@@ -5,7 +5,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -62,7 +62,10 @@ def parse_wall_time_slurm(s):
         if len(wt0) == 2:
             t = th + int(wt[0])*3600
         else:
-            t = th + int(wt[0])*60
+            try:
+                t = th + int(wt[0])*60
+            except Exception:
+                pass
 
     return t
 
@@ -153,6 +156,9 @@ class batch:
 
         if p.returncode == 0:
             return lines
+        else:
+            print(output[1])
+            return []
 
     #---------------------------------------------------------------------------
 
@@ -546,7 +552,7 @@ class batch:
         for i in range(len(batch_lines)):
             if batch_lines[i][0:4] == '#PBS':
                 batch_args = ' ' + self.__pre_parse__(batch_lines[i][4:])
-                index = string.rfind(batch_args, ' -')
+                index = batch_args.rfind(' -')
                 while index > -1:
                     arg = batch_args[index+1:]
                     batch_args = batch_args[0:index]
@@ -581,7 +587,7 @@ class batch:
                                 = int(wt[0])
                     elif arg[0:2] == '-q':
                             self.params['job_class'] = arg.split()[1]
-                    index = string.rfind(batch_args, ' -')
+                    index = batch_args.rfind(' -')
 
     #---------------------------------------------------------------------------
 
@@ -595,7 +601,7 @@ class batch:
             if batch_lines[i][0:4] == '#PBS':
                 ch = ''
                 batch_args = ' ' + self.__pre_parse__(batch_lines[i][4:])
-                index = string.rfind(batch_args, ' -')
+                index = batch_args.rfind(' -')
                 while index > -1:
                     arg = batch_args[index+1:]
                     batch_args = batch_args[0:index]
@@ -633,7 +639,7 @@ class batch:
                         ch = ' -q ' + self.params['job_class'] + ch
                     else:
                         ch = ' ' + arg + ch
-                    index = string.rfind(batch_args, ' -')
+                    index = batch_args.rfind(' -')
                 ch = '#PBS' + ch
                 batch_lines[i] = ch
 
@@ -648,7 +654,7 @@ class batch:
         for i in range(len(batch_lines)):
             if batch_lines[i][0:2] == '#$':
                 batch_args = ' ' + self.__pre_parse__(batch_lines[i][2:])
-                index = string.rfind(batch_args, ' -')
+                index = batch_args.rfind(' -')
                 while index > -1:
                     arg = batch_args[index+1:]
                     batch_args = batch_args[0:index]
@@ -672,7 +678,7 @@ class batch:
                             self.params['job_walltime'] = int(wt[0])
                     elif arg[0:2] == '-q':
                         self.params['job_class'] = arg.split()[1]
-                    index = string.rfind(batch_args, ' -')
+                    index = batch_args.rfind(' -')
 
     #---------------------------------------------------------------------------
 
@@ -686,7 +692,7 @@ class batch:
             if batch_lines[i][0:2] == '#$':
                 ch = ''
                 batch_args = ' ' + self.__pre_parse__(batch_lines[i][2:])
-                index = string.rfind(batch_args, ' -')
+                index = batch_args.rfind(' -')
                 while index > -1:
                     arg = batch_args[index+1:]
                     batch_args = batch_args[0:index]
@@ -709,7 +715,7 @@ class batch:
                         ch = ' -q ' + self.params['job_class'] + ch
                     else:
                         ch = ' ' + arg + ch
-                    index = string.rfind(batch_args, ' -')
+                    index = batch_args.rfind(' -')
                     ch = '#$' + ch
                     batch_lines[i] = ch
 
