@@ -5,7 +5,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -65,6 +65,29 @@ def adaptation(adaptation, saturne_script, case_dir):
     if cs_exec_environment.run_command(cmd) != 0:
         sys.exit(0)
 
+#-------------------------------------------------------------------------------
+
+def check_exec_dir_stamp(d):
+
+    """
+    Return path to execution directory if present in a given directory.
+    Otherwise, return given directory path
+    """
+
+    retval = d
+
+    p = os.path.join(d, 'run_status.exec_dir')
+
+    if os.path.isfile(p):
+        f = open(p, 'r')
+        l = f.readlines()
+        f.close()
+        d = l[0].rstrip()
+        if os.path.isdir(d):
+            retval = d
+
+    return retval
+
 #===============================================================================
 # Main class
 #===============================================================================
@@ -78,6 +101,8 @@ class case:
                  package,                     # main package
                  package_compute = None,      # package for compute environment
                  case_dir = None,
+                 staging_dir = None,
+                 coupling_parameters = None,
                  domains = None,
                  syr_domains = None,
                  ast_domain = None,
@@ -167,16 +192,15 @@ class case:
             self.name = os.path.split(case_dir)[1]
             self.script_dir = self.case_dir
 
+        if staging_dir:
+            staging_dir = check_exec_dir_stamp(staging_dir)
+
         for d in self.domains:
-            d.set_case_dir(self.case_dir)
+            d.set_case_dir(self.case_dir, staging_dir)
         for d in self.syr_domains:
-            d.set_case_dir(self.case_dir)
+            d.set_case_dir(self.case_dir, staging_dir)
         for d in self.ast_domains:
-            d.set_case_dir(self.case_dir)
-
-        # Mesh directory for default study structure
-
-        self.mesh_dir = os.path.join(self.study_dir, 'MESH')
+            d.set_case_dir(self.case_dir, staging_dir)
 
         # Working directory
 
@@ -204,6 +228,10 @@ class case:
                 + 'all or no domains must execute their solver.\n'
             raise RunCaseError(err_str)
 
+        # Coupling parameters if present
+
+        self.coupling_parameters = coupling_parameters
+
         # Date or other name
 
         self.run_id = None
@@ -878,20 +906,24 @@ export SALOME_INSTANCE=$3
         for d in self.syr_domains:
             s_args = d.solver_command()
             if s_args[1][0:2] == './':
-                s_path = os.path.relpath(os.path.join(s_args[0], s_args[1]),
-                                         self.exec_dir)
+                s_path = os.path.join(s_args[0], s_args[1])
+                s_path = './' + os.path.relpath(s_path, self.exec_dir)
             else:
                 s_path = s_args[1]
+            cmd = '%d-%d\t' % (rank_id, rank_id + d.n_procs - 1) \
+                   + s_path + s_args[2] \
+                   + ' -wdir ' + os.path.basename(s_args[0]) + '\n'
+            e.write(cmd)
             rank_id += d.n_procs
 
         for d in self.domains:
             s_args = d.solver_command()
             if s_args[1][0:2] == './':
-                s_path = os.path.relpath(os.path.join(s_args[0], s_args[1]),
-                                         self.exec_dir)
+                s_path = os.path.join(s_args[0], s_args[1])
+                s_path = './' + os.path.relpath(s_path, self.exec_dir)
             else:
                 s_path = s_args[1]
-            cmd = '%d-%d\n' % (rank_id, rank_id + d.n_procs - 1) \
+            cmd = '%d-%d\t' % (rank_id, rank_id + d.n_procs - 1) \
                    + s_path + s_args[2] \
                    + ' -wdir ' + os.path.basename(s_args[0]) + '\n'
             e.write(cmd)
@@ -1127,10 +1159,11 @@ export SALOME_INSTANCE=$3
 %(salomeenv)s;
 export PATH=%(exec_dir)s/bin:$PATH
 appli=%(exec_dir)s/appli
-$appli/runSession killSalome.py
 unset PYTHONHOME
-$appli/runAppli -t
-$appli/runSession $appli/bin/salome/driver -e -d 0 fsi_yacs_scheme.xml
+port_log=`pwd`/.salome_port.log
+$appli/salome start -t --ns-port-log=$port_log
+$appli/salome shell -- driver -e fsi_yacs_scheme.xml
+$appli/salome kill `cat $port_log`
 
 """
         s.write(template % {'salomeenv': self.package.config.salome_env,
@@ -1201,7 +1234,6 @@ $appli/runSession $appli/bin/salome/driver -e -d 0 fsi_yacs_scheme.xml
                 n_procs += d.n_procs
 
         # Set PATH for Windows DLL search PATH
-        # It should not harm in other circumstances
 
         if sys.platform.startswith('win'):
             cs_exec_environment.write_script_comment(s,
@@ -1236,8 +1268,6 @@ $appli/runSession $appli/bin/salome/driver -e -d 0 fsi_yacs_scheme.xml
                                                    'LD_LIBRARY_PATH',
                                                    salome_libdir)
 
-        s.write('\n')
-
         # Handle rcfile and environment modules if used
 
         rcfile = cs_exec_environment.get_rcfile(self.package_compute)
@@ -1254,6 +1284,34 @@ $appli/runSession $appli/bin/salome/driver -e -d 0 fsi_yacs_scheme.xml
                 s.write('  source ' + rcfile + '\n')
             s.write('fi\n\n')
 
+        # Add paths for plugins or dynamic library dependencies
+
+        plugin_lib_dirs, plugin_pythonpath_dirs, plugin_env_vars \
+            = self.package_compute.config.get_run_environment_dependencies()
+        for d in plugin_lib_dirs:
+            cs_exec_environment.write_prepend_path(s,
+                                                   'LD_LIBRARY_PATH',
+                                                   d)
+        for d in plugin_pythonpath_dirs:
+            cs_exec_environment.write_prepend_path(s,
+                                                   'PYTHONPATH',
+                                                   d)
+
+        # Add paths for local libraries
+        if self.package_compute.config.features['relocatable'] == "yes":
+            d = self.package_compute.get_dir('libdir')
+            if d != '/usr/lib' and d != '/usr/local/lib':
+                cs_exec_environment.write_prepend_path(s,
+                                                       'LD_LIBRARY_PATH',
+                                                       d)
+
+        # Add additional environment variables
+
+        for v in plugin_env_vars:
+            cs_exec_environment.write_export_env(s, v, plugin_env_vars[v])
+
+        s.write('\n')
+
         # Handle OpenMP if needed
 
         n_threads = exec_env.resources.n_threads
@@ -1316,35 +1374,36 @@ $appli/runSession $appli/bin/salome/driver -e -d 0 fsi_yacs_scheme.xml
     def update_scripts_tmp(self, src, dest):
 
         """
-        Create a stamp file in the scripts directory, rename it, or destroy it.
+        Create a stamp file in the execution directory, rename it, or destroy it.
         """
 
-        # Create a temporary file for SALOME (equivalent to "control_file")
+        # Create a temporary file to determine status
 
         src_tmp_name = None
         dest_tmp_name = None
 
+        base_path = os.path.join(self.exec_dir, 'run_status.')
+        if not os.path.isdir(base_path):
+            base_path = os.path.join(self.result_dir, 'run_status.')
+
         if src != None:
             if type(src) == tuple:
                 for s in src:
-                    p = os.path.join(self.script_dir, s + '.' + self.run_id)
+                    p = base_path + s
                     if os.path.isfile(p):
                         src_tmp_name = p
 
             else:
-                p = os.path.join(self.script_dir, src + '.' + self.run_id)
+                p = base_path + src
                 if os.path.isfile(p):
                     src_tmp_name = p
 
         try:
             if dest != None:
-                dest_tmp_name = os.path.join(self.script_dir,
-                                             dest + '.' + self.run_id)
+                dest_tmp_name = base_path + dest
                 if src_tmp_name == None:
                     scripts_tmp = open(dest_tmp_name, 'w')
-                    scripts_tmp.write(self.run_id + '\n')
-                    scripts_tmp.write(self.exec_dir + '\n')
-                    scripts_tmp.write(self.result_dir + '\n')
+                    scripts_tmp.write(self.case_dir + '\n')
                     scripts_tmp.close()
                 else:
                     os.rename(src_tmp_name, dest_tmp_name)
@@ -1357,6 +1416,33 @@ $appli/runSession $appli/bin/salome/driver -e -d 0 fsi_yacs_scheme.xml
 
     #---------------------------------------------------------------------------
 
+    def add_exec_dir_stamp(self):
+
+        """
+        Create a file with the execution directory path in the results directory.
+        """
+        p = os.path.join(self.result_dir, 'run_status.exec_dir')
+
+        f = open(p, 'w')
+        f.write(self.exec_dir + '\n')
+        f.close()
+
+    #---------------------------------------------------------------------------
+
+    def clear_exec_dir_stamp(self):
+
+        """
+        Remove file with the execution directory path from the results directory.
+        """
+        p = os.path.join(self.result_dir, 'run_status.exec_dir')
+
+        try:
+            os.remove(p)
+        except Exception:
+            pass
+
+    #---------------------------------------------------------------------------
+
     def set_run_id(self,
                    run_id = None):
 
@@ -1441,11 +1527,13 @@ $appli/runSession $appli/bin/salome/driver -e -d 0 fsi_yacs_scheme.xml
             if len(d.error) > 0:
                 self.error = d.error
 
-        # Rename temporary file to indicate new status
+        # Output coupling parameters for staging
 
-        # Remove the Salome temporary file
-
-        self.update_scripts_tmp('preparing', None)
+        if self.coupling_parameters:
+            s = open(os.path.join(self.result_dir,
+                                  'coupling_parameters.py'), 'w')
+            s.write(self.coupling_parameters)
+            s.close()
 
         # Rename temporary file to indicate new status
 
@@ -1611,10 +1699,7 @@ $appli/runSession $appli/bin/salome/driver -e -d 0 fsi_yacs_scheme.xml
 
         # Indicate status using temporary file for SALOME.
 
-        if self.domains[0].logging_args == '--log 0':
-            self.update_scripts_tmp('ready', 'runningstd')
-        else:
-            self.update_scripts_tmp('ready', 'runningext')
+        self.update_scripts_tmp('ready', 'running')
 
         sys.stdout.write('\n'
                          ' **********************\n'
@@ -1682,7 +1767,7 @@ $appli/runSession $appli/bin/salome/driver -e -d 0 fsi_yacs_scheme.xml
         else:
             status = 'failed'
 
-        self.update_scripts_tmp(('runningstd', 'runningext'), status)
+        self.update_scripts_tmp(('running',), status)
 
         return retcode
 
@@ -1846,6 +1931,7 @@ $appli/runSession $appli/bin/salome/driver -e -d 0 fsi_yacs_scheme.xml
         if self.exec_dir != self.result_dir:
             msg += ' Working directory (to be periodically cleaned):\n' \
                 + '   ' +  str(self.exec_dir) + '\n'
+            self.add_exec_dir_stamp()
 
         msg += '\n'
 
@@ -1870,13 +1956,13 @@ $appli/runSession $appli/bin/salome/driver -e -d 0 fsi_yacs_scheme.xml
 
             if stages['save_results'] == True:
                 self.save_results()
+                self.clear_exec_dir_stamp()
 
         finally:
             if self.run_id != None:
                 self.update_scripts_tmp(('preparing',
                                          'ready',
-                                         'runningstd',
-                                         'runningext',
+                                         'running',
                                          'finished',
                                          'failed'), None)
 
@@ -1912,6 +1998,16 @@ $appli/runSession $appli/bin/salome/driver -e -d 0 fsi_yacs_scheme.xml
 
         if len(self.domains) + len(self.syr_domains) + len(self.ast_domains) > 1:
             r += '_COUPLING'
+        elif len(self.domains) == 1 and not (run_id_prefix or run_id_suffix):
+            try:
+                d = self.domains[0]
+                if not d.exec_solver:
+                    run_id_prefix = 'import_'
+                a = d.solver_args
+                if a and a in ('--quality', '--preprocess', '--benchmark'):
+                    run_id_prefix = a[2:] + '_'
+            except Exception:
+                pass
 
         # Make sure directory does not exist already
 
