@@ -5,7 +5,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -70,8 +70,8 @@ def make_clean_dir(path):
     if not os.path.isdir(path):
         os.mkdir(path)
     else:
-        list = os.listdir(path)
-        for f in list:
+        l = os.listdir(path)
+        for f in l:
             os.remove(os.path.join(path, f))
 
 #-------------------------------------------------------------------------------
@@ -120,6 +120,7 @@ class base_domain:
         # Names, directories, and files in case structure
 
         self.case_dir = None
+        self.case_root_dir = None
 
         self.name = name # used for multiple domains only
 
@@ -127,8 +128,6 @@ class base_domain:
         self.result_dir = None
         self.src_dir = None
 
-        self.mesh_dir = None
-
         # Working directory and executable
 
         self.exec_dir = None
@@ -155,19 +154,31 @@ class base_domain:
 
     #---------------------------------------------------------------------------
 
-    def set_case_dir(self, case_dir):
+    def set_case_dir(self, case_dir, staging_dir = None):
 
         # Names, directories, and files in case structure
 
         self.case_dir = case_dir
+        self.case_root_dir = case_dir
 
         if self.name != None:
             self.case_dir = os.path.join(self.case_dir, self.name)
+            self.case_root_dir = os.path.join(case_dir, self.name)
 
         self.data_dir = os.path.join(self.case_dir, 'DATA')
         self.result_dir = os.path.join(self.case_dir, 'RESU')
         self.src_dir = os.path.join(self.case_dir, 'SRC')
 
+        # If computation is already staged, avoid reading data in upstream
+        # case directories, as it may have changed since staging.
+
+        if staging_dir:
+            self.case_dir = staging_dir
+            if self.name != None:
+                self.case_dir = os.path.join(self.case_dir, self.name)
+            self.data_dir = self.case_dir
+            self.src_dir = None
+
     #---------------------------------------------------------------------------
 
     def set_exec_dir(self, exec_dir):
@@ -239,8 +250,8 @@ class base_domain:
 
             if not os.path.isdir(dest):
                 os.mkdir(dest)
-            list = os.listdir(src)
-            for f in list:
+            l = os.listdir(src)
+            for f in l:
                 f_src = os.path.join(src, f)
                 f_dest = os.path.join(dest, f)
                 if os.path.isfile(f_src):
@@ -342,8 +353,6 @@ class base_domain:
             debug_args += 'python '
         if self.package.name != 'neptune_cfd':
             pkg_dir = self.package.get_dir('pkgpythondir')
-        else:
-            pkg_dir = self.package.get_cs_dir('pkgpythondir')
         dbg_wrapper_path = os.path.join(pkg_dir, 'cs_debug_wrapper.py')
         debug_args += dbg_wrapper_path + ' '
         for a in separate_args(debug_cmd):
@@ -450,11 +459,11 @@ class domain(base_domain):
 
     #---------------------------------------------------------------------------
 
-    def set_case_dir(self, case_dir):
+    def set_case_dir(self, case_dir, staging_dir = None):
 
         # Names, directories, and files in case structure
 
-        base_domain.set_case_dir(self, case_dir)
+        base_domain.set_case_dir(self, case_dir, staging_dir)
 
         # We may now import user python script functions if present.
 
@@ -542,12 +551,7 @@ class domain(base_domain):
         """
         # Check if there are files to compile in source path
 
-        dir_files = os.listdir(self.src_dir)
-
-        src_files = (fnmatch.filter(dir_files, '*.c')
-                     + fnmatch.filter(dir_files, '*.cxx')
-                     + fnmatch.filter(dir_files, '*.cpp')
-                     + fnmatch.filter(dir_files, '*.[fF]90'))
+        src_files = cs_compile.files_to_compile(self.src_dir)
 
         if self.exec_solver and len(src_files) > 0:
             return True
@@ -562,17 +566,13 @@ class domain(base_domain):
         """
         # Check if there are files to compile in source path
 
-        dir_files = os.listdir(self.src_dir)
-
-        src_files = (fnmatch.filter(dir_files, '*.c')
-                     + fnmatch.filter(dir_files, '*.cxx')
-                     + fnmatch.filter(dir_files, '*.cpp')
-                     + fnmatch.filter(dir_files, '*.[fF]90'))
+        src_files = cs_compile.files_to_compile(self.src_dir)
 
         if len(src_files) > 0:
 
             # Add header files to list so as not to forget to copy them
 
+            dir_files = os.listdir(self.src_dir)
             src_files = src_files + (  fnmatch.filter(dir_files, '*.h')
                                      + fnmatch.filter(dir_files, '*.hxx')
                                      + fnmatch.filter(dir_files, '*.hpp'))
@@ -629,9 +629,6 @@ class domain(base_domain):
             self.purge_result(link_path) # in case of previous run here
             self.symlink(mesh_input, link_path)
 
-        if not self.exec_solver:
-            return
-
         # Copy data files
 
         dir_files = os.listdir(self.data_dir)
@@ -646,6 +643,9 @@ class domain(base_domain):
                 if f == 'cs_user_scripts.py':  # Copy user script to results now
                     shutil.copy2(src,  os.path.join(self.result_dir, f))
 
+        if not self.exec_solver:
+            return
+
         # Call user script if necessary
 
         if self.user_locals:
@@ -662,14 +662,15 @@ class domain(base_domain):
             if not os.path.isabs(restart_input):
                 restart_input = os.path.join(self.case_dir, restart_input)
 
-            if os.path.exists(restart_input):
-
-                if not os.path.isdir(restart_input):
-                    err_str = restart_input + ' is not a directory.'
-                    raise RunCaseError(err_str)
-                else:
-                    self.symlink(restart_input,
-                                 os.path.join(self.exec_dir, 'restart'))
+            if not os.path.exists(restart_input):
+                err_str = restart_input + ' does not exist.'
+                raise RunCaseError(err_str)
+            elif not os.path.isdir(restart_input):
+                err_str = restart_input + ' is not a directory.'
+                raise RunCaseError(err_str)
+            else:
+                self.symlink(restart_input,
+                             os.path.join(self.exec_dir, 'restart'))
 
         # Partition input files
 
@@ -699,7 +700,7 @@ class domain(base_domain):
             return
 
         # Study directory
-        study_dir = os.path.split(self.case_dir)[0]
+        study_dir = os.path.split(self.case_root_dir)[0]
 
         # User config file
         u_cfg = configparser.ConfigParser()
@@ -715,7 +716,7 @@ class domain(base_domain):
         if self.mesh_dir is not None:
             mesh_dir = os.path.expanduser(self.mesh_dir)
             if not os.path.isabs(mesh_dir):
-                mesh_dir = os.path.join(self.case_dir, mesh_dir)
+                mesh_dir = os.path.join(self.case_root_dir, mesh_dir)
             mesh_dirs.append(mesh_dir)
         if os.path.isdir(os.path.join(study_dir, 'MESH')):
             mesh_dirs.append(os.path.join(study_dir, 'MESH'))
@@ -984,7 +985,7 @@ class syrthes_domain(base_domain):
 
     def __init__(self,
                  package,
-                 cmd_line = None,     # Command line to define optional syrthes4 behaviour
+                 cmd_line = None,     # Command line to define optional syrthes4 behavior
                  name = None,
                  param = 'syrthes.data',
                  log_file = None,
@@ -1002,6 +1003,9 @@ class syrthes_domain(base_domain):
 
         self.n_procs_radiation = n_procs_radiation
 
+        self.n_procs_ref = None
+        self.n_procs_radiation = None
+
         # Additional parameters for Code_Saturne/SYRTHES coupling
         # Directories, and files in case structure
 
@@ -1026,26 +1030,34 @@ class syrthes_domain(base_domain):
 
         self.syrthes_case = None
 
-        # Try to base Syrthes version on path used to import the syrthes module,
-        # for consistency with case creation parameters; it this is not enough,
-        # use existing environment or load one based on current configuration.
+        # Determine environment which should be used for Syrthes
+        # operations
 
-        try:
-            for p in sys.path:
-                if p[-14:] == '/share/syrthes' or p[-14:] == '\share\syrthes':
-                    syr_profile = os.path.join(p[:,-14], 'bin', 'syrthes.profile')
-                    if os.path.isfile(syr_profile):
-                        source_shell_script(syr_profile)
-        except Exception:
-            pass
+        ld_library_path_save = os.getenv('LD_LIBRARY_PATH')
 
         source_syrthes_env(self.package)
 
+        self.ld_library_path = os.getenv('LD_LIBRARY_PATH')
+
+        self.__ld_library_path_restore__(ld_library_path_save)
+
     #---------------------------------------------------------------------------
 
-    def set_case_dir(self, case_dir):
+    def __ld_library_path_restore__(self, ld_library_path_save):
 
-        base_domain.set_case_dir(self, case_dir)
+        if ld_library_path_save:
+            os.environ['LD_LIBRARY_PATH'] = ld_library_path_save
+        else:
+            try:
+                os.environ.pop('LD_LIBRARY_PATH')
+            except Exception:
+                pass
+
+    #---------------------------------------------------------------------------
+
+    def set_case_dir(self, case_dir, staging_dir = None):
+
+        base_domain.set_case_dir(self, case_dir, staging_dir)
 
         # Names, directories, and files in case structure
 
@@ -1116,14 +1128,51 @@ class syrthes_domain(base_domain):
 
     #---------------------------------------------------------------------------
 
-    def init_syrthes_case(self):
+    def __init_syrthes_case__(self):
         """
         Build or rebuild syrthes object
         """
 
+        # Now try to read additional data set by Syrthes GUI
+
+        part_tool_name = None
+
+        f = open(os.path.join(self.case_dir, self.param), 'r')
+        lines = f.readlines()
+        del(f)
+        for l in lines:
+            if l[0] == '/':
+                i = l.find('DOMAIN_POS=')
+                if i > -1:
+                    try:
+                        id = int(l[i + 11:].strip())
+                        if id == 0:
+                            part_tool_name = 'scotch'
+                        elif id == 1:
+                            part_tool_name = 'metis'
+                    except Exception:
+                        pass
+                i = l.find('LISTING=')
+                if i > -1:
+                    logfile = l[i + 8:].strip()
+                    if logfile:
+                        self.logfile = logfile
+                i = l.find('NBPROC_COND=')
+                if i > -1:
+                     try:
+                        self.n_procs_ref = int(l[i + 12:].strip())
+                     except Exception:
+                        pass
+                i = l.find('NBPROC_RAD=')
+                if i > -1:
+                     try:
+                        self.n_procs_radiation_ref = int(l[i + 11:].strip())
+                     except Exception:
+                        pass
+
         # Build command-line arguments
 
-        args = '-d ' + os.path.join(self.case_dir, self.param)
+        args = '-d ' + os.path.join(self.case_root_dir, self.param)
         args += ' --name ' + self.name
 
         if self.n_procs != None and self.n_procs != 1:
@@ -1132,6 +1181,9 @@ class syrthes_domain(base_domain):
         if self.n_procs_radiation > 0:
             args += ' -r ' + str(self.n_procs_radiation)
 
+        if part_tool_name:
+            args += ' -t ' + part_tool_name
+
         if self.data_dir != None:
             args += ' --data-dir ' + str(self.data_dir)
 
@@ -1173,7 +1225,10 @@ class syrthes_domain(base_domain):
         # determine whether compilation should use MPI, and this is
         # always true anyways in the coupled case.
 
-        self.init_syrthes_case()
+        ld_library_path_save = os.getenv('LD_LIBRARY_PATH')
+        os.environ['LD_LIBRARY_PATH'] = self.ld_library_path
+
+        self.__init_syrthes_case__()
 
         # Build exec_srcdir
 
@@ -1203,6 +1258,8 @@ class syrthes_domain(base_domain):
 
         self.solver_path = os.path.join('.', 'syrthes')
 
+        self.__ld_library_path_restore__(ld_library_path_save)
+
     #---------------------------------------------------------------------------
 
     def preprocess(self):
@@ -1210,8 +1267,11 @@ class syrthes_domain(base_domain):
         Partition mesh for parallel run if required by user
         """
 
+        ld_library_path_save = os.getenv('LD_LIBRARY_PATH')
+        os.environ['LD_LIBRARY_PATH'] = self.ld_library_path
+
         # Rebuild Syrthes case now that number of processes is known
-        self.init_syrthes_case()
+        self.__init_syrthes_case__()
 
         # Sumary of the parameters
         self.syrthes_case.dump()
@@ -1226,6 +1286,8 @@ class syrthes_domain(base_domain):
             err_str = '\n  Error during the SYRTHES preprocessing step\n'
             raise RunCaseError(err_str)
 
+        self.__ld_library_path_restore__(ld_library_path_save)
+
     #---------------------------------------------------------------------------
 
     def copy_results(self):
@@ -1233,6 +1295,9 @@ class syrthes_domain(base_domain):
         Retrieve results from the execution directory
         """
 
+        ld_library_path_save = os.getenv('LD_LIBRARY_PATH')
+        os.environ['LD_LIBRARY_PATH'] = self.ld_library_path
+
         # Post-processing
         if self.syrthes_case.post_mode != None:
           retval = self.syrthes_case.postprocessing(mode = \
@@ -1255,6 +1320,8 @@ class syrthes_domain(base_domain):
             err_str = '\n   Error saving SYRTHES results\n'
             raise RunCaseError(err_str)
 
+        self.__ld_library_path_restore__(ld_library_path_save)
+
     #---------------------------------------------------------------------------
 
     def summary_info(self, s):
@@ -1309,9 +1376,9 @@ class aster_domain(base_domain):
 
     #---------------------------------------------------------------------------
 
-    def set_case_dir(self, case_dir):
+    def set_case_dir(self, case_dir, staging_dir = None):
 
-        base_domain.set_case_dir(self, case_dir)
+        base_domain.set_case_dir(self, case_dir, staging_dir)
 
         e = []
         p = os.path.join(self.case_dir, self.param)
@@ -1371,7 +1438,7 @@ class aster_domain(base_domain):
         if os.path.isabs(self.conf):
             s.path = self.conf
         else:
-            s_path = os.path.join(self.aster_home, 'config.txt')
+            s_path = os.path.join(self.aster_home, 'share', 'aster', 'config.txt')
         s = open(s_path, 'r')
         template = s.read()
         s.close()
@@ -1379,7 +1446,7 @@ class aster_domain(base_domain):
         import re
 
         e_re = re.compile('profile.sh')
-        e_subst = os.path.join(self.aster_home, 'profile.sh')
+        e_subst = os.path.join(self.aster_home, 'share', 'aster', 'profile.sh')
         template = re.sub(e_re, e_subst, template)
 
         e_re = re.compile('Execution/E_SUPERV.py')
@@ -1406,10 +1473,8 @@ class aster_domain(base_domain):
         from cs_exec_environment import write_shell_shebang
         write_shell_shebang(s)
 
-        aster_root = os.path.split(self.aster_home)[0]
-
         s.write('cd ..\n')
-        s.write('export PATH=' + os.path.join(aster_root, 'bin') + ':$PATH\n')
+        s.write('export PATH=' + os.path.join(self.aster_home, 'bin') + ':$PATH\n')
         s.write('as_run ' + self.param + '\n')
 
         s.close()
