@@ -5,7 +5,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -29,7 +29,8 @@ This module describes the script used to create a study/case for Code_Saturne.
 This module defines the following functions:
 - usage
 - process_cmd_line
-- make_executable
+- set_executable
+- unset_executable
 and the following classes:
 - study
 - class
@@ -140,7 +141,7 @@ def process_cmd_line(argv, pkg):
 # Assign executable mode (chmod +x) to a file
 #-------------------------------------------------------------------------------
 
-def make_executable(filename):
+def set_executable(filename):
     """
     Give executable permission to a given file.
     Equivalent to `chmod +x` shell function.
@@ -156,6 +157,32 @@ def make_executable(filename):
 
     return
 
+#-------------------------------------------------------------------------------
+# Remove executable mode (chmod -x) from a file or files inside a directory
+#-------------------------------------------------------------------------------
+
+def unset_executable(path):
+    """
+    Remove executable permission from a given file or files inside a directory.
+    Equivalent to `chmod -x` shell function.
+    """
+
+    if os.path.isfile(path):
+        try:
+            st   = os.stat(path)
+            mode = st[stat.ST_MODE] | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH
+            mode = mode ^ (stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)
+            os.chmod(path, mode)
+        except Exception:
+            pass
+
+    elif os.path.isdir(path):
+        l = os.listdir(path)
+        for p in l:
+            unset_executable(p)
+
+    return
+
 #-------------------------------------------------------------------------------
 # Build lines necessary to import SYRTHES packages
 #-------------------------------------------------------------------------------
@@ -268,6 +295,8 @@ class Study:
         Create and initialize SYRTHES case directories.
         """
 
+        os_env_save = os.environ
+
         cs_exec_environment.source_syrthes_env(self.package)
         import syrthes
 
@@ -281,6 +310,8 @@ class Study:
                 sys.stderr.write("Cannot create SYRTHES case: '%s'\n" % s)
                 sys.exit(1)
 
+        os_environ = os_env_save
+
 
     def create_aster_case(self, repbase):
         """
@@ -345,6 +376,9 @@ domains = [
 
         for c in self.cases:
 
+            c = os.path.normpath(c)
+            base_c = os.path.basename(c)
+
             template = sep + \
 """
     {'solver': 'PACKAGE',
@@ -355,7 +389,7 @@ domains = [
      'n_procs_max': None}
 """
             template = re.sub(e_pkg, solver_name, template)
-            template = re.sub(e_dom, c, template)
+            template = re.sub(e_dom, base_c, template)
 
             dict_str += template
 
@@ -367,6 +401,9 @@ domains = [
 
         for c in self.syr_case_names:
 
+            c = os.path.normpath(c)
+            base_c = os.path.basename(c)
+
             template = \
 """
     ,
@@ -379,7 +416,7 @@ domains = [
      'opt' : ''}               # Additional SYRTHES options
                                # (ex.: postprocessing with '-v ens' or '-v med')
 """
-            template = re.sub(e_dom, c, template)
+            template = re.sub(e_dom, base_c, template)
             dict_str += template
 
         if self.ast_case_name is not None:
@@ -456,9 +493,6 @@ domains = [
 
         datadir = self.package.get_dir("pkgdatadir")
         data_distpath  = os.path.join(datadir, 'data')
-        user_distpath = os.path.join(datadir, 'user')
-        if self.package.name == 'code_saturne' :
-            user_examples_distpath = os.path.join(datadir, 'user_examples')
 
         if not self.import_only:
             try:
@@ -486,8 +520,10 @@ domains = [
                 abs_f = os.path.join(thch_distpath, f)
                 if os.path.isfile(abs_f):
                     shutil.copy(abs_f, ref)
+                    unset_executable(ref)
             abs_f = os.path.join(datadir, 'cs_user_scripts.py')
             shutil.copy(abs_f, ref)
+            unset_executable(ref)
 
         # Write a wrapper for GUI launching
 
@@ -509,7 +545,7 @@ domains = [
 
         fd.close()
 
-        make_executable(guiscript)
+        set_executable(guiscript)
 
         # User source files directory
 
@@ -519,11 +555,35 @@ domains = [
 
         if self.use_ref:
 
+            user_distpath = os.path.join(datadir, 'user')
+            user_examples_distpath = os.path.join(datadir, 'user_examples')
+
             user = os.path.join(src, 'REFERENCE')
             user_examples = os.path.join(src, 'EXAMPLES')
             shutil.copytree(user_distpath, user)
-            if self.package.name == 'code_saturne' :
-                shutil.copytree(user_examples_distpath, user_examples)
+            shutil.copytree(user_examples_distpath, user_examples)
+
+            add_datadirs = []
+            if self.package.name == 'neptune_cfd' :
+                add_datadirs.append(os.path.join(self.package.get_dir("datadir"),
+                                                 self.package.name))
+
+            for d in add_datadirs:
+                user_distpath = os.path.join(d, 'user')
+                user_examples_distpath = os.path.join(d, 'user_examples')
+
+                if os.path.isdir(user_distpath):
+                    s_files = os.listdir(user_distpath)
+                    for f in s_files:
+                        shutil.copy(os.path.join(user_distpath, f), user)
+
+                if os.path.isdir(user_examples_distpath):
+                    s_files = os.listdir(user_examples_distpath)
+                    for f in s_files:
+                        shutil.copy(os.path.join(user_examples_distpath, f), user_examples)
+
+            unset_executable(user)
+            unset_executable(user_examples)
 
         # Copy data and source files from another case
 
@@ -540,6 +600,7 @@ domains = [
                        f not in [self.package.guiname,
                                  'preprocessor_output']:
                     shutil.copy(os.path.join(ref_data, abs_f), data)
+                    unset_executable(os.path.join(data, f))
 
             # Source files
 
@@ -557,6 +618,7 @@ domains = [
             for f in c_files + h_files + f_files + \
                     cxx_files + cpp_files + hxx_files + hpp_files:
                 shutil.copy(os.path.join(ref_src, f), src)
+                unset_executable(os.path.join(src, f))
 
         # Results directory (only one for all instances)
 
