@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -142,7 +142,7 @@ def main(argv, pkg):
     else:
         images_path = os.path.join(pkg.get_dir('pkgpythondir'), 'core', 'icons')
         sys.path.insert(1, os.path.join(pkg.get_dir('pkgpythondir'), 'core'))
-        sys.path.insert(1, pkg.get_dir('cspythondir'))
+        sys.path.insert(1, pkg.get_dir('pythondir'))
 
     # Test if EOS modules could be imported
     cfg = cs_config.config()
