@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -31,7 +31,7 @@
   \section cs_user_extra_operations_examples_intro Introduction
 
   This page provides several examples of code blocks that may be used
-  to perform energy balance, 1D profiles, etc.
+  to perform data extraction or modify values
   in \ref cs_user_extra_operations.
 
 
@@ -39,8 +39,7 @@
   Here is the list of examples dedicated to different physics:
 
   - \subpage cs_user_extra_operations_examples_energy_balance_p
-  - \subpage cs_user_extra_operations_examples_scalar_balance_by_zone_p
-  - \subpage cs_user_extra_operations_examples_oned_profile_p
+  - \subpage cs_user_extra_operations_examples_balance_by_zone_p
   - \subpage cs_user_extra_operations_examples_force_temperature_p
   - \subpage cs_user_extra_operations_examples_global_efforts_p
   - \subpage cs_user_extra_operations_examples_parallel_operations_p
@@ -155,7 +154,7 @@
 // __________________________________________________________________________________
 /*!
 
-  \page cs_user_extra_operations_examples_scalar_balance_by_zone_p Scalar balance by zone
+  \page cs_user_extra_operations_examples_scalar_by_zone_p Scalar and head loss balance by zone
 
   \section cs_user_extra_operations_examples_scalar_balance_by_zone Scalar balance by zone
 
@@ -226,7 +225,7 @@
 
   Here is the corresponding code:
 
-  \snippet cs_user_extra_operations-scalar_balance_by_zone.f90 example_1
+  \snippet cs_user_extra_operations-balance_by_zone.c example_1
 
   \subsection cs_user_extra_operations_examples_example_2 Example 2
 
@@ -235,48 +234,26 @@
   on a box defined by two diagonally opposite points (the extrema in terms
   of coordinates).
 
-  The box criterium can be used as follows:
+  The box criterion can be used as follows:
   box[\f$x_{min}\f$, \f$y_{min}\f$, \f$z_{min}\f$, \f$x_{max}\f$, \f$y_{max}\f$, \f$z_{max}\f$].
 
   Here is the corresponding code:
 
-  \snippet cs_user_extra_operations-scalar_balance_by_zone.f90 example_2
+  \snippet cs_user_extra_operations-balance_by_zone.c example_2
 
-*/
-// __________________________________________________________________________________
-/*!
-
-  \page cs_user_extra_operations_examples_oned_profile_p Extract a 1D profile
-
-  \section cs_user_extra_operations_examples_oned_profile Extract a 1D profile
-
-  This is an example of \ref cs_user_extra_operations which performs 1D profile.
-
-  \subsection cs_user_extra_operations_examples_loc_var_1dp Local variables to be added
-
-  \snippet cs_user_extra_operations-extract_1d_profile.f90 loc_var_dec
-
-  \subsection cs_user_extra_operations_examples_1dp_body Body
-
-   We seek here to extract the profile of U, V, W, k and epsilon on an
-   arbitrary 1D curve based on a curvilear abscissa.
-   The profile is described in the 'profile.dat' file (do not forget to
-   define it as user data in the run script).
-
-   - the curve used here is the segment: [(0;0;0),(0;0.1;0)], but the
-     generalization to an arbitrary curve is simple.
-   - the routine handles parallelism an periodicity, as well as the different
-     turbulence models.
-   - the 1D curve is discretized into 'npoint' points. For each of these
-     points, we search for the closest cell center and we output the variable
-     values at this cell center. For better consistency, the coordinate
-     which is output is that of the cell center (instead of the initial point).
-   - we avoid using the same cell multiple times (in case several points
-     an the curve are associated with the same cell).
+  \subsection cs_user_extra_operations_examples_example_3 Scalar balance through a surface
 
+  This example computes the balance relative to a scalar named "scalar1".
   Here is the corresponding code:
 
-  \snippet cs_user_extra_operations-extract_1d_profile.f90 example_1
+  \snippet cs_user_extra_operations-balance_by_zone.c example_3
+
+  \section cs_user_extra_operations_examples_head_balance_by_zone Head loss balance by zone
+
+  This example computes the head balance  for a volume zone.
+  Here is the corresponding code:
+
+  \snippet cs_user_extra_operations-balance_by_zone.c example_4
 
 */
 // __________________________________________________________________________________
@@ -320,7 +297,7 @@
   \subsection cs_user_extra_operations_examples_geff_body Body
 
   Example: compute global efforts on a subset of faces.
-  If efforts have been calculated correctly:
+  If boundary stresses have been calculated correctly:
 
   \snippet cs_user_extra_operations-global_efforts.f90 example_1
 
