@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -22,10 +22,10 @@
   Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
-/*-----------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------------*/
 
 /*!
-  \page parameters Input of calculation parameters (C functions)
+  \page parameters Input of calculation parameters (C functions: cs_user_parameters.c)
 
   \section cs_user_parameters_h_intro Introduction
 
@@ -48,10 +48,122 @@
   Most definitions should be done in the \ref cs_user_parameters
   function, which is the C equivalent of the \ref usipsu Fortran routine.
 
-  For example, to add boundary values for all scalars, the
-  following code can be added:
+  For example, to change limiters for the convective scheme for a given scalar:
 
-  \snippet cs_user_parameters-base.c param_var_boundary_vals_1
+  \snippet cs_user_parameters-base.c param_var_limiter_choice
+
+  To set options for the solving of the Stokes problem (velocity pressure coupling
+  see \ref cs_stokes_model_t structure):
+
+  \snippet cs_user_parameters-base.c param_stokes_model
+
+  \section cs_user_parameters_h_cs_user_model_cdo Base model for CDO/HHO schemes
+
+  \subsection cs_user_parameters_h_cdo_activation Activation of CDO/HHO schemes
+
+  Two modes are available \ref CS_DOMAIN_CDO_MODE_ONLY or \ref CS_DOMAIN_CDO_MODE_WITH_FV
+
+  CDO/HHO schemes can be activated within this function as follows:
+
+  \snippet cs_user_parameters-cdo-condif.c param_cdo_activation
+
+  \subsection cs_user_parameters_h_cdo_domain_boundary Domain boundary with CDO/HHO schemes
+
+  Several types of domain boundaries can be defined. There are gathered in \ref cs_domain_boundary_type_t
+  The definition of the domain boundaries for CDO/HHO schemes can be specified
+  as follows:
+
+  \snippet cs_user_parameters-cdo-condif.c param_cdo_domain_boundary
+
+  \subsection cs_user_parameters_h_cdo_domain_output Output with CDO/HHO schemes
+
+  The management of the level and frequency of details written by the code
+  can be specified for CDO/HHO schemes as follows:
+
+  \snippet cs_user_parameters-cdo-condif.c param_cdo_domain_output
+
+  \subsection cs_user_parameters_h_cdo_time_step Time step with CDO/HHO schemes
+
+  The management of the time step with CDO/HHO schemes can be specified as
+  follows:
+
+  \snippet cs_user_parameters-cdo-condif.c param_cdo_time_step
+
+  \subsection cs_user_parameters_h_cdo_walldistance Wall distance with CDO/HHO schemes
+
+  The computation of the wall distance with CDO schemes is performed as follows:
+
+  \snippet cs_user_parameters-cdo-condif.c param_cdo_wall_distance
+
+  \subsection cs_user_parameters_h_cdo_add_user_eq Add a user-defined equation with CDO/HHO schemes
+
+  The add of a user-defined equation solved by CDO/HHO schemes is specified as
+  follows:
+
+  \snippet cs_user_parameters-cdo-condif.c param_cdo_add_user_equation
+
+  \subsection cs_user_parameters_h_cdo_add_user_pty Add user-defined properties with CDO/HHO schemes
+
+  The definition of an advection field allows one to handle flows with a frozen velocity field or
+  the transport of scalar quantities without solving the Navier-Stokes system.
+  The add of a new user-defined property or a new user-defined advection field
+  with CDO/HHO schemes is specified as follows:
+
+  \snippet cs_user_parameters-cdo-condif.c param_cdo_add_user_properties
+
+  \subsection cs_user_parameters_h_cdo_gwf Settings related to the groundwater flow module with CDO/HHO schemes
+
+  Activation of the groundwater flow module. The second argument is either 0 if no option is needed or a list of flags among
+  \ref CS_GWF_GRAVITATION, \ref CS_GWF_RICHARDS_UNSTEADY, \ref CS_GWF_SOIL_PROPERTY_UNSTEADY, \ref CS_GWF_SOIL_ALL_SATURATED
+
+  \snippet cs_user_parameters-cdo-gwf.c param_cdo_activate_gwf
+
+  Add soils (settings of the soil is performed in \ref cs_user_gwf_setup). Soils have to be added before adding tracers.
+
+  \snippet cs_user_parameters-cdo-gwf.c param_cdo_gwf_add_soil
+
+  Add tracer equations which correspond to a transport equation using
+  the darcean flux as the advection field. This call implies the
+  creation of a new equation related to a new variable field name
+  given as arguments. Advanced tracer equations can be defined using
+  \ref cs_gwf_add_tracer_user.
+
+  \snippet cs_user_parameters-cdo-gwf.c param_cdo_gwf_add_tracer
+
+  \section cs_user_parameters_h_cs_user_parameters Define or modify general numerical and physical user parameters
+
+  \subsection cs_user_parameters_h_cdo_param_eq CDO/HHO schemes
+
+  Modifiy the numerical parameters related to a given equation.
+
+  \snippet cs_user_parameters-cdo-condif.c param_cdo_numerics
+
+  \section cs_user_parameters_h_cs_user_cdo_finalize_setup Finalize the set-up for CDO/HHO schemes
+
+  \subsection cs_user_parameters_h_cdo_set_pty Set up properties with CDO/HHO schemes
+
+  When a property has been added, the second step is to define this property. According to the type
+  of property (isotropic, orthotropic or anisotropic) definitions differ.
+  Here are two examples:
+
+  \snippet cs_user_parameters-cdo-condif.c param_cdo_setup_property
+
+  \subsection cs_user_parameters_h_cdo_set_advfield Set up user-defined advection field with CDO/HHO schemes
+
+  When an advection field has been added, the second step is to define this advection field.
+  Here are is an example of definition using an anlytic function and the activation of optional features:
+
+  \snippet cs_user_parameters-cdo-condif.c param_cdo_setup_advfield
+
+  \subsection cs_user_parameters_h_cdo_set_bcs Set up the boundary conditions for an equation
+
+  \snippet cs_user_parameters-cdo-condif.c param_cdo_setup_bcs
+
+  \subsection cs_user_parameters_h_cdo_add_eq_terms Add terms to an equation
+
+  Add terms like diffusion term, advection term, unsteady term, reaction terms or source terms.
+
+  \snippet cs_user_parameters-cdo-condif.c param_cdo_add_terms
 
   \section cs_user_parameters_h_cs_user_linear_solvers  Linear solver related options
 
@@ -105,6 +217,13 @@
 
   \snippet cs_user_parameters-linear_solvers.c sles_verbosity_1
 
+  \subsection cs_user_parameters_h_sles_viz_1 Example: error visualization
+
+  The following example shows how to activate local error visualization
+  output (here for velocity and pressure).
+
+  \snippet cs_user_parameters-linear_solvers.c sles_viz_1
+
   \subsection cs_user_parameters_h_sles_mgp_1 Example: advanced multigrid settings
 
   The following example shows how to set advanced settings for the
@@ -292,13 +411,26 @@
 
   \snippet cs_user_parameters-time_moments.c tmom_simple_sum_data
 
-  In \ref cs_user_time_moments, we can not assign that function to a moments
+  In \ref cs_user_time_moments, we can now assign that function to a moments
   definition:
 
   \snippet cs_user_parameters-time_moments.c tmom_simple_sum
 
   \subsection cs_user_parameters_h_example_5 Example 5
 
+  This next example illustrates the use of another user-defined function
+  to evaluate expressions. Here, we compute the moment of the thermal flux
+  at the boundary. We also that we compute both means and variances here.
+
+  \snippet cs_user_parameters-time_moments.c tmom_b_thermal_flux_data
+
+  In \ref cs_user_time_moments, we assign that function to a moments
+  definition:
+
+  \snippet cs_user_parameters-time_moments.c tmom_b_thermal_flux
+
+  \subsection cs_user_parameters_h_example_6 Example 6
+
   In this last example, we compute components of the mean velocity
   in the case of a rotating mesh. As the mesh orientation changes
   at each time step, it is necessary to compensate for this
@@ -320,9 +452,9 @@
 
   \snippet cs_user_parameters-time_moments.c tmom_velocity_rotation
 
-  \section cs_user_parameters_h_cs_user_fans  Fan modeling options
+  \section cs_user_parameters_h_cs_user_fans  Fan modelling options
 
-  Code_Saturne allows modeling of some circular fans as volume
+  Code_Saturne allows modelling of some circular fans as volume
   regions, defined by simple geometric characteristics, and modeled
   as explicit momentum source terms in those regions.
 
@@ -335,4 +467,21 @@
 
   \snippet cs_user_parameters-fans.c fan_user_1
 
+  \section cs_user_parameters_h_postprocess Activate some automatic post-processings
+
+  To add boundary values for all scalars, the
+  following code can be added:
+
+  \snippet cs_user_parameters-base.c param_var_boundary_vals_1
+
+  You can activate the post-processing of the Q-criterion on the whole domain
+  mesh with:
+
+  \snippet cs_user_parameters-base.c param_var_q_criterion
+
+  You can activate the post-processing of clipping on turbulent quantities
+  on the whole domain mesh with:
+
+  \snippet cs_user_parameters-base.c param_var_rij_clipping
+
 */
