@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -73,7 +73,7 @@ Therefore :
  \note In \c Code_Saturne the flux is positive when it is oriented from inside to outside.
 
 
-  |  Corps                       |     Emissivity    |
+  |  Body                        |     Emissivity    |
   |------------------------------|------------------:|
   |  polished steel              |       0.06        |
   |  oxidized steel              |       0.80        |
@@ -89,79 +89,61 @@ Therefore :
 
   \subsection bound_faces Boundary faces identification
 
-   Boundary faces may be identified using the \ref getfbr subroutine. The syntax of this subroutine is described in the \ref cs_user_boundary_conditions subroutine, but a more thorough description can be found in the user guide.
+   Boundary faces may be identified using the \ref getfbr function,
+   or preferrably, through boundary zones, defined using the
+   GUI or the \ref cs_user_zones function..
 
-\note These usefull constant are definded \n
-\f$ TKELVI = 273.16D0 \f$ \n
-\f$ SIG = 5.6703D-8 \f$
+\subsection init_fin Initialization and finalization
 
+The following declaration and initialization block needs to be added
+for the following examples:
 
-  \subsection init_fin Initialization and finalization
+\snippet cs_user_radiative_transfer_bcs.c loc_var
 
+<b> Remaining initialisation</b>
 
-   The following initialization block needs to be added for the following examples:
+ivar: number of the thermal variable
 
-  \snippet cs_user_radiative_transfer_bcs.f90 allocate
+\snippet cs_user_radiative_transfer_bcs.c ivar
 
-  At the end of the subroutine, it is recommended to deallocate the work array:
+Min and Max values for the wall temperatures (clipping otherwise)
 
-  \snippet cs_user_radiative_transfer_bcs.f90 deallocate
+\f$ T_{min} \f$ and \f$T_{max} \f$ are given in Kelvin.
 
-  In theory Fortran 95 deallocates locally-allocated arrays automatically, but deallocating arrays in a symetric manner to their allocation is good pratice, and avoids using a different logic for C and Fortran.
+\snippet  cs_user_radiative_transfer_bcs.c temp
 
- <b> Remaining initialisation</b>
+\subsection assign2 Assign boundary conditions to boundary wall
 
+\section cs_user_radiative_transfer_bcs_zones  Zone definitions
 
-  ivar: number of the thermal variable
+For each boundary face face_id, a specific output (logging and
+postprocessing) zone id may be assigned. This allows realizing balance
+sheets by treating them separately for each zone. By default, the
+output zone id is set to the general (input) zone id associated to a face.
 
-   \snippet cs_user_radiative_transfer_bcs.f90 ivar
+To access output zone ids (both for reading and modifying), use the
+\ref cs_rad_transfer_get_output_b_face_zone_ids function.
+The zone id values are arbitrarily chosen by the user, but must be
+positive integers; very high numbers may also lead to higher memory
+consumption.
 
-  Min and Max values for the wall temperatures (clipping otherwise)
-
- \f$ T_{min} \f$ and \f$T_{max} \f$ are given in Kelvin.
-
-  \snippet  cs_user_radiative_transfer_bcs.f90 temp
-
-   \subsection assign2 Assign boundary conditions to boundary wall
-
-    \subsubsection zone_def Zones definition
-
- We define zones of wall boundary, and we assign a type.
-   This allows to apply the boundary conditions and realize
-   balance sheets by treating them separately for each zone.
-
- For each boundary face ifac (not just the faces of wall)
-   the user defines his own choice by a number of zone
-   \c izfrdp(ifac) from color of the boundary face
-     or more generally, their properties (color, groups ...),
-     or boundary conditions specified in \ref cs_user_boundary_conditions,
-     or even of their coordinates.
-
-\warning It is essential that ALL boundary faces
-   have been assigned to a zone.
-   The number of zones (the value of \c izfrdp(ifac)) is
-   arbitrarily chosen by the user, but must be a
-   positive integer and less than or equal to \c nbzrdm
-   (value set in parameter \ref radiat.h).
-
-
-\paragraph wall_carac Wall caracteristics
+\paragraph wall_carac Wall characteristics
 
 \warning The unit of the temperature is the Kelvin
 
 \paragraph manda Mandatory data
 
-   - \c isothp(ifac) boundary face type
-               -  \c itpimp -> Gray wall with fixed inside temperature
-               -  \c ipgrno -> Gray wall with fixed outside temperature
-               -  \c iprefl -> Reflecting wall with fixed outside temperature
-               -  \c ifgrno -> Gray wall with fixed conduction flux
-               -  \c ifrefl -> Reflecting wall with fixed conduction flux
+  - \c isothp(ifac) boundary face type
+              -  \c itpimp -> Gray wall with fixed inside temperature
+              -  \c ipgrno -> Gray wall with fixed outside temperature
+              -  \c iprefl -> Reflecting wall with fixed outside temperature
+              -  \c ifgrno -> Gray wall with fixed conduction flux
+              -  \c ifrefl -> Reflecting wall with fixed conduction flux
 
-   - \c tintp(ifac) inside wall temperature (Kelvin)
-                  initialize thwall at the first time step.
-                  If \c isothp = \c itpimp, the value of thwall is fixed to \c tintp
-                  In the other case, \c tintp is only for initialization.
+  - \c tintp(ifac) inside wall temperature (Kelvin)
+                   initialize thwall at the first time step.
+                   If \c isothp = \c itpimp, the value of thwall is fixed to \c tintp
+                   In the other case, \c tintp is only for initialization.
 \paragraph data Other data (depending of the isothp)
 
   - \c rcodcl = conduction flux
@@ -179,21 +161,21 @@ Here is a list of examples:
 
   For wall boundary faces, selection criteria: color 1  \n
 
-  \snippet   cs_user_radiative_transfer_bcs.f90  example_1
+  \snippet   cs_user_radiative_transfer_bcs.c  example_1
 
 
   \subsubsection ex2 Gray or black wall with fixed outside temperature \f$ T_{ext} \f$
 
   For wall boundary faces, selection criteria: color 2  \n
 
-  \snippet  cs_user_radiative_transfer_bcs.f90 example_2
+  \snippet  cs_user_radiative_transfer_bcs.c example_2
 
 
   \subsubsection ex3 Reflecting wall (\f$ epsp = 0 \f$) with fixed outside temperature \f$ T_{ext} \f$
 
   For wall boundary faces, selection criteria: color 3 \n
 
-  \snippet  cs_user_radiative_transfer_bcs.f90 example_3
+  \snippet  cs_user_radiative_transfer_bcs.c example_3
 
 
   \subsubsection ex4 Gray or black wall and fixed conduction flux through the wall
@@ -217,7 +199,7 @@ Flux density (< 0 if gain for the fluid)
  - For enthalpy \f$h\f$, in \f$ W.m^{-2} \f$:
  \f[ RCODC(IFAC,IVAR,3)=(viscls+\frac{visct}{\sigma})\cdot \grad{H} \cdot \vect{n}\f]
 
-\snippet  cs_user_radiative_transfer_bcs.f90 example_4
+\snippet  cs_user_radiative_transfer_bcs.c example_4
 
 
 \subsubsection ex5 Reflecting wall and fixed conduction flux through the wall
@@ -236,76 +218,77 @@ If the conduction flux is zero then the wall is adiabatic.
   - For enthalpies \f$h\f$,      in \f$ W.m^{-2} \f$:
     \f[  rcodcl(ifac,ivar,3) =    (viscls+\frac{visct}{\sigma})  \cdot \grad{H} \cdot \vect{n} \f]
 
-\snippet  cs_user_radiative_transfer_bcs.f90 example_5
+\snippet  cs_user_radiative_transfer_bcs.c example_5
 
 \subsubsection w Warning
 
 For all boundary faces that are not wall it is MANDATORY to impose a number of
 zone in the array \c izfrdp. For each zone, informations will be displayed in the listing.
 
-\snippet cs_user_radiative_transfer_bcs.f90 w
+\snippet cs_user_radiative_transfer_bcs.c w
 
 Verification that all boundary faces have been treated
 
-\snippet  cs_user_radiative_transfer_bcs.f90 check
+\snippet  cs_user_radiative_transfer_bcs.c check
 
 \subsection end_loop End of the loop on the boundary faces
 
-\snippet  cs_user_radiative_transfer_bcs.f90 end_radiative
+\snippet  cs_user_radiative_transfer_bcs.c end_radiative
 \subsection format_radiative_trans Format
-\snippet cs_user_radiative_transfer_bcs.f90 format_radiative
+\snippet cs_user_radiative_transfer_bcs.c format_radiative
 
 \section abso_flux Absorption coefficient and net radiation flux
 
 The absorption coefficient and the net radiation flux for the radiative module
-can be defined in \ref cs_user_radiative_transfer.f90 through the \ref usray3
-and \ref usray5 subroutines.
+can be defined in \ref cs_user_radiative_transfer.c through the
+\ref cs_user_rad_transfer_absorption
+and \ref cs_user_rad_transfer_net_flux subroutines.
 
 \subsection abso Absorption coefficient
 
-The absorption coefficient is defined in \ref usray3 .
+The absorption coefficient is defined in \ref cs_user_rad_transfer_absorption.
 
-\subsubsection arg Arguments of usray3
+\subsubsection arg Arguments of cs_user_rad_transfer_absorption
 
-\snippet cs_user_radiative_transfer.f90 arg_1
+\snippet cs_user_radiative_transfer.c arg_1
 
 \subsubsection var Local variables to be added
 
-\snippet cs_user_radiative_transfer.f90 loc_var_dec_1
+\snippet cs_user_radiative_transfer.c loc_var_dec_1
 
 \subsubsection abso_coeff_computation Computation of the absorption coefficient
 
-\snippet cs_user_radiative_transfer.f90 abso_coeff
+\snippet cs_user_radiative_transfer.c abso_coeff
 
 \subsubsection format_1 Format
 
-\snippet cs_user_radiative_transfer.f90 format_1
+\snippet cs_user_radiative_transfer.c format_1
 
-\subsection usray5_flux Net radiation flux
+\subsection cs_user_rad_transfer_net_flux Net radiation flux
 
-The net radiation flux is computed in \ref usray5 .
+The net radiation flux is computed in \ref cs_user_rad_transfer_net_flux.
 
-\subsubsection arg2 Arguments of usray5
+\subsubsection arg2 Arguments of cs_user_rad_transfer_net_flux
 
-\snippet cs_user_radiative_transfer.f90 arg_2
+\snippet cs_user_radiative_transfer.c arg_2
 
 \subsubsection var2 Local variables to be added
 
-\snippet cs_user_radiative_transfer.f90 loc_var_dec_2
+\snippet cs_user_radiative_transfer.c loc_var_dec_2
 
 \subsubsection init Initialization
 
 At the end of the subroutine, if \c iok is different from zero, some faces
 have been forgotten and the calculation stops.
 
-\snippet cs_user_radiative_transfer.f90 init
+\snippet cs_user_radiative_transfer.c init
 
 \subsubsection net_flux_computation Computation of the net radiation flux
 
-\snippet cs_user_radiative_transfer.f90 net_flux
+\snippet cs_user_radiative_transfer.c net_flux
 
 \subsubsection format_2 Format
 
-\snippet cs_user_radiative_transfer.f90 format_2
+\snippet cs_user_radiative_transfer.c format_2
 
 */
