@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -85,8 +85,6 @@
   refer to the user manual or to the comments on the similar command
   \ref getfbr in the routine \ref cs_user_boundary_conditions.
 
-
-
    \section loc_var1 Local variables
 
    \snippet cs_user_source_terms.f90 loc_var_dec_1
@@ -123,7 +121,14 @@ which yields:
 
  \section body1 Body
 
-   \snippet  cs_user_source_terms.f90 remaining_1
+   \snippet cs_user_source_terms.f90 remaining_1
+
+   \section example_source_terms_2 Example of a boussinesq momentum source term
+   Example to add Boussinesq source to the z component of \f$\vect{u}\f$:
+
+ \section body2 Body
+
+   \snippet cs_user_source_terms.f90 boussinesq_st 
 
 */
 //_________________________________________________________________________________________________
