@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2015 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -31,8 +31,9 @@
 !> \brief User subroutine which fills boundary conditions arrays
 !> (\c icodcl, \c rcodcl) for unknown variables.
 !>
+!> See \subpage cs_user_boundary_conditions_examples for examples.
 !>
-!> \section intro Introduction
+!> \section cs_user_boundary_conditions_intro Introduction
 !>
 !> Here one defines boundary conditions on a per-face basis.
 !>
@@ -213,11 +214,11 @@
 !>     - For the pressure P,              in  s/m:
 !>        \c rcodcl(ifac, ivar, 2) =                     dt / d
 !>     - For temperatures T,              in Watt/(m2 degres):
-!>        \c rcodcl(ifac, ivar, 2) = Cp*(viscls+visct/sigmas) / d
+!>        \c rcodcl(ifac, ivar, 2) = Cp*(viscls+visct/turb_schmidt) / d
 !>     - For enthalpies H,                in kg /(m2 s):
-!>        \c rcodcl(ifac, ivar, 2) =    (viscls+visct/sigmas) / d
+!>        \c rcodcl(ifac, ivar, 2) =    (viscls+visct/turb_schmidt) / d
 !>     - For other scalars F              in:
-!>        \c rcodcl(ifac, ivar, 2) =    (viscls+visct/sigmas) / d
+!>        \c rcodcl(ifac, ivar, 2) =    (viscls+visct/turb_schmidt) / d
 !>            (d has the dimension of a distance in m)
 !>
 !>  - \c rcodcl(ifac, ivar, 3) if \c icodcl(ifac, ivar) = 3 or 13:
@@ -227,11 +228,11 @@
 !>     - For pressure P,                  in kg/(m2 s):
 !>        \c rcodcl(ifac, ivar, 3) =                    -dt * (grad P).n
 !>     - For temperatures T,              in Watt/m2:
-!>        \c rcodcl(ifac, ivar, 3) = -Cp*(viscls+visct/sigmas) * (grad T).n
+!>        \c rcodcl(ifac, ivar, 3) = -Cp*(viscls+visct/turb_schmidt) * (grad T).n
 !>     - For enthalpies H,                in Watt/m2:
-!>        \c rcodcl(ifac, ivar, 3) = -(viscls+visct/sigmas) * (grad H).n
+!>        \c rcodcl(ifac, ivar, 3) = -(viscls+visct/turb_schmidt) * (grad H).n
 !>     - For other scalars F              in:
-!>        \c rcodcl(ifac, ivar, 3) = -(viscls+visct/sigmas) * (grad F).n
+!>        \c rcodcl(ifac, ivar, 3) = -(viscls+visct/turb_schmidt) * (grad F).n
 !>
 !>  - \c rcodcl(ifac, ivar, 3) if \c icodcl(ifac, ivar) = 6:
 !>      Roughness for the rough wall law
@@ -325,15 +326,16 @@
 !>       entering the subroutine.
 !>
 !>
-!> \subsubsection cell_id Cell values of some variables
+!> \subsubsection cs_user_bc_cell_id Cell values of some variables
 !>
 !> Cell value field ids
 !>
-!> - Density:                          \c iprpfl(ipproc(irom))
-!> - Dynamic molecular viscosity:      \c iprpfl(ipproc(iviscl))
-!> - Turbulent viscosity:              \c iprpfl(ipproc(ivisct))
-!> - Specific heat:                    \c iprpfl(ipproc(icp)
-!> - Diffusivity(lambda):              \c field_get_key_int(ivarfl(isca(iscal)), kivisl, ...)
+!> - Density:                        \c irom
+!> - Dynamic molecular viscosity:    \c iviscl
+!> - Turbulent viscosity:            \c ivisct
+!> - Specific heat:                  \c icp
+!> - Diffusivity(lambda):            \c field_get_key_int(ivarfl(isca(iscal)), &
+!>                                      kivisl, ...)
 !>
 !>
 !> \subsubsection fac_id Faces identification
@@ -344,6 +346,10 @@
 !>     using \c field_get_key_int(ivarfl(ivar), kbmasf, iflmab)
 !> - For other values: take as an approximation the value in the adjacent cell
 !>                     i.e. as above with \c iel = ifabor(ifac).
+!>
+!> Please refer to the
+!> <a href="../../theory.pdf#boundary"><b>boundary conditions</b></a>
+!> section of the theory guide for more informations.
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
@@ -387,7 +393,7 @@
 !>                                     \grad T \cdot \vect{n} \f$
 !_______________________________________________________________________________
 
-subroutine cs_user_boundary_conditions &
+subroutine cs_f_user_boundary_conditions &
  ( nvar   , nscal  ,                                              &
    icodcl , itrifb , itypfb , izfppp ,                            &
    dt     ,                                                       &
@@ -417,11 +423,12 @@ use ppcpfu
 use atincl
 use atsoil
 use ctincl
-use elincl
 use cs_fuel_incl
 use mesh
 use field
 use turbomachinery
+use iso_c_binding
+use cs_c_bindings
 
 !===============================================================================
 
@@ -431,19 +438,17 @@ implicit none
 
 integer          nvar   , nscal
 
-integer          icodcl(nfabor,nvarcl)
+integer          icodcl(nfabor,nvar)
 integer          itrifb(nfabor), itypfb(nfabor)
 integer          izfppp(nfabor)
 
 double precision dt(ncelet)
-double precision rcodcl(nfabor,nvarcl,3)
+double precision rcodcl(nfabor,nvar,3)
 
 ! Local variables
 
 integer ilelt, ifac, nlelt
 
-! INSERT_VARIABLE_DEFINITIONS_HERE
-
 integer, allocatable, dimension(:) :: lstelt
 
 !===============================================================================
@@ -455,8 +460,6 @@ integer, allocatable, dimension(:) :: lstelt
 
 allocate(lstelt(nfabor))  ! temporary array for boundary faces selection
 
-! INSERT_ADDITIONAL_INITIALIZATION_CODE_HERE
-
 !===============================================================================
 ! Assign boundary conditions to boundary faces here
 
@@ -491,4 +494,4 @@ endif
 deallocate(lstelt)  ! temporary array for boundary faces selection
 
 return
-end subroutine cs_user_boundary_conditions
+end subroutine cs_f_user_boundary_conditions
