@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -407,6 +407,7 @@ class BrowserView(QWidget, Ui_BrowserForm):
     Identity and paths
 Calculation environment
     Meshes selection
+    Notebook
 Thermophysical models
     Calculation features
     Deformable mesh
@@ -421,6 +422,7 @@ Thermophysical models
     Species transport
     Turbomachinery
     Groundwater flows
+    Fans
 Physical properties
     Reference values
     Fluid properties
@@ -454,6 +456,7 @@ Calculation control
     Surface solution control
     Lagrangian solution control
     Profiles
+    Balance by zone
 Calculation management
     Start/Restart
     Performance tuning
@@ -675,6 +678,7 @@ Calculation management
         self.setRowClose(self.tr('Groundwater laws'))
 
         if case['prepro'] == True:
+            self.setRowClose(self.tr('Notebook'))
             self.setRowClose(self.tr('Thermophysical models'))
             self.setRowClose(self.tr('Physical properties'))
             self.setRowClose(self.tr('Volume conditions'))
@@ -687,8 +691,11 @@ Calculation management
             self.setRowClose(self.tr('Surface solution control'))
             self.setRowClose(self.tr('Lagrangian solution control'))
             self.setRowClose(self.tr('Profiles'))
+            self.setRowClose(self.tr('Balance by zone'))
+            self.setRowClose(self.tr('Fans'))
             return
         else:
+            self.setRowOpen(self.tr('Notebook'))
             self.setRowOpen(self.tr('Thermophysical models'))
             self.setRowOpen(self.tr('Physical properties'))
             self.setRowOpen(self.tr('Volume conditions'))
@@ -701,6 +708,8 @@ Calculation management
             self.setRowOpen(self.tr('Surface solution control'))
             self.setRowOpen(self.tr('Lagrangian solution control'))
             self.setRowOpen(self.tr('Profiles'))
+            self.setRowOpen(self.tr('Balance by zone'))
+            self.setRowOpen(self.tr('Fans'))
 
         # Steady flow management
 
@@ -813,6 +822,8 @@ Calculation management
             self.setRowOpen(self.tr('Groundwater flows'))
             self.setRowClose(self.tr('Turbulence models'))
             self.setRowClose(self.tr('Turbomachinery'))
+            self.setRowClose(self.tr('Fans'))
+            self.setRowClose(self.tr('Physical properties'))
             self.setRowClose(self.tr('Reference values'))
             self.setRowClose(self.tr('Gravity'))
             self.setRowClose(self.tr('Fluid properties'))
