@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -39,6 +39,8 @@ This module defines the following classes:
 import sys
 import os
 import logging
+import locale
+
 Py2 = sys.version[0] == '2'
 Py3 = sys.version[0] == '3'
 
@@ -154,7 +156,10 @@ if os.environ.get('QT_API', 'pyqt') == 'pyqt':
             elif convfunc is int:
                 return int(qobj)
             elif convfunc is float:
-                return float(qobj)
+                try:
+                    return float(qobj)
+                except Exception:
+                    return locale.atof(qobj)
             else:
                 return qobj
 
@@ -485,18 +490,35 @@ class ComboModel:
             self.combo.setCurrentIndex(index)
 
 
-    def getIndex(self, str_model="", str_view=""):
+    def enableItem(self, index=None, str_model="", str_view=""):
         """
-        Get the index for a string.
+        Enable the item specified with its index or a string.
         """
-        if str_model:
+        if index is not None:
+            self.__enableItem(index)
+
+        elif str_model:
             index = self.items.index(str_model)
+            self.__enableItem(index)
 
         elif str_view:
             str_model = self.dicoV2M[str_view]
             index = self.items.index(str_model)
+            self.__enableItem(index)
 
-        return index
+
+    def hide(self):
+        """
+        Hide combobox.
+        """
+        self.combo.hide()
+
+
+    def show(self):
+        """
+        Show combobox.
+        """
+        self.combo.show()
 
 
     def getItems(self):
@@ -665,6 +687,7 @@ class DoubleValidator(QDoubleValidator):
         Initialization for validator
         """
         QDoubleValidator.__init__(self, parent)
+        self.setLocale(QLocale(QLocale.C, QLocale.AnyCountry))
         self.parent = parent
         self.state = QValidator.Invalid
         self.__min = min
@@ -749,13 +772,12 @@ class DoubleValidator(QDoubleValidator):
         """
         state = QDoubleValidator.validate(self, stri, pos)[0]
 
-        try:
-            x = from_qvariant(stri, float)
-            valid = True
-            pass
-        except (TypeError, ValueError):
-            x = 0.0
-            valid = False
+        if state == QValidator.Acceptable:
+            try:
+                x = from_qvariant(stri, float)
+            except Exception: # may be type error or localization issue
+                x = 0.0
+                state = QValidator.Intermediate
 
         if state == QValidator.Acceptable:
             if self.exclusiveMin and x == self.bottom():
@@ -765,7 +787,7 @@ class DoubleValidator(QDoubleValidator):
 
         palette = self.parent.palette()
 
-        if not valid or state == QValidator.Intermediate:
+        if state != QValidator.Acceptable:
             palette.setColor(QPalette.Text, QColor("red"))
             self.parent.setPalette(palette)
         else:
