@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -38,7 +38,7 @@ This module defines the following classes:
 # Library modules import
 #-------------------------------------------------------------------------------
 
-import os, sys, string, unittest, logging
+import os, sys, unittest, logging
 from xml.dom.minidom import Document, parse, parseString, Node
 from xml.sax.handler import ContentHandler
 from xml.sax import make_parser
@@ -49,12 +49,6 @@ from xml.sax import make_parser
 
 from code_saturne.Base.Toolbox import GuiParam
 
-#-------------------------------------------------------------------------------
-# Third-party modules
-#-------------------------------------------------------------------------------
-
-from code_saturne.Base.QtCore    import *
-
 #-------------------------------------------------------------------------------
 # log config
 #-------------------------------------------------------------------------------
@@ -105,7 +99,6 @@ class Dico:
         """
         self.data = {}
 
-        self.data['new']              = ""
         self.data['saved']            = "yes"
         self.data['prepro']           = True
         self.data['xmlfile']          = ""
@@ -389,7 +382,7 @@ class XMLElement:
     def __getitem__(self, attr):
         """
         Return the XMLElement attribute's value
-        with a dictionary syntaxe: node['attr']
+        with a dictionary syntax: node['attr']
         Return None if the searched attribute does not exist.
         """
         a = self.el.getAttributeNode(attr)
@@ -401,7 +394,7 @@ class XMLElement:
     def __setitem__(self, attr, value):
         """
         Set a XMLElement attribute an its value
-        with a dictionary syntaxe: node['attr'] = value
+        with a dictionary syntax: node['attr'] = value
         """
         if self.ca: self.ca.modified()
         self.el.setAttribute(attr, _encode(str(value)))
@@ -411,7 +404,7 @@ class XMLElement:
 
     def __delitem__(self, name):
         """
-        Delete a XMLElement attribute with a dictionary syntaxe: del node['attr']
+        Delete a XMLElement attribute with a dictionary syntax: del node['attr']
         """
         if self.ca: self.ca.modified()
         self.xmlDelAttribute(name)
@@ -552,23 +545,6 @@ class XMLElement:
         #
         nodeL = self.el.getElementsByTagName(tag)
 
-        # If there is a precision for the selection
-        #
-#TRYME
-##        for attr in attrList:
-##            for node in nodeL:
-##                if node.hasAttribute(attr) and node not in nodeList:
-##                    nodeList.append(node)
-
-##        if attrList and kwargs:
-##            nodeL = nodeList
-##            nodeList = []
-
-##        for k, v in kwargs.items():
-##            for node in nodeL:
-##                if node.getAttribute(k) == v and node not in nodeList:
-##                    nodeList.append(node)
-
         for node in nodeL:
             iok = 0
             for attr in attrList:
@@ -592,7 +568,6 @@ class XMLElement:
                     iok = 0
                     break
             if iok: nodeList.append(node)
-#TRYME
 
         if not attrList and not kwargs:
             nodeList = nodeL
@@ -680,7 +655,7 @@ class XMLElement:
     def xmlSetData(self, tag, textNode, *attrList, **kwargs):
         """
         Set the textNode in an elementNode, which is a single node.
-        If the searched 'tag' doesn't exist, it is create
+        If the searched 'tag' doesn't exist, it is created
         and textNode is added as a child TEXT_NODE nodeType.
         Return a XMLElement list of the created Node.
         """
@@ -1197,7 +1172,7 @@ class XMLDocument(XMLElement):
         if isinstance(node, XMLElement):
             node = node.el
 
-        if 'formula' not in node.tagName:
+        if 'formula' not in node.tagName or 'dirichlet_formula' in node.tagName:
             for n in node.childNodes:
                 if n.nodeType == Node.TEXT_NODE:
                     n.data = self.xmlNormalizeWhitespace(n.data)
@@ -1253,37 +1228,36 @@ class XMLDocument(XMLElement):
 # XML utility functions
 #-------------------------------------------------------------------------------
 
-class Case(Dico, XMLDocument, QObject):
-    undo_signal = pyqtSignal()
+class Case(Dico, XMLDocument):
 
-    def __init__(self, package=None, file_name=""):
+    def __init__(self, package=None, file_name="", studymanager=False):
         """
         Instantiate a new dico and a new xml doc
         """
         Dico.__init__(self)
         XMLDocument.__init__(self, case=self)
-        QObject.__init__(self)
 
         if package:
             self['package'] = package
             rootNode = '<' + self['package'].code_name +'_GUI study="" case="" version="2.0"/>'
         else:
             rootNode = '<' + '_GUI study="" case="" version="2.0"/>'
+        if studymanager:
+            rootNode = '<' + 'studymanager version="2.0"/>'
 
         if file_name:
             self.parse(file_name)
-            self['new'] = "no"
             self['saved'] = "yes"
         else:
             self.parseString(rootNode)
-            self['new'] = "yes"
-            self['saved'] = "yes"
+            self['saved'] = "no"
 
         self.record_func_prev = None
         self.record_argument_prev = None
         self.record_local = False
         self.record_global = True
         self.xml_prev = ""
+        self.xml_saved = self.toString()
 
 
     def xmlRootNode(self):
@@ -1305,7 +1279,13 @@ class Case(Dico, XMLDocument, QObject):
         """
         Return True if the xml doc is modified.
         """
-        return self['saved'] == "no"
+        # return self['saved'] == "no"
+
+        s = self.toIOString()
+        if s == self.xml_saved:
+            return False
+        else:
+            return True
 
 
     def __del__(self):
@@ -1328,6 +1308,15 @@ class Case(Dico, XMLDocument, QObject):
         return self.toPrettyUnicodeString()
 
 
+    def toIOString(self):
+        """
+        Transform to string for IO.
+        """
+        d = XMLDocument().parseString(self.toPrettyString())
+        d.xmlCleanHightLevelBlank(d.root())
+        return d.toString()
+
+
     def xmlSaveDocument(self):
         """
         This method writes the associated xml file.
@@ -1336,10 +1325,11 @@ class Case(Dico, XMLDocument, QObject):
         try:
             d = XMLDocument().parseString(self.toPrettyString())
             d.xmlCleanHightLevelBlank(d.root())
+            s = d.toString()
             file = open(self['xmlfile'], 'w')
-            file.write(d.toString())
+            file.write(s)
             file.close()
-            self['new'] = "no"
+            self.xml_saved = s
             self['saved'] = "yes"
             d.doc.unlink()
         except IOError:
@@ -1465,7 +1455,7 @@ class Case(Dico, XMLDocument, QObject):
         try:
             file = open(self['pythonfile'], 'w')
             file.write("#automatic python script\n")
-            file.write('import os, sys, string\n')
+            file.write('import os, sys\n')
             file.write('sys.path.insert(0, "' + self['package'].dirs['pythondir'][1] + '")\n')
             file.write('sys.path.insert(0, "' + os.path.join(self['package'].dirs['pythondir'][1], self['package'].name) + '")\n\n')
 
@@ -1473,7 +1463,7 @@ class Case(Dico, XMLDocument, QObject):
                 file.write('import cs_package\n')
                 file.write('from Base.XMLinitialize import XMLinit\n')
             else:
-                file.write('sys.path.insert(0, "' + os.path.join(self['package'].dirs['cspythondir'][1], "code_saturne") + '")\n\n')
+                file.write('sys.path.insert(0, "' + os.path.join(self['package'].dirs['pythondir'][1], "code_saturne") + '")\n\n')
                 file.write('import nc_package\n')
                 file.write('from core.XMLinitialize import XMLinit\n')
             file.write('from Base.XMLengine import Case\n\n')
@@ -1508,73 +1498,44 @@ class Case(Dico, XMLDocument, QObject):
 
 
     def undoStop(self):
-        self.record_local = True
+        """
+        Method to be overloaded for undo/redo in GUI.
+        """
+        return
 
 
     def undoStart(self):
-        self.record_local = False
+        """
+        Method to be overloaded for undo/redo in GUI.
+        """
+        return
 
 
     def undoStopGlobal(self):
-        self.record_global = False
-
+        """
+        Method to be overloaded for undo/redo in GUI.
+        """
+        return
 
     def undoStartGlobal(self):
-        self.record_global = True
+        """
+        Method to be overloaded for undo/redo in GUI.
+        """
+        return
 
 
     def undoGlobal(self, f, c):
-        if self['current_page'] != '' and self.record_local == False and self.record_global == True:
-            if sys.version[0] == '2':
-                self['dump_python'].append([f.__module__, f.func_name, c])
-            else:
-                self['dump_python'].append([f.__module__, f.__name__, c])
-            if self.xml_prev != self.toString() or self.xml_prev == "":
-                # control if function have same arguments
-                # last argument is value
-                same = True
-                if self.record_argument_prev == None:
-                    same = False
-                elif (len(c) == len(self.record_argument_prev) and len(c) >= 2):
-                    for i in range(0, len(c)-1):
-                        if c[i] != self.record_argument_prev[i]:
-                            same = False
-
-                if same:
-                    pass
-                else:
-                    self['undo'].append([self['current_page'], self.toString(), self['current_index'], self['current_tab']])
-                    self.xml_prev = self.toString()
-                    self.record_func_prev = None
-                    self.record_argument_prev = c
-                    self.undo_signal.emit()
+        """
+        Method to be overloaded for undo/redo in GUI.
+        """
+        return
 
 
     def undo(self, f, c):
-        if self['current_page'] != '' and self.record_local == False and self.record_global == True:
-            if sys.version[0] == '2':
-                self['dump_python'].append([f.__module__, f.func_name, c])
-            else:
-                self['dump_python'].append([f.__module__, f.__name__, c])
-            if self.xml_prev != self.toString():
-                # control if function have same arguments
-                # last argument is value
-                same = True
-                if self.record_argument_prev == None:
-                    same = False
-                elif (len(c) == len(self.record_argument_prev) and len(c) >= 2):
-                    for i in range(0, len(c)-1):
-                        if c[i] != self.record_argument_prev[i]:
-                            same = False
-
-                if self.record_func_prev == f and same:
-                    pass
-                else:
-                    self.record_func_prev = f
-                    self.record_argument_prev = c
-                    self['undo'].append([self['current_page'], self.toString(), self['current_index'], self['current_tab']])
-                    self.xml_prev = self.toString()
-                    self.undo_signal.emit()
+        """
+        Method to be overloaded for undo/redo in GUI.
+        """
+        return
 
 
 #-------------------------------------------------------------------------------
