@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -56,7 +56,7 @@ from code_saturne.Pages.CoalCombustionModel import CoalCombustionModel
 from code_saturne.Pages.ThermalScalarModel import ThermalScalarModel
 from code_saturne.Pages.ElectricalModel import ElectricalModel
 from code_saturne.Pages.GasCombustionModel import GasCombustionModel
-from code_saturne.Pages.DarcyModel import DarcyModel
+from code_saturne.Pages.GroundwaterModel import GroundwaterModel
 from code_saturne.Pages.AtmosphericFlowsModel import AtmosphericFlowsModel
 from code_saturne.Pages.LagrangianModel import LagrangianModel
 from code_saturne.Pages.ThermalRadiationModel import ThermalRadiationModel
@@ -92,27 +92,34 @@ class XMLinit(Variables):
 
             # Initialization (order is important, see turbulenceModelsList method)
 
+            grdflow = GroundwaterModel(self.case).getGroundwaterModel()
+
             self.node_models = self.case.xmlInitNode('thermophysical_models')
             node = self.node_models.xmlInitNode('velocity_pressure')
-            self.setNewVariable(node, 'pressure')
+            if grdflow == 'groundwater':
+                self.setNewVariable(node, 'hydraulic_head')
+            else:
+                self.setNewVariable(node, 'pressure')
             self.setNewVariable(node, 'velocity', dim = '3')
             self.setNewProperty(node, 'total_pressure')
-            n = self.setNewProperty(node, 'yplus')
-            n['support'] = 'boundary'
-            n['label'] = 'Yplus'
-            n = self.setNewProperty(node, 'stress')
-            n['support'] = 'boundary'
-            n['label'] = 'Stresss'
-            if not node.xmlGetChildNode('property', name='stress_tangential'):
-                n = self.setNewProperty(node, 'stress_tangential')
-                n['label'] = 'Stresss, tangential'
-                n['support'] = 'boundary'
-                n.xmlInitNode('postprocessing_recording')['status']= "off"
-            if not node.xmlGetChildNode('property', name='stress_normal'):
-                n = self.setNewProperty(node, 'stress_normal')
-                n['label'] = 'Stresss, normal'
-                n['support'] = 'boundary'
-                n.xmlInitNode('postprocessing_recording')['status']= "off"
+
+            if grdflow != 'groundwater':
+              n = self.setNewProperty(node, 'yplus')
+              n['support'] = 'boundary'
+              n['label'] = 'Yplus'
+              n = self.setNewProperty(node, 'stress')
+              n['support'] = 'boundary'
+              n['label'] = 'Stress'
+              if not node.xmlGetChildNode('property', name='stress_tangential'):
+                  n = self.setNewProperty(node, 'stress_tangential')
+                  n['label'] = 'Stress, tangential'
+                  n['support'] = 'boundary'
+                  n.xmlInitNode('postprocessing_recording')['status']= "off"
+              if not node.xmlGetChildNode('property', name='stress_normal'):
+                  n = self.setNewProperty(node, 'stress_normal')
+                  n['label'] = 'Stress, normal'
+                  n['support'] = 'boundary'
+                  n.xmlInitNode('postprocessing_recording')['status']= "off"
 
             MobileMeshModel(self.case).getMethod()
             TurbulenceModel(self.case).getTurbulenceModel()
@@ -153,9 +160,8 @@ class XMLinit(Variables):
             GasCombustionModel(self.case).getGasCombustionModel()
             ElectricalModel(self.case).getElectricalModel()
             ThermalRadiationModel(self.case).getRadiativeModel()
-            DarcyModel(self.case).getDarcyModel()
             AtmosphericFlowsModel(self.case).getAtmosphericFlowsModel()
-            LagrangianModel(self.case).getLagrangianStatus()
+            LagrangianModel(self.case).getLagrangianModel()
 
             return msg
 
@@ -194,32 +200,80 @@ class XMLinit(Variables):
         return msg
 
 
+    def __renameSingle(self, parent_tag, old_tag, new_tag):
+        """
+        Rename some nodes in order to ensure backward compatibility.
+        """
+
+        # renames
+
+        node = self.case.xmlGetNode(parent_tag)
+        if node:
+            oldnode = node.xmlGetNode(old_tag)
+            if oldnode:
+                newnode = node.xmlInitNode(new_tag)
+                newnode.xmlChildsCopy(oldnode)
+                oldnode.xmlRemoveNode()
+
+
+    def __clean_version(self, vers):
+        """
+        Simplify version history number, replacing "alpha" or "beta"
+        version with previous version  to force ensuring of backward
+        compatibility.
+        """
+        known_versions = ["3.0", "3.1", "3.2", "3.3",
+                          "4.0", "4.1", "4.2", "4.3",
+                          "5.0", "5.1"]
+        j = -2
+        for i in range(0, len(known_versions)):
+            if vers.find(known_versions[i]) == 0:
+                j = i
+                for e in ("-alpha", "-beta"):
+                    if vers.find(e) > -1:
+                        j = i-1
+                break
+        if j == -1:
+            if not vers:
+                vers = "-1.0"
+        elif j > 0:
+            vers = known_versions[j]
+        return vers
 
 
     def __backwardCompatibility(self):
         """
         Change XML in order to ensure backward compatibility.
         """
+        cur_vers = self.__clean_version(self.case['package'].version)
+
         if self.case.root()["solver_version"]:
-            vers = self.case.root()["solver_version"]
-            history = vers.split(";")
-            cur_vers = history[len(history) - 1]
-            if history[len(history) - 1] == self.case['package'].version:
+            his_r = self.case.root()["solver_version"]
+            history = his_r.split(";")
+            last_vers = history[len(history) - 1]
+            if last_vers == cur_vers:
                 self.__backwardCompatibilityCurrentVersion()
             else:
-                self.__backwardCompatibilityOldVersion(cur_vers)
+                self.__backwardCompatibilityOldVersion(last_vers)
                 self.__backwardCompatibilityCurrentVersion()
-                his = ""
-                for v in history:
-                    his = his + v + ";"
-                his = his + self.case['package'].version
+            his = ""
+            vp = ""
+            for v in history:
+                vc = self.__clean_version(v)
+                if vc != vp:
+                    his += vc + ";"
+                    vp = vc
+            if cur_vers != vp:
+                his += cur_vers + ";"
+            his = his[:-1]
+            if his != his_r:
                 self.case.root().xmlSetAttribute(solver_version = his)
 
         else:
-            vers = self.case['package'].version
+            vers = cur_vers
             self.case.root().xmlSetAttribute(solver_version = vers)
 
-            # apply all backwardCompatibility we don't know when it was create
+            # apply all backwardCompatibility we don't know when it was created
             self.__backwardCompatibilityOldVersion("-1")
             self.__backwardCompatibilityCurrentVersion()
 
@@ -228,33 +282,28 @@ class XMLinit(Variables):
         """
         Change XML in order to ensure backward compatibility for old version
         """
-        if from_vers == "-1":
+        if from_vers <= "-1.0":
             self.__backwardCompatibilityBefore_3_0()
-            self.__backwardCompatibilityFrom_3_0()
-            self.__backwardCompatibilityFrom_3_1()
-            self.__backwardCompatibilityFrom_3_2()
-            self.__backwardCompatibilityFrom_3_3()
-            self.__backwardCompatibilityFrom_4_0()
-        elif from_vers == "3.0":
-            self.__backwardCompatibilityFrom_3_0()
-            self.__backwardCompatibilityFrom_3_1()
-            self.__backwardCompatibilityFrom_3_2()
-            self.__backwardCompatibilityFrom_3_3()
-            self.__backwardCompatibilityFrom_4_0()
-        elif from_vers == "3.1":
-            self.__backwardCompatibilityFrom_3_1()
-            self.__backwardCompatibilityFrom_3_2()
-            self.__backwardCompatibilityFrom_3_3()
-            self.__backwardCompatibilityFrom_4_0()
-        elif from_vers == "3.2":
-            self.__backwardCompatibilityFrom_3_2()
-            self.__backwardCompatibilityFrom_3_3()
-            self.__backwardCompatibilityFrom_4_0()
-        elif from_vers == "3.3":
-            self.__backwardCompatibilityFrom_3_3()
-            self.__backwardCompatibilityFrom_4_0()
-        elif from_vers == "4.0":
-            self.__backwardCompatibilityFrom_4_0()
+
+        if from_vers[:3] < "4.0.0":
+            if from_vers[:3] < "3.1.0":
+                self.__backwardCompatibilityFrom_3_0()
+            if from_vers[:3] < "3.2.0":
+                self.__backwardCompatibilityFrom_3_1()
+            if from_vers[:3] < "3.3.0":
+                self.__backwardCompatibilityFrom_3_2()
+            if from_vers[:3] < "4.0.0":
+                self.__backwardCompatibilityFrom_3_3()
+
+        if from_vers[:3] < "5.0.0":
+            if from_vers[:3] < "4.1.0":
+                self.__backwardCompatibilityFrom_4_0()
+            if from_vers[:3] < "4.2.0":
+                self.__backwardCompatibilityFrom_4_1()
+            if from_vers[:3] < "4.3.0":
+                self.__backwardCompatibilityFrom_4_2()
+            if from_vers[:3] < "5.0.0":
+                self.__backwardCompatibilityFrom_4_3()
 
 
     def __backwardCompatibilityBefore_3_0(self):
@@ -513,7 +562,6 @@ class XMLinit(Variables):
                         content = re.sub(pattern, n['name'], content)
                         nf.xmlSetTextNode(content)
 
-
         # update velocity node
         nodeV = self.__XMLVelocityPressureNode.xmlGetNode('variable', name="velocity_U")
         if nodeV:
@@ -596,7 +644,7 @@ class XMLinit(Variables):
                     ("scalar",                       "user_"),
                     ("PotElecReal",                  "elec_pot_r"),
                     ("POT_EL_I",                     "elec_pot_i"),
-                    ("YM_ESL",                       "esl_fraction"),
+                    ("YM_ESL",                       "esl_fraction_"),
                     ("POT_VEC",                      "vec_potential_"),
                     ("Fra_MEL",                      "mixture_fraction"),
                     ("Var_FMe",                      "mixture_fraction_variance"),
@@ -615,6 +663,7 @@ class XMLinit(Variables):
                     ("component_R12",                "r12"),
                     ("component_R13",                "r13"),
                     ("component_R23",                "r23"),
+                    ("StressTensor",                 "rij"),
                     ("turb_k",                       "k"),
                     ("turb_eps",                     "epsilon"),
                     ("turb_phi",                     "phi"),
@@ -803,9 +852,9 @@ class XMLinit(Variables):
                     ("Bilan_O",                      "x_oxygen"),
                     ("Bilan_H",                      "x_hydrogen"),
                     ("PuisJoul",                     "joule_power"),
-                    ("Cour_re",                      "current_re_"),
-                    ("CouImag",                      "current_im_"),
-                    ("For_Lap",                      "laplace_force_"),
+                    ("Cour_re",                      "current_re"),
+                    ("CouImag",                      "current_im"),
+                    ("For_Lap",                      "laplace_force"),
                     ("Coef_Abso",                    "absorption_coeff"),
                     ("c_NO_HCN",                     "radiation_source"),
                     ("Sigma",                        "elec_sigma"),
@@ -1034,8 +1083,7 @@ class XMLinit(Variables):
                 lst[i].xmlRemoveNode()
 
 
-
-    def __backwardCompatibilityCurrentVersion(self):
+    def __backwardCompatibilityFrom_4_1(self):
         """
         Change XML in order to ensure backward compatibility.
         """
@@ -1085,8 +1133,12 @@ class XMLinit(Variables):
                     if s:
                         nn.xmlInitNode('postprocessing_recording')['status']= s
 
-        # renames
 
+    def __backwardCompatibilityFrom_4_2(self):
+        """
+        Change XML in order to ensure backward compatibility.
+        """
+        # renames
         node = self.case.xmlGetNode('calculation_management')
         if node:
             cmd = node.xmlGetString('valgrind')
@@ -1096,6 +1148,380 @@ class XMLinit(Variables):
                 if nn:
                     nn.xmlRemoveNode()
 
+        # darcy_model -> groundwater
+        XMLThermoPhysicalModelNode = self.case.xmlInitNode('thermophysical_models')
+        oldnode = XMLThermoPhysicalModelNode.xmlGetNode('darcy_model')
+        if oldnode:
+            mdl = oldnode['model']
+            newnode = XMLThermoPhysicalModelNode.xmlInitNode('groundwater_model', 'model')
+            newnode['model'] = mdl
+            newnode.xmlChildsCopy(oldnode)
+            oldnode.xmlRemoveNode()
+
+        node = XMLThermoPhysicalModelNode.xmlGetNode('groundwater_model')
+        if node:
+            n = node.xmlGetNode('diffusion')
+            if n:
+                mdl = n['model']
+                node.xmlInitNode('dispersion')['model']= mdl
+                n.xmlRemoveNode()
+            n = node.xmlGetNode('criterion')
+            if n:
+                n.xmlRemoveNode()
+
+        XMLDarcy = XMLThermoPhysicalModelNode.xmlGetNode('darcy')
+        if XMLDarcy:
+            XMLGround = XMLThermoPhysicalModelNode.xmlInitNode('groundwater')
+            XMLGround.xmlChildsCopy(XMLDarcy)
+            XMLDarcy.xmlRemoveNode()
+
+            nodelist = XMLGround.xmlGetNodeList('darcy_law')
+            for oldnode in nodelist:
+                mdl = oldnode['model']
+                zid = oldnode['zone_id']
+                newnode = XMLGround.xmlInitNode('groundwater_law', 'model')
+                newnode['model'] = mdl
+                newnode['zone_id'] = zid
+                newnode.xmlChildsCopy(oldnode)
+                oldnode.xmlRemoveNode()
+
+        XMLSolutionDomainNode = self.case.xmlInitNode('solution_domain')
+        self.__XMLVolumicConditionsNode = XMLSolutionDomainNode.xmlInitNode('volumic_conditions')
+        for node in self.__XMLVolumicConditionsNode.xmlGetNodeList('zone'):
+            law = node['darcy_law']
+            if law:
+                node['groundwater_law'] = law
+
+        # Profile titles
+        for node in self.case.xmlGetNodeList('profile'):
+            node.xmlDelAttribute('title')
+
+        # Lagrangian Model
+        stats_node = None
+        XMLLagrangianNode = self.case.xmlGetNode('lagrangian')
+        if XMLLagrangianNode:
+            stats_node = XMLLagrangianNode.xmlGetNode('statistics')
+        if stats_node:
+            v_node = stats_node.xmlGetNode('volume')
+            b_node = stats_node.xmlGetNode('boundary')
+            if v_node:
+                it_start = v_node.xmlGetInt('iteration_start_volume')
+                if it_start != None:
+                    stats_node.xmlSetData('iteration_start', it_start)
+                    v_node.xmlRemoveChild('iteration_start_volume')
+                it_start_st = v_node.xmlGetInt('iteration_steady_start_volume')
+                if it_start_st != None:
+                    stats_node.xmlSetData('iteration_steady_start', it_start_st)
+                    v_node.xmlRemoveChild('iteration_steady_start_volume')
+                threshold = v_node.xmlGetDouble('threshold_volume')
+                if threshold != None:
+                    stats_node.xmlSetData('threshold', threshold)
+                    v_node.xmlRemoveChild('threshold_volume')
+            if b_node:
+                for t in ['iteration_start_boundary', 'threshold_boundary']:
+                    n = b_node.xmlGetNode(t)
+                    if n:
+                        n.xmlRemoveNode()
+        if XMLLagrangianNode:
+            if XMLLagrangianNode['model'] == "on":
+                node = XMLLagrangianNode.xmlGetNode('coupling_mode')
+                if node:
+                    XMLLagrangianNode['model'] = node['model']
+                    node.xmlRemoveNode()
+        XMLBoundaryNode = self.case.xmlInitNode('boundary_conditions')
+        for nature in ('inlet', 'outlet', 'wall', 'symmetry',
+                       'free_inlet_outlet', 'groundwater'):
+            for node in XMLBoundaryNode.xmlGetNodeList(nature):
+                node['field_id'] = 'none'
+
+
+    def __backwardCompatibilityFrom_4_3(self):
+        """
+        Change XML in order to ensure backward compatibility.
+        """
+
+        for f_type in ['variable', 'property']:
+            for node in self.case.xmlGetNodeList(f_type):
+                n = node.xmlGetChildNode('probes')
+                if n:
+                    if node.xmlGetChildNode('probes_recording'):
+                        node.xmlRemoveChild('probes_recording')
+                    if n['choice'] == '0':
+                        node.xmlInitChildNode('probes_recording')['status'] = "off"
+                    n.xmlRemoveNode()
+
+        # rename pressure into hydraulic_head for ground water module
+        XMLThermoPhysicalModelNode = self.case.xmlGetNode('thermophysical_models')
+        XMLPhysicalModelGWNode = XMLThermoPhysicalModelNode.xmlGetNode('groundwater')
+        if XMLPhysicalModelGWNode:
+
+            for XMLGWLaw in XMLPhysicalModelGWNode.xmlGetNodeList('groundwater_law'):
+                if XMLGWLaw:
+                    for XMLGWScalar in XMLGWLaw.xmlGetNodeList('variable'):
+                        if XMLGWScalar:
+                            XMLGWScalarProp = XMLGWScalar.xmlGetNode('property')
+                            if XMLGWScalarProp:
+                                if XMLGWScalarProp.xmlGetNode('formula'):
+                                    XMLGWScalarProp.xmlRemoveChild('formula')
+                                XMLGWScalar.xmlRemoveChild('property')
+                            XMLGWLaw.xmlRemoveChild('variable')
+
+                            print("Warning: xml parameter file must be updated manually.")
+                            print("Settings of diffusivity and delay for the "
+                                  "defined scalars have been removed "
+                                  "to ensure a partial compatibility.")
+
+            XMLVelPrNode = XMLThermoPhysicalModelNode.xmlGetNode('velocity_pressure')
+            for node in XMLVelPrNode.xmlGetNodeList('variable', 'name'):
+                if node['name'] == 'pressure':
+                    node['name'] = 'hydraulic_head'
+                if node['label'] == 'Pressure':
+                    node['label'] = 'HydraulicHead'
+
+            XMLBoundaryNode = self.case.xmlGetNode('boundary_conditions')
+            XMLGWNodes = XMLBoundaryNode.xmlGetNodeList('groundwater')
+            for XMLGWNode in XMLGWNodes:
+                for node in (XMLGWNode.xmlGetNodeList('neumann') + XMLGWNode.xmlGetNodeList('dirichlet')):
+                    if node['name'] == 'pressure':
+                        node['name'] = 'hydraulic_head'
+
+        XMLPhysicalModelALE = XMLThermoPhysicalModelNode.xmlGetNode('ale_method')
+        if XMLPhysicalModelALE:
+            node = XMLPhysicalModelALE.xmlGetNode('external_coupling_post_synchronizationre')
+        if node:
+            node.xmlRemoveNode()
+
+        node = self.case.xmlGetNode('variable', name='vec_potential')
+        if node:
+            node['dim'] = '3'
+
+        for var in ['current_re', 'laplace_force', 'current_im']:
+            node = self.case.xmlGetNode('property', name=var)
+            if node:
+                node['dim'] = '3'
+
+        node = self.case.xmlGetNode('variable', name='vec_potential_01')
+        if node:
+            node.xmlRemoveNode()
+        node = self.case.xmlGetNode('variable', name='vec_potential_02')
+        if node:
+            node.xmlRemoveNode()
+        node = self.case.xmlGetNode('variable', name='vec_potential_03')
+        if node:
+            node.xmlRemoveNode()
+        node = self.case.xmlGetNode('property', name='current_re_1')
+        if node:
+            node.xmlRemoveNode()
+            nModels         = self.case.xmlGetNode('thermophysical_models')
+            self.node_joule = nModels.xmlInitNode('joule_effect',      'model')
+            self.setNewProperty(self.node_joule, 'current_re', dim = '3')
+            self.setNewProperty(self.node_joule, 'laplace_force', dim = '3')
+            self.setNewProperty(self.node_joule, 'electric_field', dim = '3')
+        node = self.case.xmlGetNode('property', name='current_re_2')
+        if node:
+            node.xmlRemoveNode()
+        node = self.case.xmlGetNode('property', name='current_re_3')
+        if node:
+            node.xmlRemoveNode()
+
+        rd_list = None
+        XMLBoundaryNode = self.case.xmlGetNode('boundary_conditions')
+        if XMLBoundaryNode:
+            rd_list = XMLBoundaryNode.xmlGetNodeList('radiative_data')
+        if rd_list:
+            for rd in rd_list:
+                node = rd.xmlGetNode('output_zone')
+                if node:
+                    node.xmlRemoveNode()
+
+        XMLThermoPhysicalModelNode = self.case.xmlGetNode('thermophysical_models')
+        node_joule = XMLThermoPhysicalModelNode.xmlGetNode('joule_effect', 'model')
+        if node_joule:
+            if node_joule['model'] != 'off':
+                node_gas = XMLThermoPhysicalModelNode.xmlGetNode('gas_combustion',
+                                                                 'model')
+                if node_gas:
+                    node = node_gas.xmlGetNode('data_file')
+                    if node:
+                        node.xmlRemoveNode()
+
+        # Update some postprocessing outputs
+
+        npr = XMLThermoPhysicalModelNode.xmlGetNode('thermal_scalar')
+        if npr:
+            if npr['model'] != 'off':
+                node = npr.xmlGetNode('property', name="input_thermal_flux")
+                node2 = npr.xmlGetNode('property', name="thermal_flux")
+                if node2 and node:
+                    node.xmlRemoveNode()
+                elif node:
+                    node['name'] = "thermal_flux"
+            if npr['model']:
+                ThermalScalarModel(self.case).setThermalModelOutputs(npr['model'])
+
+        npr = XMLThermoPhysicalModelNode.xmlGetNode('radiative_transfer')
+        if npr:
+            for name in ("wall_temp", "qrad_y", "qrad_z"):
+                node = npr.xmlGetNode('property', name=name)
+                if node:
+                    node.xmlRemoveNode()
+            node = npr.xmlGetNode('property', name="qrad_x")
+            if node:
+                node['name'] = "qrad"
+                node['label'] = "Qrad"
+
+        npr = XMLThermoPhysicalModelNode.xmlGetNode('velocity_pressure')
+        if npr:
+            node = npr.xmlGetNode('property', name="all_variables")
+            if node:
+                node.xmlRemoveNode()
+
+
+        # renames
+
+        self.__renameSingle('thermophysical_models', 'heads_losses', 'head_losses')
+        self.__renameSingle('solution_domain', 'extrude_meshes', 'extrusion')
+
+        # renumber boundary and volume zones if required
+
+        m = LocalizationModel('BoundaryZone', self.case)
+        m.renumberZones()
+
+        m = LocalizationModel('VolumicZone', self.case)
+        m.renumberZones()
+
+        # missing info from some old files
+
+        for node in self.case.xmlGetNodeList('time_average'):
+            if node:
+                if not node['name']:
+                    node['name'] = node['label']
+
+
+    def __backwardCompatibilityCurrentVersion(self):
+        """
+        Change XML in order to ensure backward compatibility.
+        """
+
+        XMLThermoPhysicalModelNode = self.case.xmlInitNode('thermophysical_models')
+
+        # fix name of thermal conductivity in radiative transfer node
+        npr = XMLThermoPhysicalModelNode.xmlGetNode('radiative_transfer')
+        if npr:
+            node = npr.xmlGetNode('property', name="thermal_conductivity")
+            if node:
+                node['name'] = "wall_thermal_conductivity"
+
+        nch = XMLThermoPhysicalModelNode.xmlGetNode('solid_fuels')
+        if nch:
+            if nch['model'] == 'homogeneous_fuel_moisture_lagr':
+                nch['model'] = 'homogeneous_fuel_moisture'
+
+        # fix name of Reynolds stress tensor
+        ntur = XMLThermoPhysicalModelNode.xmlGetNode('turbulence')
+        if ntur:
+            if ntur['model'] in  ('Rij-SSG', 'Rij-epsilon', 'Rij-EBRSM'):
+
+                node = ntur.xmlGetNode('variable', name="r11")
+                if node:
+                    node['label'] = "Rij"
+                    node['name']  = "rij"
+                    node['dimension'] = "6"
+
+                node = ntur.xmlGetNode('variable', name="r22")
+                if node:
+                    node.xmlRemoveNode()
+                node = ntur.xmlGetNode('variable', name="r33")
+                if node:
+                    node.xmlRemoveNode()
+                node = ntur.xmlGetNode('variable', name="r12")
+                if node:
+                    node.xmlRemoveNode()
+                node = ntur.xmlGetNode('variable', name="r23")
+                if node:
+                    node.xmlRemoveNode()
+                node = ntur.xmlGetNode('variable', name="r13")
+                if node:
+                    node.xmlRemoveNode()
+
+            # Update Time Averages using Rij if needed !
+            rij_lbls = ['r11', 'r22', 'r33', 'r12', 'r23', 'r13']
+            for node in self.case.xmlGetNodeList('time_average'):
+                if node:
+                    for var in node.xmlGetChildNodeList('var_prop'):
+                        varName = var['name']
+                        if varName in rij_lbls:
+                            var['name'] = 'rij'
+                            var['component'] = rij_lbls.index(varName)
+
+            # Update profiles using Rij
+            for node in self.case.xmlGetNodeList('profile'):
+                if node:
+                    for var in node.xmlGetChildNodeList('var_prop'):
+                        varName = var['name']
+                        if varName in rij_lbls:
+                            var['name'] = 'rij'
+                            var['component'] = rij_lbls.index(varName)
+
+
+            # Lagrangian model setup renames and cleanup
+            XMLBoundaryNode = self.case.xmlGetNode('boundary_conditions')
+            XMLInletNodes = XMLBoundaryNode.xmlGetNodeList('inlet')
+            for XMLInletNode in XMLInletNodes:
+                XMLParticeNodes = XMLInletNode.xmlGetNodeList('particles')
+                for XMLParticleNode in XMLParticeNodes:
+                    XMLClassNodes = XMLInletNode.xmlGetNodeList('class')
+                    for XMLClassNode in XMLClassNodes:
+                        node = XMLClassNode.xmlGetNode('velocity', choice='subroutine')
+                        if node:
+                            node['choice'] = 'components'
+                        else:
+                            node = XMLClassNode.xmlGetNode('velocity', choice='components')
+                            if node:
+                                for d in (('u', 'x'), ('v', 'y'), ('w', 'z')):
+                                    onode = node.xmlGetNode('velocity_' + d[0])
+                                    if onode:
+                                        nnode = node.xmlInitNode('velocity_' + d[1])
+                                        nnode.xmlChildsCopy(onode)
+                                        onode.xmlRemoveNode()
+                        if node:
+                            node['choice'] = 'components'
+                        for attr in ('statistical_weight', 'temperature'):
+                            node = XMLClassNode.xmlGetNode(attr, choice='subroutine')
+                            if node:
+                                node['choice'] = 'prescribed'
+                        node = XMLClassNode.xmlGetNode('diameter')
+                        if node:
+                            node.xmlDelAttribute('choice')
+                        for attr in ('coal_composition', 'statitical_weight'):
+                            node = XMLClassNode.xmlGetNode(attr)
+                            if node:
+                                node.xmlRemoveNode()
+
+            XMLLagrangianNode = self.case.xmlGetNode('lagrangian')
+            if XMLLagrangianNode:
+                for attr in ('continuous_injection', 'particles_max_number'):
+                    node = XMLLagrangianNode.xmlGetNode(attr)
+                    if node:
+                        node.xmlRemoveNode()
+                stats_node = XMLLagrangianNode.xmlGetNode('statistics')
+                if stats_node:
+                    vol_node = stats_node.xmlGetNode('volume')
+                    if vol_node:
+                        for attr in ('Part_velocity_X', 'Part_velocity_Y',
+                                     'Part_velocity_Z'):
+                            node = vol_node.xmlGetNode('property', name=attr)
+                            if node:
+                                node.xmlRemoveNode()
+                output_node = XMLLagrangianNode.xmlGetNode('output')
+                if output_node:
+                    for attr in ('trajectory', 'particles', 'number_of_particles',
+                                 'postprocessing_format', 'postprocessing_options',
+                                 'postprocessing_frequency'):
+                        node = output_node.xmlGetNode(attr)
+                        if node:
+                            node.xmlRemoveNode()
+        return
 
 #-------------------------------------------------------------------------------
 # XMLinit test case
@@ -1171,23 +1597,15 @@ class XMLinitTestCase(unittest.TestCase):
         '<physical_properties>'\
                 '<fluid_properties>'\
                         '<property choice="constant" label="Density" name="density">'\
-                                '<listing_printing status="off"/>'\
-                                '<postprocessing_recording status="off"/>'\
                                 '<initial_value>1.17862</initial_value>'\
                         '</property>'\
                         '<property choice="constant" label="LamVisc" name="molecular_viscosity">'\
-                                '<listing_printing status="off"/>'\
-                                '<postprocessing_recording status="off"/>'\
                                 '<initial_value>1.83e-05</initial_value>'\
                         '</property>'\
                         '<property choice="constant" label="SpecHeat" name="specific_heat">'\
-                                '<listing_printing status="off"/>'\
-                                '<postprocessing_recording status="off"/>'\
                                 '<initial_value>1017.24</initial_value>'\
                         '</property>'\
                         '<property choice="constant" label="ThermalCond" name="thermal_conductivity">'\
-                                '<listing_printing status="off"/>'\
-                                '<postprocessing_recording status="off"/>'\
                                 '<initial_value>0.02495</initial_value>'\
                         '</property>'\
                 '</fluid_properties>'\
