@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -36,7 +36,7 @@ This module contains the following classes and function:
 #-------------------------------------------------------------------------------
 
 import os, sys
-import string, types
+import types
 import re
 import logging
 import subprocess
@@ -313,6 +313,7 @@ class ListingDialogView(CommandMgrDialogView):
 
         self.pushButtonStop.clicked.connect(self.__slotStop)
         self.pushButtonStopAt.clicked.connect(self.__slotStopAt)
+        self.pushButtonCvgTool.clicked.connect(self.__slotCvgTool)
 
         self.scratch_dir = ""
         self.result_dir = ""
@@ -412,6 +413,26 @@ class ListingDialogView(CommandMgrDialogView):
             self.__stopExec(result['iter'], msg)
 
 
+    @pyqtSlot()
+    def __slotCvgTool(self):
+        """
+        Private slot. Running convergence tracking tool
+        """
+        import subprocess
+
+        cmd = [self.case['package'].config.python]
+        cmd.append(os.path.join(self.case['package'].dirs['bindir'][1],
+                                self.case['package'].name))
+        cmd.append("trackcvg")
+        if self.scratch_dir:
+            cmd.append("-r")
+            cmd.append(self.scratch_dir)
+        elif self.result_dir:
+            cmd.append("-r")
+            cmd.append(self.result_dir)
+        self.runProcess = subprocess.Popen(cmd)
+
+
     @pyqtSlot("QProcess::ProcessState")
     def slotStateChanged(self, state):
         """
@@ -428,11 +449,9 @@ class ListingDialogView(CommandMgrDialogView):
 
 class BatchRunningView(QWidget, Ui_BatchRunningForm):
     """
-    This class is devoted to the Computer selection.
-    When a new computer is selected, The old data frame is deleted and
-    a new apropriate frame is open.
-    If the batch script file name is known, informations are display
-    in the apropiate widget.
+    This class is devoted to the queue selection.
+    If the batch script file name is known, informations are displayed
+    in the appropiate widget.
     """
     def __init__(self, parent, case):
         """
@@ -460,6 +479,8 @@ class BatchRunningView(QWidget, Ui_BatchRunningForm):
                     self.case['runcase'] = cs_runcase.runcase(runcase_path,
                                                               package=self.case['package'])
 
+        self.jmdl = BatchRunningModel(parent, self.case)
+
         # Get MPI and OpenMP features
 
         self.have_mpi = False
@@ -469,18 +490,28 @@ class BatchRunningView(QWidget, Ui_BatchRunningForm):
         config = configparser.ConfigParser()
         config.read(self.case['package'].get_configfiles())
 
+        compute_build_id = -1
+        self.compute_versions = None
         if config.has_option('install', 'compute_versions'):
-            compute_versions = config.get('install', 'compute_versions').split(':')
-            if compute_versions[0]:
-                pkg_compute = self.case['package'].get_alternate_version(compute_versions[0])
-                config_features = pkg_compute.config.features
+            self.compute_versions = config.get('install', 'compute_versions').split(':')
+            compute_build_id = 0
+            if len(self.compute_versions) > 1:
+                run_build = self.jmdl.dictValues['run_build']
+                if self.compute_versions.count(run_build) > 0:
+                    compute_build_id = self.compute_versions.index(run_build)
+                elif run_build:
+                    compute_build_id = -1
+                    self.jmdl.dictValues['run_build'] = None
+                    self.jmdl.updateBatchFile('run_build')
+
+        if compute_build_id >= 0:
+            pkg_compute = self.case['package'].get_alternate_version(self.compute_versions[compute_build_id])
+            config_features = pkg_compute.config.features
         if config_features['mpi'] == 'yes':
             self.have_mpi = True
         if config_features['openmp'] == 'yes':
             self.have_openmp = True
 
-        self.jmdl = BatchRunningModel(parent, self.case)
-
         # Batch info
 
         self.hideBatchInfo()
@@ -491,14 +522,23 @@ class BatchRunningView(QWidget, Ui_BatchRunningForm):
         self.labelNThreads.hide()
         self.spinBoxNThreads.hide()
 
+        if compute_build_id < 0:
+            self.labelBuildType.hide()
+            self.comboBoxBuildType.hide()
+        else:
+            self.comboBoxBuildType.currentIndexChanged[int].connect(self.slotBuildType)
+            for b in self.compute_versions:
+                build_type_label = os.path.basename(b)
+                if not build_type_label:
+                    build_type_label = "[default]"
+                self.comboBoxBuildType.addItem(self.tr(build_type_label),
+                                               to_qvariant(build_type_label))
+            self.comboBoxBuildType.setCurrentIndex(compute_build_id)
+
         self.class_list = None
 
         if self.jmdl.batch.rm_type != None:
 
-            self.groupBoxArchi.setTitle("Job and script files")
-            self.labelBatch.show()
-            self.toolButtonSearchBatch.show()
-
             validatorSimpleName = RegExpValidator(self.lineEditJobName,
                                                   QRegExp("[_A-Za-z0-9]*"))
             validatorAccountName = RegExpValidator(self.lineEditJobAccount,
@@ -519,7 +559,9 @@ class BatchRunningView(QWidget, Ui_BatchRunningForm):
                     self.mdl.setString('n_procs', None)
                 except Exception:
                     pass
-
+        validatorRunId = RegExpValidator(self.lineEditRunId,
+                                         QRegExp("[_A-Za-z0-9]*"))
+        self.lineEditRunId.setValidator(validatorRunId)
 
         # Connections
 
@@ -541,10 +583,10 @@ class BatchRunningView(QWidget, Ui_BatchRunningForm):
             self.spinBoxNProcs.valueChanged[int].connect(self.slotNProcs)
             self.spinBoxNThreads.valueChanged[int].connect(self.slotNThreads)
 
-        self.toolButtonSearchBatch.clicked.connect(self.slotSearchBatchFile)
         self.comboBoxRunType.activated[str].connect(self.slotArgRunType)
         self.toolButtonAdvanced.clicked.connect(self.slotAdvancedOptions)
         self.pushButtonRunSubmit.clicked.connect(self.slotBatchRunning)
+        self.lineEditRunId.textChanged[str].connect(self.slotJobRunId)
 
         # Combomodels
 
@@ -553,27 +595,31 @@ class BatchRunningView(QWidget, Ui_BatchRunningForm):
         self.modelArg_cs_verif.addItem(self.tr("Import mesh only"), 'none')
         self.modelArg_cs_verif.addItem(self.tr("Mesh preprocessing"), 'mesh preprocess')
         self.modelArg_cs_verif.addItem(self.tr("Mesh quality criteria"), 'mesh quality')
-        self.modelArg_cs_verif.addItem(self.tr("Standard"), 'standard')
-        if self.case['prepro'] == False:
-            self.modelArg_cs_verif.enableItem(3)
+        if self.case['prepro'] == True:
+            self.modelArg_cs_verif.setItem(str_model=self.mdl.getRunType(self.case['prepro']))
+            self.labelRunType.show()
+            self.comboBoxRunType.show()
         else:
-            self.modelArg_cs_verif.disableItem(3)
-        self.modelArg_cs_verif.setItem(str_model=self.mdl.getRunType(self.case['prepro']))
+            self.labelRunType.hide()
+            self.comboBoxRunType.hide()
 
         # initialize Widgets
 
-        # Check if the script file name is already defined
-
-        if self.case['runcase']:
-            name = os.path.basename(self.case['runcase'].path)
-            self.labelBatchName.setText(str(name))
-            self.toolButtonSearchBatch.setStyleSheet("background-color: green")
-        else:
-            self.toolButtonSearchBatch.setStyleSheet("background-color: red")
-
         if self.jmdl.batch.rm_type != None and self.case['runcase']:
             self.displayBatchInfo()
 
+        #rm_type = self.jmdl.batch.rm_type
+        run_id = str(self.jmdl.dictValues['run_id'])
+
+        #if run_id == 'None' and self.case['scripts_path']:
+        #    run_id, run_title = self.__suggest_run_id()
+        #    self.__updateRuncase(run_id)
+
+        if run_id != 'None':
+            self.lineEditRunId.setText(run_id)
+        else:
+            self.lineEditRunId.setText("")
+
         # Script info is based on the XML model
 
         self.displayScriptInfo()
@@ -596,8 +642,13 @@ class BatchRunningView(QWidget, Ui_BatchRunningForm):
         """
         Increment, decrement and colorize the input argument entry
         """
+        n = int(self.spinBoxNodes.text())
         self.jmdl.batch.params['job_nodes'] = str(self.spinBoxNodes.text())
         self.jmdl.batch.update_lines(self.case['runcase'].lines, 'job_nodes')
+        if self.job_ppn:
+            ppn = int(self.jmdl.batch.params['job_ppn'])
+            tot_ranks = n*ppn
+            self.lineEditTotMPI.setText(str(tot_ranks))
 
 
     @pyqtSlot(int)
@@ -605,8 +656,17 @@ class BatchRunningView(QWidget, Ui_BatchRunningForm):
         """
         Increment, decrement and colorize the input argument entry
         """
-        self.jmdl.batch.params['job_ppn']  = str(self.spinBoxPpn.text())
+        ppn = int(self.spinBoxPpn.text())
+        self.jmdl.batch.params['job_ppn']  = str(ppn)
         self.jmdl.batch.update_lines(self.case['runcase'].lines, 'job_ppn')
+        if self.job_nodes:
+            n = int(self.jmdl.batch.params['job_nodes'])
+            tot_ranks = n*ppn
+            self.lineEditTotMPI.setText(str(tot_ranks))
+        if self.job_threads:
+            n_threads = int(self.jmdl.batch.params['job_threads'])
+            node_threads = n_threads*ppn
+            self.lineEditThreadsPerNode.setText(str(node_threads))
 
 
     @pyqtSlot(int)
@@ -623,8 +683,13 @@ class BatchRunningView(QWidget, Ui_BatchRunningForm):
         """
         Increment, decrement and colorize the input argument entry
         """
-        self.jmdl.batch.params['job_threads']  = str(self.spinBoxThreads.text())
+        n_threads = int(self.spinBoxThreads.text())
+        self.jmdl.batch.params['job_threads']  = str(n_threads)
         self.jmdl.batch.update_lines(self.case['runcase'].lines, 'job_threads')
+        if self.job_ppn:
+            ppn = int(self.jmdl.batch.params['job_ppn'])
+            node_threads = n_threads*ppn
+            self.lineEditThreadsPerNode.setText(str(node_threads))
 
 
     @pyqtSlot()
@@ -698,6 +763,48 @@ class BatchRunningView(QWidget, Ui_BatchRunningForm):
         self.jmdl.updateBatchFile('run_nthreads')
 
 
+    @pyqtSlot(int)
+    def slotBuildType(self, v):
+
+        if v == 0:
+            self.jmdl.dictValues['run_build'] = None
+        else:
+            self.jmdl.dictValues['run_build'] = str(self.compute_versions[v])
+        self.jmdl.updateBatchFile('run_build')
+        compute_build_id = v
+
+        pkg_compute = self.case['package'].get_alternate_version(self.compute_versions[compute_build_id])
+        config_features = pkg_compute.config.features
+        if config_features['mpi'] == 'yes':
+            self.have_mpi = True
+        else:
+            self.have_mpi = False
+            self.spinBoxNProcs.setValue(1)
+            self.jmdl.dictValues['run_nprocs'] = None
+            self.jmdl.updateBatchFile('run_nprocs')
+
+        if config_features['openmp'] == 'yes':
+            self.have_openmp = True
+        else:
+            self.have_openmp = False
+            self.spinBoxNThreads.setValue(1)
+            self.jmdl.dictValues['run_nthreads'] = None
+            self.jmdl.updateBatchFile('run_nthreads')
+
+        self.displayScriptInfo()
+
+
+    @pyqtSlot(str)
+    def slotJobRunId(self, v):
+        """
+        """
+        if self.lineEditRunId.validator().state == QValidator.Acceptable:
+            self.jmdl.dictValues['run_id'] = str(v)
+            self.jmdl.updateBatchFile('run_id')
+            if v != 'None':
+                self.__updateRuncase(str(v))
+
+
     @pyqtSlot()
     def slotAdvancedOptions(self):
         """
@@ -718,7 +825,14 @@ class BatchRunningView(QWidget, Ui_BatchRunningForm):
         """
         # Is the file saved?
 
-        if self.case['new'] == "yes" or len(self.case['undo']) > 0 or len(self.case['redo']) > 0:
+        xml_current = os.path.basename(self.case['xmlfile'])
+        xml_param = None
+        if self.case['runcase']:
+            xml_param = self.case['runcase'].get_parameters()
+        if not xml_current or xml_current != xml_param:
+            self.case['saved'] = "no"
+
+        if self.case['saved'] == "no" or len(self.case['undo']) > 0 or len(self.case['redo']) > 0:
 
             title = self.tr("Warning")
             msg   = self.tr("The current case must be saved before "\
@@ -761,12 +875,18 @@ class BatchRunningView(QWidget, Ui_BatchRunningForm):
         rm_type = self.jmdl.batch.rm_type
 
         batch = self.case['runcase'].path
+
         cmd = None
         run_title = None
 
+        run_id = self.jmdl.dictValues['run_id']
+
         if rm_type == None:
-            run_id, run_title = self.__suggest_run_id()
-            self.__updateRuncase(run_id)
+            if not run_id:
+                tmp_run_id, run_title = self.__suggest_run_id()
+                self.__updateRuncase(tmp_run_id)
+            else:
+                run_title = self.case['package'].code_name + run_id
             cmd = cs_exec_environment.enquote_arg(batch)
         else:
             run_title = self.case['package'].code_name + ' - Job Submission'
@@ -776,7 +896,7 @@ class BatchRunningView(QWidget, Ui_BatchRunningForm):
         dlg = ListingDialogView(self.parent, self.case, run_title, cmd)
         dlg.show()
 
-        if rm_type == None:
+        if rm_type == None and not run_id:
             self.__updateRuncase('')  # remove --id <id> from runcase
 
         os.chdir(prv_dir)
@@ -789,8 +909,9 @@ class BatchRunningView(QWidget, Ui_BatchRunningForm):
         cmd = os.path.join(self.case['package'].get_dir('bindir'),
                            self.case['package'].name)
         cmd = cs_exec_environment.enquote_arg(cmd) + " run --suggest-id"
-        if self.mdl.getRunType() != "standard":
-            cmd += " --id-prefix=preprocess"
+        xmlfile = self.case['xmlfile']
+        if xmlfile:
+            cmd += " --param " + cs_exec_environment.enquote_arg(xmlfile)
 
         r_title = subprocess.Popen(cmd,
                                    shell=True,
@@ -831,6 +952,10 @@ class BatchRunningView(QWidget, Ui_BatchRunningForm):
         self.spinBoxProcs.hide()
         self.labelThreads.hide()
         self.spinBoxThreads.hide()
+        self.labelTotMPI.hide()
+        self.lineEditTotMPI.hide()
+        self.labelThreadsPerNode.hide()
+        self.lineEditThreadsPerNode.hide()
         self.labelClass.hide()
         self.labelWTime.hide()
         self.spinBoxDays.hide()
@@ -882,11 +1007,21 @@ class BatchRunningView(QWidget, Ui_BatchRunningForm):
             self.labelProcs.show()
             self.spinBoxProcs.setValue(int(self.job_procs))
             self.spinBoxProcs.show()
+        elif self.job_nodes != None and self.job_ppn != None:
+            self.labelTotMPI.show()
+            self.lineEditTotMPI.show()
+            tot_ranks = int(self.job_nodes)*int(self.job_ppn)
+            self.lineEditTotMPI.setText(str(tot_ranks))
 
         if self.job_threads != None:
             self.labelThreads.show()
             self.spinBoxThreads.setValue(int(self.job_threads))
             self.spinBoxThreads.show()
+            if self.job_ppn != None:
+                self.labelThreadsPerNode.show()
+                self.lineEditThreadsPerNode.show()
+                th_per_node = int(self.job_threads)*int(self.job_ppn)
+                self.lineEditThreadsPerNode.setText(str(th_per_node))
 
         if self.job_walltime != None:
             seconds = self.job_walltime
@@ -975,21 +1110,26 @@ class BatchRunningView(QWidget, Ui_BatchRunningForm):
         """
         Layout of the second part of this page.
         """
-
-        if self.case['batch_type'] == None:
+        if self.jmdl.batch.rm_type == None:
             if self.have_mpi:
                 self.labelNProcs.show()
                 self.spinBoxNProcs.show()
+            else:
+                self.labelNProcs.hide()
+                self.spinBoxNProcs.hide()
             if self.have_openmp:
                 self.labelNThreads.show()
                 self.spinBoxNThreads.show()
+            else:
+                self.labelNThreads.hide()
+                self.spinBoxNThreads.hide()
         else:
             self.labelNProcs.hide()
             self.spinBoxNProcs.hide()
             self.labelNThreads.hide()
             self.spinBoxNThreads.hide()
 
-        if self.case['batch_type'] == None:
+        if self.jmdl.batch.rm_type == None:
             if self.have_mpi:
                 n_procs_s = self.jmdl.dictValues['run_nprocs']
                 if n_procs_s:
@@ -997,7 +1137,7 @@ class BatchRunningView(QWidget, Ui_BatchRunningForm):
                 else:
                     n_procs = 1
                 self.spinBoxNProcs.setValue(n_procs)
-            if self.have_openmp == 'yes':
+            if self.have_openmp:
                 n_threads_s = self.jmdl.dictValues['run_nthreads']
                 if n_threads_s:
                     n_threads = int(n_threads_s)
@@ -1008,45 +1148,6 @@ class BatchRunningView(QWidget, Ui_BatchRunningForm):
             pass
 
 
-    @pyqtSlot()
-    def slotSearchBatchFile(self):
-        """
-        Open a FileDialog in order to search the batch command file
-        in the system file.
-        """
-        file_name = ""
-        if self.case['scripts_path'] and os.path.isdir(self.case['scripts_path']):
-            path = self.case['scripts_path']
-        else:
-            path = os.getcwd()
-        title = self.tr("Select the batch script")
-        filetypes = self.tr("All Files (*)")
-        file_name = QFileDialog.getOpenFileName(self, title, path, filetypes)[0]
-        file_name = str(file_name)
-
-        if file_name:
-
-            launcher = os.path.basename(file_name)
-            self.toolButtonSearchBatch.setStyleSheet("background-color: green")
-
-            if self.case['scripts_path'] == os.path.dirname(file_name):
-                self.case['runcase'] = cs_runcase.runcase(os.path.join(self.case['scripts_path'],
-                                                                       launcher),
-                                                          package=self.case['package'])
-                self.labelBatchName.setText(str(launcher))
-                self.hideBatchInfo()
-                if self.jmdl.batch.rm_type != None:
-                    self.displayBatchInfo()
-            else:
-                title = self.tr("Warning")
-                msg   = self.tr("The new batch file is not in scripts "\
-                                "directory given in the 'Identity and paths' "\
-                                "section.\n\n" + \
-                                "Verify the existence and location of these files, "\
-                                "and the 'Identity and Pathes' section")
-                QMessageBox.warning(self, title, msg)
-
-
     def tr(self, text):
         """
         Translation
