@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -33,7 +33,6 @@ This module defines the following classes:
 # Library modules import
 #-------------------------------------------------------------------------------
 
-import string
 import unittest
 
 #-------------------------------------------------------------------------------
@@ -67,7 +66,9 @@ class DefineUserScalarsModel(Variables, Model):
         self.node_therm  = self.node_models.xmlGetNode('thermal_scalar')
         self.node_source = self.node_models.xmlGetNode('source_terms')
         self.node_bc     = self.case.xmlGetNode('boundary_conditions')
-
+        self.node_ana    = self.case.xmlInitNode('analysis_control')
+        self.node_prof   = self.node_ana.xmlInitNode('profiles')
+        self.node_ava    = self.node_ana.xmlInitNode('time_averages')
 
     def defaultScalarValues(self):
         """Return the default values - Method also used by ThermalScalarModel"""
@@ -84,10 +85,13 @@ class DefineUserScalarsModel(Variables, Model):
         else:
             default['GGDH']                  = "SGDH"
         del GroundwaterModel
-        if self.getScalarNameList():
-            default['variance']          = self.getScalarNameList()[0]
+        if self.getThermalScalarName():
+            default['variance']          = self.getThermalScalarName()[0]
         else:
-            default['variance']          = "no scalar"
+            if self.getScalarNameList():
+                default['variance']          = self.getScalarNameList()[0]
+            else:
+                default['variance']          = "no scalar"
 
         return default
 
@@ -227,6 +231,12 @@ class DefineUserScalarsModel(Variables, Model):
         node.xmlRemoveNode()
         self.__deleteScalarBoundaryConditions(name)
         self.__updateScalarNameAndDiffusivityName()
+        # delete scalar in profiles and time averages nodes
+        for node in self.node_prof.xmlGetNodeList('profile'):
+            node.xmlRemoveChild('var_prop', name=name)
+        for node in self.node_ava.xmlGetNodeList('time_average'):
+            node.xmlRemoveChild('var_prop', name=name)
+
 
 
     @Variables.noUndo
@@ -345,7 +355,7 @@ class DefineUserScalarsModel(Variables, Model):
         l= self.__defaultVarianceName(name)
         if l not in self.getScalarsVarianceList():
             self.scalar_node.xmlInitNode('variable', name=l, type="user", label=l)
-            if self.getScalarNameList() != None:
+            if (self.getThermalScalarName() != None) or (self.getScalarNameList() != None):
                 self.setScalarVariance(l, self.defaultScalarValues()['variance'])
 
         self.__updateScalarNameAndDiffusivityName()
@@ -684,7 +694,7 @@ class DefineUserScalarsModel(Variables, Model):
                 if node.xmlGetString('name') == sname:
                    node.xmlRemoveNode()
 
-        # Delete all scalars
+        # Delete scalars in list
         for scalar in lst:
             self.__deleteScalar(scalar)
 
