@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -159,19 +159,20 @@ class ElectricalModel(Variables, Model):
             self.__removeVariablesAndProperties([], [])
         else:
             listV = ['elec_pot_r']
+            listV3 = []
             gasN = self.getGasNumber()
             if gasN > 1:
                 for gas in range(0, gasN - 1):
                     name = '%s%2.2i' % ('esl_fraction_', gas + 1)
                     listV.append(name)
 
-            listP = ['temperature', 'joule_power', 'elec_sigma', 'current_re']
+            listP = ['temperature', 'joule_power', 'elec_sigma']
+            listP3 = ['current_re', 'electric_field']
 
             if model == 'arc':
-                listV.append('vec_potential_01')
-                listV.append('vec_potential_02')
-                listV.append('vec_potential_03')
-                listP.append('laplace_force')
+                listV3.append('vec_potential')
+                listP3.append('laplace_force')
+                listP3.append('magnetic_field')
                 if self.getRadiativeModel() == 'Coef_Abso':
                     listP.append('absorption_coeff')
                 elif self.getRadiativeModel() == 'TS_radia':
@@ -182,12 +183,18 @@ class ElectricalModel(Variables, Model):
                 if model == 'PotComplexe' or model == 'PotComplexe+CDLTransfo':
                     listV.append('elec_pot_i')
                 if model == 'PotComplexe+CDLTransfo':
-                    listP.append('current_im')
+                    listP3.append('current_im')
 
             for v in listV:
                 self.setNewVariable(self.node_joule, v, tpe="model", label=v)
             for v in listP:
                 self.setNewProperty(self.node_joule, v)
+            for v in listV3:
+                self.setNewVariable(self.node_joule, v, dim = '3', tpe="model", label=v)
+                listV.append(v)
+            for v in listP3:
+                self.setNewProperty(self.node_joule, v, dim = '3')
+                listP.append(v)
             self.__removeVariablesAndProperties(listV, listP)
 
 
@@ -382,24 +389,6 @@ class ElectricalModel(Variables, Model):
         node['status'] = status
 
 
-    @Variables.noUndo
-    def getPropertiesDataFileName(self):
-        """
-        Get name for properties data (return None if not defined)i
-        """
-        f = self.node_gas.xmlGetString('data_file')
-        return f
-
-
-    @Variables.undoGlobal
-    def setPropertiesDataFileName(self, name):
-        """
-        Put name for properties data and load file for number gaz and radiative model
-        """
-        self.node_gas.xmlSetData('data_file', name)
-        self.load(name)
-
-
     @Variables.noUndo
     def getScalingModel(self):
         """
