@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -312,7 +312,7 @@ class StandardItemModelCoals(QStandardItemModel):
         self.case = case
         self.model = model
 
-        self.coalModel = CoalCombustionModel.CoalCombustionModel(self.case)
+        self.coalModel = CoalCombustionModel(self.case)
         CoalsNumber = self.coalModel.getCoalNumber()
 
         for icoal in range(CoalsNumber):
