@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -57,7 +57,8 @@ class Zone(object):
     """
     Zone API
     """
-    def __new__(cls, typeZone, case = None, label = None, codeNumber = None, localization = None, nature = None):
+    def __new__(cls, typeZone, case = None, label = None, codeNumber = None,
+                localization = None, nature = None):
         """
         Factory
         """
@@ -69,7 +70,8 @@ class Zone(object):
             raise ValueError("Unknown type zone")
 
 
-    def __init__(self, typeZone, case = None, label = None, codeNumber = None, localization = None, nature = None):
+    def __init__(self, typeZone, case = None, label = None, codeNumber = None,
+                 localization = None, nature = None):
         """
         """
         self.case = case
@@ -102,6 +104,7 @@ class Zone(object):
         self._natureDict = {}
         self.case = case
 
+
     def setLabel(self, text):
         if Model().isStr(text):
             self._label = text
@@ -178,12 +181,15 @@ class BoundaryZone(Zone):
 
         if case != None:
             if self.case['package'].name == 'code_saturne':
-                self._natureList = ['wall', 'inlet', 'outlet', 'symmetry', 'free_inlet_outlet']
+                self._natureList = ['wall', 'inlet', 'outlet', 'symmetry', 'free_inlet_outlet', \
+                                    'imposed_p_outlet']
                 self._natureDict['free_inlet_outlet'] = self.tr("Free inlet/outlet")
+                self._natureDict['imposed_p_outlet'] = self.tr("Imposed P Outlet")
                 from code_saturne.Pages.MobileMeshModel import MobileMeshModel
                 if MobileMeshModel(self.case).getMethod() != "off":
                     self._natureDict['free_surface'] = self.tr("Free surface")
-                    self._natureList = ['wall', 'inlet', 'outlet', 'symmetry', 'free_inlet_outlet', 'free_surface']
+                    self._natureList = ['wall', 'inlet', 'outlet', 'symmetry', 'free_inlet_outlet', \
+                                        'free_surface', 'imposed_p_outlet']
                 del MobileMeshModel
                 from code_saturne.Pages.GroundwaterModel import GroundwaterModel
                 if GroundwaterModel(self.case).getGroundwaterModel() != "off":
@@ -194,8 +200,10 @@ class BoundaryZone(Zone):
             else:
                 self._natureList = ['wall', 'inlet', 'outlet', 'symmetry']
         else:
-            self._natureList = ['wall', 'inlet', 'outlet', 'symmetry', 'free_inlet_outlet']
+            self._natureList = ['wall', 'inlet', 'outlet', 'symmetry', 'free_inlet_outlet', \
+                                'imposed_p_outlet']
             self._natureDict['free_inlet_outlet'] = self.tr("Free inlet/outlet")
+            self._natureDict['imposed_p_outlet'] = self.tr("Imposed P Outlet")
 
 
     def defaultValues(self):
@@ -243,7 +251,7 @@ class VolumicZone(Zone):
                 self._natureDict['momentum_source_term'] = self.tr("Momentum source\n term")
             else:
                 self._natureList.append('momentum_source_term')
-                self._natureDict['momentum_source_term'] = self.tr("Volumetric source\n term")
+                self._natureDict['momentum_source_term'] = self.tr("Volumic source\n term")
             del GroundwaterModel
 
             from code_saturne.Pages.ThermalScalarModel import ThermalScalarModel
@@ -494,6 +502,7 @@ class LocalizationModel(object):
         """
         Replace a zone by another in the XML file
         """
+
         newLabel = new_zone.getLabel()
         if newLabel == new_zone.defaultValues()['label']:
             newLabel = old_zone.getLabel()
@@ -551,7 +560,7 @@ class VolumicLocalizationModel(LocalizationModel):
         self.node_models = self.case.xmlGetNode('thermophysical_models')
         self.node_veloce = self.node_models.xmlGetNode('velocity_pressure')
         self.scalar_node = self.case.xmlGetNode('additional_scalars')
-        self.losses_node = self.case.xmlGetNode('heads_losses')
+        self.losses_node = self.case.xmlGetNode('head_losses')
 
 
     @Variables.noUndo
@@ -676,7 +685,7 @@ class VolumicLocalizationModel(LocalizationModel):
         newLabel, newCodeNumber, newLocal = LocalizationModel.replaceZone(self, old_zone, new_zone)
 
         node = self.__XMLVolumicConditionsNode.xmlGetNode('zone',
-                                                           label = old_zone.getLabel())
+                                                          label = old_zone.getLabel())
 
         # if codeNumber is modified, we must modify zone in initialization of variables
         if old_zone.getCodeNumber() != newCodeNumber:
@@ -713,25 +722,12 @@ class VolumicLocalizationModel(LocalizationModel):
             name = node['id']
             node.xmlRemoveNode()
 
-        # Delete the other nodes for zone initializations
-        for tag in self._tagList:
-            nodeList = self.case.xmlGetNodeList(tag, zone_id=name)
-            for node in nodeList:
-                node.xmlRemoveNode()
+            # Delete the other nodes for zone initializations
+            n_d = self.case.xmlGetNodeWithAttrList('zone_id', zone_id=name)
+            for n in n_d:
+                n.xmlRemoveNode()
 
-        # Update Id's
-        XMLZonesNodes = self.__XMLVolumicConditionsNode.xmlGetChildNodeList('zone', 'label', 'id')
-
-        for node in XMLZonesNodes:
-            nodeid = int(node['id'])
-            if nodeid > int(name):
-                node['id'] = str(nodeid-1)
-
-        if self.losses_node is not None:
-            for node in self.losses_node.xmlGetNodeList('head_loss'):
-                nodeid = int(node['zone_id'])
-                if nodeid > int(name):
-                    node['zone_id'] = str(nodeid-1)
+            self.renumberZones(name)
 
 
     @Variables.undoGlobal
@@ -755,11 +751,40 @@ class VolumicLocalizationModel(LocalizationModel):
                 for node in nodeList:
                     node.xmlRemoveNode()
 
-        # Update Id's
+        self.renumberZones()
+
+
+    @Variables.undoGlobal
+    def renumberZones(self, start_id='0'):
+        """
+        Renumber zones in the XML file and update dependent id's
+        """
+
+        volCondNode = self.__XMLVolumicConditionsNode
+        XMLZonesNodes = volCondNode.xmlGetChildNodeList('zone', 'label', 'id')
+
+        n_d = self.case.xmlGetNodeWithAttrList('zone_id')
+
         count = 1
-        XMLZonesNodes = self.__XMLVolumicConditionsNode.xmlGetChildNodeList('zone', 'label', 'id')
+
+        l = []
         for node in XMLZonesNodes:
-            node['id'] = str(count)
+            l.append(int(node['id']))
+        l.sort()
+
+        for zoneid in l:
+            if zoneid > int(start_id):
+                node = volCondNode.xmlGetNode('zone', id=zoneid)
+
+                new_id = str(count)
+                old_id = node['id']
+                node['id'] = new_id
+
+                for n in n_d:
+                    if n['zone_id'] == old_id:
+                        n['zone_id'] = new_id
+
+            count = count + 1
 
 
 #-------------------------------------------------------------------------------
@@ -862,7 +887,8 @@ class BoundaryLocalizationModel(LocalizationModel):
         LocalizationModel.setCodeNumber(self, label, codeNumber)
         #
         # XML file updating
-        node = self.__XMLBoundaryConditionsNode.xmlGetChildNode('boundary', 'name', 'nature', label = label)
+        node = self.__XMLBoundaryConditionsNode.xmlGetChildNode('boundary', 'name', 'nature',
+                                                                label = label)
         node['name'] = str(codeNumber)
 
 
@@ -953,13 +979,16 @@ class BoundaryLocalizationModel(LocalizationModel):
         LocalizationModel.deleteZone(self, label)
 
         # Get Nature
-        node = self.__XMLBoundaryConditionsNode.xmlGetNode('boundary', 'name', 'nature', label = label)
+        node = self.__XMLBoundaryConditionsNode.xmlGetNode('boundary', 'name', 'nature',
+                                                           label = label)
         nature = node['nature']
         node.xmlRemoveNode()
 
         # Delete nature boundary
         Boundary(nature, label, self.case).delete()
 
+        self.renumberZones()
+
 
     @Variables.undoGlobal
     def mergeZones(self, label, localization, lst):
@@ -968,26 +997,38 @@ class BoundaryLocalizationModel(LocalizationModel):
         """
         LocalizationModel.mergeZones(self, label, localization, lst)
 
-        node = self.__XMLBoundaryConditionsNode.xmlGetNode('boundary', 'name', 'nature', label = label)
+        node = self.__XMLBoundaryConditionsNode.xmlGetNode('boundary', 'name', 'nature',
+                                                           label = label)
         node.xmlSetTextNode(localization)
 
         lst.reverse()
 
         for z in lst:
-            n = self.__XMLBoundaryConditionsNode.xmlGetNode('boundary', 'nature', 'label', name = z + 1)
+            n = self.__XMLBoundaryConditionsNode.xmlGetNode('boundary', 'nature', 'label',
+                                                            name = z + 1)
             label = n['label']
             nature = n['nature']
             Boundary(nature, label, self.case).delete()
             n.xmlRemoveNode()
 
+        self.renumberZones()
+
+
+    @Variables.undoGlobal
+    def renumberZones(self):
+        """
+        Merge zones in the XML file
+        """
+
         count = 1
-        for z in self.getCodeNumbersList():
-            n = self.__XMLBoundaryConditionsNode.xmlGetNode('boundary', 'nature', 'label', name = z)
-            n['name'] = count
+        l = self.getCodeNumbersList()
+        l.sort()
+        for z in l:
+            n = self.__XMLBoundaryConditionsNode.xmlGetNode('boundary', 'nature', 'label',
+                                                            name = z)
+            n['name'] = str(count)
             nature = n['nature']
             count = count + 1
-            # Delete nature boundary
-            Boundary(nature, label, self.case).delete()
 
 
 #-------------------------------------------------------------------------------
@@ -1005,173 +1046,6 @@ class LocalizationVolumicTestCase(ModelTest):
         assert model != None, 'Could not instantiate LocalizationVolumicModel'
 
 
-    def checkInitialZone(self):
-        """Check whether the zone could be initialized"""
-        model = LocalizationModel("VolumicZone", self.case)
-        node = self.case.xmlGetNode('volumic_conditions')
-
-        doc = '''<volumic_conditions>
-                    <zone head_losses="off" porosity="off" initialization="on" label="all_cells" mass_source_term="off" momentum_source_term="off" name="1" thermal_source_term="off">
-                            all[]
-                    </zone>
-                 </volumic_conditions>'''
-
-        assert node == self.xmlNodeFromString(doc),\
-           'Could not initialize default volumic zone'
-
-
-    def checkAddAndDeleteZone(self):
-        """Check whether the zone could be added and deleted"""
-        model = LocalizationModel("VolumicZone", self.case)
-        node = self.case.xmlGetNode('volumic_conditions')
-        model.addZone()
-        zone = Zone("VolumicZone", label='toto', localization="1 or door")
-        model.addZone(zone)
-        zone = Zone("VolumicZone", label='fenetre', localization="12 and window or door")
-        model.addZone(zone)
-        model.addZone()
-
-        doc = '''<volumic_conditions>
-                    <zone head_losses="off" porosity="off" initialization="on" label="all_cells" mass_source_term="off" momentum_source_term="off" name="1" thermal_source_term="off">
-                            all[]
-                    </zone>
-                    <zone head_losses="off" porosity="off" initialization="on" label="Zone_1" mass_source_term="off" momentum_source_term="off" name="2" thermal_source_term="off">
-                            not(all[])
-                    </zone>
-                    <zone head_losses="off" porosity="off" initialization="on" label="toto" mass_source_term="off" momentum_source_term="off" name="3" thermal_source_term="off">
-                            1 or door
-                    </zone>
-                    <zone head_losses="off" porosity="off" initialization="on" label="fenetre" mass_source_term="off" momentum_source_term="off" name="4" thermal_source_term="off">
-                            12 and window or door
-                    </zone>
-                    <zone head_losses="off" porosity="off" initialization="on" label="Zone_2" mass_source_term="off" momentum_source_term="off" name="5" thermal_source_term="off">
-                            not(all[];not(all[]);1 or door;12 and window or door)
-                    </zone>
-                </volumic_conditions>'''
-
-        assert node == self.xmlNodeFromString(doc),\
-           'Could not add one volumic zone'
-
-        model.deleteZone(label="toto")
-        model.deleteZone(label="Zone_2")
-
-        doc = '''<volumic_conditions>
-                    <zone head_losses="off" porosity="off" initialization="on" label="all_cells" mass_source_term="off" momentum_source_term="off" name="1" thermal_source_term="off">
-                            all[]
-                    </zone>
-                    <zone head_losses="off" porosity="off" initialization="on" label="Zone_1" mass_source_term="off" momentum_source_term="off" name="2" thermal_source_term="off">
-                            not(all[])
-                    </zone>
-                    <zone head_losses="off" porosity="off" initialization="on" label="fenetre" mass_source_term="off" momentum_source_term="off" name="4" thermal_source_term="off">
-                            12 and window or door
-                    </zone>
-                </volumic_conditions>'''
-
-        assert node == self.xmlNodeFromString(doc),\
-           'Could not delete one volumic zone'
-
-
-    def checkReplaceZone(self):
-        """Check whether the zone could be replaced"""
-        model = LocalizationModel("VolumicZone", self.case)
-        node = self.case.xmlGetNode('volumic_conditions')
-        model.addZone()
-        zone = Zone("VolumicZone", label='toto', localization="1 or door")
-        model.addZone(zone)
-        zone1 = Zone("VolumicZone", label='fenetre', localization="12 and window or door")
-        model.addZone(zone1)
-        new_zone = Zone("VolumicZone", label='window', localization="12 or window or door")
-##        new_zone = Zone("VolumicZone", localization="12 or window or door")
-##        new_zone = Zone("VolumicZone", label='window')
-
-        model.replaceZone(zone1, new_zone)
-
-        doc = '''<volumic_conditions>
-                    <zone head_losses="off" porosity="off" initialization="on" label="all_cells" mass_source_term="off" momentum_source_term="off" name="1" thermal_source_term="off">
-                            all[]
-                    </zone>
-                    <zone head_losses="off" porosity="off" initialization="on" label="Zone_1" mass_source_term="off" momentum_source_term="off" name="2" thermal_source_term="off">
-                            not(all[])
-                    </zone>
-                    <zone head_losses="off" porosity="off" initialization="on" label="toto" mass_source_term="off" momentum_source_term="off" name="3" thermal_source_term="off">
-                            1 or door
-                    </zone>
-                    <zone head_losses="off" porosity="off" initialization="on" label="window" mass_source_term="off" momentum_source_term="off" name="4" thermal_source_term="off">
-                            12 or window or door
-                    </zone>
-                </volumic_conditions>'''
-
-        assert node == self.xmlNodeFromString(doc),\
-           'Could not replace volumic zone'
-
-
-    def checkSetLabelAndLocalization(self):
-        """Check whether the zone could be set label ,and localization"""
-        model = LocalizationModel("VolumicZone", self.case)
-        node = self.case.xmlGetNode('volumic_conditions')
-        model.addZone()
-        zone = Zone("VolumicZone", label='toto', localization="1 or door")
-        model.addZone(zone)
-        zone = Zone("VolumicZone", label='fenetre', localization="12 and window or door")
-        model.addZone(zone)
-
-        model.setLocalization("Zone_1", "1 and 2 and 3 or group1")
-        doc = '''<volumic_conditions>
-                    <zone head_losses="off" porosity="off" initialization="on" label="all_cells" mass_source_term="off" momentum_source_term="off" name="1" thermal_source_term="off">
-                            all[]
-                    </zone>
-                    <zone head_losses="off" porosity="off" initialization="on" label="Zone_1" mass_source_term="off" momentum_source_term="off" name="2" thermal_source_term="off">
-                            1 and 2 and 3 or group1
-                    </zone>
-                    <zone head_losses="off" porosity="off" initialization="on" label="toto" mass_source_term="off" momentum_source_term="off" name="3" thermal_source_term="off">
-                            1 or door
-                    </zone>
-                    <zone head_losses="off" porosity="off" initialization="on" label="fenetre" mass_source_term="off" momentum_source_term="off" name="4" thermal_source_term="off">
-                            12 and window or door
-                    </zone>
-                </volumic_conditions>'''
-
-        assert node == self.xmlNodeFromString(doc),\
-           'Could not set localization on volumic zone'
-
-
-    def checkSetAndGetNature(self):
-        """Check whether the zone could be set and get nature"""
-        model = LocalizationModel("VolumicZone", self.case)
-        node = self.case.xmlGetNode('volumic_conditions')
-        model.addZone()
-        zone = Zone("VolumicZone", label='porte', localization="1 or door")
-        model.addZone(zone)
-        zone = Zone("VolumicZone", label='fenetre', localization="12 and window or door")
-        model.addZone(zone)
-
-        model.setNature('porte', "head_losses,thermal_source_term")
-
-        doc = '''<volumic_conditions>
-                    <zone head_losses="off" porosity="off" initialization="on" label="all_cells" mass_source_term="off" momentum_source_term="off" name="1" thermal_source_term="off">
-                            all[]
-                    </zone>
-                    <zone head_losses="off" porosity="off" initialization="on" label="Zone_1" mass_source_term="off" momentum_source_term="off" name="2" thermal_source_term="off">
-                            not(all[])
-                    </zone>
-                    <zone head_losses="on" porosity="off" initialization="off" label="porte" mass_source_term="off" momentum_source_term="off" name="3" thermal_source_term="on">
-                            1 or door
-                    </zone>
-                    <zone head_losses="off" porosity="off" initialization="on" label="fenetre" mass_source_term="off" momentum_source_term="off" name="4" thermal_source_term="off">
-                            12 and window or door
-                    </zone>
-                </volumic_conditions>'''
-
-        assert node == self.xmlNodeFromString(doc),\
-           'Could not set nature on volumic zone'
-
-        assert model.getNature('fenetre') == "initialization",\
-           'Could not get nature on volumic zone'
-
-        assert model.getNature('porte') == "thermal_source_term,head_losses",\
-           'Could not get nature on volumic zone'
-
-
 def suite1():
     testSuite = unittest.makeSuite(LocalizationVolumicTestCase, "check")
     return testSuite
