@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -53,6 +53,7 @@ from code_saturne.Base.QtPage    import IntValidator,  ComboModel, from_qvariant
 from code_saturne.Pages.MobileMeshModel import MobileMeshModel
 
 from code_saturne.Pages.QMeiEditorView import QMeiEditorView
+from code_saturne.Pages.NotebookModel import NotebookModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -127,6 +128,7 @@ if (xray2 < xr2) {
         self.case.undoStopGlobal()
         self.mdl = MobileMeshModel(self.case)
         self.browser = browser
+        self.notebook = NotebookModel(self.case)
 
         # Combo model VISCOSITY
         self.modelVISCOSITY = ComboModel(self.comboBoxVISCOSITY,2,1)
@@ -242,6 +244,9 @@ if (xray2 < xr2) {
                 ('t', 'current time'),
                 ('iter', 'number of iteration')]
 
+        for (nme, val) in self.notebook.getNotebookList():
+            symb.append((nme, 'value (notebook) = ' + str(val)))
+
         dialog = QMeiEditorView(self,
                                 check_syntax = self.case['package'].get_check_syntax(),
                                 expression = exp,
