@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -35,7 +35,6 @@ This module defines the following classes:
 # Library modules import
 #-------------------------------------------------------------------------------
 
-import string
 import unittest
 
 #-------------------------------------------------------------------------------
@@ -102,6 +101,7 @@ class NumericalParamEquationModel(Model):
         """ Private method: return default values """
         self.default = {}
         self.default['time_step_factor'] = 1.0
+        self.default['verbosity'] = 0
         self.default['solver_precision'] = 1e-8
         self.default['solver_precision_pressure'] = 1e-8
         if NumericalParamGlobalModel(self.case).getTimeSchemeOrder() == 2:
@@ -574,7 +574,7 @@ class NumericalParamEquationModel(Model):
         """ Put choice of solver for variable labelled name """
         self.isInList(value, ('multigrid', 'conjugate_gradient', 'jacobi',
                               'bi_cgstab', 'bi_cgstab2', 'gmres', 'automatic',
-                              'gauss_seidel', 'PCR3'))
+                              'gauss_seidel', 'symmetric_gauss_seidel', 'PCR3'))
         node = self._getSolverNameNode(name)
 
         default = self._defaultValues()['solver_choice']
@@ -682,6 +682,27 @@ class NumericalParamEquationModel(Model):
             raise ValueError("This method runs only with scalar name")
 
 
+    @Variables.noUndo
+    def getVerbosity(self, name):
+        """ Return value of verbosity for variable labelled name """
+        node = self._getSolverNameNode(name)
+        value = node.xmlGetInt('verbosity')
+        if value == None:
+            value = self._defaultValues()['verbosity']
+        return value
+
+
+    @Variables.undoLocal
+    def setVerbosity(self, name, value):
+        """ Put value of verbosity for variable labelled name """
+        self.isInt(value)
+        node = self._getSolverNameNode(name)
+        if value != self._defaultValues()['verbosity']:
+            node.xmlSetData('verbosity', value)
+        else:
+            node.xmlRemoveChild('verbosity')
+
+
     @Variables.noUndo
     def getMinValue(self, name):
         """Get minimal value from an additional_scalar with label scalar_name"""
