@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -127,6 +127,7 @@ class SolverChoiceDelegate(QItemDelegate):
         editor.addItem("BiCGstab2")
         editor.addItem("GMRES")
         editor.addItem("Gauss Seidel")
+        editor.addItem("Symmetric Gauss Seidel")
         editor.addItem("conjugate residual")
         if mg:
             editor.addItem("Multigrid")
@@ -136,8 +137,8 @@ class SolverChoiceDelegate(QItemDelegate):
 
     def setEditorData(self, comboBox, index):
         dico = {"automatic": 0, "conjugate_gradient": 1, "jacobi": 2,
-                "bi_cgstab": 3, "bi_cgstab2": 4, "gmres": 5, "multigrid": 8,
-                "gauss_seidel": 6, "PCR3": 7}
+                "bi_cgstab": 3, "bi_cgstab2": 4, "gmres": 5, "multigrid": 9,
+                "gauss_seidel": 6, "symmetric_gauss_seidel": 7, "PCR3": 8}
         row = index.row()
         string = index.model().dataSolver[row]['iresol']
         idx = dico[string]
@@ -168,11 +169,11 @@ class PreconditioningChoiceDelegate(QItemDelegate):
     def createEditor(self, parent, option, index):
         editor = QComboBox(parent)
 
+        editor.addItem("Automatic")
         editor.addItem("None")
         editor.addItem("Multigrid")
         editor.addItem("Jacobi")
         editor.addItem("Polynomial")
-        editor.addItem("Automatic")
 
         solver = index.model().dataSolver[index.row()]['iresol']
         if solver == 'multigrid':
@@ -489,7 +490,7 @@ class StandardItemModelSolver(QStandardItemModel):
         QStandardItemModel.__init__(self)
         self.NPE = NPE
         self.SM = SM
-        self.setColumnCount(5)
+        self.setColumnCount(6)
         self.dataSolver = []
         # list of items to be disabled in the view
         self.disabledItem = []
@@ -497,28 +498,30 @@ class StandardItemModelSolver(QStandardItemModel):
 
 
     def populateModel(self):
-        self.dicoV2M= {"Multigrid"          : 'multigrid',
-                       "Conjugate gradient" : 'conjugate_gradient',
-                       "Jacobi"             : 'jacobi',
-                       "BiCGstab"           : 'bi_cgstab',
-                       "BiCGstab2"          : 'bi_cgstab2',
-                       "GMRES"              : 'gmres',
-                       "Automatic"          : "automatic",
-                       "Gauss Seidel"       : "gauss_seidel",
-                       "conjugate residual" : "PCR3",
-                       "None"               : "none",
-                       "Polynomial"         : "polynomial"}
-        self.dicoM2V= {"multigrid"          : 'Multigrid',
-                       "conjugate_gradient" : 'Conjugate gradient',
-                       "jacobi"             : 'Jacobi',
-                       "bi_cgstab"          : 'BiCGstab',
-                       "bi_cgstab2"         : 'BiCGstab2',
-                       'gmres'              : "GMRES",
-                       "automatic"          : "Automatic",
-                       "gauss_seidel"       : "Gauss Seidel",
-                       "PCR3"               : "conjugate residual",
-                       "none"               : "None",
-                       "polynomial"         : "Polynomial"}
+        self.dicoV2M= {"Multigrid"              : 'multigrid',
+                       "Conjugate gradient"     : 'conjugate_gradient',
+                       "Jacobi"                 : 'jacobi',
+                       "BiCGstab"               : 'bi_cgstab',
+                       "BiCGstab2"              : 'bi_cgstab2',
+                       "GMRES"                  : 'gmres',
+                       "Automatic"              : "automatic",
+                       "Gauss Seidel"           : "gauss_seidel",
+                       "Symmetric Gauss Seidel" : "symmetric_gauss_seidel",
+                       "conjugate residual"     : "PCR3",
+                       "None"                   : "none",
+                       "Polynomial"             : "polynomial"}
+        self.dicoM2V= {"multigrid"              : 'Multigrid',
+                       "conjugate_gradient"     : 'Conjugate gradient',
+                       "jacobi"                 : 'Jacobi',
+                       "bi_cgstab"              : 'BiCGstab',
+                       "bi_cgstab2"             : 'BiCGstab2',
+                       'gmres'                  : "GMRES",
+                       "automatic"              : "Automatic",
+                       "gauss_seidel"           : "Gauss Seidel",
+                       "symmetric_gauss_seidel" : "Symmetric Gauss Seidel",
+                       "PCR3"                   : "conjugate residual",
+                       "none"                   : "None",
+                       "polynomial"             : "Polynomial"}
 
         for name in self.NPE.getSolverList():
             row = self.rowCount()
@@ -530,11 +533,12 @@ class StandardItemModelSolver(QStandardItemModel):
             dico['iresol']  = self.NPE.getSolverChoice(name)
             dico['precond'] = self.NPE.getPreconditioningChoice(name)
             dico['epsilo']  = self.NPE.getSolverPrecision(name)
+            dico['verbo']   = self.NPE.getVerbosity(name)
             if self.NPE.isScalar(name):
                 dico['cdtvar'] = self.NPE.getScalarTimeStepFactor(name)
             else:
                 dico['cdtvar'] = ""
-                self.disabledItem.append((row,4))
+                self.disabledItem.append((row,5))
 
             self.dataSolver.append(dico)
             log.debug("populateModel-> dataSolver = %s" % dico)
@@ -549,6 +553,8 @@ class StandardItemModelSolver(QStandardItemModel):
             if index.column() == 3:
                 return to_qvariant(self.tr("Code_Saturne keyword: EPSILO"))
             elif index.column() == 4:
+                return to_qvariant(self.tr("Code_Saturne keyword: IWARNI"))
+            elif index.column() == 5:
                 return to_qvariant(self.tr("Code_Saturne keyword: CDTVAR"))
 
         elif role == Qt.DisplayRole:
@@ -564,6 +570,8 @@ class StandardItemModelSolver(QStandardItemModel):
             elif index.column() == 3:
                 return to_qvariant(dico['epsilo'])
             elif index.column() == 4:
+                return to_qvariant(dico['verbo'])
+            elif index.column() == 5:
                 return to_qvariant(dico['cdtvar'])
             else:
                 return to_qvariant()
@@ -599,6 +607,8 @@ class StandardItemModelSolver(QStandardItemModel):
             elif section == 3:
                 return to_qvariant(self.tr("Solver\nPrecision"))
             elif section == 4:
+                return to_qvariant(self.tr("Verbosity"))
+            elif section == 5:
                 return to_qvariant(self.tr("Time Step\nFactor"))
             else:
                 return to_qvariant()
@@ -622,6 +632,10 @@ class StandardItemModelSolver(QStandardItemModel):
             self.NPE.setSolverPrecision(name, self.dataSolver[row]['epsilo'])
 
         elif index.column() == 4:
+            self.dataSolver[row]['verbo'] = from_qvariant(value, int)
+            self.NPE.setVerbosity(name, self.dataSolver[row]['verbo'])
+
+        elif index.column() == 5:
             self.dataSolver[row]['cdtvar'] = from_qvariant(value, float)
             self.NPE.setScalarTimeStepFactor(name, self.dataSolver[row]['cdtvar'])
 
@@ -663,6 +677,39 @@ class MinimumDelegate(QItemDelegate):
                         model.setData(idx, to_qvariant(value), Qt.DisplayRole)
 
 
+#-------------------------------------------------------------------------------
+# Line edit delegate for verbosity
+#-------------------------------------------------------------------------------
+
+class VerbosityDelegate(QItemDelegate):
+    def __init__(self, parent=None, xml_model=None):
+        super(VerbosityDelegate, self).__init__(parent)
+        self.parent = parent
+        self.turb = xml_model
+
+
+    def createEditor(self, parent, option, index):
+        editor = QLineEdit(parent)
+        validator = IntValidator(editor, min=0)
+        validator.setExclusiveMin(False)
+        editor.setValidator(validator)
+        return editor
+
+
+    def setEditorData(self, editor, index):
+        value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
+        editor.setText(value)
+
+
+    def setModelData(self, editor, model, index):
+        if editor.validator().state == QValidator.Acceptable:
+            value = from_qvariant(editor.text(), int)
+            selectionModel = self.parent.selectionModel()
+            for idx in selectionModel.selectedIndexes():
+                if idx.column() == index.column():
+                    model.setData(idx, to_qvariant(value))
+
+
 #-------------------------------------------------------------------------------
 # Line edit delegate for maximum value
 #-------------------------------------------------------------------------------
@@ -880,7 +927,7 @@ class NumericalParamEquationView(QWidget, Ui_NumericalParamEquationForm):
             self.tableViewSolver.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
 
         if self.SM.getSteadyFlowManagement() == 'on':
-            self.tableViewSolver.setColumnHidden(4, True)
+            self.tableViewSolver.setColumnHidden(5, True)
 
         delegate = SolverDelegate(self.tableViewSolver)
         self.tableViewSolver.setItemDelegate(delegate)
@@ -891,6 +938,9 @@ class NumericalParamEquationView(QWidget, Ui_NumericalParamEquationForm):
         delegatePrecondChoice = PreconditioningChoiceDelegate(self.tableViewSolver)
         self.tableViewSolver.setItemDelegateForColumn(2, delegatePrecondChoice)
 
+        delegateVerbosity = VerbosityDelegate(self.tableViewSolver)
+        self.tableViewSolver.setItemDelegateForColumn(4, delegateVerbosity)
+
         # Clipping
         self.modelClipping = StandardItemModelClipping(self, self.NPE)
         self.tableViewClipping.setModel(self.modelClipping)
