@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2015 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -84,12 +84,12 @@ class NumericalParamGlobalModel(Model):
         else:
             self.default['piso_sweep_number'] = 2
         del CompressibleModel
-        from code_saturne.Pages.DarcyModel import DarcyModel
-        if DarcyModel(self.case).getDarcyModel() != 'off':
+        from code_saturne.Pages.GroundwaterModel import GroundwaterModel
+        if GroundwaterModel(self.case).getGroundwaterModel() != 'off':
             self.default['gradient_reconstruction'] = 1
         else:
             self.default['gradient_reconstruction'] = 0
-        del DarcyModel
+        del GroundwaterModel
         return self.default
 
 
