@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -297,8 +297,8 @@ class NumericalParamGlobalView(QWidget, Ui_NumericalParamGlobalForm):
         """
         Set value for parameter IMRGRA
         """
-        imrgra = self.modelIMRGRA.getIndex(str_view=str(text))
-        self.model.setGradientReconstruction(imrgra)
+        imrgra = self.modelIMRGRA.dicoV2M[str(text)]
+        self.model.setGradientReconstruction(int(imrgra))
         log.debug("slotIMRGRA-> %s" % imrgra)
 
 
