@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -38,7 +38,7 @@ This module defines the following classes:
 # Library modules import
 #-------------------------------------------------------------------------------
 
-import string, sys, unittest
+import sys, unittest
 
 #-------------------------------------------------------------------------------
 # Application modules import
@@ -149,6 +149,7 @@ class OutputControlModel(Model):
         default['frequency_choice']          = 'none'
         default['frequency']                 = 1
         default['frequency_time']            = 1.
+        default['output_at_start']           = 'off'
         default['output_at_end']             = 'on'
         default['format']                    = 'ensight'
         default['time_dependency']           = 'fixed_mesh'
@@ -225,7 +226,7 @@ class OutputControlModel(Model):
             node = self.node_out.xmlInitNode('writer', id = "-1", label = 'results')
             node.xmlInitNode('frequency', period = 'none')
             node.xmlInitNode('output_at_end', status = 'on')
-            node.xmlInitNode('format', name = 'ensight', options = 'binary')
+            node.xmlInitNode('format', name = 'ensight', options = '')
             node.xmlInitNode('directory', name = 'postprocessing')
             node.xmlInitNode('time_dependency', choice = 'fixed_mesh')
 
@@ -239,7 +240,7 @@ class OutputControlModel(Model):
             nodeL = self.node_out.xmlInitNode('writer', id = "-3", label = 'particles')
             nodeL.xmlInitNode('frequency', period = 'none')
             nodeL.xmlInitNode('output_at_end', status = 'on')
-            nodeL.xmlInitNode('format', name = 'ensight', options = 'binary')
+            nodeL.xmlInitNode('format', name = 'ensight', options = '')
             nodeL.xmlInitNode('directory', name = 'postprocessing')
             nodeL.xmlInitNode('time_dependency', choice = 'transient_connectivity')
 
@@ -248,7 +249,7 @@ class OutputControlModel(Model):
             nodeT = self.node_out.xmlInitNode('writer', id = "-4", label = 'trajectories')
             nodeT.xmlInitNode('frequency', period = 'none')
             nodeT.xmlInitNode('output_at_end', status = 'on')
-            nodeT.xmlInitNode('format', name = 'ensight', options = 'binary')
+            nodeT.xmlInitNode('format', name = 'ensight', options = '')
             nodeT.xmlInitNode('directory', name = 'postprocessing')
             nodeT.xmlInitNode('time_dependency', choice = 'fixed_mesh')
 
@@ -407,6 +408,37 @@ class OutputControlModel(Model):
         node.xmlSetData('frequency', str(freq))
 
 
+    @Variables.noUndo
+    def getWriterOutputStartStatus(self, writer_id):
+        """
+        Return the output_at_start status of a mesh
+        """
+        self.isInList(writer_id, self.getWriterIdList())
+        node = self.node_out.xmlGetNode('writer', 'label', id = writer_id)
+        n = node.xmlGetNode('output_at_start')
+        status = None
+        if n:
+            status = n['status']
+        if status == None:
+            status = self.defaultWriterValues()['output_at_start']
+        return status
+
+
+    @Variables.undoLocal
+    def setWriterOutputStartStatus(self, writer_id, status):
+        """
+        Set the output_at_start status of a mesh
+        """
+        self.isInList(writer_id, self.getWriterIdList())
+        self.isOnOff(status)
+        node = self.node_out.xmlGetNode('writer', 'label', id = writer_id)
+        if status == self.defaultWriterValues()['output_at_start']:
+            node.xmlRemoveChild('output_at_start')
+        else:
+            n = node.xmlInitNode('output_at_start')
+            n['status'] = status
+
+
     @Variables.noUndo
     def getWriterOutputEndStatus(self, writer_id):
         """
@@ -414,11 +446,12 @@ class OutputControlModel(Model):
         """
         self.isInList(writer_id, self.getWriterIdList())
         node = self.node_out.xmlGetNode('writer', 'label', id = writer_id)
-        n = node.xmlInitNode('output_at_end')
-        status = n['status']
+        n = node.xmlGetNode('output_at_end')
+        status = None
+        if n:
+            status = n['status']
         if status == None:
             status = self.defaultWriterValues()['output_at_end']
-            self.setWriterOutputEndStatus(writer_id, status)
         return status
 
 
@@ -430,8 +463,11 @@ class OutputControlModel(Model):
         self.isInList(writer_id, self.getWriterIdList())
         self.isOnOff(status)
         node = self.node_out.xmlGetNode('writer', 'label', id = writer_id)
-        n = node.xmlInitNode('output_at_end')
-        n['status'] = status
+        if status == self.defaultWriterValues()['output_at_end']:
+            node.xmlRemoveChild('output_at_end')
+        else:
+            n = node.xmlInitNode('output_at_end')
+            n['status'] = status
 
 
     @Variables.noUndo
@@ -455,7 +491,6 @@ class OutputControlModel(Model):
         Set the format for a writer
         """
         self.isInList(writer_id, self.getWriterIdList())
-        self.isInList(format, ('ensight', 'med', 'cgns', 'catalyst', 'ccm'))
         node = self.node_out.xmlInitNode('writer', 'label', id = writer_id)
         n = node.xmlInitNode('format')
         n['name'] = format
@@ -1142,7 +1177,7 @@ class OutputControlModel(Model):
             node.xmlRemoveNode()
             self.case.xmlRemoveChild('probe_recording', name=num)
 
-            # renumerotation of all monitoring points
+            # renumbering of all monitoring points
 
             for p in range(int(num)+1, self.getNumberOfMonitoringPoints()+2):
                 probe = self.node_out.xmlGetNode('probe', name=p)
@@ -1150,22 +1185,6 @@ class OutputControlModel(Model):
                 for probe_recording in self.case.xmlGetNodeList('probe_recording', name=p):
                     probe_recording['name'] = p - 1
 
-            # update the attribute "choice" of the probes markup for variables
-
-            from code_saturne.Pages.OutputVolumicVariablesModel import OutputVolumicVariablesModel
-            listNodeVolum = OutputVolumicVariablesModel(self.case).listNodeVolum
-            del OutputVolumicVariablesModel
-            for nodeList in listNodeVolum:
-                for node in nodeList:
-                    n = node.xmlGetChildNode('probes')
-                    if n:
-                        nlist = n.xmlGetChildNodeList('probe_recording')
-                        if not nlist:
-                            n.xmlRemoveNode()
-                        else:
-                            n['choice']= str(len(nlist))
-
-
     @Variables.noUndo
     def getMonitoringPointCoordinates(self, name):
         """
