@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -34,7 +34,7 @@ This module contains the following classes and function:
 # Standard modules
 #-------------------------------------------------------------------------------
 
-import os, sys, string, logging
+import os, sys, logging
 try:
     import ConfigParser
     configparser = ConfigParser
@@ -117,7 +117,7 @@ class MeshFormatDelegate(QItemDelegate):
         self.lst = MeshModel().getBuildFormatList()
         # Compute width based on longest possible string and font metrics
         fm = self.parent.fontMetrics()
-        self.textSize = fm.size(Qt.TextSingleLine, 'pro-STAR/STAR4 (*.ngeom)')
+        self.textSize = fm.size(Qt.TextSingleLine, 'I-deas universal')
         self.textSize.setHeight(1)
         for i in range(len(self.lst)):
             w = fm.size(Qt.TextSingleLine, str(self.lst[i][1])).width()
@@ -718,7 +718,7 @@ class StandardItemModelExtrude(QStandardItemModel):
         self.headers = [ self.tr("zone id"),
                          self.tr("n layers"),
                          self.tr("thickness"),
-                         self.tr("reason"),
+                         self.tr("expansion factor"),
                          self.tr("selector")]
 
         self.setColumnCount(len(self.headers))
@@ -862,7 +862,7 @@ class MeshInputDialog(QFileDialog):
         self.setWindowTitle(caption)
 
         self.name_filter = str(self.tr("Imported or preprocessed meshes (mesh_input mesh_output)"))
-        self.setFilter(self.name_filter)
+        self.setNameFilter(self.name_filter)
 
         self.select_label = str(self.tr("Choose"))
         self.setLabelText(QFileDialog.Accept, self.select_label)
@@ -890,7 +890,7 @@ class MeshInputDialog(QFileDialog):
             if os.path.isdir(path):
                 mode = QFileDialog.Directory
         self.setFileMode(mode)
-        self.setFilter(self.name_filter)
+        self.setNameFilter(self.name_filter)
         self.setLabelText(QFileDialog.Accept, self.select_label)
 
 
@@ -1037,6 +1037,10 @@ class SolutionDomainView(QWidget, Ui_SolutionDomainForm):
         self.pushButtonAddThinWall.clicked.connect(self.slotAddThinWall)
         self.pushButtonDeleteThinWall.clicked.connect(self.slotDeleteThinWall)
 
+        # load values
+        for tw in range(self.mdl.getThinWallSelectionsCount()):
+            self.tableModelThinWall.newItem(tw)
+
         # 2.5) Extrude
 
         self.tableModelExtrude = StandardItemModelExtrude(self, self.mdl)
@@ -1119,12 +1123,11 @@ class SolutionDomainView(QWidget, Ui_SolutionDomainForm):
 
         # Connections
 
-        self.widgetFacesPerio.tableView.clicked[QModelIndex].connect(self.slotUpdatePeriodicity)
+        selectionModel = self.widgetFacesPerio.tableView.selectionModel()
+        selectionModel.selectionChanged.connect(self.slotUpdatePeriodicity)
 
         self.widgetFacesPerio.pushButtonDelete.clicked.connect(self.slotDeletePeriodicity)
 
-        # self.connect(self.modelPeriod, SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), self.slotUpdateByModel)
-
         self.comboBoxPeriodicity.activated[str].connect(self.slotPeriodicityMode)
 
         self.lineEditTX.textChanged[str].connect(self.slotTranslationX)
@@ -1670,27 +1673,21 @@ class SolutionDomainView(QWidget, Ui_SolutionDomainForm):
             self.__setValuesMixed(perio)
 
 
-    @pyqtSlot("QModelIndex, QModelIndex")
-    def slotUpdateByModel(self, index1, index2):
-        """
-        This slot updates the display for the periodicity modified in the
-        in the table view.
-        """
-        log.debug("slotUpdateByModel index.row() = %i " % index1.row())
-        self.slotUpdatePeriodicity(index1)
-
-
-    @pyqtSlot("QModelIndex")
-    def slotUpdatePeriodicity(self, index):
+    @pyqtSlot("QItemSelection")
+    def slotUpdatePeriodicity(self, current):
         """
         This slot updates the display for the periodicity selected
         in the table view.
         """
+        index = self.widgetFacesPerio.tableView.currentIndex()
         log.debug("slotUpdatePeriodicity index.row() = %i " % index.row())
 
         self.groupBoxMode.show()
 
         perio_id = index.row()
+        if perio_id < 0:
+            return
+
         perio_mode = self.mdl.getPeriodicityMode(perio_id)
         self.perio_id = perio_id
         self.perio_mode = perio_mode
