@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -67,22 +67,19 @@ class ThermalRadiationModel(Variables, Model):
         self.c_prop = {}
         self.b_prop = {}
 
-        self.c_prop['qrad_x']                     = self.tr("Qrad_x")
-        self.c_prop['qrad_y']                     = self.tr("Qrad_y")
-        self.c_prop['qrad_z']                     = self.tr("Qrad_z")
+        self.c_prop['qrad']                       = self.tr("Qrad")
         self.c_prop['radiative_source_term']      = self.tr("Radiative_source_term")
         self.c_prop['absorption']                 = self.tr("Absorption")
         self.c_prop['emission']                   = self.tr("Emission")
         self.c_prop['absorption_coefficient']     = self.tr("Absorption_coefficient")
 
-        self.b_prop['wall_temp']              = self.tr("Wall_temperature")
-        self.b_prop['flux_incident']          = self.tr("Flux_incident")
-        self.b_prop['thickness']              = self.tr("Thickness")
-        self.b_prop['thermal_conductivity']   = self.tr("Thermal_conductivity")
-        self.b_prop['emissivity']             = self.tr("Emissivity")
-        self.b_prop['flux_net']               = self.tr("Flux_net")
-        self.b_prop['flux_convectif']         = self.tr("Flux_convectif")
-        self.b_prop['coeff_ech_conv']         = self.tr("Coeff_ech_convectif")
+        self.b_prop['flux_incident']               = self.tr("Flux_incident")
+        self.b_prop['thickness']                   = self.tr("Thickness")
+        self.b_prop['wall_thermal_conductivity']   = self.tr("Thermal_conductivity")
+        self.b_prop['emissivity']                  = self.tr("Emissivity")
+        self.b_prop['flux_net']                    = self.tr("Flux_net")
+        self.b_prop['flux_convectif']              = self.tr("Flux_convectif")
+        self.b_prop['coeff_ech_conv']              = self.tr("Coeff_ech_convectif")
 
         self.classesNumber = 0
 
@@ -130,15 +127,15 @@ class ThermalRadiationModel(Variables, Model):
         """
         dico = {}
         rayName = ['srad',     'qrad',     'absorp',  'emiss',    'coefAb',
-                   'wall_temp', 'flux_incident', 'thermal_conductivity', 'thickness',
+                   'flux_incident', 'wall_thermal_conductivity', 'thickness',
                    'emissivity', 'flux_net',      'flux_convectif',  'coeff_ech_conv']
 
         raylabF = ['Srad',       'Qrad',          'Absorp',     'Emiss',    'CoefAb',
-                   'Temp_paroi', 'Flux_incident', 'Conductivite_th', 'Epaisseur',
+                   'Flux_incident', 'Conductivite_th', 'Epaisseur',
                    'Emissivite', 'Flux_net',      'Flux_convectif',  'Coeff_ech_conv']
 
         raylabE = ['Srad',      'Qrad',      'Absorp',          'Emiss',    'CoefAb',
-                   'Wall_temp', 'Incident_flux', 'Th_conductivity', 'Thickness',
+                   'Incident_flux', 'Th_conductivity', 'Thickness',
                    'Emissivity','Net_flux',      'Convective_flux',  'Conv_exch_coeff']
 
         dico['name'] = rayName
@@ -440,9 +437,8 @@ class ThermalRadiationTestCase(ModelTest):
                     <property label="Absorp" name="absorp"/>
                     <property label="Emiss" name="emiss"/>
                     <property label="CoefAb" name="coefAb"/>
-                    <property label="Wall_temp" name="wall_temp" support="boundary"/>
                     <property label="Flux_incident" name="flux_incident" support="boundary"/>
-                    <property label="Th_conductivity" name="thermal_conductivity" support="boundary"/>
+                    <property label="Th_conductivity" name="wall_thermal_conductivity" support="boundary"/>
                     <property label="Thickness" name="thickness" support="boundary"/>
                     <property label="Emissivity" name="emissivity" support="boundary"/>
                     <property label="Flux_net" name="flux_net" support="boundary"/>
@@ -472,9 +468,8 @@ class ThermalRadiationTestCase(ModelTest):
                     <property label="Absorp" name="absorp"/>
                     <property label="Emiss" name="emiss"/>
                     <property label="CoefAb" name="coefAb"/>
-                    <property label="Wall_temp" name="wall_temp" support="boundary"/>
                     <property label="Flux_incident" name="flux_incident" support="boundary"/>
-                    <property label="Th_conductivity" name="thermal_conductivity" support="boundary"/>
+                    <property label="Th_conductivity" name="wall_thermal_conductivity" support="boundary"/>
                     <property label="Thickness" name="thickness" support="boundary"/>
                     <property label="Emissivity" name="emissivity" support="boundary"/>
                     <property label="Flux_net" name="flux_net" support="boundary"/>
@@ -504,9 +499,8 @@ class ThermalRadiationTestCase(ModelTest):
                     <property label="Absorp" name="absorp"/>
                     <property label="Emiss" name="emiss"/>
                     <property label="CoefAb" name="coefAb"/>
-                    <property label="Wall_temp" name="wall_temp" support="boundary"/>
                     <property label="Flux_incident" name="flux_incident" support="boundary"/>
-                    <property label="Th_conductivity" name="thermal_conductivity" support="boundary"/>
+                    <property label="Th_conductivity" name="wall_thermal_conductivity" support="boundary"/>
                     <property label="Thickness" name="thickness" support="boundary"/>
                     <property label="Emissivity" name="emissivity" support="boundary"/>
                     <property label="Flux_net" name="flux_net" support="boundary"/>
@@ -536,9 +530,8 @@ class ThermalRadiationTestCase(ModelTest):
                     <property label="Absorp" name="absorp"/>
                     <property label="Emiss" name="emiss"/>
                     <property label="CoefAb" name="coefAb"/>
-                    <property label="Wall_temp" name="wall_temp" support="boundary"/>
                     <property label="Flux_incident" name="flux_incident" support="boundary"/>
-                    <property label="Th_conductivity" name="thermal_conductivity" support="boundary"/>
+                    <property label="Th_conductivity" name="wall_thermal_conductivity" support="boundary"/>
                     <property label="Thickness" name="thickness" support="boundary"/>
                     <property label="Emissivity" name="emissivity" support="boundary"/>
                     <property label="Flux_net" name="flux_net" support="boundary"/>
@@ -568,9 +561,8 @@ class ThermalRadiationTestCase(ModelTest):
                     <property label="Absorp" name="absorp"/>
                     <property label="Emiss" name="emiss"/>
                     <property label="CoefAb" name="coefAb"/>
-                    <property label="Wall_temp" name="wall_temp" support="boundary"/>
                     <property label="Flux_incident" name="flux_incident" support="boundary"/>
-                    <property label="Th_conductivity" name="thermal_conductivity" support="boundary"/>
+                    <property label="Th_conductivity" name="wall_thermal_conductivity" support="boundary"/>
                     <property label="Thickness" name="thickness" support="boundary"/>
                     <property label="Emissivity" name="emissivity" support="boundary"/>
                     <property label="Flux_net" name="flux_net" support="boundary"/>
@@ -601,9 +593,8 @@ class ThermalRadiationTestCase(ModelTest):
                     <property label="Absorp" name="absorp"/>
                     <property label="Emiss" name="emiss"/>
                     <property label="CoefAb" name="coefAb"/>
-                    <property label="Wall_temp" name="wall_temp" support="boundary"/>
                     <property label="Flux_incident" name="flux_incident" support="boundary"/>
-                    <property label="Th_conductivity" name="thermal_conductivity" support="boundary"/>
+                    <property label="Th_conductivity" name="wall_thermal_conductivity" support="boundary"/>
                     <property label="Thickness" name="thickness" support="boundary"/>
                     <property label="Emissivity" name="emissivity" support="boundary"/>
                     <property label="Flux_net" name="flux_net" support="boundary"/>
@@ -634,9 +625,8 @@ class ThermalRadiationTestCase(ModelTest):
                     <property label="Absorp" name="absorp"/>
                     <property label="Emiss" name="emiss"/>
                     <property label="CoefAb" name="coefAb"/>
-                    <property label="Wall_temp" name="wall_temp" support="boundary"/>
                     <property label="Flux_incident" name="flux_incident" support="boundary"/>
-                    <property label="Th_conductivity" name="thermal_conductivity" support="boundary"/>
+                    <property label="Th_conductivity" name="wall_thermal_conductivity" support="boundary"/>
                     <property label="Thickness" name="thickness" support="boundary"/>
                     <property label="Emissivity" name="emissivity" support="boundary"/>
                     <property label="Flux_net" name="flux_net" support="boundary"/>
@@ -667,9 +657,8 @@ class ThermalRadiationTestCase(ModelTest):
                     <property label="Absorp" name="absorp"/>
                     <property label="Emiss" name="emiss"/>
                     <property label="CoefAb" name="coefAb"/>
-                    <property label="Wall_temp" name="wall_temp" support="boundary"/>
                     <property label="Flux_incident" name="flux_incident" support="boundary"/>
-                    <property label="Th_conductivity" name="thermal_conductivity" support="boundary"/>
+                    <property label="Th_conductivity" name="wall_thermal_conductivity" support="boundary"/>
                     <property label="Thickness" name="thickness" support="boundary"/>
                     <property label="Emissivity" name="emissivity" support="boundary"/>
                     <property label="Flux_net" name="flux_net" support="boundary"/>
