@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -227,19 +227,27 @@ class ThermalRadiationView(QWidget, Ui_ThermalRadiationForm):
         self.lineEditNdirec.setValidator(validatorNdir)
 
         self.modelAbsorption.addItem('constant',                   'constant')
-        self.modelAbsorption.addItem('user subroutine (usray3)',   'variable')
+        self.modelAbsorption.addItem('user function (cs_user_rad_transfer_absorption)',   'variable')
         self.modelAbsorption.addItem('user law',                   'formula')
         self.modelAbsorption.addItem('H2O and CO2 mixing (Modak)', 'modak')
 
+        coal_or_gas = "off"
         from code_saturne.Pages.CoalCombustionModel import CoalCombustionModel
-        if CoalCombustionModel(self.case).getCoalCombustionModel("only") != "off":
+        coal_or_gas = CoalCombustionModel(self.case).getCoalCombustionModel("only")
+        del CoalCombustionModel
+
+        if coal_or_gas == "off":
+            from code_saturne.Pages.GasCombustionModel import GasCombustionModel
+            coal_or_gas = GasCombustionModel(self.case).getGasCombustionModel()
+            del GasCombustionModel
+
+        if coal_or_gas != "off":
             self.modelAbsorption.disableItem(str_model='variable')
             self.modelAbsorption.enableItem(str_model='modak')
         else:
             self.modelAbsorption.disableItem(str_model='modak')
             self.modelAbsorption.enableItem(str_model='variable')
 
-        del CoalCombustionModel
 
         self.modelAbsorption.disableItem(str_model='formula')
 
