@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -96,6 +96,7 @@ class TurbulenceModel(Variables, Model):
                                'r12',
                                'r13',
                                'r23',
+                               'rij',
                                'k',
                                'epsilon',
                                'phi',
@@ -190,6 +191,10 @@ class TurbulenceModel(Variables, Model):
                 for node in self.node_ava.xmlGetNodeList('time_average'):
                     node.xmlRemoveChild('var_prop', name=v)
         self.node_turb.xmlRemoveChild('property', name=propName)
+        for node in self.node_prof.xmlGetNodeList('profile'):
+            node.xmlRemoveChild('var_prop', name=propName)
+        for node in self.node_ava.xmlGetNodeList('time_average'):
+            node.xmlRemoveChild('var_prop', name=propName)
 
 
     @Variables.undoGlobal
@@ -217,19 +222,19 @@ class TurbulenceModel(Variables, Model):
             self.__removeVariablesAndProperties(lst, 'smagorinsky_constant^2')
 
         elif model_turb in ('Rij-epsilon', 'Rij-SSG'):
-            lst = ('r11', 'r22', 'r33',
-                   'r12', 'r13', 'r23',
-                   'epsilon')
+            lst = ('rij', 'epsilon')
             for v in lst:
-                self.setNewVariable(self.node_turb, v, label=v)
+                # Rij is now considered as a tensor (vector of length 6, since it is symmetric)
+                if v == 'rij':
+                    self.setNewVariable(self.node_turb, v, label='Rij', dim='6')
+                else:
+                    self.setNewVariable(self.node_turb, v, label=v)
             self.setNewProperty(self.node_turb, 'turbulent_viscosity')
             self.__updateInletsForTurbulence()
             self.__removeVariablesAndProperties(lst, 'smagorinsky_constant^2')
 
         elif model_turb == 'Rij-EBRSM':
-            lst = ('r11', 'r22', 'r33',
-                   'r12', 'r13', 'r23',
-                   'epsilon', 'alpha')
+            lst = ('rij', 'epsilon', 'alpha')
             for v in lst:
                 self.setNewVariable(self.node_turb, v, label=v)
             self.setNewProperty(self.node_turb, 'turbulent_viscosity')
@@ -252,9 +257,9 @@ class TurbulenceModel(Variables, Model):
 
             NumericalParamGlobalModel(self.case).setTimeSchemeOrder(2)
 
-            from code_saturne.Pages.NumericalParamEquationModel import NumericalParamEquatModel
-            NumericalParamEquatModel(self.case).setSchemeDefaultValues()
-            del NumericalParamEquatModel
+            from code_saturne.Pages.NumericalParamEquationModel import NumericalParamEquationModel
+            NumericalParamEquationModel(self.case).setSchemeDefaultValues()
+            del NumericalParamEquationModel
 
         elif model_turb == 'v2f-BL-v2/k':
             lst = ('k', 'epsilon', 'phi', 'alpha')
@@ -342,6 +347,8 @@ class TurbulenceModel(Variables, Model):
         Return wall function from advanced options.
         """
         wall_function = self.node_turb.xmlGetInt('wall_function')
+        if not wall_function:
+            wall_function = -1 # for next test
         if wall_function < 0 or wall_function > 5:
             wall_function = self.defaultTurbulenceValues()['wall_function']
             self.setWallFunction(wall_function)
