@@ -39,6 +39,7 @@ def run_command(cmd, stage, app, log):
 
     p = subprocess.Popen(cmd,
                          shell=True,
+                         universal_newlines=True,
                          stdout=subprocess.PIPE,
                          stderr=subprocess.STDOUT)
 
@@ -46,7 +47,7 @@ def run_command(cmd, stage, app, log):
     log.write(output[0])
 
     if p.returncode != 0:
-        sys.stderr.write("Error during " + string.lower(stage) +
+        sys.stderr.write("Error during " + stage.lower() +
                          " stage of " + app + ".\n")
         sys.stderr.write("See " + log.name + " for more information.\n")
         sys.exit(1)
@@ -65,6 +66,7 @@ def run_test(cmd):
 
     p = subprocess.Popen(cmd,
                          shell=True,
+                         universal_newlines=True,
                          stdout=subprocess.PIPE,
                          stderr=subprocess.STDOUT)
 
@@ -87,7 +89,7 @@ def check_directory():
     top_srcdir, script_name = os.path.split(script_path)
     abscwd = os.path.abspath(os.getcwd())
 
-    if abscwd.find(top_srcdir) == 0:
+    if not os.path.relpath(abscwd, top_srcdir)[0:2] == '..':
         message = \
 """
 The '%(script_name)s' installer script should not be run from inside the
@@ -133,8 +135,9 @@ def find_executable(names, env_var=None):
     # test it first.
 
     if env_var:
-        if os.environ.has_key(env_var):
-            return os.environ(env_var)
+        k = os.environ.get(env_var)
+        if k:
+            return k
         else:
             for a in sys.argv[1:]:
                 if a.find(env_var) == 0:
@@ -173,7 +176,10 @@ class Package:
         self.version = version
         self.archive = archive
         if self.archive:
-            self.url = url % self.archive
+            try:
+                self.url = url % self.archive
+            except Exception:
+                self.url = url
         else:
             self.url = None
 
@@ -262,6 +268,7 @@ class Package:
 
             p = subprocess.Popen('unzip ' + self.archive,
                                  shell=True,
+                                 universal_newlines=True,
                                  stdout=sys.stdout,
                                  stderr=sys.stderr)
             output = p.communicate()
@@ -388,6 +395,7 @@ class Package:
         try:
             p = subprocess.Popen(self.cc + ' -Xlinker --help',
                                  shell=True,
+                                 universal_newlines=True,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE)
             output = p.communicate()[0]
@@ -604,7 +612,7 @@ class Setup:
                     package="hdf5",
                     version="1.8.17",
                     archive="hdf5-1.8.17.tar.gz",
-                    url="http://www.hdfgroup.org/ftp/HDF5/releases/hdf5-1.8.17/src/%s")
+                    url="https://support.hdfgroup.org/ftp/HDF5/releases/hdf5-1.8/hdf5-1.8.17/src/%s")
 
         p = self.packages['hdf5']
         p.config_opts = "--enable-production"
@@ -614,10 +622,10 @@ class Setup:
         self.packages['cgns'] = \
             Package(name="CGNS",
                     description="CFD General Notation System",
-                    package="cgnslib",
-                    version="3.2.1",
-                    archive="cgnslib_3.2.1.tar.gz",
-                    url="http://sourceforge.net/projects/cgns/files/cgnslib_3.2/%s/download")
+                    package="cgns",
+                    version="3.3.1",
+                    archive="CGNS-3.3.1.tar.gz",
+                    url="https://github.com/CGNS/CGNS/archive/v3.3.1.tar.gz")
 
         p = self.packages['cgns']
         p.config_opts = "-DCGNS_ENABLE_64BIT=ON -DCGNS_ENABLE_SCOPING=ON"
@@ -628,8 +636,8 @@ class Setup:
             Package(name="MED",
                     description="Model for Exchange of Data",
                     package="med",
-                    version="3.2.0",
-                    archive="med-3.2.0.tar.gz",
+                    version="3.2.1",
+                    archive="med-3.2.1.tar.gz",
                     url="http://files.salome-platform.org/Salome/other/%s")
 
         p = self.packages['med']
@@ -666,7 +674,7 @@ class Setup:
                     package="scotch",
                     version="6.0.4",
                     archive="scotch_6.0.4.tar.gz",
-                    url="https://gforge.inria.fr/frs/download.php/34099/%s")
+                    url="https://gforge.inria.fr/frs/download.php/file/34618/%s")
 
     #---------------------------------------------------------------------------
 
@@ -757,7 +765,7 @@ to start the installation.
         # setup file reading
         #
         try:
-            setupFile = file('setup', mode='r')
+            setupFile = open('setup', mode='r')
         except IOError:
             sys.stderr.write('Error: opening setup file\n')
             sys.exit(1)
@@ -945,6 +953,7 @@ Check the setup file and some utilities presence.
                 sys.stdout.write("     Python version is ")
             p = subprocess.Popen(cmd,
                                  shell=True,
+                                 universal_newlines=True,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.STDOUT)
 
@@ -1063,7 +1072,8 @@ Check the setup file and some utilities presence.
             if hdf5.install_dir:
                 config_opts = config_opts + " --with-hdf5=" + hdf5.install_dir
                 med.config_opts += " --with-hdf5=" + hdf5.install_dir
-                cgns.config_opts += " -DHDF5_INCLUDE_PATH=" + hdf5.install_dir + "/include" \
+                cgns.config_opts += "-DCMAKE_PREFIX_PATH=" + hdf5.install_dir \
+                    + " -DHDF5_INCLUDE_PATH=" + hdf5.install_dir + "/include" \
                     + " -DHDF5_LIBRARY=" + hdf5.install_dir + "/lib/libhdf5.so"
 
         # CGNS
@@ -1137,7 +1147,7 @@ Check the setup file and some utilities presence.
             for lib in self.optlibs:
                 p = self.packages[lib]
                 if p.installation == 'yes':
-                    sys.stdout.write("Download of %s\n" % p.name)
+                    sys.stdout.write("Download of %s\n  (%s)\n" % (p.name, p.url))
                     p.download()
             self.download = 'no'
             self.write_setup()
@@ -1182,7 +1192,7 @@ Check the setup file and some utilities presence.
         #
         # setup file update
         #
-        sf = file(os.path.join(os.getcwd(), "setup"), mode='w')
+        sf = open(os.path.join(os.getcwd(), "setup"), mode='w')
 
         setupMain = \
 """#========================================================
@@ -1282,11 +1292,13 @@ salome    %(salome)s
 #
 #   For Linux workstations, HDF5, CGNS, and even MED
 # packages may be available through the package manager.
-# HDF5 is also often available on large systems such as
-# IBM Blue Gene or Cray XT/XE/XC/XK.
+# HDF5 is also often available on large systems.
+# When building with SALOME, the platform distribution's
+# packages may be used, by setting 'salome' in the
+# matching entry under the "Use" column.
 #
 # Scotch and Pt-Scotch are available in some Linux
-# distributtions, but may be built with options
+# distributions, but may be built with options
 # incompatible with non-threaded Code_Saturne runs.
 #
 #   To install CGNS or ParMetis, the CMake
@@ -1373,7 +1385,7 @@ You can have a look at the log file meanwhile.
 Before using Code_Saturne, please update your environment with:
 
   cspath=%(cspath)s
-  alias code_saturne="$cspath/bin"
+  alias code_saturne="$cspath/code_saturne"
 
 The documentation should then be available through the commands:
   code_saturne info -g refcard
