@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -40,8 +40,9 @@ import logging
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtGui import QDialog, QMessageBox, QDockWidget, QCursor, QApplication, QTabWidget
-from PyQt4.QtCore import Qt, QObject, SIGNAL
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Salome modules
@@ -66,7 +67,7 @@ import CFDSTUDYGUI_SolverGUI
 from CFDSTUDYGUI_Commons import sg, sgPyQt
 from CFDSTUDYGUI_Commons import CFD_Saturne, CFD_Neptune
 from CFDSTUDYGUI_Commons import CheckCFD_CodeEnv
-
+from CFDSTUDYGUI_Message import cfdstudyMess
 #-------------------------------------------------------------------------------
 # log config
 #-------------------------------------------------------------------------------
@@ -90,6 +91,10 @@ ObjectTR = QObject()
 
 DEFAULT_EDITOR_NAME = ObjectTR.tr("CFDSTUDY_PREF_EDITOR")
 
+DEFAULT_READER_NAME = ObjectTR.tr("CFDSTUDY_PREF_READER")
+
+DEFAULT_DISPLAY_VIEWER_NAME = ObjectTR.tr("CFDSTUDY_PREF_DISPLAY_VIEWER")
+
 #-------------------------------------------------------------------------------
 # Callback GUI functions
 #-------------------------------------------------------------------------------
@@ -102,6 +107,10 @@ def initialize():
     log.debug("initialize")
     if not sgPyQt.hasSetting( "CFDSTUDY", "ExternalEditor"):
         sgPyQt.addSetting( "CFDSTUDY", "ExternalEditor", DEFAULT_EDITOR_NAME )
+    if not sgPyQt.hasSetting( "CFDSTUDY", "ExternalReader"):
+        sgPyQt.addSetting( "CFDSTUDY", "ExternalReader", DEFAULT_READER_NAME )
+    if not sgPyQt.hasSetting( "CFDSTUDY", "ExternalDisplay"):
+        sgPyQt.addSetting( "CFDSTUDY", "ExternalDisplay", DEFAULT_DISPLAY_VIEWER_NAME )
     pass
 
 
@@ -163,7 +172,7 @@ def setWorkSpace(ws):
     dsk = sgPyQt.getDesktop()
     _DesktopMgr.setWorkspace(dsk, ws)
     ActionHandler = _DesktopMgr.getActionHandler(dsk)
-
+    ActionHandler.connectSolverGUI()
 
 def createPreferences():
     """
@@ -172,9 +181,15 @@ def createPreferences():
     log.debug("createPreferences")
     sgPyQt = CFDSTUDYGUI_DataModel.sgPyQt
     genTab = sgPyQt.addPreference(ObjectTR.tr("CFDSTUDY_PREF_GEN_GROUP"))
-    editorGroup = sgPyQt.addPreference("Editor",genTab)
-    editor      = sgPyQt.addPreference("External Editor",editorGroup,SalomePyQt.PT_String, "CFDSTUDY","ExternalEditor")
-    Reader      = sgPyQt.addPreference("External Reader",editorGroup,SalomePyQt.PT_String, "CFDSTUDY","ExternalReader")
+    EditorField = str(ObjectTR.tr("EDITOR"))
+    editorGroup = sgPyQt.addPreference(EditorField,genTab)
+    externalEditorField = str(ObjectTR.tr("EXTERNAL_EDITOR"))
+    externalReaderField= str(ObjectTR.tr("EXTERNAL_READER"))
+    editor      = sgPyQt.addPreference(externalEditorField,editorGroup,SalomePyQt.PT_String, "CFDSTUDY","ExternalEditor")
+    Reader      = sgPyQt.addPreference(externalReaderField,editorGroup,SalomePyQt.PT_String, "CFDSTUDY","ExternalReader")
+    externalDisplayField = str(ObjectTR.tr("EXTERNAL_DISPLAY"))
+    displayViewer        = sgPyQt.addPreference(externalDisplayField,editorGroup,SalomePyQt.PT_String, "CFDSTUDY","ExternalDisplay")
+
 
 
 def preferenceChanged( section, setting ):
@@ -191,7 +206,6 @@ def activate():
     """
     log.debug("activate")
     global d_activation, studyId
-
     dsk = sgPyQt.getDesktop()
     studyId = sgPyQt.getStudyId()
     dsk.setTabPosition(Qt.RightDockWidgetArea,QTabWidget.South)
@@ -204,7 +218,7 @@ def activate():
     # instance of the CFDSTUDYGUI_ActionsHandler class for the current desktop
     ActionHandler = _DesktopMgr.getActionHandler(dsk)
 
-    if studyId not in d_activation.keys():
+    if studyId not in list(d_activation.keys()):
         d_activation[studyId] = 1
 
     if d_activation[studyId] == 1:
@@ -225,9 +239,8 @@ def activate():
 
         if env_neptune:
             if mess2 != "":
-                Error = "Error: "+ ObjectTR.tr("CFDSTUDY_INVALID_ENV")
-                QMessageBox.critical(ActionHandler.dskAgent().workspace(),
-                                     Error, mess2, QMessageBox.Ok, 0)
+                mess = cfdstudyMess.trMessage(ObjectTR.tr("CFDSTUDY_INVALID_ENV"),[]) + " ; "+ mess2
+                cfdstudyMess.aboutMessage(mess)
                 d_activation[studyId] = 1
                 return False
             else:
@@ -235,19 +248,14 @@ def activate():
 
         elif env_saturne:
             if mess1 != "":
-                Error = "Error: "+ ObjectTR.tr("CFDSTUDY_INVALID_ENV")
-                QMessageBox.critical(ActionHandler.dskAgent().workspace(),
-                                     Error, mess1, QMessageBox.Ok, 0)
+                mess = cfdstudyMess.trMessage(ObjectTR.tr("CFDSTUDY_INVALID_ENV"),[]) + " ; "+ mess2
+                cfdstudyMess.aboutMessage(mess)
                 d_activation[studyId] = 1
                 return False
             else:
                 ActionHandler.DialogCollector.InfoDialog.setCode(env_saturne, False)
 
-        ActionHandler.slotStudyLocation()
-
-    ActionHandler.connect(ActionHandler._SalomeSelection,
-                          SIGNAL('currentSelectionChanged()'),
-                          ActionHandler.updateActions)
+    ActionHandler._SalomeSelection.currentSelectionChanged.connect(ActionHandler.updateActions)
 
     ActionHandler.connectSolverGUI()
     ActionHandler.updateObjBrowser()
@@ -256,7 +264,7 @@ def activate():
     for dock in sgPyQt.getDesktop().findChildren(QDockWidget):
         dockTitle = dock.windowTitle()
         log.debug("activate -> QDockWidget: %s" % dockTitle)
-        if dockTitle in (u"Python Console", u"Console Python",  u"Message Window"):
+        if dockTitle in ("Python Console", "Console Python",  "Message Window"):
             dock.setVisible(False)
 
     return True
@@ -265,7 +273,7 @@ def activate():
 def setSettings():
     """
     Stores the selected CFD code and updates action according with current
-    selection and study states in the dekstop manager.
+    selection and study states in the desktop manager.
     """
     log.debug("setSettings")
 
@@ -281,7 +289,9 @@ def deactivate():
     log.debug("deactivate")
     dsk = sgPyQt.getDesktop()
     ActionHandler = _DesktopMgr.getActionHandler(dsk)
-    ActionHandler.disconnect(ActionHandler._SalomeSelection, SIGNAL('currentSelectionChanged()'), ActionHandler.updateActions)
+
+    ActionHandler._SalomeSelection.currentSelectionChanged.disconnect(ActionHandler.updateActions)
+
     ActionHandler.disconnectSolverGUI()
 
 
@@ -327,5 +337,10 @@ def createPopupMenu(popup, context):
                                     popup.removeAction(ActionHandler.commonAction(CFDSTUDYGUI_ActionsHandler.DisplayOnlyGroupMESHAction))
 
                             else:
-                                if not CFDSTUDYGUI_DataModel.isLinkPathObject(sobj):
-                                    ActionHandler.customPopup(id, popup)
+                                ActionHandler.customPopup(id, popup)
+                                fathername = sobj.GetFather().GetName()
+                                if  fathername in ["RESU","RESU_COUPLING"]:
+                                    if CFDSTUDYGUI_DataModel.checkType(sobj, CFDSTUDYGUI_DataModel.dict_object["RESUSubFolder"]) or \
+                                        CFDSTUDYGUI_DataModel.checkType(sobj, CFDSTUDYGUI_DataModel.dict_object["RESU_COUPLINGSubFolder"]):
+                                        ActionHandler.customPopup(id, popup)
+                                        ActionHandler.commonAction(ActionHandler.RemoveAction).setEnabled(True)
