@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2016 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -41,8 +41,9 @@ import os, sys, string, logging
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtGui import QDockWidget, QTreeView, QMessageBox
-from PyQt4.QtCore import Qt, QObject, SIGNAL
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Salome modules
@@ -159,23 +160,21 @@ class CFDSTUDYGUI_SolverGUI(QObject):
             aTitle = "unnamed"
             if aCase != None:
                 if findDockWindow(aTitle, aCase.GetName(), aCase.GetFather().GetName()):
-                    mess = "A case not finished to be set is already opened"
+                    mess = "A new case is already opened"
                     QMessageBox.warning(None, "Warning: ",mess)
                     return
-
         if aCase != None:
             aChildList = CFDSTUDYGUI_DataModel.ScanChildren(aCase, "^DATA$")
             if not len(aChildList)== 1:
                 # no DATA folder
-                mess = "There are not DATA directory  in the case"
+                mess = "DATA directory is not present in the case"
                 QMessageBox.warning(None, "Warning: ", mess)
                 return None
 
             aStartPath = CFDSTUDYGUI_DataModel._GetPath(aChildList[0])
             if aStartPath != None and aStartPath != '':
                 os.chdir(aStartPath)
-
-        mw = self.lauchGUI(WorkSpace, aCase, sobjXML, Args)
+        mw = self.launchGUI(WorkSpace, aCase, sobjXML, Args)
         if mw != None:
             self._CurrentWindow = mw
 
@@ -203,14 +202,13 @@ class CFDSTUDYGUI_SolverGUI(QObject):
             return False
 
 
-    def onSaveXmlFile(self):
-        log.debug("onSaveXmlFile")
+    def SaveXmlFile(self):
+        log.debug("SaveXmlFile")
+        xml_file = None
         if self._CurrentWindow != None:
-            if self._CurrentWindow.case['xmlfile'] != "":
-                self._CurrentWindow.fileSave()
-            else:
-                self.SaveAsXmlFile()
-
+            self._CurrentWindow.fileSave()
+            xml_file = self._CurrentWindow.case['xmlfile']
+        return xml_file
 
     def SaveAsXmlFile(self):
         """
@@ -220,14 +218,14 @@ class CFDSTUDYGUI_SolverGUI(QObject):
         """
         old_xml_file = None
         xml_file = None
-
         if self._CurrentWindow != None:
             old_xml_file = self._CurrentWindow.case['xmlfile']
             self._CurrentWindow.fileSaveAs()
             xml_file = self._CurrentWindow.case['xmlfile']
-
             if old_xml_file == "":
                 old_xml_file = None
+            if xml_file == "":
+                xml_file = None
 
         return old_xml_file, xml_file
 
@@ -258,6 +256,13 @@ class CFDSTUDYGUI_SolverGUI(QObject):
         if self._CurrentWindow != None:
             self._CurrentWindow.slotRedo()
 
+    def onPreproMode(self):
+        if self._CurrentWindow != None:
+            self._CurrentWindow.slotPreproMode()
+
+    def onCalculationMode(self):
+        if self._CurrentWindow != None:
+            self._CurrentWindow.slotCalculationMode()
 
     def onOpenShell(self):
         if self._CurrentWindow != None:
@@ -342,13 +347,12 @@ class CFDSTUDYGUI_SolverGUI(QObject):
         return aTitle
 
 
-    def lauchGUI(self, WorkSpace, aCase, sobjXML, Args):
+    def launchGUI(self, WorkSpace, aCase, sobjXML, Args):
         """
         mw.dockWidgetBrowser is the Browser of the CFD MainView
         """
-        log.debug("lauchGUI")
+        log.debug("launchGUI")
         from cs_gui import process_cmd_line
-
         if CFD_Code() == CFD_Saturne:
             from cs_package import package
             from code_saturne.Base.MainView import MainView
@@ -364,7 +368,12 @@ class CFDSTUDYGUI_SolverGUI(QObject):
         self.Workspace = WorkSpace
         pkg = package()
         case, splash = process_cmd_line(Args)
-        mw = MainView(pkg, case, aCase)
+        try:
+            mw = MainView(pkg, case, aCase)
+        except:
+            mess = "Error in Opening CFD GUI"
+            QMessageBox.warning(None, "Warning", mess, QMessageBox.Ok, QMessageBox.NoButton)
+            return None
 
         # Put the standard panel of the MainView inside a QDockWidget
         # in the SALOME Desktop
@@ -375,7 +384,6 @@ class CFDSTUDYGUI_SolverGUI(QObject):
         dock.setWidget(mw.frame)
         dock.setMinimumWidth(520)
         dsk.addDockWidget(Qt.RightDockWidgetArea, dock)
-
         dock.setVisible(True)
         dock.show()
 
@@ -396,13 +404,10 @@ class CFDSTUDYGUI_SolverGUI(QObject):
         aCaseCFD  = aCase
         xmlFileName = str(Title)
         _c_CFDGUI.set_d_CfdCases(studyId, dock, mw.dockWidgetBrowser, mw, aStudyCFD, aCaseCFD, xmlFileName, sobjXML)
-
-        self.connect(dock, SIGNAL("visibilityChanged(bool)"), self.setdockWindowBrowserActivated)
-        self.connect(mw.dockWidgetBrowser, SIGNAL("visibilityChanged(bool)"),self.setdockWindowActivated)
-
-        self.connect(dock.toggleViewAction(), SIGNAL("toggled(bool)"), self.setdockWB)
-        self.connect(mw.dockWidgetBrowser.toggleViewAction(), SIGNAL("toggled(bool)"), self.setdock)
-
+        dock.visibilityChanged["bool"].connect(self.setdockWindowBrowserActivated)
+        mw.dockWidgetBrowser.visibilityChanged["bool"].connect(self.setdockWindowActivated)
+        dock.toggleViewAction().toggled["bool"].connect(self.setdockWB)
+        mw.dockWidgetBrowser.toggleViewAction().toggled["bool"].connect(self.setdock)
         _c_CFDGUI.tabifyDockWindows(dsk, studyId)
         self.showDockWindows(studyId, xmlFileName, aCaseCFD.GetName(), aStudyCFD.GetName())
         updateObjectBrowser()
@@ -411,6 +416,10 @@ class CFDSTUDYGUI_SolverGUI(QObject):
 
 
     def setdockWB(self, istoggled):
+        """
+        istoggled referred to CFD Window dock widget
+        """
+        log.debug("setdockWB")
         studyId = sgPyQt.getStudyId()
         dock = self.sender().parent()
         log.debug("setdockWB -> %s" % (dock,))
@@ -418,7 +427,7 @@ class CFDSTUDYGUI_SolverGUI(QObject):
         if _c_CFDGUI != None:
             dockWB = _c_CFDGUI.getdockWB(studyId, dock)
             if dockWB != None:
-                dockWB.setVisible(dock.isVisible())
+                dockWB.setVisible(istoggled) #dock.isVisible())
                 if istoggled:
                     dock.show()
                     dock.raise_()
@@ -433,6 +442,10 @@ class CFDSTUDYGUI_SolverGUI(QObject):
 
 
     def setdock(self, istoggled):
+        """
+        istoggled referred to Window browser dock widget
+        """
+        log.debug("setdock")
         studyId = sgPyQt.getStudyId()
         dockWB = self.sender().parent()
         log.debug("setdock -> %s" % (dockWB,))
@@ -440,7 +453,7 @@ class CFDSTUDYGUI_SolverGUI(QObject):
         if _c_CFDGUI != None:
             dock = _c_CFDGUI.getdock(studyId, dockWB)
             if dock != None:
-                dock.setVisible(dockWB.isVisible())
+                dock.setVisible(istoggled) #dockWB.isVisible())
                 if istoggled:
                     dock.show()
                     dock.raise_()
@@ -459,6 +472,7 @@ class CFDSTUDYGUI_SolverGUI(QObject):
         mw is the Main CFD window allocated by MainView code
         When we click on a cfd study window tab, the cfd study window appears and the associated CFD window browser raises too
         """
+
         studyId = sgPyQt.getStudyId()
         dock = self.sender()
         log.debug("setdockWindowBrowserActivated -> %s" % (dock,))
@@ -559,6 +573,17 @@ class CFDSTUDYGUI_SolverGUI(QObject):
         return case
 
 
+    def removeDockWindowfromStudyAndCaseNames(self, studyCFDName, caseName):
+        """
+        Close the CFD_study_dock_windows if opened from close Study popup menu into the object browser
+        """
+        log.debug("removeDockWindow -> %s %s" % (studyCFDName, caseName))
+        dsk = sgPyQt.getDesktop()
+        studyId = sgPyQt.getStudyId()
+        if _c_CFDGUI != None:
+           _c_CFDGUI.delDockfromStudyAndCaseNames(dsk, studyId, studyCFDName, caseName)
+
+
     def removeDockWindow(self, studyCFDName, caseName, xmlName):
         """
         Close the CFD_study_dock_windows from remove  popup menu in object browser
