@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -116,10 +116,6 @@ BEGIN_C_DECLS
  * - \f$ Rhs \f$ has already been initialized before calling bilsca!
  * - mind the minus sign
  *
- * Options for the diffusive scheme:
- * - idftnp = 1: scalar diffusivity
- * - idftnp = 6: symmetric tensor diffusivity
- *
  * Options for the convective scheme:
  * - blencp = 0: upwind scheme for the advection
  * - blencp = 1: no upwind scheme except in the slope test
@@ -144,7 +140,9 @@ BEGIN_C_DECLS
  * \param[in]     var_cal_opt   pointer to a cs_var_cal_opt_t structure which
  *                              contains variable calculation options
  * \param[in]     pvar          solved variable (current time step)
+ *                              may be NULL if pvara != NULL
  * \param[in]     pvara         solved variable (previous time step)
+ *                              may be NULL if pvar != NULL
  * \param[in]     coefap        boundary condition array for the variable
  *                               (explicit part)
  * \param[in]     coefbp        boundary condition array for the variable
@@ -223,8 +221,10 @@ cs_balance_scalar(int                idtvar,
     var_cal_opt_loc.imligr = var_cal_opt->imligr;
     var_cal_opt_loc.ircflu = var_cal_opt->ircflu;
     var_cal_opt_loc.iwgrec = 0;
+    var_cal_opt_loc.icoupl = -1;
     var_cal_opt_loc.thetav = var_cal_opt->thetav;
     var_cal_opt_loc.blencv = var_cal_opt->blencv;
+    var_cal_opt_loc.blend_st = 0.;
     var_cal_opt_loc.epsilo = -1.;
     var_cal_opt_loc.epsrsm = -1.;
     var_cal_opt_loc.epsrgr = var_cal_opt->epsrgr;
@@ -239,7 +239,7 @@ cs_balance_scalar(int                idtvar,
   }
 
   /* Scalar diffusivity */
-  if (idftnp == 1) {
+  if (idftnp & CS_ISOTROPIC_DIFFUSION) {
     if (imucpp == 0) {
       cs_convection_diffusion_scalar(idtvar,
                                      f_id,
@@ -283,7 +283,7 @@ cs_balance_scalar(int                idtvar,
     }
   }
   /* Symmetric tensor diffusivity */
-  else if (idftnp == 6) {
+  else if (idftnp & CS_ANISOTROPIC_DIFFUSION) {
     var_cal_opt_loc.idiff = 0;
     /* Convective part */
     if (imucpp == 0 && iconvp == 1) {
@@ -375,10 +375,6 @@ cs_balance_scalar(int                idtvar,
  * - \f$ \vect{Rhs} \f$ has already been initialized before calling bilscv!
  * - mind the sign minus
  *
- * Options for the diffusive scheme:
- * - idftnp = 1: scalar diffusivity
- * - idftnp = 6: symmetric tensor diffusivity
- *
  * Options for the convective scheme:
  * - blencp = 0: upwind scheme for the advection
  * - blencp = 1: no upwind scheme except in the slope test
@@ -416,6 +412,11 @@ cs_balance_scalar(int                idtvar,
  *                               at boundary faces for the r.h.s.
  * \param[in]     secvif        secondary viscosity at interior faces
  * \param[in]     secvib        secondary viscosity at boundary faces
+ * \param[in]     viscel        symmetric cell tensor \f$ \tens{\mu}_\celli \f$
+ * \param[in]     weighf        internal face weight between cells i j in case
+ *                               of tensor diffusion
+ * \param[in]     weighb        boundary face weight for cells i in case
+ *                               of tensor diffusion
  * \param[in]     icvflb        global indicator of boundary convection flux
  *                               - 0 upwind scheme at all boundary faces
  *                               - 1 imposed flux at some boundary faces
@@ -441,10 +442,13 @@ cs_balance_vector(int                  idtvar,
                   const cs_real_33_t   cofbfv[],
                   const cs_real_t      i_massflux[],
                   const cs_real_t      b_massflux[],
-                  cs_real_t            i_visc[],
+                  const cs_real_t      i_visc[],
                   const cs_real_t      b_visc[],
                   const cs_real_t      secvif[],
                   const cs_real_t      secvib[],
+                  cs_real_6_t          viscel[],
+                  const cs_real_2_t    weighf[],
+                  const cs_real_t      weighb[],
                   int                  icvflb,
                   const int            icvfli[],
                   cs_real_3_t          smbr[])
@@ -473,6 +477,7 @@ cs_balance_vector(int                  idtvar,
     var_cal_opt_loc.iwgrec = 0;
     var_cal_opt_loc.thetav = var_cal_opt->thetav;
     var_cal_opt_loc.blencv = var_cal_opt->blencv;
+    var_cal_opt_loc.blend_st = 0.;
     var_cal_opt_loc.epsilo = -1.;
     var_cal_opt_loc.epsrsm = -1.;
     var_cal_opt_loc.epsrgr = var_cal_opt->epsrgr;
@@ -487,7 +492,7 @@ cs_balance_vector(int                  idtvar,
   }
 
   /* Scalar diffusivity */
-  if (idftnp == 1) {
+  if (idftnp & CS_ISOTROPIC_DIFFUSION) {
     cs_convection_diffusion_vector(idtvar,
                                    f_id,
                                    var_cal_opt_loc,
@@ -507,10 +512,11 @@ cs_balance_vector(int                  idtvar,
                                    i_visc,
                                    b_visc,
                                    secvif,
+                                   secvib,
                                    smbr);
   }
   /* Symmetric tensor diffusivity */
-  else if (idftnp == 6) {
+  else if (idftnp & CS_ANISOTROPIC_DIFFUSION) {
     /* ! Nor diffusive part neither secondary viscosity or transpose of gradient */
     var_cal_opt_loc.idiff = 0;
     /* Convective part */
@@ -534,27 +540,53 @@ cs_balance_vector(int                  idtvar,
                                      i_visc,
                                      b_visc,
                                      secvif,
+                                     secvib,
                                      smbr);
 
     }
 
     /* Diffusive part (with a 3x3 symmetric diffusivity) */
-    if (idiffp == 1) {
-      cs_anisotropic_diffusion_vector(idtvar,
-                                      f_id,
-                                      var_cal_opt_loc,
-                                      inc,
-                                      ivisep,
-                                      pvar,
-                                      pvara,
-                                      coefav,
-                                      coefbv,
-                                      cofafv,
-                                      cofbfv,
-                                      (const cs_real_33_t *)i_visc,
-                                      b_visc,
-                                      secvif,
-                                      smbr);
+
+    /* either Daly-Harlow type i.e. Nabla(v).K */
+    if (idiffp == 1 && idftnp & CS_ANISOTROPIC_RIGHT_DIFFUSION) {
+      /* ! Neither diffusive part neither secondary viscosity
+         nor transpose of gradient */
+      cs_anisotropic_right_diffusion_vector(idtvar,
+                                            f_id,
+                                            var_cal_opt_loc,
+                                            inc,
+                                            pvar,
+                                            pvara,
+                                            coefav,
+                                            coefbv,
+                                            cofafv,
+                                            cofbfv,
+                                            i_visc,
+                                            b_visc,
+                                            secvif,
+                                            viscel,
+                                            weighf,
+                                            weighb,
+                                            smbr);
+    }
+
+    /* or K.Nabla(v) ) */
+    else if (idiffp == 1 && idftnp & CS_ANISOTROPIC_LEFT_DIFFUSION) {
+      cs_anisotropic_left_diffusion_vector(idtvar,
+                                           f_id,
+                                           var_cal_opt_loc,
+                                           inc,
+                                           ivisep,
+                                           pvar,
+                                           pvara,
+                                           coefav,
+                                           coefbv,
+                                           cofafv,
+                                           cofbfv,
+                                           (const cs_real_33_t *)i_visc,
+                                           b_visc,
+                                           secvif,
+                                           smbr);
     }
   }
 }
@@ -577,10 +609,6 @@ cs_balance_vector(int                  idtvar,
  * - \f$ \tens{Rhs} \f$ has already been initialized before calling bilscts!
  * - mind the sign minus
  *
- * Options for the diffusive scheme:
- * - idftnp = 1: scalar diffusivity
- * - idftnp = 6: symmetric tensor diffusivity
- *
  * Options for the convective scheme:
  * - blencp = 0: upwind scheme for the advection
  * - blencp = 1: no upwind scheme except in the slope test
@@ -620,7 +648,7 @@ cs_balance_vector(int                  idtvar,
  * \param[in]     icvfli        boundary face indicator array of convection flux
  *                               - 0 upwind scheme
  *                               - 1 imposed flux
- * \param[in,out] smbr          right hand side \f$ \vect{Rhs} \f$
+ * \param[in,out] smbrp         right hand side \f$ \vect{Rhs} \f$
  */
 /*----------------------------------------------------------------------------*/
 
@@ -646,7 +674,6 @@ cs_balance_tensor(int                 idtvar,
                   int                 icvflb,
                   const int           icvfli[],
                   cs_real_6_t         smbrp[])
-
 {
   /* Local variables */
   int iconvp = var_cal_opt->iconv;
@@ -672,6 +699,7 @@ cs_balance_tensor(int                 idtvar,
     var_cal_opt_loc.iwgrec = 0;
     var_cal_opt_loc.thetav = var_cal_opt->thetav;
     var_cal_opt_loc.blencv = var_cal_opt->blencv;
+    var_cal_opt_loc.blend_st = 0.;
     var_cal_opt_loc.epsilo = -1.;
     var_cal_opt_loc.epsrsm = -1.;
     var_cal_opt_loc.epsrgr = var_cal_opt->epsrgr;
@@ -686,7 +714,7 @@ cs_balance_tensor(int                 idtvar,
   }
 
   /* Scalar diffusivity */
-  if (idftnp == 1) {
+  if (idftnp & CS_ISOTROPIC_DIFFUSION) {
     cs_convection_diffusion_tensor(idtvar,
                                    f_id,
                                    var_cal_opt_loc,
@@ -706,9 +734,9 @@ cs_balance_tensor(int                 idtvar,
                                    smbrp);
   }
   /* Symmetric tensor diffusivity */
-  else if (idftnp == 6) {
-    /* Nor diffusive part neither secondary viscosity or transpose of gradient */
-    var_cal_opt_loc.idiff  = 0;
+  else if (idftnp & CS_ANISOTROPIC_RIGHT_DIFFUSION) {
+    /* No diffusive part */
+    var_cal_opt_loc.idiff = 0;
     /* Convective part */
     if (iconvp == 1) {
       cs_convection_diffusion_tensor(idtvar,
