@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -33,6 +33,7 @@
 
 #include "cs_base.h"
 #include "cs_halo.h"
+#include "cs_internal_coupling.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -54,7 +55,8 @@ typedef enum {
 
   CS_GRADIENT_ITER,              /* Iterative */
   CS_GRADIENT_LSQ,               /* Least-squares */
-  CS_GRADIENT_LSQ_ITER_OLD,      /* LSQ followed by iterative (old) */
+  CS_GRADIENT_LSQ_ITER,          /* conservative gradient reconstructed
+                                    with LSQ */
   CS_GRADIENT_ITER_OLD           /* Iterative (old) */
 
 } cs_gradient_type_t;
@@ -170,28 +172,28 @@ cs_gradient_finalize(void);
  * tensor field.
  *
  * parameters:
- *   var_name       <-- variable name
- *   gradient_type  <-- gradient type
- *   halo_type      <-- halo type
- *   inc            <-- if 0, solve on increment; 1 otherwise
- *   recompute_cocg <-- should COCG FV quantities be recomputed ?
- *   n_r_sweeps     <-- if > 1, number of reconstruction sweeps
- *   tr_dim         <-- 2 for tensor with periodicity of rotation,
- *                      0 otherwise
- *   hyd_p_flag     <-- flag for hydrostatic pressure
- *   w_stride       <-- stride for weighting coefficient
- *   verbosity      <-- verbosity level
- *   clip_mode      <-- clipping mode
- *   epsilon        <-- precision for iterative gradient calculation
- *   extrap         <-- boundary gradient extrapolation coefficient
- *   clip_coeff     <-- clipping coefficient
- *   f_ext          <-- exterior force generating the hydrostatic pressure
- *   bc_coeff_a     <-- boundary condition term a
- *   bc_coeff_b     <-- boundary condition term b
- *   var            <-> gradient's base variable
- *   c_weight       <-- weighted gradient coefficient variable,
- *                      or NULL
- *   grad           --> gradient
+ *   var_name        <-- variable name
+ *   gradient_type   <-- gradient type
+ *   halo_type       <-- halo type
+ *   inc             <-- if 0, solve on increment; 1 otherwise
+ *   recompute_cocg  <-- should COCG FV quantities be recomputed ?
+ *   n_r_sweeps      <-- if > 1, number of reconstruction sweeps
+ *   tr_dim          <-- 2 for tensor with periodicity of rotation,
+ *                       0 otherwise
+ *   hyd_p_flag      <-- flag for hydrostatic pressure
+ *   w_stride        <-- stride for weighting coefficient
+ *   verbosity       <-- verbosity level
+ *   clip_mode       <-- clipping mode
+ *   epsilon         <-- precision for iterative gradient calculation
+ *   extrap          <-- boundary gradient extrapolation coefficient
+ *   clip_coeff      <-- clipping coefficient
+ *   f_ext           <-- exterior force generating the hydrostatic pressure
+ *   bc_coeff_a      <-- boundary condition term a
+ *   bc_coeff_b      <-- boundary condition term b
+ *   var             <-> gradient's base variable
+ *   c_weight        <-> weighted gradient coefficient variable, or NULL
+ *   cpl             <-> structure associated with internal coupling, or NULL
+ *   grad            --> gradient
  *----------------------------------------------------------------------------*/
 
 void
@@ -214,26 +216,33 @@ cs_gradient_scalar(const char                *var_name,
                    const cs_real_t            bc_coeff_b[],
                    cs_real_t        *restrict var,
                    cs_real_t        *restrict c_weight,
+                   cs_internal_coupling_t    *cpl,
                    cs_real_3_t      *restrict grad);
 
-/*----------------------------------------------------------------------------
- * Compute cell gradient of a vector field.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute cell gradient of vector field.
  *
- * parameters:
- *   var_name       <-- variable name
- *   gradient_type  <-- gradient type
- *   halo_type      <-- halo type
- *   inc            <-- if 0, solve on increment; 1 otherwise
- *   n_r_sweeps     <-- if > 1, number of reconstruction sweeps
- *   verbosity      <-- verbosity level
- *   clip_mode      <-- clipping mode
- *   epsilon        <-- precision for iterative gradient calculation
- *   clip_coeff     <-- clipping coefficient
- *   bc_coeff_a     <-- boundary condition term a
- *   bc_coeff_b     <-- boundary condition term b
- *   var            <-> gradient's base variable
- *   gradv          --> gradient (du_i/dx_j : gradv[][i][j])
- *----------------------------------------------------------------------------*/
+ * \param[in]       var_name        variable name
+ * \param[in]       gradient_type   gradient type
+ * \param[in]       halo_type       halo type
+ * \param[in]       inc             if 0, solve on increment; 1 otherwise
+ * \param[in]       n_r_sweeps      if > 1, number of reconstruction sweeps
+ * \param[in]       verbosity       verbosity level
+ * \param[in]       clip_mode       clipping mode
+ * \param[in]       epsilon         precision for iterative gradient calculation
+ * \param[in]       clip_coeff      clipping coefficient
+ * \param[in]       bc_coeff_a      boundary condition term a
+ * \param[in]       bc_coeff_b      boundary condition term b
+ * \param[in, out]  var             gradient's base variable
+ * \param[in, out]  c_weight        weighted gradient coefficient variable,
+ *                                  or NULL
+ * \param[in, out]  cpl             structure associated with internal coupling,
+ *                                  or NULL
+ * \param[out]      gradv           gradient
+                                    (\f$ \der{u_i}{x_j} \f$ is gradv[][i][j])
+ */
+/*----------------------------------------------------------------------------*/
 
 void
 cs_gradient_vector(const char                *var_name,
@@ -248,6 +257,8 @@ cs_gradient_vector(const char                *var_name,
                    const cs_real_3_t          bc_coeff_a[],
                    const cs_real_33_t         bc_coeff_b[],
                    cs_real_3_t      *restrict var,
+                   cs_real_t        *restrict c_weight,
+                   cs_internal_coupling_t    *cpl,
                    cs_real_33_t     *restrict gradv);
 
 /*----------------------------------------------------------------------------*/
@@ -286,6 +297,148 @@ cs_gradient_tensor(const char                *var_name,
                    cs_real_6_t      *restrict var,
                    cs_real_63_t     *restrict grad);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute cell gradient of scalar field or component of vector or
+ *         tensor field.
+ *
+ * This variant of the \ref cs_gradient_scalar function assumes ghost cell
+ * values for input arrays (var and optionally c_weight)
+ * have already been synchronized.
+ *
+ * \param[in]       var_name        variable name
+ * \param[in]       gradient_type   gradient type
+ * \param[in]       halo_type       halo type
+ * \param[in]       inc             if 0, solve on increment; 1 otherwise
+ * \param[in]       recompute_cocg  should COCG FV quantities be recomputed ?
+ * \param[in]       n_r_sweeps      if > 1, number of reconstruction sweeps
+ * \param[in]       tr_dim          2 for tensor with periodicity of rotation,
+ *                                  0 otherwise
+ * \param[in]       hyd_p_flag      flag for hydrostatic pressure
+ * \param[in]       w_stride        stride for weighting coefficient
+ * \param[in]       verbosity       verbosity level
+ * \param[in]       clip_mode       clipping mode
+ * \param[in]       epsilon         precision for iterative gradient calculation
+ * \param[in]       extrap          boundary gradient extrapolation coefficient
+ * \param[in]       clip_coeff      clipping coefficient
+ * \param[in]       f_ext           exterior force generating
+ *                                  the hydrostatic pressure
+ * \param[in]       bc_coeff_a      boundary condition term a
+ * \param[in]       bc_coeff_b      boundary condition term b
+ * \param[in]       var             gradient's base variable
+ * \param[in]       c_weight        weighted gradient coefficient variable,
+ *                                  or NULL
+ * \param[in, out]  cpl             structure associated with internal coupling,
+ *                                  or NULL
+ * \param[out]      grad            gradient
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_gradient_scalar_synced_input(const char                 *var_name,
+                                cs_gradient_type_t          gradient_type,
+                                cs_halo_type_t              halo_type,
+                                int                         inc,
+                                bool                        recompute_cocg,
+                                int                         n_r_sweeps,
+                                int                         tr_dim,
+                                int                         hyd_p_flag,
+                                int                         w_stride,
+                                int                         verbosity,
+                                int                         clip_mode,
+                                double                      epsilon,
+                                double                      extrap,
+                                double                      clip_coeff,
+                                cs_real_t                   f_ext[][3],
+                                const cs_real_t             bc_coeff_a[],
+                                const cs_real_t             bc_coeff_b[],
+                                const cs_real_t             var[restrict],
+                                const cs_real_t             c_weight[restrict],
+                                const cs_internal_coupling_t  *cpl,
+                                cs_real_t                   grad[restrict][3]);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute cell gradient of vector field.
+ *
+ * This variant of the \ref cs_gradient_vector function assumes ghost cell
+ * values for input arrays (var and optionally c_weight)
+ * have already been synchronized.
+ *
+ * \param[in]       var_name        variable name
+ * \param[in]       gradient_type   gradient type
+ * \param[in]       halo_type       halo type
+ * \param[in]       inc             if 0, solve on increment; 1 otherwise
+ * \param[in]       n_r_sweeps      if > 1, number of reconstruction sweeps
+ * \param[in]       verbosity       verbosity level
+ * \param[in]       clip_mode       clipping mode
+ * \param[in]       epsilon         precision for iterative gradient calculation
+ * \param[in]       clip_coeff      clipping coefficient
+ * \param[in]       bc_coeff_a      boundary condition term a
+ * \param[in]       bc_coeff_b      boundary condition term b
+ * \param[in, out]  var             gradient's base variable
+ * \param[in, out]  c_weight        weighted gradient coefficient variable,
+ *                                  or NULL
+ * \param[in, out]  cpl             structure associated with internal coupling,
+ *                                  or NULL
+ * \param[out]      gradv           gradient
+                                    (\f$ \der{u_i}{x_j} \f$ is gradv[][i][j])
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_gradient_vector_synced_input(const char                *var_name,
+                                cs_gradient_type_t         gradient_type,
+                                cs_halo_type_t             halo_type,
+                                int                        inc,
+                                int                        n_r_sweeps,
+                                int                        verbosity,
+                                int                        clip_mode,
+                                double                     epsilon,
+                                double                     clip_coeff,
+                                const cs_real_t            bc_coeff_a[][3],
+                                const cs_real_t            bc_coeff_b[][3][3],
+                                const cs_real_t            var[restrict][3],
+                                const cs_real_t            c_weight[restrict],
+                                const cs_internal_coupling_t  *cpl,
+                                cs_real_33_t     *restrict gradv);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute cell gradient of tensor.
+ *
+ * \param[in]       var_name        variable name
+ * \param[in]       gradient_type   gradient type
+ * \param[in]       halo_type       halo type
+ * \param[in]       inc             if 0, solve on increment; 1 otherwise
+ * \param[in]       n_r_sweeps      if > 1, number of reconstruction sweeps
+ * \param[in]       verbosity       verbosity level
+ * \param[in]       clip_mode       clipping mode
+ * \param[in]       epsilon         precision for iterative gradient calculation
+ * \param[in]       clip_coeff      clipping coefficient
+ * \param[in]       bc_coeff_a      boundary condition term a
+ * \param[in]       bc_coeff_b      boundary condition term b
+ * \param[in, out]  var             gradient's base variable
+ * \param[out]      grad            gradient
+                                    (\f$ \der{t_ij}{x_k} \f$ is grad[][ij][k])
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_gradient_tensor_synced_input(const char                *var_name,
+                                cs_gradient_type_t         gradient_type,
+                                cs_halo_type_t             halo_type,
+                                int                        inc,
+                                int                        n_r_sweeps,
+                                int                        verbosity,
+                                int                        clip_mode,
+                                double                     epsilon,
+                                double                     clip_coeff,
+                                const cs_real_t            bc_coeff_a[][6],
+                                const cs_real_t            bc_coeff_b[][6][6],
+                                const cs_real_t            var[restrict][6],
+                                cs_real_63_t     *restrict grad);
+
 /*----------------------------------------------------------------------------
  * Determine gradient type by Fortran "imrgra" value
  *
