@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -36,6 +36,7 @@
 #include "cs_halo.h"
 #include "cs_numbering.h"
 #include "cs_halo_perio.h"
+#include "cs_matrix_assembler.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -159,7 +160,6 @@ cs_matrix_structure_create(cs_matrix_type_t       type,
                            cs_lnum_t              n_rows,
                            cs_lnum_t              n_cols_ext,
                            cs_lnum_t              n_edges,
-                           const cs_gnum_t       *cell_num,
                            const cs_lnum_2_t     *edges,
                            const cs_halo_t       *halo,
                            const cs_numbering_t  *numbering);
@@ -219,7 +219,6 @@ cs_matrix_structure_create_msr(cs_matrix_type_t        type,
  *   direct_assembly  <-- true if each value corresponds to a unique face
  *   n_rows           <-- local number of rows
  *   n_cols_ext       <-- local number of columns + ghosts
- *   cell_num         <-- global cell numbers, or NULL
  *   row_index        <-- pointer to index on rows
  *   col_id           <-- pointer to array of colum ids related to the row index
  *   halo             <-- halo structure for synchronization, or NULL
@@ -235,12 +234,28 @@ cs_matrix_structure_create_msr_shared(bool                    have_diag,
                                       bool                    direct_assmbly,
                                       cs_lnum_t               n_rows,
                                       cs_lnum_t               n_cols_ext,
-                                      const cs_gnum_t        *cell_num,
                                       const cs_lnum_t        *row_index,
                                       const cs_lnum_t        *col_id,
                                       const cs_halo_t        *halo,
                                       const cs_numbering_t   *numbering);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Create a matrix structure using a matrix assembler.
+ *
+ * Only CSR and MSR formats are handled.
+ *
+ * \param[in]  type  type of matrix considered
+ * \param[in]  ma    pointer to matrix assembler structure
+ *
+ * \return  a pointer to a created matrix structure
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_matrix_structure_t *
+cs_matrix_structure_create_from_assembler(cs_matrix_type_t        type,
+                                          cs_matrix_assembler_t  *ma);
+
 /*----------------------------------------------------------------------------
  * Destroy a matrix structure.
  *
@@ -285,6 +300,23 @@ cs_matrix_t *
 cs_matrix_create_by_variant(const cs_matrix_structure_t  *ms,
                             const cs_matrix_variant_t    *mv);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Create a matrix directly from assembler.
+ *
+ * Only CSR and MSR formats are handled.
+ *
+ * \param[in]  type  type of matrix considered
+ * \param[in]  ma    pointer to matrix assembler structure
+ *
+ * \return  a pointer to a created matrix structure
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_matrix_t *
+cs_matrix_create_from_assembler(cs_matrix_type_t        type,
+                                cs_matrix_assembler_t  *ma);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Create a matrix container by copying another
@@ -359,7 +391,7 @@ cs_matrix_get_n_rows(const cs_matrix_t  *matrix);
  *   pointer to block sizes
  *----------------------------------------------------------------------------*/
 
-const int *
+const cs_lnum_t *
 cs_matrix_get_diag_block_size(const cs_matrix_t  *matrix);
 
 /*----------------------------------------------------------------------------
@@ -376,7 +408,7 @@ cs_matrix_get_diag_block_size(const cs_matrix_t  *matrix);
  *   pointer to block sizes
  *----------------------------------------------------------------------------*/
 
-const int *
+const cs_lnum_t *
 cs_matrix_get_extra_diag_block_size(const cs_matrix_t  *matrix);
 
 /*----------------------------------------------------------------------------
@@ -409,9 +441,9 @@ cs_matrix_get_halo(const cs_matrix_t  *matrix);
  *----------------------------------------------------------------------------*/
 
 cs_matrix_fill_type_t
-cs_matrix_get_fill_type(bool        symmetric,
-                        const int  *diag_block_size,
-                        const int  *extra_diag_block_size);
+cs_matrix_get_fill_type(bool              symmetric,
+                        const cs_lnum_t  *diag_block_size,
+                        const cs_lnum_t  *extra_diag_block_size);
 
 /*----------------------------------------------------------------------------
  * Set matrix coefficients defined relative to a "native" edge graph,
@@ -445,8 +477,8 @@ cs_matrix_get_fill_type(bool        symmetric,
 void
 cs_matrix_set_coefficients(cs_matrix_t        *matrix,
                            bool                symmetric,
-                           const int          *diag_block_size,
-                           const int          *extra_diag_block_size,
+                           const cs_lnum_t    *diag_block_size,
+                           const cs_lnum_t    *extra_diag_block_size,
                            const cs_lnum_t     n_edges,
                            const cs_lnum_2_t   edges[],
                            const cs_real_t    *da,
@@ -479,8 +511,8 @@ cs_matrix_set_coefficients(cs_matrix_t        *matrix,
 void
 cs_matrix_copy_coefficients(cs_matrix_t        *matrix,
                             bool                symmetric,
-                            const int          *diag_block_size,
-                            const int          *extra_diag_block_size,
+                            const cs_lnum_t    *diag_block_size,
+                            const cs_lnum_t    *extra_diag_block_size,
                             const cs_lnum_t     n_edges,
                             const cs_lnum_2_t   edges[],
                             const cs_real_t    *da,
@@ -514,13 +546,38 @@ cs_matrix_copy_coefficients(cs_matrix_t        *matrix,
 void
 cs_matrix_transfer_coefficients_msr(cs_matrix_t         *matrix,
                                     bool                 symmetric,
-                                    const int           *diag_block_size,
-                                    const int           *extra_diag_block_size,
+                                    const cs_lnum_t     *diag_block_size,
+                                    const cs_lnum_t     *extra_diag_block_size,
                                     const cs_lnum_t      row_index[],
                                     const cs_lnum_t      col_id[],
                                     cs_real_t          **d_val,
                                     cs_real_t          **x_val);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Create and initialize a CSR matrix assembler values structure.
+ *
+ * The associated matrix's structure must have been created using
+ * \ref cs_matrix_structure_create_from_assembler.
+ *
+ * Block sizes are defined by an optional array of 4 values:
+ *   0: useful block size, 1: vector block extents,
+ *   2: matrix line extents,  3: matrix line*column extents
+ *
+ * \param[in, out]  matrix                 pointer to matrix structure
+ * \param[in]       diag_block_size        block sizes for diagonal, or NULL
+ * \param[in]       extra_diag_block_size  block sizes for extra diagonal,
+ *                                         or NULL
+ *
+ * \return  pointer to initialized matrix assembler values structure;
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_matrix_assembler_values_t *
+cs_matrix_assembler_values_init(cs_matrix_t      *matrix,
+                                const cs_lnum_t  *diag_block_size,
+                                const cs_lnum_t  *extra_diag_block_size);
+
 /*----------------------------------------------------------------------------
  * Release shared matrix coefficients.
  *
@@ -769,6 +826,20 @@ cs_matrix_exdiag_vector_multiply(cs_halo_rotation_t   rotation_mode,
                                  cs_real_t           *restrict x,
                                  cs_real_t           *restrict y);
 
+/*----------------------------------------------------------------------------
+ * Synchronize ghost values prior to matrix.vector product
+ *
+ * parameters:
+ *   rotation_mode <-- halo update option for rotational periodicity
+ *   matrix        <-- pointer to matrix structure
+ *   x             <-> multipliying vector values (ghost values updated)
+ *----------------------------------------------------------------------------*/
+
+void
+cs_matrix_pre_vector_multiply_sync(cs_halo_rotation_t   rotation_mode,
+                                   const cs_matrix_t   *matrix,
+                                   cs_real_t           *x);
+
 /*----------------------------------------------------------------------------
  * Build list of variants for tuning or testing.
  *
@@ -894,7 +965,6 @@ cs_matrix_variant_type(const cs_matrix_variant_t  *mv);
  *   n_rows         <-- number of local rows
  *   n_cols_ext     <-- number of columns + ghosts
  *   n_edges        <-- local number of (undirected) graph edges
- *   cell_num       <-- optional global cell numbers (1 to n), or NULL
  *   edges          <-- edges (symmetric row <-> column) connectivity
  *   halo           <-- cell halo structure
  *   numbering      <-- vectorization or thread-related numbering info, or NULL
@@ -904,7 +974,6 @@ void
 cs_matrix_variant_test(cs_lnum_t              n_rows,
                        cs_lnum_t              n_cols_ext,
                        cs_lnum_t              n_edges,
-                       const cs_gnum_t       *cell_num,
                        const cs_lnum_2_t     *edges,
                        const cs_halo_t       *halo,
                        const cs_numbering_t  *numbering);
