@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -53,6 +53,7 @@ BEGIN_C_DECLS
  *
  *  - Native
  *  - Compressed Sparse Row (CSR)
+ *  - Modified Compressed Sparse Row (MSR), with separate diagonal
  *  - Symmetric Compressed Sparse Row (CSR_SYM)
  */
 
@@ -219,7 +220,8 @@ typedef struct _cs_matrix_coeff_csr_sym_t {
 typedef struct _cs_matrix_coeff_msr_t {
 
   int              max_db_size;       /* Current max allocated block size */
-  int              max_eb_size;       /* Current max allocated extradiag block size */
+  int              max_eb_size;       /* Current max allocated extradiag
+                                         block size */
 
   /* Pointers to possibly shared arrays */
 
@@ -250,10 +252,11 @@ struct _cs_matrix_structure_t {
      local->local cell numbering for future info or renumbering,
      and halo) */
 
-  const cs_gnum_t       *row_num;      /* Global row numbers */
   const cs_halo_t       *halo;         /* Parallel or periodic halo */
   const cs_numbering_t  *numbering;    /* Vectorization or thread-related
                                           numbering information */
+
+  const cs_matrix_assembler_t  *assembler;   /* Associated matrix assembler */
 };
 
 /* Structure associated with Matrix (representation-independent part) */
@@ -270,13 +273,13 @@ struct _cs_matrix_t {
 
   bool                   symmetric;    /* true if coefficients are symmetric */
 
-  int                    db_size[4];   /* Diag Block size, including padding:
+  cs_lnum_t              db_size[4];   /* Diag Block size, including padding:
                                           0: useful block size
                                           1: vector block extents
                                           2: matrix line extents
                                           3: matrix line*column extents */
 
-  int                    eb_size[4];   /* Extradiag block size, including padding:
+  cs_lnum_t              eb_size[4];   /* Extradiag block size, including padding:
                                           0: useful block size
                                           1: vector block extents
                                           2: matrix line extents
@@ -284,7 +287,8 @@ struct _cs_matrix_t {
 
   /* Pointer to shared structure */
 
-  const void            *structure;    /* Matrix structure */
+  const void            *structure;    /* Possibly shared matrix structure */
+  void                  *_structure;   /* Private matrix structure */
 
   /* Pointers to arrays possibly shared from mesh structure
      (graph edges: face->cell connectivity for coefficient assignment,
@@ -295,6 +299,8 @@ struct _cs_matrix_t {
   const cs_numbering_t  *numbering;    /* Vectorization or thread-related
                                           numbering information */
 
+  const cs_matrix_assembler_t  *assembler;   /* Associated matrix assembler */
+
   /* Pointer to shared arrays from coefficient assignment from
      "native" type. This should be removed in the future, but requires
      removing the dependency to the native structure in the multigrid
