@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -50,10 +50,11 @@
 #include "bft_printf.h"
 
 #include "cs_base.h"
-#include "cs_log.h"
-#include "cs_halo.h"
 #include "cs_field.h"
 #include "cs_grid.h"
+#include "cs_halo.h"
+#include "cs_internal_coupling.h"
+#include "cs_log.h"
 #include "cs_mesh.h"
 #include "cs_matrix.h"
 #include "cs_matrix_default.h"
@@ -120,7 +121,7 @@ _sles_default_native(int                f_id,
                      cs_matrix_type_t   matrix_type,
                      bool               symmetric)
 {
-  bool multigrid = false;
+  int multigrid = 0;
   cs_sles_it_type_t sles_it_type = CS_SLES_N_IT_TYPES;
   int n_max_iter = _n_max_iter_default;
 
@@ -128,10 +129,10 @@ _sles_default_native(int                f_id,
 
     if (!strcmp(name, "wall_distance")) { /* distpr.f90 */
       sles_it_type = CS_SLES_PCG;
-      multigrid = true;
+      multigrid = 1;
     }
     if (!strcmp(name, "yplus_wall")) { /* distyp.f90 */
-      sles_it_type = CS_SLES_JACOBI;
+      sles_it_type = CS_SLES_P_SYM_GAUSS_SEIDEL;
     }
     else if (   !strcmp(name, "hydrostatic_p")  /* calhyd.f90 */
              || !strcmp(name, "potential")) {   /* predfl.f90 */
@@ -149,13 +150,13 @@ _sles_default_native(int                f_id,
       }
       /* If copying from pressure failed, default to multigrid */
       sles_it_type = CS_SLES_PCG;
-      multigrid = true;
+      multigrid = 1;
     }
     else if (!strcmp(name, "Prhydro")) { /* prehyd.f90 */
       sles_it_type = CS_SLES_PCG;
     }
     else if (!strcmp(name, "Pr compress")) { /* resopv.f90 */
-      sles_it_type = CS_SLES_JACOBI;
+      sles_it_type = CS_SLES_P_SYM_GAUSS_SEIDEL;
     }
     else if (!strcmp(name, "PoissonL")) { /* lageqp.f90 */
       sles_it_type = CS_SLES_PCG;
@@ -163,26 +164,55 @@ _sles_default_native(int                f_id,
     }
     else if (!strcmp(name, "radiation_p1")) { /* raypun.f90 */
       sles_it_type = CS_SLES_PCG;
-      multigrid = true;
+      multigrid = 1;
+    }
+    /* cs_bad_cells_regularisation.c */
+    else if (!strcmp(name, "potential_regularisation_scalar")) {
+      sles_it_type = CS_SLES_PCG;
+    }
+    else if (!strcmp(name, "potential_regularisation_vector")) {
+      sles_it_type = CS_SLES_PCG;
+    }
+    else if (!strcmp(name, "potential_regularisation_sym_tensor")) {
+      sles_it_type = CS_SLES_PCG;
+    }
+  }
+  else if (f_id > -1) {
+    const cs_field_t *f = cs_field_by_id(f_id);
+    if (!strcmp(f->name, "hydraulic_head")) {
+      sles_it_type = CS_SLES_PCG;
+      multigrid = 2;
     }
-
   }
 
   /* Final default */
 
   if (sles_it_type == CS_SLES_N_IT_TYPES) {
+
+    int coupling_id = -1;
+
+    if (f_id > -1) {
+      const cs_field_t *f = cs_field_by_id(f_id);
+      coupling_id
+        = cs_field_get_key_int(f, cs_field_key_id("coupling_entity"));
+    }
+
     if (symmetric) {
       sles_it_type = CS_SLES_PCG;
-      if (f_id > -1)
-        multigrid = true;
+      if (f_id > -1 && coupling_id < 0)
+        multigrid = 1;
+    }
+    else {
+      if (coupling_id < 0)
+        sles_it_type = CS_SLES_P_SYM_GAUSS_SEIDEL;
+      else
+        sles_it_type = CS_SLES_BICGSTAB;
     }
-    else
-      sles_it_type = CS_SLES_JACOBI;
   }
 
-  if (multigrid) {
+  if (multigrid == 1) {
 
-    /* Multigrid used as preconditionner if possible, as solver otherwise */
+    /* Multigrid used as preconditioner if possible, as solver otherwise */
 
     if ((matrix_type == CS_MATRIX_MSR) || (matrix_type == CS_MATRIX_N_TYPES)) {
       cs_sles_it_t *c = cs_sles_it_define(f_id,
@@ -194,14 +224,14 @@ _sles_default_native(int                f_id,
       cs_multigrid_t *mg = cs_sles_pc_get_context(pc);
       cs_sles_it_transfer_pc(c, &pc);
       cs_multigrid_set_solver_options(mg,
-                                      CS_SLES_P_GAUSS_SEIDEL,
-                                      CS_SLES_P_GAUSS_SEIDEL,
+                                      CS_SLES_P_SYM_GAUSS_SEIDEL,
+                                      CS_SLES_P_SYM_GAUSS_SEIDEL,
                                       CS_SLES_PCG,
                                       1,    /* n max cycles */
                                       1,    /* n max iter for descent */
                                       1,    /* n max iter for ascent */
                                       500,  /* n max iter for coarse solve */
-                                      0, 0, 0,  /* precond degree */
+                                      0, 0, -1,  /* precond degree */
                                       -1, -1, 1); /* precision multiplier */
       cs_sles_t *sc = cs_sles_find(f_id, name);
       cs_sles_set_error_handler(sc, cs_sles_default_error);
@@ -210,6 +240,9 @@ _sles_default_native(int                f_id,
       cs_multigrid_define(f_id, name);
 
   }
+  else if (multigrid == 2)
+    cs_multigrid_define(f_id, name);
+
   else
     (void)cs_sles_it_define(f_id,
                             name,
@@ -425,46 +458,70 @@ cs_sles_setup_native_conv_diff(int                  f_id,
          "If this is not an error, increase CS_SLES_DEFAULT_N_SETUPS\n"
          "  in file %s.", CS_SLES_DEFAULT_N_SETUPS, __FILE__);
 
-    if (a == NULL)
+    /* Set extended contribution for domain coupling */
+
+    if (f_id > -1) {
+
+      const cs_field_t *f = cs_field_by_id(f_id);
+      int coupling_id
+        = cs_field_get_key_int(f, cs_field_key_id("coupling_entity"));
+
+      if (coupling_id > -1) {
+        /* TODO base matrix type on solver type (for external libraries) */
+        a = cs_matrix_set_coefficients_coupled(f,
+                                               CS_MATRIX_MSR,
+                                               false, /* symmetric */
+                                               diag_block_size,
+                                               extra_diag_block_size,
+                                               da,
+                                               xa);
+      }
+
+    }
+
+    if (a == NULL) {
+
       a = cs_matrix_msr(false,
                         diag_block_size,
                         extra_diag_block_size);
 
-    cs_matrix_set_coefficients(a,
-                               false,
-                               diag_block_size,
-                               extra_diag_block_size,
-                               m->n_i_faces,
-                               (const cs_lnum_2_t *)(m->i_face_cells),
-                               da,
-                               xa);
+      cs_matrix_set_coefficients(a,
+                                 false,
+                                 diag_block_size,
+                                 extra_diag_block_size,
+                                 m->n_i_faces,
+                                 (const cs_lnum_2_t *)(m->i_face_cells),
+                                 da,
+                                 xa);
 
-    cs_matrix_t *a_ref = cs_matrix_default(false,
-                                           diag_block_size,
-                                           extra_diag_block_size);
-    if (a_conv == NULL)
-      a_conv = cs_matrix_create_by_copy(a_ref);
+      cs_matrix_t *a_ref = cs_matrix_default(false,
+                                             diag_block_size,
+                                             extra_diag_block_size);
+      if (a_conv == NULL)
+        a_conv = cs_matrix_create_by_copy(a_ref);
 
-    cs_matrix_set_coefficients(a_conv,
-                               false,
-                               diag_block_size,
-                               extra_diag_block_size,
-                               m->n_i_faces,
-                               (const cs_lnum_2_t *)(m->i_face_cells),
-                               da_conv,
-                               xa_conv);
+      cs_matrix_set_coefficients(a_conv,
+                                 false,
+                                 diag_block_size,
+                                 extra_diag_block_size,
+                                 m->n_i_faces,
+                                 (const cs_lnum_2_t *)(m->i_face_cells),
+                                 da_conv,
+                                 xa_conv);
 
-    if (a_diff == NULL)
-      a_diff = cs_matrix_create_by_copy(a_ref);
+      if (a_diff == NULL)
+        a_diff = cs_matrix_create_by_copy(a_ref);
 
-    cs_matrix_set_coefficients(a_diff,
-                               false,
-                               diag_block_size,
-                               extra_diag_block_size,
-                               m->n_i_faces,
-                               (const cs_lnum_2_t *)(m->i_face_cells),
-                               da_diff,
-                               xa_diff);
+      cs_matrix_set_coefficients(a_diff,
+                                 false,
+                                 diag_block_size,
+                                 extra_diag_block_size,
+                                 m->n_i_faces,
+                                 (const cs_lnum_2_t *)(m->i_face_cells),
+                                 da_diff,
+                                 xa_diff);
+
+    }
 
     _sles_setup[setup_id] = sc;
     _matrix_setup[setup_id][0] = a;
@@ -491,6 +548,93 @@ cs_sles_setup_native_conv_diff(int                  f_id,
   cs_multigrid_setup_conv_diff(mg, name, a, a_conv, a_diff, verbosity);
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Call sparse linear equation solver setup for systems with internal
+ *        coupling.
+ *
+ * \param[in]  f_id                   associated field id, or < 0
+ * \param[in]  name                   associated name if f_id < 0, or NULL
+ * \param[in]  symmetric              indicates if matrix coefficients
+ *                                    are symmetric
+ * \param[in]  diag_block_size        block sizes for diagonal, or NULL
+ * \param[in]  extra_diag_block_size  block sizes for extra diagonal,
+ *                                    or NULL
+ * \param[in]  da                     diagonal values (NULL if zero)
+ * \param[in]  xa                     extradiagonal values (NULL if zero)
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_sles_setup_native_coupling(int               f_id,
+                              const char       *name,
+                              bool              symmetric,
+                              const int        *diag_block_size,
+                              const int        *extra_diag_block_size,
+                              const cs_real_t  *da,
+                              const cs_real_t  *xa)
+{
+  cs_matrix_t *a = NULL;
+
+  /* Check if this system has already been setup */
+
+  cs_sles_t *sc = cs_sles_find_or_add(f_id, name);
+
+  int setup_id = 0;
+  while (setup_id < _n_setups) {
+    if (_sles_setup[setup_id] == sc)
+      break;
+    else
+      setup_id++;
+  }
+
+  if (setup_id >= _n_setups) {
+
+    _n_setups += 1;
+
+    if (_n_setups > CS_SLES_DEFAULT_N_SETUPS)
+      bft_error
+        (__FILE__, __LINE__, 0,
+         "Too many linear systems solved without calling cs_sles_free_native\n"
+         "  maximum number of systems: %d\n"
+         "If this is not an error, increase CS_SLES_DEFAULT_N_SETUPS\n"
+         "  in file %s.", CS_SLES_DEFAULT_N_SETUPS, __FILE__);
+
+    /* Set extended contribution for domain coupling */
+
+    if (f_id > -1) {
+
+      const cs_field_t *f = cs_field_by_id(f_id);
+      int coupling_id
+        = cs_field_get_key_int(f, cs_field_key_id("coupling_entity"));
+
+      if (coupling_id > -1) {
+        /* TODO base matrix type on solver type (for external libraries) */
+        a = cs_matrix_set_coefficients_coupled(f,
+                                               CS_MATRIX_MSR,
+                                               symmetric,
+                                               diag_block_size,
+                                               extra_diag_block_size,
+                                               da,
+                                               xa);
+      }
+
+    }
+
+    _sles_setup[setup_id] = sc;
+    _matrix_setup[setup_id][0] = a;
+    _matrix_setup[setup_id][1] = a; /* so it is freed later */
+
+  }
+  else {
+    a = _matrix_setup[setup_id][0];
+  }
+
+  /* Setup system */
+
+  cs_sles_setup(sc, a);
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Call sparse linear equation solver using native matrix arrays.
@@ -576,6 +720,7 @@ cs_sles_solve_native(int                  f_id,
       a = cs_matrix_native(symmetric,
                            diag_block_size,
                            extra_diag_block_size);
+
       cs_matrix_set_coefficients(a,
                                  symmetric,
                                  diag_block_size,
@@ -584,6 +729,7 @@ cs_sles_solve_native(int                  f_id,
                                  (const cs_lnum_2_t *)(m->i_face_cells),
                                  da,
                                  xa);
+
       cs_sles_define_t  *sles_default_func = cs_sles_get_default_define();
       sles_default_func(f_id, name, a);
       cs_matrix_release_coefficients(a);
@@ -596,7 +742,9 @@ cs_sles_solve_native(int                  f_id,
 
     if (strcmp(cs_sles_get_type(sc), "cs_sles_it_t") == 0) {
       cs_sles_it_t *c = cs_sles_get_context(sc);
-      if (cs_sles_it_get_type(c) == CS_SLES_P_GAUSS_SEIDEL)
+      cs_sles_it_type_t s_type = cs_sles_it_get_type(c);
+      if (   s_type == CS_SLES_P_GAUSS_SEIDEL
+          || s_type == CS_SLES_P_SYM_GAUSS_SEIDEL)
         need_msr = true;
       else {
         pc = cs_sles_it_get_pc(c);
@@ -610,10 +758,18 @@ cs_sles_solve_native(int                  f_id,
       mg = cs_sles_get_context(sc);
 
     if (mg != NULL) {
-      if (cs_multigrid_get_fine_solver_type(mg) == CS_SLES_P_GAUSS_SEIDEL)
+      cs_sles_it_type_t fs_type = cs_multigrid_get_fine_solver_type(mg);
+      if (   fs_type == CS_SLES_P_GAUSS_SEIDEL
+          || fs_type == CS_SLES_P_SYM_GAUSS_SEIDEL)
         need_msr = true;
     }
 
+    /* MSR not supported yet for full blocks */
+    if (extra_diag_block_size != NULL) {
+      if (extra_diag_block_size[0] > 1)
+        need_msr = false;
+    }
+
     if (need_msr)
       a = cs_matrix_msr(symmetric,
                         diag_block_size,
@@ -641,6 +797,33 @@ cs_sles_solve_native(int                  f_id,
   else
     a = _matrix_setup[setup_id][0];
 
+  /* If system uses specific halo (i.e. when matrix contains more than
+     face->cell nonzeroes), allocate specific buffers. */
+
+  cs_real_t *_vx = vx, *_rhs = NULL;
+  const cs_real_t *rhs_p = rhs;
+
+  const cs_halo_t *halo = cs_matrix_get_halo(a);
+  if (halo != NULL && halo != m->halo) {
+
+    size_t stride = 1;
+    if (diag_block_size != NULL)
+      stride = diag_block_size[1];
+    cs_lnum_t n_rows = cs_matrix_get_n_rows(a);
+    cs_lnum_t n_cols_ext = cs_matrix_get_n_columns(a);
+    assert(n_rows == m->n_cells);
+    cs_lnum_t _n_rows = n_rows*stride;
+    BFT_MALLOC(_rhs, n_cols_ext*stride, cs_real_t);
+    BFT_MALLOC(_vx, n_cols_ext*stride, cs_real_t);
+#   pragma omp parallel for  if(_n_rows > CS_THR_MIN)
+    for (cs_lnum_t i = 0; i < _n_rows; i++) {
+      _rhs[i] = rhs[i];
+      _vx[i] = vx[i];
+    }
+    cs_matrix_pre_vector_multiply_sync(rotation_mode, a, _rhs);
+    rhs_p = _rhs;
+  }
+
   /* Solve system */
 
   cvg = cs_sles_solve(sc,
@@ -650,11 +833,24 @@ cs_sles_solve_native(int                  f_id,
                       r_norm,
                       n_iter,
                       residue,
-                      rhs,
-                      vx,
+                      rhs_p,
+                      _vx,
                       0,
                       NULL);
 
+  BFT_FREE(_rhs);
+  if (_vx != vx) {
+    size_t stride = 1;
+    if (diag_block_size != NULL)
+      stride = diag_block_size[1];
+    cs_lnum_t n_rows = cs_matrix_get_n_rows(a);
+    cs_lnum_t _n_rows = n_rows*stride;
+#   pragma omp parallel for  if(_n_rows > CS_THR_MIN)
+    for (cs_lnum_t i = 0; i < _n_rows; i++)
+      vx[i] = _vx[i];
+    BFT_FREE(_vx);
+  }
+
   return cvg;
 }
 
@@ -684,10 +880,8 @@ cs_sles_free_native(int          f_id,
   if (setup_id < _n_setups) {
 
     cs_sles_free(sc);
-    for (int i = 0; i < 3; i++) {
-      if (_matrix_setup[setup_id][i] != NULL)
-        cs_matrix_release_coefficients(_matrix_setup[setup_id][i]);
-    }
+    if (_matrix_setup[setup_id][0] != NULL)
+      cs_matrix_release_coefficients(_matrix_setup[setup_id][0]);
     for (int i = 1; i < 3; i++) { /* Remove "copied" matrixes */
       if (_matrix_setup[setup_id][i] != NULL)
         cs_matrix_destroy(&(_matrix_setup[setup_id][i]));
@@ -710,7 +904,7 @@ cs_sles_free_native(int          f_id,
  *        for sparse linear equation solver.
  *
  * In case of divergence with an iterative solver, this error handler
- * switches to a default preconditionner, then resets the solution vector.
+ * switches to a default preconditioner, then resets the solution vector.
  *
  * The default error for the solver type handler is then  set, in case
  * the solution fails again.
@@ -811,7 +1005,8 @@ cs_sles_default_error(cs_sles_t                    *sles,
       bft_printf(_("\n\n"
                    "%s [%s]: divergence\n"
                    "  fallback from multigrid to %s-preconditionned CG solver\n"
-                   "  for re-try and subsequent solves.\n"), "Jacobi");
+                   "  for re-try and subsequent solves.\n"),
+                   "Multigrid", name, "Jacobi");
 
       cs_sles_free(sles);
 
