@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -56,14 +56,16 @@ BEGIN_C_DECLS
 
 typedef enum {
 
-  CS_SLES_PCG,             /* Preconditionned conjugate gradient */
-  CS_SLES_JACOBI,          /* Jacobi */
-  CS_SLES_BICGSTAB,        /* Bi-conjugate gradient stabilized */
-  CS_SLES_BICGSTAB2,       /* Bi-conjugate gradient stabilized - 2*/
-  CS_SLES_GMRES,           /* Generalized minimal residual */
-  CS_SLES_P_GAUSS_SEIDEL,  /* Process-local Gauss-Seidel */
-  CS_SLES_PCR3,            /* 3-layer conjugate residual */
-  CS_SLES_N_IT_TYPES       /* Number of resolution algorithms */
+  CS_SLES_PCG,                 /* Preconditionned conjugate gradient */
+  CS_SLES_IPCG,                /* Inexact preconditionned conjugate gradient */
+  CS_SLES_JACOBI,              /* Jacobi */
+  CS_SLES_BICGSTAB,            /* Bi-conjugate gradient stabilized */
+  CS_SLES_BICGSTAB2,           /* Bi-conjugate gradient stabilized - 2*/
+  CS_SLES_GMRES,               /* Generalized minimal residual */
+  CS_SLES_P_GAUSS_SEIDEL,      /* Process-local Gauss-Seidel */
+  CS_SLES_P_SYM_GAUSS_SEIDEL,  /* Process-local symmetric Gauss-Seidel */
+  CS_SLES_PCR3,                /* 3-layer conjugate residual */
+  CS_SLES_N_IT_TYPES           /* Number of resolution algorithms */
 
 } cs_sles_it_type_t;
 
@@ -395,19 +397,25 @@ void
 cs_sles_it_assign_order(cs_sles_it_t   *context,
                         cs_lnum_t     **order);
 
-/*----------------------------------------------------------------------------
- * Assign symmetric option to iterative solver.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define convergence level under which the fallback to another
+ *        solver may be used if applicable.
  *
- * This is useful only for Process-local Gauss-Seidel.
+ * Currently, this mechanism is only by default used for BiCGstab and
+ * 3-layer conjugate residual solvers with scalar matrices, which may
+ * fall back to a preconditioned GMRES solver. For those solvers, the
+ * default threshold is \ref CS_SLES_BREAKDOWN, meaning that divergence
+ * (but not breakdown) will lead to the use of the fallback mechanism.
  *
- * parameters:
- *   context   <-> pointer to iterative solver info and context
- *   symmetric <-- symmetric if true
- *----------------------------------------------------------------------------*/
+ * \param[in, out]  context    pointer to iterative solver info and context
+ * \param[in]       threshold  convergence level under which fallback is used
+ */
+/*----------------------------------------------------------------------------*/
 
 void
-cs_sles_it_set_symmetric(cs_sles_it_t   *context,
-                         bool            symmetric);
+cs_sles_it_set_fallback_threshold(cs_sles_it_t                 *context,
+                                  cs_sles_convergence_state_t   threshold);
 
 /*----------------------------------------------------------------------------
  * Query mean number of rows under which Conjugate Gradient algorithm
