@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -47,6 +47,7 @@ use atchem
 use atimbr
 use siream
 use field
+use cs_c_bindings
 
 !===============================================================================
 
@@ -55,6 +56,9 @@ implicit none
 ! Local variables
 
 integer          ii, isc, jj
+double precision turb_schmidt
+
+type(var_cal_opt) :: vcopt
 
 !===============================================================================
 
@@ -112,12 +116,12 @@ if (ippmod(iatmos).eq.1) then
 
     jj = iscapp(isc)
 
+    call field_get_key_struct_var_cal_opt(ivarfl(isca(jj)), vcopt)
+
     if (iscavr(jj).le.0) then
       visls0(jj) = viscl0
     endif
 
-    blencv(isca(jj)) = 1.d0
-
   enddo
 
 endif
@@ -136,12 +140,12 @@ if (ippmod(iatmos).eq.2) then
 
     jj = iscapp(isc)
 
+    call field_get_key_struct_var_cal_opt(ivarfl(isca(jj)), vcopt)
+
     if (iscavr(jj).le.0) then
       visls0(jj) = viscl0
     endif
 
-    blencv(isca(jj)) = 1.d0
-
   enddo
 
 endif
@@ -152,7 +156,8 @@ endif
 
 if (nscal.gt.0) then
   do ii = 1, nscal
-    sigmas(ii) = 0.7d0
+    turb_schmidt = 0.7d0
+    call field_set_key_double(ivarfl(isca(ii)), ksigmas, turb_schmidt)
   enddo
 endif
 
