@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,10 +20,11 @@
 
 !-------------------------------------------------------------------------------
 !> \file atiniv.f90
-!> \brief   Initialisation of calculation variables for the atmospheric module,
-!>              it is the counterpart of usiniv.f90.
-!>      Initialise for example the meteorological field for each cell of
-!>    the domain by interpolation of the data from the meteo file
+!> \brief Initialisation of calculation variables for the atmospheric module,
+!> it is the counterpart of usiniv.f90.
+!>
+!> Initialise for example the meteorological field for each cell of
+!> the domain by interpolation of the data from the meteo file
 !-------------------------------------------------------------------------------
 ! Arguments
 !______________________________________________________________________________.
@@ -33,8 +34,8 @@
 !> \param[in]   nscal       total number of scalars
 !> \param[in]   dt          time step value
 !-------------------------------------------------------------------------------
-subroutine atiniv  ( nvar, nscal, dt )
 
+subroutine atiniv  ( nvar, nscal, dt )
 
 !===============================================================================
 ! Module files
@@ -75,6 +76,7 @@ double precision xcent
 double precision, dimension(:,:), pointer :: vel
 double precision, dimension(:), pointer :: cvar_k, cvar_ep, cvar_phi
 double precision, dimension(:), pointer :: cvar_fb, cvar_omg, cvar_nusa
+double precision, dimension(:,:), pointer :: cvar_rij
 double precision, dimension(:), pointer :: cvar_r11, cvar_r22, cvar_r33
 double precision, dimension(:), pointer :: cvar_r12, cvar_r13, cvar_r23
 double precision, dimension(:), pointer :: cvar_despgi, cvar_sc
@@ -95,12 +97,16 @@ if (itytur.eq.2) then
   call field_get_val_s(ivarfl(ik), cvar_k)
   call field_get_val_s(ivarfl(iep), cvar_ep)
 elseif (itytur.eq.3) then
-  call field_get_val_s(ivarfl(ir11), cvar_r11)
-  call field_get_val_s(ivarfl(ir22), cvar_r22)
-  call field_get_val_s(ivarfl(ir33), cvar_r33)
-  call field_get_val_s(ivarfl(ir12), cvar_r12)
-  call field_get_val_s(ivarfl(ir23), cvar_r23)
-  call field_get_val_s(ivarfl(ir13), cvar_r13)
+  if (irijco.eq.1) then
+    call field_get_val_v(ivarfl(irij), cvar_rij)
+  else
+    call field_get_val_s(ivarfl(ir11), cvar_r11)
+    call field_get_val_s(ivarfl(ir22), cvar_r22)
+    call field_get_val_s(ivarfl(ir33), cvar_r33)
+    call field_get_val_s(ivarfl(ir12), cvar_r12)
+    call field_get_val_s(ivarfl(ir23), cvar_r23)
+    call field_get_val_s(ivarfl(ir13), cvar_r13)
+  endif
   call field_get_val_s(ivarfl(iep), cvar_ep)
 elseif (iturb.eq.50) then
   call field_get_val_s(ivarfl(ik), cvar_k)
@@ -280,12 +286,21 @@ if (isuite.eq.0) then
 
         elseif (itytur.eq.3) then
 
-          cvar_r11(iel) = d2s3*xkent
-          cvar_r22(iel) = d2s3*xkent
-          cvar_r33(iel) = d2s3*xkent
-          cvar_r12(iel) = 0.d0
-          cvar_r13(iel) = 0.d0
-          cvar_r23(iel) = 0.d0
+          if (irijco.eq.1) then
+            cvar_rij(1,iel) = d2s3*xkent
+            cvar_rij(2,iel) = d2s3*xkent
+            cvar_rij(3,iel) = d2s3*xkent
+            cvar_rij(4,iel) = 0.d0
+            cvar_rij(5,iel) = 0.d0
+            cvar_rij(6,iel) = 0.d0
+          else
+            cvar_r11(iel) = d2s3*xkent
+            cvar_r22(iel) = d2s3*xkent
+            cvar_r33(iel) = d2s3*xkent
+            cvar_r12(iel) = 0.d0
+            cvar_r13(iel) = 0.d0
+            cvar_r23(iel) = 0.d0
+          endif
           cvar_ep(iel)  = xeent
 
         elseif (iturb.eq.50) then
