@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -32,6 +32,7 @@
 !______________________________________________________________________________!
 !> \param[in]   itypfb          boundary face types
 !> \param[in]   izfppp          boundary face zone number for atmospheric module
+!> \param[out]  icodcl          face boundary condition code
 !> \param[out]  rcodcl          Boundary conditions value
 !>- rcodcl(1) = valeur du dirichlet
 !>- rcodcl(2) = valeur du coef. d'echange ext. (infinie si pas d'echange)
@@ -39,9 +40,9 @@
 !> hauteur de rugosite (m) si icodcl=6  \n
 !>    pour les vitesses (vistl+visct)*gradu \n
 !>    pour la pression             dt*gradp \n
-!>    pour les scalaires cp*(viscls+visct/sigmas)*gradt
+!>    pour les scalaires cp*(viscls+visct/turb_schmidt)*gradt
 !-------------------------------------------------------------------------------
-subroutine attycl ( itypfb, izfppp, rcodcl )
+subroutine attycl ( itypfb, izfppp, icodcl, rcodcl )
 
 !===============================================================================
 ! Module files
@@ -52,6 +53,7 @@ use numvar
 use optcal
 use cstphy
 use cstnum
+use dimens, only: nvar
 use entsor
 use parall
 use ppppar
@@ -63,6 +65,7 @@ use atincl
 use atchem
 use atimbr
 use siream
+use cs_c_bindings
 
 !===============================================================================
 
@@ -72,8 +75,8 @@ implicit none
 
 integer          itypfb(nfabor)
 integer          izfppp(nfabor)
-
-double precision rcodcl(nfabor,nvarcl,3)
+integer          icodcl(nfabor,nvar)
+double precision rcodcl(nfabor,nvar,3)
 
 ! Local variables
 
@@ -83,7 +86,7 @@ integer jsp, isc
 double precision d2s3, zent, vs, xuent, xvent
 double precision xkent, xeent, tpent, qvent,ncent
 double precision xcent
-double precision, dimension(:), pointer ::  brom
+double precision, dimension(:), pointer ::  brom, coefap
 
 ! arrays for cressman interpolation
 double precision , dimension(:),allocatable :: u_bord
@@ -216,10 +219,12 @@ if (imbrication_flag) then
 endif ! imbrication_flag
 
 ! ==============================================================================
-! the interpollated field u_bord,v_bord,  .. will replace the values from the
+! the interpolated field u_bord,v_bord,  .. will replace the values from the
 ! standard meteo profile
 ! ==============================================================================
 
+call field_get_coefa_s(ivarfl(ipr), coefap)
+
 do ifac = 1, nfabor
 
   izone = izfppp(ifac)
@@ -279,18 +284,20 @@ do ifac = 1, nfabor
 
     vs = xuent*surfbo(1,ifac) + xvent*surfbo(2,ifac)
 
-    !     On met a jour le type de face de bord s'il n'a pas ete specifie
-    !       par l'utilisateur.
-    !     Pour une entree, on remplit la condition de Dirichlet si elle n'a pas
-    !     ete  specifiee par utilisateur.
+    ! On met a jour le type de face de bord s'il n'a pas ete specifie
+    !   par l'utilisateur.
+    ! Pour une entree, on remplit la condition de Dirichlet si elle n'a pas
+    ! ete  specifiee par utilisateur.
 
-    if (vs.gt.0) then
-      if (itypfb(ifac).eq.0) itypfb(ifac) = isolib
-    else
-      if (itypfb(ifac).eq.0) itypfb(ifac) = ientre
+    if (iautom(ifac).ge.1) then
+      if (vs.gt.0) then
+        itypfb(ifac) = isolib
+      else
+        if (itypfb(ifac).eq.0) itypfb(ifac) = ientre
+      endif
     endif
 
-    if (itypfb(ifac).eq.ientre) then
+    if (itypfb(ifac).eq.ientre.or.itypfb(ifac).eq.i_convective_inlet) then
 
       if (rcodcl(ifac,iu,1).gt.rinfin*0.5d0)             &
            rcodcl(ifac,iu,1) = xuent
@@ -299,54 +306,7 @@ do ifac = 1, nfabor
       if (rcodcl(ifac,iw,1).gt.rinfin*0.5d0)             &
            rcodcl(ifac,iw,1) = 0.d0
 
-      if    (itytur.eq.2) then
-
-        if (rcodcl(ifac,ik,1).gt.rinfin*0.5d0)           &
-             rcodcl(ifac,ik,1) = xkent
-        if (rcodcl(ifac,iep,1).gt.rinfin*0.5d0)          &
-             rcodcl(ifac,iep,1) = xeent
-
-      elseif(itytur.eq.3) then
-
-        if (rcodcl(ifac,ir11,1).gt.rinfin*0.5d0)         &
-             rcodcl(ifac,ir11,1) = d2s3*xkent
-        if (rcodcl(ifac,ir22,1).gt.rinfin*0.5d0)         &
-             rcodcl(ifac,ir22,1) = d2s3*xkent
-        if (rcodcl(ifac,ir33,1).gt.rinfin*0.5d0)         &
-             rcodcl(ifac,ir33,1) = d2s3*xkent
-        if (rcodcl(ifac,ir12,1).gt.rinfin*0.5d0)         &
-             rcodcl(ifac,ir12,1) = 0.d0
-        if (rcodcl(ifac,ir23,1).gt.rinfin*0.5d0)         &
-             rcodcl(ifac,ir23,1) = 0.d0
-        if (rcodcl(ifac,ir13,1).gt.rinfin*0.5d0)         &
-             rcodcl(ifac,ir13,1) = 0.d0
-        if (rcodcl(ifac,iep,1).gt.rinfin*0.5d0)          &
-             rcodcl(ifac,iep,1) = xeent
-
-      elseif(iturb.eq.50) then
-
-        if (rcodcl(ifac,ik,1).gt.rinfin*0.5d0)           &
-             rcodcl(ifac,ik,1) = xkent
-        if (rcodcl(ifac,iep,1).gt.rinfin*0.5d0)          &
-             rcodcl(ifac,iep,1) = xeent
-        if (rcodcl(ifac,iphi,1).gt.rinfin*0.5d0)         &
-             rcodcl(ifac,iphi,1) = d2s3
-        if (rcodcl(ifac,ifb,1).gt.rinfin*0.5d0)          &
-             rcodcl(ifac,ifb,1) = 0.d0
-
-      elseif(iturb.eq.60) then
-
-        if (rcodcl(ifac,ik,1).gt.rinfin*0.5d0)           &
-             rcodcl(ifac,ik,1) = xkent
-        if (rcodcl(ifac,iomg,1).gt.rinfin*0.5d0)         &
-             rcodcl(ifac,iomg,1) = xeent/cmu/xkent
-
-      elseif(iturb.eq.70) then
-
-        if (rcodcl(ifac,inusa,1).gt.rinfin*0.5d0)         &
-             rcodcl(ifac,inusa,1) = cmu*xkent**2/xeent
-
-      endif
+      call turbulence_bc_inlet_k_eps(ifac, xkent, xeent, rcodcl)
 
       if (iscalt.ne.-1) then
 
@@ -382,6 +342,43 @@ do ifac = 1, nfabor
 
     endif
 
+    if (iatmst.gt.0) then
+
+      if (iautom(ifac).ge.1) then
+
+        ! Dirichlet on the pressure
+        icodcl(ifac,ipr) = 1
+        rcodcl(ifac,ipr,1) = coefap(ifac)
+
+        ! Dirichlet on turbulent variables
+        call turbulence_bc_inlet_k_eps(ifac, xkent, xeent, rcodcl)
+
+        if (iautom(ifac).eq.1) then
+
+          ! Homogeneous Neumann on the velocity
+          icodcl(ifac,iu) = 3
+          icodcl(ifac,iv) = 3
+          icodcl(ifac,iw) = 3
+          rcodcl(ifac,iu,3) = 0.d0
+          rcodcl(ifac,iv,3) = 0.d0
+          rcodcl(ifac,iw,3) = 0.d0
+
+        else if (iautom(ifac).eq.2) then
+
+          ! Dirichlet on the velocity
+          icodcl(ifac,iu) = 1
+          icodcl(ifac,iv) = 1
+          icodcl(ifac,iw) = 1
+          rcodcl(ifac,iu,1) = xuent
+          rcodcl(ifac,iv,1) = xvent
+          rcodcl(ifac,iw,1) = 0.d0
+
+        endif
+
+      endif
+
+    endif
+
   endif
 
 enddo
