@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -19,13 +19,24 @@
 ! Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 !-------------------------------------------------------------------------------
-!> \file atvarp.f90
-!> \brief ATMOSPHERIC AEROSOL CHEMISTRY MODULE add necessary variables
 
-!> \brief INIT DES POSITIONS DES VARIABLES POUR LE MODULE ATMOSPHERIQUE
-!>    REMPLISSAGE DES PARAMETRES (DEJA DEFINIS) POUR LES SCALAIRES PP
+!===============================================================================
+! Function:
+! ---------
+
+!> \file atvarp.f90
+!> \brief Declare additional transported variables for atmospheric module.
+!>
+!-------------------------------------------------------------------------------
+
+!-------------------------------------------------------------------------------
+! Arguments
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!_______________________________________________________________________________
+
 subroutine atvarp
-!================
 
 !===============================================================================
 ! Module files
@@ -109,7 +120,7 @@ call atini0
 !==========
 
 !===============================================================================
-! 1. DEFINITION DES POINTEURS
+! 1. Add variables
 !===============================================================================
 
 ! 1.1  Dry atmosphere
@@ -466,7 +477,7 @@ endif
 !===============================================================================
 
 ! Cp is constant
-icp = 0
+icp = -1
 
 !----
 ! Formats
