@@ -3,7 +3,7 @@
 !     This file is part of the Code_Saturne Kernel, element of the
 !     Code_Saturne CFD tool.
 
-!     Copyright (C) 1998-2016 EDF S.A., France
+!     Copyright (C) 1998-2018 EDF S.A., France
 
 !     contact: saturne-support@edf.fr
 
@@ -68,6 +68,7 @@ use lagran
 use ihmpre
 use cplsat
 use mesh
+use post
 use field
 use cs_f_interfaces
 use cs_c_bindings
@@ -84,10 +85,15 @@ integer          ii
 integer          ifcvsl, kbfid
 integer          iflid, iopchr
 integer          itycat, ityloc, idim1, idim3
+integer          f_id, potr, poti, flag
+integer          f_vis, f_log, iut, ialpha
+integer          kturt, kfturt
+integer          keycpl
+integer          ivar
 logical          iprev, inoprv, is_set
-integer          f_id, potr, poti
 
-character(len=80) :: f_name, f_label, s_label, s_name
+character(len=80) :: name, f_name, f_label, s_label, s_name
+type(var_cal_opt) :: vcopt_dfm, vcopt_alpha, vcopt
 
 !===============================================================================
 
@@ -107,16 +113,84 @@ iprev  = .true.    ! variables have previous value
 inoprv = .false.   ! variables have no previous value
 iopchr = 1         ! Postprocessing level for variables
 
+! Keys not stored globally
+call field_get_key_id('turbulent_flux_model', kturt)
+call field_get_key_id('turbulent_flux_id', kfturt)
+call field_get_key_id('coupled', keycpl)
+
 !===============================================================================
-! 1. Initialization
+! 0. Initialization
 !===============================================================================
 
 call field_get_key_id("boundary_value_id", kbfid)
 
 call field_get_key_id('log', keylog)
-call field_get_key_id('post_vis', keyvis)
 call field_get_key_id('label', keylbl)
 
+!===============================================================================
+! 1. Additional variable fields
+!===============================================================================
+
+! User variables
+!---------------
+
+do ii = 1, nscal
+
+  if (isca(ii) .gt. 0) then
+
+    ivar = isca(ii)
+    f_id = ivarfl(ivar)
+
+    call field_get_key_int(ivarfl(ivar), keyvis, f_vis)
+    call field_get_key_int(ivarfl(ivar), keylog, f_log)
+
+    if (ityturt(ii).gt.0) then
+      call field_get_name (f_id, name)
+      f_name = trim(name)//'_turbulent_flux'
+
+      if (ityturt(ii).eq.3) then
+        call add_variable_field(f_name, f_name, 3, iut)
+        iflid = ivarfl(iut)
+
+        call field_set_key_int(iflid, keycpl, 1)
+        ! Tensorial diffusivity
+        call field_get_key_struct_var_cal_opt(iflid, vcopt_dfm)
+        vcopt_dfm%idften = ANISOTROPIC_RIGHT_DIFFUSION
+        call field_set_key_struct_var_cal_opt(iflid, vcopt_dfm)
+
+      else
+        itycat = FIELD_INTENSIVE + FIELD_PROPERTY  ! for properties
+
+        call field_create(f_name, itycat, ityloc, idim3, iprev, iflid)
+
+        call field_set_key_int(iflid, keyvis, f_vis)
+        call field_set_key_int(iflid, keylog, f_log)
+      endif
+
+      call field_set_key_int(ivarfl(ivar), kturt, iturt(ii))
+      call field_set_key_int(ivarfl(ivar), kfturt, iflid)
+
+      ! Elliptic Blending (AFM or DFM)
+      if (iturt(ii).eq.11 .or. iturt(ii).eq.21 .or. iturt(ii).eq.31) then
+        f_name = trim(name)//'_alpha'
+
+        call add_variable_field(f_name, f_name, 1, ialpha)
+        iflid = ivarfl(ialpha)
+
+        ! Elliptic equation (no convection, no time term)
+        call field_get_key_struct_var_cal_opt(iflid, vcopt_alpha)
+        vcopt_alpha%iconv = 0
+        vcopt_alpha%istat = 0
+        call field_set_key_struct_var_cal_opt(iflid, vcopt_alpha)
+
+      endif
+
+    endif
+
+  endif
+
+enddo
+
 !===============================================================================
 ! 2. Additional property fields
 !===============================================================================
@@ -162,7 +236,7 @@ do ii = 1, nscal
       endif
     endif
     ! Now create matching property
-    call add_property_field_owner(f_name, f_label, 1, .false., ifcvsl)
+    call add_property_field(f_name, f_label, 1, .false., ifcvsl)
     call field_set_key_int(ivarfl(isca(ii)), kivisl, ifcvsl)
   endif
 enddo
@@ -183,12 +257,88 @@ do ii = 1, nscal
   endif
 enddo
 
+! Add a scalar density when defined as variable and different from the bulk.
+! WARNING: it must be consitent with continuity equation, this is used
+! for fluid solid computation with apssive scalars with different density in the solid.
+! The kromsl key should be equal to -1 for constant diffusivity,
+! and f_id for a variable diffusivity defined by field f_id
+! Assuming the first field created is not a diffusivity property
+! (we define variables first), f_id > 0, so we use 0 to indicate
+! the density is variable but its field has not been created yet.
+
+do ii = 1, nscal
+  f_id = ivarfl(isca(ii))
+  call field_get_key_int(f_id, kromsl, ifcvsl)
+  if (ifcvsl.eq.0 .and. iscavr(ii).le.0) then
+    ! Build name and label, using a general rule, with a
+    ! fixed name for temperature or enthalpy
+    call field_get_name(f_id, s_name)
+    call field_get_label(f_id, s_label)
+    f_name  = trim(s_name) // '_density'
+    f_label = trim(s_label) // ' Rho'
+
+    ! Now create matching property
+    call add_property_field(f_name, f_label, 1, .false., ifcvsl)
+    call field_set_key_int(ivarfl(isca(ii)), kromsl, ifcvsl)
+  endif
+enddo
+
+! For variances, the diffusivity is that of the associated scalar,
+! and must not be initialized first.
+
+do ii = 1, nscal
+  if (iscavr(ii).gt.0) then
+    f_id = ivarfl(isca(ii))
+    call field_get_key_int(ivarfl(isca(iscavr(ii))), kromsl, ifcvsl)
+    call field_is_key_set(f_id, kromsl, is_set)
+    if (is_set.eqv..true.) then
+      write(nfecra,7040) f_id, ivarfl(isca(iscavr(ii))), ifcvsl
+    else
+      call field_set_key_int(f_id, kromsl, ifcvsl)
+    endif
+  endif
+enddo
+
 ! Boundary roughness
 if (iwallf.ge.5) then
   call add_boundary_property_field_owner('boundary_roughness', 'Boundary Roughness', &
                                          iflid)
 endif
 
+
+! Van Driest damping
+if (idries.eq.-1) then
+  if (iturb.eq.40) then
+    idries = 1
+  elseif (iturb.eq.41.or.iturb.eq.42) then
+    idries = 0
+  endif
+endif
+
+! Wall distance for some turbulence models
+
+if ((iturb.eq.30.and.irijec.eq.1).or.              &
+     (itytur.eq.4.and.idries.eq.1).or.              &
+     iturb.eq.60.or.iturb.eq.70      ) then
+  ineedy = 1
+  f_name  = 'wall_distance'
+  f_label = 'Wall distance'
+  call add_variable_field(f_name, f_label, 1, ivar)
+  iflid = ivarfl(ivar)
+
+  ! Elliptic equation (no convection, no time term)
+  call field_get_key_struct_var_cal_opt(iflid, vcopt)
+  vcopt%iconv = 0
+  vcopt%istat = 0
+  vcopt%nswrsm = 2
+  vcopt%idifft = 0
+  vcopt%relaxv = 1.d0 ! No relaxation, even for steady algorithm.
+  call field_set_key_struct_var_cal_opt(iflid, vcopt)
+
+endif
+
+
+
 !===============================================================================
 ! 3. Additional postprocessing fields
 !===============================================================================
@@ -223,7 +373,7 @@ itycat = FIELD_INTENSIVE + FIELD_PROPERTY
 
 ! In case of condensation or y+ postprocessing, create appropriate field
 
-if (icond.ge.0 .or. ipstdv(ipstyp).ne.0) then
+if (icondb.ge.0.or.icondv.ge.0.or.ipstdv(ipstyp).ne.0) then
   call field_get_id_try('yplus', f_id) ! Test if pre-existing
   call field_find_or_create('yplus', itycat, ityloc, idim1, iyplbr)
   if (f_id .lt. 0) then                ! Set some properties if new
@@ -231,13 +381,19 @@ if (icond.ge.0 .or. ipstdv(ipstyp).ne.0) then
     call field_set_key_int(iyplbr, keylog, 1)
   endif
   ! yplus postprocessed if required
-  if (ipstdv(ipstyp).ne.0) call field_set_key_int(iyplbr, keyvis, 1)
+  flag = POST_ON_LOCATION
+  if (ipstdv(ipstyp).ne.0) call field_set_key_int(iyplbr, keyvis, flag)
 endif
 
 ! Some mappings
 
 call cs_field_pointer_map_boundary
 
+! Cooling towers mappings
+if (ippmod(iaeros).ge.0) then
+  call cs_ctwr_field_pointer_map
+endif
+
 call field_get_id_try('boundary_temperature', itempb)
 
 if (itempb .ge. 0) then
