@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -27,7 +27,8 @@
 !> \file alelav.f90
 !>
 !> \brief This subroutine performs the solving of a Poisson equation
-!> on the mesh velocity for ALE module.
+!> on the mesh velocity for ALE module. It also updates the mesh displacement
+!> so that it can be used to update mass fluxes (due to mesh displacement).
 !>
 !-------------------------------------------------------------------------------
 
@@ -36,11 +37,11 @@
 !______________________________________________________________________________.
 !  mode           name          role                                           !
 !______________________________________________________________________________!
-!> \param[in]     propce        physical properties at cell centers
+!> \param[in]     iterns        Navier-Stokes iteration number
 !_______________________________________________________________________________
 
 subroutine alelav &
- ( propce )
+ ( iterns )
 
 !===============================================================================
 
@@ -56,11 +57,13 @@ use optcal
 use cstnum
 use cstphy
 use pointe
-use albase, only: ialtyb
+use albase, only: ialtyb, iortvm, impale, fdiale
 use parall
 use period
 use mesh
 use field
+use field_operator
+use cs_c_bindings
 
 !===============================================================================
 
@@ -68,44 +71,48 @@ implicit none
 
 ! Arguments
 
-double precision propce(ncelet,*)
+integer          iterns
 
 ! Local variables
 
 character(len=80) :: chaine
-integer          iel   , isou  , jsou  , ifac
-integer          ipcvmx, ipcvmy, ipcvmz
+integer          iel   , isou  , jsou  , ifac, inod
 integer          iflmas, iflmab
 integer          nswrgp, imligp, iwarnp
 integer          iconvp, idiffp, ndircp
 integer          nswrsp, ircflp, ischcp, isstpp, iescap
 integer          ivisep
 integer          iswdyp, idftnp, icvflb
-
 integer          ivoid(1)
+integer          inc, iprev
 
 double precision blencp, epsilp, epsrgp, climgp, thetv
 double precision epsrsp, prosrf
 double precision relaxp
 double precision hint, distbf, srfbn2
 double precision rinfiv(3), pimpv(3)
-
 double precision rvoid(1)
 
-double precision, dimension(:,:), pointer :: cfaale, claale
-double precision, dimension(:,:,:), pointer :: cfbale, clbale
-
+double precision, allocatable, dimension(:,:) :: dproj
+double precision, allocatable, dimension(:,:,:) :: gradm
 double precision, allocatable, dimension(:) :: viscf, viscb
 double precision, allocatable, dimension(:,:) :: smbr
 double precision, allocatable, dimension(:,:,:) :: fimp
+
 double precision, dimension(:), pointer :: imasfl, bmasfl
-double precision, dimension(:), pointer :: brom
-double precision, dimension(:,:), pointer :: mshvel, mshvela
+double precision, dimension(:), pointer :: brom, cpro_vism_s
+double precision, dimension(:,:), pointer :: cfaale, claale
+double precision, dimension(:,:,:), pointer :: cfbale, clbale
+double precision, dimension(:,:), pointer :: mshvel, mshvela, cpro_vism_v
+double precision, dimension(:,:), pointer :: disale, disala
+double precision, pointer, dimension(:)   :: dt
+
+type(var_cal_opt)  vcopt
 
 !===============================================================================
 
 !===============================================================================
-! 1. INITIALIZATION
+! 1. Initialization
 !===============================================================================
 
 ! Allocate temporary arrays for the radiative equations resolution
@@ -116,9 +123,15 @@ allocate(fimp(3,3,ncelet))
 rinfiv(1) = rinfin
 rinfiv(2) = rinfin
 rinfiv(3) = rinfin
-ipcvmx = ipproc(ivisma(1))
-ipcvmy = ipproc(ivisma(2))
-ipcvmz = ipproc(ivisma(3))
+
+call field_get_val_s_by_name('dt', dt)
+
+if (iortvm.eq.0) then
+  call field_get_val_s(ivisma, cpro_vism_s)
+else
+  call field_get_val_v(ivisma, cpro_vism_v)
+endif
+
 ! The mass flux is necessary to call coditv but not used (ICONV=0)
 ! Except for the free surface, where it is used as a Boundary condition
 call field_get_key_int(ivarfl(iu), kimasf, iflmas)
@@ -129,7 +142,9 @@ call field_get_val_s(iflmab, bmasfl)
 call field_get_val_v(ivarfl(iuma), mshvel)
 call field_get_val_prev_v(ivarfl(iuma), mshvela)
 
-if(iwarni(iuma).ge.1) then
+call field_get_key_struct_var_cal_opt(ivarfl(iuma), vcopt)
+
+if(vcopt%iwarni.ge.1) then
   write(nfecra,1000)
 endif
 
@@ -151,12 +166,12 @@ do ifac = 1, nfabor
     iel = ifabor(ifac)
     distbf = distb(ifac)
     srfbn2 = surfbn(ifac)**2
-    if (ipcvmx.eq.ipcvmy) then
-      hint = propce(iel,ipproc(ivisma(1)))/distbf
+    if (iortvm.eq.0) then
+      hint = cpro_vism_s(iel)/distbf
     else !FIXME
-      hint = ( propce(iel,ipproc(ivisma(1)))*surfbo(1,ifac)**2    &
-             + propce(iel,ipproc(ivisma(2)))*surfbo(2,ifac)**2    &
-             + propce(iel,ipproc(ivisma(3)))*surfbo(3,ifac)**2 )  &
+      hint = ( cpro_vism_v(1,iel)*surfbo(1,ifac)**2    &
+             + cpro_vism_v(2,iel)*surfbo(2,ifac)**2    &
+             + cpro_vism_v(3,iel)*surfbo(3,ifac)**2 )  &
            /distbf/srfbn2
     endif
 
@@ -174,10 +189,10 @@ do ifac = 1, nfabor
 enddo
 
 !===============================================================================
-! 2. SOLVING OF THE MESH VELOCITY EQUATION
+! 2. Solving of the mesh velocity equation
 !===============================================================================
 
-if (iwarni(iuma).ge.1) then
+if (vcopt%iwarni.ge.1) then
   call field_get_name(ivarfl(iuma), chaine)
   write(nfecra,1100) chaine(1:16)
 endif
@@ -191,38 +206,38 @@ do iel = 1, ncelet
   enddo
 enddo
 
-if (ipcvmx.eq.ipcvmy) then
+if (iortvm.eq.0) then
   call viscfa &
   !==========
 ( imvisf ,                                                       &
-  propce(1,ipcvmx),                                              &
+  cpro_vism_s ,                                                  &
   viscf  , viscb  )
 else
   call visort &
   !==========
 ( imvisf ,                                                       &
-  propce(1,ipcvmx), propce(1,ipcvmy), propce(1,ipcvmz),          &
+  cpro_vism_v(1,:), cpro_vism_v(2,:), cpro_vism_v(3,:),          &
   viscf  , viscb  )
 endif
 
-iconvp = iconv (iuma)
-idiffp = idiff (iuma)
+iconvp = vcopt%iconv
+idiffp = vcopt%idiff
 ndircp = ndircl(iuma)
-nswrsp = nswrsm(iuma)
-nswrgp = nswrgr(iuma)
-imligp = imligr(iuma)
-ircflp = ircflu(iuma)
-ischcp = ischcv(iuma)
-isstpp = isstpc(iuma)
+nswrsp = vcopt%nswrsm
+nswrgp = vcopt%nswrgr
+imligp = vcopt%imligr
+ircflp = vcopt%ircflu
+ischcp = vcopt%ischcv
+isstpp = vcopt%isstpc
 iescap = 0
-idftnp = idften(iuma)
-iswdyp = iswdyn(iuma)
-iwarnp = iwarni(iuma)
-blencp = blencv(iuma)
-epsilp = epsilo(iuma)
-epsrsp = epsrsm(iuma)
-epsrgp = epsrgr(iuma)
-climgp = climgr(iuma)
+idftnp = vcopt%idften
+iswdyp = vcopt%iswdyn
+iwarnp = vcopt%iwarni
+blencp = vcopt%blencv
+epsilp = vcopt%epsilo
+epsrsp = vcopt%epsrsm
+epsrgp = vcopt%epsrgr
+climgp = vcopt%climgr
 relaxp = 1.d0
 thetv  = 1.d0
 ! all boundary convective flux with upwind
@@ -233,7 +248,7 @@ ivisep = 0
 
 call coditv &
 !==========
- ( idtvar , iuma   , iconvp , idiffp , ndircp ,                   &
+ ( idtvar , iterns , ivarfl(iuma)    , iconvp , idiffp , ndircp , &
    imrgra , nswrsp , nswrgp , imligp , ircflp , ivisep ,          &
    ischcp , isstpp , iescap , idftnp , iswdyp ,                   &
    iwarnp ,                                                       &
@@ -243,6 +258,7 @@ call coditv &
    claale , clbale , cfaale , cfbale ,                            &
    imasfl , bmasfl ,                                              &
    viscf  , viscb  , viscf  , viscb  , viscf  , viscb  ,          &
+   rvoid  , rvoid  , rvoid  ,                                     &!FIXME do a proper anisotropic version
    icvflb , ivoid  ,                                              &
    fimp   ,                                                       &
    smbr   ,                                                       &
@@ -253,8 +269,48 @@ call coditv &
 deallocate(viscf, viscb)
 deallocate(smbr, fimp)
 
+!===============================================================================
+! 3. Update nodes displacement
+!===============================================================================
+
+call field_get_val_v(fdiale, disale)
+call field_get_val_prev_v(fdiale, disala)
+
+! Allocate a temporary array
+allocate(dproj(3,nnod))
+allocate(gradm(3,3,ncelet))
+
+inc = 1
+iprev = 0
+
+call field_gradient_vector(ivarfl(iuma), iprev, imrgra, inc,      &
+                           gradm)
+
+call field_get_coefa_v(ivarfl(iuma), claale)
+call field_get_coefb_v(ivarfl(iuma), clbale)
+
+call aledis &
+ ( ialtyb ,                                                       &
+   mshvel , gradm  ,                                              &
+   claale , clbale ,                                              &
+   dt     , dproj  )
+
+!FIXME warning if nterup > 1, use itrale ?
+! Update mesh displacement only where it is not imposed by the user (ie when impale <> 1)
+do inod = 1, nnod
+  if (impale(inod).eq.0) then
+    do isou = 1, 3
+      disale(isou,inod) = disala(isou,inod) + dproj(isou,inod)
+    enddo
+  endif
+enddo
+
+! Free memory
+deallocate(dproj)
+deallocate(gradm)
+
 !--------
-! FORMATS
+! Formats
 !--------
 
 #if defined(_CS_LANG_FR)
@@ -274,7 +330,7 @@ deallocate(smbr, fimp)
 #endif
 
 !----
-! END
+! End
 !----
 
 return
