@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -67,6 +67,7 @@ use ihmpre
 use radiat
 use cplsat
 use atincl
+use ctincl
 use cfpoin
 use mesh
 use field
@@ -95,10 +96,9 @@ implicit none
 
 ! Local variables
 
-integer          iiii
+logical(kind=c_bool) mesh_modified
 
-integer          modhis, iappel, modntl, iisuit, iwarn0
-integer          ivar
+integer          modhis, iappel, modntl, iisuit, imrgrl
 integer          iel
 
 integer          inod   , idim, ifac
@@ -115,18 +115,10 @@ double precision rvoid(1)
 
 double precision, save :: ttchis
 
-character        ficsui*32
-
-integer, allocatable, dimension(:) :: isostd
-
 double precision, pointer, dimension(:)   :: dt => null()
 double precision, pointer, dimension(:)   :: porosi => null()
-double precision, pointer, dimension(:,:) :: propce => null()
 double precision, pointer, dimension(:,:) :: disale => null()
 
-double precision, pointer, dimension(:,:) :: frcxt => null()
-double precision, pointer, dimension(:)   :: prhyd => null()
-
 !===============================================================================
 ! Interfaces
 !===============================================================================
@@ -135,22 +127,13 @@ interface
 
   subroutine tridim &
   ( itrale , nvar   , nscal  ,                                     &
-    isostd ,                                                       &
-    dt     , propce ,                                              &
-    frcxt  , prhyd  )
-
-    use dimens, only: ndimfb
-    use mesh, only: nfabor
+    dt     )
 
     implicit none
 
     integer                                   :: itrale, nvar, nscal
-    integer, dimension(nfabor+1)              :: isostd
 
     double precision, pointer, dimension(:)   :: dt
-    double precision, pointer, dimension(:,:) :: propce
-    double precision, pointer, dimension(:,:) :: frcxt
-    double precision, pointer, dimension(:)   :: prhyd
 
   end subroutine tridim
 
@@ -164,24 +147,10 @@ end interface
 ! Initialization
 !===============================================================================
 
-! Initialize random number generator
-! (not always necessary, but not at all costly)
-
-! If parallelized particle-tracking is in use,
-! seed the random number generator with
-! (rank number+1) to avoid potential statistical bias
-
-if ((iilagr.gt.0).and.(irangp.ge.0)) then
-  call zufalli(irangp + 1)
-else
-  call zufalli(1)
-endif
-
 !--> Probes output tracking
 ttchis = -1.d0
 
 ! Test presence of control_file to modify ntmabs if required
-!
 
 ntmsav = ntmabs
 
@@ -207,7 +176,14 @@ endif
 ! Geometry
 !===============================================================================
 
-call cregeo
+! Filter extended neighborhood for least-squares gradients
+
+imrgrl = abs(imrgra)
+imrgrl = modulo(imrgrl,10)
+
+if (imrgrl.eq.3 .or. imrgrl.eq.6 .or. imrgrl.eq.9) then
+  call redvse(anomax)
+endif
 
 !===============================================================================
 ! End of modules initialization
@@ -227,21 +203,17 @@ iappel = 1
 allocate(izcpdc(ncel))
 allocate(izctsm(ncel))
 allocate(izftcd(ncel)) ! should be in init_pcond only
-allocate(izft1d(nfabor))
 
 ! ---------
 ! Head-loss
 ! ---------
 
-if (iihmpr.eq.1) then
-  call uikpdc &
-( iappel ,          &
-  ncepdc ,          &
-  ivoid  ,          &
-  rvoid  )
-endif
+! Initialize
 
-call cs_user_head_losses &
+ncetsm = volume_zone_n_type_cells(VOLUME_ZONE_MASS_SOURCE_TERM)
+ncepdc = volume_zone_n_type_cells(VOLUME_ZONE_HEAD_LOSS)
+
+call cs_f_user_head_losses &
 ( ncepdc , iappel ,                                              &
   ivoid  , izcpdc ,                                              &
   rvoid  ,                                                       &
@@ -269,11 +241,13 @@ if (ncpdct.gt.0) then
   ! Add matching field if not already done
   if (idtten.lt.0) then
     call field_create('dttens', FIELD_INTENSIVE, 1, 6, .false., idtten)
+    call field_set_key_int(ivarfl(ipr), kwgrec, idtten)
   endif
 else if (iporos.eq.2) then
   ! Add matching field if not already done
   if (idtten.lt.0) then
     call field_create('dttens', FIELD_INTENSIVE, 1, 6, .false., idtten)
+    call field_set_key_int(ivarfl(ipr), kwgrec, idtten)
   endif
 endif
 
@@ -324,13 +298,9 @@ endif
 ! 1D-wall module
 ! --------------
 
-call uspt1d &
- ( nvar   , nscal  , nfpt1d , iappel ,                            &
-   ivoid  , izft1d , ivoid  , ivoid  ,                            &
-   rvoid  , rvoid  , rvoid  ,                                     &
-   rvoid  , rvoid  , rvoid  ,                                     &
-   rvoid  , rvoid  , rvoid  ,                                     &
-   rvoid  )
+call init_1d_wall_thermal
+
+call cs_user_1d_wall_thermal(iappel, isuit1)
 
 nfpt1t = nfpt1d
 if (irangp.ge.0) then
@@ -342,17 +312,13 @@ if (nfpt1t.gt.0) then
   write(nfecra,3000)
 endif
 
-call vert1d &
-( nfabor , nfpt1d , iappel ,                                      &
-  ivoid  , ivoid  , ivoid  ,                                      &
-  rvoid  , rvoid  ,                                               &
-  rvoid  , rvoid  , rvoid  )
+call cs_1d_wall_thermal_check(iappel, isuit1)
 
 ! Free memory if relevant
 if (ncpdct.eq.0) deallocate(izcpdc)
 if (nctsmt.eq.0) deallocate(izctsm)
 if (nftcdt.eq.0) deallocate(izftcd)
-if (nfpt1t.eq.0) deallocate(izft1d)
+if (nfpt1t.eq.0) call cs_1d_wall_thermal_finalize
 
 ! Formats
 #if defined(_CS_LANG_FR)
@@ -390,26 +356,6 @@ if (nfpt1t.eq.0) deallocate(izft1d)
 ! Memory management
 !===============================================================================
 
-! Allocate main arrays
-
-allocate(dt(ncelet))
-allocate(propce(ncelet,nproce))
-
-! Allocate arrays on boundary faces
-
-allocate(isostd(nfabor+1))
-if (iphydr.eq.1) then
-  allocate(frcxt(3, ncelet))
-else
-  frcxt => rvoid2
-endif
-
-if (iphydr.eq.2) then
-  allocate(prhyd(ncelet))
-else
-  prhyd => rvoid1
-endif
-
 call boundary_conditions_init
 
 call init_aux_arrays(ncelet, nfabor)
@@ -424,6 +370,8 @@ if (ippmod(iatmos).ge.0) then
     call activate_imbrication
   endif
 
+  call cs_at_data_assim_build_ops
+
   if (ifilechemistry.ge.1) then
     call init_chemistry
   endif
@@ -438,7 +386,7 @@ if (ippmod(icompf).ge.0) then
   call init_compf (nfabor)
 endif
 
-if (iale.eq.1.or.imobil.eq.1) then
+if (iale.eq.1) then
   call init_ale (nfabor, nnod)
 endif
 
@@ -455,12 +403,12 @@ if (nftcdt.gt.0) then
   call init_nz_pcond
 endif
 
-if (icond.eq.1) then
+if (icondv.eq.0) then
   call init_vcond ( nvar, ncelet )
 endif
 
 if (nfpt1t.gt.0) then
-  call init_pt1d
+  call init_1d_wall_thermal_local_models
 endif
 
 ! Map arrays from Lagrangian module
@@ -472,20 +420,25 @@ endif
 ! Default initializations
 !===============================================================================
 
-call fldtri(nproce, dt, propce)
+call fldtri
 
 call field_allocate_or_map_all
 
-call iniva0(nvar, nscal, dt, propce, frcxt, prhyd)
+call field_get_val_s_by_name('dt', dt)
+
+call iniva0(nscal)
 
 ! Compute the porosity if needed
 if (iporos.ge.1) then
 
   if (iihmpr.eq.1) then
-    call uiporo(iporos)
+    call uiporo
   endif
   call usporo
 
+  ! C version
+  call user_porosity
+
   call field_get_val_s(ipori, porosi)
 
   if (irangp.ge.0.or.iperio.eq.1) then
@@ -579,7 +532,7 @@ if (isuite.eq.1) then
 
   call timer_stats_start(restart_stats_id)
 
-  call lecamo(frcxt, prhyd)
+  call lecamo
 
   ! Using ALE, geometric parameters must be recalculated
   if (iale.eq.1) then
@@ -608,7 +561,7 @@ endif
 !    dt rom romb viscl visct viscls (tpucou with periodicity)
 !===============================================================================
 
-call inivar(nvar, nscal, dt)
+call inivar(nvar, nscal)
 
 !===============================================================================
 ! Radiative transfer: possible restart
@@ -643,41 +596,19 @@ if (nfpt1t.gt.0) then
 ! Deuxieme appel : remplissage des tableaux de definition de la geometrie
 !            et de l'initialisation (IFPT1D,NPPT1D,EPPT1D,RGPT1D,TPPT1D)
   iappel = 2
-  call  uspt1d &
- ( nvar   , nscal  , nfpt1d , iappel ,                            &
-   ifpt1d , izft1d , nppt1d , iclt1d ,                            &
-   tppt1d , rgpt1d , eppt1d ,                                     &
-   tept1d , hept1d , fept1d ,                                     &
-   xlmbt1 , rcpt1d , dtpt1d ,                                     &
-   dt     )
+  call cs_user_1d_wall_thermal(iappel, isuit1)
 
   iappel = 2
-  call vert1d &
-( nfabor , nfpt1d , iappel ,                                      &
-  ifpt1d , nppt1d , iclt1d ,                                      &
-  rgpt1d , eppt1d ,                                               &
-  xlmbt1 , rcpt1d , dtpt1d )
-
-!     Calcul du max des NPPT1D (pour les fichiers suite)
-  nmxt1d = 0
-  do iiii = 1, nfpt1d
-    nmxt1d = max(nppt1d(iiii),nmxt1d)
-  enddo
-  if (irangp.ge.0) call parcmx(nmxt1d)
+  call cs_1d_wall_thermal_check(iappel, isuit1)
 
   if (isuit1.eq.1) then
 
-    ficsui = '1dwall_module'
-    call lect1d &
- ( ficsui , len(ficsui), nfpt1d , nfpt1t ,           &
-   nmxt1d , nfabor     , nppt1d , ifpt1d , eppt1d , &
-   rgpt1d , tppt1d )
+    call cs_1d_wall_thermal_read
 
   else
-!     Creation du maillage, initialisation de la temperature.
 
-    call mait1d &
- ( nfpt1d, nppt1d, eppt1d, rgpt1d, tppt1d )
+!     Creation du maillage, initialisation de la temperature.
+    call cs_1d_wall_thermal_mesh_create
 
   endif
 
@@ -711,27 +642,19 @@ endif
 ! Arrays for time block, to discard afterwards
 !===============================================================================
 
-! On appelle cs_user_head_losses lorqu'il y a sur un processeur au moins des
-!     cellules avec terme source de masse.
-!     On ne fait que remplir le tableau d'indirection des cellules
-!     On appelle cependant cs_user_head_losses avec tous les processeurs,
-!     au cas ou l'utilisateur aurait mis en oeuvre des operations globales.
-
 if (ncpdct.gt.0) then
 
-  iappel = 2
-
   if (iflow .eq.1) then
     do iel = 1, ncepdc
       icepdc(iel) = iel
     enddo
   endif
 
-  if (iihmpr.eq.1) then
-    call uikpdc(iappel, ncepdc, icepdc, ckupdc)
-  endif
+  call volume_zone_select_type_cells(VOLUME_ZONE_HEAD_LOSS, icepdc)
 
-  call cs_user_head_losses &
+  iappel = 2
+
+  call cs_f_user_head_losses &
 ( ncepdc , iappel ,                                              &
   icepdc , izcpdc ,                                              &
   dt     ,                                                       &
@@ -739,13 +662,15 @@ if (ncpdct.gt.0) then
 
 endif
 
-! On appelle cs_user_mass_source_terms lorqu'il y a sur un processeur au moins
+! On appelle cs_user_mass_source_terms lorsqu'il y a sur un processeur au moins
 !     des cellules avec terme source de masse.
 !     On ne fait que remplir le tableau d'indirection des cellules
 !     On appelle cependant cs_user_mass_source_terms avec tous les processeurs,
 !     au cas ou l'utilisateur aurait mis en oeuvre des operations globales.
 
-if(nctsmt.gt.0) then
+if (nctsmt.gt.0) then
+
+  call volume_zone_select_type_cells(VOLUME_ZONE_MASS_SOURCE_TERM, icetsm)
 
   iappel = 2
   call cs_user_mass_source_terms &
@@ -758,7 +683,6 @@ if(nctsmt.gt.0) then
 
 endif
 
-
 ! -- Methode des vortex pour la L.E.S.
 !    (dans verini on s'est deja assure que ITYTUR=4 si IVRTEX=1)
 
@@ -796,6 +720,18 @@ endif
 
 call cscini(nvar)
 
+! Lagrangian initialization
+
+if (iilagr.gt.0) then
+
+  call timer_stats_start(lagr_stats_id)
+
+  call cs_lagr_solve_initialize(dt)
+
+  call timer_stats_stop(lagr_stats_id)
+
+endif
+
 !===============================================================================
 ! Start of time loop
 !===============================================================================
@@ -805,16 +741,9 @@ write(nfecra,2000)
 ntcabs = ntpabs
 ttcabs = ttpabs
 
-if (imobil.eq.1 .or. iturbo.eq.2)  ttcmob = ttpmob
+if (iturbo.eq.2)  ttcmob = ttpmob
 
-iwarn0 = 1
-do ivar = 1, nvar
-  iwarn0 = max(iwarn0,iwarni(ivar))
-enddo
-
-if(iwarn0.gt.0) then
-  write(nfecra,3000)
-endif
+write(nfecra,3000)
 
 if(inpdt0.eq.1) then
   ntmabs = ntcabs
@@ -850,6 +779,19 @@ if (itrale.gt.0) then
 
 endif
 
+! Possible postprocessing of initialization values
+
+call timer_stats_start(post_stats_id)
+
+call post_activate_by_time_step
+call cs_user_postprocess_activate(ntmabs, ntcabs, ttcabs)
+
+call pstvar(nvar, nscal)
+
+call timer_stats_stop(post_stats_id)
+
+! Start time loop
+
  100  continue
 
 if (inpdt0.eq.0 .and. itrale.gt.0) then
@@ -860,10 +802,8 @@ if (inpdt0.eq.0 .and. itrale.gt.0) then
   else
     ttcabs = ttcabs + dtref
   endif
-  if (iwarn0.gt.0) then
-    write(nfecra,3001) ttcabs,ntcabs
-  endif
-  if (imobil.eq.1 .or. iturbo.eq.2) then
+  write(nfecra,3001) ttcabs,ntcabs
+  if (iturbo.eq.2) then
     if(idtvar.eq.0.or.idtvar.eq.1) then
       ttcmob = ttcmob + dt(1)
     else
@@ -885,15 +825,16 @@ if (      (idtvar.eq.0 .or. idtvar.eq.1)                          &
   ntmabs = ntcabs
 endif
 
+mesh_modified = .false.
+call cs_volume_zone_build_all(mesh_modified)
+call cs_boundary_zone_build_all(mesh_modified)
+
 call dmtmps(titer1)
 
 call tridim                                                       &
  ( itrale ,                                                       &
    nvar   , nscal  ,                                              &
-   isostd ,                                                       &
-   dt     ,                                                       &
-   propce ,                                                       &
-   frcxt  , prhyd  )
+   dt     )
 
 
 if (inpdt0.eq.0 .and. itrale.gt.0) then
@@ -931,6 +872,7 @@ if (itrale.gt.0) then
 
   if (iihmpr.eq.1) then
     call uiprof()
+    call uiexop()
   endif
 
   call cs_f_user_extra_operations &
@@ -953,8 +895,6 @@ if (iale.eq.1 .and. inpdt0.eq.0) then
 
     call alemav &
   ( itrale ,                                               &
-    impale , ialtyb ,                                      &
-    dt     ,                                               &
     xyzno0 )
 
   endif
@@ -990,32 +930,25 @@ if (iisuit.eq.1) then
   call timer_stats_start(restart_stats_id)
 
   if(ntcabs.lt.ntmabs) then
-    if (iwarn0.gt.0) write(nfecra,3020) ntcabs, ttcabs
+    write(nfecra,3020) ntcabs, ttcabs
   else if(ntcabs.eq.ntmabs) then
-    if(iwarn0.gt.0) write(nfecra,3021)ntcabs,ttcabs
+    write(nfecra,3021)ntcabs,ttcabs
   endif
 
-  call ecrava(frcxt, prhyd)
+  call ecrava
 
   if (iturbo.eq.2 .and. iecaux.eq.1) then
     call trbsui
   endif
 
   if (nfpt1t.gt.0) then
-    ficsui = '1dwall_module'
-    call ecrt1d                                                   &
- ( ficsui   , len(ficsui), nfpt1d   ,  nmxt1d  ,                  &
-   nfabor   , tppt1d     , ifpt1d )
+    call cs_1d_wall_thermal_write
   endif
 
   if (nent.gt.0) then
     call ecrsyn('les_inflow'//c_null_char)
   endif
 
-  if (ippmod(iaeros).ge.0) then
-    call ecrctw ('cooling_towers'//c_null_char)
-  endif
-
   if (iilagr.gt.0) then
     call lagout()
   endif
@@ -1034,6 +967,8 @@ endif ! iisuit = 1
 ! Test to determine if a visualization output is generated
 !===============================================================================
 
+call timer_stats_start(post_stats_id)
+
 call post_activate_by_time_step
 
 if (iihmpr.eq.1) then
@@ -1042,22 +977,20 @@ endif
 
 call cs_user_postprocess_activate(ntmabs, ntcabs, ttcabs)
 
-!===============================================================================
-! Standard visualization output
-!===============================================================================
-
 ! If ITRALE=0, deactivate all writers, as geometry has not
 !              been output yet.
 if (itrale.eq.0) then
   call post_activate_writer(0, .false.)
 endif
 
-call timer_stats_start(post_stats_id)
+!===============================================================================
+! Standard visualization output
+!===============================================================================
 
-call pstvar(ntcabs, nvar, nscal)
+call pstvar(nvar, nscal)
 
 !===============================================================================
-! Probes
+! Structures
 !===============================================================================
 
 if ((nthist.gt.0 .or.frhist.gt.0.d0) .and. itrale.gt.0) then
@@ -1075,12 +1008,6 @@ if ((nthist.gt.0 .or.frhist.gt.0.d0) .and. itrale.gt.0) then
 
     ttchis = ttcabs
 
-    call ecrhis(modhis)
-
-    ! if (iilagr.gt.0) then
-    !   call laghis
-    ! endif
-
     if (ihistr.eq.1) then
       call strhis(modhis)
     endif
@@ -1089,8 +1016,6 @@ if ((nthist.gt.0 .or.frhist.gt.0.d0) .and. itrale.gt.0) then
 
 endif
 
-call ushist(nvar, nscal, dt)
-
 !===============================================================================
 ! Write to "listing" every ntlist iterations
 !===============================================================================
@@ -1104,19 +1029,19 @@ else
 endif
 if (modntl.eq.0) then
 
-  call ecrlis &
-  ( ncelet , ncel   ,                                    &
-    dt     , volume )
+  call ecrlis(ncelet, ncel, dt, volume)
 
   call log_iteration
 
+  call log_l2residual
+
 endif
 
 call timer_stats_stop(post_stats_id)
 
 call dmtmps(titer2)
 
-if (iwarn0.gt.0 .and. itrale.le.0) then
+if (itrale.le.0) then
   write(nfecra,3012)titer2-titer1
 endif
 
@@ -1140,16 +1065,13 @@ endif
 ! Finalize probes
 !===============================================================================
 
-if(iwarn0.gt.0) then
-  write(nfecra,4000)
-endif
+write(nfecra,4000)
 
 ! Ici on sauve les historiques (si on en a stocke)
 
 call timer_stats_start(post_stats_id)
 
 modhis = 2
-call ecrhis(modhis)
 
 ! if (iilagr.gt.0) then
 !   call laghis
@@ -1165,20 +1087,10 @@ call timer_stats_stop(post_stats_id)
 !     ET/OU ON FERME LE LISTING LAGRANGIEN
 
 if (nfpt1d.gt.0) then
-  call lbrt1d
+  call cs_1d_wall_thermal_free
 endif
 
 ! Free main arrays
-deallocate(dt, propce)
-
-deallocate(isostd)
-if (iphydr.eq.1) then
-  deallocate(frcxt)
-endif
-
-if (iphydr.eq.2) then
-  deallocate(prhyd)
-endif
 
 if (ivrtex.eq.1) then
   deallocate(irepvo)
@@ -1200,6 +1112,8 @@ if (ippmod(iatmos).ge.0) then
     call finalize_imbrication
   endif
 
+  call cs_at_data_assim_finalize
+
   if (ifilechemistry.ge.1) then
     call finalize_chemistry
   endif
@@ -1210,7 +1124,11 @@ if (ippmod(icompf).ge.0) then
   call finalize_compf
 endif
 
-if (iale.eq.1.or.imobil.eq.1) then
+if (ippmod(igmix).ge.0) then
+  call finalize_gas_mix
+endif
+
+if (iale.eq.1) then
   call finalize_ale
 endif
 
@@ -1232,7 +1150,7 @@ if(nftcdt.gt.0) then
   endif
 endif
 
-if (icond.eq.1) then
+if (icondv.eq.0) then
   call finalize_vcond
   if (itagms.eq.1) then
     call finalize_tagms
@@ -1240,7 +1158,7 @@ if (icond.eq.1) then
 endif
 
 if (nfpt1d.gt.0) then
-  call finalize_pt1d
+  call cs_1d_wall_thermal_finalize
 endif
 
 if (ivrtex.eq.1) then
@@ -1254,7 +1172,6 @@ endif
 !     Liberation des structures liees a la lecture du fichier xml
 
 if (iihmpr.eq.1) then
-  call memui2
   call memui1(ncharb)
 endif
 
