@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -45,7 +45,7 @@
 !>
 !> Warning:
 !>
-!> - for a vector field such as the veclocity \f$ \vect{u} \f$ the boundary
+!> - for a vector field such as the velocity \f$ \vect{u} \f$ the boundary
 !>   conditions may read:
 !>   \f[
 !>   \vect{u}_\centf = \vect{A}_u^g + \tens{B}_u^g \vect{u}_\centi
@@ -56,6 +56,9 @@
 !>   \f]
 !>   where \f$ \tens{B}_u^g \f$ and \f$ \tens{B}_u^f \f$ are 3x3 tensor matrix
 !>   which coupled veclocity components next to a boundary.
+!>
+!> Please refer to the <a href="../../theory.pdf#cpltrg"><b>clptrg</b></a> section
+!> of the theory guide for more informations.
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
@@ -119,6 +122,7 @@ use numvar
 use optcal
 use cstphy
 use cstnum
+use dimens, only: nvar
 use pointe
 use entsor
 use albase
@@ -132,6 +136,7 @@ use mesh
 use field
 use lagran
 use turbomachinery
+use cs_c_bindings
 
 !===============================================================================
 
@@ -141,9 +146,9 @@ implicit none
 
 integer          nscal, isvhb
 
-integer          icodcl(nfabor,nvarcl)
+integer          icodcl(nfabor,nvar)
 
-double precision rcodcl(nfabor,nvarcl,3)
+double precision rcodcl(nfabor,nvar,3)
 double precision velipb(nfabor,ndim), rijipb(nfabor,6)
 double precision visvdr(ncelet)
 double precision hbord(nfabor),theipb(nfabor)
@@ -154,7 +159,6 @@ integer          ifac, iel, isou, ii, jj, kk
 integer          iscal
 integer          modntl
 integer          iuntur, iuiptn, f_id
-integer          ifccp
 
 double precision rnx, rny, rnz, rxnn
 double precision tx, ty, tz, txn, txn0, t2x, t2y, t2z
@@ -178,6 +182,8 @@ double precision distb0, rugd  , rugt  , ydep
 double precision dsa0
 double precision rinfiv(3)
 double precision fcoefa(6), fcoefb(6), fcofaf(6), fcofbf(6), fcofad(6), fcofbd(6)
+double precision rxx, rxy, rxz, ryy, ryz, rzz, rnnb
+double precision rttb, alpha_rnn
 
 double precision, dimension(:), pointer :: crom
 double precision, dimension(:), pointer :: viscl, visct, cpro_cp, yplbr, uetbor
@@ -185,6 +191,8 @@ double precision, dimension(:), allocatable :: byplus, buk, buet, bcfnns
 
 double precision, dimension(:), pointer :: cvar_k
 double precision, dimension(:), pointer :: cvar_r11, cvar_r22, cvar_r33
+double precision, dimension(:), pointer :: cvar_r12, cvar_r13, cvar_r23
+double precision, dimension(:,:), pointer :: cvar_rij
 double precision, dimension(:), pointer :: cvara_nusa
 
 double precision, dimension(:,:), pointer :: coefau, cofafu
@@ -221,6 +229,8 @@ integer          ntlast , iaff
 data             ntlast , iaff /-1 , 0/
 save             ntlast , iaff
 
+type(var_cal_opt) :: vcopt
+
 !===============================================================================
 
 !===============================================================================
@@ -252,6 +262,9 @@ cfnne=1.d0
 und0   = 1.d0
 deuxd0 = 2.d0
 
+! Alpha constant for a realisable BC for R12 with the SSG model
+alpha_rnn = 0.47d0
+
 ! pointers to y+ if saved
 
 yplbr => null()
@@ -460,6 +473,7 @@ if (inusa.gt.0) then
   call field_get_coefaf_s(ivarfl(inusa), coefaf_nu)
   call field_get_coefb_s(ivarfl(inusa), coefb_nu)
   call field_get_coefbf_s(ivarfl(inusa), coefbf_nu)
+  call field_get_key_struct_var_cal_opt(ivarfl(inusa), vcopt)
 else
   cvara_nusa => null()
   coefa_nu => null()
@@ -477,18 +491,22 @@ if (itytur.eq.2 .or. itytur.eq.5                             &
 endif
 
 if(itytur.eq.3) then
-  call field_get_val_s(ivarfl(ir11), cvar_r11)
-  call field_get_val_s(ivarfl(ir22), cvar_r22)
-  call field_get_val_s(ivarfl(ir33), cvar_r33)
+  if (irijco.eq.1) then
+    call field_get_val_v(ivarfl(irij), cvar_rij)
+  else
+    call field_get_val_s(ivarfl(ir11), cvar_r11)
+    call field_get_val_s(ivarfl(ir22), cvar_r22)
+    call field_get_val_s(ivarfl(ir33), cvar_r33)
+    call field_get_val_s(ivarfl(ir12), cvar_r12)
+    call field_get_val_s(ivarfl(ir13), cvar_r13)
+    call field_get_val_s(ivarfl(ir23), cvar_r23)
+  endif
 endif
 
-call field_get_val_s(iprpfl(iviscl), viscl)
-call field_get_val_s(iprpfl(ivisct), visct)
-if (icp.gt.0) then
-  ifccp  = iprpfl(icp)
-  call field_get_val_s(ifccp, cpro_cp)
-else
-  ifccp = -1
+call field_get_val_s(iviscl, viscl)
+call field_get_val_s(ivisct, visct)
+if (icp.ge.0) then
+  call field_get_val_s(icp, cpro_cp)
 endif
 
 ! min. and max. of wall tangential velocity
@@ -567,7 +585,7 @@ do ifac = 1, nfabor
     !  to be tangential (and update rcodcl for possible use)
     ! In frozen rotor (iturbo = 1), the velocity is neither tangential to the
     !  wall (absolute velocity solved in a relative frame of reference)
-    if (iale.eq.0.and.imobil.eq.0.and.iturbo.eq.0) then
+    if (iale.eq.0.and.iturbo.eq.0) then
       rcodcn = rcodcx*rnx+rcodcy*rny+rcodcz*rnz
       rcodcx = rcodcx -rcodcn*rnx
       rcodcy = rcodcy -rcodcn*rny
@@ -723,18 +741,41 @@ do ifac = 1, nfabor
     ! Dimensionless velocity
     uplus = log(yplus+1.d0)/xkappa
 
+    if (itytur.eq.2 .or. itytur.eq.5 .or. iturb.eq.60) then
+      ek = cvar_k(iel)
+    else if(itytur.eq.3) then
+      if (irijco.eq.1) then
+        ek = 0.5d0*(cvar_rij(1,iel)+cvar_rij(2,iel)+cvar_rij(3,iel))
+        rxx = cvar_rij(1,iel)
+        rxy = cvar_rij(4,iel)
+        rxz = cvar_rij(6,iel)
+        ryy = cvar_rij(2,iel)
+        ryz = cvar_rij(5,iel)
+        rzz = cvar_rij(3,iel)
+      else
+        ek = 0.5d0*(cvar_r11(iel)+cvar_r22(iel)+cvar_r33(iel))
+        rxx = cvar_r11(iel)
+        rxy = cvar_r12(iel)
+        rxz = cvar_r13(iel)
+        ryy = cvar_r22(iel)
+        ryz = cvar_r23(iel)
+        rzz = cvar_r33(iel)
+      endif
+      rnnb =   rnx * (rxx * rnx + rxy * rny + rxz * rnz) &
+        + rny * (rxy * rnx + ryy * rny + ryz * rnz) &
+        + rnz * (rxz * rnx + ryz * rny + rzz * rnz)
+
+      rttb =   tx * (rxx * tx + rxy * ty + rxz * tz) &
+        + ty * (rxy * tx + ryy * ty + ryz * tz) &
+        + tz * (rxz * tx + ryz * ty + rzz * tz)
+    endif
+
     if (iwallf.le.2) then
       uk = uet
 
     ! Si iwallf= 3, 4 ou 5 on calcule uk et uet
     else
 
-      if (itytur.eq.2 .or. itytur.eq.5 .or. iturb.eq.60) then
-        ek = cvar_k(iel)
-      else if(itytur.eq.3) then
-        ek = 0.5d0*(cvar_r11(iel)+cvar_r22(iel)+cvar_r33(iel))
-      endif
-
       uk = cmu025*sqrt(ek)
 
     endif
@@ -842,7 +883,7 @@ do ifac = 1, nfabor
       ! Clipping :
       ! On borne U_f,grad entre 0 et Utau (il y a surement mieux...)
       ! - 0    : on interdit le retournement en face de bord, qui est en
-      !          contradiction avec l'hypothse de loi log.
+      !          contradiction avec l'hypoth\E8se de loi log.
       ! - Utau : la production turbulente ne peut etre nulle
       ! On empeche U_f,flux d'etre negatif
 
@@ -996,7 +1037,27 @@ do ifac = 1, nfabor
           kk = 3
         endif
 
-        if (iclptr.eq.1) then
+        ! Coupled version: we implicit as much as possible
+        if (irijco.eq.1) then
+          coefa_rij(isou, ifac) = - (  eloglo(jj,1)*eloglo(kk,2)         &
+                                     + eloglo(jj,2)*eloglo(kk,1))        &
+                                    * alpha_rnn * sqrt(rnnb * rttb) * cfnnk
+          coefaf_rij(isou, ifac)      = -hint * coefa_rij(isou, ifac)
+          coefad_rij(isou, ifac)      = 0.d0
+          do ii = 1, 6
+            coefb_rij(isou,ii, ifac)  = alpha(ii,isou)
+            if (ii.eq.isou) then
+              coefbf_rij(isou,ii, ifac) = hint &
+                                        * (1.d0 - coefb_rij(isou,ii, ifac))
+            else
+              coefbf_rij(isou,ii, ifac) = - hint &
+                                        * coefb_rij(isou,ii, ifac)
+            endif
+            coefbd_rij(isou,ii, ifac) = coefb_rij(isou,ii, ifac)
+          enddo
+
+        else if (iclptr.eq.1) then
+
           do ii = 1, 6
             if (ii.ne.isou) then
               fcoefa(isou) = fcoefa(isou) + alpha(isou,ii) * rijipb(ifac,ii)
@@ -1009,6 +1070,7 @@ do ifac = 1, nfabor
           enddo
           fcoefb(isou) = 0.d0
         endif
+
         ! Boundary conditions for the momentum equation
         fcofad(isou) = fcoefa(isou)
         fcofbd(isou) = fcoefb(isou)
@@ -1021,15 +1083,9 @@ do ifac = 1, nfabor
         fcofaf(isou) = -hint*fcoefa(isou)
         fcofbf(isou) = hint*(1.d0-fcoefb(isou))
       enddo
-      do isou = 1, 6
-        if (irijco.eq.1) then
-          coefa_rij(isou, ifac)       = fcoefa(isou)
-          coefb_rij(isou,isou, ifac)  = fcoefb(isou)
-          coefaf_rij(isou, ifac)      = fcofaf(isou)
-          coefbf_rij(isou,isou, ifac) = fcofbf(isou)
-          coefad_rij(isou, ifac)      = fcofad(isou)
-          coefbd_rij(isou,isou, ifac) = fcofbd(isou)
-        else
+
+      if (irijco.ne.1) then
+        do isou = 1, 6
           if (isou.eq.1) then
             coefa_r11(ifac) = fcoefa(isou)
             coefb_r11(ifac) = fcoefb(isou)
@@ -1073,8 +1129,8 @@ do ifac = 1, nfabor
             coefad_r13(ifac) = fcofad(isou)
             coefbd_r13(ifac) = fcofbd(isou)
           endif
-        endif
-      enddo
+        enddo
+      endif
 
       ! ---> Epsilon
       ! same treatment as k-epsilon
@@ -1249,7 +1305,8 @@ do ifac = 1, nfabor
     elseif (iturb.eq.70) then
 
       dsa0 = rugd
-      hint = (visclc + idifft(inusa)*cvara_nusa(iel)*romc*dsa0/(distbf+dsa0) ) / distbf / csasig
+      hint = (visclc + vcopt%idifft*cvara_nusa(iel)*romc*dsa0/(distbf+dsa0) ) &
+            / distbf / csasig
 
       ! If we have a rough wall then:
       ! nusa_wall*(1- I'F/d0)=nusa_I'
@@ -1329,7 +1386,9 @@ deallocate(bcfnns)
 !       On indique aussi le numero du dernier pas de temps auquel on
 !       a rencontre des yplus hors bornes admissibles
 
-if (iwarni(iu).ge.0) then
+call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt)
+
+if (vcopt%iwarni.ge.0) then
   if (ntlist.gt.0) then
     modntl = mod(ntcabs,ntlist)
   elseif (ntlist.eq.-1.and.ntcabs.eq.ntmabs) then
@@ -1338,11 +1397,11 @@ if (iwarni(iu).ge.0) then
     modntl = 1
   endif
 
-  if ((modntl.eq.0 .or. iwarni(iu).ge.2).and.iscalt.gt.0) then
+  if ((modntl.eq.0 .or. vcopt%iwarni.ge.2).and.iscalt.gt.0) then
     write(nfecra,2011) &
          uiptmn,uiptmx,uetmin,uetmax,ukmin,ukmax,yplumn,yplumx,   &
          tetmin, tetmax, tplumn, tplumx, iuiptn
-  elseif (modntl.eq.0 .or. iwarni(iu).ge.2) then
+  elseif (modntl.eq.0 .or. vcopt%iwarni.ge.2) then
     write(nfecra,2010) &
          uiptmn,uiptmx,uetmin,uetmax,ukmin,ukmax,yplumn,yplumx,   &
          iuiptn
@@ -1504,6 +1563,7 @@ use numvar
 use optcal
 use cstphy
 use cstnum
+use dimens, only: nvar
 use pointe
 use entsor
 use albase
@@ -1517,6 +1577,7 @@ use mesh
 use field
 use lagran
 use turbomachinery
+use cs_c_bindings
 
 !===============================================================================
 
@@ -1526,9 +1587,9 @@ implicit none
 
 integer          iscal, isvhb
 
-integer          icodcl(nfabor,nvarcl)
+integer          icodcl(nfabor,nvar)
 
-double precision rcodcl(nfabor,nvarcl,3)
+double precision rcodcl(nfabor,nvar,3)
 double precision byplus(nfabor)
 double precision hbord(nfabor), theipb(nfabor)
 double precision buk(nfabor), buet(nfabor), bcfnns(nfabor)
@@ -1538,7 +1599,7 @@ double precision tetmax, tetmin, tplumx, tplumn
 
 integer          ivar, f_id, b_f_id, isvhbl
 integer          ifac, iel, isou, jsou
-integer          ifccp, ifccv, ifcvsl, itplus, itstar
+integer          ifcvsl, itplus, itstar
 
 double precision cpp, rkl, prdtl, visclc, romc, tplus, cpscv
 double precision distfi, distbf, fikis, hint, heq, hflui, hext
@@ -1547,11 +1608,12 @@ double precision viscis, visctc, cofimp
 double precision act, rugt
 double precision rinfiv(3), pimpv(3)
 double precision visci(3,3), hintt(6)
+double precision turb_schmidt, exchange_coef
 
 character(len=80) :: fname
 
 double precision, dimension(:), pointer :: val_s, bval_s, crom, viscls
-double precision, dimension(:), pointer :: viscl, visct, cpro_cp, cv
+double precision, dimension(:), pointer :: viscl, visct, cpro_cp, cpro_cv
 
 double precision, dimension(:), pointer :: bfconv, bhconv
 double precision, dimension(:), pointer :: tplusp, tstarp
@@ -1561,6 +1623,8 @@ double precision, dimension(:,:,:), pointer :: coefbut, cofbfut, cofbrut
 
 integer, save :: kbfid = -1
 
+type(var_cal_opt) :: vcopt
+
 !===============================================================================
 
 ivar = isca(iscal)
@@ -1568,8 +1632,8 @@ f_id = ivarfl(ivar)
 
 call field_get_val_s(ivarfl(ivar), val_s)
 
-call field_get_val_s(iprpfl(iviscl), viscl)
-call field_get_val_s(iprpfl(ivisct), visct)
+call field_get_val_s(iviscl, viscl)
+call field_get_val_s(ivisct, visct)
 
 call field_get_key_int (f_id, kivisl, ifcvsl)
 
@@ -1577,7 +1641,9 @@ if (ifcvsl .ge. 0) then
   call field_get_val_s(ifcvsl, viscls)
 endif
 
-if (idften(ivar).eq.6.or.ityturt(iscal).eq.3) then
+call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+
+if (iand(vcopt%idften, ANISOTROPIC_DIFFUSION).ne.0.or.ityturt(iscal).eq.3) then
   if (iturb.ne.32.or.ityturt(iscal).eq.3) then
     call field_get_val_v(ivsten, visten)
   else ! EBRSM and (GGDH or AFM)
@@ -1591,18 +1657,13 @@ call field_get_coefaf_s(f_id, cofafp)
 call field_get_coefbf_s(f_id, cofbfp)
 
 call field_get_val_s(icrom, crom)
-if (icp.gt.0) then
-  ifccp = iprpfl(icp)
-  call field_get_val_s(ifccp, cpro_cp)
-else
-  ifccp = -1
+if (icp.ge.0) then
+  call field_get_val_s(icp, cpro_cp)
 endif
 
-ifccv = -1
 if (ippmod(icompf) .ge. 0) then
-  if (icv.gt.0) then
-    ifccv  = iprpfl(icv)
-    call field_get_val_s(ifccv, cv)
+  if (icv.ge.0) then
+    call field_get_val_s(icv, cpro_cv)
   endif
 endif
 
@@ -1620,6 +1681,10 @@ if (iscal.eq.iscalt) then
   tplumn =  grand
 endif
 
+rinfiv(1) = rinfin
+rinfiv(2) = rinfin
+rinfiv(3) = rinfin
+
 if (ityturt(iscal).eq.3) then
 
   ! Name of the scalar ivar
@@ -1678,6 +1743,9 @@ else
   endif
 endif
 
+! retrieve turbulent Schmidt value for current scalar
+call field_get_key_double(ivarfl(isca(iscal)), ksigmas, turb_schmidt)
+
 ! --- Loop on boundary faces
 do ifac = 1, nfabor
 
@@ -1700,7 +1768,7 @@ do ifac = 1, nfabor
 
     cpp = 1.d0
     if (iscacp(iscal).eq.1) then
-      if (ifccp.ge.0) then
+      if (icp.ge.0) then
         cpp = cpro_cp(iel)
       else
         cpp = cp0
@@ -1723,54 +1791,54 @@ do ifac = 1, nfabor
     ! Mu*Cv/Lambda.
 
     if (iscal.eq.iscalt .and. itherm.eq.3) then
-      if (ifccp.ge.0) then
+      if (icp.ge.0) then
         prdtl = prdtl*cpro_cp(iel)
       else
         prdtl = prdtl*cp0
       endif
-      if (ifccv.ge.0) then
-        prdtl = prdtl/cv(iel)
+      if (icv.ge.0) then
+        prdtl = prdtl/cpro_cv(iel)
       else
         prdtl = prdtl/cv0
       endif
     endif
 
     ! Scalar diffusivity
-    if (idften(ivar).eq.1) then
-      ! En compressible, pour l'energie LAMBDA/CV+CP/CV*(MUT/SIGMAS)
+    if (iand(vcopt%idften, ISOTROPIC_DIFFUSION).ne.0) then
+      ! En compressible, pour l'energie LAMBDA/CV+CP/CV*(MUT/TURB_SCHMIDT)
       if (ippmod(icompf) .ge. 0) then
-        if (ifccp.ge.0) then
+        if (icp.ge.0) then
           cpscv = cpro_cp(iel)
         else
           cpscv = cp0
         endif
-        if (ifccv.ge.0) then
-          cpscv = cpscv/cv(iel)
+        if (icv.ge.0) then
+          cpscv = cpscv/cpro_cv(iel)
         else
           cpscv = cpscv/cv0
         endif
-        hint = (rkl+idifft(ivar)*cpp*cpscv*visctc/sigmas(iscal))/distbf
+        hint = (rkl+vcopt%idifft*cpp*cpscv*visctc/turb_schmidt)/distbf
       else
-        hint = (rkl+idifft(ivar)*cpp*visctc/sigmas(iscal))/distbf
+        hint = (rkl+vcopt%idifft*cpp*visctc/turb_schmidt)/distbf
       endif
 
       ! Symmetric tensor diffusivity (GGDH or AFM)
-    elseif (idften(ivar).eq.6) then
+    elseif (iand(vcopt%idften, ANISOTROPIC_DIFFUSION).ne.0) then
       ! En compressible, pour l'energie LAMBDA/CV+CP/CV*(MUT/SIGMAS)
       if (ippmod(icompf) .ge. 0) then
-        if (ifccp.ge.0) then
+        if (icp.ge.0) then
           cpscv = cpro_cp(iel)
         else
           cpscv = cp0
         endif
-        if (ifccv.ge.0) then
-          cpscv = cpscv/cv(iel)
+        if (icv.ge.0) then
+          cpscv = cpscv/cpro_cv(iel)
         else
           cpscv = cpscv/cv0
         endif
-        temp = idifft(ivar)*cpp*cpscv*ctheta(iscal)
+        temp = vcopt%idifft*cpp*cpscv*ctheta(iscal)
       else
-        temp = idifft(ivar)*cpp*ctheta(iscal)
+        temp = vcopt%idifft*cpp*ctheta(iscal)
       endif
       visci(1,1) = temp*visten(1,iel) + rkl
       visci(2,2) = temp*visten(2,iel) + rkl
@@ -1830,8 +1898,6 @@ do ifac = 1, nfabor
       tplus = romc*cpp*max(buet(ifac)*bcfnns(ifac),epzero)/hint
     endif
 
-    if (isvhbl.gt.0) hbord(ifac) = hflui
-
     hext = rcodcl(ifac,ivar,2)
     pimp = rcodcl(ifac,ivar,1)
 
@@ -1889,7 +1955,7 @@ do ifac = 1, nfabor
           pimpv(isou) = surfbo(isou,ifac)*phit/(surfbn(ifac)*cpp*romc)
         enddo
 
-        call set_dirichlet_vector_ggdh &
+        call set_dirichlet_vector_aniso &
              !========================
            ( coefaut(:,ifac)  , cofafut(:,ifac)  ,           &
              coefbut(:,:,ifac), cofbfut(:,:,ifac),           &
@@ -1905,76 +1971,45 @@ do ifac = 1, nfabor
 
       endif
 
-      !--> Radiative module:
-
-      ! On stocke le coefficient d'echange lambda/distance
-      ! (ou son equivalent en turbulent) quelle que soit la
-      ! variable thermique transportee (temperature ou enthalpie)
-      ! car on l'utilise pour realiser des bilans aux parois qui
-      ! sont faits en temperature (on cherche la temperature de
-      ! paroi quelle que soit la variable thermique transportee pour
-      ! ecrire des eps sigma T4.
-
-      !     donc :
-
-      !       lorsque la variable transportee est la temperature
-      !         abs(iscsth(ii)).eq.1 : hconv(ifac) = hint
-      !         puisque hint = visls * cp / distbr
-      !                      = lambda/distance en W/(m2 K)
-
-      !       lorsque la variable transportee est l'enthalpie
-      !         iscsth(ii).eq.2 : hconv(ifac) = hint*cpr
-      !         avec
-      !            if (ifccp.ge.0) then
-      !              cpr = cpro_cp(iel)
-      !            else
-      !              cpr = cp0
-      !            endif
-      !         puisque hint = visls / distbr
-      !                      = lambda/(cp * distance)
-
-      !       lorsque la variable transportee est l'energie (compressible)
-      !         iscsth(ii).eq.3 :
-      !         on procede comme pour l'enthalpie avec CV au lieu de CP
-      !         (rq : il n'y a pas d'hypothese, sf en non orthogonal :
-      !               le flux est le bon et le coef d'echange aussi)
-
-      !      De meme dans condli.
-
-      !               Si on rayonne et que
-      !                  le scalaire est la variable energetique
-
-      if (iirayo.ge.1 .and. iscal.eq.iscalt) then
-
-        ! We compute the exchange coefficient in W/(m2 K)
-
+      ! Storage of the thermal exchange coefficient
+      ! (conversion in case of energy or enthaly)
+      ! the exchange coefficient is in W/(m2 K)
+      ! Usefull for thermal coupling or radiative transfer
+      if (iirayo.ge.1 .and. iscal.eq.iscalt.or.isvhbl.gt.0) then
         ! Enthalpy
         if (itherm.eq.2) then
           ! If Cp is variable
-          if (ifccp.ge.0) then
-            bhconv(ifac) = hflui*cpro_cp(iel)
+          if (icp.ge.0) then
+            exchange_coef = hflui*cpro_cp(iel)
           else
-            bhconv(ifac) = hflui*cp0
+            exchange_coef = hflui*cp0
           endif
 
-          ! Energie (compressible module)
+          ! Total energy (compressible module)
         elseif (itherm.eq.3) then
           ! If Cv is variable
-          if (ifccv.ge.0) then
-            bhconv(ifac) = hflui*cv(iel)
+          if (icv.ge.0) then
+            exchange_coef = hflui*cpro_cv(iel)
           else
-            bhconv(ifac) = hflui*cv0
+            exchange_coef = hflui*cv0
           endif
 
           ! Temperature
         elseif (iscacp(iscal).eq.1) then
-          bhconv(ifac) = hflui
+          exchange_coef = hflui
         endif
+      endif
+
+      ! ---> Thermal coupling, store h = lambda/d
+      if (isvhbl.gt.0) hbord(ifac) = exchange_coef
+
+      ! ---> Radiative transfer
+      if (iirayo.ge.1 .and. iscal.eq.iscalt) then
+        bhconv(ifac) = exchange_coef
 
         ! The outgoing flux is stored (Q = h(Ti'-Tp): negative if
         !  gain for the fluid) in W/m2
-        bfconv(ifac) =   cofafp(ifac)              &
-                       + cofbfp(ifac)*theipb(ifac)
+        bfconv(ifac) = cofafp(ifac) + cofbfp(ifac)*theipb(ifac)
       endif
 
     endif ! End if icodcl=6
@@ -2019,4 +2054,3 @@ enddo
 
 return
 end subroutine
-
