@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -44,6 +44,10 @@
 #include <unistd.h>
 #endif
 
+#if defined(HAVE_DLOPEN)
+#include <dlfcn.h>
+#endif
+
 /*----------------------------------------------------------------------------
  * PLE library headers
  *----------------------------------------------------------------------------*/
@@ -60,7 +64,8 @@
 #include "bft_mem.h"
 #include "bft_printf.h"
 
-#include "cs_prototypes.h"
+#include "cs_file.h"
+#include "cs_fp_exception.h"
 #include "cs_log.h"
 #include "cs_timer.h"
 
@@ -80,6 +85,8 @@ BEGIN_C_DECLS
  * Local Macro Definitions
  *============================================================================*/
 
+#define DIR_SEPARATOR '/'
+
 /* Fortran API */
 /*-------------*/
 
@@ -147,6 +154,14 @@ static _cs_base_sighandler_t cs_glob_base_sigtrap_save = SIG_DFL;
 static _cs_base_sighandler_t cs_glob_base_sigcpu_save = SIG_DFL;
 #endif
 
+#if defined(HAVE_DLOPEN)
+#if defined(CS_DLOPEN_USE_RTLD_GLOBAL)
+static int _cs_dlopen_flags = RTLD_LAZY | RTLD_GLOBAL;
+#else
+static int _cs_dlopen_flags = RTLD_LAZY;
+#endif
+#endif
+
 /* Installation paths */
 
 static const char _cs_base_build_localedir[] = LOCALEDIR;
@@ -1079,7 +1094,7 @@ cs_base_logfile_head(int    argc,
              "                      Version %s\n\n",
              CS_APP_VERSION);
 
-  bft_printf("\n  Copyright (C) 1998-2016 EDF S.A., France\n\n");
+  bft_printf("\n  Copyright (C) 1998-2018 EDF S.A., France\n\n");
 
 #if defined(CS_REVISION)
   if (strlen(CS_REVISION) > 0)
@@ -1151,14 +1166,13 @@ cs_base_mpi_init(int    *argc,
 
 #if   defined(__bg__) || defined(__CRAYXT_COMPUTE_LINUX_TARGET)
 
-  /* Notes: Blue Gene/L also defines the BGLMPI_SIZE environment variable.
-   *        Blue Gene/P defines BG_SIZE (plus BG_MAPPING, and BG_RELEASE). */
+  /* Blue Gene/Q or Cray: assume MPI is always used. */
 
   use_mpi = true;
 
 #elif defined(MPICH) || defined(MPICH2) || defined(MSMPI_VER)
 
-  /* Notes: Microsoft MPI is based on standard MPICH2 */
+  /* Notes: Microsoft MPI is based on MPICH */
 
   if (getenv("PMI_RANK") != NULL)
     use_mpi = true;
@@ -1166,45 +1180,19 @@ cs_base_mpi_init(int    *argc,
   else if (getenv("PCMPI") != NULL) /* Platform MPI */
     use_mpi = true;
 
-#if defined(MPICH_NAME)
-#if (MPICH_NAME == 1)
-
-  /*
-    Using standard MPICH1 1.2.x with the p4 (default) mechanism,
-    the information required by MPI_Init() are transferred through
-    the command line, which is then modified by MPI_Init();
-    in this case, only rank 0 knows the "user" command line arguments
-    at program startup, the other processes obtaining them only upon
-    calling  MPI_Init(). In this case, it is thus necessary to initialize
-    MPI before parsing the command line.
-  */
-
-  for (arg_id = 0 ; arg_id < *argc ; arg_id++) {
-    if (   !strcmp((*argv)[arg_id], "-p4pg")         /* For process 0 */
-        || !strcmp((*argv)[arg_id], "-p4rmrank")) {  /* For other processes */
-      use_mpi = true;
-      break;
-    }
-  }
-
-  if (getenv("GMPI_ID") != NULL) /* In case we are using MPICH-GM */
-    use_mpi = true;
-
-#endif /* End for MPICH-1 or variants */
-#endif
-
-#elif defined(LAM_MPI)
-  if (getenv("LAMRANK") != NULL)
-    use_mpi = true;
-
 #elif defined(OPEN_MPI)
   if (getenv("OMPI_MCA_ns_nds_vpid") != NULL)         /* OpenMPI 1.2 */
     use_mpi = true;
-  else if (getenv("OMPI_COMM_WORLD_RANK") != NULL)    /* OpenMPI 1.3 */
+  else if (getenv("OMPI_COMM_WORLD_RANK") != NULL)    /* OpenMPI 1.3 + */
     use_mpi = true;
 
 #endif /* Tests for known MPI variants */
 
+  /* Test for run through SLURM's srun */
+
+  if (getenv("SLURM_SRUN_COMM_HOST") != NULL)
+    use_mpi = true;
+
   /* If we have determined from known MPI environment variables
      of command line arguments that we are running under MPI,
      initialize MPI */
@@ -2004,6 +1992,223 @@ cs_base_check_bool(bool *b)
   }
 }
 
+/*----------------------------------------------------------------------------
+ * Open a data file in read mode.
+ *
+ * If a file of the given name in the working directory is found, it
+ * will be opened. Otherwise, it will be searched for in the "data/thch"
+ * subdirectory of pkgdatadir.
+ *
+ * parameters:
+ *   base_name      <-- base file name
+ *
+ * returns:
+ *   pointer to opened file
+ *----------------------------------------------------------------------------*/
+
+FILE *
+cs_base_open_properties_data_file(const char  *base_name)
+{
+  FILE *f = NULL;
+
+  char *_f_name = NULL;
+  const char *file_name = base_name;
+
+  /* choose local file if present, default otherwise */
+
+  if (! cs_file_isreg(file_name)) {
+    const char *datadir = cs_base_get_pkgdatadir();
+    const char subdir[] = "/data/thch/";
+    BFT_MALLOC(_f_name,
+               strlen(datadir) + strlen(subdir) + strlen(base_name) + 1,
+               char);
+    sprintf(_f_name, "%s%s%s", datadir, subdir, base_name);
+    file_name = _f_name;
+  }
+
+  f = fopen(file_name, "r");
+
+  if (f == NULL)
+    bft_error(__FILE__, __LINE__, errno,
+              _("Error opening data file \"%s\""), file_name);
+
+  BFT_FREE(_f_name);
+
+  return f;
+}
+
+#if defined(HAVE_DLOPEN)
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Load a dynamic library.
+ *
+ * \param[in]  filename  path to shared library file
+ *
+ * \return  handle to shared library
+ */
+/*----------------------------------------------------------------------------*/
+
+void*
+cs_base_dlopen(const char *filename)
+{
+  void *retval = NULL;
+
+  /* Disable floating-point traps as the initialization of some libraries
+     may interfere with this (for example, embree, and optional Paraview
+     depedency) */
+
+  cs_fp_exception_disable_trap();
+
+  /* Load symbols from shared library */
+
+  retval = dlopen(filename, _cs_dlopen_flags);
+
+  if (retval == NULL)
+    bft_error(__FILE__, __LINE__, 0,
+              _("Error loading %s: %s."), filename, dlerror());
+
+  /* Restore floating-point trap behavior */
+
+  cs_fp_exception_restore_trap();
+
+  return retval;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Load a plugin's dynamic library
+ *
+ * This function is similar to \ref cs_base_dlopen, execpt that only
+ * the base plugin file name (with no extension) needs to be given.
+ * It is assumed the file is available in the code's "pkglibdir" directory,
+ *
+ * \param[in]  filename  path to shared library file
+ *
+ * \return  handle to shared library
+ */
+/*----------------------------------------------------------------------------*/
+
+void *
+cs_base_dlopen_plugin(const char *name)
+{
+  void *retval = NULL;
+
+  char  *lib_path = NULL;
+  const char *pkglibdir = cs_base_get_pkglibdir();
+
+  /* Open shared library */
+
+  BFT_MALLOC(lib_path,
+             strlen(pkglibdir) + 1 + 3 + strlen(name) + 3 + 1,
+             char);
+
+  sprintf(lib_path, "%s%c%s.so", pkglibdir, DIR_SEPARATOR, name);
+
+  retval = cs_base_dlopen(lib_path);
+
+  BFT_FREE(lib_path);
+
+  return retval;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Get flags for dlopen.
+ *
+ * \return  flags used for dlopen.
+ */
+/*----------------------------------------------------------------------------*/
+
+int
+cs_base_dlopen_get_flags(void)
+{
+  return _cs_dlopen_flags;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Set flags for dlopen.
+ *
+ * \param[in]  flags  flags to set
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_base_dlopen_set_flags(int flags)
+{
+  _cs_dlopen_flags = flags;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Unload a dynamic library.
+ *
+ * Note that the dlopen underlying mechanism uses a reference count, so
+ * a library is really unloaded only one \ref cs_base_dlclose has been called
+ * the same number of times as \ref cs_base_dlopen.
+ *
+ * \param[in]  filename  optional path to shared library file name for error
+ *                       logging, or NULL
+ * \param[in]  handle    handle to shared library
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_base_dlclose(const char  *filename,
+                void        *handle)
+{
+  int retval = 0;
+
+  if (handle != NULL)
+    retval = dlclose(handle);
+
+  if (retval != 0) {
+    if (filename != NULL)
+      bft_error(__FILE__, __LINE__, 0,
+                _("Error decrementing count or unloading %s: %s."),
+                filename, dlerror());
+    else
+      bft_error(__FILE__, __LINE__, 0,
+                _("Error decrementing count or unloading %s."),
+                dlerror());
+  }
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Get a shared library function pointer
+ *
+ * \param[in]  handle            handle to shared library
+ * \param[in]  name              name of function symbol in library
+ * \param[in]  errors_are_fatal  abort if true, silently ignore if false
+ *
+ * \return  pointer to function in shared library
+ */
+/*----------------------------------------------------------------------------*/
+
+void *
+cs_base_get_dl_function_pointer(void        *handle,
+                                const char  *name,
+                                bool         errors_are_fatal)
+{
+  void  *retval = NULL;
+  char  *error = NULL;
+
+  dlerror();    /* Clear any existing error */
+
+  retval = dlsym(handle, name);
+  error = dlerror();
+
+  if (error != NULL && errors_are_fatal)
+    bft_error(__FILE__, __LINE__, 0,
+              _("Error calling dlsym for %s: %s\n"), name, error);
+
+  return retval;
+}
+
+#endif /* defined(HAVE_DLOPEN) */
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
