@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -27,10 +27,18 @@
 
 /*----------------------------------------------------------------------------*/
 
+#include "cs_defs.h"
+
 /*----------------------------------------------------------------------------
  * Standard C library headers
  *----------------------------------------------------------------------------*/
 
+#include <stdio.h>
+
+/*----------------------------------------------------------------------------
+ *  Local headers
+ *----------------------------------------------------------------------------*/
+
 /*=============================================================================
  * Macro definitions
  *============================================================================*/
@@ -86,6 +94,25 @@ typedef void (cs_base_atexit_t) (void);
  * Public function prototypes
  *============================================================================*/
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Return a string "true" or "false" according to the boolean
+ *
+ * \param[in]  boolean  bool type
+ *
+ * \return a string "true" or "false"
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline const char *
+cs_base_strtf(bool  boolean)
+{
+  if (boolean)
+    return "true";
+  else
+    return "false";
+}
+
 /*----------------------------------------------------------------------------
  * First analysis of the command line to determine an application name.
  *
@@ -368,6 +395,115 @@ cs_base_get_pkglibdir(void);
 void
 cs_base_check_bool(bool *b);
 
+/*----------------------------------------------------------------------------
+ * Open a data file in read mode.
+ *
+ * If a file of the given name in the working directory is found, it
+ * will be opened. Otherwise, it will be searched for in the "data/thch"
+ * subdirectory of pkgdatadir.
+ *
+ * parameters:
+ *   base_name      <-- base file name
+ *
+ * returns:
+ *   pointer to opened file
+ *----------------------------------------------------------------------------*/
+
+FILE *
+cs_base_open_properties_data_file(const char  *base_name);
+
+#if defined(HAVE_DLOPEN)
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Load a dynamic library.
+ *
+ * \param[in]  filename  path to shared library file.
+ *
+ * \return  handle to shared library
+ */
+/*----------------------------------------------------------------------------*/
+
+void*
+cs_base_dlopen(const char *filename);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Load a plugin's dynamic library
+ *
+ * This function is similar to \ref cs_base_dlopen, execpt that only
+ * the base plugin file name (with no extension) needs to be given.
+ * It is assumed the file is available in the code's "pkglibdir" directory,
+ *
+ * \param[in]  filename  path to shared library file
+ *
+ * \return  handle to shared library
+ */
+/*----------------------------------------------------------------------------*/
+
+void*
+cs_base_dlopen_plugin(const char *name);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Get flags for dlopen.
+ *
+ * \return  flags used for dlopen.
+ */
+/*----------------------------------------------------------------------------*/
+
+int
+cs_base_dlopen_get_flags(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Set flags for dlopen.
+ *
+ * \param[in]  flags  flags to set
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_base_dlopen_set_flags(int flags);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Unload a dynamic library.
+ *
+ * Note that the dlopen underlying mechanism uses a reference count, so
+ * a library is really unloaded only one \ref cs_base_dlclose has been called
+ * the same number of times as \ref cs_base_dlopen.
+ *
+ * \param[in]  filename  optional path to shared library file name for error
+ *                       logging, or NULL
+ * \param[in]  handle    handle to shared library
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_base_dlclose(const char  *filename,
+                void        *handle);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Get a shared library function pointer
+ *
+ * \param[in]  handle            handle to shared library
+ * \param[in]  name              name of function symbol in library
+ * \param[in]  errors_are_fatal  abort if true, silently ignore if false
+ *
+ * \return  pointer to function in shared library
+ */
+/*----------------------------------------------------------------------------*/
+
+void *
+cs_base_get_dl_function_pointer(void        *handle,
+                                const char  *name,
+                                bool         errors_are_fatal);
+
+
+#endif /* defined(HAVE_DLOPEN) */
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
