@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -258,6 +258,7 @@ typedef enum {
   CS_CHAR,               /* character values */
   CS_FLOAT,              /* 4-byte floating point values */
   CS_DOUBLE,             /* 8-byte floating point values */
+  CS_UINT16,             /* 2-byte unsigned integer values */
   CS_INT32,              /* 4-byte signed integer values */
   CS_INT64,              /* 8-byte signed integer values */
   CS_UINT32,             /* 4-byte unsigned integer values */
@@ -313,6 +314,7 @@ typedef cs_real_t  cs_real_9_t[9];      /* Vector of 9 real values */
 
 typedef cs_real_t  cs_real_33_t[3][3];  /* Matrix of 3x3 real values */
 typedef cs_real_t  cs_real_66_t[6][6];  /* Matrix of 6x6 real values */
+typedef cs_real_t  cs_real_99_t[9][9];  /* Matrix of 9x9 real values */
 
 typedef cs_real_t  cs_real_34_t[3][4];  /* Matrix of 3x4 real values */
 
@@ -330,6 +332,14 @@ typedef struct {
 
 } cs_double_int_t;
 
+/* Vector-valued quantity stored using its measure (i.e. length) and
+   its direction given by a unitary vector */
+typedef struct {
+
+  double  meas;
+  double  unitv[3];
+
+} cs_nvec3_t;
 
 /* Mappings to MPI datatypes */
 /*---------------------------*/
@@ -358,8 +368,9 @@ typedef struct {
 #    define CS_MPI_GNUM       MPI_UNSIGNED
 #  endif
 
-#  define CS_MPI_LNUM     MPI_INT         /* MPI type for cs_lnum_t type */
-#  define CS_MPI_COORD    MPI_DOUBLE      /* MPI type for cs_coord_t type */
+#  define CS_MPI_FLAG     MPI_UNSIGNED_SHORT /* MPI type for cs_flag_t type */
+#  define CS_MPI_LNUM     MPI_INT            /* MPI type for cs_lnum_t type */
+#  define CS_MPI_COORD    MPI_DOUBLE         /* MPI type for cs_coord_t type */
 
 #endif /* defined(HAVE_MPI) && !defined(CS_IGNORE_MPI) */
 
@@ -386,6 +397,7 @@ typedef struct {
 # define CS_INT_TYPE      CS_INT32
 #endif
 
+#define CS_FLAG_TYPE      CS_UINT16
 #define CS_REAL_TYPE      CS_DOUBLE
 #define CS_COORD_TYPE     CS_DOUBLE
 
@@ -433,6 +445,7 @@ typedef enum {
  *----------------------------------------------------------------------------*/
 
 #define CS_UNUSED(x) (void)(x)
+#define CS_NO_WARN_IF_UNUSED(x) (void)(x)
 
 /*----------------------------------------------------------------------------
  * Macros for compilation with a C++ compiler
@@ -449,16 +462,6 @@ typedef enum {
 #  define   END_C_DECLS
 #endif
 
-/*----------------------------------------------------------------------------
- * Macros for scoping of examples
- *----------------------------------------------------------------------------*/
-
-#undef BEGIN_EXAMPLE_SCOPE
-#undef END_EXAMPLE_SCOPE
-
-#define BEGIN_EXAMPLE_SCOPE  {
-#define   END_EXAMPLE_SCOPE  }
-
 /*----------------------------------------------------------------------------
  * Macros for Fortran interoperability
  *----------------------------------------------------------------------------*/
