@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -113,6 +113,9 @@ typedef struct {
   cs_real_t         *ac;           /* Explicit coefficient for convection */
   cs_real_t         *bc;           /* Implicit coefficient for convection */
 
+  cs_real_t         *hint;         /* coefficient for internal coupling */
+  cs_real_t         *hext;         /* coefficient for internal coupling */
+
 } cs_field_bc_coeffs_t;
 
 /* Field descriptor */
@@ -208,31 +211,33 @@ cs_field_create(const char   *name,
                 int           dim,
                 bool          has_previous);
 
-/*----------------------------------------------------------------------------
- * Return a field matching a given name and attributes,
- * creating it if necessary.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Return a field matching a given name and attributes,
+ *        creating it if necessary.
  *
  * If a field with the same name but different attributes is present,
  * this is considered an error.
  *
  * The default number of time values associated with a field created through
- * this function is 1. To modify it, use cs_field_set_n_time_vals().
+ * this function is 1. To modify it, use \ref cs_field_set_n_time_vals.
  *
- * parameters:
- *   name        <-- field name
- *   type_flag   <-- mask of field property and category values
- *   location_id <-- id of associated location
- *   dim         <-- field dimension (number of components)
+ * \param[in]  name          field name
+ * \param[in]  type_flag     mask of field property and category values
+ * \param[in]  location_id   id of associated location
+ * \param[in]  dim           field dimension (number of components)
+ * \param[in]  has_previous  maintain values at the previous time step ?
  *
- * returns:
- *   pointer to field
- *----------------------------------------------------------------------------*/
+ * \return  pointer to field
+ */
+/*----------------------------------------------------------------------------*/
 
 cs_field_t *
 cs_field_find_or_create(const char   *name,
                         int           type_flag,
                         int           location_id,
-                        int           dim);
+                        int           dim,
+                        bool          has_previous);
 
 /*----------------------------------------------------------------------------
  * Change the number of time values managed by a field.
@@ -544,12 +549,13 @@ cs_field_define_key_str(const char  *name,
  * by the current value, and its id is returned.
  *
  * parameters:
- *   name          <-- key name
- *   default_value <-- pointer to default value associated with key
- *   log_funct     <-- pointer to logging function
- *   size          <-- sizeof structure
- *   type_flag     <-- mask associated with field types with which
- *                     the key may be associated, or 0
+ *   name             <-- key name
+ *   default_value    <-- pointer to default value associated with key
+ *   log_funct        <-- pointer to logging function
+ *   log_func_default <-- pointer to default logging function
+ *   size             <-- sizeof structure
+ *   type_flag        <-- mask associated with field types with which
+ *                        the key may be associated, or 0
  *
  * returns:
  *   id associated with key
@@ -559,6 +565,7 @@ int
 cs_field_define_key_struct(const char                 *name,
                            const void                 *default_value,
                            cs_field_log_key_struct_t  *log_func,
+                           cs_field_log_key_struct_t  *log_func_default,
                            size_t                      size,
                            int                         type_flag);
 
@@ -714,6 +721,52 @@ int
 cs_field_get_key_int(const cs_field_t  *f,
                      int                key_id);
 
+/*----------------------------------------------------------------------------
+ * Set integer bits matching a mask to 1 for a given key for a field.
+ *
+ * If the key id is not valid, CS_FIELD_INVALID_KEY_ID is returned.
+ * If the field category is not compatible with the key (as defined
+ * by its type flag), CS_FIELD_INVALID_CATEGORY is returned.
+ * If the data type does not match, CS_FIELD_INVALID_TYPE is returned.
+ * If the key value has been locked, CS_FIELD_LOCKED is returned.
+ *
+ * parameters:
+ *   f             <-- pointer to field structure
+ *   key_id        <-- id of associated key
+ *   mask          <-- mask associated with key
+ *
+ * returns:
+ *   0 in case of success, > 1 in case of error
+ *----------------------------------------------------------------------------*/
+
+int
+cs_field_set_key_int_bits(cs_field_t  *f,
+                          int          key_id,
+                          int          mask);
+
+/*----------------------------------------------------------------------------
+ * Set integer bits matching a mask to 0 for a given key for a field.
+ *
+ * If the key id is not valid, CS_FIELD_INVALID_KEY_ID is returned.
+ * If the field category is not compatible with the key (as defined
+ * by its type flag), CS_FIELD_INVALID_CATEGORY is returned.
+ * If the data type does not match, CS_FIELD_INVALID_TYPE is returned.
+ * If the key value has been locked, CS_FIELD_LOCKED is returned.
+ *
+ * parameters:
+ *   f             <-- pointer to field structure
+ *   key_id        <-- id of associated key
+ *   mask          <-- mask associated with key
+ *
+ * returns:
+ *   0 in case of success, > 1 in case of error
+ *----------------------------------------------------------------------------*/
+
+int
+cs_field_clear_key_int_bits(cs_field_t  *f,
+                            int          key_id,
+                            int          mask);
+
 /*----------------------------------------------------------------------------
  * Assign a floating point value for a given key to a field.
  *
@@ -840,6 +893,47 @@ cs_field_get_key_struct(const cs_field_t  *f,
                         int                key_id,
                         void              *s);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Return a pointer to a simple structure for a given key to a field.
+ *
+ * If the key id is not valid, the value type or field category is not
+ * compatible, or the structure has been locked, a fatal error is provoked.
+ *
+ * Note that using this function marks the field's value for this structure
+ * as set, and if no values have been set yet, the structure is set to
+ * default values.
+ *
+ * \param[in]  f       pointer to field structure
+ * \param[in]  key_id  id of associated key
+ *
+ * \return  pointer to key structure in case of success, NULL in case of error
+ */
+/*----------------------------------------------------------------------------*/
+
+void *
+cs_field_get_key_struct_ptr(cs_field_t  *f,
+                            int          key_id);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Return a read-only pointer to a simple structure for a given key
+ *        to a field.
+ *
+ * If the key id is not valid, the value type or field category is not
+ * compatible, a fatal error is provoked.
+ *
+ * \param[in]  f       pointer to field structure
+ * \param[in]  key_id  id of associated key
+ *
+ * \return  pointer to key structure in case of success, NULL in case of error
+ */
+/*----------------------------------------------------------------------------*/
+
+const void *
+cs_field_get_key_struct_const_ptr(const cs_field_t  *f,
+                                  int                key_id);
+
 /*----------------------------------------------------------------------------
  * Print info relative to all field definitions to log file.
  *----------------------------------------------------------------------------*/
@@ -909,7 +1003,6 @@ cs_field_log_all_key_vals(bool  log_defaults);
  *   "label"        (string)
  *   "log"          (integer)
  *   "post_vis"     (integer)
- *   "post_probes"  (integer)
  *   "coupled"      (integer, restricted to CS_FIELD_VARIABLE)
  *   "moment_id"    (integer, restricted to
  *                   CS_FIELD_ACCUMULATOR | CS_FIELD_POSTPROCESS);
