@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -49,6 +49,7 @@
 #include "cs_gradient.h"
 #include "cs_gradient_perio.h"
 #include "cs_halo.h"
+#include "cs_halo_perio.h"
 #include "cs_mesh.h"
 #include "cs_log.h"
 #include "cs_map.h"
@@ -57,6 +58,7 @@
 #include "cs_mesh.h"
 #include "cs_mesh_location.h"
 #include "cs_mesh_quantities.h"
+#include "cs_internal_coupling.h"
 
 /*----------------------------------------------------------------------------
  * Header for the current file
@@ -155,7 +157,7 @@ void cs_f_field_gradient_tensor(int                     f_id,
  *----------------------------------------------------------------------------*/
 
 static void
-_field_interpolate_by_mean(cs_field_t         *f,
+_field_interpolate_by_mean(const cs_field_t   *f,
                            cs_lnum_t           n_points,
                            const cs_lnum_t     point_location[],
                            cs_real_t          *val)
@@ -185,7 +187,7 @@ _field_interpolate_by_mean(cs_field_t         *f,
  *----------------------------------------------------------------------------*/
 
 static void
-_field_interpolate_by_gradient(cs_field_t         *f,
+_field_interpolate_by_gradient(const cs_field_t   *f,
                                cs_lnum_t           n_points,
                                const cs_lnum_t     point_location[],
                                const cs_real_3_t   point_coords[],
@@ -269,6 +271,88 @@ _field_interpolate_by_gradient(cs_field_t         *f,
   BFT_FREE(grad);
 }
 
+/*----------------------------------------------------------------------------
+ * For each mesh cell this function finds the local extrema of a
+ * scalar field.
+ *
+ * parameters:
+ *   pvar            <-- scalar values
+ *   halo_type       <-- halo type
+ *   local_max       --> local maximum value
+ *   local_min       --> local minimum value
+ *----------------------------------------------------------------------------*/
+
+static void
+_local_extrema_scalar(const cs_real_t *restrict pvar,
+                      cs_halo_type_t            halo_type,
+                      cs_real_t                *local_max,
+                      cs_real_t                *local_min)
+{
+  const cs_mesh_t *m = cs_glob_mesh;
+  const cs_lnum_t n_cells = m->n_cells;
+  const cs_lnum_t n_cells_ext = m->n_cells_with_ghosts;
+  const int n_i_groups = m->i_face_numbering->n_groups;
+  const int n_i_threads = m->i_face_numbering->n_threads;
+  const cs_lnum_t *restrict i_group_index = m->i_face_numbering->group_index;
+
+  const cs_lnum_t *restrict cell_cells_idx
+    = (const cs_lnum_t *restrict) m->cell_cells_idx;
+  const cs_lnum_t *restrict cell_cells_lst
+    = (const cs_lnum_t *restrict) m->cell_cells_lst;
+  const cs_lnum_2_t *restrict i_face_cells
+    = (const cs_lnum_2_t *restrict)m->i_face_cells;
+
+# pragma omp parallel for
+  for (cs_lnum_t ii = 0; ii < n_cells_ext; ii++) {
+    local_max[ii] = pvar[ii];
+    local_min[ii] = pvar[ii];
+  }
+
+  /* Contribution from interior faces */
+
+  for (int g_id = 0; g_id < n_i_groups; g_id++) {
+#   pragma omp parallel for
+    for (int t_id = 0; t_id < n_i_threads; t_id++) {
+      for (cs_lnum_t face_id = i_group_index[(t_id*n_i_groups + g_id)*2];
+          face_id < i_group_index[(t_id*n_i_groups + g_id)*2 + 1];
+          face_id++) {
+
+        cs_lnum_t ii = i_face_cells[face_id][0];
+        cs_lnum_t jj = i_face_cells[face_id][1];
+
+        cs_real_t pi = pvar[ii];
+        cs_real_t pj = pvar[jj];
+
+        local_max[ii] = CS_MAX(local_max[ii], pj);
+        local_max[jj] = CS_MAX(local_max[jj], pi);
+        local_min[ii] = CS_MIN(local_min[ii], pj);
+        local_min[jj] = CS_MIN(local_min[jj], pi);
+      }
+    }
+  }
+
+  /* Contribution from extended neighborhood */
+
+  if (halo_type == CS_HALO_EXTENDED) {
+#   pragma omp parallel for
+    for (cs_lnum_t ii = 0; ii < n_cells; ii++) {
+      for (cs_lnum_t cidx = cell_cells_idx[ii];
+           cidx < cell_cells_idx[ii + 1];
+           cidx++) {
+        cs_lnum_t jj = cell_cells_lst[cidx];
+
+        cs_real_t pi = pvar[ii];
+        cs_real_t pj = pvar[jj];
+
+        local_max[ii] = CS_MAX(local_max[ii], pj);
+        local_max[jj] = CS_MAX(local_max[jj], pi);
+        local_min[ii] = CS_MIN(local_min[ii], pj);
+        local_min[jj] = CS_MIN(local_min[jj], pi);
+      }
+    }
+  } /* End for extended neighborhood */
+}
+
 /*============================================================================
  * Fortran wrapper function definitions
  *============================================================================*/
@@ -475,19 +559,34 @@ cs_field_gradient_scalar(const cs_field_t          *f,
   int tr_dim = 0;
   int w_stride = 1;
   int key_cal_opt_id = cs_field_key_id("var_cal_opt");
-  cs_real_t *weight = NULL;
+  cs_real_t *c_weight = NULL;
+  cs_internal_coupling_t  *cpl = NULL;
   cs_var_cal_opt_t var_cal_opt;
 
   /* Get the calculation option from the field */
+
   cs_field_get_key_struct(f, key_cal_opt_id, &var_cal_opt);
   if (f->type & CS_FIELD_VARIABLE && var_cal_opt.iwgrec == 1) {
     if (var_cal_opt.idiff > 0) {
+      /* Weighted gradient coefficients */
       int key_id = cs_field_key_id("gradient_weighting_id");
       int diff_id = cs_field_get_key_int(f, key_id);
       if (diff_id > -1) {
-        cs_field_t *weight_f = cs_field_by_id(diff_id);
-        weight = weight_f->val;
-        w_stride = weight_f->dim;
+        cs_field_t *f_weight = cs_field_by_id(diff_id);
+        c_weight = f_weight->val;
+        w_stride = f_weight->dim;
+      }
+    }
+  }
+
+  if (f->type & CS_FIELD_VARIABLE) {
+    if (var_cal_opt.idiff > 0) {
+      /* Internal coupling structure */
+      int key_id = cs_field_key_id_try("coupling_entity");
+      if (key_id > -1) {
+        int coupl_id = cs_field_get_key_int(f, key_id);
+        if (coupl_id > -1)
+          cpl = cs_internal_coupling_by_id(coupl_id);
       }
     }
   }
@@ -514,7 +613,8 @@ cs_field_gradient_scalar(const cs_field_t          *f,
                      f->bc_coeffs->a,
                      f->bc_coeffs->b,
                      var,
-                     weight,
+                     c_weight,
+                     cpl, /* internal coupling */
                      grad);
 }
 
@@ -552,7 +652,7 @@ cs_field_gradient_potential(const cs_field_t          *f,
   cs_real_t *var = (use_previous_t) ? f->val_pre : f->val;
 
   int key_cal_opt_id = cs_field_key_id("var_cal_opt");
-  cs_real_t *weight = NULL;
+  cs_real_t *c_weight = NULL;
   cs_var_cal_opt_t var_cal_opt;
 
   /* Get the calculation option from the field */
@@ -562,9 +662,9 @@ cs_field_gradient_potential(const cs_field_t          *f,
       int key_id = cs_field_key_id("gradient_weighting_id");
       int diff_id = cs_field_get_key_int(f, key_id);
       if (diff_id > -1) {
-        cs_field_t *weight_f = cs_field_by_id(diff_id);
-        weight = weight_f->val;
-        w_stride = weight_f->dim;
+        cs_field_t *f_weight = cs_field_by_id(diff_id);
+        c_weight = f_weight->val;
+        w_stride = f_weight->dim;
       }
     }
   }
@@ -587,14 +687,14 @@ cs_field_gradient_potential(const cs_field_t          *f,
                      f->bc_coeffs->a,
                      f->bc_coeffs->b,
                      var,
-                     weight,
+                     c_weight,
+                     NULL, /* internal coupling */
                      grad);
 }
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Compute cell gradient of scalar field or component of vector or
- * tensor field.
+ * \brief  Compute cell gradient of vector field.
  *
  * \param[in]       f               pointer to field
  * \param[in]       use_previous_t  should we use values from the previous
@@ -616,9 +716,33 @@ cs_field_gradient_vector(const cs_field_t          *f,
 {
   int key_cal_opt_id = cs_field_key_id("var_cal_opt");
   cs_var_cal_opt_t var_cal_opt;
+  cs_real_t *c_weight = NULL;
+  cs_internal_coupling_t  *cpl = NULL;
 
   /* Get the calculation option from the field */
   cs_field_get_key_struct(f, key_cal_opt_id, &var_cal_opt);
+  if (f->type & CS_FIELD_VARIABLE && var_cal_opt.iwgrec == 1) {
+    if (var_cal_opt.idiff > 0) {
+      /* Weighted gradient coefficients */
+      int key_id = cs_field_key_id("gradient_weighting_id");
+      int diff_id = cs_field_get_key_int(f, key_id);
+      if (diff_id > -1) {
+        cs_field_t *f_weight = cs_field_by_id(diff_id);
+        c_weight = f_weight->val;
+      }
+    }
+  }
+
+  if (f->type & CS_FIELD_VARIABLE) {
+    if (var_cal_opt.idiff > 0) {
+      int key_id = cs_field_key_id_try("coupling_entity");
+      if (key_id > -1) {
+        int coupl_id = cs_field_get_key_int(f, key_id);
+        if (coupl_id > -1)
+          cpl = cs_internal_coupling_by_id(coupl_id);
+      }
+    }
+  }
 
   cs_real_3_t *var = (use_previous_t) ? (cs_real_3_t *)(f->val_pre)
                                       : (cs_real_3_t *)(f->val);
@@ -635,6 +759,8 @@ cs_field_gradient_vector(const cs_field_t          *f,
                      (const cs_real_3_t *)(f->bc_coeffs->a),
                      (const cs_real_33_t *)(f->bc_coeffs->b),
                      var,
+                     c_weight,
+                     cpl,
                      grad);
 }
 
@@ -730,6 +856,104 @@ cs_field_interpolate(cs_field_t              *f,
   }
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Find local extrema of a given scalar field at each cell
+ *
+ * This assumes the field values have been synchronized.
+ *
+ * \param[in]      field id    The scalar field id
+ * \param[in]      halo_type   Halo type
+ * \param[in,out]  local_max   The local maximum value
+ * \param[in,out]  local_min   The local minimum value
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_field_local_extrema_scalar(int              f_id,
+                              cs_halo_type_t   halo_type,
+                              cs_real_t       *local_max,
+                              cs_real_t       *local_min)
+{
+  const cs_mesh_t *m = cs_glob_mesh;
+  const cs_lnum_t n_cells_ext = m->n_cells_with_ghosts;
+
+  cs_field_t *f = cs_field_by_id(f_id);
+  const cs_real_t *restrict pvar = f->val;
+
+  _local_extrema_scalar(pvar,
+                        halo_type,
+                        local_max,
+                        local_min);
+
+  /* Initialisation of local extrema */
+
+  int key_scamax_id = cs_field_key_id("max_scalar");
+  int key_scamin_id = cs_field_key_id("min_scalar");
+
+  cs_real_t scalar_max = cs_field_get_key_double(f, key_scamax_id);
+  cs_real_t scalar_min = cs_field_get_key_double(f, key_scamin_id);
+
+  /* Bounded by the global extrema */
+# pragma omp parallel for
+  for (cs_lnum_t ii = 0; ii < n_cells_ext; ii++) {
+    local_max[ii] = CS_MIN(local_max[ii], scalar_max);
+    local_min[ii] = CS_MAX(local_min[ii], scalar_min);
+  }
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Synchronize current parallel and periodic field values.
+ *
+ * This function currently only upates fields based on CS_MESH_LOCATION_CELLS.
+ *
+ * \param[in, out]   f           pointer to field
+ * \param[in]        halo_type   halo type
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_field_synchronize(cs_field_t      *f,
+                     cs_halo_type_t   halo_type)
+{
+  if (f->location_id == CS_MESH_LOCATION_CELLS) {
+
+    const cs_halo_t *halo = cs_glob_mesh->halo;
+
+    if (halo != NULL) {
+
+      if (f->dim == 1)
+        cs_halo_sync_var(halo, halo_type, f->val);
+
+      else {
+
+        cs_halo_sync_var_strided(halo, halo_type, f->val, f->dim);
+
+        if (cs_glob_mesh->n_init_perio > 0) {
+          switch(f->dim) {
+          case 9:
+            cs_halo_perio_sync_var_tens(halo, halo_type, f->val);
+            break;
+          case 6:
+            cs_halo_perio_sync_var_sym_tens(halo, halo_type, f->val);
+            break;
+          case 3:
+            cs_halo_perio_sync_var_vect(halo, halo_type, f->val, 3);
+            break;
+          default:
+            break;
+          }
+
+        }
+
+      }
+
+    }
+
+  }
+}
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
