@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -42,17 +42,24 @@
  *----------------------------------------------------------------------------*/
 
 #include "cs_base.h"
+#include "cs_boundary_zone.h"
+#include "cs_ctwr.h"
 #include "cs_fan.h"
 #include "cs_field.h"
 #include "cs_log.h"
+#include "cs_parameters.h"
 #include "cs_physical_constants.h"
 #include "cs_sles.h"
 #include "cs_sles_default.h"
+#include "cs_stokes_model.h"
+#include "cs_thermal_model.h"
 #include "cs_time_moment.h"
 #include "cs_turbomachinery.h"
 #include "cs_rad_transfer_options.h"
 #include "cs_rotation.h"
+#include "cs_turbulence_model.h"
 #include "cs_lagr_log.h"
+#include "cs_volume_zone.h"
 
 /*----------------------------------------------------------------------------
  * Header for the current file
@@ -100,24 +107,54 @@ _log_global_model_options(void)
                   "Physical model options\n"
                   "----------------------\n"));
 
-  /* Rotation info */
+  /* Physical properties */
 
+  cs_physical_constants_log_setup();
+
+  cs_fluid_properties_log_setup();
+
+  /* TODO : Add diftl0 printing */
+
+  /* Thermal model */
+
+  cs_thermal_model_log_setup();
+
+  /* Turbulence */
+  cs_turb_model_log_setup();
+
+  cs_turb_constants_log_setup();
+
+  cs_time_step_log_setup();
+
+  /* Stokes model*/
+  cs_stokes_model_log_setup();
+
+  /* TODO : Partie iroext etc... */
+
+  /* Face viscosity */
+  cs_space_disc_log_setup();
+
+  /* Rotation info */
   if (cs_turbomachinery_get_model() == CS_TURBOMACHINERY_NONE) {
     const cs_rotation_t  *r = cs_glob_rotation;
 
-    cs_log_printf(CS_LOG_SETUP, "\n");
+    cs_log_printf(CS_LOG_SETUP, _("\nSubdomain rotation\n"
+                                  "------------------\n\n"));
 
     cs_log_printf(CS_LOG_SETUP,
-                  _("\n"
-                    "  Global domain rotation:\n"
-                    "    axis:             [%g, %g, %g]"
-                    "    invariant point:  [%g, %g, %g]"
+                  _("  Global domain rotation:\n"
+                    "    axis:             [%g, %g, %g]\n"
+                    "    invariant point:  [%g, %g, %g]\n"
                     "    angular velocity:  %g radians/s\n"),
                   r->axis[0], r->axis[1], r->axis[2],
                   r->invariant[0], r->invariant[1], r->invariant[2],
                   r->omega);
 
   }
+
+  /* Zone information */
+  cs_volume_zone_log_setup();
+  cs_boundary_zone_log_setup();
 }
 
 
@@ -156,6 +193,8 @@ cs_log_setup(void)
 
   cs_fan_log_setup();
 
+  cs_ctwr_log_setup();
+
   cs_log_printf_flush(CS_LOG_SETUP);
 }
 
