@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -107,19 +107,40 @@ BEGIN_C_DECLS
         the file \ref cs_cf_thermo.h, which is not a user source, to add new
         equations of state.
   \var  cs_fluid_properties_t::icp
-        property index of the isobaric specific heat
-        - 0: uniform isobaric specific heat (no property field defined)
+        indicates if the isobaric specific heat \f$C_p\f$ is variable
+        - -1: uniform, no property field is declared
+        -  0: variable, \f$C_p\f$ is declared as a property field\n
+        When gas or coal combustion is activated, \ref icp is automatically
+        set to 0 (constant \f$C_p\f$). With the electric module, it is
+        automatically set to 1. The user is not allowed to modify these
+        default choices.\n When \ref icp = 1 is specified, the code
+        automatically modifies this value to make \ref icp designate the
+        effective index-number of the property "specific heat". For each cell
+        iel, the value of \f$C_p\f$ is then specified by the user in the
+        appropriate subroutine (\ref cs_user_physical_properties for the
+        standard physics).\n Useful if there is
+        1\f$\leqslant\f$N\f$\leqslant\f$\ref dimens::nscal "nscal" so that
+        iscsth(n)=1 (there is a scalar temperature) or with the compressible
+        module for non perfect gases.
   \var  cs_fluid_properties_t::icv
         property index of the isochoric specific heat
-        - 0: uniform isochoric specific heat (no property field defined)
+        - -1: uniform isochoric specific heat (no property field defined)
   \var  cs_fluid_properties_t::irovar
         variable density field \f$ \rho \f$:
-        - 0: false
-        - 1: true
+        - 1: true, its variation law be given either
+        in the GUI, or in the user subroutine
+        \ref cs_user_physical_properties .\n
+        See \subpage physical_properties for more informations.
+        - 0: false, its value is the reference density
+        \ref ro0.
   \var  cs_fluid_properties_t::ivivar
-        variable viscosity field \f$ \rho \f$:
-        - 0: false
-        - 1: true
+        variable viscosity field \f$ \mu \f$:
+           - 1: true, its variation law be given either
+        in the GUI, or in the user subroutine
+        \ref cs_user_physical_properties .\n
+        See \subpage physical_properties for more informations.
+           - 0: false, its value is the reference molecular
+        dynamic viscosity \ref viscl0
   \var  cs_fluid_properties_t::ivsuth
         Sutherland law for laminar viscosity and thermal conductivity
         Only useful in gas mix (igmix) specific physics
@@ -241,7 +262,7 @@ BEGIN_C_DECLS
         stiffened gas (\ref cs_fluid_properties_t::ieos "ieos"=2) limit pressure
         (zero in perfect gas)
   \var  cs_fluid_properties_t::gammasg
-        stiffened gas (\ref ppincl::ieos "ieos"=2) polytropic coefficient
+        stiffened gas (\ref cs_fluid_properties_t::ieos "ieos"=2) polytropic coefficient
         (dimensionless)
   \var  cs_fluid_properties_t::ipthrm
         uniform variable thermodynamic pressure:
@@ -279,9 +300,9 @@ BEGIN_C_DECLS
 /* main physical constants structure and associated pointer */
 
 static cs_physical_constants_t _physical_constants = {
-  .gx = 0.,
-  .gy = 0.,
-  .gz = 0.,
+  .gravity[0] = 0.,
+  .gravity[1] = 0.,
+  .gravity[2] = 0.,
   .icorio = 0};
 
 /* main fluid properties structure and associated pointer */
@@ -289,8 +310,8 @@ static cs_physical_constants_t _physical_constants = {
 static cs_fluid_properties_t  _fluid_properties = {
   .ixyzp0   = -1,
   .ieos     = -1,
-  .icp      = 0,
-  .icv      = 0,
+  .icp      = -1,
+  .icv      = -1,
   .irovar   = 0,
   .ivivar   = 0,
   .ivsuth   = 0,
@@ -298,7 +319,7 @@ static cs_fluid_properties_t  _fluid_properties = {
   .viscl0   = 1.83337e-5,
   .p0       = 1.01325e5,
   .pred0    = 0.,
-  .xyzp0    = {-999., -999.,-999.},
+  .xyzp0    = {-1.e30, -1.e30, -1.e30},
   .t0       = 293.15,
   .cp0      = 1017.24,
   .cv0      = 0.,
@@ -327,7 +348,17 @@ const double cs_physical_constants_r = 8.31446;
 
 const double cs_physical_constants_kb = 1.38e-23;
 
-const cs_physical_constants_t  *cs_glob_physical_constants = &_physical_constants;
+/*! Conversion from Celsius to Kelvin: 275.15   */
+
+const double cs_physical_constants_celsius_to_kelvin = 273.15;
+
+/* !Stephan constant for the radiative module \f$\sigma\f$
+   in \f$W.m^{-2}.K^{-4}\f$ */
+
+const double cs_physical_constants_stephan = 5.6703e-8;
+
+const cs_physical_constants_t  *cs_glob_physical_constants =
+                                  &_physical_constants;
 
 const cs_fluid_properties_t  *cs_glob_fluid_properties = &_fluid_properties;
 
@@ -386,9 +417,9 @@ cs_f_fluid_properties_get_pointers(int     **ixyzp0,
  * enables mapping to Fortran global pointers.
  *
  * parameters:
- *   gx     --> pointer to cs_glob_physical_constants->gx
- *   gy     --> pointer to cs_glob_physical_constants->gy
- *   gz     --> pointer to cs_glob_physical_constants->gz
+ *   gx     --> pointer to cs_glob_physical_constants->gravity[0]
+ *   gy     --> pointer to cs_glob_physical_constants->gravity[1]
+ *   gz     --> pointer to cs_glob_physical_constants->gravity[2]
  *   icorio --> pointer to cs_glob_physical_constants->icorio
  *----------------------------------------------------------------------------*/
 
@@ -398,9 +429,9 @@ cs_f_physical_constants_get_pointers(double  **gx,
                                      double  **gz,
                                      int     **icorio)
 {
-  *gx     = &(_physical_constants.gx);
-  *gy     = &(_physical_constants.gy);
-  *gz     = &(_physical_constants.gz);
+  *gx     = &(_physical_constants.gravity[0]);
+  *gy     = &(_physical_constants.gravity[1]);
+  *gz     = &(_physical_constants.gravity[2]);
   *icorio = &(_physical_constants.icorio);
 }
 
@@ -524,6 +555,71 @@ cs_get_glob_fluid_properties(void)
   return &_fluid_properties;
 }
 
+/*----------------------------------------------------------------------------
+ *!
+ * \brief Print the physical constants structure to setup.log.
+ *
+ *----------------------------------------------------------------------------*/
+
+void
+cs_physical_constants_log_setup(void)
+{
+
+  cs_log_printf
+    (CS_LOG_SETUP,
+     _("\n"
+       "Physical properties\n"
+       "-------------------\n\n"));
+
+  cs_log_printf
+    (CS_LOG_SETUP,
+     _("    gravity:    [%14.5e, (Gravity x component)\n"
+       "                 %14.5e, (Gravity y component)\n"
+       "                 %14.5e] (Gravity z component)\n"
+       "    icorio:      %14d (Coriolis source terms)\n\n"),
+       cs_glob_physical_constants->gravity[0],
+       cs_glob_physical_constants->gravity[1],
+       cs_glob_physical_constants->gravity[2],
+       cs_glob_physical_constants->icorio);
+}
+
+/*----------------------------------------------------------------------------
+ *!
+ * \brief Print the fluid properties structure to setup.log.
+ *
+ *----------------------------------------------------------------------------*/
+
+void
+cs_fluid_properties_log_setup(void)
+{
+  cs_log_printf
+    (CS_LOG_SETUP,
+     _("   Continuous phase:\n"
+       "    ro0:         %14.5e (Reference density)\n"
+       "    viscl0:      %14.5e (Ref. molecular dyn. visc.)\n"
+       "    cp0:         %14.5e (Ref. specific heat)\n"
+       "    icp:         %14d (> 0: Variable cp (cs_user_physical_properties))\n"
+       "    p0:          %14.5e (Ref. total pressure)\n"
+       "    pred0:       %14.5e (Ref. reduced pressure)\n"
+       "    t0:          %14.5e (Ref. temperature)\n\n"
+       "    irovar:      %14d (Density constant(0) or not(1))\n"
+       "    ivivar:      %14d (Molec. visc cst.(0) or not(1))\n\n"
+       "    Initial reference point for pressure\n"
+       "    xyzp0:       %14.5e %14.5e %14.5e\n"),
+       cs_glob_fluid_properties->ro0,
+       cs_glob_fluid_properties->viscl0,
+       cs_glob_fluid_properties->cp0,
+       cs_glob_fluid_properties->icp,
+       cs_glob_fluid_properties->p0,
+       cs_glob_fluid_properties->pred0,
+       cs_glob_fluid_properties->t0,
+       cs_glob_fluid_properties->irovar,
+       cs_glob_fluid_properties->ivivar,
+       cs_glob_fluid_properties->xyzp0[0],
+       cs_glob_fluid_properties->xyzp0[1],
+       cs_glob_fluid_properties->xyzp0[2]);
+}
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
