@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -37,10 +37,6 @@
 #include <stdio.h>
 #include <string.h>
 
-#if defined(HAVE_DLOPEN)
-#include <dlfcn.h>
-#endif
-
 /*----------------------------------------------------------------------------
  *  Local headers
  *----------------------------------------------------------------------------*/
@@ -194,74 +190,6 @@ _thermal_table_create(void)
   return tt;
 }
 
-#if defined(HAVE_DLOPEN) && defined(HAVE_EOS)
-
-/*----------------------------------------------------------------------------
- * Get a shared library function pointer for a writer plugin
- *
- * parameters:
- *   name             <-- name of function symbol in library
- *
- * returns:
- *   pointer to function in shared library
- *----------------------------------------------------------------------------*/
-
-static void *
-_get_eos_dl_function_pointer(const char  *name)
-{
-  void  *retval = NULL;
-  char  *error = NULL;
-
-  assert(_cs_eos_dl_lib != NULL);
-
-  dlerror();    /* Clear any existing error */
-
-  retval = dlsym(_cs_eos_dl_lib, name);
-  error = dlerror();
-
-  if (error != NULL)
-    bft_error(__FILE__, __LINE__, 0,
-              _("Error calling dlsym: %s\n"), error);
-
-  return retval;
-}
-
-#endif
-
-#if defined(HAVE_PLUGINS) && defined(HAVE_COOLPROP)
-
-/*----------------------------------------------------------------------------
- * Get a shared library function pointer for a writer plugin
- *
- * parameters:
- *   name             <-- name of function symbol in library
- *
- * returns:
- *   pointer to function in shared library
- *----------------------------------------------------------------------------*/
-
-static void *
-_get_coolprop_dl_function_pointer(const char  *name)
-{
-  void  *retval = NULL;
-  char  *error = NULL;
-
-  assert(_cs_coolprop_dl_lib != NULL);
-
-  dlerror();    /* Clear any existing error */
-
-  retval = dlsym(_cs_coolprop_dl_lib, name);
-  error = dlerror();
-
-  if (error != NULL)
-    bft_error(__FILE__, __LINE__, 0,
-              _("Error calling dlsym: %s\n"), error);
-
-  return retval;
-}
-
-#endif
-
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
 /*=============================================================================
@@ -308,30 +236,20 @@ cs_thermal_table_set(const char *material,
 #if defined(HAVE_COOLPROP)
 #if defined(HAVE_PLUGINS)
     {
-      char  *lib_path = NULL;
-      const char *pkglibdir = cs_base_get_pkglibdir();
-
       /* Open from shared library */
-      BFT_MALLOC(lib_path,
-                 strlen(pkglibdir) + 1 + 3 + strlen("cs_coolprop") + 3 + 1,
-                 char);
-      sprintf(lib_path, "%s%c%s.so", pkglibdir, DIR_SEPARATOR, "cs_coolprop");
-      _cs_coolprop_dl_lib = dlopen(lib_path, RTLD_LAZY);
-      BFT_FREE(lib_path);
+      _cs_coolprop_dl_lib = cs_base_dlopen_plugin("cs_coolprop");
 
       /* Load symbols from shared library */
 
-      if (_cs_coolprop_dl_lib == NULL)
-        bft_error(__FILE__, __LINE__, 0,
-                  _("Error loading %s: %s."), lib_path, dlerror());
-
       /* Function pointers need to be double-casted so as to first convert
          a (void *) type to a memory address and then convert it back to the
          original type. Otherwise, the compiler may issue a warning.
          This is a valid ISO C construction. */
 
       _cs_phys_prop_coolprop = (cs_phys_prop_coolprop_t *)  (intptr_t)
-        _get_coolprop_dl_function_pointer("cs_phys_prop_coolprop");
+        cs_base_get_dl_function_pointer(_cs_coolprop_dl_lib,
+                                        "cs_phys_prop_coolprop",
+                                        true);
     }
 #else
     _cs_phys_prop_coolprop = (cs_phys_prop_coolprop_t *)  (intptr_t)
@@ -346,22 +264,8 @@ cs_thermal_table_set(const char *material,
     cs_glob_thermal_table->type = 2;
 #if defined(HAVE_EOS)
     {
-      char  *lib_path = NULL;
-      const char *pkglibdir = cs_base_get_pkglibdir();
-
       /* Open from shared library */
-      BFT_MALLOC(lib_path,
-                 strlen(pkglibdir) + 1 + 3 + strlen("cs_eos") + 3 + 1,
-                 char);
-      sprintf(lib_path, "%s%c%s.so", pkglibdir, DIR_SEPARATOR, "cs_eos");
-      _cs_eos_dl_lib = dlopen(lib_path, RTLD_LAZY);
-      BFT_FREE(lib_path);
-
-      /* Load symbols from shared library */
-
-      if (_cs_eos_dl_lib == NULL)
-        bft_error(__FILE__, __LINE__, 0,
-                  _("Error loading %s: %s."), lib_path, dlerror());
+      _cs_eos_dl_lib = cs_base_dlopen_plugin("cs_eos");
 
       /* Function pointers need to be double-casted so as to first convert
          a (void *) type to a memory address and then convert it back to the
@@ -369,13 +273,14 @@ cs_thermal_table_set(const char *material,
          This is a valid ISO C construction. */
 
       _cs_eos_create = (cs_eos_create_t *)  (intptr_t)
-        _get_eos_dl_function_pointer("cs_eos_create");
+        cs_base_get_dl_function_pointer(_cs_eos_dl_lib, "cs_eos_create", true);
       _cs_eos_destroy = (cs_eos_destroy_t *)  (intptr_t)
-        _get_eos_dl_function_pointer("cs_eos_destroy");
+        cs_base_get_dl_function_pointer(_cs_eos_dl_lib, "cs_eos_destroy", true);
       _cs_phys_prop_eos = (cs_phys_prop_eos_t *)  (intptr_t)
-        _get_eos_dl_function_pointer("cs_phys_prop_eos");
+        cs_base_get_dl_function_pointer(_cs_eos_dl_lib, "cs_phys_prop_eos", true);
 
-      _cs_eos_create(cs_glob_thermal_table->method, cs_glob_thermal_table->reference);
+      _cs_eos_create(cs_glob_thermal_table->method,
+                     cs_glob_thermal_table->reference);
     }
 #endif
   }
@@ -396,9 +301,7 @@ cs_thermal_table_finalize(void)
 #if defined(HAVE_EOS)
     if (cs_glob_thermal_table->type == 2) {
       _cs_eos_destroy();
-      if (dlclose(_cs_eos_dl_lib) != 0)
-        bft_error(__FILE__, __LINE__, 0,
-                  _("Error unloading library: %s."), dlerror());
+      cs_base_dlclose("cs_eos", _cs_eos_dl_lib);
       _cs_eos_create = NULL;
       _cs_eos_destroy = NULL;
       _cs_phys_prop_eos = NULL;
@@ -406,9 +309,7 @@ cs_thermal_table_finalize(void)
 #endif
 #if defined(HAVE_COOLPROP) && defined(HAVE_PLUGINS)
     if (cs_glob_thermal_table->type == 3) {
-      if (dlclose(_cs_coolprop_dl_lib) != 0)
-        bft_error(__FILE__, __LINE__, 0,
-                  _("Error unloading library: %s."), dlerror());
+      cs_base_dlclose("cs_coolprop", _cs_coolprop_dl_lib);
       _cs_phys_prop_coolprop = NULL;
     }
 #endif
@@ -422,10 +323,17 @@ cs_thermal_table_finalize(void)
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief Compute a property.
+ * \brief Compute a physical property.
+ *
+ * For values var1 and var2, we can use a stride so that accesses for a given
+ * element with id i will be of the form: var[i*stride]; this allows regular
+ * access with stride 1, and access to constant variables stored as a
+ * single-valued array with a stride of 0.
  *
  * \param[in]   property      property queried
  * \param[in]   n_vals        number of values
+ * \param[in]   var1_stride   stride between successive values of var1
+ * \param[in]   var2_stride   stride between successive values of var2
  * \param[in]   var1          values on first plane axis
  * \param[in]   var2          values on second plane axis
  * \param[out]  val           resulting property values
@@ -433,35 +341,72 @@ cs_thermal_table_finalize(void)
 /*----------------------------------------------------------------------------*/
 
 void
-cs_phys_prop_compute(cs_phys_prop_type_t                property,
-                     const cs_lnum_t                    n_vals,
-                     const cs_real_t                    var1[],
-                     const cs_real_t                    var2[],
-                     cs_real_t                          val[])
+cs_phys_prop_compute(cs_phys_prop_type_t          property,
+                     cs_lnum_t                    n_vals,
+                     cs_lnum_t                    var1_stride,
+                     cs_lnum_t                    var2_stride,
+                     const cs_real_t              var1[],
+                     const cs_real_t              var2[],
+                     cs_real_t                    val[])
 {
-  double *val_compute;
-  BFT_MALLOC(val_compute, n_vals, double);
-  for (int ii = 0; ii < n_vals; ii++) {
-    if (cs_glob_thermal_table->temp_scale == 2)
-      val_compute[ii] = var2[ii] + 273.15;
-    else
-      val_compute[ii] = var2[ii];
+  cs_lnum_t        _n_vals = n_vals;
+  cs_real_t         _var2_c_single[1];
+  cs_real_t        *_var1_c = NULL, *_var2_c = NULL;
+  const cs_real_t  *var1_c = var1, *var2_c = var2;
+
+  if (n_vals < 1)
+    return;
+
+  /* Adapt to different strides to optimize for constant arrays */
+
+  if (var1_stride == 0 && var2_stride == 0)
+    _n_vals = 1;
+
+  if (var1_stride == 0 && n_vals > 1) {
+    BFT_MALLOC(_var1_c, n_vals, cs_real_t);
+    for (cs_lnum_t ii = 0; ii < n_vals; ii++)
+      _var1_c[ii] = var1[0];
+    var1_c = _var1_c;
   }
+
+  if (cs_glob_thermal_table->temp_scale == 2) {
+    if (_n_vals == 1) {
+      _var2_c_single[0] = var2[0] + 273.15;
+      var2_c = _var2_c_single;
+    }
+    else {
+      BFT_MALLOC(_var2_c, n_vals, cs_real_t);
+      for (cs_lnum_t ii = 0; ii < n_vals; ii++)
+        _var2_c[ii] = var2[ii*var2_stride] + 273.15;
+      var2_c = _var2_c;
+    }
+  }
+  else {
+    if (var2_stride == 0 && n_vals > 1) {
+      BFT_MALLOC(_var2_c, n_vals, cs_real_t);
+      for (cs_lnum_t ii = 0; ii < n_vals; ii++)
+        _var2_c[ii] = var2[0];
+      var2_c = _var2_c;
+    }
+  }
+
+  /* Compute proper */
+
   if (cs_glob_thermal_table->type == 1) {
     cs_phys_prop_freesteam(cs_glob_thermal_table->thermo_plane,
                            property,
-                           n_vals,
-                           var1,
-                           val_compute,
+                           _n_vals,
+                           var1_c,
+                           var2_c,
                            val);
   }
 #if defined(HAVE_EOS)
   else if (cs_glob_thermal_table->type == 2) {
     _cs_phys_prop_eos(cs_glob_thermal_table->thermo_plane,
                       property,
-                      n_vals,
-                      var1,
-                      val_compute,
+                      _n_vals,
+                      var1_c,
+                      var2_c,
                       val);
   }
 #endif
@@ -470,13 +415,22 @@ cs_phys_prop_compute(cs_phys_prop_type_t                property,
     _cs_phys_prop_coolprop(cs_glob_thermal_table->material,
                            cs_glob_thermal_table->thermo_plane,
                            property,
-                           n_vals,
-                           var1,
-                           val_compute,
+                           _n_vals,
+                           var1_c,
+                           var2_c,
                            val);
   }
 #endif
-  BFT_FREE(val_compute);
+  BFT_FREE(_var1_c);
+  BFT_FREE(_var2_c);
+
+  /* In case of single value, apply to all */
+
+  if (_n_vals == 1) {
+    cs_real_t val_const = val[0];
+    for (cs_lnum_t ii = 0; ii < n_vals; ii++)
+      val[ii] = val_const;
+  }
 }
 
 /*----------------------------------------------------------------------------*/
