@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -48,8 +48,11 @@
 
 #include "cs_field.h"
 #include "cs_field_pointer.h"
+#include "cs_halo.h"
+#include "cs_halo_perio.h"
 #include "cs_log.h"
 #include "cs_map.h"
+#include "cs_mesh.h"
 #include "cs_parall.h"
 #include "cs_mesh_location.h"
 #include "cs_turbulence_model.h"
@@ -254,6 +257,52 @@ _read_legacy_field_info(cs_restart_t  *r)
 
 }
 
+/*----------------------------------------------------------------------------
+ * Synchronize cell-based field values.
+ *
+ * parameters:
+ *   f    <-> field whose values should be synchronized
+ *   t_id <-- time id (0 for current, 1 for previous, ...)
+ *----------------------------------------------------------------------------*/
+
+static void
+_sync_field_vals(cs_field_t  *f,
+                 int          t_id)
+{
+  const cs_mesh_t *m = cs_glob_mesh;
+
+  if (m->halo != NULL) {
+
+    cs_halo_type_t  halo_type = CS_HALO_EXTENDED;
+    cs_real_t      *v = f->vals[t_id];
+
+    cs_halo_sync_var_strided(m->halo, halo_type, v, f->dim);
+
+    if (m->n_init_perio > 0) {
+      if (f->dim == 3)
+        cs_halo_perio_sync_var_vect(m->halo, halo_type, v, 3);
+      else if (f->dim == 6)
+        cs_halo_perio_sync_var_sym_tens(m->halo, halo_type, v);
+      else if (f->dim == 9)
+        cs_halo_perio_sync_var_tens(m->halo, halo_type, v);
+      else if (f->dim == 1 && f == CS_F_(r13)) {
+        cs_halo_perio_sync_var_tens_ni(m->halo,
+                                       halo_type,
+                                       CS_F_(r11)->vals[t_id],
+                                       CS_F_(r12)->vals[t_id],
+                                       CS_F_(r13)->vals[t_id],
+                                       CS_F_(r12)->vals[t_id],
+                                       CS_F_(r22)->vals[t_id],
+                                       CS_F_(r23)->vals[t_id],
+                                       CS_F_(r13)->vals[t_id],
+                                       CS_F_(r23)->vals[t_id],
+                                       CS_F_(r33)->vals[t_id]);
+      }
+    }
+
+  }
+}
+
 /*----------------------------------------------------------------------------
  * Read field values from checkpoint.
  *
@@ -264,7 +313,7 @@ _read_legacy_field_info(cs_restart_t  *r)
  *   r            <-- associated restart file pointer
  *   restart_name <-- base name with which read is attempted
  *   t_id         <-- time id (0 for current, 1 for previous, ...)
- *   f            <-> file whose values should be read
+ *   f            <-> field whose values should be read
  *
  * returns:
  *   CS_RESTART_SUCCESS in case of success, CS_RESTART_ERR_... otherwise
@@ -338,6 +387,10 @@ _read_field_vals(cs_restart_t  *r,
   if (sec_name != _sec_name)
     BFT_FREE(sec_name);
 
+  if (   retcode == CS_RESTART_SUCCESS
+      && f->location_id == CS_MESH_LOCATION_CELLS)
+    _sync_field_vals(f, t_id);
+
   return retcode;
 }
 
@@ -473,15 +526,17 @@ _read_field_vals_legacy(cs_restart_t  *r,
     else if (strcmp(f->name, "dt") == 0)
       strncpy(sec_name, "dt_variable_espace_ce", 127);
 
+    else if (strcmp(f->name, "volume_forces") == 0)
+      strncpy(sec_name, "force_ext_ce_phase01", 127);
+
+    else if (strcmp(f->name, "hydrostatic_pressure_prd") == 0)
+      strncpy(sec_name, "Prhyd_pre_phase01", 127);
+
     else if (strcmp(f->name, "void_fraction") == 0)
       strncpy(sec_name, "taux_vide_ce", 127);
 
-    else if (strcmp(f->name, "mesh_viscosity_1") == 0)
-      strncpy(sec_name, "visc_maillage_x", 127);
-    else if (strcmp(f->name, "mesh_viscosity_2") == 0)
-      strncpy(sec_name, "visc_maillage_y", 127);
-    else if (strcmp(f->name, "mesh_viscosity_3") == 0)
-      strncpy(sec_name, "visc_maillage_z", 127);
+    else if (strcmp(f->name, "mesh_viscosity") == 0)
+      strncpy(sec_name, "visc_maillage", 127);
 
     else if (strcmp(f->name, "rad_st") == 0)
       strncpy(sec_name, "rayexp_ce", 127);
@@ -564,7 +619,6 @@ _read_field_vals_legacy(cs_restart_t  *r,
       snprintf(old_name_yz, 127, "r23::vals::0");
       snprintf(old_name_xz, 127, "r13::vals::0");
 
-
       retcode = cs_restart_check_section(r,
                                          old_name_xx,
                                          f->location_id,
@@ -584,6 +638,11 @@ _read_field_vals_legacy(cs_restart_t  *r,
                                                   (cs_real_6_t *)(f->vals[t_id]));
     }
   }
+
+  if (   retcode == CS_RESTART_SUCCESS
+      && f->location_id == CS_MESH_LOCATION_CELLS)
+    _sync_field_vals(f, t_id);
+
   return retcode;
 }
 
@@ -690,7 +749,7 @@ _read_linked_fields_legacy(cs_restart_t  *r,
 
     const cs_field_t *f = cs_field_by_id(f_id);
 
-    if (key_flag != 0 && !(f->type & key_flag))
+    if (key_flag == -1 || !(f->type & key_flag))
       continue;
 
     const int lnk_f_id = cs_field_get_key_int(f, key_id);
@@ -831,7 +890,7 @@ _check_field_model(cs_restart_t               *r,
 
       const cs_field_t *f = cs_field_by_id(f_id);
 
-      if (key_flag != 0 && !(f->type & key_flag))
+      if (key_flag == -1 || !(f->type & key_flag))
         continue;
 
       const char *f_name = NULL;
@@ -1075,12 +1134,8 @@ _read_turbulence_model(cs_restart_t  *r)
                                              1,
                                              &iturb_old);
 
-  if (n_opt_vals != 1) {
-    bft_error
-      (__FILE__, __LINE__, 0,
-       _("Error reading turbulence model info from restart file \"%s\"."),
-       cs_restart_get_name(r));
-  }
+  if (n_opt_vals != 1)
+    iturb_old = -999;
 
   return iturb_old;
 }
@@ -1687,6 +1742,16 @@ cs_restart_read_variables(cs_restart_t               *r,
                                 "turbulent_flux_id",
                                 _read_flag);
 
+  cs_restart_read_linked_fields(r,
+                                old_field_map,
+                                "gwf_sorbed_concentration_id",
+                                _read_flag);
+
+  cs_restart_read_linked_fields(r,
+                                old_field_map,
+                                "gwf_precip_concentration_id",
+                                _read_flag);
+
   /* Read and convert turbulence variables in case of model change */
 
   const int iturb_cur = cs_glob_turb_model->iturb;
@@ -1699,11 +1764,27 @@ cs_restart_read_variables(cs_restart_t               *r,
 
   if (t_id_flag < 1) {
 
-    if (!(_read_flag[CS_F_(u)->id] & 1) || !(_read_flag[CS_F_(p)->id] & 1))
-      bft_error
-        (__FILE__, __LINE__, 0,
-         _("Error reading velocity/pressure values in restart file \"%s\"."),
-         cs_restart_get_name(r));
+    if (cs_glob_field_pointers != NULL) {
+
+      /* standard calculation */
+      if (CS_F_(p)) {
+        if (!(_read_flag[CS_F_(u)->id] & 1) || !(_read_flag[CS_F_(p)->id] & 1))
+          bft_error
+            (__FILE__, __LINE__, 0,
+             _("Error reading velocity/pressure values in restart file \"%s\"."),
+             cs_restart_get_name(r));
+        /* ground water flow calculation */
+      } else if (CS_F_(head)) {
+        if (   !(_read_flag[CS_F_(u)->id] & 1)
+            || !(_read_flag[CS_F_(head)->id] & 1))
+          bft_error
+            (__FILE__, __LINE__, 0,
+             _("Error reading velocity/hydraulic head values in restart "
+               "file \"%s\"."),
+             cs_restart_get_name(r));
+      }
+
+    }
 
     int n_missing = 0;
 
@@ -1775,7 +1856,7 @@ cs_restart_write_variables(cs_restart_t  *r,
   /* Write metadata for turbulent flux models
      (turbulent flux model useful only for warnings, as the turbulent flux
      field id is sufficient  to determine data to be written;
-     last test on n_turbt is a minor optimization to possible avoid a call
+     last test on n_turbt is a minor optimization to possibly avoid a call
      to cs_restart_write_linked_fields, but that call would be no
      more costly than the loop to determine the turbulent flux model
      in the first case...) */
@@ -1857,6 +1938,14 @@ cs_restart_write_variables(cs_restart_t  *r,
                                    "turbulent_flux_id",
                                    _write_flag);
 
+  cs_restart_write_linked_fields(r,
+                                 "gwf_sorbed_concentration_id",
+                                 _write_flag);
+
+  cs_restart_write_linked_fields(r,
+                                 "gwf_precip_concentration_id",
+                                 _write_flag);
+
   /* Cleanup */
 
   if (_write_flag != write_flag)
@@ -1898,7 +1987,7 @@ cs_restart_read_linked_fields(cs_restart_t               *r,
   const int n_fields = cs_field_n_fields();
   const int n_o_fields = cs_map_name_to_id_size(old_field_map);
 
-  const int key_id = cs_field_key_id(key);
+  const int key_id = cs_field_key_id_try(key);
   const int key_flag = cs_field_key_flag(key_id);
   const int kr = cs_field_key_id_try("restart_name");
 
@@ -1906,8 +1995,10 @@ cs_restart_read_linked_fields(cs_restart_t               *r,
 
   for (int f_id = 0; f_id < n_fields; f_id++) {
     const cs_field_t *f = cs_field_by_id(f_id);
-    if (key_flag != 0 && !(f->type & key_flag))
-      continue;
+    if (key_flag != 0) {
+      if (key_flag == -1 || !(f->type & key_flag))
+        continue;
+    }
     if (cs_field_get_key_int(f, key_id) > -1)
       n_required += 1;
   }
@@ -1964,8 +2055,10 @@ cs_restart_read_linked_fields(cs_restart_t               *r,
 
       const cs_field_t *f = cs_field_by_id(f_id);
 
-      if (key_flag != 0 && !(f->type & key_flag))
-        continue;
+      if (key_flag != 0) {
+        if (key_flag == -1 || !(f->type & key_flag))
+          continue;
+      }
 
       const int lnk_f_id = cs_field_get_key_int(f, key_id);
 
@@ -1996,6 +2089,8 @@ cs_restart_read_linked_fields(cs_restart_t               *r,
             if (old_lnk_f_id > -1)
               f_lnk_name
                 = cs_map_name_to_id_reverse(old_field_map, old_lnk_f_id);
+            else
+              f_lnk_name = f_lnk->name;
           }
         }
 
@@ -2062,7 +2157,7 @@ cs_restart_write_linked_fields(cs_restart_t  *r,
 
   const int n_fields = cs_field_n_fields();
 
-  const int key_id = cs_field_key_id(key);
+  const int key_id = cs_field_key_id_try(key);
   const int key_flag = cs_field_key_flag(key_id);
 
   int *_write_flag = write_flag;
@@ -2086,8 +2181,10 @@ cs_restart_write_linked_fields(cs_restart_t  *r,
   for (int f_id = 0; f_id < n_fields; f_id++) {
     key_val[f_id] = -1;
     const cs_field_t *f = cs_field_by_id(f_id);
-    if (key_flag != 0 && !(f->type & key_flag))
-      continue;
+    if (key_flag != 0) {
+      if (key_flag == -1 || !(f->type & key_flag))
+        continue;
+    }
     key_val[f_id] = cs_field_get_key_int(f, key_id);
   }
 
@@ -2248,7 +2345,7 @@ cs_restart_read_bc_coeffs(cs_restart_t  *r)
                                                       old_name_yz,
                                                       old_name_xz,
                                                       f->location_id,
-                                                      (cs_real_6_t *)(f->vals));
+                                                      (cs_real_6_t *)(f->val));
           }
           else {
              retval =  cs_restart_read_real_66_t_compat(r,
@@ -2260,7 +2357,7 @@ cs_restart_read_bc_coeffs(cs_restart_t  *r)
                                                         old_name_yz,
                                                         old_name_xz,
                                                         f->location_id,
-                                                        (cs_real_66_t *)(f->vals));
+                                                        (cs_real_66_t *)(f->val));
           }
         }
         else {
@@ -2579,6 +2676,56 @@ cs_restart_read_time_step_info(cs_restart_t  *r)
     cs_time_step_define_prev(_n_ts, _ts);
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Loop over all fields and save them in the restart file which id is
+ *        passed in argument if it matches their "restart_file" key value.
+ *
+ * \param[in, out]  r        associated restart file pointer
+ * \param[in]       r_id     value of the key "restart_file"
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_restart_write_fields(cs_restart_t        *r,
+                        cs_restart_file_t    r_id)
+{
+  int n_fields = cs_field_n_fields();
+  const int restart_file_key_id = cs_field_key_id("restart_file");
+
+  for (int f_id = 0; f_id < n_fields; f_id++) {
+    cs_field_t *f = cs_field_by_id(f_id);
+    if (cs_field_get_key_int(f, restart_file_key_id) == r_id) {
+      cs_restart_write_field_vals(r, f_id, 0);
+    }
+  }
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Loop over all fields and read them in the restart file which id is
+ *        passed in argument if it matches their "restart_file" key value.
+ *
+ * \param[in, out]  r        associated restart file pointer
+ * \param[in]       r_id     value of the key "restart_file"
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_restart_read_fields(cs_restart_t       *r,
+                       cs_restart_file_t   r_id)
+{
+  int n_fields = cs_field_n_fields();
+  const int restart_file_key_id = cs_field_key_id("restart_file");
+
+  for (int f_id = 0; f_id < n_fields; f_id++) {
+    cs_field_t *f = cs_field_by_id(f_id);
+    if (cs_field_get_key_int(f, restart_file_key_id) == r_id) {
+      cs_restart_read_field_vals(r, f_id, 0);
+    }
+  }
+}
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
