@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -38,36 +38,9 @@
  *----------------------------------------------------------------------------*/
 
 #include "bft_mem.h"
-#include "bft_printf.h"
-
-#include "fvm_selector.h"
-
-#include "cs_interface.h"
 
 #include "cs_base.h"
-#include "cs_benchmark.h"
-#include "cs_gradient.h"
-#include "cs_gui.h"
-#include "cs_gui_mesh.h"
-#include "cs_gui_output.h"
-#include "cs_gradient.h"
-#include "cs_gradient_perio.h"
-#include "cs_join.h"
-#include "cs_halo.h"
-#include "cs_halo_perio.h"
-#include "cs_matrix_default.h"
 #include "cs_math.h"
-#include "cs_mesh.h"
-#include "cs_mesh_coherency.h"
-#include "cs_mesh_location.h"
-#include "cs_mesh_quantities.h"
-#include "cs_mesh_to_builder.h"
-#include "cs_multigrid.h"
-#include "cs_parall.h"
-#include "cs_post.h"
-#include "cs_preprocess.h"
-#include "cs_prototypes.h"
-#include "cs_renumber.h"
 #include "cs_time_step.h"
 #include "cs_timer.h"
 
@@ -410,11 +383,6 @@ cs_rotation_define(double  omega_x,
   for (int i = 0; i < 3; i++) {
     r->axis[i] /= r->omega;
   }
-
-  /* As rotation is global, define it both on rotation "0" (default)
-     and "1" (rotor) so as to allow both access modes */
-
-  memcpy(_glob_rotation_0 + 1, _glob_rotation_0, sizeof(cs_rotation_t));
 }
 
 /*----------------------------------------------------------------------------*/
@@ -537,6 +505,59 @@ cs_rotation_update_coords(cs_lnum_t    n_coords,
     _apply_vector_transfo(matrix, coords[i]);
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Express a vector in the cyclindrical system associated to a rotation.
+ *
+ * \param[in]   r   pointer to rotation structure
+ * \param[in]   p   cartesian coordinates of the location point
+ * \param[in]   v   vector components in cartesian coordinates system
+ * \param[out]  vc  vector components in cylindrical coordinates system
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_rotation_cyl_v(const cs_rotation_t  *r,
+                  const cs_real_t       coords[3],
+                  const cs_real_t       v[3],
+                  cs_real_t             vc[3])
+{
+  /* Axial unit vector */
+
+  const cs_real_t *e_ax = r->axis;
+
+  /* Tangential unit vector */
+
+  cs_real_t e_th[3];
+
+  e_th[0] =  e_ax[1] * (coords[2] - r->invariant[2])
+           - e_ax[2] * (coords[1] - r->invariant[1]);
+  e_th[1] =  e_ax[2] * (coords[0] - r->invariant[0])
+           - e_ax[0] * (coords[2] - r->invariant[2]);
+  e_th[2] =  e_ax[0] * (coords[1] - r->invariant[1])
+           - e_ax[1] * (coords[0] - r->invariant[0]);
+
+  cs_real_t xnrm =  sqrt(cs_math_3_square_norm(e_th));
+
+  e_th[0] /= xnrm;
+  e_th[1] /= xnrm;
+  e_th[2] /= xnrm;
+
+  /* Radial unit vector */
+
+  cs_real_t e_r[3];
+
+  e_r[0] = - e_ax[1]*e_th[2] + e_ax[2]*e_th[1];
+  e_r[1] = - e_ax[2]*e_th[0] + e_ax[0]*e_th[2];
+  e_r[2] = - e_ax[0]*e_th[1] + e_ax[1]*e_th[0];
+
+  /* Transformation into cylindrical coordinates */
+
+  vc[0] = v[0]*e_r[0]  + v[1]*e_r[1]  + v[2]*e_r[2];
+  vc[1] = v[0]*e_th[0] + v[1]*e_th[1] + v[2]*e_th[2];
+  vc[2] = v[0]*e_ax[0] + v[1]*e_ax[1] + v[2]*e_ax[2];
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Copy rotation structure values to an array
