@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -51,7 +51,9 @@ BEGIN_C_DECLS
 typedef struct {
 
   int           ivisse;       /* take viscous term of transposed velocity
-                                 gradient into account in momentum equation */
+                                 gradient into account in momentum equation
+                                 - 1: true (default)
+                                 - 0: false */
   int           irevmc;       /* reconstruction of the velocity field with the
                                  updated pressure option
                                  - 0: default */
@@ -59,10 +61,6 @@ typedef struct {
                                  continuity equation
                                  - 1: true (default)
                                  - 0: false */
-  int           irnpnw;       /* compute the normed residual for the pressure
-                                 step in the prediction step
-                                 - 1: true (default)
-                                 - 0: false */
   double        rnormp;       /* normed residual for the pressure step */
   double        arak;         /* Arakawa multiplicator for the Rhie and Chow
                                  filter (1 by default) */
@@ -96,8 +94,8 @@ typedef struct {
                                    treatment IF iphydr=1
                                  - 0: no treatment (default) */
   int           igpust;       /* improve static pressure algorithm
-                                 - 1: take user momemtum source terms in the static pressure
-                                   treatment IF iphydr=1 (default)
+                                 - 1: take user momemtum source terms in the
+                                   static pressure treatment IF iphydr=1 (default)
                                  - 0: no treatment */
   int           iifren;       /* indicates the presence of a Bernoulli boundary
                                  face (automatically computed)
@@ -108,9 +106,10 @@ typedef struct {
                                  pressure at outlets
                                  - 1: true
                                  - 0: false (default) */
-  int           irecmf;       /* use interpolated face diffusion coefficient instead of
-                                 cell diffusion coefficient for the mass flux reconstruction
-                                 for the non-orthogonalities
+  int           irecmf;       /* use interpolated face diffusion coefficient
+                                 instead of cell diffusion coefficient for the
+                                 mass flux reconstruction for the
+                                 non-orthogonalities
                                  - 1: true
                                  - 0: false (default) */
 
@@ -137,6 +136,15 @@ extern const cs_stokes_model_t  *cs_glob_stokes_model;
 cs_stokes_model_t *
 cs_get_glob_stokes_model(void);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Print the stokes model parameters to setup.log.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_stokes_model_log_setup(void);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
