@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -347,6 +347,7 @@ _cs_syr4_coupling_post_function(void                  *coupling,
           cell_temp = coupling_ent->solid_temp;
 
         cs_post_write_var(coupling_ent->post_mesh_id,
+                          CS_POST_WRITER_ALL_ASSOCIATED,
                           _("Solid T"),
                           1,
                           false,
@@ -359,6 +360,7 @@ _cs_syr4_coupling_post_function(void                  *coupling,
 
         if (type_id == 1)
           cs_post_write_var(coupling_ent->post_mesh_id,
+                            CS_POST_WRITER_ALL_ASSOCIATED,
                             _("Solid heat flux"),
                             1,
                             false,
@@ -764,7 +766,7 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
                      (unsigned long long) n_ext);
         if (ext_syr)
           bft_printf(_(" Some solid mesh elements not located on fluid mesh\n"));
-        bft_printf(_("   Extending search with tolerance factor %f..."),
+        bft_printf(_("\n   Extending search with tolerance factor %f..."),
                    tolerance);
         bft_printf_flush();
       }
@@ -808,6 +810,7 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
     cs_post_write_meshes(cs_glob_time_step);
 
     cs_post_write_var(coupling_ent->post_mesh_id,
+                      CS_POST_WRITER_ALL_ASSOCIATED,
                       _("distance_to_solid"),
                       1,
                       false,
@@ -881,6 +884,7 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
       cs_post_write_meshes(cs_glob_time_step);
 
       cs_post_write_vertex_var(mesh_id,
+                               CS_POST_WRITER_ALL_ASSOCIATED,
                                _("distance_to_fluid"),
                                1,
                                false,
@@ -1228,6 +1232,59 @@ _ensure_conservativity(cs_syr4_coupling_t   *syr_coupling,
                  " coupling with SYRTHES: %5.3e\n"), coef);
 }
 
+/*----------------------------------------------------------------------------
+ * Exchange location synchronization status
+ *
+ * parameters:
+ *   syr_coupling <-- SYRTHES coupling structure
+ *
+ * returns:
+ *   0 in case of success, 1 otherwise
+ *----------------------------------------------------------------------------*/
+
+int
+_sync_after_location(cs_syr4_coupling_t  *syr_coupling)
+{
+  int retval = 1;
+
+  char op_name_send[32 + 1];
+  char op_name_recv[32 + 1];
+
+  const cs_lnum_t verbosity = syr_coupling->verbosity;
+
+  /* Communication with SYRTHES */
+  /*----------------------------*/
+
+  /* Ready to start time iterations */
+
+  strcpy(op_name_send, "coupling:start");
+
+  _exchange_sync(syr_coupling, op_name_send, op_name_recv);
+
+  if (!strcmp(op_name_recv, "coupling:error:location")) {
+
+    cs_coupling_set_sync_flag(PLE_COUPLING_STOP);
+
+    cs_base_warn(__FILE__, __LINE__);
+
+    bft_printf(_(" Message received from SYRTHES: \"%s\"\n"
+                 " indicates meshes have not been matched correctly.\n\n"
+                 " The calculation will not run.\n\n"),
+               op_name_recv);
+
+  }
+  else if (strcmp(op_name_recv, "coupling:start"))
+    bft_error(__FILE__, __LINE__, 0,
+              _(" Message received from SYRTHES: \"%s\"\n"
+                " indicates an error or is unexpected."),
+              op_name_recv);
+
+  else
+    retval = 0;
+
+  return retval;
+}
+
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
 /*============================================================================
@@ -1522,50 +1579,30 @@ cs_syr4_coupling_init_mesh(cs_syr4_coupling_t  *syr_coupling)
 
   assert(syr_coupling->dim == 3 || syr_coupling->dim == 2);
 
-  if (syr_coupling->face_sel != NULL)
+  int match_flag = 0;
+
+  if (syr_coupling->face_sel != NULL) {
     syr_coupling->faces = _create_coupled_ent(syr_coupling,
                                               syr_coupling->face_sel,
                                               syr_coupling->dim - 1);
+    match_flag += _sync_after_location(syr_coupling);
+  }
 
   if (syr_coupling->cell_sel != NULL) {
     syr_coupling->cells = _create_coupled_ent(syr_coupling,
                                               syr_coupling->cell_sel,
                                               syr_coupling->dim);
+    match_flag += _sync_after_location(syr_coupling);
   }
 
   /* Communication with SYRTHES */
   /*----------------------------*/
 
-  /* Ready to start time iterations */
-
-  strcpy(op_name_send, "coupling:start");
-
-  _exchange_sync(syr_coupling, op_name_send, op_name_recv);
-
-  if (!strcmp(op_name_recv, "coupling:error:location")) {
-
-    cs_coupling_set_sync_flag(PLE_COUPLING_STOP);
-
-    cs_base_warn(__FILE__, __LINE__);
-
-    bft_printf(_(" Message received from SYRTHES: \"%s\"\n"
-                 " indicates meshes have not been matched correctly.\n\n"
-                 " The calculation will not run.\n\n"),
-               op_name_recv);
-
-  }
-  else if (strcmp(op_name_recv, "coupling:start"))
-    bft_error(__FILE__, __LINE__, 0,
-              _(" Message received from SYRTHES: \"%s\"\n"
-                " indicates an error or is unexpected."),
-              op_name_recv);
-
-  else if (verbosity > 0) {
+  if (match_flag == 0 && verbosity > 0) {
     bft_printf(_("\n ** Mesh located for SYRTHES coupling \"%s\".\n\n"),
                syr_coupling->syr_name);
     bft_printf_flush();
   }
-
 }
 
 /*----------------------------------------------------------------------------
