@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -45,7 +45,8 @@
 #include "cs_interface.h"
 
 #include "cs_base.h"
-#include "cs_benchmark.h"
+#include "cs_boundary_zone.h"
+#include "cs_coupling.h"
 #include "cs_gradient.h"
 #include "cs_gui.h"
 #include "cs_gui_mesh.h"
@@ -57,6 +58,7 @@
 #include "cs_halo_perio.h"
 #include "cs_matrix_default.h"
 #include "cs_mesh.h"
+#include "cs_mesh_adjacencies.h"
 #include "cs_mesh_coherency.h"
 #include "cs_mesh_location.h"
 #include "cs_mesh_quantities.h"
@@ -72,7 +74,9 @@
 #include "cs_timer.h"
 #include "cs_timer_stats.h"
 #include "cs_restart.h"
+#include "cs_sat_coupling.h"
 #include "cs_preprocessor_data.h"
+#include "cs_volume_zone.h"
 
 /*----------------------------------------------------------------------------
  * Header for the current file
@@ -94,18 +98,26 @@ BEGIN_C_DECLS
 
 typedef struct {
 
-  cs_turbomachinery_model_t  model;             /* Turbomachinery model type */
+  cs_turbomachinery_model_t  model;             /* turbomachinery model type */
 
-  int                        n_rotors;          /* Number of rotors */
+  int                        n_rotors;          /* number of rotors */
+
+  int                        n_couplings;       /* number of couplings */
 
   cs_rotation_t             *rotation;          /* rotation structures */
-  char                     **rotor_cells_c;     /* Rotor cells selection
-                                                   criteria ((for each rotor) */
+  char                     **rotor_cells_c;     /* rotor cells selection
+                                                   criteria (for each rotor) */
 
-  cs_mesh_t                 *reference_mesh;    /* Reference mesh (before
+  int                        n_max_join_tries;  /* maximum number of tries
+                                                   for joining differences */
+  double                     dt_retry;          /* time shift multiplier for
+                                                   retry position */
+  double                     t_cur;             /* current time for update */
+
+  cs_mesh_t                 *reference_mesh;    /* reference mesh (before
                                                    rotation and joining) */
 
-  cs_lnum_t                  n_b_faces_ref;     /* Reference number of
+  cs_lnum_t                  n_b_faces_ref;     /* reference number of
                                                    boundary faces */
 
   int                       *cell_rotor_num;    /* cell rotation axis number */
@@ -118,15 +130,17 @@ typedef struct {
  * Static global variables
  *============================================================================*/
 
-cs_turbomachinery_t  *cs_glob_turbomachinery = NULL;
+cs_turbomachinery_t  *_turbomachinery = NULL;
 
 /*============================================================================
  * Prototypes for functions intended for use only by Fortran wrappers.
  * (descriptions follow, with function bodies).
  *============================================================================*/
 
-void cs_f_map_turbomachinery_module(cs_int_t    *iturbo,
-                                    int        **irotce);
+void cs_f_map_turbomachinery_model(cs_int_t    *iturbo,
+                                   cs_int_t    *ityint);
+
+void cs_f_map_turbomachinery_rotor(int       **irotce);
 
 /*============================================================================
  * Private function definitions
@@ -162,6 +176,79 @@ _post_error_faces_select(void         *input,
   *face_ids = _face_ids;
 }
 
+/*----------------------------------------------------------------------------
+ * Function for tagging of coupling mesh.
+ *
+ * We use the rotor number as a tag
+ *
+ * Note: if the context pointer is non-NULL, it must point to valid data
+ * when the selection function is called, so that value or structure
+ * should not be temporary (i.e. local);
+ *
+ * parameters:
+ *   context         <-> pointer to optional (untyped) value or structure.
+ *   mesh            <-> nodal mesh which should be tagged
+ *   n_points        <-- number of points to tag
+ *   point_list_base <-- base numbering for point_list
+ *   point_list      <-- optional indirection for points
+ *   point_tag       --> point tag values (size: n_tags)
+ *----------------------------------------------------------------------------*/
+
+static void
+_turbomachinery_coupling_tag(void            *context,
+                             fvm_nodal_t     *mesh,
+                             cs_lnum_t        n_points,
+                             cs_lnum_t        point_list_base,
+                             const cs_lnum_t  point_list[],
+                             int             *point_tag)
+{
+  const cs_turbomachinery_t *tbm = context;
+  const cs_mesh_t *m = cs_glob_mesh;
+
+  /* Tag elements (boundary faces) */
+
+  if (mesh != NULL) {
+
+    cs_lnum_t n_elts = fvm_nodal_get_n_entities(mesh, 3);
+
+    const int ent_dim = 3;
+    int *elt_tag;
+    cs_lnum_t *parent_num;
+
+    BFT_MALLOC(elt_tag, n_elts, int);
+    BFT_MALLOC(parent_num, n_elts, cs_lnum_t);
+
+    fvm_nodal_get_parent_num(mesh, ent_dim, parent_num);
+    for (cs_lnum_t i = 0; i < n_elts; i++) {
+      cs_lnum_t c_id = parent_num[i] - 1;
+      elt_tag[i] = tbm->cell_rotor_num[c_id];
+    }
+
+    BFT_FREE(parent_num);
+
+    fvm_nodal_set_tag(mesh, elt_tag, ent_dim);
+
+    BFT_FREE(elt_tag);
+
+  }
+
+  /* Tag vertices */
+
+  if (point_list != NULL) {
+    for (cs_lnum_t i = 0; i < n_points; i++) {
+      cs_lnum_t f_id = point_list[i] - point_list_base;
+      cs_lnum_t c_id = m->b_face_cells[f_id];
+      point_tag[i] = tbm->cell_rotor_num[c_id];
+    }
+  }
+  else {
+    for (cs_lnum_t i = 0; i < n_points; i++) {
+      cs_lnum_t c_id = m->b_face_cells[i];
+      point_tag[i] = tbm->cell_rotor_num[c_id];
+    }
+  }
+}
+
 /*----------------------------------------------------------------------------
  * Create an empty turbomachinery structure
  *----------------------------------------------------------------------------*/
@@ -185,10 +272,15 @@ _turbomachinery_create(void)
     r->invariant[i] = 0;
   }
 
+  tbm->n_max_join_tries = 5;
+  tbm->t_cur = 0;
+  tbm->dt_retry = 1e-2;
+
   tbm->reference_mesh = cs_mesh_create();
   tbm->n_b_faces_ref = -1;
   tbm->cell_rotor_num = NULL;
   tbm->model = CS_TURBOMACHINERY_NONE;
+  tbm->n_couplings = 0;
 
   return tbm;
 }
@@ -232,7 +324,7 @@ static inline void
 _apply_vector_rotation(double     m[3][4],
                        cs_real_t  v[3])
 {
-  double  t[3] = {v[0], v[1], v[0]};
+  double  t[3] = {v[0], v[1], v[2]};
 
   for (int i = 0; i < 3; i++)
     v[i] = m[i][0]*t[0] + m[i][1]*t[1] + m[i][2]*t[2];
@@ -240,7 +332,7 @@ _apply_vector_rotation(double     m[3][4],
 
 /*----------------------------------------------------------------------------
  * Compute a matrix * tensor * Tmatrix product to apply a rotation to a
- * given symmetric interleaved tensor
+ * given symmetric tensor
  *
  * parameters:
  *   matrix[3][4]        --> transformation matrix in homogeneous coords.
@@ -322,6 +414,9 @@ _relative_velocity(double        omega,
  *
  * Note that some fields which are recomputable are not copied.
  *
+ * In case of coupling only (i.e. no joining), only a partial copy of the
+ * mesh is done.
+ *
  * parameters:
  *   mesh      <-- reference mesh
  *   mesh_copy <-> mesh copy
@@ -351,11 +446,14 @@ _copy_mesh(const cs_mesh_t  *mesh,
 
   /* Local structures */
 
-  BFT_MALLOC(mesh_copy->vtx_coord, (3*mesh->n_vertices), cs_real_t);
+  BFT_REALLOC(mesh_copy->vtx_coord, (3*mesh->n_vertices), cs_real_t);
   memcpy(mesh_copy->vtx_coord,
          mesh->vtx_coord,
          3*mesh->n_vertices*sizeof(cs_real_t));
 
+  if (cs_glob_n_joinings < 1)
+    return;
+
   BFT_MALLOC(mesh_copy->i_face_cells, mesh->n_i_faces, cs_lnum_2_t);
   memcpy(mesh_copy->i_face_cells,
          mesh->i_face_cells,
@@ -464,9 +562,9 @@ _copy_mesh(const cs_mesh_t  *mesh,
     BFT_MALLOC(mesh_copy->group_idx, mesh->n_groups + 1, cs_lnum_t);
     memcpy(mesh_copy->group_idx, mesh->group_idx,
            (mesh->n_groups + 1)*sizeof(cs_lnum_t));
-    BFT_MALLOC(mesh_copy->group_lst, mesh->group_idx[mesh->n_groups] - 1, char);
-    memcpy(mesh_copy->group_lst, mesh->group_lst,
-           (mesh->group_idx[mesh->n_groups] - 1)*sizeof(char));
+    BFT_MALLOC(mesh_copy->group, mesh->group_idx[mesh->n_groups], char);
+    memcpy(mesh_copy->group, mesh->group,
+           mesh->group_idx[mesh->n_groups]*sizeof(char));
   }
 
   mesh_copy->n_families = mesh->n_families;
@@ -497,15 +595,40 @@ _copy_mesh(const cs_mesh_t  *mesh,
  * Update mesh vertex positions
  *
  * parameters:
- *   mesh <-> mesh to update
  *   t    <-- associated time
  *----------------------------------------------------------------------------*/
 
 static void
-_update_geometry(cs_mesh_t  *mesh,
-                 cs_real_t   t)
+_update_angle(cs_real_t  t)
 {
-  cs_turbomachinery_t *tbm = cs_glob_turbomachinery;
+  cs_turbomachinery_t *tbm = _turbomachinery;
+
+  /* Now update coordinates */
+
+  double dt = t - tbm->t_cur;
+
+  if (dt > 0) {
+    for (int j = 0; j < tbm->n_rotors+1; j++) {
+      cs_rotation_t *r = tbm->rotation + j;
+      r->angle += r->omega * dt;
+    }
+    tbm->t_cur = t;
+  }
+}
+
+/*----------------------------------------------------------------------------
+ * Update mesh vertex positions
+ *
+ * parameters:
+ *   mesh <-> mesh to update
+ *   dt   <-- associated time delta (0 for current, unmodified time)
+ *----------------------------------------------------------------------------*/
+
+static void
+_update_geometry(cs_mesh_t  *mesh,
+                 cs_real_t   dt)
+{
+  cs_turbomachinery_t *tbm = _turbomachinery;
 
   cs_lnum_t  f_id, v_id;
 
@@ -559,8 +682,8 @@ _update_geometry(cs_mesh_t  *mesh,
 
   for (int j = 0; j < tbm->n_rotors+1; j++) {
     cs_rotation_t *r = tbm->rotation + j;
-    r->angle = r->omega * t;
-    cs_rotation_matrix(r->angle,
+
+    cs_rotation_matrix(r->angle + r->omega*dt,
                        r->axis,
                        r->invariant,
                        m[j]);
@@ -586,7 +709,7 @@ _update_geometry(cs_mesh_t  *mesh,
 static void
 _check_geometry(cs_mesh_t  *mesh)
 {
-  cs_turbomachinery_t *tbm = cs_glob_turbomachinery;
+  cs_turbomachinery_t *tbm = _turbomachinery;
 
   cs_gnum_t n_errors = 0;
 
@@ -639,6 +762,16 @@ _select_rotor_cells(cs_turbomachinery_t  *tbm)
     cs_selector_get_cell_list(tbm->rotor_cells_c[r_id],
                               &_n_cells,
                               _cell_list);
+    cs_gnum_t _n_g_cells = _n_cells;
+    cs_parall_counter(&_n_g_cells, 1);
+    if (_n_g_cells == 0)
+      bft_error
+        (__FILE__, __LINE__, 0,
+         _("%sd: The rotor %d with cell selection criteria\n"
+           "  \"%s\"\n"
+           "does not contain any cell.\n"
+           "This rotor should be removed or its selection criteria modified."),
+         __func__, r_id + 1, tbm->rotor_cells_c[r_id]);
     for (cs_lnum_t i = 0; i < _n_cells; i++)
       tbm->cell_rotor_num[_cell_list[i]] = r_id + 1;
   }
@@ -655,6 +788,60 @@ _select_rotor_cells(cs_turbomachinery_t  *tbm)
     _check_geometry(m);
 }
 
+/*----------------------------------------------------------------------------
+ * Update mesh for unsteady rotor/stator computation when no joining is used.
+ *
+ * parameters:
+ *   restart_mode  true for restart, false otherwise
+ *   t_cur_mob     current rotor time
+ *   t_elapsed     elapsed computation time
+ */
+/*----------------------------------------------------------------------------*/
+
+static void
+_update_mesh_coupling(double   t_cur_mob,
+                      double  *t_elapsed)
+{
+  double  t_start, t_end;
+
+  cs_turbomachinery_t *tbm = _turbomachinery;
+
+  int t_stat_id = cs_timer_stats_id_by_name("mesh_processing");
+  int t_top_id = cs_timer_stats_switch(t_stat_id);
+
+  t_start = cs_timer_wtime();
+
+  /* Indicates we are in the framework of turbomachinery */
+
+  tbm->active = true;
+
+  /* Cell and boundary face numberings can be moved from old mesh
+     to new one, as the corresponding parts of the mesh should not change */
+
+  _copy_mesh(tbm->reference_mesh, cs_glob_mesh);
+
+  /* Update geometry, if necessary */
+
+  _update_angle(t_cur_mob);
+
+  if (tbm->n_rotors > 0)
+    _update_geometry(cs_glob_mesh, 0);
+
+  /* Recompute geometric quantities related to the mesh */
+
+  cs_mesh_quantities_compute(cs_glob_mesh, cs_glob_mesh_quantities);
+
+  /* Update linear algebra APIs relative to mesh */
+
+  cs_gradient_perio_update_mesh();
+
+  t_end = cs_timer_wtime();
+
+  *t_elapsed = t_end - t_start;
+
+  cs_timer_stats_switch(t_top_id);
+}
+
 /*----------------------------------------------------------------------------
  * Update mesh for unsteady rotor/stator computation.
  *
@@ -673,7 +860,7 @@ _update_mesh(bool     restart_mode,
   double  t_start, t_end;
 
   cs_halo_type_t halo_type = cs_glob_mesh->halo_type;
-  cs_turbomachinery_t *tbm = cs_glob_turbomachinery;
+  cs_turbomachinery_t *tbm = _turbomachinery;
 
   int t_stat_id = cs_timer_stats_id_by_name("mesh_processing");
   int t_top_id = cs_timer_stats_switch(t_stat_id);
@@ -684,101 +871,153 @@ _update_mesh(bool     restart_mode,
 
   tbm->active = true;
 
+  /* In case of simple coupling, use simpler update */
+
+  if (cs_glob_n_joinings < 1) {
+    _update_mesh_coupling(t_cur_mob,
+                          t_elapsed);
+    return;
+  }
+
   /* Cell and boundary face numberings can be moved from old mesh
      to new one, as the corresponding parts of the mesh should not change */
 
-  cs_numbering_t *cell_numbering = cs_glob_mesh->cell_numbering;
-  cs_glob_mesh->cell_numbering = NULL;
+  cs_numbering_t *cell_numbering = NULL;
+
+  if (restart_mode == false) {
+    cell_numbering = cs_glob_mesh->cell_numbering;
+    cs_glob_mesh->cell_numbering = NULL;
+  }
 
   /* Destroy previous global mesh and related entities */
 
-  cs_mesh_location_finalize();
   cs_mesh_quantities_destroy(cs_glob_mesh_quantities);
 
   cs_mesh_destroy(cs_glob_mesh);
 
   /* Create new global mesh and related entities */
 
-  cs_mesh_location_initialize();
   cs_glob_mesh = cs_mesh_create();
   cs_glob_mesh->verbosity = 0;
   cs_glob_mesh_builder = cs_mesh_builder_create();
   cs_glob_mesh_quantities = cs_mesh_quantities_create();
 
+  _update_angle(t_cur_mob);
+
   if (restart_mode == false) {
 
-    _copy_mesh(tbm->reference_mesh, cs_glob_mesh);
+    int n_retry = CS_MIN(tbm->n_max_join_tries, 1);
+    cs_lnum_t boundary_changed = 0;
+    double eps_dt = 0.;
 
-    /* Update geometry, if necessary */
+    do {
 
-    if (tbm->n_rotors > 0)
-      _update_geometry(cs_glob_mesh, t_cur_mob);
+      n_retry -= 1;
 
-    /* Reset the interior faces -> cells connectivity */
-    /* (in order to properly build the halo of the joined mesh) */
+      _copy_mesh(tbm->reference_mesh, cs_glob_mesh);
 
-    cs_mesh_to_builder_perio_faces(cs_glob_mesh, cs_glob_mesh_builder);
+      /* Update geometry, if necessary */
 
-    {
-      int i;
-      cs_lnum_t f_id;
-      cs_lnum_2_t *i_face_cells = (cs_lnum_2_t *)cs_glob_mesh->i_face_cells;
-      const cs_lnum_t n_cells = cs_glob_mesh->n_cells;
-      for (f_id = 0; f_id < cs_glob_mesh->n_i_faces; f_id++) {
-        for (i = 0; i < 2; i++) {
-          if (i_face_cells[f_id][i] >= n_cells)
-            i_face_cells[f_id][i] = -1;
+      if (tbm->n_rotors > 0)
+        _update_geometry(cs_glob_mesh, eps_dt);
+
+      /* Reset the interior faces -> cells connectivity */
+      /* (in order to properly build the halo of the joined mesh) */
+
+      cs_mesh_to_builder_perio_faces(cs_glob_mesh, cs_glob_mesh_builder);
+
+      {
+        int i;
+        cs_lnum_t f_id;
+        cs_lnum_2_t *i_face_cells = (cs_lnum_2_t *)cs_glob_mesh->i_face_cells;
+        const cs_lnum_t n_cells = cs_glob_mesh->n_cells;
+        for (f_id = 0; f_id < cs_glob_mesh->n_i_faces; f_id++) {
+          for (i = 0; i < 2; i++) {
+            if (i_face_cells[f_id][i] >= n_cells)
+              i_face_cells[f_id][i] = -1;
+          }
         }
       }
-    }
 
-    /* Join meshes and build periodicity links */
+      /* Join meshes and build periodicity links */
 
-    cs_join_all(false);
+      cs_join_all(false);
 
-    cs_lnum_t boundary_changed = 0;
-    if (tbm->n_b_faces_ref > -1) {
-      if (cs_glob_mesh->n_b_faces != tbm->n_b_faces_ref)
-        boundary_changed = 1;
-    }
-    cs_parall_counter_max(&boundary_changed, 1);
+      boundary_changed = 0;
+      if (tbm->n_b_faces_ref > -1) {
+        if (cs_glob_mesh->n_b_faces != tbm->n_b_faces_ref)
+          boundary_changed = 1;
+      }
+      cs_parall_counter_max(&boundary_changed, 1);
 
-    /* Check that joining has not added or removed boundary faces.
-       Postprocess new faces appearing on boundary or inside of mesh:
-       this assumes that joining appends new faces at the end of the mesh */
+      /* Check that joining has not added or removed boundary faces.
+         Postprocess new faces appearing on boundary or inside of mesh
+         (which assumes that joining appends new faces at the end of the mesh)
+         or try again with slightly different rotation angle
+       */
 
-    if (boundary_changed) {
-      const int writer_id = -2;
-      const int writer_ids[] = {writer_id};
-      const int mesh_id = cs_post_get_free_mesh_id();
-      cs_lnum_t b_face_count[] = {tbm->n_b_faces_ref,
-                                  cs_glob_mesh->n_b_faces};
-      cs_gnum_t n_g_b_faces_ref = tbm->n_b_faces_ref;
-      cs_parall_counter(&n_g_b_faces_ref, 1);
-      cs_post_init_error_writer();
-      cs_post_define_surface_mesh_by_func(mesh_id,
-                                          _("Added boundary faces"),
-                                          NULL,
-                                          _post_error_faces_select,
-                                          NULL,
-                                          b_face_count,
-                                          false, /* time varying */
-                                          true,  /* add groups if present */
-                                          false, /* auto variables */
-                                          1,
-                                          writer_ids);
-      cs_post_activate_writer(writer_id, 1);
-      cs_post_write_meshes(NULL);
-      bft_error(__FILE__, __LINE__, 0,
-                _("Error in turbomachinery mesh update:\n"
-                  "Number of boundary faces has changed from %llu to %llu.\n"
-                  "There are probably unjoined faces, "
-                  "due to an insufficiently regular mesh;\n"
-                  "adjusting mesh joining parameters might help."),
-                (unsigned long long)n_g_b_faces_ref,
-                (unsigned long long)cs_glob_mesh->n_g_b_faces);
-    }
+      if (boundary_changed) {
 
+        const char join_err_fmt[]
+          = N_("Error in turbomachinery mesh update:\n"
+               "Number of boundary faces has changed from %llu to %llu.\n"
+               "There are probably unjoined faces, "
+               "due to an insufficiently regular mesh;\n"
+               "adjusting mesh joining parameters might help.");
+
+        cs_gnum_t n_g_b_faces_ref = tbm->n_b_faces_ref;
+        cs_parall_counter(&n_g_b_faces_ref, 1);
+
+        if (n_retry < 1)  {
+          const int writer_id = -2;
+          const int writer_ids[] = {writer_id};
+          const int mesh_id = cs_post_get_free_mesh_id();
+          cs_lnum_t b_face_count[] = {tbm->n_b_faces_ref,
+                                      cs_glob_mesh->n_b_faces};
+          cs_post_init_error_writer();
+          cs_post_define_surface_mesh_by_func(mesh_id,
+                                              _("Added boundary faces"),
+                                              NULL,
+                                              _post_error_faces_select,
+                                              NULL,
+                                              b_face_count,
+                                              false, /* time varying */
+                                              true,  /* add groups if present */
+                                              false, /* auto variables */
+                                              1,
+                                              writer_ids);
+          cs_post_activate_writer(writer_id, 1);
+          cs_post_write_meshes(NULL);
+          bft_error(__FILE__, __LINE__, 0,
+                    _(join_err_fmt),
+                    (unsigned long long)n_g_b_faces_ref,
+                    (unsigned long long)cs_glob_mesh->n_g_b_faces);
+        }
+        else {
+          double dt = cs_glob_time_step->t_cur - cs_glob_time_step->t_prev;
+          eps_dt += tbm->dt_retry * dt;
+          bft_printf(_(join_err_fmt),
+                     (unsigned long long)n_g_b_faces_ref,
+                     (unsigned long long)cs_glob_mesh->n_g_b_faces);
+          bft_printf("\nTrying again with eps_dt = %lg\n", eps_dt);
+
+          /* Destroy previous global mesh and related entities */
+
+          cs_mesh_location_finalize();
+          cs_mesh_quantities_destroy(cs_glob_mesh_quantities);
+
+          cs_mesh_destroy(cs_glob_mesh);
+
+          /* Create new global mesh and related entities */
+
+          cs_mesh_location_initialize();
+          cs_glob_mesh = cs_mesh_create();
+          cs_glob_mesh->verbosity = 0;
+          cs_glob_mesh_builder = cs_mesh_builder_create();
+          cs_glob_mesh_quantities = cs_mesh_quantities_create();
+        }
+      }
+    } while (boundary_changed && n_retry >= 0);
   }
   else {
 
@@ -846,6 +1085,8 @@ _update_mesh(bool     restart_mode,
 
   cs_mesh_init_selectors();
   cs_mesh_location_build(cs_glob_mesh, -1);
+  cs_volume_zone_build_all(true);
+  cs_boundary_zone_build_all(true);
 
   /* Check coherency if debugging */
 
@@ -873,6 +1114,8 @@ _update_mesh(bool     restart_mode,
 
   }
 
+  cs_mesh_adjacencies_update_mesh();
+
   /* Update linear algebra APIs relative to mesh */
 
   cs_gradient_perio_update_mesh();
@@ -894,29 +1137,38 @@ _update_mesh(bool     restart_mode,
  *
  * parameters:
  *   iturbo <-- turbomachinery type flag
+ *   ityint <-- 0: joining; 1: internal coupling
+ *----------------------------------------------------------------------------*/
+
+void
+cs_f_map_turbomachinery_model(cs_int_t    *iturbo,
+                              cs_int_t    *ityint)
+{
+  if (_turbomachinery != NULL)
+    *iturbo = _turbomachinery->model;
+  else
+    *iturbo = CS_TURBOMACHINERY_NONE;
+
+  if (_turbomachinery->n_couplings > 0)
+    *ityint = 1;
+  else
+    *ityint = 0;
+}
+
+/*----------------------------------------------------------------------------
+ * Map turbomachinery rotor to fortran module
+ *
+ * parameters:
  *   irotce <-- pointer to cell flag for rotation
  *----------------------------------------------------------------------------*/
 
 void
-cs_f_map_turbomachinery_module(cs_int_t    *iturbo,
-                               cs_int_t   **irotce)
+cs_f_map_turbomachinery_rotor(cs_int_t   **irotce)
 {
-  if (cs_glob_turbomachinery != NULL) {
-
-    *iturbo = cs_glob_turbomachinery->model;
-
-    /* Assign rotor cells flag array to module */
-
-    *irotce = cs_glob_turbomachinery->cell_rotor_num;
-
-  }
-  else {
-
-    *iturbo = CS_TURBOMACHINERY_NONE;
-
+  if (_turbomachinery != NULL)
+    *irotce = _turbomachinery->cell_rotor_num;
+  else
     *irotce = NULL;
-
-  }
 }
 
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
@@ -935,15 +1187,15 @@ void
 cs_turbomachinery_set_model(cs_turbomachinery_model_t  model)
 {
   if (   model == CS_TURBOMACHINERY_NONE
-      && cs_glob_turbomachinery != NULL) {
+      && _turbomachinery != NULL) {
     cs_turbomachinery_finalize();
     return;
   }
 
-  else if (cs_glob_turbomachinery == NULL)
-    cs_glob_turbomachinery = _turbomachinery_create();
+  else if (_turbomachinery == NULL)
+    _turbomachinery = _turbomachinery_create();
 
-  cs_glob_turbomachinery->model = model;
+  _turbomachinery->model = model;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -955,10 +1207,10 @@ cs_turbomachinery_set_model(cs_turbomachinery_model_t  model)
 cs_turbomachinery_model_t
 cs_turbomachinery_get_model(void)
 {
-  if (cs_glob_turbomachinery == NULL)
+  if (_turbomachinery == NULL)
    return CS_TURBOMACHINERY_NONE;
   else
-    return cs_glob_turbomachinery->model;
+    return _turbomachinery->model;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -978,7 +1230,7 @@ cs_turbomachinery_add_rotor(const char    *cell_criteria,
                             const double   rotation_axis[3],
                             const double   rotation_invariant[3])
 {
-  cs_turbomachinery_t *tbm = cs_glob_turbomachinery;
+  cs_turbomachinery_t *tbm = _turbomachinery;
   if (tbm == NULL)
     return;
 
@@ -1025,7 +1277,7 @@ cs_turbomachinery_join_add(const char  *sel_criteria,
 {
   /* Allocate and initialize a cs_join_t structure */
 
-  BFT_REALLOC(cs_glob_join_array, cs_glob_n_joinings + 1, cs_join_t *);
+  BFT_REALLOC(cs_glob_join_array,  cs_glob_n_joinings + 1, cs_join_t *);
 
   cs_glob_join_array[cs_glob_n_joinings]
     = cs_join_create(cs_glob_n_joinings + 1,
@@ -1044,12 +1296,43 @@ cs_turbomachinery_join_add(const char  *sel_criteria,
   return cs_glob_n_joinings;
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Add a cs_join_t structure to the list of rotor/stator couplings.
+ *
+ * \param[in]  sel_criteria   boundary face selection criteria
+ * \param[in]  tolerance      value of the search tolerance
+ * \param[in]  verbosity      level of verbosity required
+ *
+ * \return  number (1 to n) associated with new coupling
+ */
+/*----------------------------------------------------------------------------*/
+
+int
+cs_turbomachinery_coupling_add(const char  *sel_criteria,
+                               float        tolerance,
+                               int          verbosity)
+{
+  cs_sat_coupling_add_internal(_turbomachinery_coupling_tag,
+                               _turbomachinery,
+                               sel_criteria,
+                               NULL,
+                               NULL,
+                               "all[]",
+                               tolerance,
+                               verbosity);
+
+  _turbomachinery->n_couplings += 1;
+
+  return cs_sat_coupling_n_couplings();
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Update mesh for unsteady rotor/stator computation.
  *
- * \param[in]   t_cur_mob  current rotor time
- * \param[out]  t_elapsed  elapsed computation time
+ * \param[in]   t_cur_mob     current rotor time
+ * \param[out]  t_elapsed     elapsed computation time
  */
 /*----------------------------------------------------------------------------*/
 
@@ -1062,42 +1345,45 @@ cs_turbomachinery_update_mesh(double   t_cur_mob,
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief Read mesh from checkpoint for unsteady rotor/stator computation.
+ * \brief Update mesh for unsteady rotor/stator computation in case of restart.
  *
- * \param[in]   t_cur_mob     current rotor time
- * \param[out]  t_elapsed     elapsed computation time
+ * Reads mesh from checkpoint when available.
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_turbomachinery_restart_mesh(double   t_cur_mob,
-                               double  *t_elapsed)
+cs_turbomachinery_restart_mesh(void)
 {
-  _update_mesh(true, t_cur_mob, t_elapsed);
+  if (cs_turbomachinery_get_model() != CS_TURBOMACHINERY_TRANSIENT)
+    return;
+
+  if (cs_glob_time_step->nt_prev > 0) {
+    double t_elapsed;
+    if (cs_file_isreg("restart/mesh"))
+      _update_mesh(true, cs_glob_time_step->t_cur, &t_elapsed);
+    else
+      _update_mesh(false, cs_glob_time_step->t_cur, &t_elapsed);
+  }
 }
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief Initializations for turbomachinery computation.
- *
- * \note This function should be called before once the mesh is built,
- *       but before cs_post_init_meshes() so that postprocessing meshes are
- *       updated correctly in the transient case.
+ * \brief Definitions for turbomachinery computation.
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_turbomachinery_initialize(void)
+cs_turbomachinery_define(void)
 {
   /* Define model; could be moved anywhere before time loop. */
 
   cs_gui_turbomachinery();
   cs_user_turbomachinery();
 
-  if (cs_glob_turbomachinery == NULL)
+  if (_turbomachinery == NULL)
     return;
 
-  cs_turbomachinery_t *tbm = cs_glob_turbomachinery;
+  cs_turbomachinery_t *tbm = _turbomachinery;
 
   if (tbm->model == CS_TURBOMACHINERY_NONE)
     return;
@@ -1106,6 +1392,31 @@ cs_turbomachinery_initialize(void)
 
   cs_gui_turbomachinery_rotor();
   cs_user_turbomachinery_rotor();
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Initializations for turbomachinery computation.
+ *
+ * \note This function should be called after the mesh is built,
+ *       but before cs_post_init_meshes() so that postprocessing meshes are
+ *       updated correctly in the transient case.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_turbomachinery_initialize(void)
+{
+  if (_turbomachinery == NULL)
+    return;
+
+  cs_turbomachinery_t *tbm = _turbomachinery;
+
+  if (tbm->model == CS_TURBOMACHINERY_NONE)
+    return;
+
+  /* Select rotor cells */
+
   _select_rotor_cells(tbm);
 
   /* Build the reference mesh that duplicates the global mesh before joining;
@@ -1128,12 +1439,7 @@ cs_turbomachinery_initialize(void)
 
   if (cs_glob_n_joinings > 0) {
     cs_real_t t_elapsed;
-    if (cs_file_isreg("restart/mesh"))
-      cs_turbomachinery_restart_mesh(cs_glob_time_step->t_cur,
-                                     &t_elapsed);
-    else
-      cs_turbomachinery_update_mesh(cs_glob_time_step->t_cur,
-                                    &t_elapsed);
+    cs_turbomachinery_update_mesh(0.0, &t_elapsed);
   }
 
   /* Adapt postprocessing options if required;
@@ -1163,9 +1469,9 @@ cs_turbomachinery_initialize(void)
 void
 cs_turbomachinery_finalize(void)
 {
-  if (cs_glob_turbomachinery != NULL) {
+  if (_turbomachinery != NULL) {
 
-    cs_turbomachinery_t *tbm = cs_glob_turbomachinery;
+    cs_turbomachinery_t *tbm = _turbomachinery;
 
     for (int i = tbm->n_rotors -1; i >= 0; i--)
       BFT_FREE(tbm->rotor_cells_c[i]);
@@ -1182,7 +1488,7 @@ cs_turbomachinery_finalize(void)
     cs_glob_rotation = NULL;
   }
 
-  BFT_FREE(cs_glob_turbomachinery);
+  BFT_FREE(_turbomachinery);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -1230,12 +1536,6 @@ cs_turbomachinery_resize_cell_fields(void)
     if (   f->location_id == CS_MESH_LOCATION_CELLS
         && f->is_owner) {
 
-      if (f->dim > 1 && f->interleaved == false)
-        bft_error(__FILE__, __LINE__, 0,
-                  "%s: fields owning their values (i.e. not mapped)\n"
-                  "should be interleaved, but \"%s\" is not.",
-                  __func__, f->name);
-
       /* Ghost cell sizes may change, but the number of main cells
          is unchanged, so a simple reallocation will do */
 
@@ -1281,7 +1581,7 @@ cs_turbomachinery_rotation_matrix(int        rotor_num,
                                   double     theta,
                                   cs_real_t  matrix[3][4])
 {
-  cs_turbomachinery_t *tbm = cs_glob_turbomachinery;
+  cs_turbomachinery_t *tbm = _turbomachinery;
   const cs_rotation_t *r = tbm->rotation + rotor_num;
 
   cs_rotation_matrix(theta, r->axis, r->invariant, matrix);
@@ -1302,7 +1602,7 @@ cs_turbomachinery_rotation_matrix(int        rotor_num,
 const int *
 cs_turbomachinery_get_cell_rotor_num(void)
 {
-  return cs_glob_turbomachinery->cell_rotor_num;
+  return _turbomachinery->cell_rotor_num;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -1314,13 +1614,55 @@ cs_turbomachinery_get_cell_rotor_num(void)
 /*----------------------------------------------------------------------------*/
 
 double
-cs_turbomachinery_get_rotation_velocity(int rotor_num)
+cs_turbomachinery_get_rotation_velocity(int  rotor_num)
 {
-  cs_turbomachinery_t *tbm = cs_glob_turbomachinery;
+  cs_turbomachinery_t *tbm = _turbomachinery;
 
   return (tbm->rotation + rotor_num)->omega;
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Set rotation velocity
+ *
+ * param[in]  rotor_num  rotor number (1 to n numbering)
+ * param[in]  omega      rotation velocity
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_turbomachinery_set_rotation_velocity(int     rotor_num,
+                                        double  omega)
+{
+  cs_turbomachinery_t *tbm = _turbomachinery;
+
+  (tbm->rotation + rotor_num)->omega = omega;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Set turbomachinery joining retry parameters.
+ *
+ * When a joing leads to a different number of boundary faces from the
+ * previous position, the rotor positions may be perturbed by a small
+ * quantity to try to obtain a better joining.
+ *
+ * param[in]  n_max_join_retries   maximum number of retries before considering
+ *                                 the joining has failed
+ * param[in]  dt_retry_multiplier  time step multiplier for new position retry
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_turbomachinery_set_rotation_retry(int     n_max_join_retries,
+                                     double  dt_retry_multiplier)
+{
+  cs_turbomachinery_t *tbm = _turbomachinery;
+
+  tbm->n_max_join_tries = n_max_join_retries;
+  tbm->dt_retry = dt_retry_multiplier;
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Rotation of vector and tensor fields.
@@ -1333,7 +1675,7 @@ cs_turbomachinery_rotate_fields(const cs_real_t dt[])
   cs_lnum_t i;
   cs_real_34_t  *m;
 
-  cs_turbomachinery_t *tbm = cs_glob_turbomachinery;
+  cs_turbomachinery_t *tbm = _turbomachinery;
   cs_real_t time_step = dt[0];
 
   BFT_MALLOC(m, tbm->n_rotors+1, cs_real_34_t);
@@ -1359,28 +1701,13 @@ cs_turbomachinery_rotate_fields(const cs_real_t dt[])
     cs_lnum_t _n_elts = n_elts[2];
 
     if (f->dim == 3) {
-      if (f->interleaved == false) {
-        double v[3];
-        for (i = 0; i < _n_elts; i++) {
-          v[0] = f->val[i];
-          v[1] = f->val[i + _n_elts];
-          v[2] = f->val[i + _n_elts*2];
-          _apply_vector_rotation(m[tbm->cell_rotor_num[i]], v);
-          f->val[i]             = v[0];
-          f->val[i + _n_elts]   = v[1];
-          f->val[i + _n_elts*2] = v[2];
-        }
-      }
-      else {
-        for (i = 0; i < _n_elts; i++)
-          _apply_vector_rotation(m[tbm->cell_rotor_num[i]], f->val + i*3);
-      }
+      for (i = 0; i < _n_elts; i++)
+        _apply_vector_rotation(m[tbm->cell_rotor_num[i]], f->val + i*3);
     }
 
     else if (f->dim == 6) {
-      assert(f->interleaved);
-        for (i = 0; i < _n_elts; i++)
-          _apply_sym_tensor_rotation(m[tbm->cell_rotor_num[i]], f->val + i*6);
+      for (i = 0; i < _n_elts; i++)
+        _apply_sym_tensor_rotation(m[tbm->cell_rotor_num[i]], f->val + i*6);
     }
 
     assert(f->dim == 3 || f->dim == 6);
@@ -1442,7 +1769,7 @@ cs_turbomachinery_relative_velocity(int              rotor_num,
                                     const cs_real_t  coords[3],
                                     cs_real_t        velocity[3])
 {
-  cs_turbomachinery_t *tbm = cs_glob_turbomachinery;
+  cs_turbomachinery_t *tbm = _turbomachinery;
   const cs_rotation_t *r = tbm->rotation + rotor_num;
 
   _relative_velocity(r->omega,
@@ -1452,6 +1779,88 @@ cs_turbomachinery_relative_velocity(int              rotor_num,
                      velocity);
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Read turbomachinery metadata from restart file.
+ *
+ * The mesh is handled separately.
+ *
+ * \param[in, out]  r  associated restart file pointer
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_turbomachinery_restart_read(cs_restart_t  *r)
+{
+  cs_turbomachinery_t *tbm = _turbomachinery;
+
+  if (tbm == NULL)
+    return;
+
+  cs_real_t *t_angle;
+  BFT_MALLOC(t_angle, tbm->n_rotors+2, cs_real_t);
+
+  t_angle[0] = tbm->t_cur;
+  for (int i = 0; i < tbm->n_rotors+1; i++) {
+    cs_rotation_t *rot = tbm->rotation + i;
+    t_angle[i+1] = rot->angle;
+  }
+
+  int retcode = cs_restart_read_section(r,
+                                        "turbomachinery:rotor_time_and_angle",
+                                        CS_MESH_LOCATION_NONE,
+                                        tbm->n_rotors+2,
+                                        CS_TYPE_cs_real_t,
+                                        t_angle);
+
+  if (retcode == CS_RESTART_SUCCESS) {
+    tbm->t_cur = t_angle[0];
+    for (int i = 0; i < tbm->n_rotors+1; i++) {
+      cs_rotation_t *rot = tbm->rotation + i;
+      rot->angle = t_angle[i+1];
+    }
+  }
+
+  BFT_FREE(t_angle);
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Write turbomachinery metadata to checkpoint file.
+ *
+ * The mesh is handled separately.
+ *
+ * \param[in, out]  r  associated restart file pointer
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_turbomachinery_restart_write(cs_restart_t  *r)
+{
+  const cs_turbomachinery_t *tbm = _turbomachinery;
+
+  if (tbm == NULL)
+    return;
+
+  cs_real_t *t_angle;
+  BFT_MALLOC(t_angle, tbm->n_rotors+2, cs_real_t);
+
+  t_angle[0] = tbm->t_cur;
+  for (int i = 0; i < tbm->n_rotors+1; i++) {
+    cs_rotation_t *rot = tbm->rotation + i;
+    t_angle[i+1] = rot->angle;
+  }
+
+  cs_restart_write_section(r,
+                           "turbomachinery:rotor_time_and_angle",
+                           CS_MESH_LOCATION_NONE,
+                           tbm->n_rotors+2,
+                           CS_TYPE_cs_real_t,
+                           t_angle);
+
+  BFT_FREE(t_angle);
+}
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
