@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -44,10 +44,6 @@ module cstphy
   double precision :: tkelvi
   parameter(tkelvi = 273.15d0)
 
-  !> Temperature in degrees Celsius corresponding to 0 Kelvin (= -273,15)
-  double precision :: tkelvn
-  parameter(tkelvn = -273.15d0)
-
   !> Calories (1 cvar_al = xcal2j J)
   double precision :: xcal2j
   parameter(xcal2j = 4.1855d0)
@@ -85,20 +81,40 @@ module cstphy
   !> source, to add new equations of state.
   integer(c_int), pointer, save :: ieos
 
-  !> isobaric specific heat \f$ C_p \f$
+  !> indicates if the isobaric specific heat \f$C_p\f$ is variable:
+  !>  - 0: constant, no property field is declared
+  !>  - 1: variable, \f$C_p\f$ is declared as a property field\n
+  !> When gas or coal combustion is activated, \ref icp is automatically set to 0
+  !> (constant \f$C_p\f$). With the electric module, it is automatically set to 1.
+  !> The user is not allowed to modify these default choices.\n
+  !> When \ref icp = 1 is specified, the code automatically modifies this value to
+  !> make \ref icp designate the effective index-number of the property "specific heat".
+  !> For each cell iel, the value of \f$C_p\f$ is then specified by the user in the
+  !> appropriate subroutine (\ref cs_user_physical_properties for the standard physics).\n
+  !> Useful if there is 1\f$\leqslant\f$N\f$\leqslant\f$\ref dimens::nscal "nscal" so that
+  !> iscsth(n)=1 (there is a scalar temperature) or with the compressible module
+  !> for non perfect gases.
   integer(c_int), pointer, save :: icp
 
   !> isochoric specific heat \f$ C_v \f$
   integer(c_int), pointer, save :: icv
 
   !> variable density field \f$ \rho \f$:
-  !>    - 1: true
-  !>    - 0: false
+  !>    - 1: true, its variation law be given either
+  !> in the GUI, or in the user subroutine
+  !> \ref cs_user_physical_properties .\n
+  !> See \subpage physical_properties for more informations.
+  !>    - 0: false, its value is the reference density
+  !> \ref ro0.
   integer(c_int), pointer, save :: irovar
 
   !> variable viscosity field \f$ \mu \f$:
-  !>    - 1: true
-  !>    - 0: false
+  !>    - 1: true, its variation law be given either
+  !> in the GUI, or in the user subroutine
+  !> \ref cs_user_physical_properties .\n
+  !> See \subpage physical_properties for more informations.
+  !>    - 0: false, its value is the reference molecular
+  !> dynamic viscosity \ref viscl0
   integer(c_int), pointer, save :: ivivar
 
   !> Sutherland law for laminar viscosity and thermal conductivity
@@ -107,7 +123,8 @@ module cstphy
   !> - 0: low temperature law (linear except for helium)
   integer(c_int), pointer, save :: ivsuth
 
-  !> reference density.
+  !> reference density.\n
+  !>
   !> Negative value: not initialized.
   !> Its value is not used in gas or coal combustion modelling (it will be
   !> calculated following the perfect gas law, with \f$P0\f$ and \f$T0\f$).
@@ -134,14 +151,15 @@ module cstphy
   !> pressure
   real(c_double), pointer, save :: ro0
 
-  !> reference molecular dynamic viscosity.
-  !> Negative value: not initialized.
+  !> reference molecular dynamic viscosity.\n
   !>
+  !> Negative value: not initialized.
   !> Always useful, it is the used value unless the user specifies the
   !> viscosity in the subroutine \ref usphyv
   real(c_double), pointer, save :: viscl0
 
-  !> reference pressure for the total pressure.
+  !> reference pressure for the total pressure.\n
+  !>
   !> except with the compressible module, the total pressure \f$P\f$ is evaluated
   !> from the reduced pressure \f$P^*\f$ so that \f$P\f$
   !> is equal to \ref p0 at the reference position \f$\vect{x}_0\f$
@@ -150,7 +168,8 @@ module cstphy
   !> always Useful
   real(c_double), pointer, save :: p0
 
-  !> reference value for the reduced pressure \f$P^*\f$ (see \ref ro0).
+  !> reference value for the reduced pressure \f$P^*\f$ (see \ref ro0).\n
+  !>
   !> It is especially used to initialise the reduced pressure and as a reference
   !> value for the outlet boundary conditions.
   !> For an optimised precision in the resolution of \f$P^*\f$,
@@ -300,10 +319,10 @@ module cstphy
   !> (see \ref cv2fmu for the value of \f$C_\mu\f$ in case of v2f modelling).
   !> Useful if and only if \ref iturb = 20, 21, 30, 31 or 60
   !> (\f$k-\varepsilon\f$, \f$R_{ij}-\varepsilon\f$ or \f$k-\omega\f$)
-  double precision, save :: cmu
+  real(c_double), pointer, save :: cmu
 
   !> \f$ C_\mu^\frac{1}{4} \f$
-  double precision, save :: cmu025
+  real(c_double), pointer, save :: cmu025
 
   !> constant \f$C_{\varepsilon 1}\f$ for all the RANS turbulence models except
   !> for the v2f and the \f$k-\omega\f$ models.
@@ -335,7 +354,7 @@ module cstphy
   !> Prandtl number for \f$\varepsilon\f$.
   !> Useful if and only if \ref iturb= 20, 21, 30, 31 or 50
   !> (\f$k-\varepsilon\f$, \f$R_{ij}-\varepsilon\f$ or v2f)
-  double precision, save :: sigmae
+  real(c_double), pointer, save :: sigmae
 
   !> constant \f$C_1\f$ for the \f$R_{ij}-\varepsilon\f$ LRR model.
   !> Useful if and only if \ref iturb=30
@@ -491,6 +510,9 @@ module cstphy
   !> Useful if and only if \ref iturb=60 (\f$k-\omega\f$ SST)
   double precision, save :: ckwbt2
 
+  !> constant \f$ C_{DDES}\f$ for the \f$k-\omega\f$ SST model.
+  !> Useful if and only if \ref iturb=60 (\f$k-\omega\f$ SST) and iddes=1
+  double precision, save :: cddes
 
   !> \f$\frac{\beta_1}{C_\mu}-\frac{\kappa^2}{\sqrt{C_\mu}\sigma_{\omega 1}}\f$
   !> constant \f$\gamma_1\f$ for the \f$k-\omega\f$ SST model.
@@ -588,7 +610,11 @@ module cstphy
   real(c_double), pointer, save :: xlomlg
 
   !> constant used in the definition of LES filtering diameter:
-  !> \f$ \delta = \text{xlesfl} . (\text{ales} . volume)^{\text{bles}} \f$
+  !> \f$ \delta = \text{xlesfl} . (\text{ales} . volume)^{\text{bles}}\f$
+  !> \ref xlesfl is a constant used to define, for
+  !> each cell \f$\omega_i\f$, the width of the (implicit) filter:
+  !> \f$\overline{\Delta}=xlesfl(ales*|\Omega_i|)^{bles}\f$\n
+  !> Useful if and only if \ref iturb = 40 or 41
   double precision, save :: xlesfl
 
   !> constant used to define, for each cell \f$Omega_i\f$,
@@ -629,15 +655,21 @@ module cstphy
   double precision, save :: xlesfd
 
   !> maximum allowed value for the variable \f$C\f$ appearing in the LES dynamic
-  !> model (the "square" comes from the fact that the
-  !> variable of the dynamic model corresponds to the square of the
-  !> constant of the Smagorinsky model).
+  !> model.
   !> Any larger value yielded by the calculation
-  !> procedure of the dynamic model will be clipped to \f$ smagmx^2\f$.
+  !> procedure of the dynamic model will be clipped to \f$ smagmx\f$.
   !>
   !> Useful if and only if \ref iturb = 41
   double precision, save :: smagmx
 
+  !> minimum allowed value for the variable \f$C\f$ appearing in the LES dynamic
+  !> model.
+  !> Any smaller value yielded by the calculation
+  !> procedure of the dynamic model will be clipped to \f$ smagmn\f$.
+  !>
+  !> Useful if and only if \ref iturb = 41
+  double precision, save :: smagmn
+
   !> van Driest constant appearing in the van Driest damping function
   !> applied to the Smagorinsky constant:
   !>  - \f$ (1-\exp^{(-y^+/cdries}) \f$.
@@ -710,6 +742,11 @@ module cstphy
   double precision, save :: cthafm
   !> constant of GGDH and AFM on the thermal scalar
   double precision, save :: cthdfm
+  !> constant of EB-AFM and EB-DFM
+  double precision, save :: xclt
+  !> constant of EB-DFM
+  double precision, save :: rhebdfm
+  double precision, save :: cthebdfm
 
   !> \}
 
@@ -739,20 +776,38 @@ module cstphy
     ! Interface to C function retrieving pointers to members of the
     ! global fluid properties structure
 
-    subroutine cs_f_fluid_properties_get_pointers(ixyzp0, ieos, icp, icv,      &
-                                                  irovar, ivivar, ro0, ivsuth, &
-                                                  viscl0, p0, pred0, xyzp0, t0,&
-                                                  cp0, cv0, xmasmr, psginf,    &
-                                                  gammasg, ipthrm, pther,      &
-                                                  pthera, pthermax,            &
-                                                  sleak, kleak,roref)          &
+    subroutine cs_f_fluid_properties_get_pointers(ixyzp0,  &
+                                                  ieos,    &
+                                                  icp,     &
+                                                  icv,     &
+                                                  irovar,  &
+                                                  ivivar,  &
+                                                  ivsuth,  &
+                                                  ro0,     &
+                                                  viscl0,  &
+                                                  p0,      &
+                                                  pred0,   &
+                                                  xyzp0,   &
+                                                  t0,      &
+                                                  cp0,     &
+                                                  cv0,     &
+                                                  xmasmr,  &
+                                                  psginf,  &
+                                                  gammasg, &
+                                                  ipthrm,  &
+                                                  pther,   &
+                                                  pthera,  &
+                                                  pthermax,&
+                                                  sleak,   &
+                                                  kleak,   &
+                                                  roref)   &
       bind(C, name='cs_f_fluid_properties_get_pointers')
       use, intrinsic :: iso_c_binding
       implicit none
       type(c_ptr), intent(out) :: ixyzp0, ieos, icp, icv, irovar, ivivar, ivsuth
-      type(c_ptr), intent(out) :: ipthrm
       type(c_ptr), intent(out) :: ro0, viscl0, p0, pred0
       type(c_ptr), intent(out) :: xyzp0, t0, cp0, cv0, xmasmr, psginf, gammasg
+      type(c_ptr), intent(out) :: ipthrm
       type(c_ptr), intent(out) :: pther, pthera, pthermax
       type(c_ptr), intent(out) :: sleak, kleak, roref
     end subroutine cs_f_fluid_properties_get_pointers
@@ -783,12 +838,24 @@ module cstphy
     ! turbulence model
 
     subroutine cs_f_turb_complete_constants() &
-      bind(C, name='cs_f_turb_complete_constants')
+      bind(C, name='cs_turb_compute_constants')
       use, intrinsic :: iso_c_binding
     end subroutine cs_f_turb_complete_constants
 
     !---------------------------------------------------------------------------
 
+    ! Interface to C function retrieving pointers to constants of the
+    ! turbulence model
+
+    subroutine cs_f_turb_model_constants_get_pointers(sigmae, cmu, cmu025) &
+      bind(C, name='cs_f_turb_model_constants_get_pointers')
+      use, intrinsic :: iso_c_binding
+      implicit none
+      type(c_ptr), intent(out) :: sigmae , cmu  , cmu025
+    end subroutine cs_f_turb_model_constants_get_pointers
+
+    !---------------------------------------------------------------------------
+
     !> (DOXYGEN_SHOULD_SKIP_THIS) \endcond
 
     !---------------------------------------------------------------------------
@@ -833,10 +900,11 @@ contains
     ! Local variables
 
     type(c_ptr) :: c_ixyzp0, c_ieos, c_icp, c_icv, c_irovar, c_ivivar
-    type(c_ptr) :: c_ipthrm
     type(c_ptr) :: c_ivsuth, c_ro0, c_viscl0, c_p0
     type(c_ptr) :: c_pred0, c_xyzp0, c_t0, c_cp0, c_cv0, c_xmasmr, c_psginf
-    type(c_ptr) :: c_gammasg, c_pther, c_pthera, c_pthermax
+    type(c_ptr) :: c_gammasg
+    type(c_ptr) :: c_ipthrm
+    type(c_ptr) :: c_pther, c_pthera, c_pthermax
     type(c_ptr) :: c_sleak, c_kleak, c_roref
 
     call cs_f_fluid_properties_get_pointers(c_ixyzp0, c_ieos, c_icp, c_icv, &
@@ -897,6 +965,28 @@ contains
 
   end subroutine turb_reference_values_init
 
+  !> \brief Initialize Fortran turbulence model constants.
+  !> This maps Fortran pointers to global C real numbers.
+
+  subroutine turb_model_constants_init
+
+    use, intrinsic :: iso_c_binding
+    implicit none
+
+    ! Local variables
+
+    type(c_ptr) :: c_sigmae, c_cmu, c_cmu025
+
+    call cs_f_turb_model_constants_get_pointers(c_sigmae, c_cmu, c_cmu025)
+
+    call c_f_pointer(c_sigmae , sigmae)
+    call c_f_pointer(c_cmu    , cmu   )
+    call c_f_pointer(c_cmu025 , cmu025)
+
+    call cs_f_turb_complete_constants
+
+  end subroutine turb_model_constants_init
+
   !=============================================================================
 
 end module cstphy
