@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,10 +20,7 @@
 
 !-------------------------------------------------------------------------------
 
-subroutine ecrava &
-!================
-
- ( frcxt  , prhyd  )
+subroutine ecrava
 
 !===============================================================================
 ! Purpose:
@@ -36,9 +33,6 @@ subroutine ecrava &
 !__________________.____._____.________________________________________________.
 ! name             !type!mode ! role                                           !
 !__________________!____!_____!________________________________________________!
-! frcxt(3,ncelet)  ! tr ! <-- ! force exterieure generant la pression          !
-!                  !    !     !  hydrostatique                                 !
-! prhyd(ncelet)    ! tr ! <-- ! hydrostatic pressure predicted                 !
 !__________________!____!_____!________________________________________________!
 
 !     Type: i (integer), r (real), s (string), a (array), l (logical),
@@ -85,8 +79,6 @@ implicit none
 
 ! Arguments
 
-double precision frcxt(3,ncelet), prhyd(ncelet)
-
 ! Local variables
 
 character        rubriq*64,car2*2,car4*4,car54*54
@@ -114,6 +106,8 @@ double precision, allocatable, dimension(:) :: tparbf
 double precision, pointer, dimension(:) :: dt_s
 double precision, pointer, dimension(:,:) :: disale
 
+type(var_cal_opt) :: vcopt
+
 !===============================================================================
 !     A noter :
 !        Lorsque qu'il est necessaire d'utiliser un ordre implicite
@@ -237,12 +231,21 @@ nbval  = 1
 ival(1) = icavit
 call restart_write_section_int_t(rp,rubriq,itysup,nbval,ival)
 
+!  ---> VOF
+rubriq = 'vof'
+itysup = 0
+nbval  = 1
+ival(1) = ivofmt
+call restart_write_section_int_t(rp,rubriq,itysup,nbval,ival)
+
 rubriq = 'instant_mobile_precedent'
 itysup = 0
 nbval  = 1
 rval(1) = ttcmob
 call restart_write_section_real_t(rp,rubriq,itysup,nbval,rval)
 
+call turbomachinery_restart_write(rp)
+
 if (ichemistry.gt.0.or.iaerosol.gt.0) then
   rubriq = 'atmospheric_chem'
   itysup = 0
@@ -265,16 +268,19 @@ call restart_write_variables(rp, 0)
 
 f_id = -1
 do ivar = 1, nvar
-  if (ibdtso(ivar).gt.1) then
+  call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+  if (vcopt%ibdtso.gt.1) then
     if (f_id.ne.ivarfl(ivar)) then
       f_id = ivarfl(ivar)
-      do t_id = 1, ibdtso(ivar) - 1
+      do t_id = 1, vcopt%ibdtso - 1
         call restart_write_field_vals(rp, f_id, t_id)
       enddo
     endif
   endif
 enddo
 
+call restart_write_fields(rp, RESTART_MAIN)
+
 !===============================================================================
 ! 4. FERMETURE FICHIER SUITE DE BASE
 !===============================================================================
@@ -334,6 +340,12 @@ if (iecaux.eq.1) then
   ival(1) = icavit
   call restart_write_section_int_t(rp,rubriq,itysup,nbval,ival)
 
+  !  ---> VOF
+  rubriq = 'vof'
+  itysup = 0
+  nbval  = 1
+  ival(1) = ivofmt
+  call restart_write_section_int_t(rp,rubriq,itysup,nbval,ival)
 
 #if defined(_CS_LANG_FR)
   car54 =' Fin de l''ecriture des dimensions et des options     '
@@ -377,14 +389,14 @@ if (iecaux.eq.1) then
   endif
 
   !     Masse volumique si elle est variable uniquement
-  !     La masse volumique est egalement ecrite pour le modele de cavitation
-  if (irovar.eq.1.or.icavit.ge.0) then
+  !     La masse volumique est egalement ecrite pour l'algo. VOF
+  if (irovar.eq.1.or.ivofmt.ge.0) then
     ! Masse volumique - cellules
     call restart_write_field_vals(rp, icrom, 0)
 
     ! Masse volumique du pdt precedent - cellules
-    ! only for cavitation and dilatable models (idilat = 4, 5)
-    if (icavit.ge.0.or.idilat.ge.4) then
+    ! only for VOF algo. and dilatable models (idilat = 4, 5)
+    if (ivofmt.ge.0.or.idilat.ge.4) then
       call restart_write_field_vals(rp, icrom, 1)
     endif
 
@@ -394,7 +406,7 @@ if (iecaux.eq.1) then
     ! Scalar source terms for dilatable model (idilat = 4, 5)
     if (idilat.ge.4) then
       do iscal = 1, nscal
-        f_id = iprpfl(iustdy(iscal))
+        f_id = iustdy(iscal)
         call restart_write_field_vals(rp, f_id, 0)
       enddo
     endif
@@ -412,22 +424,22 @@ if (iecaux.eq.1) then
   !       Une exception : on ecrit egalement Cp en effet joule pour
   !         pouvoir calculer la temperature H/Cp en debut de calcul
 
-  if (iviext.gt.0.or.icavit.ge.0) then
+  if (iviext.gt.0.or.ivofmt.ge.0) then
     !  Viscosite moleculaire - cellules (si variable ou cavitation)
-    if (ivivar.eq.1.or.icavit.ge.0) then
-      call restart_write_field_vals(rp, iprpfl(iviscl), 0)
+    if (ivivar.eq.1.or.ivofmt.ge.0) then
+      call restart_write_field_vals(rp, iviscl, 0)
     endif
 
     if (iviext.gt.0) then
       ! Viscosite turbulente ou de sous-maille - cellules
-      call restart_write_field_vals(rp, iprpfl(ivisct), 0)
+      call restart_write_field_vals(rp, ivisct, 0)
     endif
   endif
 
-  if ((icpext.gt.0.and.icp.gt.0).or.              &
-       (ippmod(ieljou).ge.1.and.icp.gt.0))  then
+  if ((icpext.gt.0.and.icp.ge.0).or.              &
+       (ippmod(ieljou).ge.1.and.icp.ge.0))  then
     !  Chaleur massique - cellules
-    call restart_write_field_vals(rp, iprpfl(icp), 0)
+    call restart_write_field_vals(rp, icp, 0)
   endif
 
   call restart_write_linked_fields(rp, "scalar_diffusivity_id", iecr)
@@ -504,11 +516,8 @@ if (iecaux.eq.1) then
   iecr = 0
 
   if (ineedy.eq.1) then
-    iecr   = 1
-    itysup = 1
-    nbval  = 1
-    rubriq = 'dist_fac_par_ce_phase01'
-    call restart_write_section_real_t(rp,rubriq,itysup,nbval,dispar)
+    call field_get_id('wall_distance', f_id)
+    call restart_write_field_vals(rp, f_id, 0)
   endif
 
   if (iecr.ne.0) then
@@ -523,39 +532,15 @@ if (iecaux.eq.1) then
 ! ---> Force exterieure
 
   if (iphydr.eq.1) then
-
-    itysup = 1
-    nbval  = 3
-
-    rubriq = 'force_ext_ce_phase01'
-    call restart_write_section_real_t(rp,rubriq,itysup,nbval,frcxt)
-
-#if defined(_CS_LANG_FR)
-    car54=' Fin de l''ecriture des forces exterieures            '
-#else
-    car54=' End writing the external forces                      '
-#endif
-    write(nfecra,1110)car54
-
+    call field_get_id('volume_forces', f_id)
+    call restart_write_field_vals(rp, f_id, 0)
   endif
 
 ! ---> Pression hydrostatique predite
 
   if (iphydr.eq.2) then
-
-    itysup = 1
-    nbval  = 1
-
-    rubriq = 'Prhyd_pre_phase01'
-    call restart_write_section_real_t(rp,rubriq,itysup,nbval,prhyd)
-
-#if defined(_CS_LANG_FR)
-    car54=' Fin d''ecriture de la pression hydrostatique predite '
-#else
-    car54=' End writing the predicted hydrostatic pressure       '
-#endif
-    write(nfecra,1110)car54
-
+    call field_get_id('hydrostatic_pressure_prd', f_id)
+    call restart_write_field_vals(rp, f_id, 0)
   endif
 
 !----------------------------------------------------------------
@@ -563,7 +548,7 @@ if (iecaux.eq.1) then
 !      with or wihtout the 1D thermal model tag1D
 !----------------------------------------------------------------
 
-  if (icond.eq.0) then
+  if (icondb.eq.0) then
 
     if (nztag1d.eq.1) then
 
@@ -632,12 +617,7 @@ if (iecaux.eq.1) then
     ival(1) = iortvm
     call restart_write_section_int_t(rp,rubriq,itysup,nbval,ival)
 
-    call restart_write_field_vals(rp, iprpfl(ipproc(ivisma(1))), 0)
-
-    if (iortvm.eq.1) then
-      call restart_write_field_vals(rp, iprpfl(ipproc(ivisma(2))), 0)
-      call restart_write_field_vals(rp, iprpfl(ipproc(ivisma(3))), 0)
-    endif
+    call restart_write_field_vals(rp, ivisma, 0)
 
 #if defined(_CS_LANG_FR)
     car54=' Fin de l''ecriture des donnees ALE    '
@@ -1086,6 +1066,7 @@ if (iecaux.eq.1) then
     write(nfecra,1110)car54
   endif
 
+  call restart_write_fields(rp, RESTART_AUXILIARY)
 
   ! Fermeture du fichiers suite auxiliaire
   call restart_destroy(rp)
