@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -57,6 +57,7 @@ use ppincl
 use radiat
 use ihmpre
 use mesh
+use post
 use field
 use cs_c_bindings
 use darcy_module
@@ -69,13 +70,15 @@ implicit none
 
 ! Local variables
 
-character(len=80) :: f_label, f_name, s_name
-integer           :: ii
-integer           :: ippok
-integer           :: ipropp, idim1, idim3, idim6, iflid
-integer           :: type_flag, location_id, ipp
+character(len=80) :: f_label, f_name, s_name, s_label
+integer           :: ii, ivar, isorb, keysrb, igwfpr, keypre, ischcp
+integer           :: idim1, idim3, idim6, iflid
+integer           :: type_flag, post_flag, location_id
 logical           :: has_previous
 
+type(gwf_soilwater_partition) :: sorption_scal
+type(var_cal_opt) :: vcopt_u
+
 !===============================================================================
 ! Interfaces
 !===============================================================================
@@ -103,13 +106,6 @@ interface
 
 end interface
 
-!===============================================================================
-! 0. INITIALISATIONS
-!===============================================================================
-
-! Initialize variables to avoid compiler warnings
-ippok = 0
-
 !===============================================================================
 ! 1. PROPRIETES PRINCIPALES
 !===============================================================================
@@ -123,9 +119,9 @@ ippok = 0
 !         Il y a toujours, pour toutes les phases,  rho, viscl, visct
 !         Il y a toujours la pression totale (sauf en compressible)
 !         Lorsqu'elles sont variables, on a les proprietes suivantes :
-!           . cp    (par phase)
+!           . cp
 !           . visls (par scalaire)
-!           . csmago (par phase) en LES dynamique
+!           . csmago en LES dynamique
 !         En ALE on a la viscosite de maillage
 !         On a aussi les flux de masse porteurs :
 !           . les variables u,v,w,p,turbulence sont portees par leur
@@ -144,22 +140,46 @@ ippok = 0
 
 ! Base properties, always present
 
-call add_property_field('density', 'Density', irom)
-icrom = iprpfl(irom)
+call add_property_field_1d('density', 'Density', irom)
+icrom = irom
+! Postprocessed and in the log file by default, hidden in modini if not variable.
+call field_set_key_int(icrom, keylog, 1)
+call field_set_key_int(icrom, keyvis, 1)
 
 call add_boundary_property_field_owner('boundary_density', 'Boundary Density', &
                                        ibrom)
 
-call add_property_field('molecular_viscosity', 'Laminar Viscosity', iviscl)
+call add_property_field_1d('molecular_viscosity', 'Laminar Viscosity', iviscl)
 
+call add_property_field_1d('turbulent_viscosity', 'Turb Viscosity', ivisct)
 if (iturb.eq.0) then
-  call add_property_field_hidden('turbulent_viscosity', 1, ivisct)
-else
-  call add_property_field('turbulent_viscosity', 'Turb Viscosity', ivisct)
+  call hide_property(ivisct)
 endif
 
-call add_property_field('courant_number', 'CFL', icour)
-call add_property_field('fourier_number', 'Fourier Number', ifour)
+! If hybrid spatial scheme is activated for the velocity (ischcv=3)
+! creation of the field hybrid_blend wihich contains the
+! local blending factor for each cell
+call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt_u)
+ischcp = vcopt_u%ischcv
+if (ischcp.eq.3) then
+  call add_property_field_1d('hybrid_blend', 'Hybrid blending function', iflid)
+end if
+
+if  (iturb.eq.60) then
+  call add_property_field_1d('s2', 'S2', is2kw)
+  call hide_property(is2kw)
+  call add_property_field_1d('vel_gradient_trace', 'Vel. Gradient Trace', idivukw)
+  call hide_property(idivukw)
+  ! Hybrid RANS/LES function f_d is stored for Post Processing in hybrid_blend.
+  ! If  hybrid spatial scheme is activated for the velocity (ischcv=3) f_d
+  ! is used as blending factor and this field already exists
+  if (iddes.eq.1.and.ischcp.ne.3) then
+    call add_property_field_1d('hybrid_blend', 'Hybrid blending function', iflid)
+  end if
+endif
+
+call add_property_field_1d('courant_number', 'CFL', icour)
+call add_property_field_1d('fourier_number', 'Fourier Number', ifour)
 
 ! Total pressure is stored in property field of index iprtot
 ! if the compressible module is not enabled (otherwise Ptot=P*).
@@ -167,14 +187,14 @@ call add_property_field('fourier_number', 'Fourier Number', ifour)
 ! only used if the gravity is set.
 
 if (ippmod(icompf).lt.0.and.ippmod(idarcy).lt.0) then
-  call add_property_field('total_pressure', 'Total Pressure', iprtot)
+  call add_property_field_1d('total_pressure', 'Total Pressure', iprtot)
 else if (ippmod(idarcy).ge.0.and.darcy_gravity.ge.1) then
-  call add_property_field('total_pressure', 'Pressure head', iprtot)
+  call add_property_field_1d('total_pressure', 'Pressure head', iprtot)
 endif
 
 ! Cs^2 si on est en LES dynamique
 if (iturb.eq.41) then
-  call add_property_field('smagorinsky_constant^2', 'Csdyn2', ismago)
+  call add_property_field_1d('smagorinsky_constant^2', 'Csdyn2', ismago)
 else
   ismago = 0
 endif
@@ -183,23 +203,8 @@ endif
 !       ajouter celles relatives a la physique particuliere
 
 ! --- Modifications pour la physique particuliere
-!      des entiers NPROCE
-
-!      Sauvegarde pour la physique particuliere de IPROP
-!      afin d'initialiser les positions des variables d'etat
-!      Attention IPROPP est le dernier numero affecte pour les proprietes.
-ipropp = nproce
 
 call ppprop
-!==========
-
-! --- Verification de NPROCE
-
-if (nproce.gt.npromx) then
-  write(nfecra,7200)nproce, npromx, nproce
-  call csexit (1)
-  !==========
-endif
 
 ! --- Properties for Darcy module
 
@@ -210,29 +215,74 @@ if (ippmod(idarcy).eq.1) then
   idim6 = 6
   f_name = 'saturation'
   f_label = 'Saturation'
-  call add_property_field_owner(f_name, f_label, idim1, has_previous, iflid)
+  call add_property_field(f_name, f_label, idim1, has_previous, iflid)
   f_name = 'capacity'
   f_label = 'Capacity'
-  call add_property_field_owner(f_name, f_label, idim1, has_previous, iflid)
+  call add_property_field(f_name, f_label, idim1, has_previous, iflid)
   f_name = 'permeability'
   f_label = 'Permeability'
   if (darcy_anisotropic_permeability.eq.0) then
-    call add_property_field_owner(f_name, f_label, idim1, has_previous, iflid)
+    call add_property_field(f_name, f_label, idim1, has_previous, iflid)
   else
-    call add_property_field_owner(f_name, f_label, idim6, has_previous, iflid)
+    call add_property_field(f_name, f_label, idim6, has_previous, iflid)
   endif
+  f_name = 'soil_density'
+  f_label = 'Soil density'
+  call add_property_field(f_name, f_label, idim1, has_previous, iflid)
+
+  call field_get_key_id("gwf_sorbed_concentration_id", keysrb)
+  call field_get_key_id("gwf_precip_concentration_id", keypre)
+
   do ii = 1, nscal
+    ivar = isca(ii)
+    call field_get_key_struct_gwf_soilwater_partition(ivarfl(ivar), &
+                                                      sorption_scal)
+    call field_get_name(ivarfl(ivar), s_name)
+    call field_get_name(ivarfl(ivar), s_label)
 
-    if (isca(ii) .gt. 0) then
+    f_name = trim(s_name)//'_kd'
+    f_label = trim(s_label)//' Kd'
+    call add_property_field(f_name, f_label, idim1, has_previous, &
+                            sorption_scal%ikd)
+    call hide_property(sorption_scal%ikd)
+    f_name = trim(s_name)//'_delay'
+    f_label = trim(s_label)//' delay'
+    call add_property_field(f_name, f_label, idim1, has_previous, &
+                            sorption_scal%idel)
 
-      call field_get_name(ivarfl(isca(ii)), s_name)
-
-      f_name = trim(s_name)//'_delay'
-      f_label = trim(s_name)//'_delay'
-      call add_property_field_owner(f_name, f_label, idim1, has_previous, iflid)
+    if (sorption_scal%kinetic.eq.1) then
+      f_name = trim(s_name)//'_sorb_conc'
+      f_label = trim(s_label)//' sorb conc'
+      call add_property_field(f_name, f_label, idim1, has_previous, isorb)
+      call field_set_key_int(ivarfl(ivar), keysrb, isorb)
 
+      f_name = trim(s_name)//'_kplus'
+      f_label = trim(s_label)//' kplus'
+      call add_property_field(f_name, f_label, idim1, has_previous, &
+                              sorption_scal%ikp)
+      call hide_property(sorption_scal%ikp)
+      f_name = trim(s_name)//'_kminus'
+      f_label = trim(s_label)//' kminus'
+      call add_property_field(f_name, f_label, idim1, has_previous, &
+                              sorption_scal%ikm)
+      call hide_property(sorption_scal%ikm)
     endif
 
+    if (sorption_scal%imxsol.ge.0) then
+      f_name = trim(s_name)//'_precip_conc'
+      f_label = trim(s_label)//' precip conc'
+      call add_property_field(f_name, f_label, idim1, has_previous, igwfpr)
+      call field_set_key_int(ivarfl(ivar), keypre, igwfpr)
+
+      f_name = trim(s_name)//'_solubility_index'
+      f_label = trim(s_label)//' solubility index'
+      call add_property_field(f_name, f_label, idim1, has_previous, &
+                              sorption_scal%imxsol)
+      call hide_property(sorption_scal%imxsol)
+    endif
+
+    call field_set_key_struct_gwf_soilwater_partition(ivarfl(ivar), &
+                                                      sorption_scal)
   enddo
 
 endif
@@ -246,16 +296,14 @@ if (iale.eq.1) then
   f_name = 'disale'
   f_label = 'Mesh displacement'
   type_flag = FIELD_PROPERTY
+  post_flag = POST_ON_LOCATION
   location_id = 4 ! variables defined on vertices
 
   call field_create(f_name, type_flag, location_id, idim3, &
                     has_previous, fdiale)
-  call field_set_key_int(fdiale, keyvis, 1)
+  call field_set_key_int(fdiale, keyvis, post_flag)
   call field_set_key_int(fdiale, keylog, 1)
 
-  ipp = field_post_id(fdiale)
-  call field_set_key_int(fdiale, keyipp, ipp)
-
   call field_set_key_str(fdiale, keylbl, trim(f_label))
 
 endif
@@ -269,14 +317,7 @@ call cs_parameters_create_added_properties
 
 if (itherm.eq.2 .and. itemp.eq.0) then
   call field_get_id_try('temperature', iflid)
-  if (iflid.ge.0) then
-    do ii = 1, nproce
-      if (iflid .eq. iprpfl(ii)) then
-        itemp = ii
-        exit
-      endif
-    enddo
-  endif
+  if (iflid.ge.0) itemp = iflid
 endif
 
 ! Map pointers
@@ -290,58 +331,6 @@ return
 ! 2. Formats
 !===============================================================================
 
-#if defined(_CS_LANG_FR)
-
- 7200 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    =========                                               ',/,&
-'@     NOMBRE DE PROPRIETES TROP GRAND                        ',/,&
-'@                                                            ',/,&
-'@  Le type de calcul defini                                  ',/,&
-'@    correspond aux nombres de proprietes suivants           ',/,&
-'@      au centre des cellules       : NPROCE = ',i10          ,/,&
-'@  Le nombre de proprietes maximal prevu                     ',/,&
-'@                      dans paramx.h est NPROMX = ',i10       ,/,&
-'@                                                            ',/,&
-'@  Le calcul ne sera pas execute.                            ',/,&
-'@                                                            ',/,&
-'@  Verifier les parametres.                                  ',/,&
-'@                                                            ',/,&
-'@  NPROMX doit valoir au moins ',i10                          ,/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-
-#else
-
- 7200 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ WARNING   : STOP AT THE INITIAL DATA VERIFICATION       ',/,&
-'@    =========                                               ',/,&
-'@     NUMBER OF VARIABLES TOO LARGE                          ',/,&
-'@                                                            ',/,&
-'@  The type of calculation defined                           ',/,&
-'@    corresponds  to the following number of properties      ',/,&
-'@      at the cell centers          : NPROCE = ',i10          ,/,&
-'@  The maximum number of properties allowed                  ',/,&
-'@                      in   paramx   is  NPROMX = ',i10       ,/,&
-'@                                                            ',/,&
-'@  The calculation cannot be executed                        ',/,&
-'@                                                            ',/,&
-'@  Verify   parameters.                                      ',/,&
-'@                                                            ',/,&
-'@  NPROMX must be at least     ',i10                          ,/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-
-#endif
-
 !===============================================================================
 ! 5. End
 !===============================================================================
@@ -351,120 +340,8 @@ end subroutine fldprp
 
 !===============================================================================
 
-!> \brief add field defining a hidden property field defined on cells
-!
-!-------------------------------------------------------------------------------
-! Arguments
-!______________________________________________________________________________.
-!  mode           name          role                                           !
-!______________________________________________________________________________!
-!> \param[in]     name          field name
-!> \param[in]     dim           field dimension
-!> \param[out]    iprop         matching field property id
-!_______________________________________________________________________________
-
-subroutine add_property_field_hidden &
- ( name, dim, iprop )
-
-!===============================================================================
-! Module files
-!===============================================================================
-
-use paramx
-use dimens
-use entsor
-use numvar
-use field
-
-!===============================================================================
-
-implicit none
-
-! Arguments
-
-character(len=*), intent(in) :: name
-integer, intent(in)          :: dim
-integer, intent(out)         :: iprop
-
-! Local variables
-
-integer  id, type_flag, location_id, ii
-logical  has_previous
-
-!===============================================================================
-
-type_flag = FIELD_INTENSIVE + FIELD_PROPERTY
-location_id = 1 ! variables defined on cells
-has_previous = .false.
-
-! Test if the field has already been defined
-call field_get_id_try(trim(name), id)
-if (id .ge. 0) then
-  write(nfecra,1000) trim(name)
-  call csexit (1)
-endif
-
-! Create field
-
-call field_create(name, type_flag, location_id, dim, has_previous, id)
-
-call field_set_key_int(id, keyvis, 0)
-call field_set_key_int(id, keylog, 0)
-
-! Property number and mapping to field
-
-iprop = nproce + 1
-nproce = nproce + dim
-
-call fldprp_check_nproce
-
-do ii = 1, dim
-  iprpfl(iprop + ii -1) = id
-  ipproc(iprop + ii - 1) = iprop + ii - 1
-enddo
-
-! Postprocessing slots
-
-do ii = 1, dim
-  ipppro(iprop+ii-1) = 1
-enddo
-
-return
-
-!---
-! Formats
-!---
-
-#if defined(_CS_LANG_FR)
- 1000 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ERREUR :    ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    ========                                                ',/,&
-'@     LE CHAMP : ', a, 'EST DEJA DEFINI.                     ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-#else
- 1000 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ERROR:      STOP AT THE INITIAL DATA SETUP              ',/,&
-'@    ======                                                  ',/,&
-'@     FIELD: ', a, 'HAS ALREADY BEEN DEFINED.                ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-#endif
-
-end subroutine add_property_field_hidden
-
-!===============================================================================
-
-!> \brief add field defining a property field defined on cells,
-!>        with default options
+!> \brief add field defining a one-dimensional property field defined on cells,
+!>        with no previous time values and with default options
 !
 !> It is recommended not to define property names of more than 16
 !> characters, to get a clear execution listing (some advanced writing
@@ -477,11 +354,11 @@ end subroutine add_property_field_hidden
 !______________________________________________________________________________!
 !> \param[in]     name          field name
 !> \param[in]     label         field default label, or empty
-!> \param[out]    iprop         matching field property id
+!> \param[out]    f_id          field id
 !_______________________________________________________________________________
 
-subroutine add_property_field &
- ( name, label, iprop )
+subroutine add_property_field_1d &
+ ( name, label, f_id )
 
 !===============================================================================
 ! Module files
@@ -500,87 +377,26 @@ implicit none
 ! Arguments
 
 character(len=*), intent(in) :: name, label
-integer, intent(out)         :: iprop
+integer, intent(out)         :: f_id
 
 ! Local variables
 
-integer  type_flag, location_id, f_id, dim
+integer  dim
 logical  has_previous
 
 !===============================================================================
 
-type_flag = FIELD_INTENSIVE + FIELD_PROPERTY
-location_id = 1 ! variables defined on cells
 has_previous = .false.
 dim = 1
 
-! Test if the field has already been defined
-call field_get_id_try(trim(name), f_id)
-if (f_id .ge. 0) then
-  write(nfecra,1000) trim(name)
-  call csexit (1)
-endif
+call add_property_field(name, label, dim, has_previous, f_id)
 
-! Create field
-
-call field_create(name, type_flag, location_id, dim, has_previous, f_id)
-
-call field_set_key_int(f_id, keyvis, 1)
 call field_set_key_int(f_id, keylog, 1)
-
-if (len(trim(label)).gt.0) then
-  call field_set_key_str(f_id, keylbl, trim(label))
-endif
-
-! Property number and mapping to field
-
-iprop = nproce + 1
-nproce = nproce + 1
-
-call fldprp_check_nproce
-
-iprpfl(iprop) = f_id
-ipproc(iprop) = iprop
-
-! Postprocessing slots
-
-ipppro(iprop) = field_post_id(f_id)
-
-call field_set_key_int(f_id, keyipp, ipppro(iprop))
+call field_set_key_int(f_id, keyvis, 1)
 
 return
 
-!---
-! Formats
-!---
-
-#if defined(_CS_LANG_FR)
- 1000 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ERREUR :    ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    ========                                                ',/,&
-'@     LE CHAMP : ', a, 'EST DEJA DEFINI.                     ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-#else
- 1000 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ERROR:      STOP AT THE INITIAL DATA SETUP              ',/,&
-'@    ======                                                  ',/,&
-'@     FIELD: ', a, 'HAS ALREADY BEEN DEFINED.                ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-#endif
-
-return
-
-end subroutine add_property_field
+end subroutine add_property_field_1d
 
 !===============================================================================
 
@@ -594,7 +410,7 @@ end subroutine add_property_field
 !> \param[in]     f_id          field id
 !_______________________________________________________________________________
 
-subroutine hide_property_field &
+subroutine hide_property &
  ( f_id )
 
 !===============================================================================
@@ -617,168 +433,18 @@ integer, intent(in) :: f_id
 
 ! Local variables
 
-integer  ipp, j
-
-integer :: f_dim
-
 !===============================================================================
 
 call field_set_key_int(f_id, keyvis, 0)
 call field_set_key_int(f_id, keylog, 0)
 
-ipp = field_post_id(f_id)
-
-if (ipp .gt. 1) then
-  call field_get_dim(f_id, f_dim)
-  do j = 1, f_dim
-    ihisvr(ipp+j-1,1) = 0
-  enddo
-endif
-
 return
 
-end subroutine hide_property_field
-
-!===============================================================================
-
-!> \brief disable logging and postprocessing for a property
-!
-!-------------------------------------------------------------------------------
-! Arguments
-!______________________________________________________________________________.
-!  mode           name          role                                           !
-!______________________________________________________________________________!
-!> \param[in]     iprop         property id
-!_______________________________________________________________________________
-
-subroutine hide_property &
- ( iprop )
-
-!===============================================================================
-! Module files
-!===============================================================================
-
-use paramx
-use dimens
-use entsor
-use numvar
-use field
-
-!===============================================================================
-
-implicit none
-
-! Arguments
-
-integer, intent(in) :: iprop
-
-! Local variables
-
-integer  f_id
-
-!===============================================================================
-
-f_id = iprpfl(iprop)
-call hide_property_field(f_id)
-
 end subroutine hide_property
 
 !===============================================================================
 
-!> \brief check npromx is sufficient for the required number of properties.
-
-!-------------------------------------------------------------------------------
-! Arguments
-!______________________________________________________________________________.
-!  mode           name          role                                           !
-!______________________________________________________________________________!
-!_______________________________________________________________________________
-
-subroutine fldprp_check_nproce
-
-!===============================================================================
-! Module files
-!===============================================================================
-
-use paramx
-use dimens
-use entsor
-use numvar
-
-!===============================================================================
-
-implicit none
-
-! Arguments
-
-! Local variables
-
-if (nproce .gt. npromx) then
-  write(nfecra,1000) nproce, npromx
-  call csexit (1)
-endif
-
-return
-
-!---
-! Formats
-!---
-
-#if defined(_CS_LANG_FR)
-
- 1000 format(                                                     &
-'@'                                                            ,/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@'                                                            ,/,&
-'@ @@ ERREUR :    ARRET A L''ENTREE DES DONNEES'               ,/,&
-'@    ========'                                                ,/,&
-'@     NOMBRE DE PROPRIETES TROP GRAND'                        ,/,&
-'@'                                                            ,/,&
-'@  Le type de calcul defini'                                  ,/,&
-'@    correspond a un nombre de proprietes NPROCE >= ', i10    ,/,&
-'@  Le nombre de proprietes maximal prevu'                     ,/,&
-'@                      dans paramx    est NPROMX  = ', i10    ,/,&
-'@'                                                            ,/,&
-'@  Le calcul ne sera pas execute.'                            ,/,&
-'@'                                                            ,/,&
-'@  Verifier les parametres'                                   ,/,&
-'@'                                                            ,/,&
-'@  Si NPROMX est augmente, le code doit etre reinstalle.'     ,/,&
-'@'                                                            ,/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@'                                                            ,/)
-
-#else
-
- 1000 format(                                                     &
-'@'                                                            ,/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@'                                                            ,/,&
-'@ @@ ERROR:      STOP AT THE INITIAL DATA SETUP'              ,/,&
-'@    ======'                                                  ,/,&
-'@     NUMBER OF PROPERTIES TOO LARGE'                         ,/,&
-'@'                                                            ,/,&
-'@  The type of calculation defined'                           ,/,&
-'@    corresponds to a number of properties NPROCE >= ', i10   ,/,&
-'@  The maximum number of properties allowed'                  ,/,&
-'@                      in   paramx     is  NPROMX  = ', i10   ,/,&
-'@'                                                            ,/,&
-'@  The calculation cannot be executed'                        ,/,&
-'@'                                                            ,/,&
-'@  Verify   parameters.'                                      ,/,&
-'@'                                                            ,/,&
-'@  If NVARMX is increased, the code must be reinstalled.'     ,/,&
-'@'                                                            ,/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@'                                                            ,/)
-
-#endif
-
-end subroutine fldprp_check_nproce
-
-!===============================================================================
-
-!> \brief add owner field defining a property field defined on cells,
+!> \brief add field defining a property field defined on cells,
 !>        with default options
 !
 !> It is recommended not to define property names of more than 16
@@ -798,7 +464,7 @@ end subroutine fldprp_check_nproce
 !> \param[out]    f_id          matching field id
 !_______________________________________________________________________________
 
-subroutine add_property_field_owner &
+subroutine add_property_field &
  ( name, label, dim, has_previous, f_id )
 
 !===============================================================================
@@ -809,6 +475,7 @@ use paramx
 use dimens
 use entsor
 use numvar
+use post
 use field
 
 !===============================================================================
@@ -824,11 +491,12 @@ integer, intent(out)         :: f_id
 
 ! Local variables
 
-integer  type_flag, location_id, ipp
+integer  type_flag, post_flag, location_id
 
 !===============================================================================
 
 type_flag = FIELD_INTENSIVE + FIELD_PROPERTY
+post_flag = POST_ON_LOCATION
 location_id = 1 ! variables defined on cells
 
 ! Test if the field has already been defined
@@ -842,12 +510,9 @@ endif
 
 call field_create(name, type_flag, location_id, dim, has_previous, f_id)
 
-call field_set_key_int(f_id, keyvis, 1)
+call field_set_key_int(f_id, keyvis, post_flag)
 call field_set_key_int(f_id, keylog, 1)
 
-ipp = field_post_id(f_id)
-call field_set_key_int(f_id, keyipp, ipp)
-
 if (len(trim(label)).gt.0) then
   call field_set_key_str(f_id, keylbl, trim(label))
 endif
@@ -882,7 +547,7 @@ return
 '@                                                            ',/)
 #endif
 
-end subroutine add_property_field_owner
+end subroutine add_property_field
 
 !===============================================================================
 
