@@ -3,7 +3,7 @@
 !     This file is part of the Code_Saturne Kernel, element of the
 !     Code_Saturne CFD tool.
 
-!     Copyright (C) 1998-2015 EDF S.A., France
+!     Copyright (C) 1998-2018 EDF S.A., France
 
 !     contact: saturne-support@edf.fr
 
@@ -25,11 +25,7 @@
 
 !-------------------------------------------------------------------------------
 
-subroutine fldtri &
-!================
-
- ( nproce ,                                                            &
-   dt     , propce )
+subroutine fldtri
 
 !===============================================================================
 ! Purpose:
@@ -42,9 +38,7 @@ subroutine fldtri &
 !__________________.____._____.________________________________________________.
 ! name             !type!mode ! role                                           !
 !__________________!____!_____!________________________________________________!
-! nproce           ! i  ! <-- ! nombre de prop phy aux centres                 !
 ! dt(ncelet)       ! ra ! <-- ! time step (per cell)                           !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
 !__________________.____._____.________________________________________________.
 
 !     Type: i (integer), r (real), s (string), a (array), l (logical),
@@ -69,14 +63,12 @@ use ppppar
 use ppthch
 use ppincl
 use cfpoin
-use lagpar
-use lagdim
 use lagran
 use ihmpre
 use cplsat
 use mesh
 use field
-use numvar
+use cs_c_bindings
 
 !===============================================================================
 
@@ -84,16 +76,12 @@ implicit none
 
 ! Arguments
 
-integer          nproce, nscal
-double precision dt(ncelet)
-double precision propce(ncelet,*)
+integer nscal
 
 ! Local variables
 
-logical          lprev
-integer          ii, ivar, iprop
-integer          iflid, nfld
-integer          ipcrom, ipcroa
+integer          ii, ivar
+integer          nfld
 integer          f_id
 
 integer          ifvar(nvarmx)
@@ -102,6 +90,8 @@ character(len=80) :: fname
 
 integer, save :: ipass = 0
 
+type(var_cal_opt) :: vcopt
+
 !===============================================================================
 
 
@@ -138,11 +128,11 @@ if (ipass.eq.1) then
   call field_init_bc_coeffs(ivarfl(ivar))
 endif
 
-! Void fraction for cavitation
-!-----------------------------
+! Void fraction for VOF algo.
+!----------------------------
 
-if (icavit.ge.0) then
-  ivar = ivoidf
+if (ivofmt.ge.0) then
+  ivar = ivolf2
   if (ipass .eq. 1) then
     call field_allocate_bc_coeffs(ivarfl(ivar), .true., .false., .false.)
     call field_init_bc_coeffs(ivarfl(ivar))
@@ -256,6 +246,18 @@ if (iale.eq.1) then
   endif
 endif
 
+! Wall distance
+!--------------
+
+call field_get_id_try("wall_distance", f_id)
+
+if (f_id.ne.-1) then
+  if (ipass .eq. 1) then
+    call field_allocate_bc_coeffs(f_id, .true., .false., .false.)
+    call field_init_bc_coeffs(f_id)
+  endif
+endif
+
 ! User variables
 !---------------
 
@@ -279,48 +281,29 @@ do ii = 1, nscal
         call field_allocate_bc_coeffs(f_id, .true., .true., .false.)
         call field_init_bc_coeffs(f_id)
       endif
+      ! Elliptic Blending (AFM or DFM)
+      if (iturt(ii).eq.11 .or. iturt(ii).eq.21 .or. iturt(ii).eq.31) then
+        call field_get_name(ivarfl(ivar), fname)
+        call field_get_id(trim(fname)//'_alpha', f_id)
+        call field_allocate_bc_coeffs(f_id, .true., .false., .false.)
+        call field_init_bc_coeffs(f_id)
+      endif
     endif
   endif
 enddo
 
-! Density field
-
-ipcrom = ipproc(irom)
-call field_have_previous(iprpfl(ipcrom), lprev)
-if (lprev) then
-  ipcroa = ipproc(iroma)
-  call field_map_values(iprpfl(ipcrom), propce(1, ipcrom), propce(1, ipcroa))
-else
-  ipcroa = -1
-  call field_map_values(iprpfl(ipcrom), propce(1, ipcrom), propce(1, ipcrom))
-endif
-
-! The choice made in VARPOS specifies that we will only be interested in
-! properties at cell centers (no mass flux, nor density at the boundary).
-
-f_id = -1
-do iprop = 1, nproce
-  if (iprop.eq.ipcrom .or. iprop.eq.ipcroa) cycle
-  if (iprpfl(iprop).ne.f_id) then
-    f_id = iprpfl(iprop)
-    call field_map_values(iprpfl(iprop), propce(1, iprop), propce(1, iprop))
-  endif
-enddo
-
 ! Reserved fields whose ids are not saved (may be queried by name)
 !-----------------------------------------------------------------
 
 ! Local time step
 
-call field_get_id('dt', iflid)
-call field_map_values(iflid, dt, dt)
-
 ! Set previous values for backward n order in time
 do ivar = 1, nvar
   ! Here there is no problem if there are multiple
   ! set on non scalar fields.
-  if (ibdtso(ivar).gt.1) then
-    call field_set_n_previous(ivarfl(ivar), ibdtso(ivar))
+  call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+  if (vcopt%ibdtso.gt.1) then
+    call field_set_n_previous(ivarfl(ivar), vcopt%ibdtso)
   endif
 enddo
 
