@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -29,7 +29,7 @@
 !------------------------------------------------------------------------------
 !   mode          name          role
 !------------------------------------------------------------------------------
-!> \param[out]    nmodpp        number of activated paricle physic models
+!> \param[out]    nmodpp        number of activated particle physic models
 !______________________________________________________________________________
 
 subroutine fldvar &
@@ -47,8 +47,6 @@ use cstphy
 use cstnum
 use entsor
 use albase
-use lagpar
-use lagdim
 use lagran
 use parall
 use ppppar
@@ -73,7 +71,9 @@ integer       nmodpp
 ! Local variables
 
 integer       ipp
-integer       iok   , ippok,  keycpl
+integer       iok   , keycpl, nmodpp_compatibility
+
+type(var_cal_opt) :: vcopt
 
 !===============================================================================
 ! Interfaces
@@ -96,9 +96,6 @@ end interface
 ! 0. INITIALISATIONS
 !===============================================================================
 
-! Initialize variables to avoid compiler warnings
-ippok = 0
-
 call field_get_key_id('coupled', keycpl)
 
 !===============================================================================
@@ -123,10 +120,21 @@ nmodpp = 0
 do ipp = 2, nmodmx
   if (ippmod(ipp).ne.-1) then
     nmodpp = nmodpp+1
-    ippok = ipp
+    if (ippmod(ipp).lt.0 .or. ippmod(ipp).gt.5) then
+      write(nfecra,6001)
+      iok = iok + 1
+    endif
   endif
 enddo
-if (nmodpp.gt.1.and.(ippmod(igmix).eq.-1.or.ippmod(icompf).eq.-1)) then
+
+nmodpp_compatibility = nmodpp
+
+! Compressible module and gas mix are compatible
+if (ippmod(igmix).ne.-1 .and. ippmod(icompf) .ne. -1) then
+  nmodpp_compatibility = nmodpp_compatibility - 1
+endif
+
+if (nmodpp_compatibility.gt.1) then
   write(nfecra,6000)
   iok = iok + 1
 endif
@@ -140,13 +148,6 @@ if (ippmod(igmix).ge.0.and.ippmod(icompf).ge.0.and.ieos.ne.3) then
   write(nfecra,6002)
 endif
 
-if (nmodpp.eq.1) then
-  if (ippmod(ippok).lt.0 .or. ippmod(ippok).gt.5) then
-    write(nfecra,6001)
-    iok = iok + 1
-  endif
-endif
-
 if (iok.ne.0) then
   call csexit (1)
   !==========
@@ -181,22 +182,34 @@ call field_set_key_int(ivarfl(iu), keycpl, 1)
 iv = iu + 1
 iw = iv + 1
 
-! Pressure
+! Pressure or hydraulic head for groundwater flow module
 
-call add_variable_field('pressure', 'Pressure', 1, ipr)
-
-if (ippmod(icompf).ge.0) then
-  istat(ipr) = 1
+if (ippmod(idarcy).eq.-1) then
+  call add_variable_field('pressure', 'Pressure', 1, ipr)
 else
-  istat (ipr) = 0
+  call add_variable_field('hydraulic_head', 'Hydraulic head', 1, ipr)
 endif
-iconv (ipr) = 0
 
-! Void fraction (cavitating flows)
+! Cavitation: activate VoF
+if (icavit.ge.0.and.ivofmt.lt.0) ivofmt = 0
 
-if (icavit.ge.0) then
-  call add_variable_field('void_fraction', 'Void Fraction', 1, ivoidf)
-  idiff(ivoidf) = 0
+call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
+if (ippmod(icompf).ge.0) then
+  vcopt%istat = 1
+else
+  vcopt%istat = 0
+endif
+! VoF algorithm: activate the weighthening for the pressure
+if (ivofmt.ge.0) vcopt%iwgrec = 1
+vcopt%iconv = 0
+call field_set_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
+
+! Void fraction (Volume of Fluid algorithm)
+if (ivofmt.ge.0) then
+  call add_variable_field('void_fraction', 'Void Fraction', 1, ivolf2)
+  call field_get_key_struct_var_cal_opt(ivarfl(ivolf2), vcopt)
+  vcopt%idiff = 0
+  call field_set_key_struct_var_cal_opt(ivarfl(ivolf2), vcopt)
 endif
 
 ! --- Turbulence
@@ -228,8 +241,11 @@ else if (itytur.eq.3) then
   call add_variable_field('epsilon', 'Turb Dissipation', 1, iep)
   if (iturb.eq.32) then
     call add_variable_field('alpha', 'Alphap', 1, ial)
-    istat(ial)  = 0
-    iconv(ial)  = 0
+    ! Elliptic equation (no convection, no time term)
+    call field_get_key_struct_var_cal_opt(ivarfl(ial), vcopt)
+    vcopt%istat = 0
+    vcopt%iconv = 0
+    call field_set_key_struct_var_cal_opt(ivarfl(ial), vcopt)
     ! For alpha, we always have a diagonal term, so do not shift the diagonal
     idircl(ial) = 0
   endif
@@ -239,14 +255,18 @@ else if (itytur.eq.5) then
   call add_variable_field('phi', 'Phi', 1, iphi)
   if (iturb.eq.50) then
     call add_variable_field('f_bar', 'f_bar', 1, ifb)
-    istat(ifb)  = 0
-    iconv(ifb)  = 0
+    call field_get_key_struct_var_cal_opt(ivarfl(ifb), vcopt)
+    vcopt%istat = 0
+    vcopt%iconv = 0
+    call field_set_key_struct_var_cal_opt(ivarfl(ifb), vcopt)
     ! For fb, we always have a diagonal term, so do not shift the diagonal
     idircl(ifb) = 0
   else if (iturb.eq.51) then
     call add_variable_field('alpha', 'Alpha', 1, ial)
-    istat(ial)  = 0
-    iconv(ial)  = 0
+    call field_get_key_struct_var_cal_opt(ivarfl(ial), vcopt)
+    vcopt%istat = 0
+    vcopt%iconv = 0
+    call field_set_key_struct_var_cal_opt(ivarfl(ial), vcopt)
     ! For alpha, we always have a diagonal term, so do not shift the diagonal
     idircl(ial) = 0
   endif
@@ -264,16 +284,13 @@ if (iale.eq.1) then
   call add_variable_field('mesh_velocity', 'Mesh Velocity', 3, iuma)
   call field_set_key_int(ivarfl(iuma), keycpl, 1)
 
-  istat(iuma) = 0
-  iconv(iuma) = 0
-
-  ! All components point to same field
   ivma = iuma + 1
   iwma = ivma + 1
-  istat(ivma) = istat(iuma)
-  iconv(ivma) = iconv(iuma)
-  istat(iwma) = istat(iuma)
-  iconv(iwma) = iconv(iuma)
+
+  call field_get_key_struct_var_cal_opt(ivarfl(iuma), vcopt)
+  vcopt%istat = 0
+  vcopt%iconv = 0
+  call field_set_key_struct_var_cal_opt(ivarfl(iuma), vcopt)
 
 endif
 
@@ -641,6 +658,7 @@ use dimens
 use entsor
 use numvar
 use field
+use cs_c_bindings
 
 !===============================================================================
 
@@ -654,42 +672,21 @@ integer, intent(out)         :: ivar
 
 ! Local variables
 
-integer  id, ii, ipp
-integer  type_flag, location_id
-logical  interleaved, has_previous
+integer  id, ii
+integer  location_id
 
-integer, save :: keycpl = -1
 integer, save :: keyvar = -1
 
-type_flag = FIELD_INTENSIVE + FIELD_VARIABLE
 location_id = 1         ! variables defined on cells
-interleaved = .true.
-has_previous = .true.
-
-! Test if the field has already been defined
-call field_get_id_try(trim(name), id)
-if (id .ge. 0) then
-  write(nfecra,1000) trim(name)
-  call csexit (1)
-endif
 
 ! Create field
 
+call variable_field_create(name, label, location_id, dim, id)
+
 if (keyvar.lt.0) then
-  call field_get_key_id('coupled', keycpl)
   call field_get_key_id("variable_id", keyvar)
 endif
 
-call field_create(name, type_flag, location_id, dim, interleaved, has_previous, &
-                  id)
-
-call field_set_key_int(id, keyvis, 1)
-call field_set_key_int(id, keylog, 1)
-
-if (len(trim(label)).gt.0) then
-  call field_set_key_str(id, keylbl, trim(label))
-endif
-
 ivar = nvar + 1
 nvar = nvar + dim
 
@@ -697,12 +694,14 @@ nvar = nvar + dim
 call fldvar_check_nvar
 
 ivarfl(ivar) = id
-ipp = field_post_id(id)
 
 call field_set_key_int(id, keyvar, ivar)
 
+call init_var_cal_opt(id)
+
+call field_set_key_double(id, ksigmas, 1.d0)
+
 if (dim .gt. 1) then
-  call field_set_key_int(id, keycpl, 1)
   do ii = 2, dim
     ivarfl(ivar + ii - 1) = id
   enddo
@@ -710,34 +709,6 @@ endif
 
 return
 
-!---
-! Formats
-!---
-
-#if defined(_CS_LANG_FR)
- 1000 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ERREUR :    ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    ========                                                ',/,&
-'@     LE CHAMP : ', a, 'EST DEJA DEFINI.                     ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-#else
- 1000 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ERROR:      STOP AT THE INITIAL DATA SETUP              ',/,&
-'@    ======                                                  ',/,&
-'@     FIELD: ', a, 'HAS ALREADY BEEN DEFINED.                ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-#endif
-
 end subroutine add_variable_field
 
 !===============================================================================
@@ -773,8 +744,7 @@ implicit none
 ! Local variables
 
 integer  iscal, nfld1, nfld2
-integer  dim, id, ipp
-logical  interleaved
+integer  dim, id, ii, ivar, keycpl
 
 integer :: keyvar, keysca
 
@@ -808,29 +778,42 @@ call field_get_n_fields(nfld2)
 
 iscal = 0
 
+call field_get_key_id('coupled', keycpl)
 call field_get_key_id("scalar_id", keysca)
 call field_get_key_id("variable_id", keyvar)
 
 do id = nfld1, nfld2 - 1
 
-  call field_get_dim(id, dim, interleaved)
+  call field_get_dim(id, dim)
 
-  if (dim.ne.1) cycle ! fields of dimension > 1 may not be handled as scalars
+  if (dim.eq.3) then
+    call field_set_key_int(id, keycpl, 1)
+  else if (dim.ne.1) then
+    cycle
+  endif
 
   iscal = iscal + 1
 
-  nvar = nvar + 1
+  ivar = nvar + 1
+  nvar = nvar + dim
   nscal = nscal + 1
 
   ! Check we have enough slots
   call fldvar_check_nvar
 
-  isca(iscal) = nvar
-  ivarfl(nvar) = id
-  ipp = field_post_id(id)
+  isca(iscal) = ivar
+  ivarfl(ivar) = id
 
-  call field_set_key_int(id, keyvar, nvar)
+  call field_set_key_int(id, keyvar, ivar)
   call field_set_key_int(id, keysca, iscal)
+  call field_set_key_double(id, ksigmas, 1.d0)
+  call init_var_cal_opt(id)
+
+  if (dim .gt. 1) then
+    do ii = 2, dim
+      ivarfl(ivar + ii - 1) = id
+    enddo
+  endif
 
 enddo
 
@@ -858,7 +841,7 @@ end subroutine add_user_scalar_fields
 !______________________________________________________________________________!
 !> \param[in]  name           field name
 !> \param[in]  label          field default label, or empty
-!> \param[out] iscal          variable number for defined field
+!> \param[out] iscal          scalar number for defined field
 !_______________________________________________________________________________
 
 subroutine add_model_scalar_field &
@@ -885,95 +868,19 @@ integer, intent(out)         :: iscal
 
 ! Local variables
 
-integer  dim, id, ipp
-integer  type_flag, location_id
-logical  interleaved, has_previous
+integer  dim
 
-integer, save :: keyvar = -1
-integer, save :: keysca = -1
-
-type_flag = FIELD_INTENSIVE + FIELD_VARIABLE
 dim = 1
-location_id = 1 ! variables defined on cells
-interleaved = .true.
-has_previous = .true.
 
-! Test if the field has already been defined
-call field_get_id_try(trim(name), id)
-if (id .ge. 0) then
-  write(nfecra,1000) trim(name)
-  call csexit (1)
-endif
-
-! Create field
-
-if (keysca.lt.0) then
-  call field_get_key_id("scalar_id", keysca)
-  call field_get_key_id("variable_id", keyvar)
-endif
-
-call field_create(name, type_flag, location_id, dim, interleaved, has_previous, &
-                  id)
-
-call field_set_key_int(id, keyvis, 1)
-call field_set_key_int(id, keylog, 1)
-
-if (len(trim(label)).gt.0) then
-  call field_set_key_str(id, keylbl, trim(label))
-endif
-
-nvar = nvar + 1
-nscal = nscal + 1
-nscapp = nscapp + 1
-iscal = nscaus + nscapp
-
-! Check we have enough slots
-call fldvar_check_nvar
-call fldvar_check_nscapp
-
-isca(iscal) = nvar
-iscapp(nscapp) = iscal
-ivarfl(isca(iscal)) = id
-ipp = field_post_id(id)
-
-call field_set_key_int(id, keyvar, nvar)
-call field_set_key_int(id, keysca, iscal)
+call add_model_field(name, label, dim, iscal)
 
 return
 
-!---
-! Formats
-!---
-
-#if defined(_CS_LANG_FR)
- 1000 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ERREUR :    ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    ========                                                ',/,&
-'@     LE CHAMP : ', a, 'EST DEJA DEFINI.                     ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-#else
- 1000 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ERROR:      STOP AT THE INITIAL DATA SETUP              ',/,&
-'@    ======                                                  ',/,&
-'@     FIELD: ', a, 'HAS ALREADY BEEN DEFINED.                ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-#endif
-
 end subroutine add_model_scalar_field
 
 !===============================================================================
 !
-!> \brief add field defining a non-user solved scalar variable,
+!> \brief add field defining a non-user solved variable,
 !>        with default options
 !
 !> It is recommended not to define variable names of more than 16
@@ -1003,6 +910,7 @@ use dimens
 use entsor
 use numvar
 use field
+use cs_c_bindings
 
 !===============================================================================
 
@@ -1016,99 +924,98 @@ integer, intent(out)         :: iscal
 
 ! Local variables
 
-integer  ivar, id, ii, ipp
-integer  type_flag, location_id,  keycpl
-logical  interleaved, has_previous
+integer  id
+integer  location_id
+
+location_id = 1 ! variables defined on cells
+
+! Create field
+
+call variable_field_create(name, label, location_id, dim, id)
+
+call add_model_field_indexes(id, iscal)
+
+return
+
+end subroutine add_model_field
+
+!===============================================================================
+!
+!> \brief add field indexes associated with a new non-user solved
+!>        scalar variable, with default options
+!
+!-------------------------------------------------------------------------------
+! Arguments
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]  f_id           field id
+!> \param[out] iscal          variable number for defined field
+!_______________________________________________________________________________
+
+subroutine add_model_field_indexes &
+ ( f_id, iscal )
+
+!===============================================================================
+! Module files
+!===============================================================================
+
+use paramx
+use dimens
+use entsor
+use numvar
+use field
+
+!===============================================================================
+
+implicit none
+
+! Arguments
+
+integer, intent(in)  :: f_id
+integer, intent(out) :: iscal
+
+! Local variables
+
+integer  dim, ivar, ii
 
 integer, save :: keyvar = -1
 integer, save :: keysca = -1
 
-type_flag = FIELD_INTENSIVE + FIELD_VARIABLE
-location_id = 1 ! variables defined on cells
-interleaved = .true.
-has_previous = .true.
+! Get field dimension
 
-! Test if the field has already been defined
-call field_get_id_try(trim(name), id)
-if (id .ge. 0) then
-  write(nfecra,1000) trim(name)
-  call csexit (1)
-endif
-
-! Create field
+call field_get_dim(f_id, dim)
 
 if (keysca.lt.0) then
-  call field_get_key_id('coupled', keycpl)
   call field_get_key_id("scalar_id", keysca)
   call field_get_key_id("variable_id", keyvar)
 endif
 
-call field_create(name, type_flag, location_id, dim, interleaved, has_previous, &
-                  id)
-
-call field_set_key_int(id, keyvis, 1)
-call field_set_key_int(id, keylog, 1)
-
-if (len(trim(label)).gt.0) then
-  call field_set_key_str(id, keylbl, trim(label))
-endif
-
 ivar = nvar + 1
 nvar = nvar + dim
-nscal = nscal + dim
+nscal = nscal + 1
 iscal = nscaus + nscapp + 1
-nscapp = nscapp + dim
+nscapp = nscapp + 1
 
 ! Check we have enough slots
 call fldvar_check_nvar
 call fldvar_check_nscapp
 
+isca(iscal) = ivar
+iscapp(nscapp) = iscal
+
 do ii = 1, dim
-  isca(iscal + ii - 1) =  nvar - dim + ii
-  ivarfl(isca(iscal + ii - 1)) = id
-  iscapp(nscapp - dim + ii) = iscal + ii - 1
+  ivarfl(ivar + ii - 1) = f_id
 enddo
 
-ipp = field_post_id(id)
-
-call field_set_key_int(id, keyvar, nvar)
-call field_set_key_int(id, keysca, iscal)
-
-if (dim .gt. 1) then
-  call field_set_key_int(id, keycpl, 1)
-endif
+call field_set_key_int(f_id, keyvar, ivar)
+call field_set_key_int(f_id, keysca, iscal)
+call field_set_key_double(f_id, ksigmas, 1.d0)
+call init_var_cal_opt(f_id)
 
 return
 
-!---
-! Formats
-!---
-
-#if defined(_CS_LANG_FR)
- 1000 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ERREUR :    ARRET A L''ENTREE DES DONNEES               ',/,&
-'@    ========                                                ',/,&
-'@     LE CHAMP : ', a, 'EST DEJA DEFINI.                     ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-#else
- 1000 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ ERROR:      STOP AT THE INITIAL DATA SETUP              ',/,&
-'@    ======                                                  ',/,&
-'@     FIELD: ', a, 'HAS ALREADY BEEN DEFINED.                ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
-#endif
-
-end subroutine add_model_field
+end subroutine add_model_field_indexes
 
 !===============================================================================
 
@@ -1205,6 +1112,75 @@ end subroutine fldvar_check_nvar
 
 !===============================================================================
 
+!> \brief Initialize the given variable calculation option structure with
+!>        legacy values (iniini) allowing to later test user modification.
+
+!-------------------------------------------------------------------------------
+! Arguments
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!_______________________________________________________________________________
+
+subroutine init_var_cal_opt &
+ ( id )
+
+!===============================================================================
+! Module files
+!===============================================================================
+
+use paramx
+use dimens
+use entsor
+use numvar
+use cs_c_bindings
+use field
+use optcal
+
+!===============================================================================
+
+implicit none
+
+! Arguments
+integer id
+
+! Local variables
+type(var_cal_opt) :: vcopt
+
+call field_get_key_struct_var_cal_opt(id, vcopt)
+vcopt%iwarni = 0
+vcopt%iconv  = 1
+vcopt%istat  = 1
+vcopt%idiff  = 1
+vcopt%idifft = 1
+vcopt%idften = ISOTROPIC_DIFFUSION
+vcopt%iswdyn = 0
+vcopt%ischcv = 1
+vcopt%ibdtso = 1
+vcopt%isstpc = -999
+vcopt%nswrgr = 100
+vcopt%nswrsm = -999
+vcopt%imrgra = 0
+vcopt%imligr = -999
+vcopt%ircflu = 1
+vcopt%iwgrec = 0
+vcopt%thetav = -999.d0
+vcopt%blencv = -999.d0
+vcopt%blend_st = 0.d0
+vcopt%epsilo = -999.d0
+vcopt%epsrsm = -999.d0
+vcopt%epsrgr = 1.d-5
+vcopt%climgr = 1.5d0
+vcopt%extrag = 0.d0
+vcopt%relaxv = -999.d0
+call field_set_key_struct_var_cal_opt(id, vcopt)
+
+return
+
+end subroutine init_var_cal_opt
+
+!===============================================================================
+
 !> \brief Check nscamx is sufficient for the required number of model scalars.
 !
 !-------------------------------------------------------------------------------
@@ -1300,3 +1276,41 @@ return
 end subroutine fldvar_check_nscapp
 
 !===============================================================================
+! C bindings (reverse)
+!===============================================================================
+
+!-------------------------------------------------------------------------------
+!> \brief add field indexes associated with a new non-user solved
+!>        variable, with default options
+!
+!> \param[in]  f_id    field id
+
+!> \result             scalar number for defined field
+!-------------------------------------------------------------------------------
+
+function cs_c_add_model_field_indexes(f_id) result(iscal) &
+  bind(C, name='cs_add_model_field_indexes')
+
+  use, intrinsic :: iso_c_binding
+  use cs_c_bindings
+
+  implicit none
+
+  ! Arguments
+
+  integer(c_int), value :: f_id
+  integer(c_int) :: iscal
+
+  ! Local variables
+
+  integer f_id0, iscal0
+
+  f_id0 = f_id
+
+  call add_model_field_indexes(f_id0, iscal0)
+
+  iscal = iscal0
+
+end function cs_c_add_model_field_indexes
+
+!---------------------------------------------------------------------------
