@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -51,10 +51,13 @@ use ppincl
 use ppcpfu
 use mesh
 use field
+use vof
 use cavitation
 use darcy_module
 use radiat
+use turbomachinery
 use cs_nz_condensation, only: nzones
+use ctincl
 
 !===============================================================================
 
@@ -62,7 +65,7 @@ implicit none
 
 ! Local variables
 
-integer          ii, jj, iscal, iprop, iest
+integer          ii, jj, iscal, iest
 integer          istr
 
 !===============================================================================
@@ -110,7 +113,6 @@ write(nfecra, 900)
 call field_get_key_id("label", keylbl)
 call field_get_key_id('log', keylog)
 call field_get_key_id('post_vis', keyvis)
-call field_get_key_id("post_id", keyipp)
 
 call field_get_key_id("inner_mass_flux_id", kimasf)
 call field_get_key_id("boundary_mass_flux_id", kbmasf)
@@ -118,11 +120,15 @@ call field_get_key_id("boundary_mass_flux_id", kbmasf)
 call field_get_key_id("scalar_diffusivity_id", kivisl)
 call field_get_key_id("scalar_diffusivity_ref", kvisl0)
 
+call field_get_key_id("scalar_density_id", kromsl)
+
 call field_get_key_id("gradient_weighting_id", kwgrec)
 
 call field_get_key_id("source_term_prev_id", kstprv)
 call field_get_key_id("source_term_id", kst)
 
+call field_get_key_id("turbulent_schmidt", ksigmas)
+
 icrom = -1
 ibrom = -1
 
@@ -139,6 +145,7 @@ call thermal_model_init
 call turb_model_init
 call turb_rans_model_init
 call turb_les_model_init
+call turb_model_constants_init
 call wall_functions_init
 call stokes_options_init
 call physical_constants_init
@@ -148,17 +155,15 @@ call piso_options_init
 call turb_reference_values_init
 call listing_writing_period_init
 call radiat_init
+call gas_mix_options_init
+call ctwr_properties_init
+
+call map_turbomachinery_model(iturbo, ityint)
 
 !===============================================================================
 ! 2. ENTREES SORTIES entsor.f90
 !===============================================================================
 
-! ---> Impressions standard
-!         -10000 : non initialise : voir modini
-do ii = 1, nvarmx
-  iwarni(ii) = -10000
-enddo
-
 ! ---> NFECRA vaut 6 par defaut ou 9 en parallele (CSINIT)
 
 !    Methode des vortex : on utilise la meme unite
@@ -205,8 +210,6 @@ ficmet = 'meteo'
 !     EMPHIS : EMPlacement
 !     PREHIS : PREfixe
 !     EXTHIS : EXTension
-!     IMPUSH : Unite fichiers specifiques ushist
-!     FICUSH : Nom   fichiers specifiques ushist
 !     IMPSTH : fichier stock + unite d'ecriture des variables
 !              des structures mobiles
 
@@ -216,90 +219,36 @@ impsth(2) = 31
 emphis = 'monitoring/'
 prehis = 'probes_'
 
-do ii = 1, nushmx
-  impush(ii) = 32+ii
-  if (irangp .le. 0) then
-    write(ficush(ii),'(1a3,i3.3)')'ush',ii
-  else
-    write(ficush(ii),'(1a3,i3.3,1a3,i4.4)')'ush',ii,'.n_',irangp+1
-  endif
-enddo
-
 ! tplfmt : time plot format (1: .dat, 2: .csv, 3: both)
-! ncapt  : nombre de sondes total (limite a ncaptm)
 ! nthist : periode de sortie (> 0 ou -1 (jamais))
 ! frhist : frequence de sortie, en secondes (prioritaire sur nthist si > 0)
-! ihisvr : nb de sonde et numero par variable (-999 non initialise)
 ! ihistr : indicateur d'ecriture des historiques des structures
 !          mobiles internes (=0 ou 1)
-! ncapt  : nombre de sondes total (limite a ncaptm)
-! nodcap : element correspondant aux sondes
-! ndrcap : rang du processus contenant nodcap (parallelisme)
-! xyzcap : position demandee des sondes
 
 tplfmt = 1
-ncapt = 0
 
 nthist = 1
 frhist = -1.d0
 
-do ii = 1, nvppmx
-  do jj = 1, ncaptm+1
-    ihisvr(ii ,jj) = -999
-  enddo
-enddo
-
 ihistr = 0
 
-do ii = 1, ncaptm
-  nodcap(ii) = 1
-  ndrcap(ii) = -1
-enddo
-
-do ii = 1, ncaptm
-  xyzcap(1,ii) = 0.d0
-  xyzcap(2,ii) = 0.d0
-  xyzcap(3,ii) = 0.d0
-enddo
-
 ! ---> Fichiers utilisateurs
 
 do ii = 1, nusrmx
   impusr(ii) = 69+ii
-  if (irangp .le. 0) then
-    write(ficusr(ii),'(1a4,i2.2)')'usrf',ii
-  else
-    write(ficusr(ii),'(1a4,i2.2,1a3,i4.4)')'usrf',ii,'.n_',irangp+1
-  endif
 enddo
 
 ! ---> Sorties listing
 
-!   COMMUNES
-!     IPP*   : Pointeurs de reperage des variables pour les sorties
-!              1 pointe sur une case poubelle et constitue donc une
-!              bonne initialisation
-!     NOMVAR : Nom des variables
-!     ITRSVR : numero de variable si IPP correspond a une variable resolue (p,u,k...)
-!              0 si IPP correspond a une variable annexe (cp, mut...)ou a rien
-!     NTLIST : periode d'ecriture
-!       ( -1 : dernier pas de temps : > 0 : periode)
-
-do ii = 1, npromx
-  ipppro(ii) = 1
-enddo
-ippdt        = 1
-ipptx        = 1
-ippty        = 1
-ipptz        = 1
-
 ntlist = 1
 
 ! ---> Post traitement automatique (bord)
 
-do ii = 1, 5
-  ipstdv(ii) = 0
-enddo
+ipstdv(ipstfo) = 0 ! should be 1 by default for Navier-Stokes cases
+ipstdv(ipstyp) = 1
+ipstdv(ipsttp) = 0
+ipstdv(ipstft) = 1
+ipstdv(ipstnu) = 0
 
 ! ---> CPU
 !      TMARUS : marge (Arret du calcul avant limite CPU)
@@ -355,7 +304,6 @@ nscaus = 0
 nscapp = 0
 nscasp = 0
 nvar   = 0
-nproce = 0
 
 !===============================================================================
 ! 5. POSITION DES VARIABLES DE numvar.f90
@@ -367,10 +315,6 @@ do ii = 1, nvarmx
   ivarfl(ii) = -1
 enddo
 
-do ii = 1, npromx
-  iprpfl(ii) = -1
-enddo
-
 idtten = -1
 
 ! --- Variables de calcul resolues (RTP, RTPA)
@@ -405,22 +349,16 @@ call ppinii
 !==========
 
 ! --- Proprietes physiques au sens large
+irom   = -1
+iviscl = -1
+ivisct = -1
+icour  = -1
+ifour  = -1
+icp    = -1
+iprtot = -1
 
-do iprop  = 1, npromx
-  ipproc(iprop) = iprop
-enddo
-
-irom   = 0
-iroma  = 0
-iviscl = 0
-ivisct = 0
-icour  = 0
-ifour  = 0
-icp    = 0
-iprtot = 0
-
-itemp  = 0
-ibeta  = 0
+itemp  = -1
+ibeta  = -1
 
 iforbr = -1
 iyplbr = -1
@@ -432,21 +370,6 @@ itempb = -1
 ! 6. OPTIONS DU CALCUL : TABLEAUX DE optcal.f90
 !===============================================================================
 
-! --- Definition des equations
-!       (convection-diffusion instationnaire, avec dirichlet
-!        sauf pour la pression, diffusion instationnaire)
-!        IDIFFT en particulier multiplie la diffusion turbulente
-!           (quand elle est activee par le modele)
-
-do ii = 1, nvarmx
-  iconv (ii) = 1
-  istat (ii) = 1
-  idiff (ii) = 1
-  idifft(ii) = 1
-  idften(ii) = 1
-  iswdyn(ii) = 0
-enddo
-
 ! --- Schema en temps
 
 !     NOTER BIEN que les valeurs de THETA pour
@@ -464,18 +387,6 @@ enddo
 !     pour definir des schemas coherents dans modini.
 ischtp = -999
 
-!   -- Variables
-!     Pour un schema centre (en n+1/2) il faut prendre theta = 0.5
-!     On devrait pouvoir faire de l'explicite pur avec theta = 0 mais
-!       ce point reste a voir
-!     Ce theta sert aux termes de convection diffusion (partie implicitee
-!       d'ordinaire)
-!     Il est applique sous la forme (1-theta) ancien + theta nouveau
-!       (ce n'est pas une extrapolation, contrairement aux termes sources)
-do ii = 1, nvarmx
-  thetav(ii) =-999.d0
-enddo
-
 !   -- Flux de masse (-999 = non initialise)
 !     = 1 Standard d'ordre 1 (THETFL = -999 inutile)
 !     = 0 Explicite (THETFL = 0)
@@ -518,11 +429,6 @@ thetsn =-999.d0
 isto2t = -999
 thetst = -999.d0
 
-! Backward differential time scheme order
-do ii = 1, nvarmx
-  ibdtso(ii) = 1
-enddo
-
 !    -- Proprietes physiques
 !     I..EXT definit l'extrapolation -theta ancien + (1+theta) nouveau
 !     = 0 explicite
@@ -594,19 +500,6 @@ do iscal = 1, nscamx
 
 enddo
 
-
-
-! --- Schema convectif
-!       (a decider, centre-upwind si ordre 2,  test de pente a decider)
-
-do ii = 1, nvarmx
-  blencv(ii) = -999.d0
-enddo
-do ii = 1, nvarmx
-  ischcv(ii) = 1
-  isstpc(ii) = -999
-enddo
-
 ! Method to compute interior mass flux due to ALE mesh velocity
 ! default: based on cell center mesh velocity
 iflxmw = 0
@@ -618,22 +511,8 @@ iflxmw = 0
 !       On n'active pas l'extrapolation des gradients par defaut
 !         meme pour la pression (par securite : moins stable sur certains cas)
 
-imrgra = 0
 anomax = -grand*10.d0
 
-do ii = 1, nvarmx
-  nswrgr(ii) = 100
-  nswrsm(ii) = -999
-  imligr(ii) = -999
-  ircflu(ii) = 1
-enddo
-
-do ii = 1, nvarmx
-  epsrgr(ii) = 1.d-5
-  climgr(ii) = 1.5d0
-  extrag(ii) = 0.d0
-enddo
-
 ! --- Solveurs iteratifs
 !       La methode de resolution sera choisie selon les equations
 !       La valeur de epsilon relatif est tres faible
@@ -648,8 +527,6 @@ enddo
 do ii = 1, nvarmx
   idircl(ii) = 1
   ndircl(ii) = 0
-  epsilo(ii) = -999.d0
-  epsrsm(ii) = -999.d0
 enddo
 
 ! --- Restarted calculation
@@ -707,7 +584,6 @@ dtref  = -grand*10.d0
 
 do ii = 1, nvarmx
   cdtvar(ii) = 1.d0
-  relaxv(ii) =-999.d0
 enddo
 relxst = 0.7d0
 
@@ -734,6 +610,7 @@ ihgas = -1
 
 iturb  =-999
 itytur =-999
+iddes  = 0
 
 ! Parfois, IGRHOK=1 donne des vecteurs non physiques en paroi
 !        IGRHOK = 1
@@ -744,7 +621,6 @@ iwalfs = -999
 iwallt = 0
 iclkep = 0
 ikecou =-999
-irijco = 0
 irijnu = 0
 irijrb = 0
 irijec = 0
@@ -776,6 +652,9 @@ iporos = 0
 
 ipthrm = 0
 
+! Call predfl subroutine?
+ipredfl = 0
+
 !     by default:
 !     ----------
 !      - the thermodynamic pressure (pther) is initialized with p0 = p_atmos
@@ -791,6 +670,11 @@ pthermax= -1.d0
 sleak = 0.d0
 kleak = 2.9d0
 
+! --- VOF module (not activated by default)
+!       -1: module not activated
+!        0: module activated
+ivofmt = -1
+
 ! --- Cavitation module (not activated by default)
 !       -1: module not activated
 !        0: no vaporization/condensation model
@@ -834,14 +718,6 @@ itagms = 0
 
 imvisf = 0
 
-! --- Gradient calculation
-!     = 0 Standard
-!     = 1 Weighted (shoulf be used with imvisf = 1)
-
-do ii = 1, nvarmx
-  iwgrec(ii) = 0
-enddo
-
 ! --- Type des CL, tables de tri
 !       Sera calcule apres cs_user_boundary_conditions.
 
@@ -850,12 +726,8 @@ do ii = 1, ntypmx
   ifinty(ii) = 0
 enddo
 
-! --- Traitement de la temperature pour couplage SYRTHES
-
-do iscal = 1, nscamx
-  icpsyr(iscal) = -999
-enddo
-
+ifrslb = 0
+itbslb = 0
 
 ! --- Estimateurs d'erreur pour Navier-Stokes
 !       En attendant un retour d'experience et pour l'avoir,
@@ -917,10 +789,6 @@ nctsmt = 0
 
 nftcdt= 0
 
-! --- Somme de NFPT1D (pour les calculs paralleles)
-
-nfpt1t = 0
-
 !     Non utilisateur
 
 ! --- Calcul de la distance a la paroi
@@ -969,38 +837,10 @@ gz = 0.d0
 
 icorio = 0
 
-! --- Constantes physiques de chaque phase
-!       RO0,VISCL0 et CP0 devront etre initialises par l'utilisateur
-!       P0 est donne par phase, mais seul P0(1) est utilise, idem
-!        pour PRED0 et XYZREF
-!       T0 ne sert a rien, sauf a etre dispo pour l'utilisateur
+! --- Physical constants initialized in C (cs_physical_constants.c)
 
-!       IROVAR : en attendant mieux, IROVAR indique si rho est constant
-!         (et c'est donc RO0) : utilise uniquement pour les suites de
-!         calcul, il indique qu'il ne faut pas relire la valeur de rho
-!         qui a ete stockee dans le fichier suite. Ceci permet de ne pas
-!         ecraser la valeur de rho0 (eventuellement modifiee par
-!         l'utilisateur) par la valeur de l'ancien calcul.
-!         Le pb ne se pose pas pour la viscosite car elle n'est relue
-!         que si elle est extrapolee et elle est alors forcement variable
-!         (du moins, on l'espere...) : on adopte la meme methode pour la
-!         symetrie.
-
-irovar = 0
-ivivar = 0
-ro0    = 1.17862d0
-roref  = ro0
-viscl0 = 1.83337d-5
-p0     = 1.01325d5
 ! Reset pther to p0
 pther = p0
-pred0  = 0.d0
-xyzp0(1)= -rinfin
-xyzp0(2)= -rinfin
-xyzp0(3)= -rinfin
-ixyzp0 = -1
-t0 = 20.d0 + 273.15d0
-cp0    = 1017.24d0
 
 ! --- Turbulence
 !     YPLULI est mis a -GRAND*10. Si l'utilisateur ne l'a pas specifie dans usipsu, on
@@ -1015,22 +855,16 @@ bpow    = 1.d0/7.d0
 cpow    = apow**(2.d0/(1.d0-bpow))
 dpow    = 1.d0/(1.d0+bpow)
 
-cmu     = 0.09d0
-cmu025  = cmu**0.25d0
-
 !   pour le k-epsilon
 ce1     = 1.44d0
 ce2     = 1.92d0
 ce4     = 1.20d0
 sigmak  = 1.00d0
-! sigmae is set to 1.30d0 in modini
 
 !   pour le Rij-epsilon standard (et SSG pour CRIJ3)
 crij1  = 1.80d0
 crij2  = 0.60d0
 crij3  = 0.55d0
-! csrij is set to 0.22d0 in modini
-! sigmae is fixed to csrij/0.18d0 in modini
 crijp1 = 0.50d0
 crijp2 = 0.30d0
 
@@ -1054,11 +888,9 @@ cebmr4  = 0.625d0
 cebmr5  = 0.20d0
 ! cebmr6  is used in the buoyant term
 cebmr6  = 0.6d0
-! csrij is set to 0.21d0 in modini
 cebme2  = 1.83d0
 cebmmu  = 0.22d0
 xcl     = 0.122d0
-! sigmae is fixed to 1.15d0 in modini
 xa1     = 0.1d0
 xceta   = 80.d0
 xct     = 6.d0
@@ -1070,7 +902,8 @@ bles   = 1.d0/3.d0
 csmago = 0.065d0
 cwale  = 0.25d0
 xlesfd = 1.5d0
-smagmx = 10.d0*csmago
+smagmx = csmago**2
+smagmn = 0.d0
 cdries = 26.d0
 
 !   pour le v2f phi-model
@@ -1108,6 +941,9 @@ ckwgm2 = ckwbt2/cmu - xkappa**2/(ckwsw2*sqrt(cmu))
 ckwa1  = 0.31d0
 ckwc1  = 10.d0
 
+! pour la ddes
+cddes = 0.65d0
+
 !   pour le modele de Spalart Allmaras
 csab1    = 0.1355d0
 csab2    = 0.622d0
@@ -1119,7 +955,7 @@ csaw3    = 2.d0
 
 ! for the Spalart-Shur rotation/curvature correction
 cssr1 = 1.d0
-cssr2 = 12.d0
+cssr2 = 2.d0
 cssr3 = 1.d0
 
 ! for the Cazalbou rotation/curvature correction
@@ -1157,7 +993,6 @@ do iscal = 1, nscamx
   iscacp(iscal) =-10
   iclvfl(iscal) = -1
   visls0(iscal) =-grand*10.d0
-  sigmas(iscal) = 1.0d0
   rvarfl(iscal) = 0.8d0
 enddo
 
@@ -1179,6 +1014,11 @@ cthafm = 0.236d0
 
 ! For the DFM (tranport equation on the turbulent flux)
 cthdfm = 0.31d0
+! For the EB-DFM model (0.122*2.5d0, See F. Dehoux thesis)
+cthebdfm = 0.22d0
+xclt   = 0.305d0
+rhebdfm = 0.5d0
+
 
 ! --- Ici tout cstphy a ete initialise
 
@@ -1272,11 +1112,11 @@ ifaccp = 0
 tslagr => rvoid2
 
 !===============================================================================
-! 13. Cavitation module
+! 13. VOF/Cavitation module
 !===============================================================================
 
+call init_vof
 call init_cavitation
-!===================
 
 !===============================================================================
 ! 14. Exit
