@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -35,17 +35,16 @@ use paramx
 use optcal
 use entsor
 use ihmpre
-use ppincl, only: ippmod, nmodmx
+use ppincl, only: ippmod, nmodmx, iatmos
 use post
 use cs_c_bindings
 use field
 use lagran
-use ppincl
 use cpincl
 use dimens
-use numvar, only: itempb
 use radiat
 use cs_fuel_incl
+use cdomod
 
 !===============================================================================
 
@@ -55,13 +54,10 @@ implicit none
 
 ! Local variables
 
-integer          iok, ipp, nmodpp, imom, n_moments, f_id, f_type, nfld
-integer          keyvar, ivar
+integer          iok, ipp, nmodpp
 
 double precision ttsuit, wtsuit
 
-type(var_cal_opt) vcopt
-
 !===============================================================================
 
 interface
@@ -84,6 +80,18 @@ interface
     implicit none
   end subroutine user_linear_solvers
 
+  subroutine user_output()  &
+      bind(C, name='cs_user_output')
+    use, intrinsic :: iso_c_binding
+    implicit none
+  end subroutine user_output
+
+  subroutine user_syrthes_coupling()  &
+      bind(C, name='cs_user_syrthes_coupling')
+    use, intrinsic :: iso_c_binding
+    implicit none
+  end subroutine user_syrthes_coupling
+
 end interface
 
 !===============================================================================
@@ -123,28 +131,23 @@ call elec_option_init
 
 call cs_rad_transfer_options
 
-! Additional fields
+if (ippmod(iatmos).ge.0) call cs_at_data_assim_initialize
 
-call addfld
+! Additional fields if not in CDO mode only
+
+if (icdo.lt.2) then
+   call addfld
+endif
 
 ! Time moments
 
 call cs_gui_time_moments
 call cs_user_time_moments
 
-n_moments = cs_time_moment_n_moments()
-do imom = 1, n_moments
-  f_id = time_moment_field_id(imom)
-  if (f_id.lt.0) cycle
-  ipp = field_post_id(f_id)
-enddo
-
 ! Postprocessing and logging
 
 if (iihmpr.eq.1) then
-  call csenso                                                   &
-     ( nvppmx, ncapt,  nthist, frhist, iecaux,          &
-       ihisvr, tplfmt, xyzcap )
+  call csenso(iecaux)
 endif
 
 ! Restart
@@ -160,8 +163,6 @@ call lagran_init_map
 
 call lagopt(isuite, iccvfg, iscalt, dtref)
 
-call lagstati
-
 !===============================================================================
 ! 3. DEFINITION DES COUPLAGES AVEC SYRTHES
 !===============================================================================
@@ -173,7 +174,7 @@ if (iihmpr.eq.1) then
   call uisyrc
 endif
 
-call ussyrc
+call user_syrthes_coupling
 
 call ussatc
 
@@ -181,48 +182,41 @@ call ussatc
 ! 4. MODIFS APRES USINI1
 !===============================================================================
 
-call modini
+! Do not call this routine if CDO mode only (default variables and properties
+! are not defined anymore)
+if (icdo.lt.2) then
+   call modini
+endif
 
 !===============================================================================
 ! 5. Some additional fields and mappings
 !===============================================================================
 
-call fldini
+! Do not call this routine if CDO mode only (default variables and properties
+! are not defined anymore)
+if (icdo.lt.2) then
+   call fldini
+endif
 
 call gui_postprocess_fields
 
 call usipes(nmodpp)
+call user_output
 
 call gui_linear_solvers
 call user_linear_solvers
 
-! Number of fields
-call field_get_n_fields(nfld)
-call field_get_key_id("variable_id", keyvar)
-
-! Copy field calculation options into the field structure
-do f_id = 0, nfld - 1
-
-  call field_get_type(f_id, f_type)
-
-  ! Is the field of type FIELD_VARIABLE?
-  if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
-    call field_get_key_int(f_id, keyvar, ivar)
-    if (ivar.gt.0) then
-      call field_get_key_struct_var_cal_opt(f_id, vcopt)
-      vcopt%iwarni= iwarni(ivar)
-      call field_set_key_struct_var_cal_opt(f_id, vcopt)
-    endif
-  endif
-enddo
-
 !===============================================================================
 ! 6. Coherency checks
 !===============================================================================
 
 iok = 0
 
-call verini (iok)
+! No verification in CDO mode only. This done elsewhere
+if (icdo.lt.2) then
+   call verini (iok)
+   call parameters_check
+endif
 
 if(iok.gt.0) then
   write(nfecra,9999)iok
