@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -33,17 +33,11 @@
 !------------------------------------------------------------------------------
 !   mode          name          role
 !------------------------------------------------------------------------------
-!> \param[in]     nvar          total number of variables
 !> \param[in]     nscal         total number of scalars
-!> \param[out]    dt            time step value
-!> \param[out]    propce        physical properties at cell centers
-!> \param[out]    frcxt         external stress generating hydrostatic pressure
-!> \param[out]    prhyd         hydrostatic pressure predicted
 !______________________________________________________________________________
 
 subroutine iniva0 &
- ( nvar   , nscal  ,                                              &
-   dt     , propce , frcxt  , prhyd)
+ ( nscal  )
 
 !===============================================================================
 ! Module files
@@ -54,7 +48,6 @@ use numvar
 use optcal
 use cstphy
 use cstnum
-use dimens, only: nproce
 use pointe
 use entsor
 use albase
@@ -67,7 +60,9 @@ use cplsat
 use field
 use mesh
 use cavitation
+use vof
 use cs_cf_bindings
+use cs_c_bindings
 use darcy_module
 
 !===============================================================================
@@ -76,33 +71,29 @@ implicit none
 
 ! Arguments
 
-integer          nvar   , nscal
-
-double precision frcxt(3,ncelet), prhyd(ncelet)
-double precision dt(ncelet), propce(ncelet,*)
+integer          nscal
 
 ! Local variables
 
-integer          iis   , iscal , iprop
-integer          iel   , ifac  , isou
-integer          iclip , ii    , jj    , idim
-integer          iivism
+integer          iis   , iscal
+integer          iel   , ifac
+integer          iclip , ii    , jj    , idim, f_dim
 integer          ifcvsl
-integer          nn
 integer          iflid, nfld, ifmaip, bfmaip, iflmas, iflmab
-integer          f_id,  f_dim
 integer          kscmin, kscmax
+integer          f_type
+integer          keyvar
 
 logical          have_previous
 
 double precision xxk, xcmu, trii
 
+double precision, dimension(:), pointer :: dt
 double precision, dimension(:), pointer :: brom, crom, crom_prev2
 double precision, dimension(:), pointer :: cofbcp
 double precision, dimension(:), pointer :: porosi
 double precision, dimension(:,:), pointer :: porosf
 double precision, dimension(:), pointer :: field_s_v
-double precision, dimension(:,:), pointer :: field_v_v
 
 double precision, dimension(:), pointer :: cvar_pr
 double precision, dimension(:), pointer :: cvar_k, cvar_ep, cvar_al
@@ -112,8 +103,9 @@ double precision, dimension(:), pointer :: cvar_r12, cvar_r13, cvar_r23
 double precision, dimension(:,:), pointer :: cvar_rij
 double precision, dimension(:), pointer :: viscl, visct, cpro_cp, cpro_prtot
 double precision, dimension(:), pointer :: cpro_viscls, cproa_viscls, cvar_tempk
-double precision, dimension(:), pointer :: cproa_viscl, cproa_cp
+double precision, dimension(:), pointer :: cproa_viscl, cproa_cp, cpro_visma_s
 double precision, dimension(:), pointer :: mix_mol_mas
+double precision, dimension(:,:), pointer :: cpro_visma_v
 
 !===============================================================================
 
@@ -121,6 +113,10 @@ double precision, dimension(:), pointer :: mix_mol_mas
 ! 1.  INITIALISATION
 !===============================================================================
 
+call field_get_key_id("variable_id", keyvar)
+
+call field_get_val_s_by_name('dt', dt)
+
 ! Initialize variables to avoid compiler warnings
 
 jj = 0
@@ -134,43 +130,6 @@ if ( ippmod(icompf).ge.0 ) then
   enddo
 endif
 
-! Initialize all cell property fields to zero
-! (this is useful only for fields which are mapped;
-! fields who own their values, such as variables, are already
-! initialized after allocation).
-
-do iprop = 1, nproce
-
-  f_id = iprpfl(iprop)
-
-  if (f_id.lt.0) cycle
-
-  call field_get_dim(f_id, f_dim)
-
-  if (f_dim.gt.1) then
-    call field_get_val_v(f_id, field_v_v)
-  else if (f_dim.eq.1) then
-    call field_get_val_s(f_id, field_s_v)
-  endif
-
-  if (f_dim.gt.1) then
-    !$omp parallel do private(isou)
-    do iel = 1, ncelet
-      do isou = 1, f_dim
-        field_v_v(isou,iel) = 0.d0
-      enddo
-    enddo
-  else
-    !$omp parallel do
-    do iel = 1, ncelet
-      field_s_v(iel) = 0.d0
-    enddo
-  endif
-
-  call field_current_to_previous(f_id) ! For those properties requiring it.
-
-enddo
-
 !===============================================================================
 ! 2. PAS DE TEMPS
 !===============================================================================
@@ -192,14 +151,15 @@ call field_get_val_s(icrom, crom)
 call field_get_val_s(ibrom, brom)
 
 !     Masse volumique aux cellules (et au pdt precedent si ordre2 ou icalhy
-!     ou cavitation)
+!     ou algo. VOF)
 do iel = 1, ncel
   crom(iel)  = ro0
 enddo
-if (iroext.gt.0.or.icalhy.eq.1.or.idilat.gt.1.or.icavit.ge.0.or.ipthrm.eq.1) then
+if (    iroext.gt.0.or.icalhy.eq.1.or.idilat.gt.1 &
+    .or.ivofmt.ge.0.or.ipthrm.eq.1) then
   call field_current_to_previous(icrom)
 endif
-if (icavit.ge.0.or.idilat.gt.1) then
+if (ivofmt.ge.0.or.idilat.gt.1) then
   call field_get_val_s(icroaa, crom_prev2)
   do iel = 1, ncelet
     crom_prev2(iel) = crom(iel)
@@ -210,33 +170,33 @@ endif
 do ifac = 1, nfabor
   brom(ifac) = ro0
 enddo
-if (iroext.gt.0.or.icavit.ge.0) then
+if (iroext.gt.0.or.ivofmt.ge.0.or.idilat.gt.1) then
   call field_current_to_previous(ibrom)
 endif
 
 !     Viscosite moleculaire
-call field_get_val_s(iprpfl(iviscl), viscl)
-call field_get_val_s(iprpfl(ivisct), visct)
+call field_get_val_s(iviscl, viscl)
+call field_get_val_s(ivisct, visct)
 
 !     Viscosite moleculaire aux cellules (et au pdt precedent si ordre2)
 do iel = 1, ncel
   viscl(iel) = viscl0
 enddo
 if(iviext.gt.0) then
-  call field_get_val_prev_s(iprpfl(iviscl), cproa_viscl)
+  call field_get_val_prev_s(iviscl, cproa_viscl)
   do iel = 1, ncel
     cproa_viscl(iel) = viscl(iel)
   enddo
 endif
 
 !     Chaleur massique aux cellules (et au pdt precedent si ordre2)
-if(icp.gt.0) then
-  call field_get_val_s(iprpfl(icp), cpro_cp)
+if(icp.ge.0) then
+  call field_get_val_s(icp, cpro_cp)
   do iel = 1, ncel
     cpro_cp(iel) = cp0
   enddo
   if(icpext.gt.0) then
-    call field_get_val_prev_s(iprpfl(icp), cproa_cp)
+    call field_get_val_prev_s(icp, cproa_cp)
     do iel = 1, ncel
       cproa_cp(iel) = cpro_cp(iel)
     enddo
@@ -251,7 +211,7 @@ endif
 ! into account
 if ((ippmod(icompf).lt.0.and.ippmod(idarcy).lt.0).or.                          &
     (ippmod(idarcy).ge.0.and.darcy_gravity.ge.1)) then
-  call field_get_val_s(iprpfl(iprtot), cpro_prtot)
+  call field_get_val_s(iprtot, cpro_prtot)
   do iel = 1, ncel
     cpro_prtot(iel) = - rinfin
   enddo
@@ -260,7 +220,7 @@ endif
 ! Initialization of mix_mol_mas with default values (air)
 ! (used in cs_cf_thermo_default_init)
 if(ippmod(igmix).ge.0) then
-  call field_get_val_s(iprpfl(igmxml), mix_mol_mas)
+  call field_get_val_s(igmxml, mix_mol_mas)
   do iel =1, ncel
     mix_mol_mas(iel) = xmasmr
   enddo
@@ -304,14 +264,19 @@ enddo
 
 !     Viscosite de maillage en ALE
 if (iale.eq.1) then
-  nn = 1
-  if (iortvm.eq.1) nn = 3
-  do ii = 1, nn
-    iivism = ipproc(ivisma(ii))
-    do iel = 1, ncel
-      propce(iel,iivism) = 1.d0
+  if (iortvm.eq.1) then
+    call field_get_val_v(ivisma, cpro_visma_v)
+    do ii = 1, 3
+      do iel = 1, ncel
+        cpro_visma_v(ii,iel) = 1.d0
+      enddo
     enddo
-  enddo
+  else
+    call field_get_val_s(ivisma, cpro_visma_s)
+    do iel = 1, ncel
+      cpro_visma_s(iel) = 1.d0
+    enddo
+  endif
 endif
 
 ! Porosity
@@ -352,15 +317,15 @@ do iel = 1, ncel
 enddo
 
 ! On definit les clipping du taux de vide et on initialize au clipping inf.
-if (icavit.ge.0) then
+if (ivofmt.ge.0) then
 
   call field_get_key_id("min_scalar_clipping", kscmin)
   call field_get_key_id("max_scalar_clipping", kscmax)
 
-  call field_set_key_double(ivarfl(ivoidf), kscmin, clvfmn)
-  call field_set_key_double(ivarfl(ivoidf), kscmax, clvfmx)
+  call field_set_key_double(ivarfl(ivolf2), kscmin, clvfmn)
+  call field_set_key_double(ivarfl(ivolf2), kscmax, clvfmx)
 
-  call field_get_val_s(ivarfl(ivoidf), field_s_v)
+  call field_get_val_s(ivarfl(ivolf2), field_s_v)
   do iel = 1, ncel
     field_s_v(iel) = clvfmn
   enddo
@@ -392,8 +357,7 @@ if(itytur.eq.2 .or. itytur.eq.5) then
     enddo
 
     iclip = 1
-    call clipke(ncelet , ncel   , nvar    ,     &
-         iclip  , iwarni(ik))
+    call clipke(ncelet, ncel, iclip)
 
   else
     do iel = 1, ncel
@@ -558,7 +522,8 @@ if (nscal.gt.0) then
 
 !    Clipping des scalaires non variance
   do iis = 1, nscal
-    if(iscavr(iis).eq.0) then
+    call field_get_dim(ivarfl(isca(iis)), f_dim)
+    if(iscavr(iis).eq.0.and.f_dim.eq.1) then
       iscal = iis
       call clpsca(iscal)
       !==========
@@ -627,9 +592,7 @@ call field_get_n_fields(nfld)
 ifmaip = -1
 bfmaip = -1
 
-do ii = 1, nfld
-
-  iflid = ii - 1
+do iflid = 0, nfld - 1
 
   call field_get_key_int(iflid, kimasf, iflmas) ! interior mass flux
   call field_get_key_int(iflid, kbmasf, iflmab) ! boundary mass flux
@@ -647,29 +610,7 @@ do ii = 1, nfld
 enddo
 
 !===============================================================================
-! 8.  INITIALISATION DE LA FORCE EXTERIEURE QUAND IPHYDR=1
-!===============================================================================
-
-if(iphydr.eq.1) then
-  do iel = 1, ncel
-    frcxt(1,iel) = 0.d0
-    frcxt(2,iel) = 0.d0
-    frcxt(3,iel) = 0.d0
-  enddo
-endif
-
-!===============================================================================
-! 9.  INITIALISATION DE LA PRESSION HYDROSTATIQUE QUAND IPHYDR=2
-!===============================================================================
-
-if(iphydr.eq.2) then
-  do iel = 1, ncel
-    prhyd(iel) = 0.d0
-  enddo
-endif
-
-!===============================================================================
-! 10.  INITIALISATIONS EN ALE OU MAILLAGE MOBILE
+! 8.  INITIALISATIONS EN ALE
 !===============================================================================
 
 if (iale.eq.1) then
@@ -678,7 +619,7 @@ if (iale.eq.1) then
   enddo
 endif
 
-if (iale.eq.1.or.imobil.eq.1) then
+if (iale.eq.1) then
   do ii = 1, nnod
     do idim = 1, 3
       xyzno0(idim,ii) = xyznod(idim,ii)
@@ -686,8 +627,20 @@ if (iale.eq.1.or.imobil.eq.1) then
   enddo
 endif
 
+!===============================================================================
+! 9 Current to previous for variables
+!===============================================================================
+
+do iflid = 0, nfld - 1
+  call field_get_type(iflid, f_type)
+  ! Is the field of type FIELD_VARIABLE?
+  if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
+    call field_current_to_previous(iflid)
+  endif
+enddo
+
 !----
-! FIN
+! End
 !----
 
 return
