@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -34,12 +34,10 @@
 !------------------------------------------------------------------------------
 !> \param[in]     nvar          total number of variables
 !> \param[in]     nscal         total number of scalars
-!> \param[in]     dt            value of time step
 !______________________________________________________________________________
 
 subroutine inivar &
- ( nvar   , nscal  ,                                              &
-   dt     )
+ ( nvar   , nscal )
 
 !===============================================================================
 ! Module files
@@ -77,8 +75,6 @@ implicit none
 
 integer          nvar   , nscal
 
-double precision dt(ncelet)
-
 ! Local variables
 
 character(len=80) :: chaine
@@ -103,6 +99,7 @@ double precision scmaxp, scminp
 double precision rvoid(1)
 double precision vvoid(3)
 
+double precision, dimension(:), pointer :: dt
 double precision, dimension(:), pointer :: field_s_v
 double precision, dimension(:,:), pointer :: field_v_v
 double precision, dimension(:), pointer :: cvar_pr
@@ -113,12 +110,16 @@ double precision, dimension(:,:), pointer :: cvar_rij
 double precision, dimension(:), pointer :: cvar_var
 double precision, dimension(:), pointer :: cpro_prtot
 
+type(var_cal_opt) :: vcopt
+
 !===============================================================================
 
 !===============================================================================
 ! 1. Initialization
 !===============================================================================
 
+call field_get_val_s_by_name('dt', dt)
+
 call field_get_n_fields(n_fields)
 
 call field_get_key_id("variable_id", keyvar)
@@ -145,15 +146,17 @@ iusini = 1
 iflidp = -1
 
 do ivar = 1, nvar
-  if (iwgrec(ivar).eq.1) then
+  call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+  if (vcopt%iwgrec.eq.1) then
 
-    if (idiff(ivar).lt.1) cycle
+    if (vcopt%idiff.lt.1) cycle
     iflid = ivarfl(ivar)
     if (iflid.eq.iflidp) cycle
     iflidp = iflid
-    if (idften(ivar).eq.1) then
+
+    if (iand(vcopt%idften, ISOTROPIC_DIFFUSION).ne.0) then
       idimf = 1
-    elseif (idften(ivar).eq.6) then
+    elseif (iand(vcopt%idften, ANISOTROPIC_DIFFUSION).ne.0) then
       idimf = 6
     endif
     call field_get_key_int(iflid, kwgrec, f_id)
@@ -239,7 +242,7 @@ call user_initialization()
 
 if (ippmod(icompf).lt.0.and.ippmod(idarcy).lt.0) then
   call field_get_val_s(ivarfl(ipr), cvar_pr)
-  call field_get_val_s(iprpfl(iprtot), cpro_prtot)
+  call field_get_val_s(iprtot, cpro_prtot)
   xxp0   = xyzp0(1)
   xyp0   = xyzp0(2)
   xzp0   = xyzp0(3)
@@ -260,7 +263,7 @@ if (ippmod(icompf).lt.0.and.ippmod(idarcy).lt.0) then
   enddo
 else if ((ippmod(idarcy).ge.0).and.(darcy_gravity.ge.1)) then
   call field_get_val_s(ivarfl(ipr), cvar_pr)
-  call field_get_val_s(iprpfl(iprtot), cpro_prtot)
+  call field_get_val_s(iprtot, cpro_prtot)
   do iel = 1, ncel
     cpro_prtot(iel) = cvar_pr(iel) - xyzcen(1,iel)*darcy_gravity_x &
                                    - xyzcen(2,iel)*darcy_gravity_y &
@@ -324,7 +327,7 @@ if (iusini.eq.1.or.isuite.eq.1) then
 
     if(xekmin.ge.0.d0.and.xepmin.ge.0.d0) then
       iclip = 1
-      call clipke(ncelet, ncel, nvar, iclip, iwarni(ik))
+      call clipke(ncelet, ncel, iclip)
     else
       write(nfecra,3020) xekmin,xepmin
       iok = iok + 1
@@ -535,10 +538,11 @@ if(nscal.gt.0.and.(iusini.eq.1.or.isuite.eq.1)) then
 !     Scalaires non variance
 
   do ii = 1, nscal
-    if(iscavr(ii).le.0.or.iscavr(ii).gt.nscal) then
+    f_id = ivarfl(isca(ii))
+    call field_get_dim(f_id, f_dim)
+    if((iscavr(ii).le.0.or.iscavr(ii).gt.nscal).and.f_dim.eq.1) then
 
       ! Get the min clipping
-      f_id = ivarfl(isca(ii))
       call field_get_key_double(f_id, kscmin, scminp)
       call field_get_key_double(f_id, kscmax, scmaxp)
 
@@ -642,9 +646,8 @@ if(nscal.gt.0.and.(iusini.eq.1.or.isuite.eq.1)) then
 
 endif
 
-
 !===============================================================================
-! 6.  IMPRESSIONS DE CONTROLE POUR LES INCONNUES, LE PAS DE TEMPS
+! 5.  IMPRESSIONS DE CONTROLE POUR LES INCONNUES, LE PAS DE TEMPS
 !        LE CUMUL DES DUREE POUR LES MOYENNES
 !===============================================================================
 
@@ -725,7 +728,7 @@ endif
 call time_moment_log_iteration
 
 !===============================================================================
-! 7.  ARRET GENERAL SI PB
+! 6.  ARRET GENERAL SI PB
 !===============================================================================
 
 if (iok.gt.0) then
