@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -44,14 +44,11 @@
 !  mode           name          role                                           !
 !______________________________________________________________________________!
 !> \param[in]     oflmap        pointer to old field map
-!> \param[out]    frcxt         external forces making hydrostatic pressure
-!> \param[out]    prhyd         predicted hydrostatic pressure
 !_______________________________________________________________________________
 
 
 subroutine lecamx &
- ( oflmap ,                                                       &
-   frcxt  , prhyd  )
+ ( oflmap )
 
 !===============================================================================
 
@@ -83,9 +80,11 @@ use ppcpfu
 use mesh
 use field
 use cavitation
+use vof
 use cs_c_bindings
 use cs_nz_condensation, only: izzftcd, nztag1d, ztpar
 use cs_nz_tagmr, only: znmurx, znmur, ztmur
+
 !===============================================================================
 
 implicit none
@@ -94,8 +93,6 @@ implicit none
 
 type(c_ptr)      oflmap
 
-double precision frcxt(3,ncelet), prhyd(ncelet)
-
 ! Local variables
 
 character        rubriq*64,car4*4,car2*2
@@ -115,23 +112,25 @@ integer          nberro, inierr, ivers(1)
 integer          ilu   , ierrch
 integer          nfmtsc, nfmtfl, nfmtch, nfmtcl
 integer          nfmtst
-integer          jale, jcavit
+integer          jale, jcavit, jvolfl
 integer          f_id, iflmas, iflmab, iflvoi, iflvob
 integer          ival(1), ngbstr(2)
 double precision rval(1), tmpstr(27)
 
+character(len=80) :: fname
+
 logical(kind=c_bool) :: ncelok, nfaiok, nfabok, nsomok
 
 type(c_ptr) :: rp
 
 double precision, dimension(:), pointer :: sval
 double precision, dimension(:), pointer :: voidfl
-double precision, dimension(:,:), pointer :: disale
-
-double precision, dimension(:), pointer :: cpro_vism1, cpro_vism2, cpro_vism3
+double precision, dimension(:,:), pointer :: disale, cpro_visma_v
 
 double precision, allocatable, dimension(:,:) :: tmurbf
 double precision, allocatable, dimension(:) :: tparbf
+double precision, allocatable, dimension(:) :: vismbf
+
 !===============================================================================
 
 !===============================================================================
@@ -242,13 +241,33 @@ call restart_read_section_int_t(rp,rubriq,itysup,nbval,ival,ierror)
 jcavit = ival(1)
 nberro=nberro+ierror
 
-! --->  Message si erreur (pas de stop pour compatibilite avec les fichiers anterieurs)
+! --->  Message si erreur (pas de stop pour compatibilite avec les fichiers
+!                          anterieurs)
 !       -> on n'affiche le message que si ICAVIT>=0 (sinon RAS)
 if (nberro.ne.0) then
   if (icavit.ge.0) write(nfecra,9220)
   jcavit = -1
 endif
 
+!     VOF
+
+nberro = 0
+
+rubriq = 'vof'
+itysup = 0
+nbval  = 1
+call restart_read_section_int_t(rp,rubriq,itysup,nbval,ival,ierror)
+jvolfl = ival(1)
+nberro=nberro+ierror
+
+! --->  Message si erreur (pas de stop pour compatibilite avec les fichiers
+!                          anterieurs)
+!       -> on n'affiche le message que si IVOFMT=1 (sinon RAS)
+if (nberro.ne.0) then
+  if (ivofmt.ge.0) write(nfecra,9221)
+  jvolfl = 0
+endif
+
 car54 =' Fin de la lecture des options                        '
 write(nfecra,1110)car54
 
@@ -307,15 +326,16 @@ endif
 
 inierr = 0
 
-if (irovar.eq.1.or.(icavit.ge.0.and.jcavit.ge.0)) then
+if (    irovar.eq.1                   &
+    .or.(ivofmt.ge.0.and.jvolfl.ge.0)) then
 
   ! Masse volumique - cellules
   call restart_read_field_vals(rp, icrom, 0, ierror)
   nberro = nberro+ierror
   inierr = inierr+ierror
 
-  ! Masse volumique du pdt precedent - cellules (uniquement pour cavitation)
-  if (icavit.ge.0.and.jcavit.ge.0.or.idilat.ge.4) then
+  ! Masse volumique du pdt precedent - cellules
+  if (ivofmt.ge.0.and.jvolfl.ge.0.or.idilat.ge.4) then
     call restart_read_field_vals(rp, icrom, 1, ierror)
     nberro = nberro+ierror
     inierr = inierr+ierror
@@ -337,7 +357,7 @@ if (irovar.eq.1.or.(icavit.ge.0.and.jcavit.ge.0)) then
   ! Scalar source terms for dilatable model  (idilat = 4, 5)
   if (idilat.ge.4) then
     do iscal = 1, nscal
-      f_id = iprpfl(iustdy(iscal))
+      f_id = iustdy(iscal)
       call restart_read_field_vals(rp, f_id, 0, ierror)
     enddo
   endif
@@ -353,21 +373,23 @@ endif
 !     La viscosite moleculaire est egalement lue pour le modele de cavitation
 !     Si on reussit, on l'indique
 
-if (iviext.gt.0.or.(icavit.ge.0.and.jcavit.ge.0)) then
+if (    iviext.gt.0                   &
+    .or.(ivofmt.ge.0.and.jvolfl.ge.0)) then
 
   inierr = 0
 
   !         Viscosite moleculaire - cellules
-  !         Uniquement si elle est variable ou pour le modele de cavitation
-  if (ivivar.eq.1.or.(icavit.ge.0.and.jcavit.ge.0)) then
-    call restart_read_field_vals(rp, iprpfl(iviscl), 0, ierror)
+  !         Uniquement si elle est variable ou pour la methode VOF
+  if (    ivivar.eq.1                   &
+      .or.(ivofmt.ge.0.and.jvolfl.ge.0)) then
+    call restart_read_field_vals(rp, iviscl, 0, ierror)
     nberro = nberro+ierror
     inierr = inierr+ierror
   endif
 
   !         Viscosite turbulente ou de sous-maille - cellules
   if (iviext.gt.0) then
-    call restart_read_field_vals(rp, iprpfl(ivisct), 0, ierror)
+    call restart_read_field_vals(rp, ivisct, 0, ierror)
     nberro = nberro+ierror
     inierr = inierr+ierror
   endif
@@ -388,13 +410,13 @@ endif
 !        et quand l'utilisateur peut s'en servir pour passer
 !        de H a T, comme en effet Joule par exemple).
 
-if ((icpext.gt.0.and.icp.gt.0).or.                &
-     (ippmod(ieljou).ge.1.and.icp.gt.0)) then
+if ((icpext.gt.0.and.icp.ge.0).or.                &
+     (ippmod(ieljou).ge.1.and.icp.ge.0)) then
 
   inierr = 0
 
   ! Chaleur massique - cellules
-  call restart_read_field_vals(rp, iprpfl(icp), 0, ierror)
+  call restart_read_field_vals(rp, icp, 0, ierror)
   nberro = nberro+ierror
   inierr = inierr+ierror
 
@@ -512,17 +534,17 @@ if (nfaiok.eqv..true. .or. nfabok.eqv..true.) then
   nberro=0
 
   ! Initialization of the void fraction convective flux, if required
-  if (icavit.ge.0.and.jcavit.lt.0) then
+  if (ivofmt.ge.0.and.jvolfl.lt.0) then
 
     ! Interior faces
 
     call field_get_key_int(ivarfl(iu), kimasf, iflmas)
-    call field_get_key_int(ivarfl(ivoidf), kimasf, iflvoi)
+    call field_get_key_int(ivarfl(ivolf2), kimasf, iflvoi)
 
     call field_get_val_s(iflmas, sval)
     call field_get_val_s(iflvoi, voidfl)
     do ifac = 1, nfac
-      voidfl(ifac) = sval(ifac)/rol
+      voidfl(ifac) = sval(ifac)/rho1
     enddo
 
     call field_have_previous(iflmas, lprev)
@@ -530,27 +552,27 @@ if (nfaiok.eqv..true. .or. nfabok.eqv..true.) then
       call field_get_val_prev_s(iflmas, sval)
       call field_get_val_prev_s(iflvoi, voidfl)
       do ifac = 1, nfac
-        voidfl(ifac) = sval(ifac)/rol
+        voidfl(ifac) = sval(ifac)/rho1
       enddo
     endif
 
     call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
     call field_get_val_s(iflmab, sval)
-    call field_get_key_int(ivarfl(ivoidf), kbmasf, iflvob)
+    call field_get_key_int(ivarfl(ivolf2), kbmasf, iflvob)
     call field_get_val_s(iflvob, voidfl)
     do ifac = 1, nfabor
-      voidfl(ifac) = sval(ifac)/rol
+      voidfl(ifac) = sval(ifac)/rho1
     enddo
 
     ! Boundary faces
 
     call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
-    call field_get_key_int(ivarfl(ivoidf), kbmasf, iflvob)
+    call field_get_key_int(ivarfl(ivolf2), kbmasf, iflvob)
 
     call field_get_val_s(iflmab, sval)
     call field_get_val_s(iflvob, voidfl)
     do ifac = 1, nfabor
-      voidfl(ifac) = sval(ifac)/rol
+      voidfl(ifac) = sval(ifac)/rho1
     enddo
 
     call field_have_previous(iflmas, lprev)
@@ -558,7 +580,7 @@ if (nfaiok.eqv..true. .or. nfabok.eqv..true.) then
       call field_get_val_prev_s(iflmab, sval)
       call field_get_val_prev_s(iflvob, voidfl)
       do ifac = 1, nfabor
-        voidfl(ifac) = sval(ifac)/rol
+        voidfl(ifac) = sval(ifac)/rho1
       enddo
     endif
 
@@ -670,11 +692,9 @@ nberro = 0
 if (ineedy.eq.1) then
   if (icdpar.gt.0 .or. inpdt0.eq.1) then
     if (nfabok.eqv..true.) then
-      itysup = 1
-      nbval  = 1
-      rubriq = 'dist_fac_par_ce_phase01'
-      call restart_read_section_real_t(rp,rubriq,itysup,nbval,dispar,  &
-                                       ierror)
+      call field_get_id('wall_distance', f_id)
+      call restart_read_field_vals(rp, f_id, 0, ierror)
+
       nberro=nberro+ierror
       if (ierror.eq.0 .and. iale.eq.0 ) then
         imajdy = 1
@@ -699,23 +719,12 @@ endif
 !===============================================================================
 
 if (iphydr.eq.1) then
-  nberro=0
 
   itysup = 1
   nbval  = 3
 
-  ! TODO read the old format
-  rubriq = 'force_ext_ce_phase01'
-  call restart_read_section_real_t(rp,rubriq,itysup,nbval,frcxt,ierror)
-  nberro=nberro+ierror
-
-  if (nberro.ne.0) then
-    car54 = 'Lecture des forces exterieures                        '
-    write(nfecra,8300)car54
-  endif
-
-  car54 = ' Fin de la lecture des forces exterieures             '
-  write(nfecra,1110)car54
+  call field_get_id('volume_forces', f_id)
+  call restart_read_field_vals(rp, f_id, 0, ierror)
 
 endif
 
@@ -725,22 +734,11 @@ endif
 
 if (iphydr.eq.2) then
 
-  nberro=0
-
   itysup = 1
   nbval  = 1
 
-  rubriq = 'Prhyd_pre_phase01'
-  call restart_read_section_real_t(rp,rubriq,itysup,nbval,prhyd,ierror)
-  nberro=nberro+ierror
-
- if (nberro.ne.0) then
-   car54 = 'Lecture de la pression hydrostatique predite          '
-   write(nfecra,8300)car54
-  endif
-
-  car54 =' Fin de la lecture de la pression hydro. predite      '
-  write(nfecra,1110)car54
+  call field_get_id('hydrostatic_pressure_prd', f_id)
+  call restart_read_field_vals(rp, f_id, 0, ierror)
 
 endif
 
@@ -749,7 +747,7 @@ endif
 !      with or wihtout the 1D thermal model tag1D
 !===============================================================================
 
-if (icond.eq.0) then
+if (icondb.eq.0) then
 
   if (nztag1d.eq.1) then
 
@@ -832,20 +830,25 @@ if (iale.eq.1 .and. jale.eq.1) then
   call restart_read_section_int_t(rp,rubriq,itysup,nbval,ival,ierror)
   jortvm = ival(1)
 
-  call restart_read_field_vals(rp, iprpfl(ivisma(1)), 0, ierror)
-
   if (iortvm.eq.1) then
-    if (jortvm.eq.1) then
-      call restart_read_field_vals(rp, iprpfl(ivisma(2)), 0, ierror)
-      call restart_read_field_vals(rp, iprpfl(ivisma(3)), 0, ierror)
-    else
-      call field_get_val_s(iprpfl(ivisma(1)), cpro_vism1)
-      call field_get_val_s(iprpfl(ivisma(2)), cpro_vism2)
-      call field_get_val_s(iprpfl(ivisma(3)), cpro_vism3)
+    if (jortvm.ne.1) then
+      call field_get_name(ivisma, fname)
+      rubriq = trim(fname)//'::vals::0'
+      itysup = 1
+      nbval  = 1
+      allocate(vismbf(ncelet))
+      call restart_read_section_real_t(rp,rubriq,itysup,nbval,   &
+                                       vismbf,ierror)
+
+      call field_get_val_v(ivisma, cpro_visma_v)
       do iel = 1, ncel
-        cpro_vism2(iel) = cpro_vism1(iel)
-        cpro_vism3(iel) = cpro_vism1(iel)
+        cpro_visma_v(1,iel) = vismbf(iel)
+        cpro_visma_v(2,iel) = vismbf(iel)
+        cpro_visma_v(3,iel) = vismbf(iel)
       enddo
+      deallocate(vismbf)
+    else
+      call restart_read_field_vals(rp, ivisma, 0, ierror)
     endif
   endif
 
@@ -1495,7 +1498,13 @@ if (ilu.ne.0) then
 endif
 
 !===============================================================================
-! 16.  FERMETURE DU FICHIER SUITE AUXILAIRE
+! 16. LECTURE DES CHAMPS PAR CLEF RESTART
+!===============================================================================
+
+call restart_read_fields(rp, RESTART_AUXILIARY)
+
+!===============================================================================
+! 17.  FERMETURE DU FICHIER SUITE AUXILAIRE
 !===============================================================================
 
 !     Fermeture du fichier suite auxilaire
@@ -1504,13 +1513,13 @@ call restart_destroy(rp)
 write(nfecra,1200)
 
 !===============================================================================
-! 17. SORTIE
+! 18. SORTIE
 !===============================================================================
 
 return
 
 !===============================================================================
-! 18. FORMATS
+! 19. FORMATS
 !===============================================================================
 
 ! --- ETAPES
@@ -1904,6 +1913,27 @@ return
 '@                                                            ',/,&
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/)
+ 9221 format(                                                     &
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/,&
+'@ @@ ATTENTION : ERREUR A LA LECTURE DU FICHIER SUITE        ',/,&
+'@    =========                                     AUXILIAIRE',/,&
+'@                                                            ',/,&
+'@      ERREUR A LA LECTURE DE L''INDICATEUR DE MODELE DE     ',/,&
+'@                                                  VOF       ',/,&
+'@                                                            ',/,&
+'@    Il se peut que le fichier suite relu corresponde a une  ',/,&
+'@      version anterieure de Code_Saturne, sans modele de    ',/,&
+'@                                                 VOF.       ',/,&
+'@    Le calcul sera execute en reinitialisant toutes les     ',/,&
+'@      donnees du modele de VOF       .                      ',/,&
+'@    Verifier neanmoins que le fichier suite utilise n''a    ',/,&
+'@        pas ete endommage.                                  ',/,&
+'@                                                            ',/,&
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
  9320 format(                                                     &
 '@                                                            ',/,&
@@ -2080,6 +2110,24 @@ return
 '@    Verify that the restart file used has not been damaged. ',/,&
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/)
+ 9221 format(                                                     &
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/,&
+'@ @@ WARNING: ERROR WHILE READING THE AUXILIARY RESTART FILE ',/,&
+'@    =======                                                 ',/,&
+'@                                                            ',/,&
+'@      ERROR WHEN READING THE INDICATOR OF THE VOF MODEL     ',/,&
+'@                                                            ',/,&
+'@    It is possible that the file read corresponds to an old ',/,&
+'@      version of Code_Saturne, without the VOF model.       ',/,&
+'@    The run will be executed with reinitialising all        ',/,&
+'@      VOF model data.                                       ',/,&
+'@                                                            ',/,&
+'@    Verify that the restart file used has not been damaged. ',/,&
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
  9320 format(                                                     &
 '@                                                            ',/,&
