@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -30,6 +30,9 @@
 !> flows for one time step. Both convection-diffusion and continuity steps are
 !> performed.  The velocity components are solved together in once.
 !>
+!> Please refer to the
+!> <a href="../../theory.pdf#navstv"><b>navstv</b></a> section
+!> of the theory guide for more informations.
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
@@ -45,10 +48,8 @@
 !> \param[in]     isostd        indicator of standar outlet
 !>                               +index of the reference face
 !> \param[in]     dt            time step (per cell)
-!> \param[in]     propce        physical properties at cell centers
 !> \param[in]     frcxt         external force generating the hydrostatic
 !>                              pressure
-!> \param[in]     prhyd         hydrostatic pressure predicted at cell centers
 !> \param[in]     trava         work array for pressure velocity coupling
 !> \param[in]     ximpa         work array for pressure velocity coupling
 !> \param[in]     uvwk          work array for pressure velocity coupling
@@ -58,8 +59,8 @@
 subroutine navstv &
  ( nvar   , nscal  , iterns , icvrge , itrale ,                   &
    isostd ,                                                       &
-   dt     , propce ,                                              &
-   frcxt  , prhyd  ,                                              &
+   dt     ,                                                       &
+   frcxt  ,                                                       &
    trava  , ximpa  , uvwk   )
 
 !===============================================================================
@@ -69,7 +70,7 @@ subroutine navstv &
 !===============================================================================
 
 use paramx
-use dimens, only: ndimfb, nproce
+use dimens, only: ndimfb
 use numvar
 use entsor
 use cstphy
@@ -78,6 +79,7 @@ use optcal
 use pointe
 use albase
 use parall
+use paramx, only: isymet
 use period
 use ppppar
 use ppthch
@@ -90,7 +92,9 @@ use turbomachinery
 use ptrglo
 use field
 use cavitation
+use vof
 use cs_c_bindings
+use atincl, only: iatmst, imomst, iautom, imeteo
 
 !===============================================================================
 
@@ -103,18 +107,16 @@ integer          nvar   , nscal  , iterns , icvrge , itrale
 integer          isostd(nfabor+1)
 
 double precision, pointer, dimension(:)   :: dt
-double precision, pointer, dimension(:,:) :: propce
 double precision, pointer, dimension(:,:) :: frcxt
-double precision, pointer, dimension(:) :: prhyd
 double precision, pointer, dimension(:,:) :: trava, uvwk
 double precision, pointer, dimension(:,:,:) :: ximpa
 
 ! Local variables
 
-integer          iccocg, inc, iel, iel1, iel2, ifac, imax, imaxt
+integer          iccocg, inc, iel, iel1, iel2, ifac, imax, imaxt, imin, imint
 integer          ii    , inod, itypfl
 integer          isou, ivar, iitsm
-integer          init
+integer          init, iautof
 integer          iflmas, iflmab
 integer          iflmb0
 integer          nswrgp, imligp, iwarnp
@@ -124,16 +126,17 @@ integer          numcpl
 integer          iflvoi, iflvob
 double precision rnorm , rnormt, rnorma, rnormi, vitnor
 double precision dtsrom, unsrom, rhom, rovolsdt
-double precision epsrgp, climgp, extrap, xyzmax(3)
+double precision epsrgp, climgp, extrap, xyzmax(3), xyzmin(3)
 double precision thetap, xdu, xdv, xdw
 double precision xxp0 , xyp0 , xzp0
-double precision rhofac, dtfac, ddepx , ddepy, ddepz
+double precision rhofac, dtfac
 double precision xnrdis, xnrtmp
 double precision t1, t2, t3, t4
 double precision visclc, visctc
 double precision distbf, srfbnf, hint
 double precision rnx, rny, rnz
 double precision vr(3), vr1(3), vr2(3), vrn
+double precision disp_fac(3)
 
 double precision, allocatable, dimension(:,:,:), target :: viscf
 double precision, allocatable, dimension(:), target :: viscb
@@ -164,12 +167,16 @@ double precision, dimension(:), pointer :: ivoifl, bvoifl
 double precision, dimension(:), pointer :: coavoi, cobvoi
 double precision, dimension(:,:), pointer :: trav
 double precision, dimension(:,:), pointer :: mshvel
+double precision, dimension(:,:), pointer :: disale
 double precision, dimension(:,:), pointer :: disala
+double precision, dimension(:,:), pointer :: cpro_momst
 double precision, dimension(:), pointer :: porosi
 double precision, dimension(:), pointer :: cvar_pr, cvara_pr
 double precision, dimension(:), pointer :: cpro_prtot, c_estim
 double precision, dimension(:), pointer :: cvar_voidf, cvara_voidf
 
+type(var_cal_opt) :: vcopt_p, vcopt_u, vcopt
+
 !===============================================================================
 ! Interfaces
 !===============================================================================
@@ -177,7 +184,7 @@ double precision, dimension(:), pointer :: cvar_voidf, cvara_voidf
 interface
 
   subroutine resopv &
-   ( nvar   , ncesmp , nfbpcd , ncmast ,                          &
+   ( nvar   , iterns , ncesmp , nfbpcd , ncmast ,                 &
      icetsm , ifbpcd , ltmast , isostd ,                          &
      dt     , vel    ,                                            &
      coefav , coefbv , coefa_dp        , coefb_dp ,               &
@@ -194,7 +201,7 @@ interface
 
     ! Arguments
 
-    integer          nvar
+    integer          nvar  , iterns
     integer          ncesmp, nfbpcd, ncmast
 
     integer          icetsm(ncesmp), ifbpcd(nfbpcd)
@@ -229,10 +236,13 @@ end interface
 ! 0. Initialization
 !===============================================================================
 
+call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt_u)
+call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt_p)
+
 ! Allocate temporary arrays for the velocity-pressure resolution
-if (idften(iu).eq.1) then
+if (iand(vcopt_u%idften, ISOTROPIC_DIFFUSION).ne.0) then
   allocate(viscf(1, 1, nfac), viscb(ndimfb))
-else if (idften(iu).eq.6) then
+else if (iand(vcopt_u%idften, ANISOTROPIC_LEFT_DIFFUSION).ne.0) then
   allocate(viscf(3, 3, nfac), viscb(ndimfb))
 endif
 
@@ -249,7 +259,7 @@ if (iphydr.eq.2) then
 else
   grdphd => rvoid2
 endif
-if (idften(iu).eq.1) then
+if (iand(vcopt_u%idften, ISOTROPIC_DIFFUSION).ne.0) then
   if (itytur.eq.3.and.irijnu.eq.1) then
     allocate(wvisfi(1,1,nfac), wvisbi(ndimfb))
     viscfi => wvisfi(:,:,1:nfac)
@@ -258,7 +268,7 @@ if (idften(iu).eq.1) then
     viscfi => viscf(:,:,1:nfac)
     viscbi => viscb(1:ndimfb)
   endif
-else if(idften(iu).eq.6) then
+else if (iand(vcopt_u%idften, ANISOTROPIC_LEFT_DIFFUSION).ne.0) then
   if (itytur.eq.3.and.irijnu.eq.1) then
     allocate(wvisfi(3,3,nfac), wvisbi(ndimfb))
     viscfi => wvisfi(1:3,1:3,1:nfac)
@@ -287,16 +297,16 @@ endif
 call field_get_val_s(ivarfl(ipr), cvar_pr)
 call field_get_val_prev_s(ivarfl(ipr), cvara_pr)
 
-if (icavit.ge.0) then
-  call field_get_val_s(ivarfl(ivoidf), cvar_voidf)
-  call field_get_val_prev_s(ivarfl(ivoidf), cvara_voidf)
+if (ivofmt.ge.0) then
+  call field_get_val_s(ivarfl(ivolf2), cvar_voidf)
+  call field_get_val_prev_s(ivarfl(ivolf2), cvara_voidf)
 endif
 
 ! Allocate work arrays
 allocate(w1(ncelet))
 allocate(w7(ncelet), w8(ncelet), w9(ncelet))
 
-if (iwarni(iu).ge.1) then
+if (vcopt_u%iwarni.ge.1) then
   write(nfecra,1000)
 endif
 
@@ -356,8 +366,8 @@ if (nterup.gt.1) then
 endif
 
 ! --- Physical quantities
-call field_get_val_s(iprpfl(iviscl), viscl)
-call field_get_val_s(iprpfl(ivisct), visct)
+call field_get_val_s(iviscl, viscl)
+call field_get_val_s(ivisct, visct)
 
 ! Initialize timers
 t1 = 0.d0
@@ -370,7 +380,7 @@ t4 = 0.d0
 !===============================================================================
 
 if ((idilat.eq.2.or.idilat.eq.3).and. &
-    (ntcabs.gt.1.or.isuite.gt.0)) then
+    (ntcabs.gt.1.or.isuite.gt.0).and.ipredfl.ne.0) then
 
   call predfl(nvar, ncetsm, icetsm, dt, smacel)
 
@@ -382,7 +392,7 @@ endif
 
 if (iphydr.eq.2) then
 
-  call prehyd(prhyd, grdphd)
+  call prehyd(grdphd, iterns)
 
 endif
 
@@ -392,7 +402,7 @@ endif
 
 if ( ippmod(icompf).ge.0 ) then
 
-  if(iwarni(iu).ge.1) then
+  if(vcopt_u%iwarni.ge.1) then
     write(nfecra,1080)
   endif
 
@@ -410,7 +420,7 @@ endif
 
 if (icavit.ge.0) then
 
-  call field_get_val_s(iprpfl(iprtot), cpro_prtot)
+  call field_get_val_s(iprtot, cpro_prtot)
 
   call cavitation_compute_source_term (cpro_prtot, cvara_voidf)
 
@@ -454,6 +464,9 @@ call predvv &
   viscf  , viscb  , viscfi , viscbi , secvif , secvib ,          &
   w1     , w7     , w8     , w9     )
 
+! Bad cells regularisation
+call cs_bad_cells_regularisation_vector(vel, 1)
+
 ! --- Sortie si pas de pression continuite
 !       on met a jour les flux de masse, et on sort
 
@@ -464,11 +477,11 @@ if (iprco.le.0) then
   inc    = 1
   iflmb0 = 1
   if (iale.eq.1) iflmb0 = 0
-  nswrgp = nswrgr(iu)
-  imligp = imligr(iu)
-  iwarnp = iwarni(iu)
-  epsrgp = epsrgr(iu)
-  climgp = climgr(iu)
+  nswrgp = vcopt_u%nswrgr
+  imligp = vcopt_u%imligr
+  iwarnp = vcopt_u%iwarni
+  epsrgp = vcopt_u%epsrgr
+  climgp = vcopt_u%climgr
 
   call inimav                                                     &
  ( ivarfl(iu)      , itypfl ,                                     &
@@ -495,11 +508,12 @@ if (iprco.le.0) then
     init   = 1
     inc    = 1
     iflmb0 = 1
-    nswrgp = nswrgr(iuma)
-    imligp = imligr(iuma)
-    iwarnp = iwarni(iuma)
-    epsrgp = epsrgr(iuma)
-    climgp = climgr(iuma)
+    call field_get_key_struct_var_cal_opt(ivarfl(iuma), vcopt)
+    nswrgp = vcopt%nswrgr
+    imligp = vcopt%imligr
+    iwarnp = vcopt%iwarni
+    epsrgp = vcopt%epsrgr
+    climgp = vcopt%climgr
 
     call field_get_coefa_v(ivarfl(iuma), claale)
     call field_get_coefb_v(ivarfl(iuma), clbale)
@@ -515,27 +529,44 @@ if (iprco.le.0) then
     intflx , bouflx )
 
     ! Here we need of the opposite of the mesh velocity.
-    !$omp parallel do if(nfabor > thr_n_min)
     do ifac = 1, nfabor
-      bmasfl(ifac) = bmasfl(ifac) - bouflx(ifac)
-    enddo
-
-    !$omp parallel do private(ddepx, ddepy, ddepz, icpt, ii, inod, &
-    !$omp                     iel1, iel2, dtfac, rhofac)
-    do ifac = 1, nfac
-      ddepx = 0.d0
-      ddepy = 0.d0
-      ddepz = 0.d0
-      icpt  = 0
-      do ii = ipnfac(ifac),ipnfac(ifac+1)-1
-        inod = nodfac(ii)
-        icpt = icpt + 1
-        ddepx = ddepx + xyznod(1,inod)-xyzno0(1,inod) - disala(1,inod)
-        ddepy = ddepy + xyznod(2,inod)-xyzno0(2,inod) - disala(2,inod)
-        ddepz = ddepz + xyznod(3,inod)-xyzno0(3,inod) - disala(3,inod)
-      enddo
       ! Compute the mass flux using the nodes displacement
       if (iflxmw.eq.0) then
+        disp_fac(1) = 0.d0
+        disp_fac(2) = 0.d0
+        disp_fac(3) = 0.d0
+        icpt  = 0
+        do ii = ipnfbr(ifac),ipnfbr(ifac+1)-1
+          inod = nodfbr(ii)
+          icpt = icpt + 1
+          disp_fac(1) = disp_fac(1) + disale(1,inod) - (xyznod(1,inod)-xyzno0(1,inod))
+          disp_fac(2) = disp_fac(2) + disale(2,inod) - (xyznod(2,inod)-xyzno0(2,inod))
+          disp_fac(3) = disp_fac(3) + disale(3,inod) - (xyznod(3,inod)-xyzno0(3,inod))
+        enddo
+        iel = ifabor(ifac)
+        bmasfl(ifac) = bmasfl(ifac) - brom(ifac) * (              &
+              disp_fac(1) * surfbo(1,ifac)                        &
+             +disp_fac(2) * surfbo(2,ifac)                        &
+             +disp_fac(3) * surfbo(3,ifac) )/dt(iel)/icpt
+      else
+        bmasfl(ifac) = bmasfl(ifac) - bouflx(ifac)
+      endif
+    enddo
+
+    do ifac = 1, nfac
+      ! Compute the mass flux using the nodes displacement
+      if (iflxmw.eq.0) then
+        disp_fac(1) = 0.d0
+        disp_fac(2) = 0.d0
+        disp_fac(3) = 0.d0
+        icpt  = 0
+        do ii = ipnfac(ifac),ipnfac(ifac+1)-1
+          inod = nodfac(ii)
+          icpt = icpt + 1
+          disp_fac(1) = disp_fac(1) + disale(1,inod) - (xyznod(1,inod)-xyzno0(1,inod))
+          disp_fac(2) = disp_fac(2) + disale(2,inod) - (xyznod(2,inod)-xyzno0(2,inod))
+          disp_fac(3) = disp_fac(3) + disale(3,inod) - (xyznod(3,inod)-xyzno0(3,inod))
+        enddo
         ! For inner vertices, the mass flux due to the mesh displacement is
         !  recomputed from the nodes displacement
         iel1 = ifacel(1,ifac)
@@ -543,9 +574,9 @@ if (iprco.le.0) then
         dtfac = 0.5d0*(dt(iel1) + dt(iel2))
         rhofac = 0.5d0*(crom(iel1) + crom(iel2))
         imasfl(ifac) = imasfl(ifac) - rhofac*(                    &
-              ddepx*surfac(1,ifac)                                &
-             +ddepy*surfac(2,ifac)                                &
-             +ddepz*surfac(3,ifac) )/dtfac/icpt
+              disp_fac(1) * surfac(1,ifac)                        &
+             +disp_fac(2) * surfac(2,ifac)                        &
+             +disp_fac(3) * surfac(3,ifac) )/dtfac/icpt
       else
         imasfl(ifac) = imasfl(ifac) - intflx(ifac)
       endif
@@ -559,7 +590,7 @@ if (iprco.le.0) then
   ! Ajout de la vitesse du solide dans le flux convectif,
   ! si le maillage est mobile (solide rigide)
   ! En turbomachine, on connait exactement la vitesse de maillage a ajouter
-  if (imobil.eq.1 .or. iturbo.eq.1 .or. iturbo.eq.2) then
+  if (iturbo.ne.0) then
     !$omp parallel do private(iel1, iel2, dtfac, rhofac )
     do ifac = 1, nfac
       iel1 = ifacel(1,ifac)
@@ -612,120 +643,122 @@ if (iturbo.eq.2 .and. iterns.eq.1) then
 
   call dmtmps(t1)
 
-  do ifac = 1, nfabor
-    isympa(ifac) = 1
-  enddo
+  if (ityint.eq.0) then
 
-  ! Scratch and resize temporary internal faces arrays
-
-  deallocate(viscf)
-  if (idften(iu).eq.1) then
-    allocate(viscf(1, 1, nfac))
-  else if (idften(iu).eq.6) then
-    allocate(viscf(3, 3, nfac))
-  endif
-
-  if (allocated(wvisfi)) then
-    deallocate(viscfi)
-
-    if (idften(iu).eq.1) then
-      if (itytur.eq.3.and.irijnu.eq.1) then
-        allocate(wvisfi(1,1,nfac))
-        viscfi => wvisfi(:,:,1:nfac)
+    do ifac = 1, nfabor
+      ! --- To cancel the mass flux for symmetry BC
+      if (itypfb(ifac).eq.isymet) then
+        isympa(ifac) = 0
       else
-        viscfi => viscf(:,:,1:nfac)
+        isympa(ifac) = 1
       endif
-    else if(idften(iu).eq.6) then
-      if (itytur.eq.3.and.irijnu.eq.1) then
-        allocate(wvisfi(3,3,nfac))
-        viscfi => wvisfi(1:3,1:3,1:nfac)
-      else
-        viscfi => viscf(1:3,1:3,1:nfac)
+    enddo
+
+    ! Scratch and resize temporary internal faces arrays
+
+    deallocate(viscf)
+    if (iand(vcopt_u%idften, ISOTROPIC_DIFFUSION).ne.0) then
+      allocate(viscf(1, 1, nfac))
+    else if (iand(vcopt_u%idften, ANISOTROPIC_LEFT_DIFFUSION).ne.0) then
+      allocate(viscf(3, 3, nfac))
+    endif
+
+    if (allocated(wvisfi)) then
+      deallocate(viscfi)
+
+      if (vcopt_u%idften.eq.1) then
+        if (itytur.eq.3.and.irijnu.eq.1) then
+          allocate(wvisfi(1,1,nfac))
+          viscfi => wvisfi(:,:,1:nfac)
+        else
+          viscfi => viscf(:,:,1:nfac)
+        endif
+      else if(vcopt_u%idften.eq.6) then
+        if (itytur.eq.3.and.irijnu.eq.1) then
+          allocate(wvisfi(3,3,nfac))
+          viscfi => wvisfi(1:3,1:3,1:nfac)
+        else
+          viscfi => viscf(1:3,1:3,1:nfac)
+        endif
       endif
+
     endif
 
-  endif
-
-  if (allocated(secvif)) then
-    deallocate(secvif)
-    allocate(secvif(nfac))
-  endif
-
-  ! Scratch, resize and initialize main internal faces properties array
-
-  call turbomachinery_reinit_i_face_fields
-
-  ! Update local pointers on "internal faces" fields
-
-  call field_get_val_s(iflmas, imasfl)
-
-  if (irangp.ge.0 .or. iperio.eq.1) then
-
-    ! Scratch and resize work arrays
-
-    deallocate(w1, w7, w8, w9)
-    allocate(w1(ncelet), w7(ncelet), w8(ncelet), w9(ncelet))
-
-    ! Resize auxiliary arrays (pointe module)
-
-    call resize_aux_arrays
-
-    ! Resize main real array
-
-    call resize_main_real_array ( dt , propce )
-
-    ! Update turbomachinery module
-
-    call turbomachinery_update
-
-    ! Update field mappings ("owner" fields handled by turbomachinery_update)
-
-    call fldtri(nproce, dt, propce)
-
-    ! Resize other arrays related to the velocity-pressure resolution
-
-    call resize_vec_real_array(trav)
-    call resize_vec_real_array(dfrcxt)
-
-    ! Resize other arrays, depending on user options
-
-    if (iilagr.gt.0) &
-      call resize_n_sca_real_arrays(ntersl, tslagr)
-
-    if (iphydr.eq.1) then
-      call resize_vec_real_array(frcxt)
-    elseif (iphydr.eq.2) then
-      call resize_sca_real_array(prhyd)
-      call resize_vec_real_array(grdphd)
+    if (allocated(secvif)) then
+      deallocate(secvif)
+      allocate(secvif(nfac))
     endif
 
-    if (nterup.gt.1) then
-      call resize_vec_real_array(uvwk)
-      call resize_tens_real_array(ximpa)
-    endif
+    ! Scratch, resize and initialize main internal faces properties array
 
-    if (nterup.gt.1.or.isno2t.gt.0) then
-      call resize_vec_real_array(trava)
-    endif
+    call turbomachinery_reinit_i_face_fields
 
-    ! Update local pointers on "cells" fields
+    ! Update local pointers on "internal faces" fields
 
-    call field_get_val_s(icrom, crom)
+    call field_get_val_s(iflmas, imasfl)
 
-    call field_get_val_s(iprpfl(iviscl), viscl)
-    call field_get_val_s(iprpfl(ivisct), visct)
+    if (irangp.ge.0 .or. iperio.eq.1) then
 
-    call field_get_val_v(ivarfl(iu), vel)
-    call field_get_val_prev_v(ivarfl(iu), vela)
+      ! Scratch and resize work arrays
 
-    call field_get_val_s(ivarfl(ipr), cvar_pr)
-    call field_get_val_prev_s(ivarfl(ipr), cvara_pr)
+      deallocate(w1, w7, w8, w9)
+      allocate(w1(ncelet), w7(ncelet), w8(ncelet), w9(ncelet))
 
-    if (idtten.ge.0) call field_get_val_v(idtten, dttens)
+      ! Resize auxiliary arrays (pointe module)
+
+      call resize_aux_arrays
+
+      ! Update turbomachinery module
+
+      call turbomachinery_update
+
+      ! Update field mappings ("owner" fields handled by turbomachinery_update)
+
+      call fldtri
+      call field_get_val_s_by_name('dt', dt)
+
+      ! Resize other arrays related to the velocity-pressure resolution
+
+      call resize_vec_real_array(trav)
+      call resize_vec_real_array(dfrcxt)
+
+      ! Resize other arrays, depending on user options
+
+      if (iilagr.gt.0) &
+        call resize_n_sca_real_arrays(ntersl, tslagr)
+
+      if (iphydr.eq.1) then
+        call field_get_val_v_by_name('volume_forces', frcxt)
+      elseif (iphydr.eq.2) then
+        call resize_vec_real_array(grdphd)
+      endif
+
+      if (nterup.gt.1) then
+        call resize_vec_real_array(uvwk)
+        call resize_tens_real_array(ximpa)
+        call resize_vec_real_array(trava)
+      endif
+
+      ! Update local pointers on "cells" fields
+
+      call field_get_val_s(icrom, crom)
+
+      call field_get_val_s(iviscl, viscl)
+      call field_get_val_s(ivisct, visct)
+
+      call field_get_val_v(ivarfl(iu), vel)
+      call field_get_val_prev_v(ivarfl(iu), vela)
+
+      call field_get_val_s(ivarfl(ipr), cvar_pr)
+      call field_get_val_prev_s(ivarfl(ipr), cvara_pr)
+
+      if (idtten.ge.0) call field_get_val_v(idtten, dttens)
+
+      if (ivofmt.ge.0) then
+        call field_get_val_s(ivarfl(ivolf2), cvar_voidf)
+        call field_get_val_prev_s(ivarfl(ivolf2), cvara_voidf)
+      endif
 
-    if (icavit.ge.0) then
-      call field_get_val_s(ivarfl(ivoidf), cvar_voidf)
-      call field_get_val_prev_s(ivarfl(ivoidf), cvara_voidf)
     endif
 
   endif
@@ -791,7 +824,7 @@ endif
 ! 7. Pressure correction step
 !===============================================================================
 
-if (iwarni(iu).ge.1) then
+if (vcopt_u%iwarni.ge.1) then
   write(nfecra,1200)
 endif
 
@@ -801,7 +834,7 @@ allocate(dpvar(ncelet))
 if (ippmod(icompf).lt.0) then
 
   call resopv &
-( nvar   , ncetsm , nfbpcd , ncmast ,                            &
+( nvar   , iterns , ncetsm , nfbpcd , ncmast ,                   &
   icetsm , ifbpcd , ltmast , isostd ,                            &
   dt     , vel    ,                                              &
   coefau , coefbu , coefa_dp        , coefb_dp ,                 &
@@ -813,6 +846,9 @@ if (ippmod(icompf).lt.0) then
 
 endif
 
+! Bad cells regularisation
+call cs_bad_cells_regularisation_scalar(cvar_pr)
+
 !===============================================================================
 ! 8. Mesh velocity solving (ALE)
 !===============================================================================
@@ -820,7 +856,7 @@ endif
 if (iale.eq.1) then
 
   if (itrale.gt.nalinf) then
-    call alelav(propce)
+    call alelav(iterns)
   endif
 
 endif
@@ -829,8 +865,8 @@ endif
 ! 9. Update of the fluid velocity field
 !===============================================================================
 
-! Mass flux initialization for cavitating flows
-if (icavit.ge.0) then
+! Mass flux initialization for VOF algorithm
+if (ivofmt.ge.0) then
   do ifac = 1, nfac
     imasfl(ifac) = 0.d0
   enddo
@@ -854,7 +890,7 @@ if (ippmod(icompf).lt.0) then
     if (idtvar.lt.0) then
       !$omp parallel do
       do iel = 1, ncel
-        dpvar(iel) = (cvar_pr(iel) -cvara_pr(iel)) / relaxv(ipr)
+        dpvar(iel) = (cvar_pr(iel) -cvara_pr(iel)) / vcopt_p%relaxv
       enddo
     else
       !$omp parallel do
@@ -872,17 +908,17 @@ if (ippmod(icompf).lt.0) then
     iccocg = 1
     inc = 0
     if (iphydr.eq.1.or.iifren.eq.1) inc = 1
-    nswrgp = nswrgr(ipr)
-    imligp = imligr(ipr)
-    iwarnp = iwarni(ipr)
-    epsrgp = epsrgr(ipr)
-    climgp = climgr(ipr)
-    extrap = extrag(ipr)
+    nswrgp = vcopt_p%nswrgr
+    imligp = vcopt_p%imligr
+    iwarnp = vcopt_p%iwarni
+    epsrgp = vcopt_p%epsrgr
+    climgp = vcopt_p%climgr
+    extrap = vcopt_p%extrag
 
     !Allocation
     allocate(gradp(3, ncelet))
 
-    if (icavit.lt.0) then
+    if (ivofmt.lt.0) then
       call gradient_potential_s &
        (ivarfl(ipr)     , imrgra , inc    , iccocg , nswrgp , imligp , &
         iphydr , iwarnp , epsrgp , climgp , extrap ,                   &
@@ -897,14 +933,13 @@ if (ippmod(icompf).lt.0) then
 
       call gradient_weighted_s &
       ( ivarfl(ipr)     , imrgra , inc    , iccocg , nswrgp , imligp , &
-        iwarnp , epsrgp , climgp , extrap ,                            &
+        iphydr, iwarnp  , epsrgp , climgp , extrap , dfrcxt ,          &
         dpvar  , xinvro , coefa_dp , coefb_dp ,                        &
         gradp  )
 
       deallocate(xinvro)
     endif
 
-    thetap = thetav(ipr)
     !$omp parallel do private(isou)
     do iel = 1, ncelet
       do isou = 1, 3
@@ -917,14 +952,14 @@ if (ippmod(icompf).lt.0) then
 
     ! Update the velocity field
     !--------------------------
-    thetap = thetav(ipr)
+    thetap = vcopt_p%thetav
 
     ! Specific handling of hydrostatic pressure
     !------------------------------------------
     if (iphydr.eq.1) then
 
       ! Scalar diffusion for the pressure
-      if (idften(ipr).eq.1) then
+      if (iand(vcopt_p%idften, ISOTROPIC_DIFFUSION).ne.0) then
         !$omp parallel do private(dtsrom, isou)
         do iel = 1, ncel
           dtsrom = thetap*dt(iel)/crom(iel)
@@ -935,7 +970,7 @@ if (ippmod(icompf).lt.0) then
         enddo
 
       ! Tensorial diffusion for the pressure
-      else if (idften(ipr).eq.6) then
+      elseif (iand(vcopt_p%idften, ANISOTROPIC_DIFFUSION).ne.0) then
         !$omp parallel do private(unsrom)
         do iel = 1, ncel
           unsrom = thetap/crom(iel)
@@ -976,7 +1011,13 @@ if (ippmod(icompf).lt.0) then
       call field_get_coefa_s(ivarfl(ipr), coefa_p)
       !$omp parallel do if(nfabor > thr_n_min)
       do ifac = 1, nfabor
-        if (isostd(ifac).eq.1) then
+        iautof = 0
+        ! automatic inlet/outlet face for atmospheric flow
+        if (imeteo.gt.0) then
+          iautof = iautom(ifac)
+        endif
+
+        if (isostd(ifac).eq.1.or.iatmst.eq.1.and.iautof.eq.1) then
           coefa_p(ifac) = coefa_p(ifac) + coefa_dp(ifac)
         endif
       enddo
@@ -987,18 +1028,18 @@ if (ippmod(icompf).lt.0) then
     else
 
       ! Scalar diffusion for the pressure
-      if (idften(ipr).eq.1) then
+      if (iand(vcopt_p%idften, ISOTROPIC_DIFFUSION).ne.0) then
 
-      !$omp parallel do private(dtsrom, isou)
-      do iel = 1, ncel
-        dtsrom = thetap*dt(iel)/crom(iel)
-        do isou = 1, 3
-          vel(isou,iel) = vel(isou,iel) - dtsrom*trav(isou,iel)
+        !$omp parallel do private(dtsrom, isou)
+        do iel = 1, ncel
+          dtsrom = thetap*dt(iel)/crom(iel)
+          do isou = 1, 3
+            vel(isou,iel) = vel(isou,iel) - dtsrom*trav(isou,iel)
+          enddo
         enddo
-       enddo
 
       ! Tensorial diffusion for the pressure
-      else if (idften(ipr).eq.6) then
+      else if (iand(vcopt_p%idften, ANISOTROPIC_DIFFUSION).ne.0) then
 
         !$omp parallel do private(unsrom)
         do iel = 1, ncel
@@ -1030,11 +1071,15 @@ if (ippmod(icompf).lt.0) then
 
 endif
 
+! Bad cells regularisation
+call cs_bad_cells_regularisation_vector(vel, 1)
+
 ! In the ALE framework, we add the mesh velocity
 if (iale.eq.1) then
 
   call field_get_val_v(ivarfl(iuma), mshvel)
 
+  call field_get_val_v(fdiale, disale)
   call field_get_val_prev_v(fdiale, disala)
 
   ! One temporary array needed for internal faces, in case some internal vertices
@@ -1045,11 +1090,12 @@ if (iale.eq.1) then
   init   = 1
   inc    = 1
   iflmb0 = 1
-  nswrgp = nswrgr(iuma)
-  imligp = imligr(iuma)
-  iwarnp = iwarni(iuma)
-  epsrgp = epsrgr(iuma)
-  climgp = climgr(iuma)
+  call field_get_key_struct_var_cal_opt(ivarfl(iuma), vcopt)
+  nswrgp = vcopt%nswrgr
+  imligp = vcopt%imligr
+  iwarnp = vcopt%iwarni
+  epsrgp = vcopt%epsrgr
+  climgp = vcopt%climgr
 
   call field_get_coefa_v(ivarfl(iuma), claale)
   call field_get_coefb_v(ivarfl(iuma), clbale)
@@ -1067,35 +1113,54 @@ if (iale.eq.1) then
   ! Here we need of the opposite of the mesh velocity.
   !$omp parallel do if(nfabor > thr_n_min)
   do ifac = 1, nfabor
-    bmasfl(ifac) = bmasfl(ifac) - bouflx(ifac)
-  enddo
-
-  !$omp parallel do private(ddepx, ddepy, ddepz, icpt, ii, inod, &
-  !$omp                     iel1, iel2, dtfac, rhofac)
-  do ifac = 1, nfac
-    ddepx = 0.d0
-    ddepy = 0.d0
-    ddepz = 0.d0
-    icpt  = 0
-    do ii = ipnfac(ifac),ipnfac(ifac+1)-1
-      inod = nodfac(ii)
-      icpt = icpt + 1
-      ddepx = ddepx + xyznod(1,inod)-xyzno0(1,inod) - disala(1,inod)
-      ddepy = ddepy + xyznod(2,inod)-xyzno0(2,inod) - disala(2,inod)
-      ddepz = ddepz + xyznod(3,inod)-xyzno0(3,inod) - disala(3,inod)
-    enddo
     ! Compute the mass flux using the nodes displacement
     if (iflxmw.eq.0) then
+      disp_fac(1) = 0.d0
+      disp_fac(2) = 0.d0
+      disp_fac(3) = 0.d0
+      icpt  = 0
+      do ii = ipnfbr(ifac),ipnfbr(ifac+1)-1
+        inod = nodfbr(ii)
+        icpt = icpt + 1
+        disp_fac(1) = disp_fac(1) + disale(1,inod) - (xyznod(1,inod)-xyzno0(1,inod))
+        disp_fac(2) = disp_fac(2) + disale(2,inod) - (xyznod(2,inod)-xyzno0(2,inod))
+        disp_fac(3) = disp_fac(3) + disale(3,inod) - (xyznod(3,inod)-xyzno0(3,inod))
+      enddo
+      iel = ifabor(ifac)
+      bmasfl(ifac) = bmasfl(ifac) - brom(ifac) * (           &
+         disp_fac(1) * surfbo(1,ifac)                        &
+        +disp_fac(2) * surfbo(2,ifac)                        &
+        +disp_fac(3) * surfbo(3,ifac) )/dt(iel)/icpt
+    else
+      bmasfl(ifac) = bmasfl(ifac) - bouflx(ifac)
+    endif
+  enddo
+
+  do ifac = 1, nfac
+    ! Compute the mass flux using the nodes displacement
+    if (iflxmw.eq.0) then
+      disp_fac(1) = 0.d0
+      disp_fac(2) = 0.d0
+      disp_fac(3) = 0.d0
+      icpt  = 0
+      do ii = ipnfac(ifac),ipnfac(ifac+1)-1
+        inod = nodfac(ii)
+        icpt = icpt + 1
+        disp_fac(1) = disp_fac(1) + disale(1,inod) - (xyznod(1,inod)-xyzno0(1,inod))
+        disp_fac(2) = disp_fac(2) + disale(2,inod) - (xyznod(2,inod)-xyzno0(2,inod))
+        disp_fac(3) = disp_fac(3) + disale(3,inod) - (xyznod(3,inod)-xyzno0(3,inod))
+      enddo
+
       ! For inner vertices, the mass flux due to the mesh displacement is
       !  recomputed from the nodes displacement
       iel1 = ifacel(1,ifac)
       iel2 = ifacel(2,ifac)
       dtfac = 0.5d0*(dt(iel1) + dt(iel2))
       rhofac = 0.5d0*(crom(iel1) + crom(iel2))
-      imasfl(ifac) = imasfl(ifac) - rhofac*(      &
-            ddepx*surfac(1,ifac)                                &
-           +ddepy*surfac(2,ifac)                                &
-           +ddepz*surfac(3,ifac) )/dtfac/icpt
+      imasfl(ifac) = imasfl(ifac) - rhofac*(                    &
+            disp_fac(1) * surfac(1,ifac)                        &
+           +disp_fac(2) * surfac(2,ifac)                        &
+           +disp_fac(3) * surfac(3,ifac) )/dtfac/icpt
     else
       imasfl(ifac) = imasfl(ifac) - intflx(ifac)
     endif
@@ -1110,11 +1175,11 @@ endif
 ! Ajout de la vitesse du solide dans le flux convectif,
 ! si le maillage est mobile (solide rigide)
 ! En turbomachine, on connait exactement la vitesse de maillage a ajouter
-if (imobil.eq.1 .or. iturbo.eq.1 .or. iturbo.eq.2) then
+if (iturbo.ne.0) then
 
-  if (iturbo.eq.1.or.iturbo.eq.2) call dmtmps(t3)
+  call dmtmps(t3)
 
-  !$omp parallel do private(iel1, iel2, rhofac)
+  !$omp parallel do private(iel1, iel2, rhofac, vr1, vr2)
   do ifac = 1, nfac
     iel1 = ifacel(1,ifac)
     iel2 = ifacel(2,ifac)
@@ -1130,7 +1195,7 @@ if (imobil.eq.1 .or. iturbo.eq.1 .or. iturbo.eq.2) then
                         + surfac(3,ifac)*(vr1(3) + vr2(3)) )
     endif
   enddo
-  !$omp parallel do private(iel, rhofac) if(nfabor > thr_n_min)
+  !$omp parallel do private(iel, rhofac, vr) if(nfabor > thr_n_min)
   do ifac = 1, nfabor
     iel = ifabor(ifac)
     if (irotce(iel).ne.0) then
@@ -1144,22 +1209,22 @@ if (imobil.eq.1 .or. iturbo.eq.1 .or. iturbo.eq.2) then
     endif
   enddo
 
-  if (iturbo.eq.1.or.iturbo.eq.2) call dmtmps(t4)
+  call dmtmps(t4)
 
   rs_ell(2) = rs_ell(2) + t4-t3
 
 endif
 
 !===============================================================================
-! 10. Cavitation: void fraction solving and update the mixture density/viscosity
+! 10. VoF: void fraction solving and update the mixture density/viscosity
 !      and mass flux (resopv solved the convective flux of void fraction, divU)
 !===============================================================================
 
-if (icavit.ge.0) then
+if (ivofmt.ge.0) then
 
   ! Void fraction solving
 
-  call resvoi(dt)
+  call resvoi(dt, iterns)
 
   ! Halo synchronization
 
@@ -1167,21 +1232,21 @@ if (icavit.ge.0) then
 
   ! Get the void fraction boundary conditions
 
-  call field_get_coefa_s(ivarfl(ivoidf), coavoi)
-  call field_get_coefb_s(ivarfl(ivoidf), cobvoi)
+  call field_get_coefa_s(ivarfl(ivolf2), coavoi)
+  call field_get_coefb_s(ivarfl(ivolf2), cobvoi)
 
   ! Get the convective flux of the void fraction
 
-  call field_get_key_int(ivarfl(ivoidf), kimasf, iflvoi)
-  call field_get_key_int(ivarfl(ivoidf), kbmasf, iflvob)
+  call field_get_key_int(ivarfl(ivolf2), kimasf, iflvoi)
+  call field_get_key_int(ivarfl(ivolf2), kbmasf, iflvob)
   call field_get_val_s(iflvoi, ivoifl)
   call field_get_val_s(iflvob, bvoifl)
 
   ! Update mixture density/viscosity and mass flux
 
-  call cavitation_update_phys_prop &
+  call vof_update_phys_prop &
  ( cvar_voidf, coavoi, cobvoi, ivoifl, bvoifl, &
-   crom, brom, propce(:,ipproc(iviscl)), imasfl, bmasfl )
+   crom, brom, imasfl, bmasfl )
 
   ! Verbosity
 
@@ -1189,7 +1254,7 @@ if (icavit.ge.0) then
 
     call field_get_val_prev_s(icrom, croma)
 
-    call cavitation_print_mass_budget &
+    call vof_print_mass_budget &
    ( crom, croma, brom, dt, imasfl, bmasfl )
 
   endif
@@ -1238,11 +1303,11 @@ if (iestim(iescor).ge.0.or.iestim(iestot).ge.0) then
   inc    = 1
   iflmb0 = 1
   if (iale.eq.1) iflmb0 = 0
-  nswrgp = nswrgr(iu)
-  imligp = imligr(iu)
-  iwarnp = iwarni(iu)
-  epsrgp = epsrgr(iu)
-  climgp = climgr(iu)
+  nswrgp = vcopt_u%nswrgr
+  imligp = vcopt_u%imligr
+  iwarnp = vcopt_u%iwarni
+  epsrgp = vcopt_u%epsrgr
+  climgp = vcopt_u%climgr
 
   call inimav                                                     &
  ( ivarfl(iu)      , itypfl ,                                     &
@@ -1375,30 +1440,46 @@ endif
 ! En compressible, la pression resolue est deja la pression totale
 
 if (ippmod(icompf).lt.0) then
-  call field_get_val_s(iprpfl(iprtot), cpro_prtot)
+  call field_get_val_s(iprtot, cpro_prtot)
   xxp0   = xyzp0(1)
   xyp0   = xyzp0(2)
   xzp0   = xyzp0(3)
-  do iel=1,ncel
-    cpro_prtot(iel)= cvar_pr(iel)               &
-         + ro0*( gx*(xyzcen(1,iel)-xxp0)               &
-         + gy*(xyzcen(2,iel)-xyp0)                     &
-         + gz*(xyzcen(3,iel)-xzp0) )                   &
-         + p0 - pred0
-  enddo
+
+  if (iatmst.eq.0) then
+    do iel=1,ncel
+      cpro_prtot(iel)= cvar_pr(iel)               &
+           + ro0*( gx*(xyzcen(1,iel)-xxp0)               &
+           + gy*(xyzcen(2,iel)-xyp0)                     &
+           + gz*(xyzcen(3,iel)-xzp0) )                   &
+           + p0 - pred0
+    enddo
+  else
+    call field_get_val_v(imomst, cpro_momst)
+
+    do iel=1,ncel
+      cpro_prtot(iel)= cvar_pr(iel)                      &
+           + ro0*(gx*(xyzcen(1,iel)-xxp0)                &
+           + gy*(xyzcen(2,iel)-xyp0)                     &
+           + gz*(xyzcen(3,iel)-xzp0))                    &
+           + p0 - pred0                                  &
+           - cpro_momst(1,iel)*(xyzcen(1,iel)-xxp0)      &
+           - cpro_momst(2,iel)*(xyzcen(2,iel)-xyp0)      &
+           - cpro_momst(3,iel)*(xyzcen(3,iel)-xzp0)
+    enddo
+  endif
 endif
 
 !===============================================================================
 ! 13. Printing
 !===============================================================================
 
-if (iwarni(iu).ge.1) then
+if (vcopt_u%iwarni.ge.1) then
 
   write(nfecra,2000)
 
   rnorm = -1.d0
   do iel = 1, ncel
-    rnorm  = max(rnorm,abs(cvar_pr(iel)))
+    rnorm = max(rnorm,abs(cvar_pr(iel)))
   enddo
   if (irangp.ge.0) call parmax (rnorm)
 
@@ -1430,7 +1511,32 @@ if (iwarni(iu).ge.1) then
 
   write(nfecra,2200) rnorm,xyzmax(1),xyzmax(2),xyzmax(3)
 
-  ! Pour la periodicite et le parallelisme, rom est echange dans phyvar
+  rnorm = sqrt(vel(1,1)**2+vel(2,1)**2+vel(3,1)**2)
+  imin = 1
+  rnormt = rnorm
+  do iel = 1, ncel
+    vitnor = sqrt(vel(1,iel)**2+vel(2,iel)**2+vel(3,iel)**2)
+    if (vitnor.le.rnormt) then
+      rnormt = vitnor
+      imint  = iel
+    endif
+  enddo
+  if (rnormt .lt. rnorm) then
+    rnorm = rnormt
+    imin = imint
+  endif
+
+  xyzmin(1) = xyzcen(1,imin)
+  xyzmin(2) = xyzcen(2,imin)
+  xyzmin(3) = xyzcen(3,imin)
+
+  if (irangp.ge.0) then
+    nbrval = 3
+    call parmnl (nbrval, rnorm, xyzmin)
+  endif
+
+  write(nfecra,2201) rnorm,xyzmin(1),xyzmin(2),xyzmin(3)
+
 
   ! With porosity
   if (iporos.ge.1) then
@@ -1560,6 +1666,8 @@ deallocate(coefa_dp, coefb_dp)
 ' Pression max.',E12.4   ,' (max. de la valeur absolue)       ',/)
  2200 format(                                                           &
 ' Vitesse  max.',E12.4   ,' en',3E11.3                         ,/)
+ 2201 format(                                                           &
+' Vitesse  min.',E12.4   ,' en',3E11.3                         ,/)
  2300 format(                                                           &
 ' Vitesse  en face interne max.',E12.4   ,' ; min.',E12.4        )
  2400 format(                                                           &
@@ -1597,7 +1705,9 @@ deallocate(coefa_dp, coefb_dp)
  2100 format(                                                           &
 ' Max. pressure',E12.4   ,' (max. absolute value)'             ,/)
  2200 format(                                                           &
-' Max. velocity',E12.4   ,' en',3E11.3                         ,/)
+' Max. velocity',E12.4   ,' in',3E11.3                         ,/)
+ 2201 format(                                                           &
+' Min. velocity',E12.4   ,' in',3E11.3                         ,/)
  2300 format(                                                           &
 ' Max. velocity at interior face',E12.4   ,' ; min.',E12.4       )
  2400 format(                                                           &
