@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -36,16 +36,6 @@ module paramx
 
   !=============================================================================
 
-  ! npromx : nombre max de proprietes physiques aux cellules (total) =
-  !          nscamx (Lambda) + 7 (rho,Cp,viscl,visct,cou,fou,iprtot)
-  !                          + 4 (estim)
-  ! ngrdmx : nombre max de grandeurs =
-  !          nvarmx + npromx
-  ! nsmamx : nombre max de cases pour les tableaux termes source de masse
-  !          nvarmx + 1 pour smacel
-  ! nvppmx : nombre de variables pour affichages
-  !          ngrdmx + 20 (20 couvre dt, tpucou, et une marge de 16 ...)
-
   !> maximum number of scalars solutions of an
   !> advection equation, apart from the variables of the turbulence model
   !> \f$ (k, \varepsilon, R_{ij}, \omega, \varphi, \overline{f}, \alpha, \nu_t\f$)
@@ -56,28 +46,8 @@ module paramx
   !> maximal number of variables = nscamx + 12 (u,v,w,P,Rij,e,alp)
   integer   nvarmx
 
-  !> maximal number of physical properties at cells.
-  !> = nscamx (Lambda) + 100 (margin for most specific physics)
-  integer   npromx
-
-  !> maximal number of physical quantities
-  !> = nvarmx + npromx
-  integer   ngrdmx
-
-  !> maximal size of mass source terms arrays.
-  !> (= nvarmx + 1 for smacel)
-  integer   nsmamx
-
-  !> number of displayed variables.
-  !> = ngrdmx + 20 (20 > dt, tpucou, increased by 16 ...)
-  integer   nvppmx
-
   parameter(nscamx=200)
   parameter(nvarmx=nscamx+12)
-  parameter(npromx=nscamx+100)
-  parameter(ngrdmx=nvarmx+npromx)
-  parameter(nsmamx=nvarmx+1)
-  parameter(nvppmx=ngrdmx+20)
 
   !> Maximal possible boundary condition types
   integer    ntypmx
@@ -201,7 +171,7 @@ module paramx
   !> code/code coupling condition
   integer   icscpl
 
-  !> \anchor icscpl
+  !> \anchor icscpd
   !> code/code coupling condition with decentered flux
   integer   icscpd
 
@@ -367,6 +337,7 @@ module paramx
   !> maximum number of structures in ALE
   integer    nstrmx
   parameter (nstrmx=200)
+
   !=============================================================================
 
   !> \}
@@ -375,9 +346,6 @@ module paramx
   ! do not compile if the deprecated name (now used for the C routine) is used.
   integer cs_user_boundary_conditions, cs_user_parameters,     &
           cs_user_initialization, cs_user_physical_properties, &
-          cs_user_extra_operations
-
-
-
+          cs_user_extra_operations, ushist
 
 end module paramx
