@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -41,14 +41,17 @@
 !______________________________________________________________________________!
 !> \param[in]     nvar          total number of variables
 !> \param[in]     nscal         total number of scalars
+!> \param[in]     iterns        Navier-Stokes sub-iterations indicator:
+!>                              - if strictly negative, indicate that this
+!>                                function is called outside Navier-Stokes loop
+!>                              - if positive, Navier-Stokes iteration number.
 !> \param[in]     dt            time step (per cell)
-!> \param[in]     propce        physical properties at cell centers
 !_______________________________________________________________________________
 
 
 subroutine phyvar &
  ( nvar   , nscal  ,                                              &
-   dt     , propce )
+   iterns , dt     )
 
 !===============================================================================
 
@@ -73,7 +76,9 @@ use ppincl
 use mesh
 use field
 use cavitation
+use vof
 use darcy_module
+use cs_c_bindings
 
 !===============================================================================
 
@@ -81,19 +86,16 @@ implicit none
 
 ! Arguments
 
-integer          nvar   , nscal
-
+integer          nvar, nscal, iterns
 double precision dt(ncelet)
-double precision propce(ncelet,*)
 
 ! Local variables
 
 character(len=80) :: chaine
 integer          ivar  , iel   , ifac  , iscal
-integer          ii    , iok   , iok1  , iok2  , iisct, idfm, iggafm
+integer          ii    , iok   , iok1  , iok2  , iisct, idfm, iggafm, iebdfm
 integer          nn    , isou
-integer          mbrom , ipcvst, ifcvsl
-integer          ipccp , ipcvis, ipcvma
+integer          mbrom , ifcvsl
 integer          iclipc
 Double precision xk, xe, xnu, xrom, vismax(nscamx), vismin(nscamx)
 double precision nusa, xi3, fv1, cv13
@@ -106,14 +108,17 @@ double precision, dimension(:), pointer :: cvar_r11, cvar_r22, cvar_r33
 double precision, dimension(:), pointer :: cvar_r12, cvar_r13, cvar_r23
 double precision, dimension(:,:), pointer :: cvar_rij
 double precision, dimension(:), pointer :: sval
-double precision, dimension(:,:), pointer :: visten, vistes
+double precision, dimension(:,:), pointer :: visten, vistes, cpro_visma_v
 double precision, dimension(:), pointer :: viscl, visct, cpro_vis
 double precision, dimension(:), pointer :: cvar_voidf
+double precision, dimension(:), pointer :: cpro_var, cpro_beta, cpro_visma_s
 
 integer          ipass
 data             ipass /0/
 save             ipass
 
+type(var_cal_opt) :: vcopt
+
 !===============================================================================
 
 !===============================================================================
@@ -127,7 +132,8 @@ ipass = ipass + 1
 !===============================================================================
 
 if (iperot.gt.0 .and. itytur.eq.3) then
-  call perinr(imrgra, iwarni(ir11), epsrgr(ir11), extrag(ir11))
+  call field_get_key_struct_var_cal_opt(ivarfl(ir11), vcopt)
+  call perinr(imrgra, vcopt%iwarni, vcopt%epsrgr, vcopt%extrag)
 endif
 
 !===============================================================================
@@ -155,7 +161,8 @@ if (iihmpr.eq.1) then
     ( darcy_anisotropic_permeability,                                     &
       darcy_anisotropic_dispersion,                                       &
       darcy_gravity,                                                      &
-      darcy_gravity_x, darcy_gravity_y, darcy_gravity_z)
+      darcy_gravity_x, darcy_gravity_y, darcy_gravity_z,                  &
+      darcy_unsaturated)
   endif
 endif
 
@@ -168,7 +175,7 @@ call usphyv &
 
 ! Finalization of physical properties for specific physics
 ! AFTER the user
-if (ippmod(iphpar).ge.1) then
+if (ippmod(iphpar).ge.1 .or. ippmod(idarcy).ge.1) then
   call cs_physical_properties2
 endif
 
@@ -183,13 +190,26 @@ if (mbrom.eq.0) then
   enddo
 endif
 
+!
+! Parallelism and periodicity
+!
+! In navstv and visecv, we need rho in the halo
+if (irangp.ge.0.or.iperio.eq.1) then
+  call field_get_val_s(icrom, crom)
+  call synsca(crom)
+endif
+
+!
+! Only density may be updated in Navier Stokes loop
+if (iterns.ge.1) return
+
 !  Au premier pas de temps du calcul
 !     Si on a indique que rho (visc) etait constant
 !       et qu'on l'a modifie dans usphyv, ca ne va pas
 !     On se sert de irovar (ivivar) pour ecrire et lire
 !       rho (visc) dans le fichier suite
 
-if (ntcabs.eq.ntpabs+1 .and. icavit.lt.0) then
+if (ntcabs.eq.ntpabs+1 .and. ivofmt.lt.0) then
 
   ! Masse volumique aux cellules et aux faces de bord
   iok1 = 0
@@ -214,7 +234,7 @@ if (ntcabs.eq.ntpabs+1 .and. icavit.lt.0) then
   ! Viscosite moleculaire aux cellules
   iok2 = 0
   if (ivivar.eq.0) then
-    call field_get_val_s(iprpfl(iviscl), viscl)
+    call field_get_val_s(iviscl, viscl)
     do iel = 1, ncel
       if ( abs(viscl(iel)-viscl0).gt.epzero) then
         iok2 = 1
@@ -244,7 +264,7 @@ if     (iturb.eq. 0) then
 ! 4.1 Laminar
 ! ===========
 
-  call field_get_val_s(iprpfl(ivisct), visct)
+  call field_get_val_s(ivisct, visct)
 
   do iel = 1, ncel
     visct(iel) = 0.d0
@@ -262,7 +282,7 @@ elseif (itytur.eq.2) then
 ! 4.3 k-epsilon
 ! =============
 
-  call field_get_val_s(iprpfl(ivisct), visct)
+  call field_get_val_s(ivisct, visct)
   call field_get_val_s(icrom, crom)
   call field_get_val_s(ivarfl(ik), cvar_k)
   call field_get_val_s(ivarfl(iep), cvar_ep)
@@ -278,7 +298,7 @@ elseif (itytur.eq.3) then
 ! 4.4 Rij-epsilon
 ! ===============
 
-  call field_get_val_s(iprpfl(ivisct), visct)
+  call field_get_val_s(ivisct, visct)
   call field_get_val_s(icrom, crom)
   call field_get_val_s(ivarfl(iep), cvar_ep)
 
@@ -318,8 +338,7 @@ elseif (iturb.eq.41) then
    ncepdc , ncetsm ,                                              &
    icepdc , icetsm , itypsm ,                                     &
    dt     ,                                                       &
-   ckupdc , smacel ,                                              &
-   propce(1,ipproc(ismago)) )
+   ckupdc , smacel )
 
 elseif (iturb.eq.42) then
 
@@ -335,8 +354,8 @@ elseif (itytur.eq.5) then
 
   if (iturb.eq.50) then
 
-    call field_get_val_s(iprpfl(iviscl), viscl)
-    call field_get_val_s(iprpfl(ivisct), visct)
+    call field_get_val_s(iviscl, viscl)
+    call field_get_val_s(ivisct, visct)
     call field_get_val_s(icrom, crom)
     call field_get_val_s(ivarfl(ik), cvar_k)
     call field_get_val_s(ivarfl(iep), cvar_ep)
@@ -374,9 +393,9 @@ elseif (iturb.eq.70) then
   cv13 = csav1**3
 
   call field_get_val_s(ivarfl(inusa), cvar_nusa)
-  call field_get_val_s(iprpfl(ivisct), visct)
+  call field_get_val_s(ivisct, visct)
   call field_get_val_s(icrom, crom)
-  call field_get_val_s(iprpfl(iviscl), viscl)
+  call field_get_val_s(iviscl, viscl)
 
   do iel = 1, ncel
     xrom = crom(iel)
@@ -393,9 +412,11 @@ endif
 !===============================================================================
 idfm = 0
 iggafm = 0
+iebdfm = 0
 
 do iscal = 1, nscal
   if (ityturt(iscal).eq.3) idfm = 1
+  if (iturt(iscal).eq.31) iebdfm = 1
   ! GGDH or AFM on current scalar
   ! and if DFM, GGDH on the scalar variance
   if (ityturt(iscal).gt.0) iggafm = 1
@@ -408,7 +429,7 @@ if (idfm.eq.1 .or. itytur.eq.3 .and. idirsm.eq.1) then
   if (itytur.eq.3) then
     if (irijco.eq.1) then
       call field_get_val_s(icrom, crom)
-      call field_get_val_s(iprpfl(iviscl), viscl)
+      call field_get_val_s(iviscl, viscl)
 
       call field_get_val_s(ivarfl(iep), cvar_ep)
 
@@ -429,8 +450,21 @@ if (idfm.eq.1 .or. itytur.eq.3 .and. idirsm.eq.1) then
           enddo
         enddo
 
-        ! No damping with Durbing scale for the scalar
-        if (iggafm.eq.1) then
+        ! Other damping for EBDFM model (see F. Dehoux thesis)
+        if (iebdfm.eq.1) then
+          call field_get_val_v(ivstes, vistes) !FIXME one by scalar
+
+          do iel = 1, ncel
+            trrij = 0.5d0*(cvar_rij(1,iel)+cvar_rij(2,iel)+cvar_rij(3,iel))
+            rottke  = csrij * crom(iel) * trrij / cvar_ep(iel)
+
+            do isou = 1, 6
+              vistes(isou, iel) = rottke*cvar_rij(isou, iel)
+            enddo
+          enddo
+
+          ! No damping with Durbing scale for the scalar
+        else if (iggafm.eq.1) then
           call field_get_val_v(ivstes, vistes)
 
           do iel = 1, ncel
@@ -458,7 +492,7 @@ if (idfm.eq.1 .or. itytur.eq.3 .and. idirsm.eq.1) then
     ! Uncoupled version
     else
       call field_get_val_s(icrom, crom)
-      call field_get_val_s(iprpfl(iviscl), viscl)
+      call field_get_val_s(iviscl, viscl)
 
       call field_get_val_s(ivarfl(iep), cvar_ep)
 
@@ -487,8 +521,29 @@ if (idfm.eq.1 .or. itytur.eq.3 .and. idirsm.eq.1) then
           visten(6,iel) = rottke*cvar_r13(iel)
         enddo
 
-        ! No damping with Durbing scale for the scalar
-        if (iggafm.eq.1) then
+        ! Other damping for EBDFM model (see F. Dehoux thesis)
+        if (iebdfm.eq.1) then
+          call field_get_val_v(ivstes, vistes)
+
+          do iel = 1, ncel
+            trrij = 0.5d0*(cvar_r11(iel)+cvar_r22(iel)+cvar_r33(iel))
+            ttke  = trrij/cvar_ep(iel)
+            ! Durbin scale
+            xttkmg = xct*sqrt(viscl(iel)/crom(iel)/cvar_ep(iel))
+            xttdrb = max(ttke,xttkmg)
+            !FIXME xttdrbt = xttdrb*sqrt((1.d0-alpha3)*PR/XRH + alpha3)
+            rottke  = csrij * crom(iel) * xttdrb
+
+            vistes(1,iel) = rottke*cvar_r11(iel)
+            vistes(2,iel) = rottke*cvar_r22(iel)
+            vistes(3,iel) = rottke*cvar_r33(iel)
+            vistes(4,iel) = rottke*cvar_r12(iel)
+            vistes(5,iel) = rottke*cvar_r23(iel)
+            vistes(6,iel) = rottke*cvar_r13(iel)
+          enddo
+
+          ! No damping with Durbing scale for the scalar
+        else if (iggafm.eq.1) then
           call field_get_val_v(ivstes, vistes)
 
           do iel = 1, ncel
@@ -541,11 +596,11 @@ endif
 if (icavit.ge.0 .and. icvevm.eq.1) then
   if (itytur.eq.2 .or. itytur.eq.5 .or. iturb.eq.60 .or. iturb.eq.70) then
 
-    ipcvst = ipproc(ivisct)
     call field_get_val_s(icrom, crom)
-    call field_get_val_s(ivarfl(ivoidf), cvar_voidf)
+    call field_get_val_s(ivarfl(ivolf2), cvar_voidf)
+    call field_get_val_s(ivisct, visct)
 
-    call cavitation_correct_visc_turb (crom, cvar_voidf, propce(:,ipcvst))
+    call cavitation_correct_visc_turb (crom, cvar_voidf, visct)
 
   endif
 endif
@@ -571,9 +626,11 @@ call usvist &
 ! La diffusivite turbulente des scalaires (mu_t/sigma), elle, sera clippee a 0
 ! dans covofi
 
+call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt)
+
 if (iturb.eq.41) then
-  call field_get_val_s(iprpfl(iviscl), viscl)
-  call field_get_val_s(iprpfl(ivisct), visct)
+  call field_get_val_s(iviscl, viscl)
+  call field_get_val_s(ivisct, visct)
   iclipc = 0
   do iel = 1, ncel
     viscto = viscl(iel) + visct(iel)
@@ -582,7 +639,7 @@ if (iturb.eq.41) then
       iclipc = iclipc + 1
     endif
   enddo
-  if (iwarni(iu).ge.1) then
+  if (vcopt%iwarni.ge.1) then
     if (irangp.ge.0) then
       call parcpt(iclipc)
     endif
@@ -601,18 +658,13 @@ if (iale.eq.1.and.ntcabs.eq.0) then
 
   if (iihmpr.eq.1) then
 
-    call uivima &
-  ( propce(1,ipproc(ivisma(1))),      &
-    propce(1,ipproc(ivisma(2))),      &
-    propce(1,ipproc(ivisma(3))) )
+    call uivima
 
   endif
 
   call usvima &
  ( nvar   , nscal  ,                                              &
-   dt     ,                                                       &
-   propce(1,ipproc(ivisma(1))) ,                                  &
-   propce(1,ipproc(ivisma(2))) , propce(1,ipproc(ivisma(3))) )
+   dt     )
 
 endif
 
@@ -625,20 +677,12 @@ endif
 ! Indicateur d'erreur
 iok = 0
 
-! Rang des variables dans PROPCE
+nn = 3
+
+! Retrieve values of some physical properties fields
 call field_get_val_s(icrom, crom)
-call field_get_val_s(iprpfl(iviscl), viscl)
-ipcvis = ipproc(iviscl)
-ipcvst = ipproc(ivisct)
-if (icp.gt.0) then
-  ipccp  = ipproc(icp   )
-  nn     = 4
-else
-  ipccp = 0
-  nn    = 3
-endif
-
-
+call field_get_val_s(iviscl, viscl)
+if (icp.ge.0) nn  = 4
 call field_get_val_s(ibrom, brom)
 
 ! Min et max sur les cellules
@@ -653,17 +697,16 @@ do ii = 1, nn
       varmn(ii) = min(varmn(ii),sval(iel))
     enddo
   else
-    if (ii.eq.2) ivar = ipcvis
-    if (ii.eq.3) ivar = ipcvst
-    if (ii.eq.4) ivar = ipccp
-    if (ivar.gt.0) then
-      varmx(ii) = propce(1,ivar)
-      varmn(ii) = propce(1,ivar)
-      do iel = 2, ncel
-        varmx(ii) = max(varmx(ii),propce(iel,ivar))
-        varmn(ii) = min(varmn(ii),propce(iel,ivar))
-      enddo
-    endif
+    ivar = 1
+    if (ii.eq.2) call field_get_val_s(iviscl, cpro_var)
+    if (ii.eq.3) call field_get_val_s(ivisct, cpro_var)
+    if (ii.eq.4) call field_get_val_s(icp, cpro_var)
+    varmx(ii) = cpro_var(1)
+    varmn(ii) = cpro_var(1)
+    do iel = 2, ncel
+      varmx(ii) = max(varmx(ii),cpro_var(iel))
+      varmn(ii) = min(varmn(ii),cpro_var(iel))
+    enddo
   endif
   if (ivar.gt.0) then
     if (irangp.ge.0) then
@@ -689,10 +732,10 @@ endif
 iok1 = 0
 do ii = 1, nn
   if (ii.eq.1) call field_get_name(icrom, chaine)
-  if (ii.eq.2) call field_get_name(iprpfl(iviscl), chaine)
-  if (ii.eq.3) call field_get_name(iprpfl(ivisct), chaine)
-  if (ii.eq.4) call field_get_name(iprpfl(icp), chaine)
-  if (iwarni(iu).ge.1.or.ipass.eq.1.or.varmn(ii).lt.0.d0) then
+  if (ii.eq.2) call field_get_name(iviscl, chaine)
+  if (ii.eq.3) call field_get_name(ivisct, chaine)
+  if (ii.eq.4) call field_get_name(icp, chaine)
+  if (vcopt%iwarni.ge.1.or.ipass.eq.1.or.varmn(ii).lt.0.d0) then
     if (iok1.eq.0) then
       write(nfecra,3010)
       iok1 = 1
@@ -715,7 +758,7 @@ endif
 
 ! Viscosite moleculaire definie
 ii = 2
-call field_get_name(iprpfl(iviscl), chaine)
+call field_get_name(iviscl, chaine)
 if (varmn(ii).lt.0.d0) then
   write(nfecra,9011)chaine(1:16),varmn(ii)
   iok = iok + 1
@@ -725,16 +768,16 @@ endif
 ! on ne clippe pas mu_t en modele LES dynamique, car on a fait
 ! un clipping sur la viscosite totale
 ii = 3
-call field_get_name(iprpfl(ivisct), chaine)
+call field_get_name(ivisct, chaine)
 if (varmn(ii).lt.0.d0.and.iturb.ne.41) then
   write(nfecra,9012)varmn(ii)
   iok = iok + 1
 endif
 
 ! Chaleur specifique definie
-if (icp.gt.0) then
+if (icp.ge.0) then
   ii = 4
-  call field_get_name(iprpfl(icp), chaine)
+  call field_get_name(icp, chaine)
   if (varmn(ii).lt.0.d0) then
     iisct = 0
     if (itherm.ne.0) iisct = 1
@@ -779,7 +822,8 @@ if (nscal.ge.1) then
     endif
 
     ivar = isca(iscal)
-    if (iwarni(ivar).ge.1.or.ipass.eq.1.or.vismin(iscal).le.0.d0) then
+    call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+    if (vcopt%iwarni.ge.1.or.ipass.eq.1.or.vismin(iscal).le.0.d0) then
       call field_get_label(ivarfl(ivar), chaine)
       if (iok1.eq.0) then
         write(nfecra,3110)
@@ -805,10 +849,11 @@ if (nscal.ge.1) then
       iok = iok + 1
     endif
 
-    if (iscal.eq.iscalt.and.irovar.eq.1.and.ityturt(iscal).eq.2) then
+    if (ibeta.ge.0) then
       iok1 = 0
+      call field_get_val_s(ibeta, cpro_beta)
       do iel = 1, ncel
-        if (propce(iel,ipproc(ibeta)).le.0.d0) iok1 = 1
+        if (cpro_beta(iel).lt.0.d0) iok1 = 1
       enddo
       if (iok1.eq.1) write(nfecra,9013)
     endif
@@ -820,18 +865,51 @@ endif
 
 if (iale.eq.1.and.ntcabs.eq.0) then
 
+  call field_get_key_struct_var_cal_opt(ivarfl(iuma), vcopt)
   iok1 = 0
-  nn = 1
-  if (iortvm.eq.1) nn = 3
-  do ii = 1, nn
-    ipcvma = ipproc(ivisma(ii))
+  if (iortvm.eq.1) then
+    call field_get_val_v(ivisma, cpro_visma_v)
+    do ii = 1, 3
+      ! Min et max sur les cellules
+      varmx(1) = cpro_visma_v(ii,1)
+      varmn(1) = cpro_visma_v(ii,1)
+      do iel = 2, ncel
+        varmx(1) = max(varmx(1),cpro_visma_v(ii,iel))
+        varmn(1) = min(varmn(1),cpro_visma_v(ii,iel))
+      enddo
+      if (irangp.ge.0) then
+        call parmax (varmx(1))
+        call parmin (varmn(1))
+      endif
 
+      ! Writings
+      call field_get_name(ivisma, chaine)
+      if (vcopt%iwarni.ge.1.or.ipass.eq.1.or.varmn(1).lt.0.d0) then
+        if (iok1.eq.0) then
+          write(nfecra,3210)
+          iok1 = 1
+        endif
+        write(nfecra,3211)chaine(1:16),varmn(1),varmx(1)
+      endif
+
+      ! Verifications de valeur physique
+
+      ! Viscosite de maillage definie
+      call field_get_name(ivisma, chaine)
+      if (varmn(1).le.0.d0) then
+        write(nfecra,9211) varmn(1)
+        iok = iok + 1
+      endif
+
+    enddo
+  else
+    call field_get_val_s(ivisma, cpro_visma_s)
     ! Min et max sur les cellules
-    varmx(1) = propce(1,ipcvma)
-    varmn(1) = propce(1,ipcvma)
+    varmx(1) = cpro_visma_s(1)
+    varmn(1) = cpro_visma_s(1)
     do iel = 2, ncel
-      varmx(1) = max(varmx(1),propce(iel,ipcvma))
-      varmn(1) = min(varmn(1),propce(iel,ipcvma))
+      varmx(1) = max(varmx(1),cpro_visma_s(iel))
+      varmn(1) = min(varmn(1),cpro_visma_s(iel))
     enddo
     if (irangp.ge.0) then
       call parmax (varmx(1))
@@ -839,8 +917,8 @@ if (iale.eq.1.and.ntcabs.eq.0) then
     endif
 
     ! Writings
-    call field_get_name(iprpfl(ipcvma), chaine)
-    if (iwarni(iuma).ge.1.or.ipass.eq.1.or.varmn(1).lt.0.d0) then
+    call field_get_name(ivisma, chaine)
+    if (vcopt%iwarni.ge.1.or.ipass.eq.1.or.varmn(1).lt.0.d0) then
       if (iok1.eq.0) then
         write(nfecra,3210)
         iok1 = 1
@@ -851,13 +929,13 @@ if (iale.eq.1.and.ntcabs.eq.0) then
     ! Verifications de valeur physique
 
     ! Viscosite de maillage definie
-    call field_get_name(iprpfl(ipcvma), chaine)
+    call field_get_name(ivisma, chaine)
     if (varmn(1).le.0.d0) then
       write(nfecra,9211) varmn(1)
       iok = iok + 1
     endif
 
-  enddo
+  endif
 
   if (iok1.eq.1) write(nfecra,3212)
 
@@ -870,18 +948,6 @@ if (iok.ne.0) then
   call csexit (1)
 endif
 
-!===============================================================================
-! 11. Parallelism and periodicity
-!===============================================================================
-
-! Pour navstv et visecv on a besoin de ROM dans le halo
-
-call field_get_val_s(icrom, crom)
-
-if (irangp.ge.0.or.iperio.eq.1) then
-  call synsca(crom)
-endif
-
 !--------
 ! Formats
 !--------
