@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -27,6 +27,7 @@ module pointe
 
   !=============================================================================
 
+  use, intrinsic :: iso_c_binding
   use paramx
 
   implicit none
@@ -38,7 +39,7 @@ module pointe
   !> \{
 
   !> \defgroup fortran_pointer_containers Containers to Fortran array pointers.
-  !> An array of one of these these derived types can be used to manage a set of
+  !> An array of one of these derived types can be used to manage a set of
   !> pointers (Fortran not allow arrays of pointers directly, so this
   !> technique is a classical workaround.
 
@@ -95,13 +96,6 @@ module pointe
 
   !... Auxiliaires
 
-  !> distance between the center of a given volume and the closest wall,
-  !> when it is necessary (\f$R_{ij}-\varepsilon\f$ with wall echo,
-  !> LES with van Driest-wall damping, or \f$k-\omega\f$ (SST) turbulence model).
-  !> The distance between the center of the cell
-  !> \c iel and the closest wall is \c dispar(iel)
-  double precision, allocatable, dimension(:)   :: dispar
-
   !> non-dimensional distance \f$y^+\f$ between a given volume and the closest
   !> wall, when it is necessary (LES with van Driest-wall damping).
   !> The adimensional distance \f$y^+\f$ between the center of the cell \c iel
@@ -119,39 +113,20 @@ module pointe
   !> \anchor itypfb
   !> boundary condition type at the boundary face \c ifac
   !> (see user subroutine \ref cs\_user\_boundary\_conditions)
-  integer, dimension(:), pointer :: itypfb(:)
+  integer, dimension(:), pointer, save :: itypfb
 
   !> indirection array allowing to sort the boundary faces
   !> according to their boundary condition type \c itypfb
   integer, allocatable, dimension(:) :: itrifb
 
   !> to identify boundary zones associated with boundary faces
-  !> (particular physics)
-  integer, allocatable, dimension(:) :: izfppp
-
-  !> to identify boundary zones associated with boundary faces
-  !> (radiative transfer)
-  integer, allocatable, dimension(:) :: izfrad
+  !> (specific physics models)
+  integer, dimension(:), pointer :: izfppp
 
   !> the index of the structure, (\c idfstr(ifac) where \c ifac is the index
   !> of the face), 0 if the face is not coupled to any structure.
   integer, allocatable, dimension(:) :: idfstr
 
-  !> square of the norm of the deviatoric part of the deformation
-  !> rate tensor (\f$S^2=2S_{ij}^D S_{ij}^D\f$). This array is defined only
-  !> with the \f$k-\omega\f$ (SST) turbulence model
-  double precision, allocatable, dimension(:)   :: s2kw
-
-  !> divergence of the velocity. More precisely it is the trace of the velocity
-  !> gradient (and not a finite volume divergence term). In the
-  !> cell \c iel,  \f$div(\vect{u})\f$ is given by \c divukw(iel1).
-  !> This array is defined only with the \f$k-\omega\f$ SST turbulence model
-  !> (because in this case it may be calculated at the same time as \f$S^2\f$)
-  double precision, allocatable, dimension(:)   :: divukw
-
-  !> strain rate tensor at the previous time step
-  double precision, allocatable, dimension(:,:) :: straio
-
   !> \}
 
   !=============================================================================
@@ -162,73 +137,14 @@ module pointe
   !> \addtogroup thermal_1D
   !> \{
 
-
   !> number of boundary faces which are coupled
-  !> with a wall 1D thermal module. See the user subroutine \ref uspt1d
-  integer, save :: nfpt1d
+  !> with a wall 1D thermal module. See the user subroutine
+  !> \ref cs_user_1d_wall_thermal
+  integer(c_int), pointer, save :: nfpt1d
 
-  ! TODO
-  integer, save :: nmxt1d
-
-  !> zones of t1d, dimensioned with nfabor (TODO)
-  integer, allocatable, dimension(:) :: izft1d
-
-  !> number of discretisation cells in the 1D wall for the
-  !> \c nfpt1d boundary faces which are coupled with a wall 1D thermal module.
-  !> The number of cells for these boundary faces is given by
-  !> \c nppt1d(ii), with 1 <= ii <= nfpt1d.
-  !> See the user subroutine \ref uspt1d
-  integer, allocatable, dimension(:) :: nppt1d
-
-  !> array allowing to mark out the numbers of
-  !> the \c nfpt1d boundary faces which are coupled with a wall 1D
-  !> thermal module. The numbers of these boundary faces are
-  !> given by \c ifpt1d(ii), with 1 <= ii <= \c nfpt1d.
-  !> See the user subroutine \ref uspt1d
-  integer, allocatable, dimension(:) :: ifpt1d
-
-  !> typical boundary condition at the external (pseudo) wall:
-  !> Dirichlet condition (\c iclt1d=1) or flux condition (\c iclt1d=3)
-  integer, allocatable, dimension(:) :: iclt1d
-
-  !> thickness of the 1D wall for the \c nfpt1d boundary faces
-  !> which are coupled with a wall 1D thermal module.
-  !> The wall thickness for these boundary faces is therefore given by
-  !> \c eppt1d(ii), with 1 <= ii <= \c nfpt1d.
-  !> See the user subroutine \ref uspt1d
-  double precision, allocatable, dimension(:) :: eppt1d
-
-  !> geometry of the pseudo wall mesh (refined as a fluid
-  !> if \c rgt1d is smaller than 1
-  double precision, allocatable, dimension(:) :: rgpt1d
-
-  !> initialisation temperature of the wall (uniform in thickness).
-  !> In the course of the calculation, the array stores the temperature
-  !> of the solid at the fluid/solid interface.
-  double precision, allocatable, dimension(:) :: tppt1d
-
-  !> external temperature of the pseudo wall in the Dirichlet case.
-  double precision, allocatable, dimension(:) :: tept1d
-
-  !> external coefficient of transfer in the pseudo wall under Dirichlet conditions
-  !> (in \f$W.m^{-2}.K^.\f$).
-  double precision, allocatable, dimension(:) :: hept1d
-
-  ! fept1d ! nfpt1d                  ! flux thermique exterieur
-  !> external heat flux in the pseudo wall under the flux conditions
-  !> (in \f$W.m^{-2}\f$, negative value for energy entering the wall).
-  double precision, allocatable, dimension(:) :: fept1d
-
-  !> thermal diffusivity
-  double precision, allocatable, dimension(:) :: xlmbt1
-
-  !> volumetric heat capacity \f$\rho C_p\f$ of the wall uniform in thickness
-  !> (in \f$J.m^{-3}.K^{-1}\f$).
-  double precision, allocatable, dimension(:) :: rcpt1d
-
-  !> physical time step associated with the solved 1D equation of the pseudo wall
-  !> (which can be different from the time step in the calculation).
-  double precision, allocatable, dimension(:) :: dtpt1d
+  !> global number of boundary faces which are coupled with
+  !> a wall 1D thermal module. (ie sum over all ranks of nfpt1d)
+  integer(c_int), pointer, save :: nfpt1t
 
   !> \}
 
@@ -238,22 +154,24 @@ module pointe
   !> \addtogroup auxiliary
   !> \{
 
-
+  !> \anchor ncepdc
   !> number of cells in which a pressure drop is imposed.
-  !> See the user subroutine \ref cs_user_head_losses
+  !> See the user subroutine \ref cs_f_user_head_losses
   integer, save :: ncepdc
 
+  !> \anchor icepdc
   !> number of the \c ncepdc cells in which a pressure drop is imposed.
-  !> See \c {iicepd} and the user subroutine \ref cs_user_head_losses
+  !> See \c {iicepd} and the user subroutine \ref cs_f_user_head_losses
   integer, allocatable, dimension(:) :: icepdc
 
   !> zone with head losses
   integer, allocatable, dimension(:) :: izcpdc
 
+  !> \anchor ckupdc
   !> value of the coefficients of the pressure drop tensor of the
   !> \c ncepdc cells in which a pressure drop is imposed.
   !> Note the 6 values are sorted as follows: (k11, k22, k33, k12, k23, k33).
-  !> See \c ickpdc and the user subroutine \ref cs_user_head_losses
+  !> See \c ickpdc and the user subroutine \ref cs_f_user_head_losses
   double precision, allocatable, dimension(:,:) :: ckupdc
 
   !> Head loss factor of the fluid outside the domain, between infinity and
@@ -263,10 +181,12 @@ module pointe
   !> \c rcodcl(ifac, \ref ipr, 2).
   double precision, allocatable, dimension(:) :: b_head_loss
 
+  !> \anchor ncetsm
   !> number of the \c ncetsm cells in which a mass source term is imposed.
   !> See \c iicesm and the user subroutine \ref cs_user_mass_source_terms
   integer, save :: ncetsm
 
+  !> \anchor icetsm
   !> number of the \c ncetsm cells in which a mass source term is imposed.
   !> See \c iicesm and the user subroutine \ref cs_user_mass_source_terms}}
   integer, allocatable, dimension(:) :: icetsm
@@ -274,12 +194,14 @@ module pointe
   !> zone where a mass source term is imposed.
   integer, allocatable, dimension(:) :: izctsm
 
+  !> \anchor itypsm
   !> type of mass source term for each variable
   !> - 0 for an injection at ambient value,
   !> - 1 for an injection at imposed value.
   !> See the user subroutine \ref cs_user_mass_source_terms
   integer, allocatable, dimension(:,:) :: itypsm
 
+  !> \anchor smacel
   !> value of the mass source term for pressure.
   !> For the other variables, eventual imposed injection value.
   !> See the user subroutine \ref cs_user_mass_source_terms
@@ -396,10 +318,10 @@ contains
     use entsor
     use ppincl
     use lagran
-    use radiat, only: iirayo
     use albase
     use ihmpre
     use field
+    use cs_c_bindings
 
     implicit none
 
@@ -408,27 +330,12 @@ contains
     integer, intent(in) :: ncelet, nfabor
 
     ! Local variables
-    integer                ifac
-    integer                kdiftn
 
-    ! Key id for diffusivity tensor
-    call field_get_key_id("diffusivity_tensor", kdiftn)
+    type(var_cal_opt) :: vcopt
 
     ! Boundary-face related arrays
 
     allocate(itrifb(nfabor))
-    if (ippmod(iphpar).ge.1 .or. iihmpr.eq.1) then
-      allocate(izfppp(nfabor))
-      do ifac = 1, nfabor
-        izfppp(ifac) = 0
-      enddo
-    endif
-    if (iirayo.gt.0) then
-      allocate(izfrad(nfabor))
-      do ifac = 1, nfabor
-        izfrad(ifac) = 0
-      enddo
-    endif
 
     ! ALE array for structure definition
 
@@ -438,38 +345,23 @@ contains
 
     ! Also tensorial diffusion for the velocity in case of tensorial porosity
     if (iporos.eq.2) then
-      idften(iu) = 6
-      ! Key word: tensorial diffusivity
-      call field_set_key_int(ivarfl(iu), kdiftn, idften(iu))
+      ! Tensorial diffusivity
+      call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt)
+      vcopt%idften = ANISOTROPIC_LEFT_DIFFUSION
+      call field_set_key_struct_var_cal_opt(ivarfl(iu), vcopt)
     endif
 
     ! Diagonal cell tensor for the pressure solving when needed
     if (ncpdct.gt.0.or.ipucou.eq.1.or.iporos.eq.2) then
-      idften(ipr) = 6
+      call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
+      vcopt%idften = ANISOTROPIC_LEFT_DIFFUSION
+      call field_set_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
     endif
 
     ! Wall-distance calculation
 
-    if (ineedy.eq.1) then
-      allocate(dispar(ncelet))
-      if (itytur.eq.4 .and. idries.eq.1) then
-        allocate(yplpar(ncelet))
-      endif
-    endif
-
-    ! Temporary storage arrays for k-omega model
-
-    if (iturb.eq.60) then
-      allocate(s2kw(ncelet))
-      allocate(divukw(ncelet))
-    endif
-
-    ! Strain rate tensor at the previous time step
-    ! if rotation curvature correction of eddy viscosity
-    if (irccor.eq.1) then
-      if (idtvar.ge.0) then
-        allocate(straio(ncelet,6))
-      endif
+    if (itytur.eq.4 .and. idries.eq.1) then
+      allocate(yplpar(ncelet))
     endif
 
     ! liquid-vapour mass transfer term for cavitating flows
@@ -496,28 +388,13 @@ contains
 
     ! Local variables
 
-    integer iel, isou
+    integer iel
     double precision, allocatable, dimension(:) :: buffer
-    double precision, allocatable, dimension(:,:) :: buff2
 
     ! Resize/copy arrays
 
     allocate(buffer(ncelet))
 
-    ! Wall-distance calculation
-
-    if (allocated(dispar)) then
-      do iel = 1, ncel
-        buffer(iel) = dispar(iel)
-      enddo
-      deallocate(dispar)
-      call synsca (buffer)
-      allocate(dispar(ncelet))
-      do iel = 1, ncelet
-        dispar(iel) = buffer(iel)
-      enddo
-    endif
-
     if (allocated(yplpar)) then
       do iel = 1, ncel
         buffer(iel) = yplpar(iel)
@@ -530,53 +407,6 @@ contains
       enddo
     endif
 
-    ! Temporary storage arrays for k-omega model
-
-    if (allocated(s2kw)) then
-      do iel = 1, ncel
-        buffer(iel) = s2kw(iel)
-      enddo
-      deallocate(s2kw)
-      call synsca (buffer)
-      allocate(s2kw(ncelet))
-      do iel = 1, ncelet
-        s2kw(iel) = buffer(iel)
-      enddo
-
-      do iel = 1, ncel
-        buffer(iel) = divukw(iel)
-      enddo
-      deallocate(divukw)
-      call synsca (buffer)
-      allocate(divukw(ncelet))
-      do iel = 1, ncelet
-        divukw(iel) = buffer(iel)
-      enddo
-    endif
-
-    ! Strain rate tensor at the previous time step
-    ! for rotation-curvature correction of eddy viscosity
-
-    if (allocated(straio)) then
-      allocate(buff2(ncel,6))
-      do isou = 1, 6
-        do iel = 1, ncel
-          buff2(iel,isou) = straio(iel,isou)
-        enddo
-      enddo
-      deallocate(straio)
-      allocate(straio(ncelet,6))
-      do isou = 1, 6
-        do iel = 1, ncel
-          straio(iel,isou) = buff2(iel,isou)
-        enddo
-      enddo
-      deallocate(buff2)
-      call synten (straio(1,1), straio(1,4), straio(1,5), &
-                   straio(1,4), straio(1,2), straio(1,6), &
-                   straio(1,5), straio(1,6), straio(1,3))
-    endif
-
     ! liquid-vapour mass transfer term for cavitating flows
     ! and its part implicit in pressure
 
@@ -615,16 +445,10 @@ contains
   subroutine finalize_aux_arrays
 
     deallocate(itrifb)
-    if (allocated(izfppp)) deallocate(izfppp)
-    if (allocated(izfrad)) deallocate(izfrad)
     if (allocated(idfstr)) deallocate(idfstr)
     if (allocated(izcpdc)) deallocate(izcpdc)
     if (allocated(izctsm)) deallocate(izctsm)
-    if (allocated(izft1d)) deallocate(izft1d)
-    if (allocated(dispar)) deallocate(dispar)
     if (allocated(yplpar)) deallocate(yplpar)
-    if (allocated(s2kw)) deallocate(s2kw, divukw)
-    if (allocated(straio))  deallocate(straio)
     if (allocated(b_head_loss)) deallocate(b_head_loss)
     if (allocated(gamcav)) deallocate(gamcav, dgdpca)
 
@@ -736,49 +560,6 @@ contains
 
   !=============================================================================
 
-  subroutine init_pt1d
-
-    use cstnum
-
-    allocate(nppt1d(nfpt1d), ifpt1d(nfpt1d), iclt1d(nfpt1d))
-    allocate(eppt1d(nfpt1d), rgpt1d(nfpt1d), tppt1d(nfpt1d))
-    allocate(tept1d(nfpt1d), hept1d(nfpt1d), fept1d(nfpt1d))
-    allocate(xlmbt1(nfpt1d), rcpt1d(nfpt1d), dtpt1d(nfpt1d))
-
-    !---> INITIALISATION DES TABLEAUX
-    !     a des valeurs sortant en erreur dans vert1d
-    !     sauf pour les variables de conditions aux limites
-    !     qui sont initialisees a des valeurs standard
-    !     (flux nul, Timpose=0, coef d'echange infini)
-
-    ifpt1d(:) = -999
-    nppt1d(:) = -999
-    iclt1d(:) = 3
-    eppt1d(:) = -999.d0
-    rgpt1d(:) = -999.d0
-    tppt1d(:) = 0.d0
-    tept1d(:) = 0.d0
-    hept1d(:) = rinfin
-    fept1d(:) = 0.d0
-    xlmbt1(:) = -999.d0
-    rcpt1d(:) = -999.d0
-    dtpt1d(:) = -999.d0
-
-  end subroutine init_pt1d
-
-  !=============================================================================
-
-  subroutine finalize_pt1d
-
-    deallocate(nppt1d, ifpt1d, iclt1d)
-    deallocate(eppt1d, rgpt1d, tppt1d)
-    deallocate(tept1d, hept1d, fept1d)
-    deallocate(xlmbt1, rcpt1d, dtpt1d)
-
-  end subroutine finalize_pt1d
-
-  !=============================================================================
-
   subroutine boundary_conditions_init
 
     use, intrinsic :: iso_c_binding
@@ -789,13 +570,14 @@ contains
 
     ! Local variables
 
-    type(c_ptr) :: c_itypfb
+    type(c_ptr) :: c_itypfb, c_izfppp
 
-    call cs_f_boundary_conditions_type_create
+    call cs_f_boundary_conditions_create
 
-    call cs_f_boundary_conditions_type_get_pointer(c_itypfb)
+    call cs_f_boundary_conditions_get_pointers(c_itypfb, c_izfppp)
 
     call c_f_pointer(c_itypfb, itypfb, [nfabor])
+    call c_f_pointer(c_izfppp, izfppp, [nfabor])
 
   end subroutine boundary_conditions_init
 
@@ -807,8 +589,83 @@ contains
 
     implicit none
 
-    call cs_f_boundary_conditions_type_free
+    call cs_f_boundary_conditions_free
 
   end subroutine boundary_conditions_finalize
 
+  !=============================================================================
+
+  !> \brief Allocate the cs_glob_1d_wall_thermal structure.
+
+  subroutine init_1d_wall_thermal
+
+    use, intrinsic :: iso_c_binding
+    use cs_c_bindings
+
+    implicit none
+
+    ! Local variables
+    type(c_ptr) :: c_nfpt1d, c_nfpt1t
+
+    call cs_1d_wall_thermal_create
+
+    call cs_f_1d_wall_thermal_get_pointers(c_nfpt1d, c_nfpt1t)
+
+    call c_f_pointer(c_nfpt1d, nfpt1d)
+    call c_f_pointer(c_nfpt1t, nfpt1t)
+
+    return
+
+  end subroutine init_1d_wall_thermal
+
+  !=============================================================================
+
+  !> \brief Return pointer to the ifpt1d array for the 1D wall thermal module.
+
+  !> \param[out]    ifpt1d         pointer to ifpt1d
+
+  subroutine cs_1d_wall_thermal_get_faces(ifpt1d)
+
+    use, intrinsic :: iso_c_binding
+    use cs_c_bindings
+
+    implicit none
+
+    integer, dimension(:), pointer, intent(out) :: ifpt1d
+
+    ! Local variables
+
+    type(c_ptr) :: c_ifpt1d
+
+    call cs_f_1d_wall_thermal_get_faces(c_ifpt1d)
+    call c_f_pointer(c_ifpt1d, ifpt1d, [nfpt1d])
+
+  end subroutine cs_1d_wall_thermal_get_faces
+
+  !=============================================================================
+
+  !> \brief Return pointer to the tppt1d array for the 1D wall thermal module.
+
+  !> \param[out]    tppt1d         pointer to tppt1d
+
+  subroutine cs_1d_wall_thermal_get_temp(tppt1d)
+
+    use, intrinsic :: iso_c_binding
+    use cs_c_bindings
+
+    implicit none
+
+    double precision, dimension(:), pointer, intent(out) :: tppt1d
+
+    ! Local variables
+
+    type(c_ptr) :: c_tppt1d
+
+    call cs_f_1d_wall_thermal_get_temp(c_tppt1d)
+    call c_f_pointer(c_tppt1d, tppt1d, [nfpt1d])
+
+  end subroutine cs_1d_wall_thermal_get_temp
+
+  !=============================================================================
+
 end module pointe
