@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -36,6 +36,9 @@
 !> - at the second call, a global estimator on Navier Stokes is computed.
 !>   This second call is done after the correction step (\ref resopv).
 !>
+!> Please refer to the
+!> <a href="../../theory.pdf#predvv"><b>predvv</b></b></a> section
+!> of the theory guide for more informations.
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
@@ -126,7 +129,6 @@ subroutine predvv &
 !===============================================================================
 
 use paramx
-use dimens, only: ndimfb
 use numvar
 use entsor
 use cstphy
@@ -150,7 +152,9 @@ use field
 use field_operator
 use pointe, only: gamcav
 use cavitation
+use vof
 use cs_tagms, only:s_metal
+use atincl, only: kopint, iatmst
 
 !===============================================================================
 
@@ -183,13 +187,13 @@ double precision viscfi(*), viscbi(nfabor)
 double precision secvif(nfac), secvib(nfabor)
 double precision w1(ncelet)
 double precision w7(ncelet), w8(ncelet), w9(ncelet)
-double precision coefav(3  ,ndimfb)
-double precision cofafv(3  ,ndimfb)
-double precision coefbv(3,3,ndimfb)
-double precision cofbfv(3,3,ndimfb)
+double precision coefav(3  ,nfabor)
+double precision cofafv(3  ,nfabor)
+double precision coefbv(3,3,nfabor)
+double precision cofbfv(3,3,nfabor)
 
-double precision vel   (3  ,ncelet)
-double precision vela  (3  ,ncelet)
+double precision vel   (3, ncelet)
+double precision vela  (3, ncelet)
 
 ! Local variables
 
@@ -200,8 +204,9 @@ integer          iswdyp, idftnp
 integer          iconvp, idiffp, ndircp, nswrsp
 integer          ircflp, ischcp, isstpp, iescap
 integer          iflmb0, nswrp
-integer          idtva0, icvflb
+integer          idtva0, icvflb, f_oi_id
 integer          jsou  , ivisep, imasac
+integer          f_dim , iflwgr
 integer          ivoid(1)
 
 double precision rnorm , vitnor
@@ -219,7 +224,7 @@ double precision rvoid(1)
 
 ! Working arrays
 double precision, allocatable, dimension(:,:) :: eswork
-double precision, allocatable, dimension(:,:) :: grad
+double precision, allocatable, dimension(:,:), target :: grad
 double precision, dimension(:,:), allocatable :: smbr
 double precision, dimension(:,:,:), allocatable :: fimp
 double precision, dimension(:,:), allocatable :: gavinj
@@ -239,7 +244,6 @@ double precision, dimension(:,:), allocatable :: coefat
 double precision, dimension(:,:,:), allocatable :: coefbt
 double precision, dimension(:,:), allocatable :: tflmas, tflmab
 double precision, dimension(:,:), allocatable :: divt
-double precision, dimension(:), allocatable ::xnormp
 double precision, dimension(:,:), pointer :: forbr, c_st_vel
 double precision, dimension(:), pointer :: cvara_pr, cvara_k
 double precision, dimension(:), pointer :: cvara_r11, cvara_r22, cvara_r33
@@ -247,8 +251,13 @@ double precision, dimension(:), pointer :: cvara_r12, cvara_r23, cvara_r13
 double precision, dimension(:,:), pointer :: cvara_rij
 double precision, dimension(:), pointer :: viscl, visct, c_estim
 double precision, allocatable, dimension(:) :: surfbm
-double precision, dimension(:,:), pointer :: lapla
+double precision, dimension(:,:), pointer :: lapla, lagr_st_vel
 double precision, dimension(:), pointer :: cpro_tsrho
+double precision, dimension(:,:), pointer :: cpro_gradp
+double precision, dimension(:), pointer :: cpro_wgrec_s
+double precision, dimension(:,:), pointer :: cpro_wgrec_v
+
+type(var_cal_opt) :: vcopt_p, vcopt_u, vcopt
 
 !===============================================================================
 
@@ -261,7 +270,12 @@ allocate(smbr(3,ncelet))
 allocate(fimp(3,3,ncelet))
 allocate(tsexp(3,ncelet))
 allocate(tsimp(3,3,ncelet))
-if (idften(iu).eq.6) allocate(viscce(6,ncelet))
+call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt_u)
+call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt_p)
+
+idftnp = vcopt_u%idften
+
+if (iand(idftnp, ANISOTROPIC_LEFT_DIFFUSION).ne.0) allocate(viscce(6,ncelet))
 
 ! Allocate a temporary array for the prediction-stage error estimator
 if (iescal(iespre).gt.0) then
@@ -277,9 +291,8 @@ if (iroext.gt.0.or.idilat.gt.1) then
   call field_get_val_prev_s(icrom, croma)
 endif
 
-
 if (iappel.eq.2) then
-  if (iforbr.ge.0 .and. iterns.eq.1 .or. icavit.ge.0) then
+  if (iforbr.ge.0 .and. iterns.eq.1 .or. ivofmt.ge.0) then
     call field_get_val_s(ivarfl(ipr), cvara_pr)
   endif
   if(iforbr.ge.0 .and. iterns.eq.1                                          &
@@ -299,7 +312,7 @@ if (iappel.eq.2) then
     endif
   endif
 else
-  if (iforbr.ge.0 .and. iterns.eq.1 .or. icavit.ge.0) then
+  if (iforbr.ge.0 .and. iterns.eq.1 .or. ivofmt.ge.0) then
     call field_get_val_prev_s(ivarfl(ipr), cvara_pr)
   endif
   if(iforbr.ge.0 .and. iterns.eq.1                                          &
@@ -331,17 +344,6 @@ else
   c_st_vel => null()
 endif
 
-! Coefficient of the "Coriolis-type" term
-if (icorio.eq.1) then
-  ! Relative velocity formulation
-  ccorio = 2.d0
-else if (iturbo.eq.1) then
-  ! Mixed relative/absolute velocity formulation
-  ccorio = 1.d0
-else
-  ccorio = 0.d0
-endif
-
 !===============================================================================
 ! 2. Potential forces (pressure gradient and gravity)
 !===============================================================================
@@ -349,8 +351,13 @@ endif
 !-------------------------------------------------------------------------------
 ! ---> Pressure gradient
 
-! Allocate a work array for the gradient calculation
-allocate(grad(3,ncelet))
+call field_get_id_try("pressure_gradient", f_id)
+if (f_id.ge.0) then
+  call field_get_val_v(f_id, cpro_gradp)
+else
+  allocate(grad(3,ncelet))
+ cpro_gradp => grad
+endif
 
 iccocg = 1
 inc    = 1
@@ -364,43 +371,38 @@ else
   iprev = 1
 endif
 
-if (icavit.lt.0) then
-  call field_gradient_potential(ivarfl(ipr), iprev, imrgra, inc,    &
-                                iccocg, iphydr,                     &
-                                frcxt, grad)
-
-else
-
-  ! Cavitating flows: consistency of the gradient with the diffusive flux scheme
-  ! of the correction step
-
-  call field_get_coefa_s (ivarfl(ipr), coefa_p)
-  call field_get_coefb_s (ivarfl(ipr), coefb_p)
-
-  allocate(xinvro(ncelet))
-
-  do iel = 1, ncel
-    xinvro(iel) = 1.d0/crom(iel)
-  enddo
-
-  iccocg = 1
-  inc    = 1
-  nswrgp = nswrgr(ipr)
-  imligp = imligr(ipr)
-  iwarnp = iwarni(ipr)
-  epsrgp = epsrgr(ipr)
-  climgp = climgr(ipr)
-  extrap = extrag(ipr)
-
-  call gradient_weighted_s(ivarfl(ipr), imrgra, inc, iccocg, nswrgp, imligp,  &
-                           iwarnp, epsrgp, climgp, extrap,                    &
-                           cvara_pr, xinvro, coefa_p, coefb_p,                &
-                           grad )
-
-  deallocate(xinvro)
+! Namely for the VOF algorithm: consistency of the gradient
+! with the diffusive flux scheme of the correction step
+if (vcopt_p%iwgrec.eq.1) then
+  ! Id weighting field for gradient
+  call field_get_key_int(ivarfl(ipr), kwgrec, iflwgr)
+  call field_get_dim(iflwgr, f_dim)
+  if (f_dim.gt.1) then
+    call field_get_val_v(iflwgr, cpro_wgrec_v)
+    do iel = 1, ncel
+      cpro_wgrec_v(1,iel) = dt(iel) / crom(iel) ! FIXME should take headlosses into account, not compatible neither with ipucou=1...
+      cpro_wgrec_v(2,iel) = dt(iel) / crom(iel)
+      cpro_wgrec_v(3,iel) = dt(iel) / crom(iel)
+      cpro_wgrec_v(4,iel) = 0.d0
+      cpro_wgrec_v(5,iel) = 0.d0
+      cpro_wgrec_v(6,iel) = 0.d0
+    enddo
+    call syntis(cpro_wgrec_v)
 
+  else
+    call field_get_val_s(iflwgr, cpro_wgrec_s)
+    do iel = 1, ncel
+      cpro_wgrec_s(iel) = dt(iel) / crom(iel)
+    enddo
+    call synsca(cpro_wgrec_s)
+  endif
 endif
 
+! Pressure gradient
+call field_gradient_potential(ivarfl(ipr), iprev, imrgra, inc,    &
+                              iccocg, iphydr,                     &
+                              frcxt, cpro_gradp)
+
 !    Calcul des efforts aux parois (partie 2/5), si demande
 !    La pression a la face est calculee comme dans gradrc/gradmc
 !    et on la transforme en pression totale
@@ -415,14 +417,14 @@ if (iforbr.ge.0 .and. iterns.eq.1) then
     diipby = diipb(2,ifac)
     diipbz = diipb(3,ifac)
     pip = cvara_pr(iel) &
-        + diipbx*grad(1,iel) + diipby*grad(2,iel) + diipbz*grad(3,iel)
+        + diipbx*cpro_gradp(1,iel) + diipby*cpro_gradp(2,iel) + diipbz*cpro_gradp(3,iel)
     pfac = coefa_p(ifac) +coefb_p(ifac)*pip
     pfac1= cvara_pr(iel)                                              &
-         +(cdgfbo(1,ifac)-xyzcen(1,iel))*grad(1,iel)              &
-         +(cdgfbo(2,ifac)-xyzcen(2,iel))*grad(2,iel)              &
-         +(cdgfbo(3,ifac)-xyzcen(3,iel))*grad(3,iel)
-    pfac = coefb_p(ifac)*(extrag(ipr)*pfac1                       &
-         +(1.d0-extrag(ipr))*pfac)                                &
+         +(cdgfbo(1,ifac)-xyzcen(1,iel))*cpro_gradp(1,iel)              &
+         +(cdgfbo(2,ifac)-xyzcen(2,iel))*cpro_gradp(2,iel)              &
+         +(cdgfbo(3,ifac)-xyzcen(3,iel))*cpro_gradp(3,iel)
+    pfac = coefb_p(ifac)*(vcopt_p%extrag*pfac1                       &
+         +(1.d0-vcopt_p%extrag)*pfac)                                &
          +(1.d0-coefb_p(ifac))*pfac                               &
          + ro0*(gx*(cdgfbo(1,ifac)-xyzp0(1))                      &
          + gy*(cdgfbo(2,ifac)-xyzp0(2))                           &
@@ -434,131 +436,6 @@ if (iforbr.ge.0 .and. iterns.eq.1) then
   enddo
 endif
 
-!-------------------------------------------------------------------------------
-! ---> RESIDU DE NORMALISATION POUR RESOLP
-!     Test d'un residu de normalisation de l'etape de pression
-!       plus comprehensible = div(rho u* + dt gradP^(n))-Gamma
-!       i.e. second membre du systeme en pression hormis la partie
-!       pression (sinon a convergence, on tend vers 0)
-!       Represente les termes que la pression doit equilibrer
-!     On calcule ici div(rho dt/rho gradP^(n)) et on complete a la fin
-!       avec  div(rho u*)
-!     Pour grad P^(n) on suppose que des CL de Neumann homogenes
-!       s'appliquent partout : on peut donc utiliser les CL de la
-!       vitesse pour u*+dt/rho gradP^(n). Comme on calcule en deux fois,
-!       on utilise les CL de vitesse homogenes pour dt/rho gradP^(n)
-!       ci-dessous et les CL de vitesse completes pour u* a la fin.
-
-if (iappel.eq.1.and.irnpnw.eq.1) then
-
-!     Calcul de dt/rho*grad P
-  do iel = 1, ncel
-    dtsrom = dt(iel)/crom(iel)
-    trav(1,iel) = grad(1,iel)*dtsrom
-    trav(2,iel) = grad(2,iel)*dtsrom
-    trav(3,iel) = grad(3,iel)*dtsrom
-  enddo
-
-  if (irangp.ge.0.or.iperio.eq.1) then
-    call synvin(trav)
-  endif
-
-!     Calcul de rho dt/rho*grad P.n aux faces
-!       Pour gagner du temps, on ne reconstruit pas.
-  itypfl = 1
-  ! Cavitation algorithm: the pressure step corresponds to the
-  ! correction of the volumetric flux, not the mass flux
-  if (icavit.ge.0)  itypfl = 0
-  init   = 1
-  inc    = 0
-  iflmb0 = 1
-  nswrp  = 1
-  imligp = imligr(iu )
-  iwarnp = iwarni(ipr)
-  epsrgp = epsrgr(iu )
-  climgp = climgr(iu )
-
-  call inimav                                                     &
- ( ivarfl(iu)      , itypfl ,                                     &
-   iflmb0 , init   , inc    , imrgra , nswrp  , imligp ,          &
-   iwarnp ,                                                       &
-   epsrgp , climgp ,                                              &
-   crom   , brom   ,                                              &
-   trav   ,                                                       &
-   coefav , coefbv ,                                              &
-   viscf  , viscb  )
-
-  ! Compute div(rho dt/rho*grad P)
-  allocate(xnormp(ncelet))
-
-  init = 1
-  call divmas(init,viscf,viscb,xnormp)
-
-!-- Volumic Gamma source term adding for volumic mass flow rate
-  if (ncesmp.gt.0) then
-    do ii = 1, ncesmp
-      iel = icetsm(ii)
-      xnormp(iel) = xnormp(iel) - cell_f_vol(iel)*smacel(ii,ipr)
-    enddo
-  endif
-
-!-- Surface Gamma source term adding for surface condensation modelling
-  if (nfbpcd.gt.0) then
-    do ii = 1, nfbpcd
-      ifac= ifbpcd(ii)
-      iel = ifabor(ifac)
-      xnormp(iel) = xnormp(iel) - surfbn(ifac) * spcond(ii,ipr)
-    enddo
-  endif
-
-! --- volume Gamma source term adding for volume condensation modelling
-  if (icond.eq.1) then
-    allocate(surfbm(ncelet))
-    surfbm(:) = 0.d0
-
-    do ii = 1, ncmast
-      iel= ltmast(ii)
-      surfbm(iel) = s_metal*volume(iel)/voltot
-      xnormp(iel) = xnormp(iel)  - surfbm(iel)*svcond(iel,ipr)
-    enddo
-
-    deallocate(surfbm)
-  endif
-
-  if (idilat.ge.4) then
-    call field_get_val_s(iprpfl(iustdy(itsrho)), cpro_tsrho)
-  endif
-
-  ! Dilatable mass conservative algorithm
-  if (idilat.eq.2) then
-    do iel = 1, ncel
-      drom = crom(iel) - croma(iel)
-      xnormp(iel) = xnormp(iel) + drom*cell_f_vol(iel)/dt(iel)
-    enddo
-  ! Semi-analytic weakly compressible algorithm add + 1/rho Drho/Dt
-  else if (idilat.eq.4)then
-    do iel = 1, ncel
-      xnormp(iel) = xnormp(iel) + cpro_tsrho(iel)/crom(iel)
-    enddo
-  else if (idilat.eq.5) then
-    do iel = 1, ncel
-      xnormp(iel) = xnormp(iel) + cpro_tsrho(iel)
-    enddo
-  endif
-
-  ! Cavitation source term
-  if (icavit.gt.0) then
-    do iel = 1, ncel
-      xnormp(iel) = xnormp(iel) -cell_f_vol(iel)*gamcav(iel)*(1.d0/rov - 1.d0/rol)
-    enddo
-  endif
-
-!     On conserve XNORMP, on complete avec u* a la fin et
-!       on le transfere a resopv
-
-endif
-
-
 !     Au premier appel, TRAV est construit directement ici.
 !     Au second  appel (estimateurs), TRAV contient deja
 !       l'increment temporel.
@@ -573,30 +450,30 @@ endif
 if (iappel.eq.1) then
   if (iphydr.eq.1) then
     do iel = 1, ncel
-      trav(1,iel) = (frcxt(1 ,iel) - grad(1,iel)) * cell_f_vol(iel)
-      trav(2,iel) = (frcxt(2 ,iel) - grad(2,iel)) * cell_f_vol(iel)
-      trav(3,iel) = (frcxt(3 ,iel) - grad(3,iel)) * cell_f_vol(iel)
+      trav(1,iel) = (frcxt(1 ,iel) - cpro_gradp(1,iel)) * cell_f_vol(iel)
+      trav(2,iel) = (frcxt(2 ,iel) - cpro_gradp(2,iel)) * cell_f_vol(iel)
+      trav(3,iel) = (frcxt(3 ,iel) - cpro_gradp(3,iel)) * cell_f_vol(iel)
     enddo
   else if (iphydr.eq.2) then
     do iel = 1, ncel
       rom = crom(iel)
-      trav(1,iel) = (rom*gx - grdphd(1,iel) - grad(1,iel)) * cell_f_vol(iel)
-      trav(2,iel) = (rom*gy - grdphd(2,iel) - grad(2,iel)) * cell_f_vol(iel)
-      trav(3,iel) = (rom*gz - grdphd(3,iel) - grad(3,iel)) * cell_f_vol(iel)
+      trav(1,iel) = (rom*gx - grdphd(1,iel) - cpro_gradp(1,iel)) * cell_f_vol(iel)
+      trav(2,iel) = (rom*gy - grdphd(2,iel) - cpro_gradp(2,iel)) * cell_f_vol(iel)
+      trav(3,iel) = (rom*gz - grdphd(3,iel) - cpro_gradp(3,iel)) * cell_f_vol(iel)
     enddo
   else if (ippmod(icompf).ge.0) then
     do iel = 1, ncel
       rom = crom(iel)
-      trav(1,iel) = (rom*gx - grad(1,iel)) * cell_f_vol(iel)
-      trav(2,iel) = (rom*gy - grad(2,iel)) * cell_f_vol(iel)
-      trav(3,iel) = (rom*gz - grad(3,iel)) * cell_f_vol(iel)
+      trav(1,iel) = (rom*gx - cpro_gradp(1,iel)) * cell_f_vol(iel)
+      trav(2,iel) = (rom*gy - cpro_gradp(2,iel)) * cell_f_vol(iel)
+      trav(3,iel) = (rom*gz - cpro_gradp(3,iel)) * cell_f_vol(iel)
     enddo
   else
     do iel = 1, ncel
       drom = (crom(iel)-ro0)
-      trav(1,iel) = (drom*gx - grad(1,iel) ) * cell_f_vol(iel)
-      trav(2,iel) = (drom*gy - grad(2,iel) ) * cell_f_vol(iel)
-      trav(3,iel) = (drom*gz - grad(3,iel) ) * cell_f_vol(iel)
+      trav(1,iel) = (drom*gx - cpro_gradp(1,iel) ) * cell_f_vol(iel)
+      trav(2,iel) = (drom*gy - cpro_gradp(2,iel) ) * cell_f_vol(iel)
+      trav(3,iel) = (drom*gz - cpro_gradp(3,iel) ) * cell_f_vol(iel)
     enddo
   endif
 
@@ -604,30 +481,30 @@ else if(iappel.eq.2) then
 
   if (iphydr.eq.1) then
     do iel = 1, ncel
-      trav(1,iel) = trav(1,iel) + (frcxt(1 ,iel) - grad(1,iel))*cell_f_vol(iel)
-      trav(2,iel) = trav(2,iel) + (frcxt(2 ,iel) - grad(2,iel))*cell_f_vol(iel)
-      trav(3,iel) = trav(3,iel) + (frcxt(3 ,iel) - grad(3,iel))*cell_f_vol(iel)
+      trav(1,iel) = trav(1,iel) + (frcxt(1 ,iel) - cpro_gradp(1,iel))*cell_f_vol(iel)
+      trav(2,iel) = trav(2,iel) + (frcxt(2 ,iel) - cpro_gradp(2,iel))*cell_f_vol(iel)
+      trav(3,iel) = trav(3,iel) + (frcxt(3 ,iel) - cpro_gradp(3,iel))*cell_f_vol(iel)
     enddo
   else if (iphydr.eq.2) then
     do iel = 1, ncel
       rom = crom(iel)
-      trav(1,iel) = trav(1,iel) + (rom*gx - grdphd(1,iel) - grad(1,iel))*cell_f_vol(iel)
-      trav(2,iel) = trav(2,iel) + (rom*gy - grdphd(2,iel) - grad(2,iel))*cell_f_vol(iel)
-      trav(3,iel) = trav(3,iel) + (rom*gz - grdphd(3,iel) - grad(3,iel))*cell_f_vol(iel)
+      trav(1,iel) = trav(1,iel) + (rom*gx - grdphd(1,iel) - cpro_gradp(1,iel))*cell_f_vol(iel)
+      trav(2,iel) = trav(2,iel) + (rom*gy - grdphd(2,iel) - cpro_gradp(2,iel))*cell_f_vol(iel)
+      trav(3,iel) = trav(3,iel) + (rom*gz - grdphd(3,iel) - cpro_gradp(3,iel))*cell_f_vol(iel)
     enddo
   else
     do iel = 1, ncel
       drom = (crom(iel)-ro0)
-      trav(1,iel) = trav(1,iel) + (drom*gx - grad(1,iel))*cell_f_vol(iel)
-      trav(2,iel) = trav(2,iel) + (drom*gy - grad(2,iel))*cell_f_vol(iel)
-      trav(3,iel) = trav(3,iel) + (drom*gz - grad(3,iel))*cell_f_vol(iel)
+      trav(1,iel) = trav(1,iel) + (drom*gx - cpro_gradp(1,iel))*cell_f_vol(iel)
+      trav(2,iel) = trav(2,iel) + (drom*gy - cpro_gradp(2,iel))*cell_f_vol(iel)
+      trav(3,iel) = trav(3,iel) + (drom*gz - cpro_gradp(3,iel))*cell_f_vol(iel)
     enddo
   endif
 
 endif
 
 ! Free memory
-deallocate(grad)
+if (allocated(grad)) deallocate(grad)
 
 
 !   Pour IAPPEL = 1 (ie appel standard sans les estimateurs)
@@ -704,8 +581,8 @@ if (iappel.eq.1) then
   if (idilat.gt.1.or.ippmod(icompf).ge.0) then
     call field_get_val_prev_s(icrom, pcrom)
 
-  ! Cavitation
-  else if (icavit.ge.0) then
+  ! VOF algorithm
+  else if (ivofmt.ge.0) then
     call field_get_val_s(icroaa, pcrom)
 
   ! Standard algo
@@ -716,7 +593,7 @@ if (iappel.eq.1) then
 
   do iel = 1, ncel
     do isou = 1, 3
-      fimp(isou,isou,iel) = istat(iu)*pcrom(iel)/dt(iel)*cell_f_vol(iel)
+      fimp(isou,isou,iel) = vcopt_u%istat*pcrom(iel)/dt(iel)*cell_f_vol(iel)
       do jsou = 1, 3
         if(jsou.ne.isou) fimp(isou,jsou,iel) = 0.d0
       enddo
@@ -884,7 +761,7 @@ endif
 if (iappel.eq.1) then
   if (ncepdp.gt.0) then
     ! The theta-scheme for the head loss is the same as the other terms
-    thetap = thetav(iu)
+    thetap = vcopt_u%thetav
     do ielpdc = 1, ncepdp
       iel = icepdc(ielpdc)
       romvom = crom(iel)*cell_f_vol(iel)*thetap
@@ -924,19 +801,33 @@ if ((icorio.eq.1.or.iturbo.eq.1) .and. iphydr.ne.1) then
     ! Si on n'itere pas sur navsto : TRAV
     if (nterup.eq.1) then
 
-      call field_get_val_s(icrom, crom)
-
+      ! Reference frame rotation
       do iel = 1, ncel
-        romvom = -ccorio*crom(iel)*cell_f_vol(iel)
-        call add_coriolis_v(irotce(iel), romvom, vela(:,iel), trav(:,iel))
+        romvom = -2.d0*crom(iel)*cell_f_vol(iel)
+        call add_coriolis_v(0, romvom, vela(:,iel), trav(:,iel))
+      enddo
+      ! Turbomachinery frozen rotors rotation
+      do iel = 1, ncel
+        if (irotce(iel).gt.0) then
+          romvom = -crom(iel)*cell_f_vol(iel)
+          call add_coriolis_v(irotce(iel), romvom, vela(:,iel), trav(:,iel))
+        endif
       enddo
 
     ! Si on itere sur navsto : TRAVA
     else
 
+      ! Reference frame rotation
       do iel = 1, ncel
-        romvom = -ccorio*crom(iel)*cell_f_vol(iel)
-        call add_coriolis_v(irotce(iel), romvom, vela(:,iel), trava(:,iel))
+        romvom = -2.d0*crom(iel)*cell_f_vol(iel)
+        call add_coriolis_v(0, romvom, vela(:,iel), trava(:,iel))
+      enddo
+      ! Turbomachinery frozen rotors rotation
+      do iel = 1, ncel
+        if (irotce(iel).gt.0) then
+          romvom = -crom(iel)*cell_f_vol(iel)
+          call add_coriolis_v(irotce(iel), romvom, vela(:,iel), trava(:,iel))
+        endif
       enddo
 
     endif
@@ -949,12 +840,20 @@ endif
 if(iappel.eq.1) then
   if (icorio.eq.1 .or. iturbo.eq.1) then
     ! The theta-scheme for the Coriolis term is the same as the other terms
-    thetap = thetav(iu)
+    thetap = vcopt_u%thetav
 
+    ! Reference frame rotation
     do iel = 1, ncel
-      romvom = -ccorio*crom(iel)*cell_f_vol(iel)*thetap
+      romvom = -2.d0*crom(iel)*cell_f_vol(iel)*thetap
       call add_coriolis_t(irotce(iel), romvom, fimp(:,:,iel))
     enddo
+    ! Turbomachinery frozen rotors rotation
+    do iel = 1, ncel
+      if (irotce(iel).gt.0) then
+        romvom = -crom(iel)*cell_f_vol(iel)*thetap
+        call add_coriolis_t(irotce(iel), romvom, fimp(:,:,iel))
+      endif
+    enddo
 
   endif
 endif
@@ -1041,11 +940,12 @@ if(itytur.eq.3.and.iterns.eq.1) then
   init = 1;
   inc  = 1;
   iflmb0 = 0;
-  nswrgp = nswrgr(ir11);
-  imligp = imligr(ir11);
-  iwarnp = iwarni(ir11);
-  epsrgp = epsrgr(ir11);
-  climgp = climgr(ir11);
+  call field_get_key_struct_var_cal_opt(ivarfl(ir11), vcopt)
+  nswrgp = vcopt%nswrgr;
+  imligp = vcopt%imligr;
+  iwarnp = vcopt%iwarni;
+  epsrgp = vcopt%epsrgr;
+  climgp = vcopt%climgr;
   itypfl = 1;
 
   allocate(tflmas(3,nfac))
@@ -1118,10 +1018,10 @@ endif
 !-------------------------------------------------------------------------------
 ! ---> Face diffusivity for the velocity
 
-if (idiff(iu).ge. 1) then
+if (vcopt_u%idiff.ge. 1) then
 
-  call field_get_val_s(iprpfl(iviscl), viscl)
-  call field_get_val_s(iprpfl(ivisct), visct)
+  call field_get_val_s(iviscl, viscl)
+  call field_get_val_s(ivisct, visct)
 
   if (itytur.eq.3) then
     do iel = 1, ncel
@@ -1129,12 +1029,12 @@ if (idiff(iu).ge. 1) then
     enddo
   else
     do iel = 1, ncel
-      w1(iel) = viscl(iel) + idifft(iu)*visct(iel)
+      w1(iel) = viscl(iel) + vcopt_u%idifft*visct(iel)
     enddo
   endif
 
   ! Scalar diffusivity (Default)
-  if (idften(iu).eq.1) then
+  if (iand(idftnp, ISOTROPIC_DIFFUSION).ne.0) then
 
     call viscfa &
    ( imvisf ,                                                       &
@@ -1146,7 +1046,7 @@ if (idiff(iu).ge. 1) then
     if(itytur.eq.3.and.irijnu.eq.1) then
 
       do iel = 1, ncel
-        w1(iel) = viscl(iel) + idifft(iu)*visct(iel)
+        w1(iel) = viscl(iel) + vcopt_u%idifft*visct(iel)
       enddo
 
       call viscfa &
@@ -1156,7 +1056,7 @@ if (idiff(iu).ge. 1) then
     endif
 
   ! Tensorial diffusion of the velocity (in case of tensorial porosity)
-  else if (idften(iu).eq.6) then
+  else if (iand(idftnp, ANISOTROPIC_LEFT_DIFFUSION).ne.0) then
 
     do iel = 1, ncel
       do isou = 1, 3
@@ -1177,7 +1077,7 @@ if (idiff(iu).ge. 1) then
     if(itytur.eq.3.and.irijnu.eq.1) then
 
       do iel = 1, ncel
-        w1(iel) = viscl(iel) + idifft(iu)*visct(iel)
+        w1(iel) = viscl(iel) + vcopt_u%idifft*visct(iel)
       enddo
 
       do iel = 1, ncel
@@ -1264,9 +1164,18 @@ if (iappel.eq.1.and.iphydr.eq.1.and.iterns.eq.1) then
 
   ! Add Coriolis force
   if (icorio.eq.1 .or. iturbo.eq.1) then
+
+    ! Reference frame rotation
     do iel = 1, ncel
-      rom = -ccorio*crom(iel)
-      call add_coriolis_v(irotce(iel), rom, vela(:,iel), dfrcxt(:,iel))
+      rom = -2.d0*crom(iel)
+      call add_coriolis_v(0, rom, vela(:,iel), dfrcxt(:,iel))
+    enddo
+    ! Turbomachinery frozen rotors rotation
+    do iel = 1, ncel
+      if (irotce(iel).gt.0) then
+        rom = -crom(iel)
+        call add_coriolis_v(irotce(iel), rom, vela(:,iel), dfrcxt(:,iel))
+      endif
     enddo
   endif
 
@@ -1280,7 +1189,6 @@ if (iappel.eq.1.and.iphydr.eq.1.and.iterns.eq.1) then
     enddo
   endif
 
-
   if (irangp.ge.0.or.iperio.eq.1) then
     call synvin(dfrcxt)
   endif
@@ -1328,7 +1236,7 @@ do iel = 1, ncel
   enddo
 enddo
 
-! The computation of esplicit and implicit source terms is performed
+! The computation of explicit and implicit source terms is performed
 ! at the first iteration only.
 if (iterns.eq.1) then
 
@@ -1351,7 +1259,7 @@ if (iterns.eq.1) then
    dt     ,                                                       &
    ckupdc , smacel , tsexp  , tsimp  )
 
-  if (ibdtso(iu).gt.1.and.ntcabs.gt.ntinit &
+  if (vcopt_u%ibdtso.gt.1.and.ntcabs.gt.ntinit &
       .and.(idtvar.eq.0.or.idtvar.eq.1)) then
     ! TODO: remove test on ntcabs and implemente a "proper" condition for
     ! initialization.
@@ -1359,7 +1267,21 @@ if (iterns.eq.1) then
     call cs_backward_differentiation_in_time(f_id, tsexp, tsimp)
   endif
   ! Skip first time step after restart if previous values have not been read.
-  if (ibdtso(iu).lt.0) ibdtso(iu) = iabs(ibdtso(iu))
+  if (vcopt_u%ibdtso.lt.0) then
+    vcopt_u%ibdtso = iabs(vcopt_u%ibdtso)
+    call field_set_key_struct_var_cal_opt(ivarfl(iu), vcopt_u)
+  endif
+
+  ! Nudging towards optimal interpolation for velocity
+  if (ippmod(iatmos).ge.0) then
+    call field_get_key_int(ivarfl(iu), kopint, f_oi_id)
+    if (f_oi_id.ge.0) then
+      call cs_at_data_assim_source_term(ivarfl(iu), tsexp, tsimp)
+    endif
+    if (iatmst.eq.1) then
+      call cs_at_source_term_for_inlet(tsexp)
+    endif
+  endif
 
   ! Coupling between two Code_Saturne
   if (nbrcpl.gt.0) then
@@ -1368,7 +1290,6 @@ if (iterns.eq.1) then
   endif
 
   if (iphydr.eq.1.and.igpust.eq.1) then
-
     do iel = 1, ncel
       !FIXME when using porosity
       dvol = 1.d0/cell_f_vol(iel)
@@ -1403,7 +1324,7 @@ if (iterns.eq.1) then
       do isou = 1, 3
         do jsou = 1, 3
           trava(isou,iel) = trava(isou,iel)                                  &
-                          + tsimp(isou,jsou,iel)*vela(jsou,iel)
+                          + tsimp(jsou,isou,iel)*vela(jsou,iel)
         enddo
       enddo
     enddo
@@ -1412,7 +1333,7 @@ if (iterns.eq.1) then
       do isou = 1, 3
         do jsou = 1, 3
           trav(isou,iel) = trav(isou,iel)                                    &
-                         + tsimp(isou,jsou,iel)*vela(jsou,iel)
+                         + tsimp(jsou,isou,iel)*vela(jsou,iel)
         enddo
       enddo
     enddo
@@ -1452,7 +1373,7 @@ endif
 if (iappel.eq.1) then
   ! If source terms are time-extrapolated
   if (isno2t.gt.0) then
-    thetap = thetav(iu)
+    thetap = vcopt_u%thetav
     if (iterns.gt.1) then
       do iel = 1, ncel
         do isou = 1, 3
@@ -1553,7 +1474,7 @@ if (ncesmp.gt.0) then
 
 endif
 
-! ---> Right Han Side initialization
+! ---> Right Hand Side initialization
 
 ! If source terms are extrapolated in time
 if (isno2t.gt.0) then
@@ -1603,9 +1524,11 @@ endif
 
 if (iilagr.eq.2 .and. ltsdyn.eq.1)  then
 
+  call field_get_val_v_by_name('velocity_st_lagr', lagr_st_vel)
+
   do iel = 1, ncel
     do isou = 1, 3
-      smbr(isou,iel) = smbr(isou,iel) + tslagr(iel,itsvx+isou-1)
+      smbr(isou,iel) = smbr(isou,iel) + lagr_st_vel(isou,iel)
     enddo
   enddo
   ! At the second call, fimp is unused
@@ -1624,33 +1547,32 @@ endif
 if (ippmod(ielarc).ge.1) then
   call field_get_val_v_by_name('laplace_force', lapla)
   do iel = 1, ncel
-      smbr(1,iel) = smbr(1,iel) + cell_f_vol(iel) * lapla(1,iel)
-      smbr(2,iel) = smbr(2,iel) + cell_f_vol(iel) * lapla(2,iel)
-      smbr(3,iel) = smbr(3,iel) + cell_f_vol(iel) * lapla(3,iel)
+    smbr(1,iel) = smbr(1,iel) + cell_f_vol(iel) * lapla(1,iel)
+    smbr(2,iel) = smbr(2,iel) + cell_f_vol(iel) * lapla(2,iel)
+    smbr(3,iel) = smbr(3,iel) + cell_f_vol(iel) * lapla(3,iel)
   enddo
 endif
 
 ! Solver parameters
-iconvp = iconv (iu)
-idiffp = idiff (iu)
+iconvp = vcopt_u%iconv
+idiffp = vcopt_u%idiff
 ndircp = ndircl(iu)
-nswrsp = nswrsm(iu)
-nswrgp = nswrgr(iu)
-imligp = imligr(iu)
-ircflp = ircflu(iu)
-ischcp = ischcv(iu)
-isstpp = isstpc(iu)
-idftnp = idften(iu)
-iswdyp = iswdyn(iu)
-iwarnp = iwarni(iu)
-blencp = blencv(iu)
-epsilp = epsilo(iu)
-epsrsp = epsrsm(iu)
-epsrgp = epsrgr(iu)
-climgp = climgr(iu)
-extrap = extrag(iu)
-relaxp = relaxv(iu)
-thetap = thetav(iu)
+nswrsp = vcopt_u%nswrsm
+nswrgp = vcopt_u%nswrgr
+imligp = vcopt_u%imligr
+ircflp = vcopt_u%ircflu
+ischcp = vcopt_u%ischcv
+isstpp = vcopt_u%isstpc
+iswdyp = vcopt_u%iswdyn
+iwarnp = vcopt_u%iwarni
+blencp = vcopt_u%blencv
+epsilp = vcopt_u%epsilo
+epsrsp = vcopt_u%epsrsm
+epsrgp = vcopt_u%epsrgr
+climgp = vcopt_u%climgr
+extrap = vcopt_u%extrag
+relaxp = vcopt_u%relaxv
+thetap = vcopt_u%thetav
 
 if (ippmod(icompf).ge.0) then
   ! impose boundary convective flux at some faces (face indicator icvfli)
@@ -1669,7 +1591,7 @@ if (iappel.eq.1) then
     ! Warning: in case of convergence estimators, eswork give the estimator
     ! of the predicted velocity
     call coditv &
- ( idtvar , iu     , iconvp , idiffp , ndircp ,                   &
+ ( idtvar , iterns , ivarfl(iu)      , iconvp , idiffp , ndircp ,&
    imrgra , nswrsp , nswrgp , imligp , ircflp , ivisse ,          &
    ischcp , isstpp , iescap , idftnp , iswdyp ,                   &
    iwarnp ,                                                       &
@@ -1679,6 +1601,7 @@ if (iappel.eq.1) then
    coefav , coefbv , cofafv , cofbfv ,                            &
    flumas , flumab ,                                              &
    viscfi , viscbi , viscf  , viscb  , secvif , secvib ,          &
+   rvoid  , rvoid  , rvoid  ,                                     &
    icvflb , icvfli ,                                              &
    fimp   ,                                                       &
    smbr   ,                                                       &
@@ -1688,7 +1611,7 @@ if (iappel.eq.1) then
   else if(iterns.gt.1) then
 
     call coditv &
- ( idtvar , iu     , iconvp , idiffp , ndircp ,                   &
+ ( idtvar , iterns , ivarfl(iu)      , iconvp , idiffp , ndircp , &
    imrgra , nswrsp , nswrgp , imligp , ircflp , ivisse ,          &
    ischcp , isstpp , iescap , idftnp , iswdyp ,                   &
    iwarnp ,                                                       &
@@ -1698,6 +1621,7 @@ if (iappel.eq.1) then
    coefav , coefbv , cofafv , cofbfv ,                            &
    flumas , flumab ,                                              &
    viscfi , viscbi , viscf  , viscb  , secvif , secvib ,          &
+   rvoid  , rvoid  , rvoid  ,                                     &
    icvflb , icvfli ,                                              &
    fimp   ,                                                       &
    smbr   ,                                                       &
@@ -1733,7 +1657,7 @@ if (iappel.eq.1) then
     ivisep = 0
 
     call coditv &
- ( idtvar , iu     , iconvp , idiffp , ndircp ,                   &
+ ( idtvar , iterns , ivarfl(iu)      , iconvp , idiffp , ndircp , &
    imrgra , nswrsp , nswrgp , imligp , ircflp , ivisep ,          &
    ischcp , isstpp , iescap , idftnp , iswdyp ,                   &
    iwarnp ,                                                       &
@@ -1743,6 +1667,7 @@ if (iappel.eq.1) then
    coefav , coefbv , cofafv , cofbfv ,                            &
    flumas , flumab ,                                              &
    viscfi , viscbi , viscf  , viscb  , secvif , secvib ,          &
+   rvoid  , rvoid  , rvoid  ,                                     &
    icvflb , ivoid  ,                                              &
    fimp   ,                                                       &
    smbr   ,                                                       &
@@ -1785,14 +1710,15 @@ else if (iappel.eq.2) then
   imasac = 0
 
   call bilscv &
- ( idtva0 , iu     , iconvp , idiffp , nswrgp , imligp , ircflp , &
-   ischcp , isstpp , inc    , imrgra , ivisse ,                   &
-   iwarnp , idftnp , imasac ,                                     &
-   blencp , epsrgp , climgp , relaxp , thetap ,                   &
-   vel    , vel    ,                                              &
-   coefav , coefbv , cofafv , cofbfv ,                            &
-   flumas , flumab , viscf  , viscb  , secvif , secvib ,          &
-   icvflb , icvfli ,                                              &
+ ( idtva0 , ivarfl(iu)      , iconvp , idiffp , nswrgp , imligp , ircflp , &
+   ischcp , isstpp , inc    , imrgra , ivisse ,                            &
+   iwarnp , idftnp , imasac ,                                              &
+   blencp , epsrgp , climgp , relaxp , thetap ,                            &
+   vel    , vel    ,                                                       &
+   coefav , coefbv , cofafv , cofbfv ,                                     &
+   flumas , flumab , viscf  , viscb  , secvif , secvib ,                   &
+   rvoid  , rvoid  , rvoid  ,                                              &
+   icvflb , icvfli ,                                                       &
    smbr   )
 
   call field_get_val_s(iestim(iestot), c_estim)
@@ -1803,52 +1729,6 @@ else if (iappel.eq.2) then
   enddo
 endif
 
-!===============================================================================
-! 4. Finalize the norm of the pressure step (see resopv)
-!===============================================================================
-
-if (iappel.eq.1.and.irnpnw.eq.1) then
-
-  ! Compute div(rho u*)
-
-  if (irangp.ge.0.or.iperio.eq.1) then
-    call synvin(vel)
-  endif
-
-  ! To save time, no space reconstruction
-  itypfl = 1
-  ! Cavitation algorithm: the pressure step corresponds to the
-  ! correction of the volumetric flux, not the mass flux
-  if (icavit.ge.0)  itypfl = 0
-  init   = 1
-  inc    = 1
-  iflmb0 = 1
-  nswrp  = 1
-  imligp = imligr(iu )
-  iwarnp = iwarni(ipr)
-  epsrgp = epsrgr(iu )
-  climgp = climgr(iu )
-
-  call inimav &
- ( ivarfl(iu)      , itypfl ,                                     &
-   iflmb0 , init   , inc    , imrgra , nswrp  , imligp ,          &
-   iwarnp ,                                                       &
-   epsrgp , climgp ,                                              &
-   crom   , brom   ,                                              &
-   vel    ,                                                       &
-   coefav , coefbv ,                                              &
-   viscf  , viscb  )
-
-  init = 0
-  call divmas(init,viscf,viscb,xnormp)
-
-  ! Compute the norm rnormp used in resopv
-  rnormp = sqrt(cs_gdot(ncel,xnormp,xnormp))
-
-  ! Free memory
-  deallocate(xnormp)
-endif
-
 ! ---> Finilaze estimators + Printings
 
 if (iappel.eq.1) then
@@ -1869,7 +1749,7 @@ if (iappel.eq.1) then
   endif
 
   ! ---> Norm printings
-  if (iwarni(iu).ge.2) then
+  if (vcopt_u%iwarni.ge.2) then
     rnorm = -1.d0
     do iel = 1, ncel
       vitnor = sqrt(vel(1,iel)**2+vel(2,iel)**2+vel(3,iel)**2)
