@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,35 +20,31 @@
 
 !-------------------------------------------------------------------------------
 
-subroutine schtmp &
-!================
-
- ( nscal  , iappel ,                                              &
-   propce )
-
 !===============================================================================
-! FONCTION :
+! Function :
 ! ----------
 
-! TRAITEMENT DU FLUX DE MASSE, DE LA VISCOSITE, DE LA MASSE
-! VOLUMIQUE, DE LA CHALEUR SPECIFIQUE ET DU TABLEAU TSNSA
-! DANS LE CAS D'UN THETA SCHEMA
+!> \file schtmp.f90
+!>
+!> \brief Management of the mass flux, the viscosity, the density, the specific
+!> heat  and the tsnsa array in case of a theta-scheme.
+!>
+!> Please refer to the
+!> <a href="../../theory.pdf#massflux"><b>mass flux</b></a> section
+!> of the theory guide for more informations.
+!-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
-! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-! nscal            ! i  ! <-- ! total number of scalars                        !
-! iappel           ! e  ! <-- ! numero de l'appel (avant ou apres              !
-!                  !    !     ! phyvar                                         !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
-!__________________!____!_____!________________________________________________!
+! Arguments                                                                    !
+!______________________________________________________________________________!
+!  mode           name               role                                      !
+!______________________________________________________________________________!
+!> \param[in]     nscal              total number of scalars
+!> \param[in]     iappel             call number (before of after phyvar
+!______________________________________________________________________________
 
-!     Type: i (integer), r (real), s (string), a (array), l (logical),
-!           and composite types (ex: ra real array)
-!     mode: <-- input, --> output, <-> modifies data, --- work array
-!===============================================================================
+subroutine schtmp &
+ ( nscal  , iappel )
 
 !===============================================================================
 ! Module files
@@ -73,20 +69,20 @@ implicit none
 
 integer          nscal  , iappel
 
-double precision propce(ncelet,*)
-
 ! Local variables
 
 integer          iel    , ifac   , iscal
 integer          iflmas , iflmab
-integer          ipcvsa , ipcvta , ifcvsl
-integer          iicpa
+integer          ifcvsl
 double precision flux   , theta  , aa, bb, viscos, xmasvo, varcp
 double precision, dimension(:), pointer :: i_mass_flux, b_mass_flux
 double precision, dimension(:), pointer :: i_mass_flux_prev, b_mass_flux_prev
 double precision, dimension(:), pointer :: brom, crom, broma, croma
-double precision, dimension(:), pointer :: viscl, visct
-double precision, dimension(:), pointer :: cpro_cp, cpro_viscls, cproa_viscls
+double precision, dimension(:), pointer :: cpro_viscl, cpro_visct
+double precision, dimension(:), pointer :: cpro_cp, cpro_visls
+double precision, dimension(:), pointer :: cproa_cp, cproa_visls, cproa_visct
+double precision, dimension(:), pointer :: cpro_romls
+double precision, dimension(:), pointer :: cproa_romls
 
 !===============================================================================
 
@@ -148,37 +144,49 @@ if (iappel.eq.1) then
     call field_current_to_previous(ibrom)
   endif
   if (iviext.gt.0) then
-    call field_get_val_s(iprpfl(iviscl), viscl)
-    call field_get_val_s(iprpfl(ivisct), visct)
-    ipcvsa = ipproc(ivisla)
-    ipcvta = ipproc(ivista)
+    call field_get_val_s(iviscl, cpro_viscl)
+    call field_get_val_s(ivisct, cpro_visct)
+    call field_get_val_prev_s(iviscl, cproa_visls)
+    call field_get_val_prev_s(ivisct, cproa_visct)
     do iel = 1, ncel
-      propce(iel,ipcvsa) = viscl(iel)
-      propce(iel,ipcvta) = visct(iel)
+      cproa_visls(iel) = cpro_viscl(iel)
+      cproa_visct(iel) = cpro_visct(iel)
     enddo
   endif
   if (icpext.gt.0) then
-    if (icp.gt.0) then
-      call field_get_val_s(iprpfl(icp), cpro_cp)
-      iicpa  = ipproc(icpa  )
+    if (icp.ge.0) then
+      call field_get_val_s(icp, cpro_cp)
+      call field_get_val_prev_s(icp, cproa_cp)
       do iel = 1, ncel
-        propce(iel,iicpa ) = cpro_cp(iel)
+        cproa_cp(iel) = cpro_cp(iel)
       enddo
     endif
   endif
 
-!     Remarque : si on faisait cette operation pour tous les
-!       scalaires, on la ferait plusieurs fois pour les scalaires
-!       ayant une variance
+  ! Remarque : si on faisait cette operation pour tous les
+  ! scalaires, on la ferait plusieurs fois pour les scalaires
+  ! ayant une variance
   if (nscal.ge.1) then
     do iscal = 1, nscal
+      ! Diffusivity
       call field_get_key_int (ivarfl(isca(iscal)), kivisl, ifcvsl)
       if (ifcvsl.ge.0.and.iscavr(iscal).le.0) then
         if (ivsext(iscal).gt.0) then
-          call field_get_val_s(ifcvsl, cpro_viscls)
-          call field_get_val_prev_s(ifcvsl, cproa_viscls)
+          call field_get_val_s(ifcvsl, cpro_visls)
+          call field_get_val_prev_s(ifcvsl, cproa_visls)
           do iel = 1, ncel
-            cproa_viscls(iel) = cpro_viscls(iel)
+            cproa_visls(iel) = cpro_visls(iel)
+          enddo
+        endif
+      endif
+      ! Densisty
+      call field_get_key_int (ivarfl(isca(iscal)), kromsl, ifcvsl)
+      if (ifcvsl.ge.0.and.iscavr(iscal).le.0) then
+        if (iroext.gt.0) then
+          call field_get_val_s(ifcvsl, cpro_romls)
+          call field_get_val_prev_s(ifcvsl, cproa_romls)
+          do iel = 1, ncel
+            cproa_romls(iel) = cpro_romls(iel)
           enddo
         endif
       endif
@@ -214,13 +222,13 @@ elseif (iappel.eq.2) then
   if (initvi.ne.1) then
     initvi = 1
     if (iviext.gt.0) then
-      call field_get_val_s(iprpfl(iviscl), viscl)
-      call field_get_val_s(iprpfl(ivisct), visct)
-      ipcvsa = ipproc(ivisla)
-      ipcvta = ipproc(ivista)
+      call field_get_val_s(iviscl, cpro_viscl)
+      call field_get_val_s(ivisct, cpro_visct)
+      call field_get_val_prev_s(iviscl, cproa_visls)
+      call field_get_val_prev_s(ivisct, cproa_visct)
       do iel = 1, ncel
-        propce(iel,ipcvsa) = viscl(iel)
-        propce(iel,ipcvta) = visct(iel)
+        cproa_visls(iel) = cpro_viscl(iel)
+        cproa_visct(iel) = cpro_visct(iel)
       enddo
     endif
   endif
@@ -228,29 +236,30 @@ elseif (iappel.eq.2) then
     initcp = 1
     if (icpext.gt.0) then
       if (icp   .gt.0) then
-        call field_get_val_s(iprpfl(icp), cpro_cp)
-        iicpa  = ipproc(icpa)
+        call field_get_val_s(icp, cpro_cp)
+        call field_get_val_prev_s(icp, cproa_cp)
         do iel = 1, ncel
-          propce(iel,iicpa) = cpro_cp(iel)
+          cproa_cp(iel) = cpro_cp(iel)
         enddo
       endif
     endif
   endif
 
-!     Remarque : si on faisant cette operation pour tous les
-!       scalaires, on la ferait plusieurs fois pour les scalaires
-!       ayant une variance
+  ! Remarque : si on faisant cette operation pour tous les
+  ! scalaires, on la ferait plusieurs fois pour les scalaires
+  ! ayant une variance
   if (nscal.ge.1) then
     do iscal = 1, nscal
+      ! Diffusivity
       if (initvs(iscal).ne.1) then
         initvs(iscal) = 1
         call field_get_key_int (ivarfl(isca(iscal)), kivisl, ifcvsl)
         if (ifcvsl.ge.0.and.iscavr(iscal).le.0) then
           if (ivsext(iscal).gt.0) then
-            call field_get_val_s(ifcvsl, cpro_viscls)
-            call field_get_val_prev_s(ifcvsl, cproa_viscls)
+            call field_get_val_s(ifcvsl, cpro_visls)
+            call field_get_val_prev_s(ifcvsl, cproa_visls)
             do iel = 1, ncel
-              cproa_viscls(iel) = cpro_viscls(iel)
+              cproa_visls(iel) = cpro_visls(iel)
             enddo
           endif
         endif
@@ -288,55 +297,72 @@ elseif (iappel.eq.2) then
     enddo
   endif
   if (iviext.gt.0) then
-    call field_get_val_s(iprpfl(iviscl), viscl)
-    call field_get_val_s(iprpfl(ivisct), visct)
-    ipcvsa = ipproc(ivisla)
-    ipcvta = ipproc(ivista)
+    call field_get_val_s(iviscl, cpro_viscl)
+    call field_get_val_s(ivisct, cpro_visct)
+    call field_get_val_prev_s(iviscl, cproa_visls)
+    call field_get_val_prev_s(ivisct, cproa_visct)
     theta  = thetvi
     do iel = 1, ncel
-      viscos = viscl(iel)
-      viscl(iel) = (1.d0+theta) * viscl(iel)    &
-           -       theta  * propce(iel,ipcvsa)
-      propce(iel,ipcvsa) = viscos
-      viscos = visct(iel)
-      visct(iel) = (1.d0+theta) * visct(iel)    &
-           -       theta  * propce(iel,ipcvta)
-      propce(iel,ipcvta) = viscos
+      viscos = cpro_viscl(iel)
+      cpro_viscl(iel) = (1.d0+theta) * cpro_viscl(iel)    &
+           -       theta  * cproa_visls(iel)
+      cproa_visls(iel) = viscos
+      viscos = cpro_visct(iel)
+      cpro_visct(iel) = (1.d0+theta) * cpro_visct(iel)    &
+           -       theta  * cproa_visct(iel)
+      cproa_visct(iel) = viscos
     enddo
   endif
   if (icpext.gt.0) then
-    if (icp.gt.0) then
-      call field_get_val_s(iprpfl(icp), cpro_cp)
-      iicpa  = ipproc(icpa)
+    if (icp.ge.0) then
+      call field_get_val_s(icp, cpro_cp)
+      call field_get_val_prev_s(icp, cproa_cp)
       theta  = thetcp
       do iel = 1, ncel
         varcp  = cpro_cp(iel)
         cpro_cp(iel) = (1.d0+theta) * cpro_cp(iel)      &
-                      -      theta  * propce(iel,iicpa)
-        propce(iel,iicpa ) = varcp
+                      -      theta  * cproa_cp(iel)
+        cproa_cp(iel) = varcp
       enddo
     endif
   endif
 
-!     Remarque : si on faisant cette operation pour tous les
-!       scalaires, on la ferait plusieurs fois pour les scalaires
-!       ayant une variance ET CE SERAIT FAUX
+  !     Remarque : si on faisant cette operation pour tous les
+  !       scalaires, on la ferait plusieurs fois pour les scalaires
+  !       ayant une variance ET CE SERAIT FAUX
   if (nscal.ge.1) then
     do iscal = 1, nscal
+      ! Diffusivity
       call field_get_key_int (ivarfl(isca(iscal)), kivisl, ifcvsl)
       if (ifcvsl.ge.0.and.iscavr(iscal).le.0) then
         if (ivsext(iscal).gt.0) then
           theta  = thetvs(iscal)
-          call field_get_val_s(ifcvsl, cpro_viscls)
-          call field_get_val_prev_s(ifcvsl, cproa_viscls)
+          call field_get_val_s(ifcvsl, cpro_visls)
+          call field_get_val_prev_s(ifcvsl, cproa_visls)
           do iel = 1, ncel
-            viscos = cpro_viscls(iel)
-            cpro_viscls(iel) = (1.d0+theta)*cpro_viscls(iel) &
-                                    -theta *cproa_viscls(iel)
-            cproa_viscls(iel) = viscos
+            viscos = cpro_visls(iel)
+            cpro_visls(iel) = (1.d0+theta)*cpro_visls(iel) &
+                                    -theta *cproa_visls(iel)
+            cproa_visls(iel) = viscos
           enddo
         endif
       endif
+      ! Density
+      call field_get_key_int (ivarfl(isca(iscal)), kromsl, ifcvsl)
+      if (ifcvsl.ge.0.and.iscavr(iscal).le.0) then
+        if (iroext.gt.0) then
+          theta = thetvs(iscal)
+          call field_get_val_s(ifcvsl, cpro_romls)
+          call field_get_val_prev_s(ifcvsl, cproa_romls)
+          do iel = 1, ncel
+            viscos = cpro_romls(iel)
+            cpro_romls(iel) = (1.d0+theta)*cpro_romls(iel) &
+                                    -theta *cproa_romls(iel)
+            cproa_romls(iel) = viscos
+          enddo
+        endif
+      endif
+
     enddo
   endif
 
@@ -426,17 +452,8 @@ elseif (iappel.eq.4) then
   call field_get_val_prev_s(iflmas, i_mass_flux_prev)
   call field_get_val_prev_s(iflmab, b_mass_flux_prev)
 
-  if (istmpf.eq.2) then
-    theta  = thetfl
-    aa = 1.d0/(2.d0-theta)
-    bb = (1.d0-theta)/(2.d0-theta)
-    do ifac = 1 , nfac
-      i_mass_flux(ifac) = aa * i_mass_flux(ifac) + bb * i_mass_flux_prev(ifac)
-    enddo
-    do ifac = 1 , nfabor
-      b_mass_flux(ifac) = aa * b_mass_flux(ifac) + bb * b_mass_flux_prev(ifac)
-    enddo
-  else if (istmpf.eq.0) then
+  ! Already checked
+  if (istmpf.eq.0) then
     do ifac = 1 , nfac
       flux = i_mass_flux(ifac)
       i_mass_flux(ifac) = i_mass_flux_prev(ifac)
@@ -502,37 +519,49 @@ elseif (iappel.eq.5) then
     enddo
   endif
   if (iviext.gt.0) then
-    call field_get_val_s(iprpfl(iviscl), viscl)
-    call field_get_val_s(iprpfl(ivisct), visct)
-    ipcvsa = ipproc(ivisla)
-    ipcvta = ipproc(ivista)
+    call field_get_val_s(iviscl, cpro_viscl)
+    call field_get_val_s(ivisct, cpro_visct)
+    call field_get_val_prev_s(iviscl, cproa_visls)
+    call field_get_val_prev_s(ivisct, cproa_visct)
     do iel = 1, ncel
-      viscl(iel) = propce(iel,ipcvsa)
-      visct(iel) = propce(iel,ipcvta)
+      cpro_viscl(iel) = cproa_visls(iel)
+      cpro_visct(iel) = cproa_visct(iel)
     enddo
   endif
   if (icpext.gt.0) then
-    if (icp.gt.0) then
-      call field_get_val_s(iprpfl(icp), cpro_cp)
-      iicpa  = ipproc(icpa)
+    if (icp.ge.0) then
+      call field_get_val_s(icp, cpro_cp)
+      call field_get_val_prev_s(icp, cproa_cp)
       do iel = 1, ncel
-        cpro_cp(iel) = propce(iel,iicpa)
+        cpro_cp(iel) = cproa_cp(iel)
       enddo
     endif
   endif
 
-!     Remarque : si on faisant cette operation pour tous les
-!       scalaires, on la ferait plusieurs fois pour les scalaires
-!       ayant une variance
+  ! Remarque : si on faisant cette operation pour tous les
+  ! scalaires, on la ferait plusieurs fois pour les scalaires
+  ! ayant une variance
   if (nscal.ge.1) then
     do iscal = 1, nscal
+      ! Diffusivity
       call field_get_key_int (ivarfl(isca(iscal)), kivisl, ifcvsl)
       if (ifcvsl.ge.0.and.iscavr(iscal).le.0) then
         if (ivsext(iscal).gt.0) then
-          call field_get_val_s(ifcvsl, cpro_viscls)
-          call field_get_val_prev_s(ifcvsl, cproa_viscls)
+          call field_get_val_s(ifcvsl, cpro_visls)
+          call field_get_val_prev_s(ifcvsl, cproa_visls)
           do iel = 1, ncel
-            cpro_viscls(iel) = cproa_viscls(iel)
+            cpro_visls(iel) = cproa_visls(iel)
+          enddo
+        endif
+      endif
+      ! Density
+      call field_get_key_int (ivarfl(isca(iscal)), kivisl, ifcvsl)
+      if (ifcvsl.ge.0.and.iscavr(iscal).le.0) then
+        if (iroext.gt.0) then
+          call field_get_val_s(ifcvsl, cpro_romls)
+          call field_get_val_prev_s(ifcvsl, cproa_romls)
+          do iel = 1, ncel
+            cpro_romls(iel) = cproa_romls(iel)
           enddo
         endif
       endif
