@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -64,13 +64,12 @@
 !>                                 -# for a scalar \f$ cp \left( K +
 !>                                     \dfrac{K_T}{\sigma_T} \right)
 !>                                     \grad T \cdot \vect{n} \f$
-!> \param[in]     frcxt         external force generating hydrostatic pressure
 !______________________________________________________________________________
 
 subroutine typecl &
  ( nvar   , nscal  ,                                              &
    itypfb , itrifb , icodcl , isostd ,                            &
-   rcodcl , frcxt  )
+   rcodcl )
 
 !===============================================================================
 ! Module files
@@ -102,39 +101,48 @@ implicit none
 
 integer          nvar   , nscal
 
-integer          icodcl(ndimfb,nvarcl)
+integer          icodcl(ndimfb,nvar)
 integer          itypfb(ndimfb) , itrifb(ndimfb)
 integer          isostd(ndimfb+1)
 
-double precision rcodcl(ndimfb,nvarcl,3)
-double precision frcxt(3,ncelet)
+double precision rcodcl(ndimfb,nvar,3)
 
 ! Local variables
 
 integer          ifac, ivar, iel
-integer          iok, inc, iccocg, iprev, ideb, ifin, inb, isum, iwrnp
-integer          ifrslb, itbslb
-integer          ityp, ii, jj, iwaru, iflmab
-integer          irangd
+integer          iok, inc, iccocg, iprev, ideb, ifin, inb, isum
+integer          ityp, ii, jj, iflmab
 integer          ifadir
 integer          iut  , ivt   , iwt, iscal
-double precision pref, pipb
+integer          keyvar, keycpl
+integer          iivar, icpl
+integer          f_id, i_dim, f_type, nfld, f_dim
+
+double precision pref
 double precision diipbx, diipby, diipbz
 double precision flumbf, flumty(ntypmx)
-double precision xxp0, xyp0, xzp0, d0, d0min
-double precision xyzref(4) ! xyzref(3) + PI' for broadcast
+double precision d0, d0min
+double precision xyzref(3)
+
+character(len=80) :: fname
 
 double precision, allocatable, dimension(:) :: pripb
 double precision, allocatable, dimension(:,:) :: grad
 double precision, dimension(:), pointer :: bmasfl
 
+double precision, pointer, dimension(:,:) :: frcxt
 double precision, dimension(:), pointer :: cvara_pr
 double precision, dimension(:), pointer :: cpro_prtot
+double precision, dimension(1,1), target :: rvoid2
+
+integer, save :: irangd
 
 integer          ipass
 data             ipass /0/
 save             ipass
 
+type(var_cal_opt) :: vcopt
+
 !===============================================================================
 
 !===============================================================================
@@ -146,7 +154,6 @@ allocate(pripb(ndimfb))
 
 ! Initialize variables to avoid compiler warnings
 
-pipb = 0.d0
 pref = 0.d0
 
 call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
@@ -154,6 +161,13 @@ call field_get_val_s(iflmab, bmasfl)
 
 call field_get_val_prev_s(ivarfl(ipr), cvara_pr)
 
+call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt)
+
+call field_get_key_id("variable_id", keyvar)
+
+! Number of fields
+call field_get_n_fields(nfld)
+
 !===============================================================================
 ! 2.  Check consistency of types given in cs_user_boundary_conditions
 !===============================================================================
@@ -169,7 +183,8 @@ do ifac = 1, nfabor
 enddo
 
 if (irangp.ge.0) call parcmx(iok)
-if(iok.ne.0) then
+
+if (iok.ne.0) then
   call boundary_conditions_error(itypfb)
 endif
 
@@ -256,7 +271,7 @@ endif
 ! ---> On ecrit les types de faces avec la borne inf et sup et le nb
 !       pour chaque type de face trouve (tjrs pour les types par defaut)
 
-if(ipass.eq.0.or.iwarni(iu).ge.2) then
+if(ipass.eq.0.or.vcopt%iwarni.ge.2) then
 
   ipass = 1
 
@@ -318,10 +333,14 @@ if(ipass.eq.0.or.iwarni(iu).ge.2) then
     ! Presence of free entrance face(s)
     if (inb.gt.0) then
       iifren = 1
-      if (.not.(allocated(b_head_loss))) allocate(b_head_loss(ndimfb))
     else
       iifren = 0
     endif
+    if (irangp.ge.0) call parcmx(iifren)
+
+    if (iifren.eq.1) then
+      if (.not.(allocated(b_head_loss))) allocate(b_head_loss(ndimfb))
+    endif
 
     ii = i_convective_inlet
     inb = ifinty(ii)-idebty(ii)+1
@@ -474,14 +493,14 @@ if(ipass.eq.0.or.iwarni(iu).ge.2) then
 
     do ii = 1, ntypmx
       if (ii.ne.iesicf .and. &
-           ii.ne.isspcf .and. &
-           ii.ne.ieqhcf .and. &
-           ii.ne.iephcf .and. &
-           ii.ne.isopcf .and. &
-           ii.ne.iparoi .and. &
-           ii.ne.iparug .and. &
-           ii.ne.isymet .and. &
-           ii.ne.iindef ) then
+          ii.ne.isspcf .and. &
+          ii.ne.ieqhcf .and. &
+          ii.ne.iephcf .and. &
+          ii.ne.isopcf .and. &
+          ii.ne.iparoi .and. &
+          ii.ne.iparug .and. &
+          ii.ne.isymet .and. &
+          ii.ne.iindef ) then
         inb = ifinty(ii)-idebty(ii)+1
         if (irangp.ge.0) call parcpt (inb)
         if(inb.gt.0) then
@@ -506,7 +525,7 @@ endif
 !     isolib, ifrent and ientre are handled later.
 !================================================================================
 
-do ivar = 1, nvarcl
+do ivar = 1, nvar
   do ifac = 1, nfabor
     if ((itypfb(ifac) .ne. isolib)            .and. &
         (itypfb(ifac) .ne. ifrent)            .and. &
@@ -524,10 +543,6 @@ enddo
 !     (if we need it, that is if there are outlet boudary faces).
 !===============================================================================
 
-xxp0   = xyzp0(1)
-xyp0   = xyzp0(2)
-xzp0   = xyzp0(3)
-
 ! ifrslb = closest free standard outlet face to xyzp0 (icodcl not modified)
 !          (or closest free inlet)
 ! itbslb = max of ifrslb on all ranks, standard outlet face presence indicator
@@ -536,101 +551,54 @@ xzp0   = xyzp0(3)
 ! origin), we choose the face whose center is closest to it, so
 ! as to be mesh numbering (and partitioning) independent.
 
-d0min = rinfin
+if (ntcabs.eq.ntpabs+1) then
 
-ifrslb = 0
+  d0min = rinfin
 
-ideb = idebty(isolib)
-ifin = ifinty(isolib)
+  ideb = idebty(isolib)
+  ifin = ifinty(isolib)
 
-do ii = ideb, ifin
-  ifac = itrifb(ii)
-  if (icodcl(ifac,ipr).eq.0) then
-    d0 =   (cdgfbo(1,ifac)-xxp0)**2  &
-         + (cdgfbo(2,ifac)-xyp0)**2  &
-         + (cdgfbo(3,ifac)-xzp0)**2
-    if (d0.lt.d0min) then
-      ifrslb = ifac
-      d0min = d0
+  do ii = ideb, ifin
+    ifac = itrifb(ii)
+    if (icodcl(ifac,ipr).eq.0) then
+      d0 =   (cdgfbo(1,ifac)-xyzp0(1))**2  &
+           + (cdgfbo(2,ifac)-xyzp0(2))**2  &
+           + (cdgfbo(3,ifac)-xyzp0(3))**2
+      if (d0.lt.d0min) then
+        ifrslb = ifac
+        d0min = d0
+      endif
     endif
-  endif
-enddo
+  enddo
 
+  ideb = idebty(ifrent)
+  ifin = ifinty(ifrent)
 
-ideb = idebty(ifrent)
-ifin = ifinty(ifrent)
-
-do ii = ideb, ifin
-  ifac = itrifb(ii)
-  if (icodcl(ifac,ipr).eq.0) then
-    d0 =   (cdgfbo(1,ifac)-xxp0)**2  &
-         + (cdgfbo(2,ifac)-xyp0)**2  &
-         + (cdgfbo(3,ifac)-xzp0)**2
-    if (d0.lt.d0min) then
-      ifrslb = ifac
-      d0min = d0
+  do ii = ideb, ifin
+    ifac = itrifb(ii)
+    if (icodcl(ifac,ipr).eq.0) then
+      d0 =   (cdgfbo(1,ifac)-xyzp0(1))**2  &
+           + (cdgfbo(2,ifac)-xyzp0(2))**2  &
+           + (cdgfbo(3,ifac)-xyzp0(3))**2
+      if (d0.lt.d0min) then
+        ifrslb = ifac
+        d0min = d0
+      endif
     endif
+  enddo
+
+  ! If we have free outlet faces, irangd and itbslb will
+  ! contain respectively the rank having the boundary face whose
+  ! center is closest to xyzp0, and the local number of that face
+  ! on that rank (also equal to ifrslb on that rank).
+  ! If we do not have free outlet faces, than itbslb = 0
+  ! (as it was initialized that way on all ranks).
+
+  itbslb = ifrslb
+  irangd = irangp
+  if (irangp.ge.0) then
+    call parfpt(itbslb, irangd, d0min)
   endif
-enddo
-
-! If we have free outlet faces, irangd and itbslb will
-! contain respectively the rank having the boundary face whose
-! center is closest to xyzp0, and the local number of that face
-! on that rank (also equal to ifrslb on that rank).
-! If we do not have free outlet faces, than itbslb = 0
-! (as it was initialized that way on all ranks).
-
-itbslb = ifrslb
-irangd = irangp
-if (irangp.ge.0) then
-  call parfpt(itbslb, irangd, d0min)
-endif
-
-if (itbslb.gt.0) then
-
-  ! Allocate a work array for the gradient calculation
-  allocate(grad(3,ncelet))
-
-  iprev = 1
-  inc = 1
-  iccocg = 1
-
-  call field_gradient_potential(ivarfl(ipr), iprev, imrgra, inc,      &
-                                iccocg, iphydr,                       &
-                                frcxt, grad)
-
-  !  Put in pripb the value at I' or F (depending on iphydr) of the
-  !  total pressure, computed from P*
-
-  if ((iphydr.eq.0.or.iphydr.eq.2).and.iifren.eq.0) then
-    do ifac = 1, nfabor
-      ii = ifabor(ifac)
-      diipbx = diipb(1,ifac)
-      diipby = diipb(2,ifac)
-      diipbz = diipb(3,ifac)
-      pripb(ifac) = cvara_pr(ii)                                          &
-           + diipbx*grad(1,ii)+ diipby*grad(2,ii) + diipbz*grad(3,ii) &
-           + ro0*( gx*(cdgfbo(1,ifac)-xxp0)                           &
-           + gy*(cdgfbo(2,ifac)-xyp0)                                 &
-           + gz*(cdgfbo(3,ifac)-xzp0))                                &
-           + p0 - pred0
-    enddo
-  else
-    do ifac = 1, nfabor
-      ii = ifabor(ifac)
-      pripb(ifac) = cvara_pr(ii)                                    &
-           + (cdgfbo(1,ifac)-xyzcen(1,ii))*grad(1,ii)           &
-           + (cdgfbo(2,ifac)-xyzcen(2,ii))*grad(2,ii)           &
-           + (cdgfbo(3,ifac)-xyzcen(3,ii))*grad(3,ii)           &
-           + ro0*(  gx*(cdgfbo(1,ifac)-xxp0)                    &
-           + gy*(cdgfbo(2,ifac)-xyp0)                           &
-           + gz*(cdgfbo(3,ifac)-xzp0))                          &
-           + p0 - pred0
-    enddo
-  endif
-
-  ! Free memory
-  deallocate(grad)
 
 endif
 
@@ -729,26 +697,22 @@ if (itbslb.gt.0) then
     xyzref(1) = cdgfbo(1,ifrslb)
     xyzref(2) = cdgfbo(2,ifrslb)
     xyzref(3) = cdgfbo(3,ifrslb)
-    xyzref(4) = pripb(ifrslb)
     if (iphydr.eq.1.or.iifren.eq.1) isostd(nfabor+1) = ifrslb
   endif
 
   ! Broadcast PI' and pressure reference
   ! from irangd to all other ranks.
   if (irangp.ge.0) then
-    inb = 4
+    inb = 3
     call parbcr(irangd, inb, xyzref)
   endif
 
-  pipb = xyzref(4)
-  xyzref(4) = 0.d0
-
   ! If the user has not specified anything, we set ixyzp0 to 2 so as
   ! to update the reference point.
 
   if (ixyzp0.eq.-1) ixyzp0 = 2
 
-elseif (ixyzp0.lt.0) then
+elseif (ixyzp0.lt.0.and.ntcabs.eq.ntpabs+1) then
 
   ! If there are no outlet faces, we search for possible Dirichlets
   ! specified by the user so as to locate the reference point.
@@ -760,9 +724,9 @@ elseif (ixyzp0.lt.0) then
   ifadir = -1
   do ifac = 1, nfabor
     if (icodcl(ifac,ipr).eq.1) then
-      d0 =   (cdgfbo(1,ifac)-xxp0)**2  &
-           + (cdgfbo(2,ifac)-xyp0)**2  &
-           + (cdgfbo(3,ifac)-xzp0)**2
+      d0 =   (cdgfbo(1,ifac)-xyzp0(1))**2  &
+           + (cdgfbo(2,ifac)-xyzp0(2))**2  &
+           + (cdgfbo(3,ifac)-xyzp0(3))**2
       if (d0.lt.d0min) then
         ifadir = ifac
         d0min = d0
@@ -794,7 +758,6 @@ elseif (ixyzp0.lt.0) then
 
 endif
 
-
 !   Si le point de reference n'a pas ete specifie par l'utilisateur
 !   on le change et on decale alors COEFU s'il y a des sorties.
 !   La pression totale est aussi decalee (c'est a priori
@@ -802,27 +765,25 @@ endif
 
 if (ixyzp0.eq.2) then
   ixyzp0 = 1
-  xxp0 = xyzref(1) - xyzp0(1)
-  xyp0 = xyzref(2) - xyzp0(2)
-  xzp0 = xyzref(3) - xyzp0(3)
+  if (ippmod(icompf).lt.0.and.ippmod(idarcy).lt.0) then
+    call field_get_val_s(iprtot, cpro_prtot)
+    do iel = 1, ncelet
+      cpro_prtot(iel) = cpro_prtot(iel) - ro0*( gx*(xyzref(1) - xyzp0(1)) &
+                                              + gy*(xyzref(2) - xyzp0(2)) &
+                                              + gz*(xyzref(3) - xyzp0(3)))
+    enddo
+  endif
+
   xyzp0(1) = xyzref(1)
   xyzp0(2) = xyzref(2)
   xyzp0(3) = xyzref(3)
-  if (ippmod(icompf).lt.0.and.ippmod(idarcy).lt.0) then
-    call field_get_val_s(iprpfl(iprtot), cpro_prtot)
-    do iel = 1, ncelet
-      cpro_prtot(iel) = cpro_prtot(iel) - ro0*( gx*xxp0 + gy*xyp0 + gz*xzp0 )
-    enddo
-  endif
+
   if (itbslb.gt.0) then
-    write(nfecra,8000)xxp0,xyp0,xzp0
-    do ifac = 1, nfabor
-      pripb(ifac) = pripb(ifac) - ro0*( gx*xxp0 + gy*xyp0 + gz*xzp0 )
-    enddo
-    pipb = pipb - ro0*( gx*xxp0 + gy*xyp0 + gz*xzp0 )
+    write(nfecra,8000) xyzp0(1), xyzp0(2), xyzp0(3)
   else
-    write(nfecra,8001)xxp0,xyp0,xzp0
+    write(nfecra,8001) xyzp0(1), xyzp0(2), xyzp0(3)
   endif
+
 elseif (ixyzp0.eq.-1) then
   !     Il n'y a pas de sorties ni de Dirichlet et l'utilisateur n'a
   !     rien specifie -> on met IXYZP0 a 0 pour ne plus y toucher, tout
@@ -830,19 +791,45 @@ elseif (ixyzp0.eq.-1) then
   ixyzp0 = 0
 endif
 
-!     La pression totale doit etre recalee en Xref a la valeur
-!     Po + rho_0*g.(Xref-X0)
 if (itbslb.gt.0) then
-  xxp0 = xyzp0(1)
-  xyp0 = xyzp0(2)
-  xzp0 = xyzp0(3)
-  pref = p0                                            &
-       + ro0*( gx*(xyzref(1)-xxp0)                     &
-       + gy*(xyzref(2)-xyp0)                           &
-       + gz*(xyzref(3)-xzp0) )                         &
-       - pipb
-endif
 
+  if (iphydr.eq.1) then
+    call field_get_val_v_by_name('volume_forces', frcxt)
+  else
+    frcxt => rvoid2
+  endif
+
+  ! Allocate a work array for the gradient calculation
+  allocate(grad(3,ncelet))
+
+  iprev = 1
+  inc = 1
+  iccocg = 1
+
+  call field_gradient_potential(ivarfl(ipr), iprev, imrgra, inc,      &
+                                iccocg, iphydr,                       &
+                                frcxt, grad)
+
+  ! Put in pripb the value at I' or F (depending on iphydr) of the
+  ! total pressure, computed from P* shifted from the ro0*g.(X-Xref)
+
+  do ifac = 1, nfabor
+    ii = ifabor(ifac)
+    pripb(ifac) = cvara_pr(ii)                                   &
+                + (cdgfbo(1,ifac)-xyzcen(1,ii))*grad(1,ii)       &
+                + (cdgfbo(2,ifac)-xyzcen(2,ii))*grad(2,ii)       &
+                + (cdgfbo(3,ifac)-xyzcen(3,ii))*grad(3,ii)
+  enddo
+
+  ! Free memory
+  deallocate(grad)
+
+  if (irangp.eq.irangd) pref = pripb(ifrslb)
+  if (irangp.ge.0) then
+    call parall_bcast_r(irangd, pref)
+  endif
+
+endif
 
 ! ---> Entree/Sortie libre
 
@@ -855,9 +842,15 @@ do ivar = 1, nvar
       ifac = itrifb(ii)
       if(icodcl(ifac,ivar).eq.0) then
         icodcl(ifac,ivar)   = 1
-        rcodcl(ifac,ivar,1) = pripb(ifac) + pref
+        rcodcl(ifac,ivar,1) = pripb(ifac) - pref
         rcodcl(ifac,ivar,2) = rinfin
         rcodcl(ifac,ivar,3) = 0.d0
+      else
+       rcodcl(ifac,ivar,1) =  rcodcl(ifac,ivar,1)                   &
+                            - ro0*( gx*(cdgfbo(1,ifac) - xyzp0(1))  &
+                                  + gy*(cdgfbo(2,ifac) - xyzp0(2))  &
+                                  + gz*(cdgfbo(3,ifac) - xyzp0(3))) &
+                            - p0
       endif
     enddo
   elseif(ivar.eq.iu.or.ivar.eq.iv.or.ivar.eq.iw) then
@@ -874,7 +867,7 @@ do ivar = 1, nvar
 enddo
 
 ! ---> Free entrance (Bernoulli relation), std free outlet
-!      (a specific treatment si performed on the increment of pressure)
+!      (a specific treatment is performed on the increment of pressure)
 
 ideb = idebty(ifrent)
 ifin = ifinty(ifrent)
@@ -884,6 +877,7 @@ do ivar = 1, nvar
     do ii = ideb, ifin
       ifac = itrifb(ii)
       iel  = ifabor(ifac)
+
       if (icodcl(ifac,ivar).eq.0) then
 
         ! If the user has given a value of boundary head loss
@@ -895,7 +889,7 @@ do ivar = 1, nvar
 
         ! Std outlet
         icodcl(ifac,ivar)   = 1
-        rcodcl(ifac,ivar,1) = pripb(ifac) + pref
+        rcodcl(ifac,ivar,1) = pripb(ifac) - pref
         rcodcl(ifac,ivar,2) = rinfin
         rcodcl(ifac,ivar,3) = 0.d0
 
@@ -951,37 +945,53 @@ enddo
 deallocate(pripb)
 
 
-! 6.3 SYMETRIE
-! =============
+! Symmetry
+! =========
 
-! ---> Les vecteurs et tenseurs ont un traitement particulier
-!        le reste Neumann sera traite plus tard
+! ---> Vectors and tensors have a special treatment.
+!      Scalars have an homogeneous Neumann.
 
 ideb = idebty(isymet)
 ifin = ifinty(isymet)
 
-do ivar = 1, nvar
-  if (ivar.eq.iu.or.ivar.eq.iv.or.ivar.eq.iw                     &
-     .or.(itytur.eq.3                                            &
-         .and.(ivar.eq.ir11.or.ivar.eq.ir22.or.ivar.eq.ir33.or.  &
-               ivar.eq.ir12.or.ivar.eq.ir13.or.ivar.eq.ir23))) then
+do f_id = 0, nfld - 1
+
+  call field_get_type(f_id, f_type)
+
+  ! Is the field of type FIELD_VARIABLE?
+  if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
+    call field_get_dim (f_id, f_dim)
+    call field_get_key_int(f_id, keyvar, ivar)
+
+    ! Loop over faces
     do ii = ideb, ifin
       ifac = itrifb(ii)
-      if(icodcl(ifac,ivar).eq.0) then
+      ! Special treatment for uncoupled version of Rij models,
+      ! will be removed with the coupled solver
+      if (icodcl(ifac,ivar).eq.0.and.irijco.eq.0.and.        &
+          (ivar.eq.ir11.or.ivar.eq.ir22.or.ivar.eq.ir33.or.  &
+           ivar.eq.ir12.or.ivar.eq.ir13.or.ivar.eq.ir23)) then
         icodcl(ifac,ivar)   = 4
-        !         rcodcl(ifac,ivar,1) = Modifie eventuellement par l'ALE
+        rcodcl(ifac,ivar,1) = 0.d0
         rcodcl(ifac,ivar,2) = rinfin
         rcodcl(ifac,ivar,3) = 0.d0
       endif
-    enddo
-  elseif(ivar.eq.ipr) then
-    do ii = ideb, ifin
-      ifac = itrifb(ii)
-      if(icodcl(ifac,ivar).eq.0) then
+
+      ! Homogeneous Neumann on scalars
+      if (f_dim.eq.1.and.icodcl(ifac,ivar).eq.0) then
         icodcl(ifac,ivar)   = 3
         rcodcl(ifac,ivar,1) = 0.d0
         rcodcl(ifac,ivar,2) = rinfin
         rcodcl(ifac,ivar,3) = 0.d0
+
+      ! Symmetry BC if nothing is set by the user on vector and tensors
+      else if (icodcl(ifac,ivar).eq.0) then
+        do i_dim = 0, f_dim - 1
+          icodcl(ifac,ivar + i_dim)    = 4
+          rcodcl(ifac,ivar + i_dim, 1) = 0.d0
+          rcodcl(ifac,ivar + i_dim, 2) = rinfin
+          rcodcl(ifac,ivar + i_dim, 3) = 0.d0
+        enddo
       endif
     enddo
   endif
@@ -1052,10 +1062,17 @@ enddo
 ! Turbulent fluxes: 0 Dirichlet if Dirichlet on the scalar
 do iscal = 1, nscal
   if (ityturt(iscal).eq.3) then
+    ! Name of the scalar ivar
+    call field_get_name(ivarfl(isca(iscal)), fname)
+
+    ! Index of the corresponding turbulent flux
+    call field_get_id(trim(fname)//'_turbulent_flux', f_id)
+
+
     ! Set pointer values of turbulent fluxes in icodcl
-    iut = nvar + 3*(ifltur(iscal) - 1) + 1
-    ivt = nvar + 3*(ifltur(iscal) - 1) + 2
-    iwt = nvar + 3*(ifltur(iscal) - 1) + 3
+    call field_get_key_int(f_id, keyvar, iut)
+    ivt = iut + 1
+    iwt = iut + 2
 
     do ii = ideb, ifin
       ifac = itrifb(ii)
@@ -1066,6 +1083,13 @@ do iscal = 1, nscal
         rcodcl(ifac,iut,1) = 0.d0
         rcodcl(ifac,ivt,1) = 0.d0
         rcodcl(ifac,iwt,1) = 0.d0
+      else if (icodcl(ifac, isca(iscal)).eq.5.and.icodcl(ifac, iut).eq.0) then
+        icodcl(ifac,iut) = 5
+        icodcl(ifac,ivt) = 5
+        icodcl(ifac,iwt) = 5
+        rcodcl(ifac,iut,1) = 0.d0
+        rcodcl(ifac,ivt,1) = 0.d0
+        rcodcl(ifac,iwt,1) = 0.d0
       endif
     enddo
   endif
@@ -1131,6 +1155,42 @@ do ivar = 1, nvar
   endif
 enddo
 
+! Turbulent fluxes: 0 Dirichlet if Dirichlet on the scalar
+do iscal = 1, nscal
+  if (ityturt(iscal).eq.3) then
+    ! Name of the scalar ivar
+    call field_get_name(ivarfl(isca(iscal)), fname)
+
+    ! Index of the corresponding turbulent flux
+    call field_get_id(trim(fname)//'_turbulent_flux', f_id)
+
+
+    ! Set pointer values of turbulent fluxes in icodcl
+    call field_get_key_int(f_id, keyvar, iut)
+    ivt = iut + 1
+    iwt = iut + 2
+
+    do ii = ideb, ifin
+      ifac = itrifb(ii)
+      if (icodcl(ifac, isca(iscal)).eq.1.and.icodcl(ifac, iut).eq.0) then
+        icodcl(ifac,iut) = 1
+        icodcl(ifac,ivt) = 1
+        icodcl(ifac,iwt) = 1
+        rcodcl(ifac,iut,1) = 0.d0
+        rcodcl(ifac,ivt,1) = 0.d0
+        rcodcl(ifac,iwt,1) = 0.d0
+      else if (icodcl(ifac, isca(iscal)).eq.6.and.icodcl(ifac, iut).eq.0) then
+        icodcl(ifac,iut) = 6
+        icodcl(ifac,ivt) = 6
+        icodcl(ifac,iwt) = 6
+        rcodcl(ifac,iut,1) = 0.d0
+        rcodcl(ifac,ivt,1) = 0.d0
+        rcodcl(ifac,iwt,1) = 0.d0
+      endif
+    enddo
+  endif
+enddo
+
 !===============================================================================
 ! 6.bis  CONVERSION EN RCODCL ICODCL
 !   (SI CE DERNIER N'A PAS DEJA ETE RENSEIGNE PAR L'UTILISATEUR)
@@ -1151,38 +1211,52 @@ ideb = idebty(ientre)
 ifin = ifinty(ientre)
 
 iok = 0
+iivar = 0
 do ivar = 1, nvar
   do ii = ideb, ifin
     ifac = itrifb(ii)
     if(icodcl(ifac,ivar).eq.0) then
 
-      if (ivar.eq.iu.or.ivar.eq.iv.or.ivar.eq.iw)      &
-           then
+      call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+
+      ! For not convected variables, if nothing is defined: homogeneous Neumann
+      if (vcopt%iconv .eq. 0 .and. rcodcl(ifac,ivar,1).gt.rinfin*0.5d0) then
+        icodcl(ifac,ivar)   = 3
+        rcodcl(ifac,ivar,1) = 0.d0
+        rcodcl(ifac,ivar,2) = rinfin
+        rcodcl(ifac,ivar,3) = 0.d0
+      else if (ivar.eq.iu.or.ivar.eq.iv.or.ivar.eq.iw) then
         if (rcodcl(ifac,ivar,1).gt.rinfin*0.5d0) then
           itypfb(ifac) = - abs(itypfb(ifac))
-          if (iok.eq.0.or.iok.eq.2) iok = iok + 1
+          if (iok.eq.0.or.iok.eq.2) then
+            iok = iok + 1
+          endif
         else
           icodcl(ifac,ivar) = 1
-          !           rcodcl(ifac,ivar,1) = Utilisateur
+          ! rcodcl(ifac,ivar,1) = given by the user
           rcodcl(ifac,ivar,2) = rinfin
           rcodcl(ifac,ivar,3) = 0.d0
         endif
 
-      elseif (rcodcl(ifac,ivar,1).gt.rinfin*0.5d0) then
+      else if (rcodcl(ifac,ivar,1).gt.rinfin*0.5d0) then
 
         flumbf = bmasfl(ifac)
-        if( flumbf.ge.-epzero) then
+        ! Outgoing flux
+        if (flumbf.ge.-epzero) then
           icodcl(ifac,ivar)   = 3
           rcodcl(ifac,ivar,1) = 0.d0
           rcodcl(ifac,ivar,2) = rinfin
           rcodcl(ifac,ivar,3) = 0.d0
         else
           itypfb(ifac) = - abs(itypfb(ifac))
-          if (iok.lt.2) iok = iok + 2
+          if (iok.lt.2) then
+            iok = iok + 2
+            iivar = max(iivar, ivar)
+          endif
         endif
       else
         icodcl(ifac,ivar) = 1
-        !         rcodcl(ifac,ivar,1) = Utilisateur
+        ! rcodcl(ifac,ivar,1) = given by the user
         rcodcl(ifac,ivar,2) = rinfin
         rcodcl(ifac,ivar,3) = 0.d0
       endif
@@ -1191,10 +1265,16 @@ do ivar = 1, nvar
   enddo
 enddo
 
-if (irangp.ge.0) call parcmx(iok)
+if (irangp.ge.0) then
+  call parcmx(iok)
+  call parcmx(iivar)
+endif
 if (iok.gt.0) then
   if (iok.eq.1 .or. iok.eq.3) write(nfecra,6060)
-  if (iok.eq.2 .or. iok.eq.3) write(nfecra,6070)
+  if (iok.eq.2 .or. iok.eq.3) then
+    call field_get_name(ivarfl(iivar), fname)
+    write(nfecra,6070) trim(fname)
+  endif
   call boundary_conditions_error(itypfb)
 endif
 
@@ -1210,12 +1290,22 @@ ideb = idebty(i_convective_inlet)
 ifin = ifinty(i_convective_inlet)
 
 iok = 0
+iivar = 0
 do ivar = 1, nvar
   do ii = ideb, ifin
     ifac = itrifb(ii)
     if(icodcl(ifac,ivar).eq.0) then
 
-      if (ivar.eq.iu.or.ivar.eq.iv.or.ivar.eq.iw) then
+      call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+
+      ! For not convected variables, if nothing is defined: homogeneous Neumann
+      if (vcopt%iconv .eq. 0 .and. rcodcl(ifac,ivar,1).gt.rinfin*0.5d0) then
+        icodcl(ifac,ivar)   = 3
+        rcodcl(ifac,ivar,1) = 0.d0
+        rcodcl(ifac,ivar,2) = rinfin
+        rcodcl(ifac,ivar,3) = 0.d0
+
+      else if (ivar.eq.iu.or.ivar.eq.iv.or.ivar.eq.iw) then
         if (rcodcl(ifac,ivar,1).gt.rinfin*0.5d0) then
           itypfb(ifac) = - abs(itypfb(ifac))
           if (iok.eq.0.or.iok.eq.2) iok = iok + 1
@@ -1226,17 +1316,21 @@ do ivar = 1, nvar
           rcodcl(ifac,ivar,3) = 0.d0
         endif
 
-      elseif (rcodcl(ifac,ivar,1).gt.rinfin*0.5d0) then
+      else if (rcodcl(ifac,ivar,1).gt.rinfin*0.5d0) then
 
         flumbf = bmasfl(ifac)
-        if( flumbf.ge.-epzero) then
+        ! Outgoing flux
+        if (flumbf.ge.-epzero) then
           icodcl(ifac,ivar)   = 3
           rcodcl(ifac,ivar,1) = 0.d0
           rcodcl(ifac,ivar,2) = rinfin
           rcodcl(ifac,ivar,3) = 0.d0
         else
           itypfb(ifac) = - abs(itypfb(ifac))
-          if (iok.lt.2) iok = iok + 2
+          if (iok.lt.2) then
+            iok = iok + 2
+            iivar = max(iivar, ivar)
+          endif
         endif
       else
         icodcl(ifac,ivar) = 13
@@ -1249,10 +1343,16 @@ do ivar = 1, nvar
   enddo
 enddo
 
-if (irangp.ge.0) call parcmx(iok)
+if (irangp.ge.0) then
+  call parcmx(iok)
+  call parcmx(iivar)
+endif
 if (iok.gt.0) then
   if (iok.eq.1 .or. iok.eq.3) write(nfecra,6060)
-  if (iok.eq.2 .or. iok.eq.3) write(nfecra,6070)
+  if (iok.eq.2 .or. iok.eq.3) then
+    call field_get_name(ivarfl(iivar), fname)
+    write(nfecra,6070) trim(fname)
+  endif
   call boundary_conditions_error(itypfb)
 endif
 
@@ -1324,14 +1424,14 @@ do ivar = 1, nvar
 enddo
 
 ! 6.2.c Free surface
-! ===================================
+! ==================
 
 ! ---> Free surface
 
 ideb = idebty(ifresf)
 ifin = ifinty(ifresf)
 
-do ivar = 1, nvarcl
+do ivar = 1, nvar
   do ii = ideb, ifin
     ifac = itrifb(ii)
     if(icodcl(ifac,ivar).eq.0) then
@@ -1351,28 +1451,6 @@ do ivar = 1, nvarcl
   enddo
 enddo
 
-! 6.3 SYMETRIE bis
-! =============
-
-! ---> Les vecteurs et tenseurs ont un traitement particulier
-!        traite plus haut
-!        le reste Neumann
-
-ideb = idebty(isymet)
-ifin = ifinty(isymet)
-
-do ivar = 1, nvarcl
-  do ii = ideb, ifin
-    ifac = itrifb(ii)
-    if(icodcl(ifac,ivar).eq.0) then
-      icodcl(ifac,ivar)   = 3
-      rcodcl(ifac,ivar,1) = 0.d0
-      rcodcl(ifac,ivar,2) = rinfin
-      rcodcl(ifac,ivar,3) = 0.d0
-    endif
-  enddo
-enddo
-
 ! 6.4 PAROI LISSE bis
 ! ===============
 
@@ -1383,16 +1461,25 @@ enddo
 ideb = idebty(iparoi)
 ifin = ifinty(iparoi)
 
-do ivar = 1, nvarcl
-  do ii = ideb, ifin
-    ifac = itrifb(ii)
-    if(icodcl(ifac,ivar).eq.0) then
-      icodcl(ifac,ivar)   = 3
-      rcodcl(ifac,ivar,1) = 0.d0
-      rcodcl(ifac,ivar,2) = rinfin
-      rcodcl(ifac,ivar,3) = 0.d0
-    endif
-  enddo
+do f_id = 0, nfld - 1
+  call field_get_type(f_id, f_type)
+  ! Is the field of type FIELD_VARIABLE?
+  if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
+    call field_get_dim (f_id, f_dim)
+    call field_get_key_int(f_id, keyvar, ivar)
+
+    do ii = ideb, ifin
+      ifac = itrifb(ii)
+      if(icodcl(ifac,ivar).eq.0) then
+        do i_dim = 0, f_dim-1
+          icodcl(ifac,ivar+i_dim) = 3
+          rcodcl(ifac,ivar+i_dim,1) = 0.d0
+          rcodcl(ifac,ivar+i_dim,2) = rinfin
+          rcodcl(ifac,ivar+i_dim,3) = 0.d0
+        enddo
+      endif
+    enddo
+  endif
 enddo
 
 ! 6.5 PAROI RUGUEUSE bis
@@ -1405,16 +1492,47 @@ enddo
 ideb = idebty(iparug)
 ifin = ifinty(iparug)
 
-do ivar = 1, nvarcl
-  do ii = ideb, ifin
-    ifac = itrifb(ii)
-    if(icodcl(ifac,ivar).eq.0) then
-      icodcl(ifac,ivar)   = 3
-      rcodcl(ifac,ivar,1) = 0.d0
-      rcodcl(ifac,ivar,2) = rinfin
-      rcodcl(ifac,ivar,3) = 0.d0
+do f_id = 0, nfld - 1
+  call field_get_type(f_id, f_type)
+  ! Is the field of type FIELD_VARIABLE?
+  if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
+    call field_get_dim (f_id, f_dim)
+    call field_get_key_int(f_id, keyvar, ivar)
+
+    do ii = ideb, ifin
+      ifac = itrifb(ii)
+      if(icodcl(ifac,ivar).eq.0) then
+        do i_dim = 0, f_dim-1
+          icodcl(ifac,ivar+i_dim) = 3
+          rcodcl(ifac,ivar+i_dim,1) = 0.d0
+          rcodcl(ifac,ivar+i_dim,2) = rinfin
+          rcodcl(ifac,ivar+i_dim,3) = 0.d0
+        enddo
+      endif
+    enddo
+  endif
+enddo
+
+! ensure that for all variables of dimension higher than 1 and which components
+! are coupled, icodcl is the same for all the components
+call field_get_key_id('coupled', keycpl)
+
+do f_id = 0, nfld - 1
+  call field_get_type(f_id, f_type)
+  ! Is the field of type FIELD_VARIABLE?
+  if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
+    call field_get_dim (f_id, f_dim)
+    call field_get_key_int(f_id, keycpl, icpl)
+
+    if (f_dim.gt.1.and.icpl.eq.1) then
+      call field_get_key_int(f_id, keyvar, ivar)
+      do ifac = 1, nfabor
+        do i_dim = 1, f_dim-1
+          icodcl(ifac,ivar+i_dim) = icodcl(ifac,ivar)
+        enddo
+      enddo
     endif
-  enddo
+  endif
 enddo
 
 !===============================================================================
@@ -1426,7 +1544,8 @@ enddo
 
 do ivar = 1, nvar
   ndircl(ivar) = 0
-  if ( istat(ivar).gt.0 .or. idircl(ivar).eq.0 ) ndircl(ivar) = 1
+  call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+  if ( vcopt%istat.gt.0 .or. idircl(ivar).eq.0 ) ndircl(ivar) = 1
 enddo
 
 do ivar = 1, nvar
@@ -1451,22 +1570,16 @@ enddo
 
 !===============================================================================
 
-iwaru = -1
-iwaru = max(iwarni(iu),iwaru)
-if (irangp.ge.0) call parcmx(iwaru)
+call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt)
 
-if(iwaru.ge.1 .or. mod(ntcabs,ntlist).eq.0                        &
+if(vcopt%iwarni.ge.1 .or. mod(ntcabs,ntlist).eq.0                        &
        .or.(ntcabs.le.ntpabs+2).or.(ntcabs.ge.ntmabs-1)) then
   write(nfecra,7010)
 endif
 
-iwrnp = iwarni(iu)
-if (irangp.ge.0) call parcmx (iwrnp)
-!==========
-
 !     On ecrit le flux de masse si IWARNI>0, a la periodicite NTLIST
 !     et au deux premiers et deux derniers pas de temps.
-if(iwrnp.ge.1 .or. mod(ntcabs,ntlist).eq.0                      &
+if(vcopt%iwarni.ge.1 .or. mod(ntcabs,ntlist).eq.0                      &
      .or.(ntcabs.le.ntpabs+2).or.(ntcabs.ge.ntmabs-1)) then
 
   do ii = 1, ntypmx
@@ -1875,20 +1988,20 @@ endif
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/,&
 '@ @@ ATTENTION : ARRET LORS DE LA VERIFICATION DES COND. LIM.',/,&
-'@    =========                                               ',/,&
-'@    CONDITIONS AUX LIMITES INCORRECTES OU INCOMPLETES       ',/,&
-'@                                                            ',/,&
-'@    Au moins une face de bord declaree en entree            ',/,&
-'@      (ou sortie) a vitesse imposee avec un flux rentrant   ',/,&
-'@      pour laquelle la valeur d''une variable n''a pas ete  ',/,&
-'@      specifiee (condition de Dirichlet).                   ',/,&
-'@    Le calcul ne sera pas execute                           ',/,&
-'@                                                            ',/,&
-'@    Verifier les conditions aux limites dans l''Interface   ',/,&
-'@    ou dans le sous-programme utilisateur correspondant.    ',/,&
-'@                                                            ',/,&
+'@    ========='                                               ,/,&
+'@    CONDITIONS AUX LIMITES INCORRECTES OU INCOMPLETES'       ,/,&
+'@'                                                            ,/,&
+'@    Au moins une face de bord declaree en entree'            ,/,&
+'@      (ou sortie) a vitesse imposee avec un flux rentrant'   ,/,&
+'@      pour laquelle la valeur de', a,' n''a pas ete'         ,/,&
+'@      specifiee (condition de Dirichlet).'                   ,/,&
+'@    Le calcul ne sera pas execute'                           ,/,&
+'@'                                                            ,/,&
+'@    Verifier les conditions aux limites dans l''Interface'   ,/,&
+'@    ou dans le sous-programme utilisateur correspondant.'    ,/,&
+'@'                                                            ,/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
+'@'                                                            ,/)
 
 
  7010 format ( /,/,                                               &
@@ -2030,7 +2143,7 @@ endif
 '@'                                                            ,/,&
 '@    At least one boundary face declared as inlet (or'        ,/,&
 '@      outlet) with prescribed velocity with an entering'     ,/,&
-'@      flow for which the value of a variable has not been'   ,/,&
+'@      flow for which the value of ',a,' has not been'        ,/,&
 '@      specified (Dirichlet condition).'                      ,/,&
 '@    The calculation will not be run.                        ',/,&
 '@'                                                            ,/,&
