@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -57,6 +57,7 @@ use ppincl
 use radiat
 use ihmpre
 use mesh
+use post
 use field
 use cs_c_bindings
 
@@ -68,17 +69,17 @@ implicit none
 
 ! Local variables
 
-integer       iscal , id, ityloc, itycat, ifcvsl
+integer       iscal , id, ityloc, itycat, ifcvsl, pflag
 integer       ii
-integer       iok   , ippok
+integer       iok
 integer       ivisph, iest
-integer       nprmax
-integer       idttur
 
 double precision gravn2
 
 character(len=80) :: f_name, f_label, s_label, s_name
 
+type(var_cal_opt) :: vcopt
+
 !===============================================================================
 ! Interfaces
 !===============================================================================
@@ -109,9 +110,6 @@ endif
 ! Initialization
 !===============================================================================
 
-! Initialize variables to avoid compiler warnings
-ippok = 0
-
 ! Determine itycor now that irccor is known (iturb/itytur known much earlier)
 ! type of rotation/curvature correction for turbulent viscosity models
 if (irccor.eq.1.and.(itytur.eq.2.or.itytur.eq.5)) then
@@ -120,32 +118,42 @@ else if (irccor.eq.1.and.(iturb.eq.60.or.iturb.eq.70)) then
   itycor = 2
 endif
 
+pflag = POST_ON_LOCATION + POST_MONITOR
+
 !===============================================================================
 ! Additional physical properties
 !===============================================================================
 
 ! CP when variable
-if (icp.ne.0) then
-  call add_property_field('specific_heat', 'Specific Heat', icp)
+if (icp.ge.0) then
+  call add_property_field_1d('specific_heat', 'Specific Heat', icp)
+
+  call field_set_key_int(icp, keyvis, 1)
+  call field_set_key_int(icp, keylog, 1)
 endif
 
-! Density at the second previous time step for cavitation algorithm
-if (icavit.ge.0.or.idilat.gt.1) then
-  call add_property_field('density_old', 'Density Old', iromaa)
-  icroaa = iprpfl(iromaa)
+! Density at the second previous time step for VOF algorithm
+! or dilatable algorithm
+if (ivofmt.ge.0.or.idilat.gt.1) then
+  call add_property_field_1d('density_old', 'Density Old', iromaa)
+  icroaa = iromaa
+  call hide_property(iromaa)
 endif
 
 ! ALE mesh viscosity
 if (iale.eq.1) then
-  call add_property_field('mesh_viscosity_1', 'Mesh ViscX', ivisma(1))
-  ! si la viscosite est isotrope, les trois composantes pointent
-  !  au meme endroit
   if (iortvm.eq.0) then
-    ivisma(2) = ivisma(1)
-    ivisma(3) = ivisma(1)
+    call add_property_field('mesh_viscosity', 'Mesh Visc', 1, .false., ivisma)
   else
-    call add_property_field('mesh_viscosity_2', 'Mesh ViscY', ivisma(2))
-    call add_property_field('mesh_viscosity_3', 'Mesh ViscZ', ivisma(3))
+    call add_property_field('mesh_viscosity', 'Mesh Visc', 3, .false., ivisma)
+  endif
+endif
+
+if (irccor.eq.1) then
+  if (idtvar.ge.0) then
+     call add_property_field('strain_rate_tensor', 'Strain Rate Tensor', 6, &
+                             .false., istraio)
+     call hide_property(istraio)
   endif
 endif
 
@@ -175,7 +183,7 @@ iok = 0
 !  Pression hydrostatique
 if (iphydr.eq.0.or.iphydr.eq.2) then
   icalhy = 0
-else if (iphydr.eq.1) then
+else if (iphydr.eq.1.and.icalhy.eq.-1) then
   gravn2 = gx**2+gy**2+gz**2
   if (gravn2.lt.epzero**2) then
     icalhy = 0
@@ -261,8 +269,6 @@ else if (itytur.eq.2.or.iturb.eq.50.or.iturb.ne.60) then
   iok = iok + 1
 endif
 
-idttur = 0
-
 do iscal = 1, nscal
   ! Termes sources Scalaires,
   if (isso2t(iscal).eq.-999) then
@@ -290,16 +296,10 @@ do iscal = 1, nscal
 
   if (iscal.eq.iscalt) then
     if (iturt(iscalt).gt.0.and.irovar.eq.1) then
-      call add_property_field('thermal_expansion', 'Beta', ibeta)
+      call add_property_field_1d('thermal_expansion', 'Beta', ibeta)
     endif
   endif
 
-  ! Index of the turbulent flux
-  if (ityturt(iscal).eq.3) then
-    idttur = idttur + 1
-    ifltur(iscal) = idttur
-  endif
-
 enddo
 
 ! Pression hydrostatique
@@ -405,10 +405,6 @@ if (iok.gt.0) then
   call csexit(1)
 endif
 
-! Numeros de propriete
-
-nprmax = nproce
-
 ! Source term for weakly compressible algorithm (semi analytic scheme)
 if (idilat.ge.4) then
   do iscal = 1, nscal
@@ -416,56 +412,46 @@ if (idilat.ge.4) then
     call field_get_name(id, s_name)
     call field_get_label(id, s_label)
     f_name  = trim(s_name) // '_dila_st'
-    call add_property_field(f_name, '', iustdy(iscal))
-    id = iprpfl(iustdy(iscal))
-    call field_set_key_int(id, keyipp, -1)
+    call add_property_field_1d(f_name, '', iustdy(iscal))
+    id = iustdy(iscal)
     call field_set_key_int(id, keyvis, 0)
   enddo
   itsrho = nscal + 1
-  call add_property_field('dila_st', '', iustdy(itsrho))
-  id = iprpfl(iustdy(iscal))
-  call field_set_key_int(id, keyipp, -1)
+  call add_property_field_1d('dila_st', '', iustdy(itsrho))
+  id = iustdy(iscal)
   call field_set_key_int(id, keyvis, 0)
 endif
 
-! The density at the previous time step is required if idilat>1 or icavit>=0 or
-! if we perform a hydrostatic pressure correction (icalhy=1)
-if (iroext.gt.0.or.icalhy.eq.1.or.idilat.gt.1.or.icavit.ge.0.or.ipthrm.eq.1 &
-     .or.ippmod(icompf).ge.0) then
-  nproce = nproce + 1
-  iroma  = nproce
-  call field_set_n_previous(iprpfl(irom), 1)
+! The density at the previous time step is required if idilat>1 or ivofmt>=0
+! or if we perform a hydrostatic pressure correction (icalhy=1)
+if (iroext.gt.0.or.icalhy.eq.1.or.idilat.gt.1.or.ivofmt.ge.0 &
+    .or.ipthrm.eq.1.or.ippmod(icompf).ge.0) then
+  call field_set_n_previous(irom, 1)
   if (iroext.gt.0.or.idilat.gt.1) then
     call field_set_n_previous(ibrom, 1)
   endif
 endif
 ! Dans le cas d'une extrapolation de la viscosite totale
 if (iviext.gt.0) then
-  nproce = nproce + 1
-  ivisla = nproce
-  call field_set_n_previous(iprpfl(iviscl), 1)
-  nproce = nproce + 1
-  ivista = nproce
-  call field_set_n_previous(iprpfl(ivisct), 1)
+  call field_set_n_previous(iviscl, 1)
+  call field_set_n_previous(ivisct, 1)
 endif
 
 ! CP s'il est variable
-if (icp.ne.0) then
+if (icp.ge.0) then
   if (icpext.gt.0) then
-    nproce = nproce + 1
-    icpa   = nproce
-    call field_set_n_previous(iprpfl(icp), 1)
+    call field_set_n_previous(icp, 1)
   endif
 endif
 
 ! On a besoin d'un tableau pour les termes sources de Navier Stokes
 !  a extrapoler. Ce tableau est NDIM dans le cas general et NDIM+1
 !  si on extrapole aussi les termes sources de l equation sur le taux
-!  de vide pour le modele de cavitation.
+!  de vide pour l'algo. VOF.
 if (isno2t.gt.0) then
   call add_source_term_prev_field(ivarfl(iu))
-  if (icavit.ge.0) then
-    call add_source_term_prev_field(ivarfl(ivoidf))
+  if (ivofmt.ge.0) then
+    call add_source_term_prev_field(ivarfl(ivolf2))
   endif
 endif
 
@@ -510,15 +496,24 @@ if (nscal.ge.1) then
       call add_source_term_prev_field(ivarfl(isca(ii)))
     endif
     ! Only usefull for Min/Max limiter
-    if (isstpc(isca(ii)).eq.2) then
+    call field_get_key_struct_var_cal_opt(ivarfl(isca(ii)), vcopt)
+    if (vcopt%isstpc.eq.2) then
       call add_source_term_field(ivarfl(isca(ii)))
     endif
+    ! Diffusivity
     call field_get_key_int (ivarfl(isca(ii)), kivisl, ifcvsl)
     if (ifcvsl.ge.0.and.iscavr(ii).le.0) then
       if (ivsext(ii).gt.0) then
         call field_set_n_previous(ifcvsl, 1)
       endif
     endif
+    ! Density
+    call field_get_key_int (ivarfl(isca(ii)), kromsl, ifcvsl)
+    if (ifcvsl.ge.0.and.iscavr(ii).le.0) then
+      if (ivsext(ii).gt.0) then
+        call field_set_n_previous(ifcvsl, 1)
+      endif
+    endif
   enddo
 endif
 
@@ -530,7 +525,7 @@ if (iporos.ge.1) then
   f_name = 'porosity'
   call field_create(f_name, itycat, ityloc, 1, .false., ipori)
   call field_set_key_int(ipori, keylog, 1)
-  call field_set_key_int(ipori, keyvis, 1)
+  call field_set_key_int(ipori, keyvis, pflag)
   if (iporos.eq.2) then
     f_name = 'tensorial_porosity'
     call field_create(f_name, itycat, ityloc, 6, .false., iporf)
@@ -551,9 +546,8 @@ call field_set_key_str(id, keylbl, 'Local Time Step')
 if (idtvar.gt.0) then
   if (idtvar.eq.2) then
     call field_set_key_int(id, keylog, 1)
-    call field_set_key_int(id, keyvis, 1)
+    call field_set_key_int(id, keyvis, pflag)
   endif
-  ippdt = field_post_id(id)
 endif
 
 itycat = FIELD_INTENSIVE
@@ -563,11 +557,8 @@ itycat = FIELD_INTENSIVE
 
 if (ipucou.ne.0 .or. ncpdct.gt.0) then
   call field_create('dttens', itycat, ityloc, 6, .false., idtten)
-  call field_set_key_int(idtten, keyvis, 1)
+  call field_set_key_int(idtten, keyvis, POST_ON_LOCATION)
   call field_set_key_int(idtten, keylog, 1)
-  ipptx = field_post_id(id)
-  ippty = ipptx + 1
-  ipptz = ippty + 1
 endif
 
 ! Error estimators
@@ -579,22 +570,22 @@ enddo
 if (iescal(iespre).gt.0) then
   write(f_name,  '(a14,i1)') 'est_error_pre_', iescal(iespre)
   write(f_label, '(a5,i1)') 'EsPre', iescal(iespre)
-  call add_property_field_owner(f_name, f_label, 1, .false., iestim(iespre))
+  call add_property_field(f_name, f_label, 1, .false., iestim(iespre))
 endif
 if (iescal(iesder).gt.0) then
   write(f_name,  '(a14,i1)') 'est_error_der_', iescal(iesder)
   write(f_label, '(a5,i1)') 'EsDer', iescal(iesder)
-  call add_property_field_owner(f_name, f_label, 1, .false., iestim(iesder))
+  call add_property_field(f_name, f_label, 1, .false., iestim(iesder))
 endif
 if (iescal(iescor).gt.0) then
   write(f_name,  '(a14,i1)') 'est_error_cor_', iescal(iescor)
   write(f_label, '(a5,i1)') 'EsCor', iescal(iescor)
-  call add_property_field_owner(f_name, f_label, 1, .false., iestim(iescor))
+  call add_property_field(f_name, f_label, 1, .false., iestim(iescor))
 endif
 if (iescal(iestot).gt.0) then
   write(f_name,  '(a14,i1)') 'est_error_tot_', iescal(iestot)
   write(f_label, '(a5,i1)') 'EsTot', iescal(iestot)
-  call add_property_field_owner(f_name, f_label, 1, .false., iestim(iestot))
+  call add_property_field(f_name, f_label, 1, .false., iestim(iestot))
 endif
 
 !===============================================================================
@@ -1175,4 +1166,3 @@ call field_set_key_int(f_id, kst, st_id)
 return
 
 end subroutine add_source_term_field
-
