@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -32,10 +32,11 @@
  *----------------------------------------------------------------------------*/
 
 #include "cs_base.h"
+#include "cs_cdo_connect.h"
+#include "cs_flag.h"
+#include "cs_math.h"
 #include "cs_mesh.h"
 #include "cs_mesh_quantities.h"
-#include "cs_cdo.h"
-#include "cs_cdo_connect.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -45,34 +46,40 @@ BEGIN_C_DECLS
  * Macro definitions
  *============================================================================*/
 
+/* Information useful to get simpler algo. */
+#define CS_CDO_ORTHO      (1 << 0)  // Orthogonality condition is checked
+
 /*============================================================================
  * Type definitions
  *============================================================================*/
 
-/* Type of algorithm to compute geometrical quantities */
+/* Type of algorithm used to compute the cell center */
 typedef enum {
 
-  CS_CDO_CC_MEANV,   // Cell center is computed as the mean of cell vertices
-  CS_CDO_CC_BARYC,   // Cell center is computed as the real cell barycenter
-  CS_CDO_CC_SATURNE, // Cell center is given by Code_Saturne
-  CS_CDO_CC_ORTHO,   /* Cell center is optimized to enforce orthogonality
-                        between cell-face edge and face plane */
-  CS_CDO_N_CC_ALGOS
+  /* Center is computed as the mean of cell vertices */
+  CS_CDO_QUANTITIES_MEANV_CENTER,
 
-} cs_cdo_cc_algo_t;
+  /* Center is computed as the real cell barycenter */
+  CS_CDO_QUANTITIES_BARYC_CENTER,
+
+  /* Use the cell center computed in cs_mesh_quantities.c (Default behavior) */
+  CS_CDO_QUANTITIES_SATURNE_CENTER,
+
+  CS_CDO_QUANTITIES_N_CENTER_ALGOS
+
+} cs_cdo_quantities_algo_ccenter_t;
 
 /* Structure storing information about variation of entities accros the
    mesh for a given type of entity (cell, face and edge) */
 typedef struct {
 
-  double   meas_min;  // Min. value of the entity measure (vol, surf or lenght)
-  double   meas_max;  // Max. value of the entity measure (vol, surf or lenght)
+  /* Measure is either a volume for cells, a surface for faces or a length
+     for edges */
+  double   meas_min;  // Min. value of the entity measure
+  double   meas_max;  // Max. value of the entity measure
   double   h_min;     // Estimation of the min. value of the diameter
   double   h_max;     // Estimation of the max. value of the diameter
 
-  cs_lnum_t   min_id; // Entity if related to the min. value
-  cs_lnum_t   max_id; // Entity if related to the max. value
-
 } cs_quant_info_t;
 
 /* For primal vector quantities (edge or face) */
@@ -84,55 +91,81 @@ typedef struct {
 
 } cs_quant_t;
 
-/* For dual face quantities. Add also a link to entity id related to this dual
-   quantities */
-
-typedef struct { /* TODO: remove what is the less necessary in order to
-                    save memory comsumption */
-
-  cs_lnum_t   parent_id[2];  /* parent entity id of (primal) faces f0 and f1 */
-  cs_nvec3_t  sface[2];      /* area and unit normal vector for each
-                                triangle s(e,f,c) for f in {f0, f1} */
-  double      vect[3];       /* dual face vector */
-
-} cs_dface_t;
-
 typedef struct { /* Specific mesh quantities */
 
   /* Global mesh quantities */
-  double        vol_tot;
+  double           vol_tot;
 
   /* Cell-based quantities */
-  cs_lnum_t        n_cells;
-  cs_real_t       *cell_centers;
-  cs_real_t       *cell_vol;
-  cs_quant_info_t  cell_info;
+  /* ===================== */
+
+  cs_lnum_t         n_cells;        /* Local number of cells */
+  cs_gnum_t         n_g_cells;      /* Global number of cells */
+  cs_real_t        *cell_centers;
+  cs_real_t        *cell_vol;
+  cs_flag_t        *cell_flag;      /* Flag attached to cell to associate
+                                       metadata like boundary cell or
+                                       orthogonality */
+
+  cs_quant_info_t   cell_info;
 
   /* Face-based quantities */
-  cs_lnum_t        n_i_faces;
-  cs_lnum_t        n_b_faces;
-  cs_lnum_t        n_faces;   /* n_i_faces + n_b_faces */
-  cs_quant_t      *face;      /* Face quantities */
-  cs_nvec3_t      *dedge;     /* Dual edge quantities (length and unit vector)
-                                 Scan with the c2f connectivity */
-  cs_quant_info_t  face_info;
+  /* ===================== */
+
+  cs_lnum_t         n_i_faces;      // Local number of interior faces
+  const cs_real_t  *i_face_normal;  // Shared with cs_mesh_quantities_t
+  const cs_real_t  *i_face_center;  // Shared with cs_mesh_quantities_t
+  const cs_real_t  *i_face_surf;    // Shared with cs_mesh_quantities_t
+
+  cs_lnum_t         n_b_faces;      // Local number of border faces
+  const cs_real_t  *b_face_normal;  // Shared with cs_mesh_quantities_t
+  const cs_real_t  *b_face_center;  // Shared with cs_mesh_quantities_t
+  const cs_real_t  *b_face_surf;    // Shared with cs_mesh_quantities_t
+
+  cs_lnum_t         n_faces;        /* n_i_faces + n_b_faces */
+  cs_gnum_t         n_g_faces;      /* Global number of faces */
+
+  /* cs_quant_t structure attached to a face (interior or border) is build
+     on-the-fly  cs_quant_get(flag, f_id, quant) */
+
+  cs_real_t        *dedge_vector;
+
+  /* cs_nvec3_t structure attached to a dual edge is build on-the-fly
+     Dual edge quantities (length and unit vector)
+     Scan with the c2f connectivity
+  */
+
+  cs_quant_info_t   face_info;
 
   /* Edge-based quantities */
-  cs_lnum_t        n_edges;
-  cs_quant_t      *edge;      /* Edge quantities */
-  cs_dface_t      *dface;     /* For each edge belonging to a cell, two
-                                 contributions coming from 2 triangles
-                                 s(x_cell, x_face, x_edge) for face in Face_edge
-                                 are considered.
-                                 Scan with the c2e connectivity */
-  cs_quant_info_t  edge_info;
+  /* ===================== */
+
+  cs_lnum_t         n_edges;        /* Local number of edges */
+  cs_gnum_t         n_g_edges;      /* Global number of edges */
+
+  cs_real_t        *edge_vector;    /* norm of the vector is equal to the
+                                       distance between two vertices.
+                                       unit vector is the tangential direction
+                                       attached to the edge */
+
+  /* For each edge belonging to a cell, two contributions coming from 2
+     triangles  s(x_cell, x_face, x_edge) for face in Face_edge are considered.
+     Scan with the c2e connectivity */
+
+  cs_real_t        *sface_normal;   /* 2 triangle-face normals by edge in a
+                                       cell */
+
+  cs_quant_info_t   edge_info;
 
   /* Vertex-based quantities */
-  cs_lnum_t   n_vertices;
-  double     *dcell_vol;       /* Dual volume related to each vertex.
-                                  Scan with the c2v connectivity */
-  const cs_real_t  *vtx_coord; /* Pointer to the one stored in cs_mesh_t */
+  /* ======================= */
 
+  cs_lnum_t         n_vertices;      /* Local number of vertices */
+  cs_gnum_t         n_g_vertices;    /* Global number of vertices */
+
+  cs_real_t        *dcell_vol;       /* Dual volume related to each vertex.
+                                        Scan with the c2v connectivity */
+  const cs_real_t  *vtx_coord;       /* Shared with the cs_mesh_t structure */
 
 } cs_cdo_quantities_t;
 
@@ -144,6 +177,33 @@ typedef struct { /* Specific mesh quantities */
  * Public function prototypes
  *============================================================================*/
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute the area of the triangle of base given by q (related to a
+ *         segment) with apex located at xa
+ *
+ * \param[in]  qa   pointer to a cs_quant_t structure related to a segment
+ * \param[in]  xb   coordinates of the apex to consider
+ *
+ * \return the value the area of the triangle
+ */
+/*----------------------------------------------------------------------------*/
+
+double
+cs_compute_area_from_quant(const cs_quant_t   qa,
+                           const cs_real_t   *xb);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Set the type of algorithm to use for computing the cell center
+ *
+ * \param[in]  algo     type of algorithm
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_cdo_quantities_set_algo_ccenter(cs_cdo_quantities_algo_ccenter_t   algo);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Build a cs_cdo_quantities_t structure
@@ -178,7 +238,7 @@ cs_cdo_quantities_free(cs_cdo_quantities_t   *q);
 /*!
  * \brief Summarize generic information about the cdo mesh quantities
  *
- * \param[in]  cdoq     pointer to cs_cdo_quantities_t structure
+ * \param[in]  quant   pointer to cs_cdo_quantities_t structure
  *
  */
 /*----------------------------------------------------------------------------*/
@@ -198,6 +258,89 @@ cs_cdo_quantities_summary(const cs_cdo_quantities_t  *quant);
 void
 cs_cdo_quantities_dump(const cs_cdo_quantities_t  *cdoq);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define a cs_quant_t structure for a primal face (interior or border)
+ *
+ * \param[in]  f_id     id related to the face (f_id > n_i_face -> border face)
+ * \param[in]  cdoq     pointer to a cs_cdo_quantities_t structure
+ *
+ * \return a initialize structure
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_quant_t
+cs_quant_set_face(cs_lnum_t                    f_id,
+                  const cs_cdo_quantities_t   *cdoq);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Retrieve the face center for a primal face (interior or border)
+ *
+ * \param[in]  f_id     id related to the face (f_id > n_i_face -> border face)
+ * \param[in]  cdoq     pointer to a cs_cdo_quantities_t structure
+ *
+ * \return a pointer to the face center coordinates
+ */
+/*----------------------------------------------------------------------------*/
+
+inline static const cs_real_t *
+cs_quant_set_face_center(cs_lnum_t                    f_id,
+                         const cs_cdo_quantities_t   *cdoq)
+{
+  const cs_lnum_t  bf_id = f_id - cdoq->n_i_faces;
+  if (bf_id > -1)  // Border face
+    return cdoq->b_face_center + 3*bf_id;
+  else             // Interior face
+    return cdoq->i_face_center + 3*f_id;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Retrieve the face surface and its unit normal vector for a primal
+ *        face (interior or border)
+ *
+ * \param[in]  f_id     id related to the face (f_id > n_i_face -> border face)
+ * \param[in]  cdoq     pointer to a cs_cdo_quantities_t structure
+ *
+ * \return a pointer to the face normalized vector
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_nvec3_t
+cs_quant_set_face_nvec(cs_lnum_t                    f_id,
+                       const cs_cdo_quantities_t   *cdoq);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Get the two normalized vector associated to a primal edge
+ *
+ * \param[in]  e_id     id related to the face (f_id > n_i_face -> border face)
+ * \param[in]  cdoq     pointer to a cs_cdo_quantities_t structure
+ *
+ * \return  a pointer to the edge normalized vector
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_nvec3_t
+cs_quant_set_edge_nvec(cs_lnum_t                    e_id,
+                       const cs_cdo_quantities_t   *cdoq);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Get the two normalized vector associated to a dual edge
+ *
+ * \param[in]  f_shift    position in c2f_idx
+ * \param[in]  cdoq       pointer to a cs_cdo_quantities_t structure
+ *
+ * \return  a pointer to the dual edge normalized vector
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_nvec3_t
+cs_quant_set_dedge_nvec(cs_lnum_t                     f_shift,
+                        const cs_cdo_quantities_t    *cdoq);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Dump a cs_quant_t structure
@@ -213,56 +356,6 @@ cs_quant_dump(FILE             *f,
               cs_lnum_t         num,
               const cs_quant_t  q);
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Compute for each vertex the dual cell volume which is also
- *
- *                sum    |celld(v) cap c| = pvol_v
- *              c in C_v
- *
- * \param[in]      connect   pointer to a cs_cdo_connect_t structure
- * \param[in]      quant     pointer to a cs_cdo_quantites_t structure
- * \param[in, out] p_pvol    pvol (if NULL, allocated in this routine)
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_compute_pvol_vtx(const cs_cdo_connect_t     *connect,
-                    const cs_cdo_quantities_t  *quant,
-                    double                     *p_pvol[]);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Compute for each edge a related volume pvol_e which constitutes
- *         a partition of unity
- *
- * \param[in]      connect   pointer to a cs_cdo_connect_t structure
- * \param[in]      quant     pointer to a cs_cdo_quantites_t structure
- * \param[in, out] p_pvol    pvol (if NULL, allocated in this routine)
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_compute_pvol_edge(const cs_cdo_connect_t      *connect,
-                     const cs_cdo_quantities_t   *quant,
-                     double                      *p_pvol[]);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Compute for each face a related volume pvol_f which constitutes
- *         a partition of unity
- *
- * \param[in]      connect   pointer to a cs_cdo_connect_t structure
- * \param[in]      quant     pointer to a cs_cdo_quantites_t structure
- * \param[in, out] p_pvol    pvol (if NULL, allocated in this routine)
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_compute_pvol_face(const cs_cdo_connect_t     *connect,
-                     const cs_cdo_quantities_t  *quant,
-                     double                     *p_pvol[]);
-
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
