@@ -2,14 +2,14 @@
 #define __CS_CDOFB_SCALEQ_H__
 
 /*============================================================================
- * Build an algebraic CDO face-based system for convection/diffusion equation
- * with source terms
+ * Build an algebraic CDO face-based system for unsteady convection/diffusion
+ * reaction of scalar-valued equations with source terms
  *============================================================================*/
 
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -39,12 +39,16 @@
  *----------------------------------------------------------------------------*/
 
 #include "cs_base.h"
-#include "cs_mesh.h"
-#include "cs_field.h"
 #include "cs_cdo_connect.h"
+#include "cs_cdo_local.h"
 #include "cs_cdo_quantities.h"
+#include "cs_equation_common.h"
 #include "cs_equation_param.h"
+#include "cs_field.h"
+#include "cs_matrix.h"
+#include "cs_mesh.h"
 #include "cs_source_term.h"
+#include "cs_time_step.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -59,7 +63,7 @@ BEGIN_C_DECLS
  *============================================================================*/
 
 /* Algebraic system for CDO face-based discretization */
-typedef struct _cs_cdofb_scaleq_t cs_cdofb_scaleq_t;
+typedef struct _cs_cdofb_t cs_cdofb_scaleq_t;
 
 /*============================================================================
  * Public function prototypes
@@ -67,28 +71,35 @@ typedef struct _cs_cdofb_scaleq_t cs_cdofb_scaleq_t;
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Set shared pointers from the main domain members
+ * \brief  Allocate work buffer and general structures related to CDO
+ *         scalar-valued face-based schemes.
+ *         Set shared pointers from the main domain members
  *
  * \param[in]  quant       additional mesh quantities struct.
  * \param[in]  connect     pointer to a cs_cdo_connect_t struct.
  * \param[in]  time_step   pointer to a time step structure
+ * \param[in]  ms          pointer to a cs_matrix_structure_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_cdofb_scaleq_set_shared_pointers(const cs_cdo_quantities_t    *quant,
-                                    const cs_cdo_connect_t       *connect,
-                                    const cs_time_step_t         *time_step);
+cs_cdofb_scaleq_init_common(const cs_cdo_quantities_t     *quant,
+                            const cs_cdo_connect_t        *connect,
+                            const cs_time_step_t          *time_step,
+                            const cs_matrix_structure_t   *ms);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Allocate work buffer and general structures related to CDO
- *         face-based schemes
+ * \brief  Retrieve work buffers used for building a CDO system cellwise
+ *
+ * \param[out]  csys   pointer to a pointer on a cs_cell_sys_t structure
+ * \param[out]  cb     pointer to a pointer on a cs_cell_builder_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_cdofb_scaleq_initialize(void);
+cs_cdofb_scaleq_get(cs_cell_sys_t       **csys,
+                    cs_cell_builder_t   **cb);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -98,81 +109,99 @@ cs_cdofb_scaleq_initialize(void);
 /*----------------------------------------------------------------------------*/
 
 void
-cs_cdofb_scaleq_finalize(void);
+cs_cdofb_scaleq_finalize_common(void);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Initialize a cs_cdofb_scaleq_t structure
+ * \brief  Initialize a cs_cdofb_scaleq_t structure storing data useful for
+ *         managing such a scheme
  *
- * \param[in]  eqp        pointer to a cs_equation_param_t structure
- * \param[in]  mesh       pointer to a cs_mesh_t structure
+ * \param[in]      eqp    pointer to a cs_equation_param_t structure
+ * \param[in, out] eqb    pointer to a cs_equation_builder_t structure
  *
  * \return a pointer to a new allocated cs_cdofb_scaleq_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 void *
-cs_cdofb_scaleq_init(const cs_equation_param_t   *eqp,
-                     const cs_mesh_t             *mesh);
+cs_cdofb_scaleq_init_context(const cs_equation_param_t   *eqp,
+                             cs_equation_builder_t       *eqb);
 
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Destroy a cs_cdofb_scaleq_t structure
  *
- * \param[in, out]  builder   pointer to a cs_cdofb_scaleq_t structure
+ * \param[in, out]  data   pointer to a cs_cdofb_scaleq_t structure
  *
  * \return a NULL pointer
  */
 /*----------------------------------------------------------------------------*/
 
 void *
-cs_cdofb_scaleq_free(void       *builder);
+cs_cdofb_scaleq_free_context(void   *data);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Destroy a cs_sla_matrix_t related to the system to solve
+ * \brief   Compute the contributions of source terms (store inside data)
  *
- * \param[in, out]  builder   pointer to a builder structure
- * \param[in, out]  matrix    pointer to a cs_sla_matrix_t structure
+ * \param[in]      eqp    pointer to a cs_equation_param_t structure
+ * \param[in, out] eqb    pointer to a cs_equation_builder_t structure
+ * \param[in, out] data     pointer to a cs_cdofb_scaleq_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_cdofb_scaleq_free_sysmat(void              *builder,
-                            cs_sla_matrix_t   *matrix);
+cs_cdofb_scaleq_compute_source(const cs_equation_param_t    *eqp,
+                               cs_equation_builder_t        *eqb,
+                               void                         *data);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief   Compute the contributions of source terms (store inside builder)
+ * \brief  Create the matrix of the current algebraic system.
+ *         Allocate and initialize the right-hand side associated to the given
+ *         data structure
  *
- * \param[in, out] builder     pointer to a cs_cdofb_scaleq_t structure
+ * \param[in]      eqp            pointer to a cs_equation_param_t structure
+ * \param[in, out] eqb            pointer to a cs_equation_builder_t structure
+ * \param[in, out] data           pointer to cs_cdofb_scaleq_t structure
+ * \param[in, out] system_matrix  pointer of pointer to a cs_matrix_t struct.
+ * \param[in, out] system_rhs     pointer of pointer to an array of double
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_cdofb_scaleq_compute_source(void     *builder);
+cs_cdofb_scaleq_initialize_system(const cs_equation_param_t  *eqp,
+                                  cs_equation_builder_t      *eqb,
+                                  void                       *data,
+                                  cs_matrix_t               **system_matrix,
+                                  cs_real_t                 **system_rhs);
 
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Build the linear system arising from a scalar convection/diffusion
  *         equation with a CDO face-based scheme.
+ *         One works cellwise and then process to the assembly
  *
  * \param[in]      mesh       pointer to a cs_mesh_t structure
  * \param[in]      field_val  pointer to the current value of the field
  * \param[in]      dt_cur     current value of the time step
- * \param[in, out] builder    pointer to cs_cdofb_scaleq_t structure
- * \param[in, out] rhs        pointer to a right-hand side array pointer
- * \param[in, out] sla_mat    pointer to cs_sla_matrix_t structure pointer
+ * \param[in]      eqp        pointer to a cs_equation_param_t structure
+ * \param[in, out] eqb        pointer to a cs_equation_builder_t structure
+ * \param[in, out] data       pointer to cs_cdofb_scaleq_t structure
+ * \param[in, out] rhs        right-hand side
+ * \param[in, out] matrix     pointer to cs_matrix_t structure to compute
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_cdofb_scaleq_build_system(const cs_mesh_t        *mesh,
-                             const cs_real_t        *field_val,
-                             double                  dt_cur,
-                             void                   *builder,
-                             cs_real_t             **rhs,
-                             cs_sla_matrix_t       **sla_mat);
+cs_cdofb_scaleq_build_system(const cs_mesh_t            *mesh,
+                             const cs_real_t            *field_val,
+                             double                      dt_cur,
+                             const cs_equation_param_t  *eqp,
+                             cs_equation_builder_t      *eqb,
+                             void                       *data,
+                             cs_real_t                  *rhs,
+                             cs_matrix_t                *matrix);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -181,44 +210,52 @@ cs_cdofb_scaleq_build_system(const cs_mesh_t        *mesh,
  *
  * \param[in]      solu       solution array
  * \param[in]      rhs        rhs associated to this solution array
- * \param[in, out] builder    pointer to cs_cdovb_scaleq_t structure
+ * \param[in]      eqp        pointer to a cs_equation_param_t structure
+ * \param[in, out] eqb        pointer to a cs_equation_builder_t structure
+ * \param[in, out] data       pointer to cs_cdofb_scaleq_t structure
  * \param[in, out] field_val  pointer to the current value of the field
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_cdofb_scaleq_update_field(const cs_real_t            *solu,
-                             const cs_real_t            *rhs,
-                             void                       *builder,
-                             cs_real_t                  *field_val);
+cs_cdofb_scaleq_update_field(const cs_real_t              *solu,
+                             const cs_real_t              *rhs,
+                             const cs_equation_param_t    *eqp,
+                             cs_equation_builder_t        *eqb,
+                             void                         *data,
+                             cs_real_t                    *field_val);
 
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Predefined extra-operations related to this equation
  *
  * \param[in]       eqname     name of the equation
- * \param[in]       field      pointer to a field strufcture
- * \param[in, out]  builder    pointer to builder structure
+ * \param[in]       field      pointer to a field structure
+ * \param[in]       eqp        pointer to a cs_equation_param_t structure
+ * \param[in, out]  eqb        pointer to a cs_equation_builder_t structure
+ * \param[in, out]  data       pointer to cs_cdofb_scaleq_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_cdofb_scaleq_extra_op(const char            *eqname,
-                         const cs_field_t      *field,
-                         void                  *builder);
+cs_cdofb_scaleq_extra_op(const char                 *eqname,
+                         const cs_field_t           *field,
+                         const cs_equation_param_t  *eqp,
+                         cs_equation_builder_t      *eqb,
+                         void                       *data);
 
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Get the computed values at each face
  *
- * \param[in]  builder    pointer to a cs_cdofb_scaleq_t structure
+ * \param[in] data       pointer to cs_cdofb_scaleq_t structure
  *
  * \return  a pointer to an array of double (size n_faces)
  */
 /*----------------------------------------------------------------------------*/
 
 double *
-cs_cdofb_scaleq_get_face_values(const void         *builder);
+cs_cdofb_scaleq_get_face_values(const void    *data);
 
 /*----------------------------------------------------------------------------*/
 
