@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -29,12 +29,13 @@
  *  Local headers
  *----------------------------------------------------------------------------*/
 
+#include "cs_cdo_connect.h"
 #include "cs_cdo_quantities.h"
 #include "cs_equation_param.h"
+#include "cs_equation_common.h"
 #include "cs_field.h"
 #include "cs_param.h"
 #include "cs_mesh.h"
-#include "cs_source_term.h"
 #include "cs_time_step.h"
 
 /*----------------------------------------------------------------------------*/
@@ -51,150 +52,237 @@ BEGIN_C_DECLS
 
 typedef struct _cs_equation_t cs_equation_t;
 
-/* List of available keys for setting an equation */
-typedef enum {
-
-  CS_EQKEY_ADV_FORMULATION,
-  CS_EQKEY_ADV_SCHEME,
-  CS_EQKEY_ADV_FLUX_QUADRA,
-  CS_EQKEY_BC_ENFORCEMENT,
-  CS_EQKEY_BC_QUADRATURE,
-  CS_EQKEY_EXTRA_OP,
-  CS_EQKEY_HODGE_DIFF_ALGO,
-  CS_EQKEY_HODGE_DIFF_COEF,
-  CS_EQKEY_HODGE_TIME_ALGO,
-  CS_EQKEY_HODGE_TIME_COEF,
-  CS_EQKEY_HODGE_REAC_ALGO,
-  CS_EQKEY_HODGE_REAC_COEF,
-  CS_EQKEY_ITSOL,
-  CS_EQKEY_ITSOL_EPS,
-  CS_EQKEY_ITSOL_MAX_ITER,
-  CS_EQKEY_ITSOL_RESNORM,
-  CS_EQKEY_ITSOL_VERBOSITY,
-  CS_EQKEY_PRECOND,
-  CS_EQKEY_SOLVER_FAMILY,
-  CS_EQKEY_SPACE_SCHEME,
-  CS_EQKEY_TIME_SCHEME,
-  CS_EQKEY_TIME_THETA,
-  CS_EQKEY_VERBOSITY,
-  CS_EQKEY_N_KEYS
-
-} cs_equation_key_t;
-
 /*============================================================================
  * Public function prototypes
  *============================================================================*/
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Allocate a pointer to a buffer of size at least the 2*n_cells for
- *         managing temporary usage of memory when dealing with equations
- *         Call specific structure allocation related to a numerical scheme
- *         according the scheme flag
- *         The size of the temporary buffer can be bigger according to the
- *         numerical settings
- *         Set also shared pointers from the main domain members
+ * \brief  Retrieve the number of equations
  *
- * \param[in]  connect       pointer to a cs_cdo_connect_t structure
- * \param[in]  quant         pointer to additional mesh quantities struct.
- * \param[in]  time_step     pointer to a time step structure
- * \param[in]  scheme_flag   flag to identify which kind of numerical scheme is
- *                           requested to solve the computational domain
+ * \return the current number of cs_equation_t structure allocated
  */
 /*----------------------------------------------------------------------------*/
 
-void
-cs_equation_allocate_common_structures(const cs_cdo_connect_t     *connect,
-                                       const cs_cdo_quantities_t  *quant,
-                                       const cs_time_step_t       *time_step,
-                                       cs_flag_t                   scheme_flag);
+int
+cs_equation_get_n_equations(void);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Allocate a pointer to a buffer of size at least the 2*n_cells for
- *         managing temporary usage of memory when dealing with equations
- *         Call specific structure allocation related to a numerical scheme
- *         according the scheme flag
- *         The size of the temporary buffer can be bigger according to the
- *         numerical settings
+ * \brief  Find the cs_equation_t structure with name eqname
+ *         Return NULL if not find
  *
- * \param[in]  scheme_flag   flag to identify which kind of numerical scheme is
- *                           requested to solve the computational domain
+ * \param[in]  eqname    name of the equation to find
+ *
+ * \return a pointer to a cs_equation_t structure or NULL if not found
  */
 /*----------------------------------------------------------------------------*/
 
-void
-cs_equation_free_common_structures(cs_flag_t   scheme_flag);
+cs_equation_t *
+cs_equation_by_name(const char    *eqname);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Retrieve a pointer to a buffer of size at least the 2*n_cells
- *         The size of the temporary buffer can be bigger according to the
- *         numerical settings
+ * \brief  Return the cs_equation_param_t structure associated to a
+ *         cs_equation_t structure thanks to the equation name
  *
- * \return  a pointer to an array of double
+ * \param[in]  eqname       name of the equation
+ *
+ * \return a cs_equation_param_t structure or NULL if not found
  */
 /*----------------------------------------------------------------------------*/
 
-cs_real_t *
-cs_equation_get_tmpbuf(void);
+cs_equation_param_t *
+cs_equation_param_by_name(const char    *eqname);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Get the allocation size of the temporary buffer
+ * \brief  Return the cs_equation_param_t structure associated to a
+ *         cs_equation_t structure
  *
- * \return  the size of the temporary buffer
+ * \param[in]  eq       pointer to a cs_equation_t structure
+ *
+ * \return a cs_equation_param_t structure or NULL if not found
  */
 /*----------------------------------------------------------------------------*/
 
-size_t
-cs_equation_get_tmpbuf_size(void);
+cs_equation_param_t *
+cs_equation_get_param(const cs_equation_t    *eq);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Define and initialize a new structure to store parameters related
- *         to an equation
+ * \brief  Find the cs_equation_t structure with name eqname
+ *         Return NULL if not find
  *
- * \param[in] eqname           name of the equation
- * \param[in] varname          name of the variable associated to this equation
- * \param[in] eqtype           type of equation (user, predefined...)
- * \param[in] vartype          type of variable (scalar, vector, tensor...)
- * \param[in] default_bc       type of boundary condition set by default
+ * \param[in]  eq_id    id of the equation to find
+ *
+ * \return a pointer to a cs_equation_t structure or NULL if not found
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_equation_t *
+cs_equation_by_id(int   eq_id);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Return true is the given equation is steady otherwise false
+ *
+ * \param[in]  eq       pointer to a cs_equation_t structure
+ *
+ * \return true or false
+ */
+/*----------------------------------------------------------------------------*/
+
+bool
+cs_equation_is_steady(const cs_equation_t    *eq);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Return the name related to the given cs_equation_t structure
+ *
+ * \param[in]  eq       pointer to a cs_equation_t structure
+ *
+ * \return a name or NULL if not found
+ */
+/*----------------------------------------------------------------------------*/
+
+const char *
+cs_equation_get_name(const cs_equation_t    *eq);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Return the id number related to the given cs_equation_t structure
+ *
+ * \param[in]  eq       pointer to a cs_equation_t structure
+ *
+ * \return an id (0 ... n-1) or -1 if not found
+ */
+/*----------------------------------------------------------------------------*/
+
+int
+cs_equation_get_id(const cs_equation_t    *eq);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Return the field structure associated to a cs_equation_t structure
+ *
+ * \param[in]  eq       pointer to a cs_equation_t structure
+ *
+ * \return a cs_field_t structure or NULL if not found
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_field_t *
+cs_equation_get_field(const cs_equation_t    *eq);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Return the flag associated to an equation
+ *
+ * \param[in]  eq       pointer to a cs_equation_t structure
+ *
+ * \return a flag (cs_flag_t type)
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_flag_t
+cs_equation_get_flag(const cs_equation_t    *eq);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Return the cs_equation_builder_t structure associated to a
+ *         cs_equation_t structure. Only for an advanced usage.
+ *
+ * \param[in]  eq       pointer to a cs_equation_t structure
+ *
+ * \return a cs_equation_builder_t structure or NULL if not found
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_equation_builder_t *
+cs_equation_get_builder(const cs_equation_t    *eq);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Return a pointer to a structure useful to handle low-level
+ *         operations for the given equation
+ *
+ * \param[in]  eq       pointer to a cs_equation_t structure
+ *
+ * \return a pointer to a structure to cast on-the-fly or NULL if not found
+ */
+/*----------------------------------------------------------------------------*/
+
+void *
+cs_equation_get_scheme_context(const cs_equation_t    *eq);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Add a new equation structure and set a first set of parameters
+ *
+ * \param[in] eqname        name of the equation
+ * \param[in] varname       name of the variable associated to this equation
+ * \param[in] eqtype        type of equation (user, predefined...)
+ * \param[in] dim           dimension of the unknow attached to this equation
+ * \param[in] default_bc    type of boundary condition set by default
  *
  * \return  a pointer to the new allocated cs_equation_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 cs_equation_t *
-cs_equation_create(const char            *eqname,
-                   const char            *varname,
-                   cs_equation_type_t     eqtype,
-                   cs_param_var_type_t    vartype,
-                   cs_param_bc_type_t     default_bc);
+cs_equation_add(const char            *eqname,
+                const char            *varname,
+                cs_equation_type_t     eqtype,
+                int                    dim,
+                cs_param_bc_type_t     default_bc);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Destroy a cs_equation_t structure
+ * \brief  Add a new user equation structure and set a first set of parameters
  *
- * \param[in, out] eq    pointer to a cs_equation_t structure
+ * \param[in] eqname        name of the equation
+ * \param[in] varname       name of the variable associated to this equation
+ * \param[in] dim           dimension of the unknow attached to this equation
+ * \param[in] default_bc    type of boundary condition set by default
  *
- * \return  a NULL pointer
+ * \return  a pointer to the new allocated cs_equation_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 cs_equation_t *
-cs_equation_free(cs_equation_t  *eq);
+cs_equation_add_user(const char            *eqname,
+                     const char            *varname,
+                     int                    dim,
+                     cs_param_bc_type_t     default_bc);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Summary of a cs_equation_t structure
- *
- * \param[in]  eq       pointer to a cs_equation_t structure
+ * \brief  Destroy all cs_equation_t structures
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_equation_summary(const cs_equation_t  *eq);
+cs_equation_destroy_all(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Print a synthesis of the monitoring information in the performance
+ *         file
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_equation_log_monitoring(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Summarize all cs_equation_t structures
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_equation_log_setup(void);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -212,215 +300,45 @@ cs_equation_set_timer_stats(cs_equation_t  *eq);
  * \brief  Assign a set of pointer functions for managing the cs_equation_t
  *         structure during the computation
  *
- * \param[in, out]  eq       pointer to a cs_equation_t structure
+ * \param[in]  connect        pointer to a cs_cdo_connect_t structure
+ * \param[in]  do_profiling   true or false
+ *
+ * \return true if all equations are steady-state otherwise false
  */
 /*----------------------------------------------------------------------------*/
 
-void
-cs_equation_last_setup(cs_equation_t  *eq);
+bool
+cs_equation_finalize_setup(const cs_cdo_connect_t   *connect,
+                           bool                      do_profiling);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Set a parameter in a cs_equation_t structure attached to keyname
- *
- * \param[in, out]  eq       pointer to a cs_equation_t structure
- * \param[in]       key      key related to the member of eq to set
- * \param[in]       keyval   accessor to the value to set
+ * \brief  Create a field structure related to all cs_equation_t structures
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_equation_set_param(cs_equation_t       *eq,
-                      cs_equation_key_t    key,
-                      const char          *keyval);
+cs_equation_create_fields(void);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Associate a material property or an advection field with an equation
- *         for a given term (diffusion, time, convection)
+ * \brief  Allocate and initialize the builder of the algebraic system.
+ *         Set the initialize condition to all variable fields associated to
+ *         each cs_equation_t structure.
+ *         Compute the initial source term.
  *
- * \param[in, out]  eq        pointer to a cs_equation_t structure
- * \param[in]       keyword   "time", "diffusion", "advection"
- * \param[in]       pointer   pointer to a given structure
+ * \param[in]  mesh      pointer to a cs_mesh_t structure
+ * \param[in]  connect   pointer to a cs_cdo_connect_t structure
+ * \param[in]  quant     pointer to a cs_cdo_quantities_t structure
+ * \param[in]  ts        pointer to a cs_time_step_t structure
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_equation_link(cs_equation_t       *eq,
-                 const char          *keyword,
-                 void                *pointer);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Define the initial condition of the unknown related to this equation
- *         This definition can be done by mesh location
- *         Available types of definition are: "value" and "analytic"
- *
- * \param[in, out]  eq        pointer to a cs_equation_t structure
- * \param[in]       ml_name   name of the associated mesh location (if NULL or
- *                            "" all entities are considered)
- * \param[in]       def_key   way of defining the value of the BC
- * \param[in]       val       pointer to the value
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_equation_set_ic(cs_equation_t    *eq,
-                   const char       *ml_name,
-                   const char       *def_key,
-                   void             *val);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Define and initialize a new structure to store parameters related
- *         to an equation
- *         bc_key among "dirichlet", "neumann" or "robin"
- *         def_key among "value", "analytic", "user"
- *
- * \param[in, out]  eq        pointer to a cs_equation_t structure
- * \param[in]       ml_name   name of the related mesh location
- * \param[in]       bc_key    type of boundary condition to add
- * \param[in]       def_key   way of defining the value of the bc
- * \param[in]       val       pointer to the value
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_equation_add_bc(cs_equation_t    *eq,
-                   const char       *ml_name,
-                   const char       *bc_key,
-                   const char       *def_key,
-                   const void       *val);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Define and initialize a new structure to store parameters related
- *         to a reaction term
- *
- * \param[in, out] eq         pointer to a cs_equation_t structure
- * \param[in]      property   pointer to a cs_property_t struct.
- * \param[in]      r_name     name of the reaction term (optional, i.e. NULL)
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_equation_add_linear_reaction(cs_equation_t   *eq,
-                                cs_property_t   *property,
-                                const char      *r_name);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Define and initialize a new structure to store parameters related
- *         to a source term
- *         def_key among "value", "analytic", "user"...
- *
- * \param[in, out] eq            pointer to a cs_equation_t structure
- * \param[in]      ml_id         id related to a mesh location
- * \param[in]      array_desc    short description of this array (mask of bits)
- * \param[in]      array_values  pointer to the array values
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_equation_add_gravity_source_term(cs_equation_t   *eq,
-                                    int              ml_id,
-                                    cs_desc_t        array_desc,
-                                    cs_real_t       *array_values);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Define and initialize by value a new structure to store parameters
- *         related to a source term defined by a user
- *
- * \param[in, out]  eq        pointer to a cs_equation_t structure
- * \param[in]       st_name   name of the source term or NULL
- * \param[in]       ml_name   name of the related mesh location
- * \param[in]       val       pointer to the value
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_equation_add_source_term_by_val(cs_equation_t   *eq,
-                                   const char      *st_name,
-                                   const char      *ml_name,
-                                   const void      *val);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Define and initialize by an analytical function a new structure
- *         related to a source term defined by a user
- *
- * \param[in, out]  eq        pointer to a cs_equation_t structure
- * \param[in]       st_name   name of the source term or NULL
- * \param[in]       ml_name   name of the related mesh location
- * \param[in]       ana       pointer to an analytical function
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_equation_add_source_term_by_analytic(cs_equation_t        *eq,
-                                        const char           *st_name,
-                                        const char           *ml_name,
-                                        cs_analytic_func_t   *ana);
-
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Set the type of quadrature to use for computing a source term
- *         If st_name is NULL, all source terms of the given equation are set
- *
- * \param[in, out]  eq         pointer to a cs_equation_t structure
- * \param[in]       st_name    name of the source term
- * \param[in]       quad_type  type of quadrature to use
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_equation_set_source_term_quadrature(cs_equation_t      *eq,
-                                       const char         *st_name,
-                                       cs_quadra_type_t    quad_type);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Set the type of quadrature to use for computing a source term
- *         If st_name is NULL, all source terms of the given equation are set
- *
- * \param[in, out]  eq         pointer to a cs_equation_t structure
- * \param[in]       st_name    name of the source term
- * \param[in]       type       type of reduction to apply
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_equation_set_source_term_reduction(cs_equation_t               *eq,
-                                      const char                  *st_name,
-                                      cs_source_term_reduction_t   type);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Create a field structure related to this cs_equation_t structure
- *         to an equation
- *
- * \param[in, out]  eq       pointer to a cs_equation_t structure
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_equation_create_field(cs_equation_t      *eq);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Initialize the values of a field according to the initial condition
- *         related to its equation
- *
- * \param[in]       mesh       pointer to the mesh structure
- * \param[in, out]  eq         pointer to a cs_equation_t structure
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_equation_init_system(const cs_mesh_t            *mesh,
-                        cs_equation_t              *eq);
+cs_equation_initialize(const cs_mesh_t             *mesh,
+                       const cs_cdo_connect_t      *connect,
+                       const cs_cdo_quantities_t   *quant,
+                       const cs_time_step_t        *ts);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -439,7 +357,7 @@ cs_equation_needs_build(const cs_equation_t    *eq);
 /*!
  * \brief  Build the linear system for this equation
  *
- * \param[in]       m           pointer to a cs_mesh_t structure
+ * \param[in]       mesh        pointer to a cs_mesh_t structure
  * \param[in]       time_step   pointer to a time step structure
  * \param[in]       dt_cur      value of the current time step
  * \param[in, out]  eq          pointer to a cs_equation_t structure
@@ -447,7 +365,7 @@ cs_equation_needs_build(const cs_equation_t    *eq);
 /*----------------------------------------------------------------------------*/
 
 void
-cs_equation_build_system(const cs_mesh_t            *m,
+cs_equation_build_system(const cs_mesh_t            *mesh,
                          const cs_time_step_t       *time_step,
                          double                      dt_cur,
                          cs_equation_t              *eq);
@@ -457,41 +375,110 @@ cs_equation_build_system(const cs_mesh_t            *m,
  * \brief  Solve the linear system for this equation
  *
  * \param[in, out]  eq          pointer to a cs_equation_t structure
- * \param[in]       do_logcvg   output information on convergence or not
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_equation_solve(cs_equation_t   *eq,
-                  bool             do_logcvg);
+cs_equation_solve(cs_equation_t   *eq);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Predefined extra-operations related to this equation
- *
- * \param[in]  eq      pointer to a cs_equation_t structure
- * \param[in]  ts      pointer to a cs_time_step_t struct.
- * \param[in]  dt      value of the cureent time step
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_equation_extra_op(const cs_equation_t     *eq,
-                     const cs_time_step_t    *ts,
-                     double                   dt);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Return true is the given equation is steady otherwise false
+ * \brief  Return a pointer to the cs_property_t structure associated to the
+ *         diffusion term for this equation (NULL if not activated).
  *
  * \param[in]  eq       pointer to a cs_equation_t structure
  *
- * \return true or false
+ * \return a pointer to a cs_property_t structure
  */
 /*----------------------------------------------------------------------------*/
 
-bool
-cs_equation_is_steady(const cs_equation_t    *eq);
+cs_property_t *
+cs_equation_get_diffusion_property(const cs_equation_t    *eq);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Return a pointer to the cs_property_t structure associated to the
+ *         unsteady term for this equation (NULL if not activated).
+ *
+ * \param[in]  eq       pointer to a cs_equation_t structure
+ *
+ * \return a pointer to a cs_property_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_property_t *
+cs_equation_get_time_property(const cs_equation_t    *eq);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Return a pointer to the cs_property_t structure associated to the
+ *         reaction term called r_name and related to this equation
+ *
+ *
+ * \param[in]  eq            pointer to a cs_equation_t structure
+ * \param[in]  reaction_id   id related to this reaction term
+ *
+ * \return a pointer to a cs_property_t structure or NULL if not found
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_property_t *
+cs_equation_get_reaction_property(const cs_equation_t    *eq,
+                                  const int               reaction_id);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Return the type of numerical scheme used for the discretization in
+ *         space
+ *
+ * \param[in]  eq       pointer to a cs_equation_t structure
+ *
+ * \return  a cs_param_space_scheme_t variable
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_param_space_scheme_t
+cs_equation_get_space_scheme(const cs_equation_t    *eq);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Return the max. degree used in the polynomial basis for the space
+ *         discretization
+ *
+ * \param[in]  eq       pointer to a cs_equation_t structure
+ *
+ * \return  the polynomial order
+ */
+/*----------------------------------------------------------------------------*/
+
+int
+cs_equation_get_space_poly_degree(const cs_equation_t    *eq);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Return the dimension of the variable solved by this equation
+ *
+ * \param[in]  eq       pointer to a cs_equation_t structure
+ *
+ * \return  an integer corresponding to the dimension of the variable
+ */
+/*----------------------------------------------------------------------------*/
+
+int
+cs_equation_get_var_dim(const cs_equation_t    *eq);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Return the type of equation for the given equation structure
+ *
+ * \param[in]  eq       pointer to a cs_equation_t structure
+ *
+ * \return  the type of the given equation
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_equation_type_t
+cs_equation_get_type(const cs_equation_t    *eq);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -521,144 +508,6 @@ cs_equation_get_face_values(const cs_equation_t    *eq);
 const cs_real_t *
 cs_equation_get_cell_values(const cs_equation_t    *eq);
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Return the name related to the given cs_equation_t structure
- *         to an equation
- *
- * \param[in]  eq       pointer to a cs_equation_t structure
- *
- * \return a name or NULL if not found
- */
-/*----------------------------------------------------------------------------*/
-
-const char *
-cs_equation_get_name(const cs_equation_t    *eq);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Return the field structure associated to a cs_equation_t structure
- *
- * \param[in]  eq       pointer to a cs_equation_t structure
- *
- * \return a cs_field_t structure or NULL if not found
- */
-/*----------------------------------------------------------------------------*/
-
-cs_field_t *
-cs_equation_get_field(const cs_equation_t    *eq);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Return the flag associated to an equation
- *
- * \param[in]  eq       pointer to a cs_equation_t structure
- *
- * \return a flag (cs_flag_t type)
- */
-/*----------------------------------------------------------------------------*/
-
-cs_flag_t
-cs_equation_get_flag(const cs_equation_t    *eq);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Return the cs_equation_param_t structure associated to a
- *         cs_equation_t structure
- *
- * \param[in]  eq       pointer to a cs_equation_t structure
- *
- * \return a cs_equation_param_t structure or NULL if not found
- */
-/*----------------------------------------------------------------------------*/
-
-const cs_equation_param_t *
-cs_equation_get_param(const cs_equation_t    *eq);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Return a pointer to the cs_property_t structure associated to the
- *         diffusion term for this equation.
- *
- * \param[in]  eq       pointer to a cs_equation_t structure
- *
- * \return a pointer to a cs_property_t structure or NULL if not found
- */
-/*----------------------------------------------------------------------------*/
-
-cs_property_t *
-cs_equation_get_diffusion_property(const cs_equation_t    *eq);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Return a pointer to the cs_property_t structure associated to the
- *         unsteady term for this equation.
- *
- * \param[in]  eq       pointer to a cs_equation_t structure
- *
- * \return a pointer to a cs_property_t structure or NULL if not found
- */
-/*----------------------------------------------------------------------------*/
-
-cs_property_t *
-cs_equation_get_time_property(const cs_equation_t    *eq);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Return a pointer to the cs_property_t structure associated to the
- *         reaction term called r_name and related to this equation
- *
- *
- * \param[in]  eq       pointer to a cs_equation_t structure
- *
- * \return a pointer to a cs_property_t structure or NULL if not found
- */
-/*----------------------------------------------------------------------------*/
-
-cs_property_t *
-cs_equation_get_reaction_property(const cs_equation_t    *eq,
-                                  const char             *r_name);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Return the type of numerical scheme used for the discretization in
- *         space
- *
- * \param[in]  eq       pointer to a cs_equation_t structure
- *
- * \return  a cs_space_scheme_t variable
- */
-/*----------------------------------------------------------------------------*/
-
-cs_space_scheme_t
-cs_equation_get_space_scheme(const cs_equation_t    *eq);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Return the type of variable solved by this equation
- *
- * \param[in]  eq       pointer to a cs_equation_t structure
- *
- * \return  the type of variable (sclar, vector...) associated to this equation
- */
-/*----------------------------------------------------------------------------*/
-
-cs_param_var_type_t
-cs_equation_get_var_type(const cs_equation_t    *eq);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Return the type of equation for the given equation structure
- *
- * \param[in]  eq       pointer to a cs_equation_t structure
- *
- * \return  the type of the given equation
- */
-/*----------------------------------------------------------------------------*/
-
-cs_equation_type_t
-cs_equation_get_type(const cs_equation_t    *eq);
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the diffusive and convective flux accross a plane defined
@@ -669,7 +518,7 @@ cs_equation_get_type(const cs_equation_t    *eq);
  * \param[in]      direction   vector indicating in which direction flux is > 0
  * \param[in, out] diff_flux   value of the diffusive part of the flux
  * \param[in, out] conv_flux   value of the convective part of the flux
- */
+  */
 /*----------------------------------------------------------------------------*/
 
 void
@@ -685,13 +534,41 @@ cs_equation_compute_flux_across_plane(const cs_equation_t   *eq,
  *         Primal or dual faces are considered according to the space scheme.
  *
  * \param[in]      eq          pointer to a cs_equation_t structure
+ * \param[in]      location    indicate where the flux has to be computed
  * \param[in, out] diff_flux   value of the diffusive flux
   */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_equation_compute_diff_flux(const cs_equation_t   *eq,
-                              cs_real_t             *diff_flux);
+cs_equation_compute_diff_flux_cellwise(const cs_equation_t   *eq,
+                                       cs_flag_t              location,
+                                       cs_real_t             *diff_flux);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Cellwise computation of the discrete gradient at vertices
+ *
+ * \param[in]      eq          pointer to a cs_equation_t structure
+ * \param[in, out] v_gradient  gradient at vertices
+  */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_equation_compute_vtx_field_gradient(const cs_equation_t   *eq,
+                                       cs_real_t             *v_gradient);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Predefined extra-operations related to all equations
+ *
+ * \param[in]  ts      pointer to a cs_time_step_t struct.
+ * \param[in]  dt      value of the current time step
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_equation_extra_post_all(const cs_time_step_t    *ts,
+                           double                   dt);
 
 /*----------------------------------------------------------------------------*/
 
