@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -31,9 +31,12 @@
  *  Local headers
  *----------------------------------------------------------------------------*/
 
+#include "assert.h"
+
 #include "cs_cdo_connect.h"
 #include "cs_cdo_local.h"
 #include "cs_cdo_quantities.h"
+#include "cs_flag.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -72,6 +75,24 @@ cs_reco_conf_vtx_dofs(const cs_cdo_connect_t      *connect,
                       double                      *p_crec[],
                       double                      *p_frec[]);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Reconstruct the value at all cell centers from an array of values
+ *         defined on primal vertices.
+ *
+ *  \param[in]      c2v      cell -> vertices connectivity
+ *  \param[in]      quant    pointer to the additional quantities struct.
+ *  \param[in]      array    pointer to the array of values
+ *  \param[in, out] val_xc   values of the reconstruction at the cell center
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_reco_pv_at_cell_centers(const cs_adjacency_t        *c2v,
+                           const cs_cdo_quantities_t   *quant,
+                           const double                *array,
+                           cs_real_t                   *val_xc);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Reconstruct the value at the cell center from an array of values
@@ -87,39 +108,11 @@ cs_reco_conf_vtx_dofs(const cs_cdo_connect_t      *connect,
 
 void
 cs_reco_pv_at_cell_center(cs_lnum_t                    c_id,
-                          const cs_connect_index_t    *c2v,
+                          const cs_adjacency_t        *c2v,
                           const cs_cdo_quantities_t   *quant,
                           const double                *array,
                           cs_real_t                   *val_xc);
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Reconstruct the value at the face center from an array of values
- *         defined on primal vertices.
- *
- *  \param[in]      fm     pointer to cs_face_mesh_t structure
- *  \param[in]      p_v    pointer to an array of values (local to this face)
- *  \param[in, out] p_f    value of the reconstruction at the face center
- */
-/*----------------------------------------------------------------------------*/
-
-static inline void
-cs_reco_pv_at_face_center(const cs_face_mesh_t    *fm,
-                          const double            *p_v,
-                          double                  *p_f)
-{
-  *p_f = 0.;
-
-  if (p_v == NULL)
-    return;
-
-  const cs_quant_t  pfq = fm->face;
-
-  for (short int e = 0; e < fm->n_ef; e++)
-    *p_f += (p_v[fm->e2v_ids[2*e]] + p_v[fm->e2v_ids[2*e+1]]) * fm->tef[e];
-  *p_f *= 0.5 / pfq.meas;
-}
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Reconstruct the value at the face center from an array of values
@@ -156,58 +149,52 @@ cs_reco_pf_from_pv(cs_lnum_t                     f_id,
 
 void
 cs_reco_dfbyc_at_cell_center(cs_lnum_t                    c_id,
-                             const cs_connect_index_t    *c2e,
+                             const cs_adjacency_t        *c2e,
                              const cs_cdo_quantities_t   *quant,
                              const double                *array,
                              cs_real_3_t                  val_xc);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Reconstruct a constant vector inside pec which is a volume
- *         surrounding the edge e inside the cell c.
- *         array is scanned thanks to the c2e connectivity.
+ * \brief  Reconstruct a constant vector inside the cell c.
+ *         array is scanned thanks to the c2e connectivity. Pointer is already
+ *         located at the beginning of the cell sequence.
  *         Reconstruction used is based on DGA (stabilization = 1/d where d is
  *         the space dimension)
  *
- *  \param[in]      c_id      cell id
- *  \param[in]      e_id      edge id
- *  \param[in]      c2e       cell -> edges connectivity
- *  \param[in]      quant     pointer to the additional quantities struct.
- *  \param[in]      array     pointer to the array of values
+ *  \param[in]      cm        pointer to a cs_cell_mesh_t structure
+ *  \param[in]      array     local pointer to the array of values
+ *  \param[in, out] val_c     value of the reconstructed vector in the cell
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_reco_dfbyc_in_cell(const cs_cell_mesh_t        *cm,
+                      const cs_real_t             *array,
+                      cs_real_3_t                  val_c);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Reconstruct a constant vector inside pec which is a volume
+ *         surrounding the edge e inside the cell c.
+ *         array is scanned thanks to the c2e connectivity. Pointer is already
+ *         located at the beginning of the cell sequence.
+ *         Reconstruction used is based on DGA (stabilization = 1/d where d is
+ *         the space dimension)
+ *
+ *  \param[in]      cm        pointer to a cs_cell_mesh_t structure
+ *  \param[in]      e         local edge id
+ *  \param[in]      array     local pointer to the array of values
  *  \param[in, out] val_pec   value of the reconstruction in pec
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_reco_dfbyc_in_pec(cs_lnum_t                    c_id,
-                     cs_lnum_t                    e_id,
-                     const cs_connect_index_t    *c2e,
-                     const cs_cdo_quantities_t   *quant,
-                     const double                *array,
+cs_reco_dfbyc_in_pec(const cs_cell_mesh_t        *cm,
+                     short int                    e,
+                     const cs_real_t             *array,
                      cs_real_3_t                  val_pec);
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Reconstruct by a constant vector a field of edge-based DoFs
- *         in a volume surrounding an edge
- *
- *  \param[in]      cid     cell id
- *  \param[in]      e1_id   sub-volume related to this edge id
- *  \param[in]      c2e     cell -> edges connectivity
- *  \param[in]      quant   pointer to the additional quantities struct.
- *  \param[in]      dof     pointer to the field of edge-based DoFs
- *  \param[in, out] reco    value of the reconstructed field in this sub-volume
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_reco_cost_edge_dof(cs_lnum_t                    cid,
-                      cs_lnum_t                    e1_id,
-                      const cs_connect_index_t    *c2e,
-                      const cs_cdo_quantities_t   *quant,
-                      const double                *dof,
-                      double                       reco[]);
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Reconstruct the value at the cell center of the gradient of a field
@@ -222,11 +209,30 @@ cs_reco_cost_edge_dof(cs_lnum_t                    cid,
 /*----------------------------------------------------------------------------*/
 
 void
-cs_reco_grd_cell_from_pv(cs_lnum_t                    c_id,
-                         const cs_cdo_connect_t      *connect,
-                         const cs_cdo_quantities_t   *quant,
-                         const double                *pdi,
-                         cs_real_t                    val_xc[]);
+cs_reco_grad_cell_from_pv(cs_lnum_t                    c_id,
+                          const cs_cdo_connect_t      *connect,
+                          const cs_cdo_quantities_t   *quant,
+                          const cs_real_t             *pdi,
+                          cs_real_t                    val_xc[]);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Reconstruct the value of a scalar potential at a point inside a cell
+ *         The scalar potential has DoFs located at primal vertices
+ *
+ * \param[in]      cm             pointer to a cs_cell_mesh_t structure
+ * \param[in]      pdi            array of DoF values at vertices
+ * \param[out]     cell_gradient  gradient inside the cell
+ *
+ * \return the value of the reconstructed potential at the evaluation point
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_reco_cw_cell_grad_from_scalar_pv(const cs_cell_mesh_t    *cm,
+                                    const cs_real_t          pdi[],
+                                    cs_real_t               *cell_gradient);
+
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -242,7 +248,7 @@ cs_reco_grd_cell_from_pv(cs_lnum_t                    c_id,
 
 void
 cs_reco_ccen_edge_dof(cs_lnum_t                   cid,
-                      const cs_connect_index_t   *c2e,
+                      const cs_adjacency_t       *c2e,
                       const cs_cdo_quantities_t  *quant,
                       const double               *dof,
                       double                      reco[]);
@@ -264,6 +270,128 @@ cs_reco_ccen_edge_dofs(const cs_cdo_connect_t     *connect,
                        const double               *dof,
                        double                     *p_ccrec[]);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Reconstruct the value of a scalar potential at a point inside a cell
+ *         The scalar potential has DoFs located at primal vertices
+ *
+ * \param[in]      cm           pointer to a cs_cell_mesh_t structure
+ * \param[in]      pdi          array of DoF values at vertices
+ * \param[in]      length_xcxp  lenght of the segment [x_c, x_p]
+ * \param[in]      unitv_xcxp   unitary vector pointed from x_c to x_p
+ * \param[in, out] wbuf         pointer to a temporary buffer
+ *
+ * \return the value of the reconstructed potential at the evaluation point
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_real_t
+cs_reco_cw_scalar_pv_inside_cell(const cs_cell_mesh_t    *cm,
+                                 const cs_real_t          pdi[],
+                                 const cs_real_t          length_xcxp,
+                                 const cs_real_t          unitv_xcxp[],
+                                 cs_real_t                wbuf[]);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief   Compute the weighted (by volume) gradient inside a given primal
+ *          cell for the related vertices.
+ *          Use the WBS algo. for approximating the gradient.
+ *          The computation takes into account a subdivision into tetrahedra of
+ *          the current cell based on p_{ef,c}
+ *
+ * \param[in]      cm       pointer to a cs_cell_mesh_t structure
+ * \param[in]      pot      values of the potential fields at vertices + cell
+ * \param[in, out] cb       auxiliary structure for computing the flux
+ * \param[in, out] vgrd     gradient at vertices inside this cell
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_reco_cw_vgrd_wbs_from_pvc(const cs_cell_mesh_t   *cm,
+                             const cs_real_t        *pot,
+                             cs_cell_builder_t      *cb,
+                             cs_real_t              *vgrd);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief   Compute the mean value of a gradient inside a given primal cell.
+ *          Use the WBS algo. for approximating the gradient.
+ *          The computation takes into account a subdivision into tetrahedra of
+ *          the current cell based on p_{ef,c}
+ *
+ * \param[in]      cm       pointer to a cs_cell_mesh_t structure
+ * \param[in]      pot      values of the potential fields at vertices + cell
+ * \param[in, out] cb       auxiliary structure for computing the flux
+ * \param[in, out] cgrd     mean-value of the cell gradient
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_reco_cw_cgrd_wbs_from_pvc(const cs_cell_mesh_t   *cm,
+                             const cs_real_t        *pot,
+                             cs_cell_builder_t      *cb,
+                             cs_real_t              *cgrd);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Reconstruct the value at the face center from an array of values
+ *         defined on primal vertices.
+ *
+ *  \param[in]      fm     pointer to cs_face_mesh_t structure
+ *  \param[in]      p_v    pointer to an array of values (local to this face)
+ *
+ * \return the value of the reconstruction at the face center
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline cs_real_t
+cs_reco_fw_scalar_pv_at_face_center(const cs_face_mesh_t    *fm,
+                                    const cs_real_t         *p_v)
+{
+  cs_real_t  p_f = 0.;
+
+  if (p_v == NULL)
+    return p_f;
+
+  const cs_quant_t  pfq = fm->face;
+
+  for (short int e = 0; e < fm->n_ef; e++)
+    p_f += (p_v[fm->e2v_ids[2*e]] + p_v[fm->e2v_ids[2*e+1]]) * fm->tef[e];
+  p_f *= 0.5 / pfq.meas;
+
+  return p_f;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Reconstruct the value of a scalar potential at the cell center from
+ *         an array of values defined on primal vertices.
+ *         Algorithm based on the cs_cell_mesh_t structure.
+ *
+ *  \param[in]      cm       pointer to a cs_cell_mesh_t structure
+ *  \param[in]      array    pointer to the array of values
+ *
+ *  \return the value of the reconstruction at the cell center
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline cs_real_t
+cs_reco_cw_scalar_pv_at_cell_center(const cs_cell_mesh_t     *cm,
+                                    const cs_real_t          *array)
+{
+  /* Sanity checks */
+  assert(cm != NULL && array != NULL);
+  assert(cs_flag_test(cm->flag, CS_CDO_LOCAL_PV | CS_CDO_LOCAL_PVQ));
+
+  /* Reconstruct the value at the cell center */
+  cs_real_t  pc = 0.;
+  for (short int v = 0; v < cm->n_vc; v++)
+    pc += cm->wvc[v] * array[cm->v_ids[v]];
+
+  return pc;
+}
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
