@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -45,16 +45,18 @@
  *  Local headers
  *----------------------------------------------------------------------------*/
 
-#include <bft_mem.h>
-#include <bft_printf.h>
+#include "bft_mem.h"
+#include "bft_printf.h"
 
+#include "cs_blas.h"
+#include "cs_equation.h"
+#include "cs_field.h"
+#include "cs_log.h"
 #include "cs_math.h"
 #include "cs_mesh.h"
-#include "cs_post.h"
 #include "cs_mesh_location.h"
-#include "cs_field.h"
-#include "cs_cdo.h"
 #include "cs_param.h"
+#include "cs_post.h"
 #include "cs_reco.h"
 
 /*----------------------------------------------------------------------------
@@ -73,14 +75,252 @@ BEGIN_C_DECLS
 
 /* Redefined the name of functions from cs_math to get shorter names */
 #define _dp3 cs_math_3_dot_product
+#define CS_WALLDISTANCE_DBG 0
+
+/*============================================================================
+ * Private variables
+ *============================================================================*/
+
+static cs_equation_t  *cs_wd_poisson_eq = NULL;
 
 /*============================================================================
  * Private function prototypes
  *============================================================================*/
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute the wall distance for a vertex+cell-based scheme
+ *         Estimation based on a Poisson equation
+ *
+ * \param[in]      connect   pointer to a cs_cdo_connect_t structure
+ * \param[in]      cdoq      pointer to a cs_cdo_quantities_t structure
+ * \param[in]      field     pointer to a cs_field_t structure
+ * \param[in]      eq        pointer to the related cs_equation_t structure
+ * \param[in, out] dist      array storing the wall distance to compute
+ */
+/*----------------------------------------------------------------------------*/
+
+static void
+_compute_poisson_cdovcb(const cs_cdo_connect_t     *connect,
+                        const cs_cdo_quantities_t  *cdoq,
+                        const cs_field_t           *field,
+                        const cs_equation_t        *eq,
+                        cs_real_t                   dist[])
+{
+  CS_UNUSED(connect);
+  cs_real_3_t  *vtx_gradient = NULL;
+
+  BFT_MALLOC(vtx_gradient, cdoq->n_vertices, cs_real_3_t);
+
+  /* Perform the computation of the gradient at vertices using a cellwise
+     algorithm */
+  cs_equation_compute_vtx_field_gradient(eq, (cs_real_t *)vtx_gradient);
+
+#if defined(DEBUG) && !defined(NDEBUG) && CS_WALLDISTANCE_DBG > 1
+  /* Post-processing */
+  cs_post_write_vertex_var(CS_POST_MESH_VOLUME,
+                           CS_POST_WRITER_ALL_ASSOCIATED,
+                           "Poisson_Sol",
+                           1,               // dim
+                           true,            // interlace
+                           true,            // true = original mesh
+                           CS_POST_TYPE_cs_real_t,
+                           field->val,      // values on vertices
+                           NULL);           // time step management structure
+
+  cs_post_write_vertex_var(CS_POST_MESH_VOLUME,
+                           CS_POST_WRITER_ALL_ASSOCIATED,
+                           "GrdVtx",
+                           3,               // dim
+                           true,           // interlace
+                           true,            // true = original mesh
+                           CS_POST_TYPE_cs_real_t,
+                           vtx_gradient,             // values on vertices
+                           NULL);           // time step management structure
+#endif
+
+  /* Compute now wall distance at each vertex */
+  const cs_real_t  *var = field->val;
+  /* Compute now the wall distance at each vertex */
+  int  count = 0;
+
+# pragma omp parallel for if (cdoq->n_vertices > CS_THR_MIN)    \
+  reduction(+:count)
+  for (cs_lnum_t i = 0; i < cdoq->n_vertices; i++) {
+    cs_real_t  vgrd2 = _dp3(vtx_gradient[i], vtx_gradient[i]);
+    if (vgrd2 + 2*var[i] < 0)
+      count++;
+    cs_real_t  d1 = sqrt(vgrd2 + 2*fabs(var[i])), d2 = sqrt(vgrd2);
+    if (d1 - d2 < 0)
+      dist[i] =  d1 + d2;
+    else
+      dist[i] = d1 - d2;
+  }
+
+  if (count > 0) {
+    cs_base_warn(__FILE__, __LINE__);
+    bft_printf(" %d degree(s) of freedom have a negative value and have been"
+               " modified\n"
+               " This may result from a bad mesh quality.", count);
+  }
+
+  /* Post-processing */
+  cs_post_write_vertex_var(CS_POST_MESH_VOLUME,
+                           CS_POST_WRITER_ALL_ASSOCIATED,
+                           field->name,
+                           1,               // dim
+                           false,           // interlace
+                           true,            // true = original mesh
+                           CS_POST_TYPE_cs_real_t,
+                           dist,            // values on vertices
+                           NULL);           // time step management structure
+
+  /* Free memory */
+  BFT_FREE(vtx_gradient);
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Compute the wall distance for a vertex-based scheme
+ *         Estimation based on a Poisson equation
+ *
+ * \param[in]      connect   pointer to a cs_cdo_connect_t structure
+ * \param[in]      cdoq      pointer to a cs_cdo_quantities_t structure
+ * \param[in]      field     pointer to a cs_field_t structure
+ * \param[in, out] dist      array storing the wall distance to compute
+ */
+/*----------------------------------------------------------------------------*/
+
+static void
+_compute_poisson_cdovb(const cs_cdo_connect_t     *connect,
+                       const cs_cdo_quantities_t  *cdoq,
+                       const cs_field_t           *field,
+                       cs_real_t                   dist[])
+{
+  /* Initialize arrays */
+  cs_real_t  *dualcell_vol = NULL;
+  cs_real_3_t  *vtx_gradient = NULL;
+
+  BFT_MALLOC(vtx_gradient, cdoq->n_vertices, cs_real_3_t);
+  BFT_MALLOC(dualcell_vol, cdoq->n_vertices, cs_real_t);
+
+# pragma omp parallel for if (cdoq->n_vertices > CS_THR_MIN)
+  for (cs_lnum_t i = 0; i < cdoq->n_vertices; i++) {
+    vtx_gradient[i][0] = vtx_gradient[i][1] = vtx_gradient[i][2] = 0.;
+    dualcell_vol[i] = 0.;
+  }
+
+  /* Reconstruct gradient at vertices from gradient at cells */
+  const cs_adjacency_t  *c2v = connect->c2v;
+  const cs_real_t  *var = field->val;
+
+  for (cs_lnum_t  c_id = 0; c_id < cdoq->n_cells; c_id++) {
+
+    cs_real_3_t  cell_gradient;
+    cs_reco_grad_cell_from_pv(c_id, connect, cdoq, var, cell_gradient);
+
+    for (cs_lnum_t i = c2v->idx[c_id]; i < c2v->idx[c_id+1]; i++) {
+
+      cs_lnum_t  v_id = c2v->ids[i];
+
+      dualcell_vol[v_id] += cdoq->dcell_vol[i];
+      for (int k = 0; k < 3; k++)
+        vtx_gradient[v_id][k] += cdoq->dcell_vol[i]*cell_gradient[k];
+
+    } // Loop on cell vertices
+
+  } // Loop on cells
+
+  if (cs_glob_n_ranks > 1) {
+
+    cs_interface_set_sum(connect->interfaces[CS_CDO_CONNECT_VTX_SCAL],
+                         connect->n_vertices,
+                         1,
+                         true, // interlace
+                         CS_REAL_TYPE,
+                         dualcell_vol);
+
+    cs_interface_set_sum(connect->interfaces[CS_CDO_CONNECT_VTX_SCAL],
+                         connect->n_vertices,
+                         3,
+                         true, // interlace
+                         CS_REAL_TYPE,
+                         vtx_gradient);
+
+  }
+
+# pragma omp parallel for if (cdoq->n_vertices > CS_THR_MIN)
+  for (cs_lnum_t i = 0; i < cdoq->n_vertices; i++) {
+    cs_real_t  inv_dualcell_vol = 1/dualcell_vol[i];
+    for (int k = 0; k < 3; k++)
+      vtx_gradient[i][k] *= inv_dualcell_vol;
+  }
+
+#if defined(DEBUG) && !defined(NDEBUG) && CS_WALLDISTANCE_DBG > 1
+  /* Post-processing */
+  cs_post_write_vertex_var(CS_POST_MESH_VOLUME,
+                           CS_POST_WRITER_ALL_ASSOCIATED,
+                           "Poisson_Sol",
+                           1,               // dim
+                           true,            // interlace
+                           true,            // true = original mesh
+                           CS_POST_TYPE_cs_real_t,
+                           var,             // values on vertices
+                           NULL);           // time step management structure
+
+  cs_post_write_vertex_var(CS_POST_MESH_VOLUME,
+                           CS_POST_WRITER_ALL_ASSOCIATED,
+                           "GrdVtx",
+                           3,               // dim
+                           true,            // interlace
+                           true,            // true = original mesh
+                           CS_POST_TYPE_cs_real_t,
+                           vtx_gradient,    // values on vertices
+                           NULL);           // time step management structure
+#endif
+
+  /* Compute now the wall distance at each vertex */
+  int  count = 0;
+# pragma omp parallel for if (cdoq->n_vertices > CS_THR_MIN) \
+  reduction(+:count)
+  for (cs_lnum_t i = 0; i < cdoq->n_vertices; i++) {
+    cs_real_t  vgrd2 = _dp3(vtx_gradient[i], vtx_gradient[i]);
+    if (vgrd2 + 2*var[i] < 0)
+      count++;
+    cs_real_t  d1 = sqrt(vgrd2 + 2*fabs(var[i])), d2 = sqrt(vgrd2);
+    if (d1 - d2 < 0)
+      dist[i] =  d1 + d2;
+    else
+      dist[i] = d1 - d2;
+  }
+
+  if (count > 0) {
+    cs_base_warn(__FILE__, __LINE__);
+    bft_printf(" %d degree(s) of freedom have a negative value and have been"
+               " modified\n"
+               " This may result from a bad mesh quality.", count);
+  }
+
+  /* Post-processing */
+  cs_post_write_vertex_var(CS_POST_MESH_VOLUME,
+                           CS_POST_WRITER_ALL_ASSOCIATED,
+                           field->name,
+                           1,               // dim
+                           false,           // interlace
+                           true,            // true = original mesh
+                           CS_POST_TYPE_cs_real_t,
+                           dist,            // values on vertices
+                           NULL);           // time step management structure
+
+  /* Free memory */
+  BFT_FREE(dualcell_vol);
+  BFT_FREE(vtx_gradient);
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute the wall distance for a face-based scheme
+ *         Estimation based on a Poisson equation
  *
  * \param[in]      connect   pointer to a cs_cdo_connect_t structure
  * \param[in]      cdoq      pointer to a cs_cdo_quantities_t structure
@@ -91,11 +331,11 @@ BEGIN_C_DECLS
 /*----------------------------------------------------------------------------*/
 
 static void
-_compute_cdofb(const cs_cdo_connect_t     *connect,
-               const cs_cdo_quantities_t  *cdoq,
-               const cs_equation_t        *eq,
-               const cs_field_t           *field,
-               cs_real_t                   dist[])
+_compute_poisson_cdofb(const cs_cdo_connect_t     *connect,
+                       const cs_cdo_quantities_t  *cdoq,
+                       const cs_equation_t        *eq,
+                       const cs_field_t           *field,
+                       cs_real_t                   dist[])
 {
   cs_lnum_t  i, k;
 
@@ -110,8 +350,8 @@ _compute_cdofb(const cs_cdo_connect_t     *connect,
 
     for (i = connect->c2f->idx[c_id]; i < connect->c2f->idx[c_id+1]; i++) {
 
-      cs_lnum_t  f_id = connect->c2f->col_id[i];
-      cs_quant_t  fq = cdoq->face[f_id];
+      cs_lnum_t  f_id = connect->c2f->ids[i];
+      cs_nvec3_t  fq = cs_quant_set_face_nvec(f_id, cdoq);
       int  sgn = connect->c2f->sgn[i];
       cs_real_t  dualedge_contrib = fq.meas*sgn*(f_var[f_id] - c_var[c_id]);
 
@@ -131,7 +371,8 @@ _compute_cdofb(const cs_cdo_connect_t     *connect,
 
   } // Loop on cells
 
-  cs_post_write_var(-1,              // id du maillage de post
+  cs_post_write_var(CS_POST_MESH_VOLUME,
+                    CS_POST_WRITER_ALL_ASSOCIATED,
                     field->name,
                     1,               // dim
                     false,           // interlace
@@ -142,112 +383,6 @@ _compute_cdofb(const cs_cdo_connect_t     *connect,
                     NULL,            // values at border faces
                     NULL);           // time step management structure
 
-  cs_data_info_t  dinfo = cs_analysis_data(cdoq->n_cells, // n_elts
-                                           1,             // stride
-                                           CS_DOUBLE,     // cs_datatype_t
-                                           dist,          // data
-                                           false);        // absolute values ?
-
-  bft_printf("\n -bnd- WallDistance.Max   % 10.6e\n", dinfo.max.value);
-  bft_printf(" -bnd- WallDistance.Mean  % 10.6e\n", dinfo.mean);
-  bft_printf(" -bnd- WallDistance.Sigma % 10.6e\n\n", dinfo.sigma);
-  bft_printf("%s", msepline);
-}
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Compute the wall distance for a vertex-based scheme
- *
- * \param[in]      connect   pointer to a cs_cdo_connect_t structure
- * \param[in]      cdoq      pointer to a cs_cdo_quantities_t structure
- * \param[in]      field     pointer to a cs_field_t structure
- * \param[in, out] dist      array storing the wall distance to compute
- */
-/*----------------------------------------------------------------------------*/
-
-static void
-_compute_cdovb(const cs_cdo_connect_t     *connect,
-               const cs_cdo_quantities_t  *cdoq,
-               const cs_field_t           *field,
-               cs_real_t                   dist[])
-{
-  cs_real_t  *gdi = NULL, *dualcell_vol = NULL, *cell_gradient = NULL;
-  cs_real_3_t  *vtx_gradient = NULL;
-
-  const cs_connect_index_t  *c2v = connect->c2v;
-  const cs_real_t  *var = field->val;
-
-  /* Compute a discrete gradient of var along each edge */
-  cs_sla_matvec(connect->e2v, var, &gdi, true);
-
-  /* Reconstruct a vector field at each cell center associated to gdi */
-  cs_reco_ccen_edge_dofs(connect, cdoq, gdi, &cell_gradient);
-
-  /* Reconstruct gradient at vertices from gradient at cells */
-  BFT_MALLOC(vtx_gradient, cdoq->n_vertices, cs_real_3_t);
-  BFT_MALLOC(dualcell_vol, cdoq->n_vertices, cs_real_t);
-# pragma omp parallel for if (cdoq->n_vertices > CS_THR_MIN)
-  for (cs_lnum_t i = 0; i < cdoq->n_vertices; i++) {
-    vtx_gradient[i][0] = vtx_gradient[i][1] = vtx_gradient[i][2] = 0.;
-    dualcell_vol[i] = 0.;
-  }
-
-  for (cs_lnum_t  c_id = 0; c_id < cdoq->n_cells; c_id++) {
-
-    cs_lnum_t  cshift = 3*c_id;
-
-    for (cs_lnum_t i = c2v->idx[c_id]; i < c2v->idx[c_id+1]; i++) {
-
-      cs_lnum_t  v_id = c2v->ids[i];
-
-      dualcell_vol[v_id] += cdoq->dcell_vol[i];
-      for (int k = 0; k < 3; k++)
-        vtx_gradient[v_id][k] += cdoq->dcell_vol[i]*cell_gradient[cshift+k];
-
-    } // Loop on cell vertices
-
-  } // Loop on cells
-
-# pragma omp parallel for if (cdoq->n_vertices > CS_THR_MIN)
-  for (cs_lnum_t i = 0; i < cdoq->n_vertices; i++) {
-    cs_real_t  inv_dualcell_vol = 1/dualcell_vol[i];
-    for (int k = 0; k < 3; k++)
-      vtx_gradient[i][k] *= inv_dualcell_vol;
-  }
-
-  /* Compute now wall distance at each vertex */
-# pragma omp parallel for if (cdoq->n_vertices > CS_THR_MIN)
-  for (cs_lnum_t i = 0; i < cdoq->n_vertices; i++) {
-    cs_real_t  tmp = _dp3(vtx_gradient[i], vtx_gradient[i]) + 2*var[i];
-    assert(tmp >= 0); // Sanity check
-    dist[i] = sqrt(tmp) - cs_math_3_norm(vtx_gradient[i]);
-  }
-
-  /* Post-processing */
-  cs_post_write_vertex_var(-1,              // id du maillage de post
-                           field->name,
-                           1,               // dim
-                           false,           // interlace
-                           true,            // true = original mesh
-                           CS_POST_TYPE_cs_real_t,
-                           dist,            // values on vertices
-                           NULL);           // time step management structure
-
-  cs_data_info_t  dinfo = cs_analysis_data(cdoq->n_vertices, // n_elts
-                                           1,                // stride
-                                           CS_DOUBLE,        // cs_datatype_t
-                                           dist,             // data
-                                           false);           // abs. values ?
-
-  bft_printf("\n -bnd- WallDistance.Max   % 10.6e\n", dinfo.max.value);
-  bft_printf(" -bnd- WallDistance.Mean  % 10.6e\n", dinfo.mean);
-  bft_printf(" -bnd- WallDistance.Sigma % 10.6e\n", dinfo.sigma);
-
-  /* Free memory */
-  BFT_FREE(gdi);
-  BFT_FREE(dualcell_vol);
-  BFT_FREE(cell_gradient);
-  BFT_FREE(vtx_gradient);
 }
 
 /*============================================================================
@@ -256,51 +391,119 @@ _compute_cdovb(const cs_cdo_connect_t     *connect,
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Setup an new equation related to the wall distance
+ * \brief  Test if the computation of the wall distance is activated
  *
- * \param[in]  eq          pointer to the associated cs_equation_t structure
- * \param[in]  diff_pty    pointer to a cs_property_t structure
- * \param[in]  wall_ml_id  id of the mesh location related to wall boundaries
+ * \return true if the wall distance computation is requested, false otherwise
+ */
+/*----------------------------------------------------------------------------*/
+
+bool
+cs_walldistance_is_activated(void)
+{
+  if (cs_wd_poisson_eq != NULL)
+    return true;
+  else
+    return false;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Activate the future computation of the wall distance
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_walldistance_setup(cs_equation_t   *eq,
-                      cs_property_t   *diff_pty,
-                      int              wall_ml_id)
+cs_walldistance_activate(void)
 {
   /* Sanity check */
-  assert(!strcmp(cs_equation_get_name(eq), "WallDistance"));
+  assert(cs_wd_poisson_eq == NULL);
 
-  /* Unity is a material property defined by default */
-  cs_equation_link(eq, "diffusion", diff_pty);
+  cs_equation_t  *eq =
+    cs_equation_add("WallDistance",              // equation name
+                    "WallDistance",              // variable name
+                    CS_EQUATION_TYPE_PREDEFINED, // type of the equation
+                    1,                           // dimension of the variable
+                    CS_PARAM_BC_HMG_NEUMANN);    // default BC
 
-  /* Add boundary conditions */
-  cs_equation_add_bc(eq,
-                     cs_mesh_location_get_name(wall_ml_id),
-                     "dirichlet",  // type of boundary condition
-                     "value",      // type of definition
-                     "0.0");       // value to set
-
-  /* Add source term */
-  cs_equation_add_source_term_by_val(eq,
-                                     "WallDist.st",   // label
-                                     "cells",         // mesh location name
-                                     "1.0");          // value to set
+  /* Set now the default numerical parameters for this equation */
+  cs_equation_param_t  *eqp = cs_equation_get_param(eq);
 
   /* Enforcement of the Dirichlet boundary conditions */
-  cs_equation_set_param(eq, CS_EQKEY_BC_ENFORCEMENT, "penalization");
+  cs_equation_set_param(eqp, CS_EQKEY_BC_ENFORCEMENT, "penalization");
 
   /* System to solve is SPD by construction */
-  cs_equation_set_param(eq, CS_EQKEY_ITSOL, "cg");
+  cs_equation_set_param(eqp, CS_EQKEY_ITSOL, "cg");
 
 #if defined(HAVE_PETSC)  /* Modify the default settings */
-  cs_equation_set_param(eq, CS_EQKEY_SOLVER_FAMILY, "petsc");
-  cs_equation_set_param(eq, CS_EQKEY_PRECOND, "amg");
+  cs_equation_set_param(eqp, CS_EQKEY_SOLVER_FAMILY, "petsc");
+  cs_equation_set_param(eqp, CS_EQKEY_PRECOND, "amg");
 #else
-  cs_equation_set_param(eq, CS_EQKEY_PRECOND, "jacobi");
+  cs_equation_set_param(eqp, CS_EQKEY_PRECOND, "jacobi");
 #endif
 
+  /* Set the static pointer */
+  cs_wd_poisson_eq = eq;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Setup the equation related to the wall distance
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_walldistance_setup(void)
+{
+  cs_equation_t  *eq = cs_wd_poisson_eq;
+
+  if (cs_wd_poisson_eq == NULL)
+    bft_error(__FILE__, __LINE__, 0,
+              " %s: Stop setting the wall distance equation.\n"
+              " The wall distance computation has not been activated.",
+              __func__);
+
+  cs_equation_param_t  *eqp = cs_equation_get_param(eq);
+
+  /* Unity is a property defined by default */
+  cs_equation_add_diffusion(eqp, cs_property_by_name("unity"));
+
+  /* Add boundary conditions */
+  cs_real_t  zero_value = 0.;
+  const char  bc_zone_name[] = "domain_walls";
+
+  cs_equation_add_bc_by_value(eqp,
+                              CS_PARAM_BC_DIRICHLET,
+                              bc_zone_name,
+                              &zero_value);
+
+  /* Add source term */
+  const char *st_zone_name =
+    cs_mesh_location_get_name(CS_MESH_LOCATION_CELLS);
+  cs_real_t  unity = 1.0;
+
+  cs_equation_add_source_term_by_val(eqp,
+                                     st_zone_name,   // zone name
+                                     &unity);        // value to set
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Finalize the setup stage for the equation related to the wall
+ *         distance. Only useful for Hamilton-Jacobi equation
+ *
+ * \param[in]      connect    pointer to a cs_cdo_connect_t structure
+ * \param[in]      cdoq       pointer to a cs_cdo_quantities_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_walldistance_finalize_setup(const cs_cdo_connect_t       *connect,
+                               const cs_cdo_quantities_t    *cdoq)
+{
+  CS_UNUSED(connect);
+  CS_UNUSED(cdoq);
+
+  return;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -312,8 +515,6 @@ cs_walldistance_setup(cs_equation_t   *eq,
  * \param[in]      dt_cur     current value of the time step
  * \param[in]      connect    pointer to a cs_cdo_connect_t structure
  * \param[in]      cdoq       pointer to a cs_cdo_quantities_t structure
- * \param[in]      do_logcvg  output information on convergence or not
- * \param[in, out] eq         pointer to the related cs_equation_t structure
  */
 /*----------------------------------------------------------------------------*/
 
@@ -322,38 +523,32 @@ cs_walldistance_compute(const cs_mesh_t              *mesh,
                         const cs_time_step_t         *time_step,
                         double                        dt_cur,
                         const cs_cdo_connect_t       *connect,
-                        const cs_cdo_quantities_t    *cdoq,
-                        bool                          do_logcvg,
-                        cs_equation_t                *eq)
+                        const cs_cdo_quantities_t    *cdoq)
 {
   /* First step:
      Solve the equation related to the definition of the wall distance. */
 
+  cs_equation_t  *eq = cs_wd_poisson_eq;
+
   /* Sanity check */
   assert(cs_equation_is_steady(eq));
 
-  /* Initialize system before resolution for all equations
-     - create system builder
-     - initialize field according to initial conditions
-     - initialize source term */
-  cs_equation_init_system(mesh, eq);
-
   /* Define the algebraic system */
   cs_equation_build_system(mesh, time_step, dt_cur, eq);
 
   /* Solve the algebraic system */
-  cs_equation_solve(eq, do_logcvg);
+  cs_equation_solve(eq);
 
   /* Second step:
      Compute the wall distance. */
 
-  cs_field_t  *field = cs_equation_get_field(eq);
+  cs_field_t  *field_p = cs_equation_get_field(eq);
 
-  const cs_lnum_t *n_elts = cs_mesh_location_get_n_elts(field->location_id);
+  const cs_lnum_t *n_elts = cs_mesh_location_get_n_elts(field_p->location_id);
 
   /* Sanity checks */
-  assert(field->is_owner);
-  assert(field->dim == 1);
+  assert(field_p->is_owner);
+  assert(field_p->dim == 1);
 
   /* Initialize dist array */
   cs_real_t  *dist = NULL;
@@ -362,23 +557,22 @@ cs_walldistance_compute(const cs_mesh_t              *mesh,
   for (cs_lnum_t i = 0; i < n_elts[0]; i++)
     dist[i] = 0;
 
-  cs_space_scheme_t  space_scheme = cs_equation_get_space_scheme(eq);
+  cs_param_space_scheme_t  space_scheme = cs_equation_get_space_scheme(eq);
   switch (space_scheme) {
 
   case CS_SPACE_SCHEME_CDOVB:
     assert(n_elts[0] == cdoq->n_vertices);
-    _compute_cdovb(connect, cdoq, field, dist);
+    _compute_poisson_cdovb(connect, cdoq, field_p, dist);
     break;
 
   case CS_SPACE_SCHEME_CDOFB:
     assert(n_elts[0] == cdoq->n_cells);
-    _compute_cdofb(connect, cdoq, eq, field, dist);
+    _compute_poisson_cdofb(connect, cdoq, eq, field_p, dist);
     break;
 
   case CS_SPACE_SCHEME_CDOVCB:
-    bft_error(__FILE__, __LINE__, 0,
-              " CDO Vertex+Cell-based is not yet implemented to compute"
-              " the wall distance.");
+    assert(n_elts[0] == cdoq->n_vertices);
+    _compute_poisson_cdovcb(connect, cdoq, field_p, eq, dist);
     break;
 
   default:
@@ -389,7 +583,7 @@ cs_walldistance_compute(const cs_mesh_t              *mesh,
   /* Replace field values by dist */
 # pragma omp parallel for if (n_elts[0] > CS_THR_MIN)
   for (cs_lnum_t i = 0; i < n_elts[0]; i++)
-    field->val[i] = dist[i];
+    field_p->val[i] = dist[i];
 
   /* Free memory */
   BFT_FREE(dist);
