@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -52,18 +52,44 @@ BEGIN_C_DECLS
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Setup an new equation related to the wall distance
- *
- * \param[in]  eq          pointer to the associated cs_equation_t structure
- * \param[in]  diff_pty    pointer to a cs_property_t structure
- * \param[in]  wall_ml_id  id of the mesh location related to wall boundaries
+ * \brief  Test if the computation of the wall distance is activated
+ */
+/*----------------------------------------------------------------------------*/
+
+bool
+cs_walldistance_is_activated(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Activate the future computation of the wall distance
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_walldistance_setup(cs_equation_t   *eq,
-                      cs_property_t   *diff_pty,
-                      int              wall_ml_id);
+cs_walldistance_activate(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Setup the equation related to the wall distance
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_walldistance_setup(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Finalize the setup stage for the equation related to the wall
+ *         distance. Only useful for Hamilton-Jacobi equation
+ *
+ * \param[in]      connect    pointer to a cs_cdo_connect_t structure
+ * \param[in]      cdoq       pointer to a cs_cdo_quantities_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_walldistance_finalize_setup(const cs_cdo_connect_t       *connect,
+                               const cs_cdo_quantities_t    *cdoq);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -74,8 +100,6 @@ cs_walldistance_setup(cs_equation_t   *eq,
  * \param[in]      dt_cur     current value of the time step
  * \param[in]      connect    pointer to a cs_cdo_connect_t structure
  * \param[in]      cdoq       pointer to a cs_cdo_quantities_t structure
- * \param[in]      do_logcvg  output information on convergence or not
- * \param[in, out] eq         pointer to the related cs_equation_t structure
  */
 /*----------------------------------------------------------------------------*/
 
@@ -84,9 +108,7 @@ cs_walldistance_compute(const cs_mesh_t              *mesh,
                         const cs_time_step_t         *time_step,
                         double                        dt_cur,
                         const cs_cdo_connect_t       *connect,
-                        const cs_cdo_quantities_t    *cdoq,
-                        bool                          do_logcvg,
-                        cs_equation_t                *eq);
+                        const cs_cdo_quantities_t    *cdoq);
 
 /*----------------------------------------------------------------------------*/
 
