@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -23,7 +23,7 @@
 subroutine cfmsfp &
 !================
 
- ( nvar   , nscal  , iterns , ncepdp , ncesmp ,                   &
+ ( nvar   , nscal  , idtcfl , iterns , ncepdp , ncesmp ,          &
    icepdc , icetsm , itypsm ,                                     &
    dt     , vela   ,                                              &
    ckupdc , smacel ,                                              &
@@ -43,6 +43,7 @@ subroutine cfmsfp &
 !__________________!____!_____!________________________________________________!
 ! nvar             ! i  ! <-- ! total number of variables                      !
 ! nscal            ! i  ! <-- ! total number of scalars                        !
+! idtcfl           ! i  ! <-- ! flux used in CFL mass (1:true, 0:false)        !
 ! iterns           ! i  ! <-- ! Navier-Stokes iteration number                 !
 ! ncepdp           ! i  ! <-- ! number of cells with head loss                 !
 ! ncesmp           ! i  ! <-- ! number of cells with mass source term          !
@@ -93,7 +94,7 @@ implicit none
 ! Arguments
 
 integer          nvar   , nscal, iterns
-integer          ncepdp , ncesmp
+integer          ncepdp , ncesmp, idtcfl
 
 integer          icepdc(ncepdp)
 integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
@@ -105,7 +106,6 @@ double precision vela  (3  ,ncelet)
 
 ! Local variables
 
-integer          ivar
 integer          ifac  , iel, ischcp, idftnp, ircflp
 integer          init  , inc   , iccocg, isstpp
 integer          nswrgp, imligp, iwarnp, iconvp, idiffp
@@ -117,8 +117,7 @@ integer          itsqdm, imasac
 double precision epsrgp, climgp, extrap, thetap, blencp, relaxp
 double precision rom
 
-double precision, allocatable, dimension(:) :: w1, w8, w9
-double precision, allocatable, dimension(:) :: w10, w11, w12
+double precision, allocatable, dimension(:) :: w1
 double precision, dimension(:), pointer :: crom, brom
 double precision, dimension(:), pointer :: viscl, visct
 double precision, dimension(:,:), allocatable :: tsexp, gavinj
@@ -133,6 +132,9 @@ double precision, allocatable, dimension(:,:,:) :: coefbv
 double precision, dimension(:,:), pointer :: coefau, cofafu
 double precision, dimension(:,:,:), pointer :: coefbu, cofbfu
 
+type(var_cal_opt) :: vcopt_u, vcopt_p
+
+double precision rvoid(1)
 !===============================================================================
 
 !===============================================================================
@@ -145,17 +147,19 @@ call field_get_coefaf_v(ivarfl(iu), cofafu)
 call field_get_coefbf_v(ivarfl(iu), cofbfu)
 
 ! Allocate work arrays
-allocate(w1(ncelet), w8(ncelet), w9(ncelet))
-allocate(w10(ncelet), w11(ncelet), w12(ncelet))
+allocate(w1(ncelet))
 allocate(tsexp(3,ncelet))
 allocate(gavinj(3,ncelet))
 allocate(tsimp(3,3,ncelet))
 allocate(coefbv(3,3,nfabor))
 allocate(vel0(3,ncelet))
 
-if (idften(iu).eq.1) then
+call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt_u)
+call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt_p)
+
+if (iand(vcopt_u%idften, ISOTROPIC_DIFFUSION).ne.0) then
   allocate(viscf(1, 1, nfac), viscb(nfabor))
-else if (idften(iu).eq.6) then
+else if (iand(vcopt_u%idften, ANISOTROPIC_LEFT_DIFFUSION).ne.0) then
   allocate(viscf(3, 3, nfac), viscb(nfabor))
   allocate(viscce(6,ncelet))
 endif
@@ -166,10 +170,6 @@ endif
 
 f_id0 = -1
 
-! --- Number of the computational variable
-!     Pressure
-ivar     = ipr
-
 ! Density
 
 call field_get_val_s(icrom, crom)
@@ -201,16 +201,6 @@ call field_get_val_s(ibrom, brom)
 !     (Without these terms we are in the configuration Euler + gravity)
 
 ! --- Initialization
-do iel = 1, ncel
-  w10(iel) = 0.d0
-enddo
-do iel = 1, ncel
-  w11(iel) = 0.d0
-enddo
-do iel = 1, ncel
-  w12(iel) = 0.d0
-enddo
-
 do iel = 1, ncel
   do isou = 1, 3
     tsexp(isou,iel) = 0.d0
@@ -236,20 +226,20 @@ if (itsqdm.ne.0) then
 
   ! Convective of the momentum equation
   ! in upwind and without reconstruction
-  iconvp = iconv(iu)
+  iconvp = vcopt_u%iconv
 
   init   = 1
   inc    = 1
   iccocg = 1
   iflmb0 = 1
-  nswrgp = nswrgr(iu)
-  imligp = imligr(iu)
-  iwarnp = iwarni(iu)
-  epsrgp = epsrgr(iu)
-  climgp = climgr(iu)
-  extrap = extrag(iu)
-  relaxp = relaxv(iu)
-  thetap = thetav(iu)
+  nswrgp = vcopt_u%nswrgr
+  imligp = vcopt_u%imligr
+  iwarnp = vcopt_u%iwarni
+  epsrgp = vcopt_u%epsrgr
+  climgp = vcopt_u%climgr
+  extrap = vcopt_u%extrag
+  relaxp = vcopt_u%relaxv
+  thetap = vcopt_u%thetav
 
   itypfl = 1
 
@@ -266,11 +256,11 @@ if (itsqdm.ne.0) then
   flumas , flumab )
 
   ! ---> Face diffusivity for the velocity
-  idiffp = idiff(iu)
+  idiffp = vcopt_u%idiff
   if (idiffp.ge. 1) then
 
-     call field_get_val_s(iprpfl(iviscl), viscl)
-     call field_get_val_s(iprpfl(ivisct), visct)
+     call field_get_val_s(iviscl, viscl)
+     call field_get_val_s(ivisct, visct)
 
      if (itytur.eq.3) then
         do iel = 1, ncel
@@ -278,12 +268,12 @@ if (itsqdm.ne.0) then
         enddo
      else
         do iel = 1, ncel
-           w1(iel) = viscl(iel) + idifft(iu)*visct(iel)
+           w1(iel) = viscl(iel) + vcopt_u%idifft*visct(iel)
         enddo
      endif
 
      ! Scalar diffusivity (Default)
-     if (idften(iu).eq.1) then
+     if (iand(vcopt_u%idften, ISOTROPIC_DIFFUSION).ne.0) then
 
         call viscfa &
         !==========
@@ -292,7 +282,7 @@ if (itsqdm.ne.0) then
        viscf  , viscb  )
 
      ! Tensorial diffusion of the velocity (in case of tensorial porosity)
-     else if (idften(iu).eq.6) then
+     else if (iand(vcopt_u%idften, ANISOTROPIC_LEFT_DIFFUSION).ne.0) then
 
         do iel = 1, ncel
           do isou = 1, 3
@@ -327,7 +317,7 @@ if (itsqdm.ne.0) then
 
   endif
 
-  idftnp = idften(iu)
+  idftnp = vcopt_u%idften
 
   ! no recontruction
   ircflp = 0;
@@ -349,14 +339,15 @@ if (itsqdm.ne.0) then
 
   call bilscv &
   !==========
-( idtvar , iu     , iconvp , idiffp , nswrgp , imligp , ircflp , &
-  ischcp , isstpp , inc    , imrgra , ivisse ,                   &
-  iwarnp , idftnp , imasac ,                                     &
-  blencp , epsrgp , climgp , relaxp , thetap ,                   &
-  vela   , vela   ,                                              &
-  coefau , coefbu , cofafu , cofbfu ,                            &
-  flumas , flumab , viscf  , viscb  , secvif , secvib ,          &
-  icvflb , icvfli ,                                              &
+( idtvar , ivarfl(iu)      , iconvp , idiffp , nswrgp , imligp , ircflp , &
+  ischcp , isstpp , inc    , imrgra , ivisse ,                            &
+  iwarnp , idftnp , imasac ,                                              &
+  blencp , epsrgp , climgp , relaxp , thetap ,                            &
+  vela   , vela   ,                                                       &
+  coefau , coefbu , cofafu , cofbfu ,                                     &
+  flumas , flumab , viscf  , viscb  , secvif , secvib ,                   &
+  rvoid  , rvoid  , rvoid  ,                                              &
+  icvflb , icvfli ,                                                       &
   tsexp  )
 
 endif
@@ -404,16 +395,23 @@ enddo
 
 do iel = 1, ncel
   do isou = 1, 3
-    tsexp(isou,iel) = vela(isou,iel) + dt(iel)*tsexp(isou,iel)
+    tsexp(isou,iel) = dt(iel)*tsexp(isou,iel)
   enddo
 enddo
 
+if (idtcfl.eq.1) then
+  do iel = 1, ncel
+    do isou = 1, 3
+      tsexp(isou,iel) = vela(isou,iel) + tsexp(isou,iel)
+    enddo
+  enddo
+endif
+
 ! Computation of the flux
 
+! volumic flux part based on dt*f^n
 ! In order to avoid a misfit boundary condition, we impose a homogeneous
-! Neumann condition. Note that it is only useful for gradient
-! reconstruction. The boundary value does not matter since the flux
-! is updated afterwards.
+! Neumann condition.
 
 ! Initialization of the mass flux
 init   = 1
@@ -422,12 +420,12 @@ inc    = 0
 iccocg = 1
 iflmb0 = 1
 ! Reconstruction is useless here
-nswrgp = 0
-imligp = imligr(ivar)
-iwarnp = iwarni(ivar)
-epsrgp = epsrgr(ivar)
-climgp = climgr(ivar)
-extrap = extrag(ivar)
+nswrgp = 0 ! FIXME
+imligp = vcopt_p%imligr
+iwarnp = vcopt_p%iwarni
+epsrgp = vcopt_p%epsrgr
+climgp = vcopt_p%climgr
+extrap = vcopt_p%extrag
 
 ! Velocity flux (crom, brom not used)
 itypfl = 0
@@ -455,9 +453,26 @@ call inimav                                                      &
   coefau , coefbv ,                                              &
   flumas , flumab )
 
+if (idtcfl.eq.0) then
+  ! volumic flux part based on velocity u^n
+  init   = 0
+  ! take into account Dirichlet velocity boundary conditions
+  inc    = 1
+
+  call inimav                                                      &
+  !==========
+  ( ivarfl(iu)      , itypfl ,                                     &
+    iflmb0 , init   , inc    , imrgra , nswrgp , imligp ,          &
+    iwarnp ,                                                       &
+    epsrgp , climgp ,                                              &
+    crom, brom,                                                    &
+    vela,                                                          &
+    coefau , coefbu ,                                              &
+    flumas , flumab )
+endif
+
 ! Free memory
-deallocate(w1, w8, w9)
-deallocate(w10, w11, w12)
+deallocate(w1)
 deallocate(tsexp)
 deallocate(gavinj)
 deallocate(tsimp)
