@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -98,7 +98,6 @@ double precision vela  (3  ,ncelet)
 ! Local variables
 
 character(len=80) :: chaine
-integer          ivar
 integer          ifac  , iel
 integer          init  , inc   , iccocg, ii, jj
 integer          iflmas, iflmab
@@ -112,7 +111,8 @@ double precision epsrsp
 double precision sclnor
 
 integer          imucpp, idftnp, iswdyp
-integer          imvis1, f_id0
+integer          imvis1, f_id0, idtcfl
+integer          f_id
 
 double precision thetv, relaxp, hint
 
@@ -123,7 +123,7 @@ double precision, allocatable, dimension(:) :: smbrs, rovsdt
 double precision, allocatable, dimension(:) :: w1
 double precision, allocatable, dimension(:) :: w7, w8, w9
 double precision, allocatable, dimension(:) :: w10
-double precision, allocatable, dimension(:) :: wflmas, wflmab
+double precision, allocatable, dimension(:) :: wflmas, wflmab, ivolfl, bvolfl
 double precision, allocatable, dimension(:) :: wbfa, wbfb
 double precision, allocatable, dimension(:) :: dpvar
 
@@ -131,8 +131,13 @@ double precision, allocatable, dimension(:) :: c2
 
 double precision, dimension(:), pointer :: coefaf_p, coefbf_p
 double precision, dimension(:), pointer :: imasfl, bmasfl
-double precision, dimension(:), pointer :: rhopre, crom
+double precision, dimension(:), pointer :: rhopre, crom, brom
 double precision, dimension(:), pointer :: cvar_pr, cvara_pr, cpro_cp, cpro_cv
+double precision, dimension(:,:), pointer :: coefau
+double precision, dimension(:,:,:), pointer :: coefbu
+double precision, dimension(:), pointer :: cpro_divq
+
+type(var_cal_opt) :: vcopt_p
 
 !===============================================================================
 !===============================================================================
@@ -143,6 +148,7 @@ double precision, dimension(:), pointer :: cvar_pr, cvara_pr, cpro_cp, cpro_cv
 allocate(viscf(nfac), viscb(nfabor))
 allocate(smbrs(ncelet), rovsdt(ncelet))
 allocate(wflmas(nfac), wflmab(nfabor))
+allocate(ivolfl(nfac), bvolfl(nfabor))
 
 allocate(wbfa(nfabor), wbfb(nfabor))
 
@@ -152,9 +158,11 @@ allocate(w7(ncelet), w8(ncelet), w9(ncelet))
 allocate(w10(ncelet))
 allocate(dpvar(ncelet))
 
+call field_get_coefa_v(ivarfl(iu), coefau)
+call field_get_coefb_v(ivarfl(iu), coefbu)
+
 ! --- Number of computational variable and post for pressure
 
-ivar   = ipr
 f_id0 = -1
 
 ! --- Mass flux associated to energy
@@ -164,6 +172,7 @@ call field_get_val_s(iflmas, imasfl)
 call field_get_val_s(iflmab, bmasfl)
 
 call field_get_val_s(icrom, crom)
+call field_get_val_s(ibrom, brom)
 call field_get_val_prev_s(icrom, rhopre)
 
 call field_get_val_s(ivarfl(ipr), cvar_pr)
@@ -171,21 +180,23 @@ call field_get_val_prev_s(ivarfl(ipr), cvara_pr)
 
 call field_get_label(ivarfl(ipr), chaine)
 
-if(iwarni(ivar).ge.1) then
+call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt_p)
+
+if(vcopt_p%iwarni.ge.1) then
   write(nfecra,1000) chaine(1:8)
 endif
 
 call field_get_coefaf_s(ivarfl(ipr), coefaf_p)
 call field_get_coefbf_s(ivarfl(ipr), coefbf_p)
 
-if (icp.gt.0) then
-  call field_get_val_s(iprpfl(icp), cpro_cp)
+if (icp.ge.0) then
+  call field_get_val_s(icp, cpro_cp)
 else
   cpro_cp => rvoid1
 endif
 
-if (icv.gt.0) then
-  call field_get_val_s(iprpfl(icv), cpro_cv)
+if (icv.ge.0) then
+  call field_get_val_s(icv, cpro_cv)
 else
   cpro_cv => rvoid1
 endif
@@ -236,67 +247,73 @@ endif
 allocate(c2(ncelet))
 
 call cs_cf_thermo_c_square(cpro_cp, cpro_cv, cvar_pr, crom, c2, ncel)
-!=========================
 
 do iel = 1, ncel
-  rovsdt(iel) = rovsdt(iel) + istat(ivar)*(cell_f_vol(iel)/(dt(iel)*c2(iel)))
+  rovsdt(iel) = rovsdt(iel) + vcopt_p%istat*(cell_f_vol(iel)/(dt(iel)*c2(iel)))
 enddo
 
 !===============================================================================
 ! 3. "MASS FLUX" AND FACE "VISCOSITY" CALCULATION
 !===============================================================================
 
-! Computation of the "convective flux" for the density:
-! (u + dt f) is computed at internal faces and stored in wflmas,
-! the values at boundary faces in wflmab won't be taken into account.
+! computation of the "convective flux" for the density
 
-call cfmsfp                                                                     &
-!==========
-( nvar   , nscal  , iterns , ncepdp , ncesmp ,                                  &
-  icepdc , icetsm , itypsm ,                                                    &
-  dt     , vela   ,                                                             &
-  ckupdc , smacel ,                                                             &
-  wflmas , wflmab )
+! volumic flux (u + dt f)
+idtcfl = 0
+call cfmsfp                                                                    &
+( nvar   , nscal  , idtcfl , iterns , ncepdp , ncesmp ,                        &
+  icepdc , icetsm , itypsm ,                                                   &
+  dt     , vela   ,                                                            &
+  ckupdc , smacel ,                                                            &
+  ivolfl , bvolfl )
 
-! Mass flux at internal faces (upwind scheme for the density).
+! mass flux at internal faces (upwind scheme for the density)
+! (negative because added to RHS)
 do ifac = 1, nfac
   ii = ifacel(1,ifac)
   jj = ifacel(2,ifac)
-  wflmas(ifac) = -0.5d0*                                                        &
-                 ( crom(ii)*(wflmas(ifac)+abs(wflmas(ifac)))                    &
-                 + crom(jj)*(wflmas(ifac)-abs(wflmas(ifac))))
+  wflmas(ifac) = -0.5d0*                                                       &
+                 ( crom(ii)*(ivolfl(ifac)+abs(ivolfl(ifac)))                   &
+                 + crom(jj)*(ivolfl(ifac)-abs(ivolfl(ifac))))
+enddo
+
+! mass flux at boundary faces
+! (negative because added to RHS)
+do ifac = 1, nfabor
+  iel = ifabor(ifac)
+  wflmab(ifac) = -brom(ifac)*bvolfl(ifac)
 enddo
 
-! Mass flux at boundary faces.
 if (icfgrp.eq.1) then
   ! The hydrostatic pressure gradient contribution has to be added to the mass
   ! flux if it has been taken into account in the pressure B.C. at walls.
   do ifac = 1, nfabor
     iel = ifabor(ifac)
     if (itypfb(ifac).eq.iparoi) then
-      wflmab(ifac) = -bmasfl(ifac)                                              &
-                     -dt(iel)*crom(iel)*(  gx*surfbo(1,ifac)                    &
-                                         + gy*surfbo(2,ifac)                    &
+      wflmab(ifac) = wflmab(ifac)                                              &
+                     -dt(iel)*crom(iel)*(  gx*surfbo(1,ifac)                   &
+                                         + gy*surfbo(2,ifac)                   &
                                          + gz*surfbo(3,ifac))
-    else
-      wflmab(ifac) = -bmasfl(ifac)
     endif
   enddo
-else
-  do ifac = 1, nfabor
-    iel = ifabor(ifac)
-    wflmab(ifac) = -bmasfl(ifac)
-  enddo
 endif
 
 init = 0
 call divmas(init,wflmas,wflmab,smbrs)
 
-! (Delta t)_ij is calculated as the "viscocity" associated to the pressure
+
+call field_get_id_try("predicted_vel_divergence", f_id)
+if (f_id.ge.0) then
+  call field_get_val_s(f_id, cpro_divq)
+  do iel = 1, ncel
+    cpro_divq(iel) = smbrs(iel)
+  enddo
+endif
+
+! (Delta t)_ij is calculated as the "viscosity" associated to the pressure
 imvis1 = 1
 
 call viscfa                                                                    &
-!==========
 ( imvis1 ,                                                                     &
   dt     ,                                                                     &
   viscf  , viscb  )
@@ -305,34 +322,33 @@ call viscfa                                                                    &
 ! 4. SOLVING
 !===============================================================================
 
-istatp = istat (ivar)
-iconvp = iconv (ivar)
-idiffp = idiff (ivar)
-ndircp = ndircl(ivar)
-nswrsp = nswrsm(ivar)
-nswrgp = nswrgr(ivar)
-imligp = imligr(ivar)
-ircflp = ircflu(ivar)
-ischcp = ischcv(ivar)
-isstpp = isstpc(ivar)
+istatp = vcopt_p%istat
+iconvp = vcopt_p%iconv
+idiffp = vcopt_p%idiff
+ndircp = ndircl(ipr)
+nswrsp = vcopt_p%nswrsm
+nswrgp = vcopt_p%nswrgr
+imligp = vcopt_p%imligr
+ircflp = vcopt_p%ircflu
+ischcp = vcopt_p%ischcv
+isstpp = vcopt_p%isstpc
 iescap = 0
 imucpp = 0
-idftnp = idften(ivar)
-iswdyp = iswdyn(ivar)
-iwarnp = iwarni(ivar)
-blencp = blencv(ivar)
-epsilp = epsilo(ivar)
-epsrsp = epsrsm(ivar)
-epsrgp = epsrgr(ivar)
-climgp = climgr(ivar)
-extrap = extrag(ivar)
-relaxp = relaxv(ivar)
-thetv  = thetav(ivar)
+idftnp = vcopt_p%idften
+iswdyp = vcopt_p%iswdyn
+iwarnp = vcopt_p%iwarni
+blencp = vcopt_p%blencv
+epsilp = vcopt_p%epsilo
+epsrsp = vcopt_p%epsrsm
+epsrgp = vcopt_p%epsrgr
+climgp = vcopt_p%climgr
+extrap = vcopt_p%extrag
+relaxp = vcopt_p%relaxv
+thetv  = vcopt_p%thetav
 icvflb = 0
 
 call codits                                                                    &
-!==========
-( idtvar , ivar   , iconvp , idiffp , ndircp ,                                 &
+( idtvar , init   , ivarfl(ipr)     , iconvp , idiffp , ndircp ,               &
   imrgra , nswrsp , nswrgp , imligp , ircflp ,                                 &
   ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,                        &
   iwarnp ,                                                                     &
@@ -356,16 +372,15 @@ call codits                                                                    &
 !       corrective treatement.
 
 call cs_cf_check_pressure(cvar_pr, ncel)
-!========================
 
 ! --- Explicit balance (see codits : the increment is withdrawn)
 
-if (iwarni(ivar).ge.2) then
+if (vcopt_p%iwarni.ge.2) then
   do iel = 1, ncel
     smbrs(iel) = smbrs(iel)                                                    &
-                 - istat(ivar)*(cell_f_vol(iel)/dt(iel))                       &
+                 - vcopt_p%istat*(cell_f_vol(iel)/dt(iel))                     &
                    *(cvar_pr(iel)-cvara_pr(iel))                               &
-                   * max(0,min(nswrsm(ivar)-2,1))
+                   * max(0,min(vcopt_p%nswrsm-2,1))
   enddo
   sclnor = sqrt(cs_gdot(ncel,smbrs,smbrs))
   write(nfecra,1200) chaine(1:8) ,sclnor
@@ -396,16 +411,15 @@ iphydp = 0
 ! This flux is stored as the mass flux of the energy
 
 call itrmas                                                                    &
-!==========
 ( f_id0  , init   , inc    , imrgra , iccocg , nswrgp , imligp ,               &
-  iphydp , iwarnp ,                                                            &
+  iphydp , 0      , iwarnp ,                                                   &
   epsrgp , climgp , extrap ,                                                   &
   rvoid  ,                                                                     &
   cvar_pr,                                                                     &
   wbfa   , wbfb   ,                                                            &
   coefaf_p        , coefbf_p        ,                                          &
   viscf  , viscb  ,                                                            &
-  dt     , dt     , dt     ,                                                   &
+  dt     ,                                                                     &
   imasfl, bmasfl)
 
 ! Incrementation of the flux with [rho (u + dt f)].n = wflmas
@@ -450,6 +464,7 @@ deallocate(c2)
 deallocate(viscf, viscb)
 deallocate(smbrs, rovsdt)
 deallocate(wflmas, wflmab)
+deallocate(ivolfl, bvolfl)
 deallocate(w1)
 deallocate(w7, w8, w9)
 deallocate(w10)
