@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -24,6 +24,9 @@
 !> \brief Handle boundary condition type code (\ref itypfb) when the
 !> compressible model is enabled.
 !>
+!> Please refer to the
+!> <a href="../../theory.pdf#cfxtcl"><b>cfxtcl</b></a>
+!> section of the theory guide for more informations.
 !-------------------------------------------------------------------------------
 
 !------------------------------------------------------------------------------
@@ -95,18 +98,17 @@ implicit none
 
 integer          nvar
 
-integer          icodcl(nfabor,nvarcl)
+integer          icodcl(nfabor,nvar)
 integer          itypfb(nfabor)
 
 double precision dt(ncelet)
-double precision rcodcl(nfabor,nvarcl,3)
+double precision rcodcl(nfabor,nvar,3)
 
 ! Local variables
 
 integer          ivar  , ifac  , iel, l_size
 integer          ii    , iii   , iccfth
 integer          icalep
-integer          iflmab
 integer          ien   , itk
 integer          nvarcf
 
@@ -114,7 +116,7 @@ integer          nvcfmx
 parameter       (nvcfmx=6)
 integer          ivarcf(nvcfmx)
 
-double precision hint, cpb(1), cvb(1)
+double precision hint, cpb(1), cvb(1), bmasfl
 
 double precision, allocatable, dimension(:) :: w1, w2
 double precision, allocatable, dimension(:) :: w4, w5, w6
@@ -123,7 +125,6 @@ double precision, allocatable, dimension(:) :: wbfb, wbfa
 double precision, allocatable, dimension(:) :: bc_en, bc_pr, bc_tk
 double precision, allocatable, dimension(:,:) :: bc_vel
 
-double precision, dimension(:), pointer :: bmasfl
 double precision, dimension(:), pointer :: coefbp
 double precision, dimension(:), pointer :: crom, brom, cpro_cp, cpro_cv, cvar_en
 double precision, dimension(:,:), pointer :: vel
@@ -149,16 +150,13 @@ allocate(bc_vel(3,nfabor))
 ien = isca(ienerg)
 itk = isca(itempk)
 
-call field_get_key_int(ivarfl(ien), kbmasf, iflmab)
-call field_get_val_s(iflmab, bmasfl)
-
 call field_get_val_s(icrom, crom)
 call field_get_val_s(ibrom, brom)
 
 call field_get_val_s(ivarfl(ien), cvar_en)
 
-if (icp.gt.0) call field_get_val_s(iprpfl(icp), cpro_cp)
-if (icv.gt.0) call field_get_val_s(iprpfl(icv), cpro_cv)
+if (icp.ge.0) call field_get_val_s(icp, cpro_cp)
+if (icv.ge.0) call field_get_val_s(icv, cpro_cv)
 
 ! list of the variables of the compressible model
 ivarcf(1) = ipr
@@ -202,19 +200,6 @@ do ifac = 1, nfabor
 
   if ( itypfb(ifac).eq.iparoi) then
 
-    ! rcodcl elements have been initialized at -RINFIN to allow to check wether
-    ! they have been modified by the user. Here those that have not been
-    ! modified by the user are set back to zero.
-    ! At walls all variables are treated.
-    do ivar = 1, nvar
-      if(rcodcl(ifac,ivar,1).le.-rinfin*0.5d0) then
-        rcodcl(ifac,ivar,1) = 0.d0
-      endif
-    enddo
-
-    ! zero mass flux
-    bmasfl(ifac) = 0.d0
-
     ! pressure :
     ! if the gravity is prevailing: hydrostatic pressure
     ! (warning: the density is here explicit and the term is an approximation)
@@ -279,7 +264,7 @@ do ifac = 1, nfabor
       ! difficult.
 
       icodcl(ifac,ien) = 5
-      if(icv.eq.0) then
+      if(icv.eq.-1) then
         rcodcl(ifac,ien,1) = cv0*rcodcl(ifac,itk,1)
       else
         rcodcl(ifac,ien,1) = cpro_cv(iel)*rcodcl(ifac,itk,1)
@@ -314,46 +299,12 @@ do ifac = 1, nfabor
 
     endif
 
-
-!     Scalars : zero flux (by default in typecl for iparoi code)
-
-
 !===============================================================================
-! 3. Treatment of all symmetry boundary faces
+! 3. Treatment of all inlet/outlet boundary faces and thermo step
 !===============================================================================
 
-  elseif ( itypfb(ifac).eq.isymet ) then
-
-    ! rcodcl elements have been initialized at -RINFIN to allow to check wether
-    ! they have been modified by the user. Here those that have not been
-    ! modified by the user are set back to zero.
-    ! At symmetry faces, all variables are treated.
-    do ivar = 1, nvar
-      if(rcodcl(ifac,ivar,1).le.-rinfin*0.5d0) then
-        rcodcl(ifac,ivar,1) = 0.d0
-      endif
-    enddo
-
-    ! zero mass flux
-    bmasfl(ifac) = 0.d0
-
-    ! Pressure condition:
-    ! homogeneous Neumann condition, nothing to be done.
-    icodcl(ifac,ipr) = 3
-    rcodcl(ifac,ipr,1) = 0.d0
-    rcodcl(ifac,ipr,2) = rinfin
-    rcodcl(ifac,ipr,3) = 0.d0
-
-    ! zero flux for all other variables (except for the normal velocity which is
-    ! itself zero) : by default in typecl for isymet code.
-
 !===============================================================================
-! 4. Treatment of all inlet/outlet boundary faces and thermo step
-!===============================================================================
-
-
-!===============================================================================
-! 4.1 Imposed Inlet/outlet (for example: supersonic inlet)
+! 3.1 Imposed Inlet/outlet (for example: supersonic inlet)
 !===============================================================================
 
   elseif ( itypfb(ifac).eq.iesicf ) then
@@ -369,29 +320,17 @@ do ifac = 1, nfabor
     ! be considered as not initialized and the computation will stop
     ! displaying an error message
     iccfth = 10000
-    if(rcodcl(ifac,ipr,1).gt.0.d0) iccfth = 2*iccfth
-    if(brom(ifac).gt.0.d0)         iccfth = 3*iccfth
-    if(rcodcl(ifac,itk,1).gt.0.d0) iccfth = 5*iccfth
-    if(rcodcl(ifac,ien,1).gt.0.d0) iccfth = 7*iccfth
-    if((iccfth.le.70000.and.iccfth.ne.60000).or.                &
-         (iccfth.eq.350000)) then
+    if(rcodcl(ifac,ipr,1).lt.rinfin*0.5d0) iccfth = 2*iccfth
+    if(brom(ifac).gt.0.d0                ) iccfth = 3*iccfth
+    if(rcodcl(ifac,itk,1).lt.rinfin*0.5d0) iccfth = 5*iccfth
+    if(rcodcl(ifac,ien,1).lt.rinfin*0.5d0) iccfth = 7*iccfth
+
+    if(iccfth.le.70000.and.iccfth.ne.60000.or.iccfth.eq.350000) then
       write(nfecra,1000)iccfth
       call csexit (1)
     endif
     iccfth = iccfth + 900
 
-    ! rcodcl elements have been initialized at -RINFIN to allow to check wether
-    ! they have been modified by the user. Here those that have not been
-    ! modified by the user are set back to zero.
-    ! Firstly variables other than turbulent ones and passive scalars are
-    ! handled, the others are handled further below.
-    do iii = 1, nvarcf
-      ivar = ivarcf(iii)
-      if(rcodcl(ifac,ivar,1).le.-rinfin*0.5d0) then
-        rcodcl(ifac,ivar,1) = 0.d0
-      endif
-    enddo
-
     ! missing thermo variables among P, rho, T, E are computed
     bc_en(ifac) = rcodcl(ifac,ien,1)
     bc_pr(ifac) = rcodcl(ifac,ipr,1)
@@ -406,54 +345,7 @@ do ifac = 1, nfabor
     ! dealt with further below
 
 !===============================================================================
-! 4.2 Supersonic outlet
-!===============================================================================
-
-  elseif ( itypfb(ifac).eq.isspcf ) then
-
-    ! A Dirichlet value equal to the bulk value is imposed for the velocity
-    ! and the energy (for the other variables a deduced Dirichlet value is
-    ! imposed). The computation of a convection flux is not needed here.
-    ! Reconstruction of those bulk cell values would be necessary by using their
-    ! cell gradient: for now only cell center values are used (not consistant on
-    ! non orthogonal meshes but potentially more stable).
-    ! Another solution may be to impose zero fluxes which would avoid
-    ! reconstruction (to be tested).
-    ! rcodcl elements have been initialized at -RINFIN to allow to check wether
-    ! they have been modified by the user. Here those that have not been
-    ! modified by the user are set back to zero.
-    ! Firstly variables other than turbulent ones and passive scalars are
-    ! handled, the others are handled further below.
-    do iii = 1, nvarcf
-      ivar = ivarcf(iii)
-      if(rcodcl(ifac,ivar,1).le.-rinfin*0.5d0) then
-        rcodcl(ifac,ivar,1) = 0.d0
-      endif
-    enddo
-
-    ! density, velocity and total energy values
-    brom(ifac) = crom(iel) ! TODO: test without (already done in phyvar)
-    bc_en(ifac) = cvar_en(iel)
-    bc_pr(ifac) = rcodcl(ifac,ipr,1)
-    bc_tk(ifac) = rcodcl(ifac,itk,1)
-    bc_vel(1,ifac) = vel(1,iel)
-    bc_vel(2,ifac) = vel(2,iel)
-    bc_vel(3,ifac) = vel(3,iel)
-
-    cpb(1) = 0.d0
-    cvb(1) = 0.d0
-    if (icp.gt.0) cpb(1) = cpro_cp(iel)
-    if (icv.gt.0) cvb(1) = cpro_cv(iel)
-
-    l_size = 1
-    call cs_cf_thermo_pt_from_de(cpb, cvb, brom(ifac:ifac), bc_en(ifac:ifac), &
-                                 bc_pr(ifac:ifac), bc_tk(ifac:ifac), &
-                                 bc_vel(:,ifac:ifac), l_size)
-
-    ! mass fluxes and boundary conditions codes, see further below.
-
-!===============================================================================
-! 4.3 Outlet with imposed pressure
+! 3.2 Outlet with imposed pressure
 !===============================================================================
 
   elseif ( itypfb(ifac).eq.isopcf ) then
@@ -466,18 +358,6 @@ do ifac = 1, nfabor
       call csexit (1)
     endif
 
-    ! rcodcl elements have been initialized at -RINFIN to allow to check wether
-    ! they have been modified by the user. Here those that have not been
-    ! modified by the user are set back to zero.
-    ! Firstly variables other than turbulent ones and passive scalars are
-    ! handled, the others are handled further below.
-    do iii = 1, nvarcf
-      ivar = ivarcf(iii)
-      if(rcodcl(ifac,ivar,1).le.-rinfin*0.5d0) then
-        rcodcl(ifac,ivar,1) = 0.d0
-      endif
-    enddo
-
     bc_en(ifac) = rcodcl(ifac,ien,1)
     bc_pr(ifac) = rcodcl(ifac,ipr,1)
     bc_tk(ifac) = rcodcl(ifac,itk,1)
@@ -490,7 +370,7 @@ do ifac = 1, nfabor
     ! mass fluxes and boundary conditions codes, see further below.
 
 !===============================================================================
-! 4.4 Inlet with Ptot, Htot imposed (reservoir boundary conditions)
+! 3.3 Inlet with Ptot, Htot imposed (reservoir boundary conditions)
 !===============================================================================
 
   elseif ( itypfb(ifac).eq.iephcf ) then
@@ -506,18 +386,6 @@ do ifac = 1, nfabor
       call csexit (1)
     endif
 
-    ! rcodcl elements have been initialized at -RINFIN to allow to check wether
-    ! they have been modified by the user. Here those that have not been
-    ! modified by the user are set back to zero.
-    ! Firstly variables other than turbulent ones and passive scalars are
-    ! handled, the others are handled further below.
-    do iii = 1, nvarcf
-      ivar = ivarcf(iii)
-      if(rcodcl(ifac,ivar,1).le.-rinfin*0.5d0) then
-        rcodcl(ifac,ivar,1) = 0.d0
-      endif
-    enddo
-
     bc_en(ifac) = rcodcl(ifac,ien,1)
     bc_pr(ifac) = rcodcl(ifac,ipr,1)
     bc_tk(ifac) = rcodcl(ifac,itk,1)
@@ -530,7 +398,7 @@ do ifac = 1, nfabor
     ! mass fluxes and boundary conditions codes, see further below.
 
 !===============================================================================
-! 4.5 Inlet with imposed rho*U and rho*U*H
+! 3.4 Inlet with imposed rho*U and rho*U*H
 !===============================================================================
 
   elseif ( itypfb(ifac).eq.ieqhcf ) then
@@ -550,131 +418,93 @@ do ifac = 1, nfabor
       call csexit (1)
     endif
 
-    ! rcodcl elements have been initialized at -RINFIN to allow to check wether
-    ! they have been modified by the user. Here those that have not been
-    ! modified by the user are set back to zero.
-    ! Firstly variables other than turbulent ones and passive scalars are
-    ! handled, the others are handled further below.
-    do iii = 1, nvarcf
-      ivar = ivarcf(iii)
-      if(rcodcl(ifac,ivar,1).le.-rinfin*0.5d0) then
-          rcodcl(ifac,ivar,1) = 0.d0
-      endif
-    enddo
-
-
-!===============================================================================
-! 5. Unexpected boundary condition type
-!===============================================================================
-
-  elseif ( itypfb(ifac).ne.iindef ) then
-
-    ! The computation stops.
-    write(nfecra,1400)
-    call csexit (1)
-
   endif ! end of test on boundary condition types
 
-
 !===============================================================================
-! 6. Complete the treatment for inlets and outlets:
+! 4. Complete the treatment for inlets and outlets:
 !    - mass fluxes computation
 !    - boundary convective fluxes computation (analytical or Rusanov) if needed
 !    - B.C. code (Dirichlet or Neumann)
 !===============================================================================
 
-  if ( ( itypfb(ifac).eq.iesicf ) .or.                    &
-       ( itypfb(ifac).eq.isspcf ) .or.                    &
-       ( itypfb(ifac).eq.iephcf ) .or.                    &
-       ( itypfb(ifac).eq.isopcf ) .or.                    &
-       ( itypfb(ifac).eq.ieqhcf ) ) then
+  if (itypfb(ifac).eq.iesicf.or.                    &
+      itypfb(ifac).eq.isspcf.or.                    &
+      itypfb(ifac).eq.iephcf.or.                    &
+      itypfb(ifac).eq.isopcf.or.                    &
+      itypfb(ifac).eq.ieqhcf) then
 
 !===============================================================================
-! 6.1 Mass fluxes computation and
+! 4.1 Mass fluxes computation and
 !     boundary convective fluxes computation (analytical or Rusanov) if needed
 !     (gamma should already have been computed if Rusanov fluxes are computed)
 !===============================================================================
 
-    ! Supersonic outlet
-    if ( itypfb(ifac).eq.isspcf ) then
+    ! Rusanov fluxes are computed only for the imposed inlet for stability
+    ! reasons (the mass flux computation is concluded)
+    if (itypfb(ifac).eq.iesicf) then
 
-      ! only the mass flux is computed
-      bmasfl(ifac) = brom(ifac) *                                              &
-                     (  bc_vel(1,ifac)*suffbo(1,ifac)                          &
-                      + bc_vel(2,ifac)*suffbo(2,ifac)                          &
-                      + bc_vel(3,ifac)*suffbo(3,ifac) )
+      call cfrusb(ifac, bc_en, bc_pr, bc_vel)
 
-    ! other inlets/outlets
-    else
+    ! For the other types of inlets/outlets (subsonic outlet, QH inlet,
+    ! PH inlet), analytical fluxes are computed
+    elseif (itypfb(ifac).ne.isspcf) then
 
-      ! Rusanov fluxes are computed only for the imposed inlet for stability
-      ! reasons (the mass flux computation is concluded)
-      if (itypfb(ifac).eq.iesicf) then
-
-        call cfrusb(ifac, bc_en, bc_pr, bc_vel)
-
-      ! For the other types of inlets/outlets (subsonic outlet, QH inlet,
-      ! PH inlet), analytical fluxes are computed
-      else
-
-        ! the pressure part of the boundary analytical flux is not added here,
-        ! but set through the pressure gradient boundary conditions (Dirichlet)
-        call cffana(ifac, bc_en, bc_pr, bc_vel)
-
-      endif
+      ! the pressure part of the boundary analytical flux is not added here,
+      ! but set through the pressure gradient boundary conditions (Dirichlet)
+      call cffana(ifac, bc_en, bc_pr, bc_vel)
 
     endif
 
 !===============================================================================
-! 6.2 Copy of boundary values into the Dirichlet values array
+! 4.2 Copy of boundary values into the Dirichlet values array
 !===============================================================================
 
-    rcodcl(ifac,ien,1) = bc_en(ifac)
-    rcodcl(ifac,ipr,1) = bc_pr(ifac)
-    rcodcl(ifac,itk,1) = bc_tk(ifac)
-    rcodcl(ifac,iu,1)  = bc_vel(1,ifac)
-    rcodcl(ifac,iv,1)  = bc_vel(2,ifac)
-    rcodcl(ifac,iw,1)  = bc_vel(3,ifac)
+    if (itypfb(ifac).ne.isspcf) then
+      rcodcl(ifac,ien,1) = bc_en(ifac)
+      rcodcl(ifac,ipr,1) = bc_pr(ifac)
+      rcodcl(ifac,itk,1) = bc_tk(ifac)
+      rcodcl(ifac,iu,1)  = bc_vel(1,ifac)
+      rcodcl(ifac,iv,1)  = bc_vel(2,ifac)
+      rcodcl(ifac,iw,1)  = bc_vel(3,ifac)
+    else ! supersonic outlet
+      rcodcl(ifac,ien,3) = 0.d0
+      rcodcl(ifac,ipr,3) = 0.d0
+      rcodcl(ifac,itk,3) = 0.d0
+      rcodcl(ifac,iu,3)  = 0.d0
+      rcodcl(ifac,iv,3)  = 0.d0
+      rcodcl(ifac,iw,3)  = 0.d0
+    endif
 
 !===============================================================================
-! 6.3 Boundary conditions codes (Dirichlet or Neumann)
+! 4.3 Boundary conditions codes (Dirichlet or Neumann)
 !===============================================================================
 
-!     P               : Dirichlet except for iesicf : Neumann (arbitrary choice)
+!     P               : Neumann but pressure part of momentum flux is imposed
+!                       as a Dirichlet BC for the pressure gradient (code 13)
 !     rho, U, E, T    : Dirichlet
 !     k, R, eps, scal : Dirichlet/Neumann depending on the flux mass value
 
-! For the pressure, a Neumann B.C. seems to be less worth for gradient
-! reconstruction if the value of P provided by the user is very different from
-! the internal value. The choice is however arbitrary.
-
-! At this point, the following values are assumed to be in the array rcodcl
-! rcodcl(IFAC,ivar,1) = user or computed above
-! rcodcl(IFAC,ivar,2) = RINFIN
-! rcodcl(IFAC,ivar,3) = 0.D0
-! and if icodcl(IFAC,ivar) = 3, only rcodcl(IFAC,ivar,3) is used
-
-
-!-------------------------------------------------------------------------------
-! Pressure : - Dirichlet for the gradient computation, allowing to have the
-!            pressure part of the convective flux at the boundary
-!            - Homogeneous Neumann for the diffusion
-!-------------------------------------------------------------------------------
-
-    icodcl(ifac,ipr)   = 13
-
-!-------------------------------------------------------------------------------
-! U E T : Dirichlet
-!-------------------------------------------------------------------------------
-
-    ! velocity
-    icodcl(ifac,iu)    = 1
-    icodcl(ifac,iv)    = 1
-    icodcl(ifac,iw)    = 1
-    ! total energy
-    icodcl(ifac,ien)   = 1
-    ! temperature
-    icodcl(ifac,itk)   = 1
+    if (itypfb(ifac).ne.isspcf) then
+      ! Pressure : - Dirichlet for the gradient computation, allowing to have the
+      !              pressure part of the convective flux at the boundary
+      !            - Homogeneous Neumann for the diffusion
+      icodcl(ifac,ipr)   = 13
+      ! velocity
+      icodcl(ifac,iu)    = 1
+      icodcl(ifac,iv)    = 1
+      icodcl(ifac,iw)    = 1
+      ! total energy
+      icodcl(ifac,ien)   = 1
+      ! temperature
+      icodcl(ifac,itk)   = 1
+    else ! supersonic outlet
+      icodcl(ifac,ipr)   = 3
+      icodcl(ifac,iu)    = 3
+      icodcl(ifac,iv)    = 3
+      icodcl(ifac,iw)    = 3
+      icodcl(ifac,ien)   = 3
+      icodcl(ifac,itk)   = 3
+    endif
 
 !-------------------------------------------------------------------------------
 ! Turbulence and passive scalars: Dirichlet / Neumann depending on the mass flux
@@ -684,7 +514,12 @@ do ifac = 1, nfabor
     ! A Dirichlet is chosen if the mass flux is ingoing and if the user provided
     ! a value in rcodcl(ifac,ivar,1)
 
-    if (bmasfl(ifac).ge.0.d0) then
+    bmasfl =  brom(ifac)                                                &
+             *(  bc_vel(1,ifac)*suffbo(1,ifac)                          &
+               + bc_vel(2,ifac)*suffbo(2,ifac)                          &
+               + bc_vel(3,ifac)*suffbo(3,ifac) )
+
+    if (itypfb(ifac).ne.isspcf.and.bmasfl.ge.0.d0) then
       if(itytur.eq.2) then
         icodcl(ifac,ik ) = 3
         icodcl(ifac,iep) = 3
@@ -719,8 +554,8 @@ do ifac = 1, nfabor
       endif
     else
       if(itytur.eq.2) then
-        if(rcodcl(ifac,ik ,1).gt.0.d0.and.               &
-             rcodcl(ifac,iep,1).gt.0.d0) then
+        if(rcodcl(ifac,ik ,1).lt.rinfin*0.5d0  .and.    &
+           rcodcl(ifac,iep,1).lt.rinfin*0.5d0) then
           icodcl(ifac,ik ) = 1
           icodcl(ifac,iep) = 1
         else
@@ -728,13 +563,13 @@ do ifac = 1, nfabor
           icodcl(ifac,iep) = 3
         endif
       elseif(itytur.eq.3) then
-        if(rcodcl(ifac,ir11,1).gt.0.d0.and.              &
-             rcodcl(ifac,ir22,1).gt.0.d0.and.              &
-             rcodcl(ifac,ir33,1).gt.0.d0.and.              &
-             rcodcl(ifac,ir12,1).gt.-rinfin*0.5d0.and.     &
-             rcodcl(ifac,ir13,1).gt.-rinfin*0.5d0.and.     &
-             rcodcl(ifac,ir23,1).gt.-rinfin*0.5d0.and.     &
-             rcodcl(ifac,iep ,1).gt.0.d0) then
+        if(rcodcl(ifac,ir11,1).lt.rinfin*0.5d0  .and.      &
+           rcodcl(ifac,ir22,1).lt.rinfin*0.5d0  .and.      &
+           rcodcl(ifac,ir33,1).lt.rinfin*0.5d0  .and.      &
+           rcodcl(ifac,ir12,1).lt.rinfin*0.5d0  .and.      &
+           rcodcl(ifac,ir13,1).lt.rinfin*0.5d0  .and.      &
+           rcodcl(ifac,ir23,1).lt.rinfin*0.5d0  .and.      &
+           rcodcl(ifac,iep ,1).lt.rinfin*0.5d0) then
           icodcl(ifac,ir11) = 1
           icodcl(ifac,ir22) = 1
           icodcl(ifac,ir33) = 1
@@ -752,10 +587,10 @@ do ifac = 1, nfabor
           icodcl(ifac,iep ) = 3
         endif
       elseif(iturb.eq.50) then
-        if(rcodcl(ifac,ik  ,1).gt.0.d0.and.              &
-             rcodcl(ifac,iep ,1).gt.0.d0.and.              &
-             rcodcl(ifac,iphi,1).gt.0.d0.and.              &
-             rcodcl(ifac,ifb ,1).gt.-rinfin*0.5d0 ) then
+        if(rcodcl(ifac,ik  ,1).lt.rinfin*0.5d0  .and.      &
+           rcodcl(ifac,iep ,1).lt.rinfin*0.5d0  .and.      &
+           rcodcl(ifac,iphi,1).lt.rinfin*0.5d0  .and.      &
+           rcodcl(ifac,ifb ,1).lt.rinfin*0.5d0) then
           icodcl(ifac,ik  ) = 1
           icodcl(ifac,iep ) = 1
           icodcl(ifac,iphi) = 1
@@ -767,8 +602,8 @@ do ifac = 1, nfabor
           icodcl(ifac,ifb ) = 3
         endif
       elseif(iturb.eq.60) then
-         if(rcodcl(ifac,ik  ,1).gt.0.d0.and.               &
-              rcodcl(ifac,iomg,1).gt.0.d0 ) then
+         if(rcodcl(ifac,ik  ,1).lt.rinfin*0.5d0  .and.      &
+            rcodcl(ifac,iomg,1).lt.rinfin*0.5d0) then
            icodcl(ifac,ik  ) = 1
            icodcl(ifac,iomg) = 1
          else
@@ -784,7 +619,7 @@ do ifac = 1, nfabor
        endif
        if (nscaus.gt.0) then
          do ii = 1, nscaus
-           if(rcodcl(ifac,isca(ii),1).gt.-rinfin*0.5d0) then
+           if(rcodcl(ifac,isca(ii),1).lt.rinfin*0.5d0) then
              icodcl(ifac,isca(ii)) = 1
            else
              icodcl(ifac,isca(ii)) = 3
@@ -793,7 +628,7 @@ do ifac = 1, nfabor
        endif
        if (nscasp.gt.0) then
          do ii = 1, nscasp
-           if(rcodcl(ifac,iscasp(ii),1).gt.-rinfin*0.5d0) then
+           if(rcodcl(ifac,iscasp(ii),1).lt.rinfin*0.5d0) then
              icodcl(ifac,iscasp(ii)) = 1
            else
              icodcl(ifac,iscasp(ii)) = 3
@@ -802,18 +637,6 @@ do ifac = 1, nfabor
        endif
      endif
 
-     ! rcodcl elements have been initialized at -RINFIN to allow to check wether
-     ! they have been modified by the user. Here those that have not been
-     ! modified by the user are set back to zero.
-     ! Turbulence and passive scalars are treated so here (to simplify the loop,
-     ! all variables are treated, hence compressible variables are treated again
-     ! here).
-     do ivar = 1, nvar
-       if(rcodcl(ifac,ivar,1).le.-rinfin*0.5d0) then
-         rcodcl(ifac,ivar,1) = 0.d0
-       endif
-     enddo
-
    endif ! end of test on inlet/outlet faces
 
  enddo ! end of loop on boundary faces
@@ -908,22 +731,7 @@ deallocate(bc_en, bc_pr, bc_tk, bc_vel)
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
- 1400 format(                                                           &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ WARNING : Error during execution,                       ',/,&
-'@    =========                                               ',/,&
-'@    Unexpected type of predefined compressible boundary     ',/,&
-'@      conditions.                                           ',/,&
-'@                                                            ',/,&
-'@    The computation will stop.                              ',/,&
-'@                                                            ',/,&
-'@    Check the boundary conditions in                        ',/,&
-'@    cs_user_boundary_conditions                             ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
+
 !----
 ! END
 !----
