@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -710,7 +710,7 @@ endif
 
 ! Although we are in enthalpy formulation, we keep Cp constant
 
-icp = 0
+icp = -1
 
 return
 end subroutine
