@@ -6,7 +6,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -234,7 +234,8 @@ _add_catalyst_mesh(fvm_to_catalyst_t  *writer,
   /* Add a new Catalyst mesh structure */
 
   int id = writer->mb->GetNumberOfBlocks();
-  vtkUnstructuredGrid* ugrid = vtkUnstructuredGrid::New();
+  vtkSmartPointer<vtkUnstructuredGrid> ugrid
+    = vtkSmartPointer<vtkUnstructuredGrid>::New();
 
   writer->mb->SetBlock(id, ugrid);
   writer->mb->GetMetaData(id)->Set(vtkCompositeDataSet::NAME(), mesh->name);
@@ -477,7 +478,7 @@ _add_catalyst_field(fvm_to_catalyst_t         *writer,
               writer->n_fields+ 1,
               fvm_catalyst_field_t *);
 
-  vtkUnstructuredGrid *f =  NULL;
+  vtkUnstructuredGrid *f = NULL;
   vtkDoubleArray *tmp = NULL;
 
   const int dest_dim = (dim == 6) ? 9 : dim;
@@ -923,25 +924,27 @@ _export_field_values_e(const fvm_nodal_t         *mesh,
                       field_values,
                       values + start_id);
 
-    /* Special case for symmetric tensors */
-
-    if (dim == 6) {
-      for (cs_lnum_t i = 0; i < mesh->n_vertices; i++) {
-        values[9*i + 8] = values[9*i + 2];
-        values[9*i + 7] = values[9*i + 4];
-        values[9*i + 6] = values[9*i + 5];
-        values[9*i + 4] = values[9*i + 1];
-        values[9*i + 2] = values[9*i + 5];
-        values[9*i + 1] = values[9*i + 3];
-        values[9*i + 5] = values[9*i + 7];
-      }
-    }
-
     start_id += section->n_elements*dest_dim;
     if (n_parent_lists == 0)
       src_shift += section->n_elements;
 
   }
+
+  /* Special case for symmetric tensors */
+
+  if (dim == 6) {
+
+    cs_lnum_t n_elts = f->GetNumberOfCells();
+    for (cs_lnum_t i = 0; i < n_elts; i++) {
+      values[9*i + 8] = values[9*i + 2];
+      values[9*i + 7] = values[9*i + 4];
+      values[9*i + 6] = values[9*i + 5];
+      values[9*i + 4] = values[9*i + 1];
+      values[9*i + 2] = values[9*i + 5];
+      values[9*i + 1] = values[9*i + 3];
+      values[9*i + 5] = values[9*i + 7];
+    }
+  }
 }
 
 /*============================================================================
