@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -892,11 +892,32 @@ fvm_to_ensight_case_get_var_file(fvm_to_ensight_case_t       *const this_case,
 
   /* First, find if variable is already defined */
 
+  char _lw_name[128], _clw_name[128];
+  char *lw_name = _lw_name, *clw_name = _clw_name;
+
+  size_t l_n = strlen(name);
+  if (l_n > 128)
+    BFT_MALLOC(lw_name, l_n + 1, char);
+  for (size_t j = 0; j < l_n; j++)
+    lw_name[j] = tolower(name[j]);
+  lw_name[l_n] = '\0';
+
   for (i = 0 ; i < this_case->n_vars ; i++) {
 
     var = this_case->var[i];
 
-    if (strcmp(var->name, name) == 0) {
+    size_t l_c = strlen(var->name);
+    if (l_c > 128) {
+      if (clw_name == _clw_name)
+        BFT_MALLOC(clw_name, l_c + 1, char);
+      else
+        BFT_REALLOC(clw_name, l_c + 1, char);
+    }
+    for (size_t j = 0; j < l_c; j++)
+      clw_name[j] = tolower(var->name[j]);
+    clw_name[l_c] = '\0';
+
+    if (strcmp(clw_name, lw_name) == 0) {
 
       /* Variable is already defined, so check consistency */
 
@@ -929,6 +950,11 @@ fvm_to_ensight_case_get_var_file(fvm_to_ensight_case_t       *const this_case,
 
   }
 
+  if (lw_name != _lw_name)
+    BFT_FREE(lw_name);
+  if (clw_name != _clw_name)
+    BFT_FREE(clw_name);
+
   /* Second, update time step */
 
   if (time_step > -1) {
@@ -1080,8 +1106,18 @@ fvm_to_ensight_case_write_case(fvm_to_ensight_case_t  *this_case,
       fprintf(f, "filename increment:    1\n");
       fprintf(f, "time values:\n");
 
-      for (j = 0 ; j < ts->n_time_values ; j++)
-        fprintf(f, "            %g\n", ts->time_value[j]);
+      for (j = 0 ; j < ts->n_time_values ; j++) {
+        char tmp[64];
+        snprintf(tmp, 63, "%.12f", ts->time_value[j]);
+        tmp[63] = '\0';
+        for (int k = strlen(tmp)-1; k > 0; k--) {
+          if (tmp[k] == '0')
+            tmp[k] = '\0';
+          else
+            break;
+        }
+        fprintf(f, "            %s\n", tmp);
+      }
 
     }
 
