@@ -6,7 +6,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -157,6 +157,8 @@ _field_output(void           *context,
               cs_gnum_t       block_end,
               void           *buffer)
 {
+  CS_UNUSED(datatype);
+
   _plot_context_t *c = context;
 
   fvm_to_plot_writer_t  *w = c->writer;
@@ -250,7 +252,7 @@ _field_output(void           *context,
     }
 
     if (w->format == CS_PLOT_DAT) {
-      if (w->n_cols > 1)
+      if (w->n_cols > 0)
         fprintf(w->f, " | %s", name_buf);
       else
         fprintf(w->f, " %s", name_buf);
@@ -313,6 +315,8 @@ fvm_to_plot_init_writer(const char             *name,
                         fvm_writer_time_dep_t   time_dependency)
 #endif
 {
+  CS_UNUSED(time_dependency);
+
   fvm_to_plot_writer_t  *w = NULL;
 
   /* Initialize writer */
@@ -558,7 +562,7 @@ fvm_to_plot_export_field(void                  *writer,
     = fvm_writer_field_helper_create(mesh,
                                      NULL, /* section list */
                                      dimension,
-                                     CS_NO_INTERLACE,
+                                     CS_INTERLACE,
                                      dest_datatype,
                                      location);
 
