@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -69,13 +69,16 @@
 #include "fvm_to_cgns.h"
 #include "fvm_to_med.h"
 #include "fvm_to_ensight.h"
+#include "fvm_to_histogram.h"
 #include "fvm_to_plot.h"
 #include "fvm_to_time_plot.h"
 
-#if !defined(HAVE_PLUGINS)
-#if defined(HAVE_CATALYST)
+#if defined(HAVE_CATALYST) && !defined(HAVE_PLUGIN_CATALYST)
 #include "fvm_to_catalyst.h"
 #endif
+
+#if defined(HAVE_MELISSA) && !defined(HAVE_PLUGIN_MELISSA)
+#include "fvm_to_melissa.h"
 #endif
 
 /*----------------------------------------------------------------------------*/
@@ -104,9 +107,9 @@ BEGIN_C_DECLS
 
 /* Number and status of defined formats */
 
-static const int _fvm_writer_n_formats = 8;
+static const int _fvm_writer_n_formats = 10;
 
-static fvm_writer_format_t _fvm_writer_format_list[8] = {
+static fvm_writer_format_t _fvm_writer_format_list[10] = {
 
   /* Built-in EnSight Gold writer */
   {
@@ -202,16 +205,21 @@ static fvm_writer_format_t _fvm_writer_format_list[8] = {
   /* Catalyst (VTK) writer (plugin) */
   {
     "Catalyst",
-    "4.0 +",
+    "4.2 +",
     (  FVM_WRITER_FORMAT_USE_EXTERNAL
      | FVM_WRITER_FORMAT_HAS_POLYGON
      | FVM_WRITER_FORMAT_HAS_POLYHEDRON),
     FVM_WRITER_TRANSIENT_CONNECT,
-#if !defined(HAVE_CATALYST) || defined(HAVE_PLUGINS)
+#if !defined(HAVE_CATALYST) || defined(HAVE_PLUGIN_CATALYST)
     0,                                 /* dynamic library count */
     NULL,                              /* dynamic library */
+#if defined(HAVE_CATALYST)
     "fvm_catalyst",                    /* dynamic library name */
     "fvm_to_catalyst_",                /* dynamic library prefix */
+#else
+    NULL,
+    NULL,
+#endif
     NULL,
     NULL,
     NULL,
@@ -244,11 +252,16 @@ static fvm_writer_format_t _fvm_writer_format_list[8] = {
     "6.3 +",
     (  FVM_WRITER_FORMAT_USE_EXTERNAL
      | FVM_WRITER_FORMAT_HAS_POLYGON),
-    FVM_WRITER_FIXED_MESH,
+    FVM_WRITER_TRANSIENT_CONNECT,
     0,                                 /* dynamic library count */
     NULL,                              /* dynamic library */
+#if defined(HAVE_MEDCOUPLING)
     "fvm_medcoupling",                 /* dynamic library name */
     "fvm_to_medcoupling_",             /* dynamic library prefix */
+#else
+    NULL,
+    NULL,
+#endif
     NULL,
     NULL,
     NULL,
@@ -260,13 +273,82 @@ static fvm_writer_format_t _fvm_writer_format_list[8] = {
     NULL
   },
 
+  /* Built-in Melissa writer */
+  {
+    "Melissa",
+    "",
+    (  FVM_WRITER_FORMAT_USE_EXTERNAL
+     | FVM_WRITER_FORMAT_HAS_POLYGON
+     | FVM_WRITER_FORMAT_HAS_POLYHEDRON),
+    FVM_WRITER_TRANSIENT_CONNECT,
+#if !defined(HAVE_MELISSA) || defined(HAVE_PLUGIN_MELISSA)
+    0,                                 /* dynamic library count */
+    NULL,                              /* dynamic library */
+#if defined(HAVE_MELISSA)
+    "fvm_melissa",                     /* dynamic library name */
+    "fvm_to_melissa_",                 /* dynamic library prefix */
+#else
+    NULL,
+    NULL,
+#endif
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL
+#else
+    0,                                 /* dynamic library count */
+    NULL,                              /* dynamic library */
+    NULL,                              /* dynamic library name */
+    NULL,                              /* dynamic library prefix */
+    NULL,                              /* n_version_strings_func */
+    NULL,                              /* version_string_func */
+    fvm_to_melissa_init_writer,        /* init_func */
+    fvm_to_melissa_finalize_writer,    /* finalize_func */
+    fvm_to_melissa_set_mesh_time,      /* set_mesh_time_func */
+    NULL,                              /* needs_tesselation_func */
+    fvm_to_melissa_export_nodal,       /* export_nodal_func */
+    fvm_to_melissa_export_field,       /* export_field_func */
+    NULL                               /* flush_func */
+#endif
+  },
+
+  /* Built-in histogram writer */
+  {
+    "histogram",
+    "",
+    (  FVM_WRITER_FORMAT_HAS_POLYGON
+     | FVM_WRITER_FORMAT_HAS_POLYHEDRON
+     | FVM_WRITER_FORMAT_SEPARATE_MESHES
+     | FVM_WRITER_FORMAT_NAME_IS_OPTIONAL),
+       FVM_WRITER_TRANSIENT_CONNECT,
+    0,                                 /* dynamic library count */
+    NULL,                              /* dynamic library */
+    NULL,                              /* dynamic library name */
+    NULL,                              /* dynamic library prefix */
+    NULL,                              /* n_version_strings_func */
+    NULL,                              /* version_string_func */
+    fvm_to_histogram_init_writer,      /* init_func */
+    fvm_to_histogram_finalize_writer,  /* finalize_func */
+    fvm_to_histogram_set_mesh_time,    /* set_mesh_time_func */
+    NULL,                              /* needs_tesselation_func */
+    NULL,                              /* export_nodal_func */
+    fvm_to_histogram_export_field,     /* export_field_func */
+    fvm_to_histogram_flush             /* flush_func */
+  },
+
   /* Built-in plot writer */
   {
     "plot",
     "",
     (  FVM_WRITER_FORMAT_HAS_POLYGON
      | FVM_WRITER_FORMAT_HAS_POLYHEDRON
-     | FVM_WRITER_FORMAT_SEPARATE_MESHES),
+     | FVM_WRITER_FORMAT_SEPARATE_MESHES
+     | FVM_WRITER_FORMAT_NAME_IS_OPTIONAL),
     FVM_WRITER_TRANSIENT_CONNECT,
     0,                                 /* dynamic library count */
     NULL,                              /* dynamic library */
@@ -289,7 +371,8 @@ static fvm_writer_format_t _fvm_writer_format_list[8] = {
     "",
     (  FVM_WRITER_FORMAT_HAS_POLYGON
      | FVM_WRITER_FORMAT_HAS_POLYHEDRON
-     | FVM_WRITER_FORMAT_SEPARATE_MESHES),
+     | FVM_WRITER_FORMAT_SEPARATE_MESHES
+     | FVM_WRITER_FORMAT_NAME_IS_OPTIONAL),
     FVM_WRITER_FIXED_MESH,
     0,                                 /* dynamic library count */
     NULL,                              /* dynamic library */
@@ -427,27 +510,23 @@ _get_dl_function_pointer(fvm_writer_format_t  *wf,
                          bool                  errors_are_fatal)
 {
   void  *retval = NULL;
-  char  *error = NULL;
 
   assert(wf != NULL);
   assert(wf->dl_lib != NULL);
 
-  dlerror();    /* Clear any existing error */
-
   if (wf->dl_name == NULL)
-    retval = dlsym(wf->dl_lib, name);
+    retval = cs_base_get_dl_function_pointer(wf->dl_lib,
+                                             name,
+                                             errors_are_fatal);
   else {
     char *_name;
     BFT_MALLOC(_name, strlen(wf->dl_prefix) + strlen(name) + 1, char);
     sprintf(_name, "%s%s", wf->dl_prefix, name);
-    retval = dlsym(wf->dl_lib, _name);
+    retval = cs_base_get_dl_function_pointer(wf->dl_lib,
+                                             _name,
+                                             errors_are_fatal);
     BFT_FREE(_name);
   }
-  error = dlerror();
-
-  if (error != NULL && errors_are_fatal)
-    bft_error(__FILE__, __LINE__, 0,
-              _("Error calling dlsym: %s\n"), error);
 
   return retval;
 }
@@ -462,19 +541,9 @@ _get_dl_function_pointer(fvm_writer_format_t  *wf,
 static void
 _load_plugin(fvm_writer_format_t  *wf)
 {
-  char  *lib_path = NULL;
-  const char *pkglibdir = cs_base_get_pkglibdir();
-
   /* Open from shared library */
 
-  BFT_MALLOC(lib_path,
-             strlen(pkglibdir) + 1 + 3 + strlen(wf->dl_name) + 3 + 1,
-             char);
-  sprintf(lib_path, "%s%c%s.so", pkglibdir, DIR_SEPARATOR, wf->dl_name);
-
-  wf->dl_lib = dlopen(lib_path, RTLD_LAZY);
-
-  BFT_FREE(lib_path);
+  wf->dl_lib = cs_base_dlopen_plugin(wf->dl_name);
 
   /* Increment reference count */
 
@@ -482,10 +551,6 @@ _load_plugin(fvm_writer_format_t  *wf)
 
   /* Load symbols from shared library */
 
-  if (wf->dl_lib == NULL)
-    bft_error(__FILE__, __LINE__, 0,
-              _("Error loading %s: %s."), lib_path, dlerror());
-
   /* Function pointers need to be double-casted so as to first convert
      a (void *) type to a memory address and then convert it back to the
      original type. Otherwise, the compiler may issue a warning.
@@ -534,9 +599,7 @@ _close_plugin(fvm_writer_format_t  *wf)
   if (wf->dl_lib == NULL)
     return;
 
-  if (dlclose(wf->dl_lib) != 0)
-    bft_error(__FILE__, __LINE__, 0,
-              _("Error unloading library: %s."), dlerror());
+  cs_base_dlclose(wf->name, wf->dl_lib);
 
   wf->dl_lib = NULL;
 
@@ -577,6 +640,9 @@ static void *
 _format_writer_init(fvm_writer_t  *this_writer,
                     const char    *mesh_name)
 {
+  const  char empty[] = "";
+  const  char untitled[] = "[untitled]";
+
   char   local_dir[] = ".";
   char  *tmp_path = NULL, *tmp_name = NULL;
 
@@ -609,19 +675,32 @@ _format_writer_init(fvm_writer_t  *this_writer,
 
   const char *name = this_writer->name;
 
-  if (mesh_name != NULL) {
-    if (strlen(mesh_name) > 0) {
-      size_t lw = strlen(this_writer->name);
-      size_t l = lw + 1 + strlen(mesh_name);
-      BFT_MALLOC(tmp_name, l + 1, char);
-      sprintf(tmp_name, "%s_%s", this_writer->name, mesh_name);
-      for (size_t i = lw + 1, j = 0; i < l; i++, j++) {
-        if (tmp_name[i] == ' ')
-          tmp_name[i] = '_';
+  if (name != NULL) {
+    size_t lw = strlen(this_writer->name);
+    if (lw == 0)
+      name = mesh_name;
+    else if (mesh_name != NULL) {
+      if (strlen(mesh_name) > 0) {
+        size_t l = lw + 1 + strlen(mesh_name);
+        BFT_MALLOC(tmp_name, l + 1, char);
+        sprintf(tmp_name, "%s_%s", this_writer->name, mesh_name);
+        for (size_t i = lw + 1, j = 0; i < l; i++, j++) {
+          if (tmp_name[i] == ' ')
+            tmp_name[i] = '_';
+        }
+        name = tmp_name;
       }
-      name = tmp_name;
     }
   }
+  else
+    name = mesh_name;
+
+  if (name == NULL) {
+    if (this_writer->format->info_mask & FVM_WRITER_FORMAT_NAME_IS_OPTIONAL)
+      name = empty;
+    else
+      name = untitled;
+  }
 
   /* Initialize format-specific writer */
 
@@ -905,6 +984,72 @@ fvm_writer_version_string(int format_index,
   return retval;
 }
 
+/*----------------------------------------------------------------------------
+ * Remove a given option from a format options list if present.
+ *
+ * The possible separators are also transformed to whitespace and merged.
+ *
+ * parameters:
+ *   format_options  <-> options for the selected format (case-independent,
+ *                       whitespace or comma separated list)
+ *   exclude_option  <-- option to be excluded
+ *----------------------------------------------------------------------------*/
+
+void
+fvm_writer_filter_option(char        *format_options,
+                         const char  *exclude_option)
+{
+  char  *tmp_options = NULL;
+
+  size_t l = strlen(exclude_option);
+
+  /* Find corresponding format and check coherency */
+
+  tmp_options = _fvm_writer_option_list(format_options);
+
+  /* Parse top-level options (consuming those handled here);
+     the options string now contains options separated by a single
+     whitespace. */
+
+  if (tmp_options != NULL) {
+
+    int i0 = 0, i1;
+
+    while (tmp_options[i0] != '\0') {
+
+      for (i1 = i0; tmp_options[i1] != '\0' && tmp_options[i1] != ' '; i1++);
+      size_t l_opt = i1 - i0;
+
+      if (   (l_opt == l)
+          && (strncmp(tmp_options + i0, exclude_option, l_opt) == 0)) {
+        if (tmp_options[i1] == ' ')
+          strcpy(tmp_options + i0, tmp_options + i1 + 1);
+        else {
+          if (i0 > 1) {
+            assert(tmp_options[i0-1] = ' ');
+            i0--;
+          }
+          tmp_options[i0] = '\0';
+        }
+      }
+      else {
+        i0 = i1;
+        if (tmp_options[i0] == ' ')
+          i0++;
+      }
+
+      i1 = strlen(tmp_options);
+      BFT_REALLOC(tmp_options, i1+1, char);
+
+    }
+
+  }
+
+  strcpy(format_options, tmp_options);
+
+  BFT_FREE(tmp_options);
+}
+
 /*----------------------------------------------------------------------------
  * Initialize FVM mesh and field output writer.
  *
@@ -1372,6 +1517,7 @@ fvm_writer_export_field(fvm_writer_t                 *this_writer,
                       time_step,
                       time_value,
                       field_values);
+
     cs_fp_exception_restore_trap();
   }
 
