@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -32,6 +32,7 @@
  *----------------------------------------------------------------------------*/
 
 #include "cs_base.h"
+#include "cs_volume_zone.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -193,13 +194,7 @@ void CS_PROCF (cstime, CSTIME) (void);
  *
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (uinum1, UINUM1) (double *blencv,
-                                   int *ischcv,
-                                   int *isstpc,
-                                   int *ircflu,
-                                double *cdtvar,
-                                double *epsilo,
-                                   int *nswrsm);
+void CS_PROCF (uinum1, UINUM1) (double *cdtvar);
 
 /*----------------------------------------------------------------------------
  * Global numerical parameters.
@@ -218,9 +213,9 @@ void CS_PROCF (csnum2, CSNUM2) (double *relaxp,
                                 double *extrag,
                                    int *imrgra);
 
-void CS_PROCF (csphys, CSPHYS) (const    int *nmodpp,
-                                      double *viscv0,
-                                      double *visls0,
+void CS_PROCF (csphys, CSPHYS) (
+                                double       *viscv0,
+                                double       *visls0,
                                 const    int *itempk);
 
 /*----------------------------------------------------------------------------
@@ -269,12 +264,9 @@ void CS_PROCF (uiipsu, UIIPSU) (int *iporos);
  * Fortran Interface:
  *
  * SUBROUTINE UIPORO
- * *****************
- *
- * INTEGER          IPOROS     <--   porosity
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (uiporo, UIPORO) (const int *iporos);
+void CS_PROCF (uiporo, UIPORO) (void);
 
 /*----------------------------------------------------------------------------
  * User momentum source terms.
@@ -370,25 +362,6 @@ void CS_PROCF(uiphyv, UIPHYV)(const cs_int_t  *iviscv,
                               const cs_real_t *visls0,
                               const cs_real_t *viscv0);
 
-/*----------------------------------------------------------------------------
- * Head losses definition
- *
- * Fortran Interface:
- *
- * subroutine uikpdc
- * *****************
- *
- * integer          iappel   <--  number of calls during a time step
- * integer          ncepdp  -->   number of cells with head losses
- * integer          icepdc  -->   ncepdp cells number with head losses
- * double precision ckupdc  -->   head losses matrix
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF(uikpdc, UIKPDC)(const int*   iappel,
-                                    int*   ncepdp,
-                                    int    icepdc[],
-                                    double ckupdc[]);
-
 /*----------------------------------------------------------------------------
  * 1D profile postprocessing
  *
@@ -401,6 +374,18 @@ void CS_PROCF(uikpdc, UIKPDC)(const int*   iappel,
 
 void CS_PROCF (uiprof, UIPROF)(void);
 
+/*----------------------------------------------------------------------------
+ * extra operations
+ *
+ * Fortran Interface:
+ *
+ * SUBROUTINE UIEXOP
+ * *****************
+ *
+ *----------------------------------------------------------------------------*/
+
+void CS_PROCF (uiexop, UIEXOP)(void);
+
 /*----------------------------------------------------------------------------
  * groundwater model : read laws for capacity, saturation and permeability
  *
@@ -411,14 +396,19 @@ void CS_PROCF (uiprof, UIPROF)(void);
  * integer         permeability    <--  permeability type
  * integer         diffusion       <--  diffusion type
  * integer         gravity         <--  check if gravity is taken into account
+ * double          gravity_x       <--  gravity direction
+ * double          gravity_y       <--  gravity direction
+ * double          gravity_z       <--  gravity direction
+ * integer         unsaturated     <--  unsaturated zone taken into account
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (uidapp, UIDAPP) (const cs_int_t   *permeability,
-                                const cs_int_t   *diffusion,
-                                const cs_int_t   *gravity,
-                                const cs_real_t  *gravity_x,
-                                const cs_real_t  *gravity_y,
-                                const cs_real_t  *gravity_z);
+void CS_PROCF (uidapp, UIDAPP) (const int       *permeability,
+                                const int       *diffusion,
+                                const int       *gravity,
+                                const cs_real_t *gravity_x,
+                                const cs_real_t *gravity_y,
+                                const cs_real_t *gravity_z,
+                                const int       *unsaturated);
 
 /*----------------------------------------------------------------------------
  * Free memory: clean global private variables and libxml2 variables.
@@ -433,6 +423,34 @@ void CS_PROCF (uidapp, UIDAPP) (const cs_int_t   *permeability,
 
 void CS_PROCF (memui1, MEMUI1) (const int *ncharb);
 
+/*----------------------------------------------------------------------------
+ * Define fans with GUI
+ *
+ * Fortran Interface:
+ *
+ * SUBROUTINE UIFANS
+ * *****************
+ *
+ *----------------------------------------------------------------------------*/
+
+void CS_PROCF (uifans, UIFANS) (void);
+
+/*----------------------------------------------------------------------------
+ * Define error estimators
+ *
+ * Fortran Interface:
+ *
+ * SUBROUTINE UIERES
+ * *****************
+ *
+ *----------------------------------------------------------------------------*/
+
+void CS_PROCF (uieres, UIERES) (int *iescal,
+                                int *iespre,
+                                int *iesder,
+                                int *iescor,
+                                int *iestot);
+
 /*=============================================================================
  * Public function prototypes
  *============================================================================*/
@@ -451,6 +469,22 @@ cs_gui_init(void);
 void
 cs_gui_finalize(void);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Compute GUI-defined head losses for a given volume zone.
+ *
+ * Head loss tensor coefficients for each cell are organized as follows:
+ * cku11, cku22, cku33, cku12, cku13, cku23.
+ *
+ * \param[in]       zone  pointer to zone structure
+ * \param[in, out]  cku   head loss coefficients
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_gui_head_losses(const cs_zone_t  *zone,
+                   cs_real_t         cku[][6]);
+
 /*-----------------------------------------------------------------------------
  * Selection of linear solvers.
  *----------------------------------------------------------------------------*/
@@ -548,6 +582,45 @@ cs_gui_user_scalar_labels(void);
 void
 cs_gui_user_variables(void);
 
+/*----------------------------------------------------------------------------
+ * Define balance by zone through the GUI.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_balance_by_zone(void);
+
+/*----------------------------------------------------------------------------
+ * Define pressure drop through the GUI.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_pressure_drop_by_zone(void);
+
+/*----------------------------------------------------------------------------
+ * Define fans through the GUI.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_define_fans(void);
+
+/*----------------------------------------------------------------------------
+ * Define error estimator through the GUI.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_error_estimator(int *iescal,
+                       int *iespre,
+                       int *iesder,
+                       int *iescor,
+                       int *iestot);
+
+/*----------------------------------------------------------------------------
+ * Define volume and boundary zones through the GUI.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_zones(void);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
