@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -37,6 +37,8 @@
  * Local headers
  *----------------------------------------------------------------------------*/
 
+#include <ple_locator.h>
+
 #include "cs_base.h"
 
 /*----------------------------------------------------------------------------*/
@@ -63,46 +65,47 @@ typedef struct {
 } cs_meteo_t;
 
 typedef struct {
-  char      **label;       /* label for each boundary zone                    */
-  char      **nature;      /* nature for each boundary zone                   */
-  int        *iqimp;       /* 1 if a flow rate is applied                     */
-  int        *ientfu;      /* 1  for a fuel flow inlet (gas combustion - D3P) */
-  int        *ientox;      /* 1 for an air flow inlet (gas combustion - D3P)  */
-  int        *ientgb;      /* 1 for burned gas inlet (gas combustion)         */
-  int        *ientgf;      /* 1 for unburned gas inlet (gas combustion)       */
-  int        *ientat;      /* 1 if inlet for oxydant (coal combustion)        */
-  int        *ientcp;      /* 1 if inlet for oxydant+coal (coal combustion)   */
-  int        *icalke;      /* automatic boundaries for turbulent variables    */
-  double     *qimp;        /* oxydant flow rate (coal combustion)             */
-  int        *inmoxy;      /* oxydant number (coal combustion)                */
-  double     *timpat;      /* inlet temperature of oxydant (coal combustion)  */
-  double     *tkent;       /* inlet temperature (gas combustion)              */
-  double    **qimpcp;      /* inlet coal flow rate (coal combustion)          */
-  double    **timpcp;      /* inlet coal temperature (coal combustion)        */
-  double     *fment;       /* Mean Mixture Fraction at Inlet (gas combustion) */
-  int        *itype;       /* type of inlet/outlet (compressible model)       */
-  double     *prein;       /* inlet pressure (compressible model)             */
-  double     *rhoin;       /* inlet density  (compressible model)             */
-  double     *tempin;      /* inlet temperature (compressible model)          */
-  double     *entin;       /* inlet total energy (compressible model)         */
-  double     *preout;      /* outlet pressure for subsonic(compressible model)*/
-  double     *dh;          /* inlet hydraulic diameter                        */
-  double     *xintur;      /* inlet turbulent intensity                       */
-  int       **type_code;   /* type of boundary for each variables             */
-  cs_val_t  **values;      /* fortran array RCODCL mapping                    */
-  double   ***distch;      /* ratio for each coal                             */
-  double     *rough;       /* roughness size                                  */
-  double     *norm;        /* norm of velocity vector                         */
-  double     *dirx;        /* directions x inlet velocity                     */
-  double     *diry;        /* directions y inlet velocity                     */
-  double     *dirz;        /* directions z inlet velocity                     */
-  mei_tree_t **velocity;   /* formula for norm or mass flow rate of velocity  */
-  mei_tree_t **direction;  /* formula for direction of velocity               */
-  cs_meteo_t  *meteo;      /* inlet or outlet info for atmospheric flow       */
-  mei_tree_t ***scalar;    /* formula for scalar (neumann, dirichlet or
-                              exchange coefficient)*/
-  mei_tree_t **headLoss;   /* formula for head loss (free inlet/outlet)       */
-  mei_tree_t **groundwat;  /* formula for hydraulic head (groundwater)        */
+  char         **label;       /* label for each boundary zone                    */
+  char         **nature;      /* nature for each boundary zone                   */
+  int           *iqimp;       /* 1 if a flow rate is applied                     */
+  int           *ientfu;      /* 1  for a fuel flow inlet (gas combustion - D3P) */
+  int           *ientox;      /* 1 for an air flow inlet (gas combustion - D3P)  */
+  int           *ientgb;      /* 1 for burned gas inlet (gas combustion)         */
+  int           *ientgf;      /* 1 for unburned gas inlet (gas combustion)       */
+  int           *ientat;      /* 1 if inlet for oxydant (coal combustion)        */
+  int           *ientcp;      /* 1 if inlet for oxydant+coal (coal combustion)   */
+  int           *icalke;      /* automatic boundaries for turbulent variables    */
+  double        *qimp;        /* oxydant flow rate (coal combustion)             */
+  int           *inmoxy;      /* oxydant number (coal combustion)                */
+  double        *timpat;      /* inlet temperature of oxydant (coal combustion)  */
+  double        *tkent;       /* inlet temperature (gas combustion)              */
+  double       **qimpcp;      /* inlet coal flow rate (coal combustion)          */
+  double       **timpcp;      /* inlet coal temperature (coal combustion)        */
+  double        *fment;       /* Mean Mixture Fraction at Inlet (gas combustion) */
+  int           *itype;       /* type of inlet/outlet (compressible model)       */
+  double        *prein;       /* inlet pressure (compressible model)             */
+  double        *rhoin;       /* inlet density  (compressible model)             */
+  double        *tempin;      /* inlet temperature (compressible model)          */
+  double        *entin;       /* inlet total energy (compressible model)         */
+  double        *preout;      /* outlet pressure for subsonic(compressible model)*/
+  double        *dh;          /* inlet hydraulic diameter                        */
+  double        *xintur;      /* inlet turbulent intensity                       */
+  int          **type_code;   /* type of boundary for each variables             */
+  cs_val_t     **values;      /* fortran array RCODCL mapping                    */
+  double      ***distch;      /* ratio for each coal                             */
+  double        *rough;       /* roughness size                                  */
+  double        *norm;        /* norm of velocity vector                         */
+  double        *dirx;        /* directions x inlet velocity                     */
+  double        *diry;        /* directions y inlet velocity                     */
+  double        *dirz;        /* directions z inlet velocity                     */
+  mei_tree_t    **velocity;   /* formula for norm or mass flow rate of velocity  */
+  mei_tree_t    **direction;  /* formula for direction of velocity               */
+  cs_meteo_t     *meteo;      /* inlet or outlet info for atmospheric flow       */
+  mei_tree_t    ***scalar;    /* formula for scalar (neumann, dirichlet or
+                                 exchange coefficient)*/
+  mei_tree_t    **headLoss;   /* formula for head loss (free inlet/outlet)       */
+  mei_tree_t    **groundwat;  /* formula for hydraulic head (groundwater)        */
+  ple_locator_t **locator;    /* locator for inlet mapped                        */
 } cs_boundary_t;
 
 /*============================================================================
@@ -127,25 +130,11 @@ extern cs_boundary_t *boundaries;
  * *****************
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (uiclim, UICLIM)(const int  *ntcabs,
-                               const int  *nfabor,
-                               const int  *idarcy,
+void CS_PROCF (uiclim, UICLIM)(const int  *idarcy,
                                const int  *nozppm,
                                const int  *ncharm,
                                const int  *ncharb,
                                const int  *nclpch,
-                               const int  *iindef,
-                               const int  *ientre,
-                               const int  *iesicf,
-                               const int  *isspcf,
-                               const int  *iephcf,
-                               const int  *isopcf,
-                               const int  *iparoi,
-                               const int  *iparug,
-                               const int  *isymet,
-                               const int  *isolib,
-                               const int  *ifrent,
-                               const int  *ifresf,
                                int        *iqimp,
                                int        *icalke,
                                int        *ientat,
@@ -156,11 +145,10 @@ void CS_PROCF (uiclim, UICLIM)(const int  *ntcabs,
                                int        *ientgf,
                                int        *ientgb,
                                int        *iprofm,
+                               int        *iautom,
                                int        *itypfb,
                                int        *izfppp,
                                int        *icodcl,
-                               double     *dtref,
-                               double     *ttcabs,
                                double     *surfbo,
                                double     *cdgfbo,
                                double     *qimp,
@@ -173,7 +161,7 @@ void CS_PROCF (uiclim, UICLIM)(const int  *ntcabs,
                                double     *tkent,
                                double     *fment,
                                double     *distch,
-                               int        *nvarcl,
+                               int        *nvar,
                                double     *rcodcl);
 
 /*----------------------------------------------------------------------------
@@ -184,40 +172,13 @@ void CS_PROCF (uiclim, UICLIM)(const int  *ntcabs,
  * SUBROUTINE UICLVE
  * *****************
  *
- * integer          nfabor  <-- number of boundary faces
  * integer          nozppm  <-- max number of boundary conditions zone
- * integer          iindef  <-- type of boundary: not defined
- * integer          ientre  <-- type of boundary: inlet
- * integer          iesicf  --> type of boundary: imposed inlet (compressible)
- * integer          isspcf  --> type of boundary: supersonic outlet (compressible)
- * integer          iephcf  --> type of boundary: subsonic inlet at imposed
- *                              total pressure and enthalpy (compressible)
- * integer          isopcf  --> type of boundary: subsonic outlet (compressible)
- * integer          iparoi  <-- type of boundary: smooth wall
- * integer          iparug  <-- type of boundary: rough wall
- * integer          isymet  <-- type of boundary: symetry
- * integer          isolib  <-- type of boundary: outlet
- * integer          ifrent  <-- type of boundary: free inlet outlet
- * integer          ifresf  <-- type of boundary: free surface
  * integer          iale    <-- ale module activated
  * integer          itypfb  <-- type of boundary for each face
  * integer          izfppp  <-- zone number
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (uiclve, UICLVE)(const int  *nfabor,
-                               const int  *nozppm,
-                               const int  *iindef,
-                               const int  *ientre,
-                               const int  *iesicf,
-                               const int  *iephcf,
-                               const int  *isspcf,
-                               const int  *isopcf,
-                               const int  *iparoi,
-                               const int  *iparug,
-                               const int  *isymet,
-                               const int  *isolib,
-                               const int  *ifrent,
-                               const int  *ifresf,
+void CS_PROCF (uiclve, UICLVE)(const int  *nozppm,
                                const int  *iale,
                                int        *itypfb,
                                int        *izfppp);
@@ -268,19 +229,16 @@ cs_gui_boundary_zone_localization(const char *label);
  * Helper to get the face list for the izone
  *
  * parameters:
- *   izone   <--  zone index
- *   label   <--  boundary label
- *   nfabor  <--  number of boundary faces
- *   nozppm  <--  max number of boundary zone for preefined physics
- *   faces   -->  number of faces
+ *   label     <--  boundary label
+ *   n_faces   -->  number of faces
+ *
+ * returns:
+ *   pointer to face list
  *----------------------------------------------------------------------------*/
 
-cs_lnum_t*
-cs_gui_get_faces_list(int          izone,
-                      const char  *label,
-                      cs_lnum_t    nfabor,
-                      int          nozppm,
-                      cs_lnum_t   *faces);
+const cs_lnum_t *
+cs_gui_get_boundary_faces(const char   *label,
+                          cs_lnum_t    *n_faces);
 
 /*----------------------------------------------------------------------------
  * Free boundary conditions structures
