@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -49,6 +49,7 @@
 #include "mei_evaluate.h"
 
 #include "cs_base.h"
+#include "cs_field_pointer.h"
 #include "cs_gui_util.h"
 #include "cs_gui_variables.h"
 #include "cs_gui_boundary_conditions.h"
@@ -86,6 +87,7 @@ BEGIN_C_DECLS
 
 enum ale_boundary_nature
 {
+  ale_boundary_nature_none,
   ale_boundary_nature_fixed_wall,
   ale_boundary_nature_sliding_wall,
   ale_boundary_nature_internal_coupling,
@@ -164,6 +166,43 @@ _get_ale_status(int  *const keyword)
   BFT_FREE(path);
 }
 
+/*-----------------------------------------------------------------------------
+ * add notebook variable to formula
+ *----------------------------------------------------------------------------*/
+
+static void
+_add_notebook_variables(mei_tree_t *ev_law)
+{
+  char *path = NULL;
+
+  /* number of variable */
+  int nbvar = cs_gui_get_tag_count("/physical_properties/notebook/var\n", 1);
+
+  if (nbvar == 0)
+    return;
+
+  for (int ivar = 0; ivar < nbvar; ivar++) {
+    path = cs_xpath_init_path();
+    cs_xpath_add_elements(&path, 2, "physical_properties", "notebook");
+    cs_xpath_add_element_num(&path, "var", ivar +1);
+    cs_xpath_add_attribute(&path, "name");
+    char *name = cs_gui_get_attribute_value(path);
+    BFT_FREE(path);
+
+    path = cs_xpath_init_path();
+    cs_xpath_add_elements(&path, 2, "physical_properties", "notebook");
+    cs_xpath_add_element_num(&path, "var", ivar +1);
+    cs_xpath_add_attribute(&path, "value");
+    char *value = cs_gui_get_attribute_value(path);
+    double val = atof(value);
+    BFT_FREE(path);
+
+    mei_tree_insert(ev_law, name, val);
+    BFT_FREE(name);
+    BFT_FREE(value);
+  }
+}
+
 /*-----------------------------------------------------------------------------
  * Initialize mei tree and check for symbols existence
  *
@@ -179,15 +218,15 @@ _get_ale_status(int  *const keyword)
  *----------------------------------------------------------------------------*/
 
 static mei_tree_t *
-cs_gui_init_mei_tree(char         *formula,
-                     const char   **symbols,
-                           int    symbol_nbr,
-                     const char   **variables,
-                     const double *variables_value,
-                           int    variable_nbr,
-                     const double dtref,
-                     const double ttcabs,
-                     const int    ntcabs)
+_init_mei_tree(char          *formula,
+               const char   **symbols,
+               int            symbol_nbr,
+               const char   **variables,
+               const double  *variables_value,
+               int            variable_nbr,
+               const double   dtref,
+               const double   ttcabs,
+               const int      ntcabs)
 {
   int i = 0;
 
@@ -209,6 +248,9 @@ cs_gui_init_mei_tree(char         *formula,
   mei_tree_insert(tree, "t",    ttcabs);
   mei_tree_insert(tree, "iter", ntcabs);
 
+  /* add variable from notebook */
+  _add_notebook_variables(tree);
+
   /* try to build the interpreter */
   if (mei_tree_builder(tree))
     bft_error(__FILE__, __LINE__, 0,
@@ -335,8 +377,8 @@ _uialcl_fixed_displacement(const char       *label,
               _("Boundary nature formula is null for %s.\n"), label);
 
   /* Init mei */
-  ev = cs_gui_init_mei_tree(formula, variables, variable_nbr,
-                            0, 0, 0, dtref, ttcabs, ntcabs);
+  ev = _init_mei_tree(formula, variables, variable_nbr,
+                      0, 0, 0, dtref, ttcabs, ntcabs);
 
   mei_evaluate(ev);
 
@@ -364,29 +406,29 @@ _uialcl_fixed_displacement(const char       *label,
  * Get uialcl data for fixed velocity
  *
  * parameters:
- *   label        --> boundary label
- *   iuma         --> IUMA
- *   ivma         --> IVMA
- *   iwma         --> IWMA
- *   nfabor       --> Number of boundary faces
- *   ifbr         --> ifbr
- *   rcodcl       <-- RCODCL
- *   dtref        --> time step
- *   ttcabs       --> current time
- *   ntcabs       --> current iteration number
+ *   label        <-- boundary label
+ *   iuma         <-- IUMA
+ *   ivma         <-- IVMA
+ *   iwma         <-- IWMA
+ *   nfabor       <-- Number of boundary faces
+ *   ifbr         <-- ifbr
+ *   rcodcl       --@ RCODCL
+ *   dtref        <-- time step
+ *   ttcabs       <-- current time
+ *   ntcabs       <-- current iteration number
  *----------------------------------------------------------------------------*/
 
 static void
-uialcl_fixed_velocity(const char*   label,
-                      const int     iuma,
-                      const int     ivma,
-                      const int     iwma,
-                      const int     nfabor,
-                      const int     ifbr,
-                      double *const rcodcl,
-                      const double  dtref,
-                      const double  ttcabs,
-                      const int     ntcabs)
+_uialcl_fixed_velocity(const char*   label,
+                       const int     iuma,
+                       const int     ivma,
+                       const int     iwma,
+                       const int     nfabor,
+                       const int     ifbr,
+                       double *const rcodcl,
+                       const double  dtref,
+                       const double  ttcabs,
+                       const int     ntcabs)
 {
   mei_tree_t *ev;
   const char*  variables[3] = { "mesh_velocity_U",
@@ -401,8 +443,8 @@ uialcl_fixed_velocity(const char*   label,
               _("Boundary nature formula is null for %s.\n"), label);
 
   /* Init MEI */
-  ev = cs_gui_init_mei_tree(formula, variables, 3, 0, 0, 0,
-                            dtref, ttcabs, ntcabs);
+  ev = _init_mei_tree(formula, variables, 3, 0, 0, 0,
+                      dtref, ttcabs, ntcabs);
 
   mei_evaluate(ev);
 
@@ -419,15 +461,15 @@ uialcl_fixed_velocity(const char*   label,
  * Return the ale boundary nature
  *
  * parameters:
- *   label  -->  label of boundary zone
+ *   label  <--  label of boundary zone
  *----------------------------------------------------------------------------*/
 
 static enum ale_boundary_nature
 _get_ale_boundary_nature(const char *const label)
 {
-  char *ale_boundary_nature;
+  char *nat;
 
-  enum ale_boundary_nature nature = ale_boundary_nature_fixed_wall;
+  enum ale_boundary_nature nature = ale_boundary_nature_none;
 
   char *path = cs_xpath_init_path();
   cs_xpath_add_elements(&path, 2, "boundary_conditions",  "wall");
@@ -435,23 +477,23 @@ _get_ale_boundary_nature(const char *const label)
   cs_xpath_add_test_attribute(&path, "label", label);
   cs_xpath_add_element(&path, "ale");
   cs_xpath_add_attribute(&path, "choice");
-  ale_boundary_nature = cs_gui_get_attribute_value(path);
+  nat = cs_gui_get_attribute_value(path);
 
-  if (cs_gui_strcmp(ale_boundary_nature, "fixed_boundary"))
+  if (cs_gui_strcmp(nat, "fixed_boundary"))
     nature = ale_boundary_nature_fixed_wall;
-  if (cs_gui_strcmp(ale_boundary_nature, "sliding_boundary"))
+  if (cs_gui_strcmp(nat, "sliding_boundary"))
     nature = ale_boundary_nature_sliding_wall;
-  else if (cs_gui_strcmp(ale_boundary_nature, "internal_coupling"))
+  else if (cs_gui_strcmp(nat, "internal_coupling"))
     nature = ale_boundary_nature_internal_coupling;
-  else if (cs_gui_strcmp(ale_boundary_nature, "external_coupling"))
+  else if (cs_gui_strcmp(nat, "external_coupling"))
     nature = ale_boundary_nature_external_coupling;
-  else if (cs_gui_strcmp(ale_boundary_nature, "fixed_velocity"))
+  else if (cs_gui_strcmp(nat, "fixed_velocity"))
     nature = ale_boundary_nature_fixed_velocity;
-  else if (cs_gui_strcmp(ale_boundary_nature, "fixed_displacement"))
+  else if (cs_gui_strcmp(nat, "fixed_displacement"))
     nature = ale_boundary_nature_fixed_displacement;
 
   BFT_FREE(path);
-  BFT_FREE(ale_boundary_nature);
+  BFT_FREE(nat);
 
   return nature;
 }
@@ -460,13 +502,13 @@ _get_ale_boundary_nature(const char *const label)
  * Get boundary attribute like nature or label
  *
  * parameters:
- *   ith_zone  --> boundary index
- *   nodeName  --> xml attribute name. for example, "nature" or "label"
+ *   ith_zone  <-- boundary index
+ *   nodeName  <-- xml attribute name. for example, "nature" or "label"
  *----------------------------------------------------------------------------*/
 
-static char*
-get_boundary_attribute(      int  ith_zone,
-                       const char *nodeName)
+static char *
+_get_boundary_attribute(int         ith_zone,
+                        const char *nodeName)
 {
   char *result;
   char *path = cs_xpath_init_path();
@@ -488,15 +530,15 @@ get_boundary_attribute(      int  ith_zone,
  * ale[choice=internal_coupling]/node_name, node_sub_name/text()
  *
  * parameters:
- *   label            --> boundary label
- *   node_name        --> xml node name ("initial_displacement")
- *   node_sub_name    --> xml child node of node_name ("X")
+ *   label            <-- boundary label
+ *   node_name        <-- xml node name ("initial_displacement")
+ *   node_sub_name    <-- xml child node of node_name ("X")
  *----------------------------------------------------------------------------*/
 
 static char*
-init_internal_coupling_xpath(const char* label,
-                             const char* node_name,
-                             const char* node_sub_name)
+_init_internal_coupling_xpath(const char *label,
+                              const char *node_name,
+                              const char *node_sub_name)
 {
   char *path = NULL;
 
@@ -517,18 +559,18 @@ init_internal_coupling_xpath(const char* label,
  * Get internal coupling double
  *
  * parameters:
- *   label            --> boundary label
- *   node_name        --> xml node name ("initial_displacement")
- *   node_sub_name    --> xml child node of node_name ("X")
+ *   label            <-- boundary label
+ *   node_name        <-- xml node name ("initial_displacement")
+ *   node_sub_name    <-- xml child node of node_name ("X")
  *----------------------------------------------------------------------------*/
 
 static double
-get_internal_coupling_double(const char* label,
-                             const char* node_name,
-                             const char* node_sub_name)
+_get_internal_coupling_double(const char *label,
+                              const char *node_name,
+                              const char *node_sub_name)
 {
   double value = 0;
-  char *path = init_internal_coupling_xpath(label, node_name, node_sub_name);
+  char *path = _init_internal_coupling_xpath(label, node_name, node_sub_name);
 
   if (!cs_gui_get_double(path, &value))
   {
@@ -545,18 +587,18 @@ get_internal_coupling_double(const char* label,
  * Get internal coupling string
  *
  * parameters:
- *   label            --> boundary label
- *   node_name        --> xml node name ("initial_displacement")
- *   node_sub_name    --> xml child node of node_name ("formula")
+ *   label            <-- boundary label
+ *   node_name        <-- xml node name ("initial_displacement")
+ *   node_sub_name    <-- xml child node of node_name ("formula")
  *----------------------------------------------------------------------------*/
 
 static char*
-get_internal_coupling_string(const char* label,
-                             const char* node_name,
-                             const char* node_sub_name)
+_get_internal_coupling_string(const char *label,
+                              const char *node_name,
+                              const char *node_sub_name)
 {
-  char *path = init_internal_coupling_xpath(label, node_name, node_sub_name);
-  char* str  = cs_gui_get_text_value(path);
+  char *path = _init_internal_coupling_xpath(label, node_name, node_sub_name);
+  char *str  = cs_gui_get_text_value(path);
 
   BFT_FREE(path);
 
@@ -568,31 +610,31 @@ get_internal_coupling_string(const char* label,
  * Retreive internal coupling x, y and z XML values
  *
  * parameters:
- *   label      --> boundary label
- *   node_name  --> xml node name ("initial_displacement")
- *   xyz        <-- result matrix
+ *   label      <-- boundary label
+ *   node_name  <-- xml node name ("initial_displacement")
+ *   xyz        --@ result matrix
  *----------------------------------------------------------------------------*/
 
 static void
-get_internal_coupling_xyz_values(const char *label,
-                                 const char *node_name,
-                                 double      xyz[3])
+_get_internal_coupling_xyz_values(const char *label,
+                                  const char *node_name,
+                                  double      xyz[3])
 {
-  xyz[0] = get_internal_coupling_double(label, node_name, "X");
-  xyz[1] = get_internal_coupling_double(label, node_name, "Y");
-  xyz[2] = get_internal_coupling_double(label, node_name, "Z");
+  xyz[0] = _get_internal_coupling_double(label, node_name, "X");
+  xyz[1] = _get_internal_coupling_double(label, node_name, "Y");
+  xyz[2] = _get_internal_coupling_double(label, node_name, "Z");
 }
 
 /*-----------------------------------------------------------------------------
  * Retreive internal coupling advanced windows double value
  *
  * parameters:
- *   node_name  --> xml node name ("displacement_prediction_alpha")
+ *   node_name  <-- xml node name ("displacement_prediction_alpha")
  *----------------------------------------------------------------------------*/
 
 static void
-get_uistr1_advanced_double(const char *const keyword,
-                               double *const value)
+_get_uistr1_advanced_double(const char *const keyword,
+                            double     *const value)
 {
   double result = 0;
   char   *path = cs_xpath_init_path();
@@ -609,11 +651,12 @@ get_uistr1_advanced_double(const char *const keyword,
  * Retreive internal coupling advanced windows checkbox value
  *
  * parameters:
- *   node_name  --> xml node name ("monitor_point_synchronisation")
+ *   node_name  <-- xml node name ("monitor_point_synchronisation")
  *----------------------------------------------------------------------------*/
 
 static void
-get_uistr1_advanced_checkbox(const char *const keyword, int *const value)
+_get_uistr1_advanced_checkbox(const char *const keyword,
+                              int        *const value)
 {
   int result = 0;
   char *path = cs_xpath_init_path();
@@ -630,46 +673,46 @@ get_uistr1_advanced_checkbox(const char *const keyword, int *const value)
  * Retreive data the internal coupling matrices
  *
  * parameters:
- *   label            --> boundary label
- *   node_name        --> xml matrix node name
- *   symbols          --> see cs_gui_init_mei_tree
- *   symbol_nbr       --> see cs_gui_init_mei_tree
- *   variables        --> see cs_gui_init_mei_tree
- *   variables_value  --> see cs_gui_init_mei_tree
- *   variable_nbr     --> see cs_gui_init_mei_tree
- *   output_matrix,   <-- result matrix
- *   dtref            --> time step
- *   ttcabs           --> current time
- *   ntcabs           --> current iteration number
+ *   label            <-- boundary label
+ *   node_name        <-- xml matrix node name
+ *   symbols          <-- see _init_mei_tree
+ *   symbol_nbr       <-- see _init_mei_tree
+ *   variables        <-- see _init_mei_tree
+ *   variables_value  <-- see _init_mei_tree
+ *   variable_nbr     <-- see _init_mei_tree
+ *   output_matrix,   --> result matrix
+ *   dtref            <-- time step
+ *   ttcabs           <-- current time
+ *   ntcabs           <-- current iteration number
  *----------------------------------------------------------------------------*/
 
 static void
-get_internal_coupling_matrix(const char    *label,
-                             const char    *node_name,
-                             const char    *symbols[],
-                                   int     symbol_nbr,
-                             const char    **variables,
-                             const double  *variables_value,
-                                   int     variable_nbr,
-                                   double  *output_matrix,
-                             const double  dtref,
-                             const double  ttcabs,
-                             const int     ntcabs)
+_get_internal_coupling_matrix(const char    *label,
+                              const char    *node_name,
+                              const char    *symbols[],
+                              int            symbol_nbr,
+                              const char   **variables,
+                              const double  *variables_value,
+                              int            variable_nbr,
+                              double        *output_matrix,
+                              const double   dtref,
+                              const double   ttcabs,
+                              const int      ntcabs)
 {
   /* Get the formula */
   mei_tree_t *tree;
 
   int i = 0;
-  char *matrix = get_internal_coupling_string(label, node_name, "formula");
+  char *matrix = _get_internal_coupling_string(label, node_name, "formula");
 
   if (!matrix)
     bft_error(__FILE__, __LINE__, 0,
               _("Formula is null for %s %s"), label, node_name);
 
   /* Initialize mei */
-  tree = cs_gui_init_mei_tree(matrix, symbols, symbol_nbr,
-                              variables, variables_value, variable_nbr,
-                              dtref, ttcabs, ntcabs);
+  tree = _init_mei_tree(matrix, symbols, symbol_nbr,
+                        variables, variables_value, variable_nbr,
+                        dtref, ttcabs, ntcabs);
   mei_evaluate(tree);
 
   /* Read matrix values */
@@ -685,27 +728,27 @@ get_internal_coupling_matrix(const char    *label,
  * Retreive data for internal coupling for a specific boundary
  *
  * parameters:
- *   label    --> boundary label
- *   xmstru   <-- Mass matrix
- *   xcstr    <-- Damping matrix
- *   xkstru   <-- Stiffness matrix
- *   forstr   <-- Fluid force matrix
- *   istruc   --> internal coupling boundary index
- *   dtref    --> time step
- *   ttcabs   --> current time
- *   ntcabs   --> current iteration number
+ *   label    <-- boundary label
+ *   xmstru   --> Mass matrix
+ *   xcstr    --> Damping matrix
+ *   xkstru   --> Stiffness matrix
+ *   forstr   --> Fluid force matrix
+ *   istruc   <-- internal coupling boundary index
+ *   dtref    <-- time step
+ *   ttcabs   <-- current time
+ *   ntcabs   <-- current iteration number
  *----------------------------------------------------------------------------*/
 
 static void
-get_uistr2_data(const char    *label,
-                double *const xmstru,
-                double *const xcstru,
-                double *const xkstru,
-                double *const forstr,
-                      int     istruc,
-                const double  dtref,
-                const double  ttcabs,
-                const int     ntcabs)
+_get_uistr2_data(const char    *label,
+                 double *const xmstru,
+                 double *const xcstru,
+                 double *const xkstru,
+                 double *const forstr,
+                 int           istruc,
+                 const double  dtref,
+                 const double  ttcabs,
+                 const int     ntcabs)
 {
   const char  *m_symbols[] = {"m11", "m12", "m13",
                               "m21", "m22", "m23",
@@ -728,20 +771,20 @@ get_uistr2_data(const char    *label,
 
   /* Get mass matrix, damping matrix and stiffness matrix */
 
-  get_internal_coupling_matrix(label, "mass_matrix", m_symbols,
-                               symbol_nbr, 0, 0, 0,
-                               &xmstru[istruc * symbol_nbr],
-                               dtref, ttcabs, ntcabs);
+  _get_internal_coupling_matrix(label, "mass_matrix", m_symbols,
+                                symbol_nbr, 0, 0, 0,
+                                &xmstru[istruc * symbol_nbr],
+                                dtref, ttcabs, ntcabs);
 
-  get_internal_coupling_matrix(label, "damping_matrix", c_symbols,
-                               symbol_nbr, 0, 0, 0,
-                               &xcstru[istruc * symbol_nbr],
-                               dtref, ttcabs, ntcabs);
+  _get_internal_coupling_matrix(label, "damping_matrix", c_symbols,
+                                symbol_nbr, 0, 0, 0,
+                                &xcstru[istruc * symbol_nbr],
+                                dtref, ttcabs, ntcabs);
 
-  get_internal_coupling_matrix(label, "stiffness_matrix", k_symbols,
-                               symbol_nbr, 0, 0, 0,
-                               &xkstru[istruc * symbol_nbr],
-                               dtref, ttcabs, ntcabs);
+  _get_internal_coupling_matrix(label, "stiffness_matrix", k_symbols,
+                                symbol_nbr, 0, 0, 0,
+                                &xkstru[istruc * symbol_nbr],
+                                dtref, ttcabs, ntcabs);
 
   /* Set variable for fluid force matrix */
   variable_values[0] = forstr[istruc * force_symbol_nbr + 0];
@@ -749,11 +792,48 @@ get_uistr2_data(const char    *label,
   variable_values[2] = forstr[istruc * force_symbol_nbr + 2];
 
   /* Get fluid force matrix */
-  get_internal_coupling_matrix(label, "fluid_force_matrix",
-                               force_symbols, force_symbol_nbr,
-                               variables, variable_values, variable_nbr,
-                               &forstr[istruc * force_symbol_nbr],
-                               dtref, ttcabs, ntcabs);
+  _get_internal_coupling_matrix(label, "fluid_force_matrix",
+                                force_symbols, force_symbol_nbr,
+                                variables, variable_values, variable_nbr,
+                                &forstr[istruc * force_symbol_nbr],
+                                dtref, ttcabs, ntcabs);
+}
+
+/*-----------------------------------------------------------------------------
+ * Return the external coupling dof ("DDL") value
+ *
+ *  <boundary_conditions>
+ *      <wall label=label_argument">
+ *          <ale choice="external_coupling">
+ *              <node_name_argument choice="off"/>
+ *
+ * parameters:
+ *   label     <-- boundary label
+ *   node_name <--  Node name: DDLX, DDLY or DDLZ.
+ *----------------------------------------------------------------------------*/
+
+static int
+_get_external_coupling_dof(const char *const label,
+                           const char *const node_name)
+{
+    char* choice;
+    int isOn;
+
+    char *path = cs_xpath_init_path();
+    cs_xpath_add_elements(&path, 2, "boundary_conditions", "wall");
+    cs_xpath_add_test_attribute(&path, "label", label);
+    cs_xpath_add_element(&path, "ale");
+    cs_xpath_add_test_attribute(&path, "choice", "external_coupling");
+    cs_xpath_add_element(&path, node_name);
+    cs_xpath_add_attribute(&path, "choice");
+
+    choice = cs_gui_get_attribute_value(path);
+    isOn = cs_gui_strcmp(choice, "on");
+
+    BFT_FREE(choice);
+    BFT_FREE(path);
+
+    return isOn;
 }
 
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
@@ -764,25 +844,20 @@ get_uistr2_data(const char    *label,
 
 /*----------------------------------------------------------------------------
  *  uivima
- *
- * viscmx   <--  VISCMX
- * viscmy   <--  VISCMY
- * viscmz   <--  VISCMZ
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (uivima, UIVIMA) (double         *const viscmx,
-                                double         *const viscmy,
-                                double         *const viscmz)
+void CS_PROCF (uivima, UIVIMA) (void)
 {
   const cs_real_3_t *restrict cell_cen
     = (const cs_real_3_t *restrict)cs_glob_mesh_quantities->cell_cen;
   const cs_lnum_t n_cells     = cs_glob_mesh->n_cells;
-  int          iel            = 0;
-  const char*  symbols[3]     = { "x", "y", "z" };
-  const char*  variables[3]   = { "mesh_viscosity_1",
+  cs_lnum_t    iel            = 0;
+  const char * symbols[3]     = { "x", "y", "z" };
+  const char * variables[3]   = { "mesh_viscosity_1",
                                   "mesh_viscosity_2",
                                   "mesh_viscosity_3" };
   int variable_nbr   = 1;
+  int ivar           = 0;
 
   /* Get formula */
   mei_tree_t *ev;
@@ -796,21 +871,19 @@ void CS_PROCF (uivima, UIVIMA) (double         *const viscmx,
   if (!aleFormula) {
     bft_printf("Warning : Formula is null for ale. Use constant value\n");
     for (iel = 0; iel < n_cells; iel++) {
-      viscmx[iel] = 1.;
-      if (isOrthotrop) {
-        viscmy[iel] = 1.;
-        viscmz[iel] = 1.;
+      for (ivar = 0; ivar < variable_nbr; ivar++) {
+        CS_F_(vism)->val[variable_nbr*iel + ivar] = 1.;
       }
     }
   }
   else {
 
     /* Init mei */
-    ev = cs_gui_init_mei_tree(aleFormula, variables,
-                              variable_nbr, symbols, 0, 3,
-                              cs_glob_time_step_options->dtref,
-                              cs_glob_time_step->t_cur,
-                              cs_glob_time_step->nt_cur);
+    ev = _init_mei_tree(aleFormula, variables,
+                        variable_nbr, symbols, 0, 3,
+                        cs_glob_time_step_options->dtref,
+                        cs_glob_time_step->t_cur,
+                        cs_glob_time_step->nt_cur);
 
     /* for each cell, update the value of the table of symbols for each scalar
        (including the thermal scalar), and evaluate the interpreter */
@@ -822,11 +895,11 @@ void CS_PROCF (uivima, UIVIMA) (double         *const viscmx,
 
       mei_evaluate(ev);
 
-      /* Set viscmx, viscmy and viscmz */
-      viscmx[iel] = mei_tree_lookup(ev, "mesh_viscosity_1");
+      /* Set mesh_u components */
+      CS_F_(vism)->val[variable_nbr*iel] = mei_tree_lookup(ev, "mesh_viscosity_1");
       if (isOrthotrop) {
-        viscmy[iel] = mei_tree_lookup(ev, "mesh_viscosity_2");
-        viscmz[iel] = mei_tree_lookup(ev, "mesh_viscosity_3");
+        CS_F_(vism)->val[variable_nbr*iel + 1] = mei_tree_lookup(ev, "mesh_viscosity_2");
+        CS_F_(vism)->val[variable_nbr*iel + 2] = mei_tree_lookup(ev, "mesh_viscosity_3");
       }
     }
     mei_tree_destroy(ev);
@@ -893,33 +966,25 @@ void CS_PROCF (uialin, UIALIN) (int    *const iale,
 }
 
 /*-----------------------------------------------------------------------------
- *  uialcl
+ * uialcl
  *
  * parameters:
- *   nozppm       --> Max number of boundary conditions zone
- *   ialtyb       --> ialtyb
- *   impale       --> uialcl_fixed_displacement
- *   disale       --> See uialcl_fixed_displacement
- *   dtref        --> time step
- *   ttcabs       --> current time
- *   ntcabs       --> current iteration number
- *   iuma         --> See uialcl_fixed_velocity
- *   ivma         --> See uialcl_fixed_velocity
- *   iwma         --> See uialcl_fixed_velocity
- *   rcodcl       --> See uialcl_fixed_velocity
+ *   ialtyb       <-- ialtyb
+ *   impale       <-- uialcl_fixed_displacement
+ *   disale       <-- See uialcl_fixed_displacement
+ *   iuma         <-- See uialcl_fixed_velocity
+ *   ivma         <-- See uialcl_fixed_velocity
+ *   iwma         <-- See uialcl_fixed_velocity
+ *   rcodcl       <-- See uialcl_fixed_velocity
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (uialcl, UIALCL) (const int *const    nozppm,
-                                const int *const    ibfixe,
+void CS_PROCF (uialcl, UIALCL) (const int *const    ibfixe,
                                 const int *const    igliss,
                                 const int *const    ivimpo,
                                 const int *const    ifresf,
                                 int       *const    ialtyb,
                                 int       *const    impale,
                                 cs_real_3_t        *disale,
-                                double    *const    dtref,
-                                double    *const    ttcabs,
-                                const int *const    ntcabs,
                                 const int *const    iuma,
                                 const int *const    ivma,
                                 const int *const    iwma,
@@ -935,27 +1000,24 @@ void CS_PROCF (uialcl, UIALCL) (const int *const    nozppm,
   int zones = cs_gui_boundary_zones_number();
 
   /* At each time-step, loop on boundary faces: */
-  for (izone=0 ; izone < zones ; izone++) {
-    cs_lnum_t* faces_list = cs_gui_get_faces_list(izone,
-                                                  boundaries->label[izone],
-                                                  m->n_b_faces,
-                                                  *nozppm,
-                                                  &faces);
+  for (izone = 0; izone < zones; izone++) {
+    const cs_lnum_t *faces_list
+      = cs_gui_get_boundary_faces(boundaries->label[izone], &faces);
 
     /* get the ale choice */
-    const char* label = boundaries->label[izone];
+    const char *label = boundaries->label[izone];
     enum ale_boundary_nature nature =_get_ale_boundary_nature(label);
 
     if (nature ==  ale_boundary_nature_fixed_wall) {
       for (ifac = 0; ifac < faces; ifac++) {
         cs_lnum_t ifbr = faces_list[ifac];
-        ialtyb[ifbr]  = *ibfixe;
+        ialtyb[ifbr] = *ibfixe;
       }
     }
     else if (nature ==  ale_boundary_nature_sliding_wall) {
       for (ifac = 0; ifac < faces; ifac++) {
         cs_lnum_t ifbr = faces_list[ifac];
-        ialtyb[ifbr]  = *igliss;
+        ialtyb[ifbr] = *igliss;
       }
     }
     else if (nature == ale_boundary_nature_fixed_displacement) {
@@ -966,7 +1028,9 @@ void CS_PROCF (uialcl, UIALCL) (const int *const    nozppm,
                                    m->b_face_vtx_idx[ifbr],
                                    m->b_face_vtx_idx[ifbr+1],
                                    m->b_face_vtx_lst, impale, disale,
-                                   *dtref, *ttcabs, *ntcabs);
+                                   cs_glob_time_step_options->dtref,
+                                   cs_glob_time_step->t_cur,
+                                   cs_glob_time_step->nt_cur);
       }
       cs_gui_add_mei_time(cs_timer_wtime() - t0);
     }
@@ -974,15 +1038,18 @@ void CS_PROCF (uialcl, UIALCL) (const int *const    nozppm,
       t0 = cs_timer_wtime();
       for (ifac = 0; ifac < faces; ifac++) {
         cs_lnum_t ifbr = faces_list[ifac];
-        uialcl_fixed_velocity(label, *iuma, *ivma, *iwma,
-                              m->n_b_faces, ifbr, rcodcl,
-                              *dtref, *ttcabs, *ntcabs);
+        _uialcl_fixed_velocity(label, *iuma, *ivma, *iwma,
+                               m->n_b_faces, ifbr, rcodcl,
+                               cs_glob_time_step_options->dtref,
+                               cs_glob_time_step->t_cur,
+                               cs_glob_time_step->nt_cur);
         ialtyb[ifbr]  = *ivimpo;
       }
       cs_gui_add_mei_time(cs_timer_wtime() - t0);
     }
     else {
-      char *nat = cs_gui_boundary_zone_nature(izone);
+      int ith_zone = izone + 1;
+      char *nat = cs_gui_boundary_zone_nature(ith_zone);
       if (cs_gui_strcmp(nat, "free_surface")) {
         for (ifac = 0; ifac < faces; ifac++) {
           cs_lnum_t ifbr = faces_list[ifac];
@@ -991,7 +1058,6 @@ void CS_PROCF (uialcl, UIALCL) (const int *const    nozppm,
       }
       BFT_FREE(nat);
     }
-    BFT_FREE(faces_list);
   }
 }
 
@@ -999,7 +1065,6 @@ void CS_PROCF (uialcl, UIALCL) (const int *const    nozppm,
  * Retreive data for internal coupling. Called once at initialization
  *
  * parameters:
- *   nfabor   <-- Number of boundary faces
  *   idfstr   --> Structure definition
  *   mbstru   <-- number of previous structures (-999 or by restart)
  *   aexxst   --> Displacement prediction alpha
@@ -1011,8 +1076,7 @@ void CS_PROCF (uialcl, UIALCL) (const int *const    nozppm,
  *   vstr0    <-> Values of the initial velocity
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (uistr1, UISTR1) (const cs_lnum_t  *nfabor,
-                                cs_lnum_t        *idfstr,
+void CS_PROCF (uistr1, UISTR1) (cs_lnum_t        *idfstr,
                                 const int        *mbstru,
                                 double           *aexxst,
                                 double           *bexxst,
@@ -1027,21 +1091,20 @@ void CS_PROCF (uistr1, UISTR1) (const cs_lnum_t  *nfabor,
   int  ifac         = 0;
   cs_lnum_t  faces        = 0;
   cs_lnum_t  ifbr         = 0;
-  cs_lnum_t  *faces_list  = NULL;
   int        istruct = 0;
 
   /* Get advanced data */
-  get_uistr1_advanced_double("displacement_prediction_alpha", aexxst);
-  get_uistr1_advanced_double("displacement_prediction_beta", bexxst);
-  get_uistr1_advanced_double("stress_prediction_alpha", cfopre);
-  get_uistr1_advanced_checkbox("monitor_point_synchronisation", ihistr);
+  _get_uistr1_advanced_double("displacement_prediction_alpha", aexxst);
+  _get_uistr1_advanced_double("displacement_prediction_beta", bexxst);
+  _get_uistr1_advanced_double("stress_prediction_alpha", cfopre);
+  _get_uistr1_advanced_checkbox("monitor_point_synchronisation", ihistr);
 
   zones = cs_gui_boundary_zones_number();
 
   /* At each time-step, loop on boundary faces */
   for (izone=0 ; izone < zones ; izone++) {
-    char *nature = get_boundary_attribute(izone + 1, "nature");
-    char *label  = get_boundary_attribute(izone + 1, "label");
+    char *nature = _get_boundary_attribute(izone + 1, "nature");
+    char *label  = _get_boundary_attribute(izone + 1, "label");
 
     /* Keep only internal coupling */
     if (  _get_ale_boundary_nature(label)
@@ -1049,22 +1112,22 @@ void CS_PROCF (uistr1, UISTR1) (const cs_lnum_t  *nfabor,
 
       if (istruct+1 > *mbstru) { /* Do not overwrite restart data */
         /* Read initial_displacement, equilibrium_displacement and initial_velocity */
-        get_internal_coupling_xyz_values(label, "initial_displacement",
-                                         &xstr0[3 * istruct]);
-        get_internal_coupling_xyz_values(label, "equilibrium_displacement",
-                                         &xstreq[3 * istruct]);
-        get_internal_coupling_xyz_values(label, "initial_velocity",
-                                         &vstr0[3 * istruct]);
+        _get_internal_coupling_xyz_values(label, "initial_displacement",
+                                          &xstr0[3 * istruct]);
+        _get_internal_coupling_xyz_values(label, "equilibrium_displacement",
+                                          &xstreq[3 * istruct]);
+        _get_internal_coupling_xyz_values(label, "initial_velocity",
+                                          &vstr0[3 * istruct]);
       }
 
-      faces_list = cs_gui_get_faces_list(izone, label, *nfabor, 0, &faces);
+      const cs_lnum_t *faces_list
+        = cs_gui_get_boundary_faces(label, &faces);
       /* Set idfstr to positiv index starting at 1 */
       for (ifac = 0; ifac < faces; ifac++) {
         ifbr = faces_list[ifac];
         idfstr[ifbr] = istruct + 1;
       }
       ++istruct;
-      BFT_FREE(faces_list);
     }
     BFT_FREE(nature);
     BFT_FREE(label);
@@ -1072,16 +1135,16 @@ void CS_PROCF (uistr1, UISTR1) (const cs_lnum_t  *nfabor,
 }
 
 /*-----------------------------------------------------------------------------
- * Retreive data for internal coupling. Called at each step
+ * Retrieve data for internal coupling. Called at each step
  *
  * parameters:
- *   xmstru       <-- Mass matrix
- *   xcstr        <-- Damping matrix
- *   xkstru       <-- Stiffness matrix
- *   forstr       <-- Fluid force matrix
- *   dtref        -->   time step
- *   ttcabs       --> current time
- *   ntcabs       --> current iteration number
+ *   xmstru       --> Mass matrix
+ *   xcstr        --> Damping matrix
+ *   xkstru       --> Stiffness matrix
+ *   forstr       --> Fluid force matrix
+ *   dtref        <--  time step
+ *   ttcabs       <-- current time
+ *   ntcabs       <-- current iteration number
  *----------------------------------------------------------------------------*/
 
 void CS_PROCF (uistr2, UISTR2) (double *const  xmstru,
@@ -1113,20 +1176,68 @@ void CS_PROCF (uistr2, UISTR2) (double *const  xmstru,
       }
 #endif
 
-      get_uistr2_data(label,
-                      xmstru,
-                      xcstru,
-                      xkstru,
-                      forstr,
-                      istru,
-                      *dtref,
-                      *ttcabs,
-                      *ntcabs);
+      _get_uistr2_data(label,
+                       xmstru,
+                       xcstru,
+                       xkstru,
+                       forstr,
+                       istru,
+                       *dtref,
+                       *ttcabs,
+                       *ntcabs);
       ++istru;
     }
   }
 }
 
+/*-----------------------------------------------------------------------------
+ * Retrieve data for external coupling
+ *
+ * parameters:
+ *   idfstr    <-- Structure definition
+ *   asddlf    --> Block of the DDL forces
+ *----------------------------------------------------------------------------*/
+
+void
+CS_PROCF (uiaste, UIASTE) (int     *const idfstr,
+                           double  *const asddlf)
+{
+  if (!cs_gui_file_is_loaded())
+    return;
+
+  cs_lnum_t faces = 0;
+  int izone       = 0;
+  int istruct     = 0;
+
+  int zones = cs_gui_boundary_zones_number();
+
+  /* Loop on boundary faces */
+
+  for (izone = 0; izone < zones; izone++) {
+
+    char *label = cs_gui_boundary_zone_label(izone+1);
+
+    if (_get_ale_boundary_nature(label) == ale_boundary_nature_external_coupling) {
+
+      const cs_lnum_t *faces_list = cs_gui_get_boundary_faces(label, &faces);
+
+      /* Get DDLX, DDLY and DDLZ values */
+      asddlf[istruct * 3 + 0] = _get_external_coupling_dof(label, "DDLX") ? 0 : 1;
+      asddlf[istruct * 3 + 1] = _get_external_coupling_dof(label, "DDLY") ? 0 : 1;
+      asddlf[istruct * 3 + 2] = _get_external_coupling_dof(label, "DDLZ") ? 0 : 1;
+
+      /* Set idfstr with negative value starting from -1 */
+      for (cs_lnum_t ifac = 0; ifac < faces; ifac++) {
+        cs_lnum_t ifbr = faces_list[ifac];
+        idfstr[ifbr]  = -istruct - 1;
+      }
+      ++istruct;
+    }
+
+    BFT_FREE(label);
+  }
+}
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
