@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -71,14 +71,7 @@ void CS_PROCF (cspstb, CSPSTB) (cs_int_t        *ipstdv);
  * Determine output options
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (csenso, CSENSO) (const cs_int_t  *nvppmx,
-                                cs_int_t        *ncapt,
-                                cs_int_t        *nthist,
-                                cs_real_t       *frhist,
-                                cs_int_t        *iecaux,
-                                cs_int_t        *ihisvr,
-                                cs_int_t        *tplfmt,
-                                cs_real_t       *xyzcap);
+void CS_PROCF (csenso, CSENSO) (cs_int_t        *iecaux);
 
 /*============================================================================
  * Public function prototypes
