@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -55,6 +55,7 @@
 #include "cs_gui_specific_physics.h"
 #include "cs_gui.h"
 #include "cs_mesh.h"
+#include "cs_post.h"
 #include "cs_parameters.h"
 #include "cs_field.h"
 #include "cs_field_pointer.h"
@@ -91,7 +92,6 @@ BEGIN_C_DECLS
 typedef struct {
   char     **label;                /* label for each boundary zone            */
   char     **nature;               /* nature for each boundary zone           */
-  int      *output_zone;
   int      *type;
   double   *emissivity;
   double   *conductivity;
@@ -100,21 +100,13 @@ typedef struct {
   double   *external_temp;
   double   *internal_temp;
   double   *conduction_flux;
-} cs_rad_transferive_boundary_t;
+} cs_radiative_transfer_boundary_t;
 
 /*----------------------------------------------------------------------------
  * Private global variables for boundary conditions
  *----------------------------------------------------------------------------*/
 
-static cs_rad_transferive_boundary_t *boundary = NULL;
-
-/*----------------------------------------------------------------------------
- * Private global variables for the treatment
- * of NOMVAR. NOMVAR is a characters fortran array
- *----------------------------------------------------------------------------*/
-
-static int      _cs_gui_max_vars = 0;
-static char  ** _cs_gui_var_rayt = NULL;
+static cs_radiative_transfer_boundary_t  *_boundary = NULL;
 
 /*============================================================================
  * Private function definitions
@@ -385,89 +377,38 @@ _radiative_boundary_type(const char  *label)
   return result;
 }
 
-/*----------------------------------------------------------------------------
- *  Return maximum value of output zone
- *----------------------------------------------------------------------------*/
-
-static int
-_radiative_boundary_output_zone_max(void)
-{
-  char *path;
-  int nb_zone, zone_max = 0;
-
-  path = cs_xpath_init_path();
-  cs_xpath_add_elements(&path, 4,
-                        "boundary_conditions",
-                        "wall",
-                        "radiative_data",
-                        "output_zone" );
-
-  nb_zone = cs_gui_get_nb_element(path);
-
-  if (nb_zone > 0) {
-    cs_xpath_add_function_text(&path);
-    zone_max = cs_gui_get_max_value(path);
-  }
-
-  BFT_FREE(path);
-
-  return zone_max;
-}
-
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
-/*============================================================================
- * Public Fortran function definitions
- *============================================================================*/
-
-/*-----------------------------------------------------------------------------
- * Free memory: clean global private variables.
- *
- * Fortran Interface:
- *
- * subroutine memui2
- * *****************
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (memui2, MEMUI2) (void)
-{
-  int zones = 0;
-  int i;
-
-  if (boundary != NULL) {
-
-  /* clean memory for global private structure boundaries */
-
-    zones = cs_gui_boundary_zones_number();
-    for (i=0 ; i < zones ; i++) {
-      BFT_FREE(boundary->label[i]);
-      BFT_FREE(boundary->nature[i]);
-    }
-    BFT_FREE(boundary->label);
-    BFT_FREE(boundary->nature);
-    BFT_FREE(boundary->output_zone);
-    BFT_FREE(boundary->type);
-    BFT_FREE(boundary->emissivity);
-    BFT_FREE(boundary->thickness);
-    BFT_FREE(boundary->thermal_conductivity);
-    BFT_FREE(boundary->external_temp);
-    BFT_FREE(boundary->internal_temp);
-    BFT_FREE(boundary->conduction_flux);
-    BFT_FREE(boundary);
-  }
-
-  /* clean memory for fortran name of variables */
-
-  for (i = 0; i < _cs_gui_max_vars; i++)
-    BFT_FREE(_cs_gui_var_rayt[i]);
-  BFT_FREE(_cs_gui_var_rayt);
-
-}
-
 /*============================================================================
  * Public function definitions
  *============================================================================*/
 
+/*-----------------------------------------------------------------------------
+ * Free GUI-defined radiative transfer parameters
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_radiative_transfers_finalize(void)
+{
+  if (_boundary != NULL) {
+    int zones = cs_gui_boundary_zones_number();
+    for (int i = 0; i < zones; i++) {
+      BFT_FREE(_boundary->label[i]);
+      BFT_FREE(_boundary->nature[i]);
+    }
+    BFT_FREE(_boundary->label);
+    BFT_FREE(_boundary->nature);
+    BFT_FREE(_boundary->type);
+    BFT_FREE(_boundary->emissivity);
+    BFT_FREE(_boundary->thickness);
+    BFT_FREE(_boundary->thermal_conductivity);
+    BFT_FREE(_boundary->external_temp);
+    BFT_FREE(_boundary->internal_temp);
+    BFT_FREE(_boundary->conduction_flux);
+    BFT_FREE(_boundary);
+  }
+}
+
 /*----------------------------------------------------------------------------
  * Read GUI-defined radiative transfer parameters
  *----------------------------------------------------------------------------*/
@@ -486,13 +427,13 @@ cs_gui_radiative_transfer_parameters(void)
   int ac_type = 0;
 
   if (cs_gui_strcmp(model, "off"))
-    cs_glob_rad_transfer_params->iirayo = 0;
+    cs_glob_rad_transfer_params->type = CS_RAD_TRANSFER_NONE;
   else if (cs_gui_strcmp(model, "dom"))
-    cs_glob_rad_transfer_params->iirayo = 1;
+    cs_glob_rad_transfer_params->type = CS_RAD_TRANSFER_DOM;
   else if (cs_gui_strcmp(model, "p-1"))
-    cs_glob_rad_transfer_params->iirayo = 2;
+    cs_glob_rad_transfer_params->type = CS_RAD_TRANSFER_P1;
 
-  if (cs_glob_rad_transfer_params->iirayo) {
+  if (cs_glob_rad_transfer_params->type > CS_RAD_TRANSFER_NONE) {
     _radiative_transfer_char("restart", &isuird);
     if (isuird)
       cs_glob_rad_transfer_params->restart = true;
@@ -510,13 +451,14 @@ cs_gui_radiative_transfer_parameters(void)
     if (!cs_gui_get_activ_thermophysical_model()) {
       _radiative_transfer_type("absorption_coefficient", &ac_type);
       if (ac_type == 3)
-	cs_glob_rad_transfer_params->imodak = 1;
+        cs_glob_rad_transfer_params->imodak = 1;
     }
   }
 #if _XML_DEBUG_
   bft_printf("==> cs_gui_radiative_transfer_parameters\n");
-  bft_printf("--rayonnement : %s  (iirayo = %i)\n", model, cs_glob_rad_transfer_params->iirayo);
-  if (cs_glob_rad_transfer_params->iirayo) {
+  bft_printf("--radiative model: %s  (type = %i)\n",
+             model, cs_glob_rad_transfer_params->type);
+  if (cs_glob_rad_transfer_params->type > 0) {
     bft_printf("--isuird = %d\n", isuird);
     bft_printf("--quadra = %d\n", cs_glob_rad_transfer_params->i_quadrature);
     bft_printf("--ndirec = %d\n", cs_glob_rad_transfer_params->ndirec);
@@ -526,7 +468,7 @@ cs_gui_radiative_transfer_parameters(void)
     bft_printf("--iimlum = %d\n", cs_glob_rad_transfer_params->iimlum);
     bft_printf("--absorption coefficient type: %d\n", ac_type);
     bft_printf("--absorption coefficient by modak: %i\n",
-	       cs_glob_rad_transfer_params->imodak);
+               cs_glob_rad_transfer_params->imodak);
   }
 #endif
   BFT_FREE(model);
@@ -573,10 +515,9 @@ cs_gui_radiative_transfer_postprocess(void)
   if (!cs_gui_file_is_loaded())
     return;
 
-  const int n_rad_b_f = 8;
+  const int n_rad_b_f = 7;
 
-  const char  *b_rad_names[8] = {
-    "wall_temp",
+  const char  *b_rad_names[7] = {
     "flux_incident",
     "thickness",
     "wall_thermal_conductivity",
@@ -585,8 +526,7 @@ cs_gui_radiative_transfer_postprocess(void)
     "flux_convectif",
     "coeff_ech_conv"};
 
-  cs_field_t * b_rad_f[8] = {
-    CS_F_(t_b),
+  cs_field_t * b_rad_f[7] = {
     CS_F_(qinci),
     CS_F_(epa),
     CS_F_(xlam),
@@ -600,7 +540,7 @@ cs_gui_radiative_transfer_postprocess(void)
   bft_printf("%s\n", __func__);
 #endif
 
-  if (cs_glob_rad_transfer_params->iirayo) {
+  if (cs_glob_rad_transfer_params->type > CS_RAD_TRANSFER_NONE) {
     int k_lbl = cs_field_key_id("label");
     int k_vis = cs_field_key_id("post_vis");
     int k_log = cs_field_key_id("log");
@@ -608,7 +548,7 @@ cs_gui_radiative_transfer_postprocess(void)
       int f_post_vis =  0;
       int f_log =  1;
       if (i == 0)
-        f_post_vis = 1;
+        f_post_vis = CS_POST_ON_LOCATION;
       char *label = _radiative_transfer_char_post(b_rad_names[i],
                                                   &f_log,
                                                   &f_post_vis);
@@ -634,11 +574,9 @@ cs_gui_radiative_transfer_postprocess(void)
 
 void
 cs_gui_radiative_transfer_bcs(const    int   itypfb[],
-                              cs_lnum_t      nvarcl,
+                              cs_lnum_t      nvar,
                               cs_lnum_t      ivart,
-                              int           *izfrdp,
                               int           *isothp,
-                              const    int  *nozppm,
                               double        *epsp,
                               double        *epap,
                               double        *tintp,
@@ -649,39 +587,31 @@ cs_gui_radiative_transfer_bcs(const    int   itypfb[],
   if (!cs_gui_file_is_loaded())
     return;
 
-  int zones = 0;
-  int output_zone_max = 0;
   int izone;
   int ith_zone;
   int ifbr;
-  cs_lnum_t j, n;
-  cs_lnum_t *faces_list = NULL;
   cs_lnum_t faces = 0;
-  int iok = 0;
-  double tmp = 0.;
   char *nature = NULL;
   char *label = NULL;
 
   const cs_lnum_t  n_b_faces = cs_glob_mesh->n_b_faces;
 
-  zones   = cs_gui_boundary_zones_number();
-  output_zone_max = _radiative_boundary_output_zone_max();
+  int zones = cs_gui_boundary_zones_number();
 
   /* First call only: memory allocation */
 
-  if (boundary == NULL) {
+  if (_boundary == NULL) {
 
-    BFT_MALLOC(boundary, 1, cs_rad_transferive_boundary_t);
-    BFT_MALLOC(boundary->label,                zones, char *);
-    BFT_MALLOC(boundary->nature,               zones, char *);
-    BFT_MALLOC(boundary->output_zone,          zones, int);
-    BFT_MALLOC(boundary->type,                 zones, int);
-    BFT_MALLOC(boundary->emissivity,           zones, double);
-    BFT_MALLOC(boundary->thickness,            zones, double);
-    BFT_MALLOC(boundary->thermal_conductivity, zones, double);
-    BFT_MALLOC(boundary->external_temp,        zones, double);
-    BFT_MALLOC(boundary->internal_temp,        zones, double);
-    BFT_MALLOC(boundary->conduction_flux,      zones, double);
+    BFT_MALLOC(_boundary, 1, cs_radiative_transfer_boundary_t);
+    BFT_MALLOC(_boundary->label,                zones, char *);
+    BFT_MALLOC(_boundary->nature,               zones, char *);
+    BFT_MALLOC(_boundary->type,                 zones, int);
+    BFT_MALLOC(_boundary->emissivity,           zones, double);
+    BFT_MALLOC(_boundary->thickness,            zones, double);
+    BFT_MALLOC(_boundary->thermal_conductivity, zones, double);
+    BFT_MALLOC(_boundary->external_temp,        zones, double);
+    BFT_MALLOC(_boundary->internal_temp,        zones, double);
+    BFT_MALLOC(_boundary->conduction_flux,      zones, double);
 
     for (izone = 0; izone < zones; izone++) {
 
@@ -694,39 +624,33 @@ cs_gui_radiative_transfer_bcs(const    int   itypfb[],
 
       label = cs_gui_boundary_zone_label(ith_zone);
 
-      BFT_MALLOC(boundary->label[izone], strlen(label)+1, char);
-      strcpy(boundary->label[izone], label);
+      BFT_MALLOC(_boundary->label[izone], strlen(label)+1, char);
+      strcpy(_boundary->label[izone], label);
 
-      BFT_MALLOC(boundary->nature[izone], strlen(nature)+1, char);
-      strcpy(boundary->nature[izone], nature);
+      BFT_MALLOC(_boundary->nature[izone], strlen(nature)+1, char);
+      strcpy(_boundary->nature[izone], nature);
 
-      /* Default initialization: these values are the same that in
-         cs_rad_tranfer_bcs but given on each face there whereas here one does
-         not necessarily have boundary faces (parallism) -> duplication */
-      boundary->type[izone] = -1;
-      boundary->output_zone[izone] = -1;
-      boundary->emissivity[izone] = -1.e12;
-      boundary->thickness[izone] = -1.e12;
-      boundary->thermal_conductivity[izone] = -1.e12;
-      boundary->external_temp[izone] = -1.e12;
-      boundary->internal_temp[izone] = -1.e12;
-      boundary->conduction_flux[izone] = 1.e30;
+      /* Default initialization */
+      _boundary->type[izone] = -1;
+      _boundary->emissivity[izone] = -1.e12;
+      _boundary->thickness[izone] = -1.e12;
+      _boundary->thermal_conductivity[izone] = -1.e12;
+      _boundary->external_temp[izone] = -1.e12;
+      _boundary->internal_temp[izone] = -1.e12;
+      _boundary->conduction_flux[izone] = 1.e30;
 
       if (cs_gui_strcmp(nature, "wall")) {
-        boundary->type[izone] = _radiative_boundary_type(label);
-        tmp = (double) boundary->output_zone[izone];
-        _radiative_boundary(label, "output_zone", &tmp);
-        boundary->output_zone[izone] = (int) tmp;
-        _radiative_boundary(label, "emissivity", &boundary->emissivity[izone]);
-        _radiative_boundary(label, "thickness", &boundary->thickness[izone]);
+        _boundary->type[izone] = _radiative_boundary_type(label);
+        _radiative_boundary(label, "emissivity", &_boundary->emissivity[izone]);
+        _radiative_boundary(label, "thickness", &_boundary->thickness[izone]);
         _radiative_boundary(label, "wall_thermal_conductivity",
-                            &boundary->thermal_conductivity[izone]);
+                            &_boundary->thermal_conductivity[izone]);
         _radiative_boundary(label, "external_temperature_profile",
-                            &boundary->external_temp[izone]);
+                            &_boundary->external_temp[izone]);
         _radiative_boundary(label, "internal_temperature_profile",
-                            &boundary->internal_temp[izone]);
+                            &_boundary->internal_temp[izone]);
         _radiative_boundary(label, "flux",
-                            &boundary->conduction_flux[izone]);
+                            &_boundary->conduction_flux[izone]);
 
       } /* if (cs_gui_strcmp(nature, "wall")) */
 
@@ -741,13 +665,12 @@ cs_gui_radiative_transfer_bcs(const    int   itypfb[],
 
     /* list of faces building */
 
-    faces_list = cs_gui_get_faces_list(izone,
-                                       boundaries->label[izone],
-                                       n_b_faces, *nozppm, &faces);
+    const cs_lnum_t *faces_list
+      = cs_gui_get_boundary_faces(boundaries->label[izone], &faces);
 
-    if (cs_gui_strcmp(boundary->nature[izone], "wall")) {
-      for (n = 0; n < faces; n++) {
-        ifbr = faces_list[n];
+    if (cs_gui_strcmp(_boundary->nature[izone], "wall")) {
+      for (cs_lnum_t i = 0; i < faces; i++) {
+        ifbr = faces_list[i];
 
         if (itypfb[ifbr] != CS_SMOOTHWALL && itypfb[ifbr] != CS_ROUGHWALL)
           bft_error
@@ -762,83 +685,49 @@ cs_gui_radiative_transfer_bcs(const    int   itypfb[],
                " must be coherent\n"
                "with these new natures.\n"));
 
-        izfrdp[ifbr] = boundary->output_zone[izone];
-        isothp[ifbr] = boundary->type[izone];
+        isothp[ifbr] = _boundary->type[izone];
         if (isothp[ifbr] == cs_glob_rad_transfer_params->itpimp) {
-          epsp[ifbr] = boundary->emissivity[izone];
-          tintp[ifbr] = boundary->internal_temp[izone];
+          epsp[ifbr] = _boundary->emissivity[izone];
+          tintp[ifbr] = _boundary->internal_temp[izone];
         }
         else if (isothp[ifbr] == cs_glob_rad_transfer_params->ipgrno) {
-          xlamp[ifbr] = boundary->thermal_conductivity[izone];
-          epap[ifbr] = boundary->thickness[izone];
-          textp[ifbr] = boundary->external_temp[izone];
-          tintp[ifbr] = boundary->internal_temp[izone];
-          epsp[ifbr] = boundary->emissivity[izone];
-          if (cs_gui_is_equal_real(boundary->emissivity[izone], 0.))
+          xlamp[ifbr] = _boundary->thermal_conductivity[izone];
+          epap[ifbr] = _boundary->thickness[izone];
+          textp[ifbr] = _boundary->external_temp[izone];
+          tintp[ifbr] = _boundary->internal_temp[izone];
+          epsp[ifbr] = _boundary->emissivity[izone];
+          if (cs_gui_is_equal_real(_boundary->emissivity[izone], 0.))
             isothp[ifbr] = cs_glob_rad_transfer_params->iprefl;
         }
         else if (isothp[ifbr] == cs_glob_rad_transfer_params->ifgrno) {
-          rcodcl[2 * n_b_faces * nvarcl + ivart * n_b_faces + ifbr]
-            = boundary->conduction_flux[izone];
-          tintp[ifbr] = boundary->internal_temp[izone];
-          if (!cs_gui_is_equal_real(boundary->emissivity[izone], 0.))
+          rcodcl[2 * n_b_faces * nvar + ivart * n_b_faces + ifbr]
+            = _boundary->conduction_flux[izone];
+          tintp[ifbr] = _boundary->internal_temp[izone];
+          if (!cs_gui_is_equal_real(_boundary->emissivity[izone], 0.))
             isothp[ifbr] = cs_glob_rad_transfer_params->ifrefl;
           else
-            epsp[ifbr] = boundary->emissivity[izone];
+            epsp[ifbr] = _boundary->emissivity[izone];
         }
       }
 
-    } else {
-      j = output_zone_max++;
-      for (n = 0; n < faces; n++) {
-        ifbr = faces_list[n];
-        izfrdp[ifbr] = j;
-      }
     } /* if nature == "wall" */
 
-    BFT_FREE(faces_list);
-
   } /* for izone */
 
-  iok = 0;
-  for (n = 0; n < n_b_faces; n++) {
-    if (izfrdp[n] == -1) iok = 1;
-  }
-  if (iok == 1) {
-    bft_printf("Warning: radiative boundary conditions in GUI are not totally defined \n");
-    if (zones)
-      bft_printf("These are radiative boundary conditions defined in GUI: \n");
-    for (izone = 0; izone < zones; izone++) {
-       bft_printf("  nature: %s label: %s\n", boundary->nature[izone],
-                  boundary->label[izone]);
-       if (cs_gui_strcmp(boundary->nature[izone], "wall")) {
-         bft_printf("    output_zone = %i\n", boundary->output_zone[izone]);
-         bft_printf("    type = %i\n", boundary->type[izone]);
-         bft_printf("    emissivity = %f\n", boundary->emissivity[izone]);
-         bft_printf("    thickness= %f\n", boundary->thickness[izone]);
-         bft_printf("    wall_thermal_conductivity = %f\n",
-                    boundary->thermal_conductivity[izone]);
-         bft_printf("    external_temp = %f\n", boundary->external_temp[izone]);
-         bft_printf("    internal_temp = %f\n", boundary->internal_temp[izone]);
-         bft_printf("    conduction_flux= %f\n", boundary->conduction_flux[izone]);
-       }
-    }
-  }
-
 #if _XML_DEBUG_
   bft_printf("==>UIRAY2\n");
   for (izone = 0; izone < zones; izone++) {
-     bft_printf("--label zone = %s\n", boundary->label[izone]);
-     if (cs_gui_strcmp(boundary->nature[izone], "wall")) {
-       bft_printf("----output_zone = %i\n", boundary->output_zone[izone]);
-       bft_printf("----type = %i\n", boundary->type[izone]);
-       bft_printf("----emissivity = %f\n", boundary->emissivity[izone]);
-       bft_printf("----thickness= %f\n", boundary->thickness[izone]);
+     bft_printf("--label zone = %s\n", _boundary->label[izone]);
+     if (cs_gui_strcmp(_boundary->nature[izone], "wall")) {
+       bft_printf("----type = %i\n", _boundary->type[izone]);
+       bft_printf("----emissivity = %f\n", _boundary->emissivity[izone]);
+       bft_printf("----thickness= %f\n", _boundary->thickness[izone]);
        bft_printf("----wall_thermal_conductivity = %f\n",
-                  boundary->thermal_conductivity[izone]);
-       bft_printf("----external_temp = %f\n", boundary->external_temp[izone]);
-       bft_printf("----internal_temp = %f\n", boundary->internal_temp[izone]);
-       bft_printf("----conduction_flux= %f\n", boundary->conduction_flux[izone]);
+                  _boundary->thermal_conductivity[izone]);
+       bft_printf("----external_temp = %f\n", _boundary->external_temp[izone]);
+       bft_printf("----internal_temp = %f\n", _boundary->internal_temp[izone]);
+       bft_printf("----conduction_flux= %f\n",
+                  _boundary->conduction_flux[izone]);
     }
   }
 #endif
