@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -38,23 +38,15 @@
 BEGIN_C_DECLS
 
 /*============================================================================
- * Public function prototypes for Fortran API
+ * Public function definitions
  *============================================================================*/
 
 /*-----------------------------------------------------------------------------
- * Free memory: clean global private variables.
- *
- * Fortran Interface:
- *
- * subroutine memui2
- * *****************
+ * Free GUI-defined radiative transfer parameters
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (memui2, MEMUI2) (void);
-
-/*============================================================================
- * Public function definitions
- *============================================================================*/
+void
+cs_gui_radiative_transfers_finalize(void);
 
 /*----------------------------------------------------------------------------
  * Read GUI-defined radiative transfer parameters
@@ -86,11 +78,9 @@ cs_gui_radiative_transfer_postprocess(void);
 
 void
 cs_gui_radiative_transfer_bcs(const    int   itypfb[],
-                              cs_lnum_t      nvarcl,
+                              cs_lnum_t      nvar,
                               cs_lnum_t      ivart,
-                              int           *izfrdp,
                               int           *isothp,
-                              const    int  *nozppm,
                               double        *epsp,
                               double        *epap,
                               double        *tintp,
