@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -49,7 +49,6 @@
 #include "fvm_selector.h"
 
 #include "mei_evaluate.h"
-#include "mei_math_util.h"
 
 #include "cs_base.h"
 #include "cs_gui_util.h"
@@ -62,6 +61,7 @@
 #include "cs_prototypes.h"
 #include "cs_selector.h"
 #include "cs_elec_model.h"
+#include "cs_gwf_physical_properties.h"
 
 /*----------------------------------------------------------------------------
  * Header for the current file
@@ -485,8 +485,9 @@ _get_solid_fuel_thermal_conductivity(const int icha)
   cs_xpath_add_element(&path, "thermal_conductivity");
   cs_xpath_add_function_text(&path);
 
-  if (!cs_gui_get_double(path, &result))
-    bft_error(__FILE__, __LINE__, 0, _("Invalid xpath: %s\n"), path);
+  if (!cs_gui_get_double(path, &result)) {
+    result = 1.e-5; /* default, used with Lagrangian model only */
+  }
 
   BFT_FREE(path);
   return result;
@@ -656,56 +657,54 @@ _get_PCI_type(int icha)
   cs_xpath_add_attribute(&path,"choice");
   buff = cs_gui_get_attribute_value(path);
   if (buff != NULL) {
-    if (cs_gui_strcmp(buff, "LHV"))
-      {
-        path2 = cs_xpath_init_path();
-        cs_xpath_add_elements(&path2, 2,"thermophysical_models", "solid_fuels");;
-        cs_xpath_add_element_num(&path2, "solid_fuel", icha);
-        cs_xpath_add_element(&path2, "Heating_model");
-        cs_xpath_add_element(&path2, "type");
-        cs_xpath_add_function_text(&path2);
-        buff2 = cs_gui_get_text_value(path2);
-        if (buff2 == NULL) {
+    if (cs_gui_strcmp(buff, "LHV")) {
+      path2 = cs_xpath_init_path();
+      cs_xpath_add_elements(&path2, 2,"thermophysical_models", "solid_fuels");;
+      cs_xpath_add_element_num(&path2, "solid_fuel", icha);
+      cs_xpath_add_element(&path2, "Heating_model");
+      cs_xpath_add_element(&path2, "type");
+      cs_xpath_add_function_text(&path2);
+      buff2 = cs_gui_get_text_value(path2);
+      if (buff2 == NULL) {
+        ichoice = 1;
+      } else {
+        if (cs_gui_strcmp(buff2, "dry_basis"))
           ichoice = 1;
-        } else {
-          if (cs_gui_strcmp(buff2, "dry_basis"))
-            ichoice = 1;
-          else if (cs_gui_strcmp(buff2, "dry_ash_free"))
-            ichoice = 0;
-          else if (cs_gui_strcmp(buff2, "as_received"))
-            ichoice = 2;
-          else
-            bft_error(__FILE__, __LINE__, 0, _("Invalid xpath: %s\n"), path2);
+        else if (cs_gui_strcmp(buff2, "dry_ash_free"))
+          ichoice = 0;
+        else if (cs_gui_strcmp(buff2, "as_received"))
+          ichoice = 2;
+        else
+          bft_error(__FILE__, __LINE__, 0, _("Invalid xpath: %s\n"), path2);
 
-          BFT_FREE(path2);
-          BFT_FREE(buff2);
-        }
+        BFT_FREE(buff2);
       }
-    else if (cs_gui_strcmp(buff, "HHV"))
-      {
-        path2 = cs_xpath_init_path();
-        cs_xpath_add_elements(&path2, 2,"thermophysical_models", "solid_fuels");
-        cs_xpath_add_element_num(&path2, "solid_fuel", icha);
-        cs_xpath_add_element(&path2, "Heating_model");
-        cs_xpath_add_element(&path2, "type");
-        cs_xpath_add_function_text(&path2);
-        buff2 = cs_gui_get_text_value(path2);
-        if (buff2 == NULL) {
+      BFT_FREE(path2);
+    }
+    else if (cs_gui_strcmp(buff, "HHV")) {
+      path2 = cs_xpath_init_path();
+      cs_xpath_add_elements(&path2, 2,"thermophysical_models", "solid_fuels");
+      cs_xpath_add_element_num(&path2, "solid_fuel", icha);
+      cs_xpath_add_element(&path2, "Heating_model");
+      cs_xpath_add_element(&path2, "type");
+      cs_xpath_add_function_text(&path2);
+      buff2 = cs_gui_get_text_value(path2);
+      if (buff2 == NULL) {
+        ichoice = 4;
+      } else {
+        if (cs_gui_strcmp(buff2, "dry_basis"))
           ichoice = 4;
-        } else {
-          if (cs_gui_strcmp(buff2, "dry_basis"))
-            ichoice = 4;
-          else if (cs_gui_strcmp(buff2, "dry_ash_free"))
-            ichoice = 3;
-          else if (cs_gui_strcmp(buff2, "as_received"))
-            ichoice = 5;
-          else
-            bft_error(__FILE__, __LINE__, 0, _("Invalid xpath: %s\n"), path2);
+        else if (cs_gui_strcmp(buff2, "dry_ash_free"))
+          ichoice = 3;
+        else if (cs_gui_strcmp(buff2, "as_received"))
+          ichoice = 5;
+        else
+          bft_error(__FILE__, __LINE__, 0, _("Invalid xpath: %s\n"), path2);
 
-          BFT_FREE(path2);
-          BFT_FREE(buff2);
-        }
+        BFT_FREE(buff2);
       }
+      BFT_FREE(path2);
+    }
     else if (cs_gui_strcmp(buff, "IGT_correlation"))
       ichoice = 6;
     else
@@ -1895,8 +1894,8 @@ void CS_PROCF (uisofu, UISOFU) (const int    *const iirayo,
     rho0ch[icha] = _get_solid_fuel_density(icha+1);
 
     /* ---- Thermal conductivity of the coal (W/m/K) */
-    if (vars != NULL)
-      if (cs_gui_strcmp(vars->model_value, "homogeneous_fuel_moisture_lagr"))
+    if (vars != NULL && thcdch != NULL)
+      if (cs_gui_strcmp(vars->model_value, "homogeneous_fuel_moisture"))
         thcdch[icha] = _get_solid_fuel_thermal_conductivity(icha+1);
 
     /* ---- Caracteristiques cendres */
@@ -2088,7 +2087,6 @@ void CS_PROCF(cfnmtd, CFNMTD) (char          *fstr,    /* --> Fortran string */
 
 }
 
-
 /*----------------------------------------------------------------------------
  * groundwater model : read parameters
  *
@@ -2100,65 +2098,65 @@ void CS_PROCF(cfnmtd, CFNMTD) (char          *fstr,    /* --> Fortran string */
  * integer         dispersion      <--   dispersion type
  * integer         unsteady        <--   steady flow
  * integer         gravity         <--   check if gravity is taken into account
+ * integer         unsaturated     <--   take into account unsaturated zone
  *----------------------------------------------------------------------------*/
 
 void CS_PROCF (uidai1, UIDAI1) (int    *const permeability,
                                 int    *const dispersion,
                                 int    *const unsteady,
-                                int    *const gravity)
+                                int    *const gravity,
+                                int    *const unsaturated)
 {
   char *path   = NULL;
   char *mdl    = NULL;
   int   result;
 
+  /* Get dispersion type */
   path = cs_xpath_init_path();
   cs_xpath_add_elements(&path, 3, "thermophysical_models",
                                   "groundwater_model",
                                   "dispersion");
-
   cs_xpath_add_attribute(&path, "model");
   mdl = cs_gui_get_attribute_value(path);
   BFT_FREE(path);
+
   if (cs_gui_strcmp(mdl, "anisotropic"))
     *dispersion = 1;
   else
     *dispersion = 0;
-
   BFT_FREE(mdl);
-  BFT_FREE(path);
 
+  /* Get flow type */
   path = cs_xpath_init_path();
   cs_xpath_add_elements(&path, 3, "thermophysical_models",
                                   "groundwater_model",
                                   "flowType");
-
   cs_xpath_add_attribute(&path, "model");
   mdl = cs_gui_get_attribute_value(path);
   BFT_FREE(path);
+
   if (cs_gui_strcmp(mdl, "steady"))
     *unsteady = 0;
   else
     *unsteady = 1;
-
   BFT_FREE(mdl);
-  BFT_FREE(path);
 
+  /* Get permeability type */
   path = cs_xpath_init_path();
   cs_xpath_add_elements(&path, 3, "thermophysical_models",
                                   "groundwater_model",
                                   "permeability");
-
   cs_xpath_add_attribute(&path, "model");
   mdl = cs_gui_get_attribute_value(path);
   BFT_FREE(path);
+
   if (cs_gui_strcmp(mdl, "anisotropic"))
     *permeability = 1;
   else
     *permeability = 0;
-
   BFT_FREE(mdl);
-  BFT_FREE(path);
 
+  /* Get gravity */
   path = cs_xpath_init_path();
   cs_xpath_add_elements(&path, 3, "thermophysical_models",
                                   "groundwater_model",
@@ -2167,14 +2165,74 @@ void CS_PROCF (uidai1, UIDAI1) (int    *const permeability,
 
   if (cs_gui_get_status(path, &result))
     *gravity = result;
-
   BFT_FREE(path);
 
+  /* Get the possible presence of unsaturated zone */
+  path = cs_xpath_init_path();
+  cs_xpath_add_elements(&path, 3, "thermophysical_models",
+                                  "groundwater_model",
+                                  "unsaturatedZone");
+  cs_xpath_add_attribute(&path, "model");
+  mdl = cs_gui_get_attribute_value(path);
+  BFT_FREE(path);
+
+  if (cs_gui_strcmp(mdl, "true"))
+    *unsaturated = 1;
+  else
+    *unsaturated = 0;
+  BFT_FREE(mdl);
+
+  /* Get first-order decay rate and chemistry model */
+  int n_fields = cs_field_n_fields();
+  for (int f_id = 0; f_id < n_fields; f_id++) {
+    cs_field_t *f = cs_field_by_id(f_id);
+    if (   (f->type & CS_FIELD_VARIABLE)
+           && (f->type & CS_FIELD_USER)) {
+
+      /* get first-order decay rate */
+      cs_real_t decay = 0.;
+      path = cs_xpath_init_path();
+      cs_xpath_add_elements(&path, 2, "thermophysical_models",
+                                      "groundwater_model");
+      cs_xpath_add_element(&path, "scalar");
+      cs_xpath_add_test_attribute(&path, "name", f->name);
+      cs_xpath_add_element(&path, "fo_decay_rate");
+      cs_xpath_add_function_text(&path);
+      cs_gui_get_double(path, &decay);
+      BFT_FREE(path);
+
+      int key_decay = cs_field_key_id("fo_decay_rate");
+      cs_field_set_key_double(f, key_decay, decay);
+
+      /* get chemistry model */
+      path = cs_xpath_init_path();
+      cs_xpath_add_elements(&path, 2, "thermophysical_models",
+                                      "groundwater_model");
+      cs_xpath_add_element(&path, "scalar");
+      cs_xpath_add_test_attribute(&path, "name", f->name);
+      cs_xpath_add_attribute(&path, "chemistry_model");
+      mdl = cs_gui_get_attribute_value(path);
+      BFT_FREE(path);
+
+      cs_gwf_soilwater_partition_t sorption_scal;
+      int key_part = cs_field_key_id("gwf_soilwater_partition");
+      cs_field_get_key_struct(f, key_part, &sorption_scal);
+      if (cs_gui_strcmp(mdl, "EK"))
+        sorption_scal.kinetic = 1;
+      else
+        sorption_scal.kinetic = 0;
+      cs_field_set_key_struct(f, key_part, &sorption_scal);
+      BFT_FREE(mdl);
+    }
+  }
+
 #if _XML_DEBUG_
   bft_printf("==>UIDAI1\n");
   bft_printf("--groundwater_anisotropic_permeability  = %i\n", *permeability);
   bft_printf("--groundwater_anisotropic_dispersion    = %f\n", *dispersion);
   bft_printf("--groundwater_unsteady                  = %f\n", *unsteady);
+  bft_printf("--groundwater_gravity                   = %f\n", *gravity);
+  bft_printf("--groundwater_unsaturated               = %f\n", *unsaturated);
 #endif
 }
 
@@ -2212,10 +2270,8 @@ cs_gui_physical_model_select(cs_int_t  *ieos,
     if (cs_gui_strcmp(vars->model, "solid_fuels")) {
       if (cs_gui_strcmp(vars->model_value, "homogeneous_fuel"))
         cs_glob_physical_model_flag[CS_COMBUSTION_COAL] = 0;
-      else if (   cs_gui_strcmp(vars->model_value,
-                                "homogeneous_fuel_moisture")
-               || cs_gui_strcmp(vars->model_value,
-                                "homogeneous_fuel_moisture_lagr"))
+      else if (cs_gui_strcmp(vars->model_value,
+                             "homogeneous_fuel_moisture"))
         cs_glob_physical_model_flag[CS_COMBUSTION_COAL] = 1;
       else
         bft_error(__FILE__, __LINE__, 0,
@@ -2349,7 +2405,10 @@ cs_gui_elec_model(void)
     elec_opt->ielcor = status;
   BFT_FREE(path);
 
-  if (cs_glob_elec_option->ieljou > 0) {
+  int ieljou = cs_glob_physical_model_flag[CS_JOULE_EFFECT];
+  int ielarc = cs_glob_physical_model_flag[CS_ELECTRIC_ARCS];
+
+  if (ieljou > 0) {
     path = cs_xpath_init_path();
     cs_xpath_add_elements(&path, 3, "thermophysical_models",
                                     "joule_effect",
@@ -2362,7 +2421,7 @@ cs_gui_elec_model(void)
     BFT_FREE(path);
   }
 
-  if (cs_glob_elec_option->ielarc > 0) {
+  if (ielarc > 0) {
     path = cs_xpath_init_path();
     cs_xpath_add_elements(&path, 3, "thermophysical_models",
                                     "joule_effect",
@@ -2816,7 +2875,7 @@ cs_gui_labels_electric_arcs(int  n_gasses)
   for (i = 0; i < n_gasses - 1; i++) {
     f = CS_FI_(ycoel, i);
     if (f != NULL) {
-      snprintf(name, 63, "%s%2.2i", "esl_fraction", i+1); name[63] = '\0';
+      snprintf(name, 63, "%s%2.2i", "esl_fraction_", i+1); name[63] = '\0';
       _set_scalar_name_label(f, "joule_effect", name);
     }
   }
