@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -181,13 +181,14 @@ void CS_PROCF(cfnmtd, CFNMTD) (char          *fstr,    /* --> Fortran string */
  * integer         dispersion      <--   dispersion type
  * integer         unsteady        <--   steady flow
  * integer         gravity         <--   check if gravity is taken into account
+ * integer         unsaturated     <--   take into account unsaturated zone
  *----------------------------------------------------------------------------*/
 
 void CS_PROCF (uidai1, UIDAI1) (int    *const permeability,
                                 int    *const dispersion,
                                 int    *const unsteady,
-                                int    *const gravity);
-
+                                int    *const gravity,
+                                int    *const unsaturated);
 
 /*=============================================================================
  * Public function prototypes
