@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -623,7 +623,8 @@ cs_gui_get_attribute_value(char  *path)
   BFT_MALLOC(attr, strlen(array[0])+1, char);
   strcpy(attr, array[0]);
 
-  BFT_FREE(array[0]);
+  for (int i = 0; i < size; i++)
+    BFT_FREE(array[i]);
   BFT_FREE(array);
 
   return attr;
