@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -56,10 +56,12 @@
 #include "bft_error.h"
 
 #include "cs_physical_constants.h"
+#include "cs_random.h"
 
 #include "cs_lagr.h"
 #include "cs_lagr_tracking.h"
 #include "cs_lagr_roughness.h"
+#include "cs_lagr_clogging.h"
 
 /*----------------------------------------------------------------------------
  *  Header for the current file
@@ -88,9 +90,6 @@ BEGIN_C_DECLS
 /* Cut-off distance for adhesion forces (assumed to be the Born distance) */
 static const cs_real_t  _d_cut_off = 1.65e-10;
 
-/* Characteristic retardation wavelength (m) for Hamaker constant */
-static const cs_real_t _lambda_wl = 1000.0e-9;
-
 /* Boltzmann constant */
 static const double _k_boltz = 1.38e-23;
 
@@ -107,46 +106,49 @@ static const cs_real_t _faraday_cst = 9.648e4;
  *============================================================================*/
 
 /*----------------------------------------------------------------------------*/
+/* VDW interaction between a sphere and a plate                               */
 /*----------------------------------------------------------------------------*/
 
 static void
-_vdwsp(cs_real_t *distp,
+_vdwsp(cs_real_t  cstham,
+       cs_real_t  lambda_vdw,
+       cs_real_t *distp,
        cs_real_t *rpart,
        cs_real_t *var)
 {
-  cs_lagr_physico_chemical_t *lag_pc = cs_glob_lagr_physico_chemical;
-
-  if (*distp < _lambda_wl / 2.0 / cs_math_pi)
-    *var = - lag_pc->cstham * *rpart / (6.0 * *distp)
-           * (1.0 / (1.0 + 14.0 * *distp / _lambda_wl + 5.0 * cs_math_pi / 4.9 * pow (*distp, 3.0)
-                   / _lambda_wl / pow (*rpart, 2.0)));
+  if (*distp < lambda_vdw / 2.0 / cs_math_pi)
+    *var = - cstham * *rpart / (6.0 * *distp)
+           * (1.0 / (1.0 + 14.0 * *distp / lambda_vdw + 5.0 * cs_math_pi / 4.9 * pow (*distp, 3.0)
+                   / lambda_vdw / pow (*rpart, 2.0)));
   else
-    *var = lag_pc->cstham * (2.45 / 60.0 / cs_math_pi * _lambda_wl
+    *var = cstham * (2.45 / 60.0 / cs_math_pi * lambda_vdw
                      * (  (*distp - *rpart) / pow (*distp, 2)
                         - (*distp + 3.0 * *rpart) / pow ((*distp + 2.0 * *rpart), 2.0))
-                     - 2.17 / 720.0 / pow (cs_math_pi, 2) * pow (_lambda_wl, 2.0)
+                     - 2.17 / 720.0 / pow (cs_math_pi, 2) * pow (lambda_vdw, 2.0)
                        * ((*distp - 2.0* *rpart) / pow (*distp, 3.0) - (*distp + 4.0 * *rpart) / pow ((*distp + 2.0 * *rpart), 3.0))
-                     + 0.59 / 5040.0 / pow (cs_math_pi, 3.0) * pow (_lambda_wl, 3.0)
+                     + 0.59 / 5040.0 / pow (cs_math_pi, 3.0) * pow (lambda_vdw, 3.0)
                        * ((*distp - 3.0 * *rpart) / pow (*distp, 4.0) - (*distp + 5.0 * *rpart) / pow ((*distp + 2.0 * *rpart), 4.0)));
 
 }
 
 /*----------------------------------------------------------------------------*/
+/* VDW interaction between two spheres                                        */
 /*----------------------------------------------------------------------------*/
 
 static void
-_vdwsa(cs_real_t *distcc,
+_vdwss(cs_real_t  cstham,
+       cs_real_t  lambda_vdw,
+       cs_real_t *distcc,
        cs_real_t *rpart1,
        cs_real_t *rpart2,
        cs_real_t *var)
 {
 
-  cs_lagr_physico_chemical_t *lag_pc = cs_glob_lagr_physico_chemical;
-  *var = - lag_pc->cstham * *rpart1 * *rpart2
+  *var = - cstham * *rpart1 * *rpart2
         / (6.0 * (*distcc - *rpart1 - *rpart2) * (*rpart1 + *rpart2))
         * (  1.0
-           -   5.32 * (*distcc - *rpart1 - *rpart2) / _lambda_wl
-             * log (1.0 + _lambda_wl / (*distcc - *rpart1 - *rpart2) / 5.32));
+           -   5.32 * (*distcc - *rpart1 - *rpart2) / lambda_vdw
+             * log (1.0 + lambda_vdw / (*distcc - *rpart1 - *rpart2) / 5.32));
 
 }
 
@@ -156,6 +158,8 @@ _vdwsa(cs_real_t *distcc,
 static void
 _edlsp(cs_real_t *distp,
        cs_real_t *rpart,
+       cs_real_t  phi_p,
+       cs_real_t  phi_s,
        cs_real_t  tempf,
        cs_real_t *var)
 {
@@ -166,8 +170,8 @@ _edlsp(cs_real_t *distp,
                 / (lag_pc->epseau * _free_space_permit * cs_physical_constants_r * tempf), -0.5);
 
   /* Reduced zeta potential    */
-  cs_real_t lphi1  = lag_pc->valen * charge * lag_pc->phi_p / _k_boltz / tempf;
-  cs_real_t lphi2  = lag_pc->valen * charge * lag_pc->phi_s / _k_boltz / tempf;
+  cs_real_t lphi1  = lag_pc->valen * charge * phi_p / _k_boltz / tempf;
+  cs_real_t lphi2  = lag_pc->valen * charge * phi_s / _k_boltz / tempf;
 
   /* xtended reduced zeta potential */
   /*  (following the work from Ohshima et al, 1982, JCIS, 90, 17-26)   */
@@ -202,9 +206,11 @@ _edlsp(cs_real_t *distp,
 /*----------------------------------------------------------------------------*/
 
 static void
-_edlsa(cs_real_t *distcc,
+_edlss(cs_real_t *distcc,
        cs_real_t *rpart1,
        cs_real_t *rpart2,
+       cs_real_t  phi1,
+       cs_real_t  phi2,
        cs_real_t  tempf,
        cs_real_t *var)
 {
@@ -216,8 +222,8 @@ _edlsa(cs_real_t *distcc,
 
   /* Reduced zeta potential    */
 
-  cs_real_t lphi1  = lag_pc->valen * charge * lag_pc->phi_p / _k_boltz / tempf;
-  cs_real_t lphi2  = lag_pc->valen * charge * lag_pc->phi_s / _k_boltz / tempf;
+  cs_real_t lphi1  = lag_pc->valen * charge * phi1 / _k_boltz / tempf;
+  cs_real_t lphi2  = lag_pc->valen * charge * phi2 / _k_boltz / tempf;
 
   /* xtended reduced zeta potential */
   /*  (following the work from Ohshima et al, 1982, JCIS, 90, 17-26)   */
@@ -278,6 +284,7 @@ cs_lagr_adh(cs_lnum_t   ip,
   cs_lagr_particle_set_t *p_set = cs_glob_lagr_particle_set;
   const cs_lagr_attribute_map_t *p_am = p_set->p_am;
   unsigned char *part = p_set->p_buffer + p_am->extents * ip;
+  cs_lagr_physico_chemical_t *lag_pc = cs_glob_lagr_physico_chemical;
 
   /*     step = step used to calculate the adhesion force following    */
   /*                         F = U(_d_cut_off+step)-U(_d_cut_off-step)/(2*step)     */
@@ -313,7 +320,7 @@ cs_lagr_adh(cs_lnum_t   ip,
     cs_lnum_t tmp = 1;
     cs_real_t rtmp;
 
-    CS_PROCF(normalen,NORMALEN) (&tmp, &rtmp);
+    cs_random_normal(1, &rtmp);
 
     tmp = (int)nmoyag + sqrt(nmoyag) * rtmp;
     tmp = CS_MAX(0, tmp);
@@ -323,10 +330,9 @@ cs_lagr_adh(cs_lnum_t   ip,
   }
   else {
 
-    cs_lnum_t tmp = 1;
     cs_lnum_t ntmp;
 
-    CS_PROCF(fische, FISCHE)(&tmp, &nmoyag, &ntmp);
+    cs_random_poisson(1, nmoyag, &ntmp);
 
     cs_lagr_particle_set_lnum(part, p_am, CS_LAGR_N_LARGE_ASPERITIES, ntmp);
 
@@ -341,10 +347,9 @@ cs_lagr_adh(cs_lnum_t   ip,
 
     if (nmoyag > 600.0) {
 
-      cs_lnum_t tmp = 1;
       cs_real_t rtmp;
 
-      CS_PROCF(normalen,NORMALEN) (&tmp, &rtmp);
+      cs_random_normal(1, &rtmp);
 
       nbasg = (int)nmoyag + sqrt (nmoyag) * rtmp;
       nbasg = CS_MAX(0, nbasg);
@@ -352,10 +357,9 @@ cs_lagr_adh(cs_lnum_t   ip,
     }
     else {
 
-      cs_lnum_t tmp = 1;
       cs_lnum_t ntmp;
 
-      CS_PROCF(fische, FISCHE)(&tmp, &nmoyag, &ntmp);
+      cs_random_poisson(1, nmoyag, &ntmp);
 
       nbasg = ntmp;
 
@@ -385,7 +389,7 @@ cs_lagr_adh(cs_lnum_t   ip,
       cs_lnum_t tmp = 1;
       cs_real_t rtmp;
 
-      CS_PROCF(normalen,NORMALEN) (&tmp, &rtmp);
+      cs_random_normal(1, &rtmp);
 
       tmp = (int)nmoyap + sqrt(nmoyap) * rtmp;
       tmp = CS_MAX(0, tmp);
@@ -395,10 +399,9 @@ cs_lagr_adh(cs_lnum_t   ip,
     }
     else {
 
-      cs_lnum_t tmp = 1;
       cs_lnum_t ntmp;
 
-      CS_PROCF(fische, FISCHE)(&tmp, &nmoyap, &ntmp);
+      cs_random_poisson(1, nmoyap, &ntmp);
 
       cs_lagr_particle_set_lnum(part, p_am, CS_LAGR_N_SMALL_ASPERITIES, ntmp);
 
@@ -412,10 +415,9 @@ cs_lagr_adh(cs_lnum_t   ip,
 
       if (nmoyap > 600.0) {
 
-        cs_lnum_t tmp = 1;
         cs_real_t rtmp;
 
-        CS_PROCF(normalen,NORMALEN) (&tmp, &rtmp);
+        cs_random_normal(1, &rtmp);
 
         nbasp = (int)nmoyap + sqrt (nmoyap) * rtmp;
         nbasp = CS_MAX(0, nbasp);
@@ -425,10 +427,9 @@ cs_lagr_adh(cs_lnum_t   ip,
       }
       else {
 
-        cs_lnum_t tmp = 1;
         cs_lnum_t ntmp;
 
-        CS_PROCF(fische, FISCHE)(&tmp, &nmoyap, &ntmp);
+        cs_random_poisson(1, nmoyap, &ntmp);
         nbasp = ntmp;
 
       }
@@ -458,7 +459,7 @@ cs_lagr_adh(cs_lnum_t   ip,
       cs_lnum_t tmp = 1;
       cs_real_t rtmp;
 
-      CS_PROCF(normalen,NORMALEN) (&tmp, &rtmp);
+      cs_random_normal(1, &rtmp);
 
       tmp = (int)nmoyap + sqrt(nmoyap) * rtmp;
       tmp = CS_MAX(0, tmp);
@@ -468,10 +469,9 @@ cs_lagr_adh(cs_lnum_t   ip,
     }
     else {
 
-      cs_lnum_t tmp = 1;
       cs_lnum_t ntmp;
 
-      CS_PROCF(fische, FISCHE)(&tmp, &nmoyap, &ntmp);
+      cs_random_poisson(1, nmoyap, &ntmp);
 
       cs_lagr_particle_set_lnum(part, p_am, CS_LAGR_N_SMALL_ASPERITIES, ntmp);
 
@@ -485,10 +485,9 @@ cs_lagr_adh(cs_lnum_t   ip,
 
       if (nmoyap > 600.0) {
 
-        cs_lnum_t tmp = 1;
         cs_real_t rtmp;
 
-        CS_PROCF(normalen,NORMALEN) (&tmp, &rtmp);
+        cs_random_normal(1, &rtmp);
 
         nbasp = (int)nmoyap + sqrt (nmoyap) * rtmp;
         nbasp = CS_MAX(0, nbasp);
@@ -496,10 +495,9 @@ cs_lagr_adh(cs_lnum_t   ip,
       }
       else {
 
-        cs_lnum_t tmp = 1;
         cs_lnum_t ntmp;
 
-        CS_PROCF(fische, FISCHE)(&tmp, &nmoyap, &ntmp);
+        cs_random_poisson(1, nmoyap, &ntmp);
 
         nbasp = ntmp;
 
@@ -532,8 +530,8 @@ cs_lagr_adh(cs_lnum_t   ip,
     cs_real_t distp = dismin + _d_cut_off + step * (3 - 2 * (np+1));
 
     cs_real_t uvdwsp, uedlsp;
-    _vdwsp(&distp, &rpart, &uvdwsp);
-    _edlsp(&distp, &rpart, tempf, &uedlsp);
+    _vdwsp(lag_pc->cstham, lag_pc->lambda_vdw, &distp, &rpart, &uvdwsp);
+    _edlsp(&distp, &rpart, lag_pc->phi_p, lag_pc->phi_s, tempf, &uedlsp);
 
     udlvor[np] = (uvdwsp + uedlsp) * (1 - scovag - scovap);
 
@@ -552,8 +550,8 @@ cs_lagr_adh(cs_lnum_t   ip,
 
     cs_real_t uvdwss, uedlss;
 
-    _vdwsa(&distcc, &rpart, &rayasp, &uvdwss);
-    _edlsa(&distcc, &rpart, &rayasp, tempf, &uedlss);
+    _vdwss(lag_pc->cstham, lag_pc->lambda_vdw, &distcc, &rpart, &rayasp, &uvdwss);
+    _edlss(&distcc, &rpart, &rayasp, lag_pc->phi_p, lag_pc->phi_s, tempf, &uedlss);
 
     udlvor[np] = uvdwss + uedlss;
 
@@ -577,8 +575,8 @@ cs_lagr_adh(cs_lnum_t   ip,
 
     cs_real_t uvdwss, uedlss;
 
-    _vdwsa(&distcc, &rpart, &rayasg, &uvdwss);
-    _edlsa(&distcc, &rpart, &rayasg, tempf, &uedlss);
+    _vdwss(lag_pc->cstham, lag_pc->lambda_vdw, &distcc, &rpart, &rayasg, &uvdwss);
+    _edlss(&distcc, &rpart, &rayasg, lag_pc->phi_p, lag_pc->phi_s, tempf, &uedlss);
 
     udlvor[np] = uvdwss + uedlss;
 
@@ -603,11 +601,9 @@ cs_lagr_adh(cs_lnum_t   ip,
   else
     *adhesion_energ = CS_ABS(*adhesion_energ);
 
-
-  /*  Calculation of adhesion torques exerted on the particle     */
-  cs_lnum_t tmp = 1;
+  /*  Calculation of adhesion torques exerted on the particle */
   cs_real_t rtmp;
-  CS_PROCF(zufall,ZUFALL)(&tmp, &rtmp);
+  cs_random_uniform(1, &rtmp);
 
   cs_real_t dismom = rtmp;
   if (nbasp > 0)
@@ -619,7 +615,7 @@ cs_lagr_adh(cs_lnum_t   ip,
     /* In the sphere-plate case, we use the deformation given by the DMT theory,
      * which is close to our approach  */
 
-    cs_real_t omsurf = cs_glob_lagr_physico_chemical->cstham / (24.0 * cs_math_pi * pow (_d_cut_off, 2));
+    cs_real_t omsurf = lag_pc->cstham / (24.0 * cs_math_pi * pow (_d_cut_off, 2));
     dismom = pow ((4.0 * cs_math_pi * omsurf * (pow (rpart, 2.0)) / modyeq), (1.0 / 3.0));
     /* dismom = (12.0d0 * cs_math_pi * omsurf * (rpart**2)/modyeq)**(1.0d0/3.0d0) */
 
@@ -631,5 +627,68 @@ cs_lagr_adh(cs_lnum_t   ip,
 
 }
 
+/*============================================================================
+ * Public function definitions
+ *============================================================================*/
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Calculation of the adhesion force and adhesion energy
+ *        between two particles
+ *
+ * \param[in]  dpart            particle diameter
+ * \param[in]  tempf            thermal scalar value at current time step
+ * \param[out] adhesion_energ   particle adhesion energy
+ * \param[out] adhesion_force   particle adhesion force
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_lagr_adh_pp(cs_real_t   dpart,
+               cs_real_t   tempf,
+               cs_real_t  *adhesion_energ,
+               cs_real_t  *adhesion_force)
+{
+  /* ================================================================  */
+  /* 0.    initialization */
+  /* ================================================================  */
+
+  cs_lagr_physico_chemical_t *lag_pc = cs_glob_lagr_physico_chemical;
+  cs_lagr_clogging_model_t *lag_cm = cs_glob_lagr_clogging_model;
+
+  cs_real_t step = 1e-11;
+
+  /* ================================================================  */
+  /* 1.    calculation of the adhesion force  */
+  /* ================================================================  */
+
+  cs_real_t rpart = 0.5 * dpart;
+
+  /* Sum of {particle-plane} and {particle-asperity} energies     */
+
+  cs_real_t udlvor[2];
+
+  /* Interaction between the spheres */
+
+  for (cs_lnum_t np = 0; np < 2; np++) {
+
+    udlvor[np] = 0.0;
+
+    cs_real_t distcc = _d_cut_off + step * (3 - 2 * (np + 1)) + 2.0 * rpart;
+
+    cs_real_t uvdwss, uedlss;
+
+    _vdwss(lag_cm->csthpp, lag_pc->lambda_vdw, &distcc, &rpart, &rpart, &uvdwss);
+    _edlss(&distcc, &rpart, &rpart, lag_pc->phi_p, lag_pc->phi_p, tempf, &uedlss);
+
+    udlvor[np] = uvdwss + uedlss;
+
+  }
+
+  *adhesion_force = CS_MAX( -(udlvor[1] - udlvor[0]) / (2.0 * step), 0.0);
+
+  *adhesion_energ  = CS_MAX(-udlvor[0], 0.0);
+
+}
 
 END_C_DECLS
