@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -79,7 +79,7 @@ BEGIN_C_DECLS
  *============================================================================*/
 
 /*!
-  \file cs_lagr_listing.c
+  \file cs_lagr_log.c
 */
 
 /*! \cond DOXYGEN_SHOULD_SKIP_THIS */
@@ -354,59 +354,37 @@ cs_lagr_log_setup(void)
 
   cs_log_printf
     (CS_LOG_SETUP,
-     _("\n  Trajectory/particle postprocessing options:\n"
-       "    fluid velocity seen:                          %s\n"
-       "    velocity:                                     %s\n"
-       "    residence time:                               %s\n"
-       "    diameter:                                     %s\n"
-       "    temperature:                                  %s\n"
-       "    mass:                                         %s\n"),
-     _status(cs_glob_lagr_post_options->ivisv1),
-     _status(cs_glob_lagr_post_options->ivisv2),
-     _status(cs_glob_lagr_post_options->ivistp),
-     _status(cs_glob_lagr_post_options->ivisdm),
-     _status(cs_glob_lagr_post_options->iviste),
-     _status(cs_glob_lagr_post_options->ivismp));
-
-  if (cs_glob_lagr_model->physical_model == 2)
-    cs_log_printf
-      (CS_LOG_SETUP,
-       _("    shrinking core diameter:                      %s\n"
-         "    moisture mass:                                %s\n"
-         "    active coal mass:                             %s\n"
-         "    coke mass:                                    %s\n"),
-       _status(cs_glob_lagr_post_options->ivisdk),
-       _status(cs_glob_lagr_post_options->iviswat),
-       _status(cs_glob_lagr_post_options->ivisch),
-       _status(cs_glob_lagr_post_options->ivisck));
+     _("\n  Trajectory/particle postprocessing options:\n"));
+     for (int attr = 0; attr < CS_LAGR_N_ATTRIBUTES; attr++) {
+       if (cs_lagr_post_get_attr(attr))
+         cs_log_printf(CS_LOG_SETUP,
+                       "    %s\n", cs_lagr_attribute_name[attr]);
+     }
 
   cs_log_printf
     (CS_LOG_SETUP,
-     _("\n  Statistics for particles/boundary interaction:\n"
-       "    compute wall statistics: %s\n"),
-     _status(cs_glob_lagr_post_options->iensi3));
+     _("\n  Statistics for particles/boundary interaction:\n"));
 
-  if (cs_glob_lagr_post_options->iensi3)
-    cs_log_printf
-      (CS_LOG_SETUP,
-       _("    number of interactions:                       %s\n"
-         "    particle mass flow:                           %s\n"
-         "    impact angle:                                 %s\n"
-         "    impact velocity:                              %s\n"
-         "    interactions with fouling:                    %s\n"
-         "    fouling coal mass flux:                       %s\n"
-         "    fouling coal diameter:                        %s\n"
-         "    fouling coal coke fraction:                   %s\n"
-         "    number of additional user statistics:         %d\n"),
-       _status(cs_glob_lagr_boundary_interactions->inbrbd),
-       _status(cs_glob_lagr_boundary_interactions->iflmbd),
-       _status(cs_glob_lagr_boundary_interactions->iangbd),
-       _status(cs_glob_lagr_boundary_interactions->ivitbd),
-       _status(cs_glob_lagr_boundary_interactions->iencnbbd),
-       _status(cs_glob_lagr_boundary_interactions->iencmabd),
-       _status(cs_glob_lagr_boundary_interactions->iencdibd),
-       _status(cs_glob_lagr_boundary_interactions->iencckbd),
-       cs_glob_lagr_boundary_interactions->nusbor);
+  if (cs_glob_lagr_boundary_interactions->inbrbd)
+    cs_log_printf(CS_LOG_SETUP, "    %s\n", "number of interactions");
+  if (cs_glob_lagr_boundary_interactions->iflmbd)
+    cs_log_printf(CS_LOG_SETUP, "    %s\n", "particle mass flow");
+  if (cs_glob_lagr_boundary_interactions->iangbd)
+    cs_log_printf(CS_LOG_SETUP, "    %s\n", "impact angle");
+  if (cs_glob_lagr_boundary_interactions->ivitbd)
+    cs_log_printf(CS_LOG_SETUP, "    %s\n", "impact velocity");
+  if (cs_glob_lagr_boundary_interactions->iencnbbd)
+    cs_log_printf(CS_LOG_SETUP, "    %s\n", "interactions with fouling");
+  if (cs_glob_lagr_boundary_interactions->iencmabd)
+    cs_log_printf(CS_LOG_SETUP, "    %s\n", "fouling coal mass flux");
+  if (cs_glob_lagr_boundary_interactions->iencdibd)
+    cs_log_printf(CS_LOG_SETUP, "    %s\n", "fouling coal diameter");
+  if (cs_glob_lagr_boundary_interactions->iencckbd)
+    cs_log_printf(CS_LOG_SETUP, "    %s\n", "fouling coal coke fraction");
+  if (cs_glob_lagr_boundary_interactions->nusbor)
+    cs_log_printf(CS_LOG_SETUP,
+                  _("    number of additional user statistics: %d\n"),
+                  cs_glob_lagr_boundary_interactions->nusbor);
 
   /* Volumic statistics   */
 
@@ -417,7 +395,7 @@ cs_lagr_log_setup(void)
 
   cs_log_printf
     (CS_LOG_SETUP,
-     _("  Start of calculation from absolute iteration n°: %10d\n"),
+     _("  Start of calculation from absolute iteration number: %10d\n"),
      cs_glob_lagr_stat_options->idstnt);
 
   if (cs_glob_time_step->nt_cur >= cs_glob_lagr_stat_options->idstnt) {
@@ -426,7 +404,7 @@ cs_lagr_log_setup(void)
       cs_log_printf
         (CS_LOG_SETUP,
          _("  Start of steady-state statistics from Lagrangian "
-           "iteration n°: %10d)\n"),
+           "iteration number: %10d)\n"),
          cs_glob_lagr_stat_options->nstist);
 
     }
@@ -463,13 +441,11 @@ cs_lagr_log_iteration(void)
   const cs_lagr_particle_counter_t *pc = cs_lagr_update_particle_counter();
 
   /* Number of particles  */
-  cs_log_printf(CS_LOG_DEFAULT,
-                _("\n"));
+  cs_log_printf(CS_LOG_DEFAULT, "\n");
   cs_log_printf(CS_LOG_DEFAULT,
                 _("   Current number of particles "
                   "(with and without statistical weight) :\n"));
-  cs_log_printf(CS_LOG_DEFAULT,
-                _("\n"));
+  cs_log_printf(CS_LOG_DEFAULT, "\n");
   cs_log_printf
     (CS_LOG_DEFAULT,
      _("ln  newly injected                           %8llu   %14.5E\n"),
@@ -517,82 +493,81 @@ cs_lagr_log_iteration(void)
   cs_log_printf(CS_LOG_DEFAULT,
                 _("   Zone     Mass flow rate(kg/s)      Boundary type\n"));
 
-  int nbfr = 0;
-  cs_lagr_bdy_condition_t *bdy_cond = cs_lagr_get_bdy_conditions();
+  cs_lagr_zone_data_t *bdy_cond = cs_lagr_get_boundary_conditions();
 
-  /* TODO log for internal zone */
+  /* TODO log for volume conditions and internal zone */
+#if 0
   cs_lagr_internal_condition_t *internal_cond = cs_lagr_get_internal_conditions();
+#endif
 
-  for (cs_lnum_t ii = 0; ii < bdy_cond->n_b_zones; ii++) {
+  int n_stats = cs_glob_lagr_model->n_stat_classes + 1;
 
-    if (bdy_cond->b_zone_id[ii] >= nbfr-1)
-      nbfr = bdy_cond->b_zone_id[ii] + 1;
+  cs_real_t *flow_rate;
+  int flow_rate_size = bdy_cond->n_zones*n_stats;
+  BFT_MALLOC(flow_rate, flow_rate_size, cs_real_t);
 
-  }
+  for (int i = 0; i < flow_rate_size; i++)
+    flow_rate[i] = bdy_cond->particle_flow_rate[i];
 
-  if (cs_glob_rank_id >= 0)
-    cs_parall_counter_max(&nbfr, 1);
+  cs_parall_sum(flow_rate_size, CS_REAL_TYPE, flow_rate);
 
-  cs_real_t debloc[2];
-  for (cs_lnum_t nb = 0; nb < nbfr; nb++) {
+  for (cs_lnum_t z_id = 0; z_id < bdy_cond->n_zones; z_id++) {
 
-    debloc[0] = 0.0;
-    debloc[1] = 0.0;
+    if (CS_ABS(flow_rate[z_id*n_stats]) > 0.) {
 
-    for (cs_lnum_t ii = 0; ii < bdy_cond->n_b_zones; ii++) {
+      const char *chcond;
 
-      if (bdy_cond->b_zone_id[ii] == nb) {
-        debloc[0] = 1.0;
-        debloc[1] = bdy_cond->particle_flow_rate[nb];
-      }
+      if (bdy_cond->zone_type[z_id] == CS_LAGR_INLET)
+        chcond = _("inlet");
 
-    }
+      else if (bdy_cond->zone_type[z_id] == CS_LAGR_REBOUND)
+        chcond = _("rebound");
 
-    cs_parall_sum(2, CS_REAL_TYPE, debloc);
+      else if (bdy_cond->zone_type[z_id] == CS_LAGR_OUTLET)
+        chcond = _("outlet");
 
-    if (debloc[0] > 0.5) {
+      else if (   bdy_cond->zone_type[z_id] == CS_LAGR_DEPO1
+               || bdy_cond->zone_type[z_id] == CS_LAGR_DEPO2)
+        chcond = _("deposition");
 
-      char const *chcond;
+      else if (bdy_cond->zone_type[z_id] == CS_LAGR_FOULING)
+        chcond = _("fouling");
 
-      if (bdy_cond->b_zone_natures[nb] == CS_LAGR_INLET)
-        chcond = "INLET";
+      else if (bdy_cond->zone_type[z_id] == CS_LAGR_DEPO_DLVO)
+        chcond = _("dlvo conditions");
 
-      else if (bdy_cond->b_zone_natures[nb] == CS_LAGR_REBOUND)
-        chcond = "REBOUND";
-
-      else if (bdy_cond->b_zone_natures[nb] == CS_LAGR_OUTLET)
-        chcond = "OUTLET";
-
-      else if (   bdy_cond->b_zone_natures[nb] == CS_LAGR_DEPO1
-               || bdy_cond->b_zone_natures[nb] == CS_LAGR_DEPO2)
-        chcond = "DEPOSITION";
-
-      else if (bdy_cond->b_zone_natures[nb] == CS_LAGR_FOULING)
-        chcond = "FOULING";
-
-      else if (bdy_cond->b_zone_natures[nb] == CS_LAGR_DEPO_DLVO)
-        chcond = "DLVO CONDITIONS";
-
-      else if (bdy_cond->b_zone_natures[nb] == CS_LAGR_SYM)
-        chcond = "SYMMETRY";
+      else if (bdy_cond->zone_type[z_id] == CS_LAGR_SYM)
+        chcond = _("symmetry");
 
       else
-        chcond = "USER";
+        chcond = _("user");
 
       cs_log_printf(CS_LOG_DEFAULT,
-                    "  %3d          %12.5E         %s\n",
-                    nb,
-                    debloc[1]/cs_glob_lagr_time_step->dtp,
+                    "  %3d          %12.5e         %s\n",
+                    z_id,
+                    flow_rate[z_id*n_stats]/cs_glob_lagr_time_step->dtp,
                     chcond);
 
+      for (int j = 1; j < n_stats; j++) {
+        if (CS_ABS(flow_rate[z_id*n_stats + j]) > 0)
+          cs_log_printf(CS_LOG_DEFAULT,
+                        "    class %3d  %12.5e         %s\n",
+                        j,
+                        flow_rate[z_id*n_stats]/cs_glob_lagr_time_step->dtp,
+                        chcond);
+      }
+
     }
 
   }
 
   cs_log_separator(CS_LOG_DEFAULT);
 
+  BFT_FREE(flow_rate);
+
   /* Boundary statistics  */
-  if (cs_glob_lagr_post_options->iensi3 == 1) {
+
+  if (cs_glob_lagr_dim->nvisbr > 0) {
 
     cs_log_printf(CS_LOG_DEFAULT,
                   _("   Boundary statistics :\n"));
@@ -723,21 +698,23 @@ cs_lagr_log_iteration(void)
 
     }
 
+    cs_real_t vmax[2] = {cs_glob_lagr_source_terms->vmax,
+                         cs_glob_lagr_source_terms->tmamax};
+    cs_gnum_t cpt = cs_glob_lagr_source_terms->ntxerr;
+
+    cs_parall_max(2, CS_REAL_TYPE, vmax);
+    cs_parall_counter(&cpt, 1);
+
     cs_log_printf(CS_LOG_DEFAULT,
-                  _("Maximum particle volume fraction : %14.5E\n"),
-                  cs_glob_lagr_source_terms->vmax);
+                  _("Maximum particle volume fraction: %14.5e\n"), vmax[0]);
     cs_log_printf(CS_LOG_DEFAULT,
-                  _("Maximum particle mass fraction :  %14.5E\n"),
-                  cs_glob_lagr_source_terms->tmamax);
+                  _("Maximum particle mass fraction:   %14.5e\n"), vmax[1]);
     cs_log_printf(CS_LOG_DEFAULT,
-                  _("Number of cells with a part. volume fraction greater than 0.8 :%10d\n"),
-                  cs_glob_lagr_source_terms->ntxerr);
+                  _("Number of cells with a particle volume fraction "
+                    "greater than 0.8: %10llu\n"), (unsigned long long)cpt);
     cs_log_separator(CS_LOG_DEFAULT);
 
   }
-
-  return;
-
 }
 
 /*----------------------------------------------------------------------------*/
