@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -53,7 +53,7 @@ typedef enum {
   CS_LAGR_CELL_NUM,            /*!< local cell number (1 to n) */
   CS_LAGR_RANK_ID,             /*!< local parallel rank id */
 
-  CS_LAGR_SWITCH_ORDER_1,
+  CS_LAGR_REBOUND_ID,          /*!< number of time steps since rebound, or -1 */
 
   CS_LAGR_RANDOM_VALUE,        /*!< associated random value (for sampling) */
 
@@ -66,11 +66,18 @@ typedef enum {
   CS_LAGR_VELOCITY,
   CS_LAGR_VELOCITY_SEEN,
 
+  CS_LAGR_TR_TRUNCATE,         /*!< portion of trajectory truncated */
+  CS_LAGR_TR_REPOSITION,       /*!< number of times the particle is repositioned
+                                    at the cell center, or -1 when lost (to be
+                                    discarded at the next displacement step) */
+
   /* Arrays for 2nd order scheme */
 
   CS_LAGR_TURB_STATE_1,        /* turbulence characteristics of first pass */
   CS_LAGR_PRED_VELOCITY,       /* 1st step prediction for particle velocity */
   CS_LAGR_PRED_VELOCITY_SEEN,  /* 1st step prediction for relative velocity */
+  CS_LAGR_V_GAUSS,             /* 1st step Gaussian variable */
+  CS_LAGR_BR_GAUSS,            /* 1st step Brownian motion Gaussian variable */
 
   /* Deposition submodel additional parameters */
 
@@ -299,6 +306,28 @@ cs_lagr_get_attr_info(const cs_lagr_particle_set_t  *particles,
                       cs_datatype_t                 *datatype,
                       int                           *count);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Check that query for a given particle attribute is valid.
+ *
+ * \param[in]   particles             associated particle set
+ * \param[in]   attr                  attribute whose values are required
+ * \param[in]   datatype              associated value type
+ * \param[in]   stride                number of values per particle
+ * \param[in]   component_id          if -1 : extract the whole attribute
+ *                                    if >0 : id of the component to extract
+ *
+ * \return 0 in case of success, 1 in case of error
+ */
+/*----------------------------------------------------------------------------*/
+
+int
+cs_lagr_check_attr_query(const cs_lagr_particle_set_t  *particles,
+                         cs_lagr_attribute_t            attr,
+                         cs_datatype_t                  datatype,
+                         int                            stride,
+                         int                            component_id);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Check if a particle attribute is in a valid range.
@@ -927,19 +956,22 @@ cs_lagr_particle_get_cell_id(const void                     *particle,
 /*!
  * \brief Set cell id of a given particle in a set
  *
- * \param[in]  particle_set  pointer to particle set
- * \param[in]  particle_id   particle number
- * \param[in]  cell_id       containing cell (0-based)
+ * \param[in]  particle  pointer to particle data
+ * \param[in]  attr_map  pointer to attribute map
+ * \param[in]  cell_id   containing cell (0-based)
  */
 /*----------------------------------------------------------------------------*/
 
 inline static void
-cs_lagr_particle_set_cell_id(cs_lagr_particle_set_t  *particle_set,
-                             cs_lnum_t                particle_id,
-                             cs_lnum_t                cell_id)
+cs_lagr_particle_set_cell_id(void                           *particle,
+                             const cs_lagr_attribute_map_t  *attr_map,
+                             cs_lnum_t                       cell_id)
 {
-  cs_lagr_particles_set_lnum(particle_set, particle_id, CS_LAGR_CELL_NUM,
-                             cell_id + 1);
+  assert(attr_map->count[0][CS_LAGR_CELL_NUM] > 0);
+
+  *((cs_lnum_t *)(  (unsigned char *)particle
+                  + attr_map->displ[0][CS_LAGR_CELL_NUM]))
+    = cell_id + 1;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -1289,15 +1321,30 @@ cs_lagr_particle_set_resize(cs_lnum_t  n_min_particles);
 void
 cs_lagr_set_reallocation_factor(double f);
 
-/*----------------------------------------------------------------------------
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Get global maximum number of particles.
+ *
+ * By default, the number is limited only by local \ref cs_lnum_t and global
+ * \ref cs_gnum_t data representation limits.
+ *
+ * \return  global maximum number of particles
+ */
+/*----------------------------------------------------------------------------*/
+
+unsigned long long
+cs_lagr_get_n_g_particles_max(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
  * \brief Set global maximum number of particles.
  *
- * By default, the number is limited only by local cs_lnum_t and global
- * cs_gnum_t data representation limits.
+ * By default, the number is limited only by local \ref cs_lnum_t and global
+ * \ref cs_gnum_t data representation limits.
  *
- * parameters:
- *   n_g_particles_max <-- global maximum number of particles
-*----------------------------------------------------------------------------*/
+ * \param[in]  n_g_particles_max  global maximum number of particles
+ */
+/*----------------------------------------------------------------------------*/
 
 void
 cs_lagr_set_n_g_particles_max(unsigned long long  n_g_particles_max);
