@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -114,18 +114,12 @@ _lagr_section_name(cs_lagr_attribute_t  attr,
                    int                  comp_id,
                    char                 sec_name[128])
 {
-  size_t l;
-
   if (comp_id < 0)
-    l= snprintf(sec_name, 127, "particle_%s::vals::0",
-                cs_lagr_attribute_name[attr] + strlen("cs_lagr_"));
+    snprintf(sec_name, 127, "particle_%s::vals::0",
+             cs_lagr_attribute_name[attr]);
   else
-    l= snprintf(sec_name, 127, "particle%s::vals::%d::0",
-                cs_lagr_attribute_name[attr]+ strlen("cs_lagr_"), comp_id);
-
-  sec_name[127] = '\0';
-  for (size_t i = 0; i < l; i++)
-    sec_name[i] = tolower(sec_name[i]);
+    snprintf(sec_name, 127, "particle_%s::vals::%d::0",
+             cs_lagr_attribute_name[attr], comp_id);
 }
 
 /*----------------------------------------------------------------------------
@@ -177,9 +171,6 @@ _legacy_section_names(cs_lagr_attribute_t  attr,
   sec_name_2[0] = '\0';
 
   switch(attr) {
-  case CS_LAGR_SWITCH_ORDER_1:
-    strcpy(sec_name_0, "order_1");
-    break;
   case CS_LAGR_RANDOM_VALUE:
     strcpy(sec_name_0, "random_value");
     break;
@@ -333,44 +324,12 @@ _set_particle_values(cs_lagr_particle_set_t  *particles,
 
   /* Check consistency */
 
-  if (datatype != _datatype || stride != _count) {
-    char attr_name[32];
-    const char *_attr_name = attr_name;
-    if (attr < CS_LAGR_N_ATTRIBUTES)
-      _attr_name = cs_lagr_attribute_name[attr];
-    else {
-      snprintf(attr_name, 31, "%d", (int)attr);
-      attr_name[31] = '\0';
-    }
-    bft_error(__FILE__, __LINE__, 0,
-              _("Attribute %s is of datatype %s and stride %d\n"
-                "but %s and %d were requested."),
-              _attr_name,
-              cs_datatype_name[_datatype], _count,
-              cs_datatype_name[datatype], stride);
+  if (cs_lagr_check_attr_query(particles,
+                               attr,
+                               datatype,
+                               stride,
+                               component_id) != 0)
     return 1;
-  }
-
-  /* Check component_id */
-  if (component_id != -1) {
-    if (component_id < 0 || component_id >= stride) {
-      char attr_name[32];
-      const char *_attr_name = attr_name;
-      if (attr < CS_LAGR_N_ATTRIBUTES)
-        _attr_name = cs_lagr_attribute_name[attr];
-      else {
-        snprintf(attr_name, 31, "%d", (int)attr);
-        attr_name[31] = '\0';
-      }
-      bft_error(__FILE__, __LINE__, 0,
-                _("Attribute %s has a number of components equal to %d\n"
-                  "but component %d is requested."),
-                _attr_name,
-                stride,
-                component_id);
-      return 1;
-    }
-  }
 
   /* No offset if export of the whole attribute */
   if (component_id == -1)
@@ -423,24 +382,26 @@ _init_particle_values(cs_lagr_particle_set_t  *particles,
     return 1;
 
   /* Check component_id */
-  if (component_id != -1) {
-    if (component_id < 0 || component_id >= stride) {
-      char attr_name[32];
-      const char *_attr_name = attr_name;
-      if (attr < CS_LAGR_N_ATTRIBUTES)
-        _attr_name = cs_lagr_attribute_name[attr];
-      else {
-        snprintf(attr_name, 31, "%d", (int)attr);
-        attr_name[31] = '\0';
-      }
-      bft_error(__FILE__, __LINE__, 0,
-                _("Attribute %s has a number of components equal to %d\n"
-                  "but component %d is requested."),
-                _attr_name,
-                stride,
-                component_id);
-      return 1;
+  if (component_id < -1 || component_id >= stride) {
+    char attr_name[128] = "";
+    attr_name[127] = '\0';
+    const char *_attr_name = attr_name;
+    if (attr < CS_LAGR_N_ATTRIBUTES) {
+      snprintf(attr_name, 127, "CS_LAGR_%s", cs_lagr_attribute_name[attr]);
+      size_t l = strlen(attr_name);
+      for (size_t i = 0; i < l; i++)
+        attr_name[i] = toupper(attr_name[i]);
     }
+    else {
+      snprintf(attr_name, 127, "%d", (int)attr);
+    }
+    bft_error(__FILE__, __LINE__, 0,
+              _("Attribute %s has a number of components equal to %d\n"
+                "but component %d is requested."),
+              _attr_name,
+              stride,
+              component_id);
+    return 1;
   }
 
   /* No offset if export of the whole attribute */
@@ -693,7 +654,6 @@ cs_lagr_restart_read_particle_data(cs_restart_t  *r)
 
     case CS_LAGR_COORDS:
     case CS_LAGR_RANK_ID:
-    case CS_LAGR_SWITCH_ORDER_1:
       break;
 
     case CS_LAGR_NEIGHBOR_FACE_ID:
@@ -901,7 +861,6 @@ cs_lagr_restart_write_particle_data(cs_restart_t  *r)
 
     case CS_LAGR_COORDS:
     case CS_LAGR_RANK_ID:
-    case CS_LAGR_SWITCH_ORDER_1:
       break;
 
     case CS_LAGR_NEIGHBOR_FACE_ID:
@@ -941,6 +900,7 @@ cs_lagr_restart_write_particle_data(cs_restart_t  *r)
       for (int s_id = 0; s_id < n_sections; s_id++) {
 
         int comp_id = (n_sections == 1) ? -1 : s_id;
+        int c_stride = (n_sections == 1) ? stride : 1;
 
         cs_lagr_get_particle_values(p_set,
                                     attr,
@@ -969,13 +929,13 @@ cs_lagr_restart_write_particle_data(cs_restart_t  *r)
         cs_restart_write_section(r,
                                  sec_name,
                                  particles_location_id,
-                                 stride,
+                                 c_stride,
                                  restart_type,
                                  vals);
 
         retval += 1;
 
-        }
+      }
 
     }
 
