@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -586,6 +586,8 @@ _intersect_edges(cs_join_t               *this_join,
                                         sync_block,
                                         inter_edges);
 
+      cs_join_inter_edges_destroy(&sync_block);
+
       /* Add new vertices to edge and mesh description if necessary */
 
       cs_join_intersect_update_struct(param.verbosity,
@@ -593,8 +595,6 @@ _intersect_edges(cs_join_t               *this_join,
                                       work_jmesh,
                                       &inter_edges);
 
-      cs_join_inter_edges_destroy(&sync_block);
-
       cs_join_mesh_sync_vertices(work_jmesh);
 
     }
@@ -704,6 +704,11 @@ _get_local_o2n_vtx_gnum(cs_join_param_t    param,
     }
   }
 
+  if (bi.rank_step > 1) {
+    for (cs_lnum_t i = 0; i < n_tot_vertices; i++)
+      dest_rank[i] *= bi.rank_step;
+  }
+
   /* Send old vtx gnum to matching block */
 
   d = cs_all_to_all_create(n_tot_vertices,
@@ -1192,11 +1197,11 @@ _split_faces(cs_join_t           *this_join,
 
     *join_type = _join_type;
 
-    if (join_type == CS_JOIN_TYPE_NULL)
+    if (*join_type == CS_JOIN_TYPE_NULL)
       bft_printf(_("\n  Joining operation is null.\n"));
-    else if (join_type == CS_JOIN_TYPE_CONFORMING)
+    else if (*join_type == CS_JOIN_TYPE_CONFORMING)
       bft_printf(_("\n  Joining operation is conforming.\n"));
-    else if (join_type == CS_JOIN_TYPE_NON_CONFORMING)
+    else if (*join_type == CS_JOIN_TYPE_NON_CONFORMING)
       bft_printf(_("\n  Joining operation is non-conforming.\n"));
     bft_printf_flush();
 
@@ -1700,6 +1705,10 @@ cs_join_all(bool  preprocess)
 
   full_clock_start = cs_timer_wtime();
 
+  /* Disable all writers until explicitely enabled for this stage */
+
+  cs_post_disable_writer(0);
+
   cs_join_post_init();
 
   /* Loop on each defined joining to deal with */
@@ -1948,6 +1957,10 @@ cs_join_all(bool  preprocess)
     cs_glob_n_joinings = 0;
   }
 
+  /* Re-enable writers disabled when entering this stage */
+
+  cs_post_enable_writer(0);
+
   full_clock_end = cs_timer_wtime();
 
   if (mesh->verbosity > 0) {
