@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -317,35 +317,22 @@ cs_join_mesh_minmax_tol(cs_join_param_t    param,
  * ranks.
  *
  * parameters:
- *   n_ranks         <-- number of ranks in the MPI communicator
- *   send_rank_index <-- index on ranks for the face distribution
- *   send_faces      <-- list of face ids to send
- *   send_mesh       <-- pointer to the sending cs_join_mesh_t structure
- *   recv_mesh       <-> pointer to the receiving cs_join_mesh_t structure
- *   comm            <-- mpi communicator on which take places comm.
+ *   n_send     <-- number of face/rank couples to send
+ *   send_rank  <-- index on ranks for the face distribution
+ *   send_faces <-- list of face ids to send
+ *   send_mesh  <-- pointer to the sending cs_join_mesh_t structure
+ *   recv_mesh  <-> pointer to the receiving cs_join_mesh_t structure
+ *   comm       <-- mpi communicator on which take places comm.
  *---------------------------------------------------------------------------*/
 
 void
-cs_join_mesh_exchange(int                    n_ranks,
-                      const cs_lnum_t        send_rank_index[],
+cs_join_mesh_exchange(cs_lnum_t              n_send,
+                      const cs_lnum_t        send_rank[],
                       const cs_lnum_t        send_faces[],
                       const cs_join_mesh_t  *send_mesh,
                       cs_join_mesh_t        *recv_mesh,
                       MPI_Comm               comm);
 
-/*----------------------------------------------------------------------------
- * Synchronize vertices definition over the ranks.
- *
- * For vertices with the same global number but a different tolerance,
- * we keep the smallest tolerance.
- *
- * parameters:
- *  mesh <->  pointer to the cs_join_mesh_t structure to synchronize
- *---------------------------------------------------------------------------*/
-
-void
-cs_join_mesh_sync_vertices(cs_join_mesh_t  *mesh);
-
 #endif /* defined(HAVE_MPI) */
 
 /*----------------------------------------------------------------------------
