@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2015 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -41,6 +41,7 @@
 #include "cs_halo.h"
 #include "cs_interface.h"
 #include "cs_numbering.h"
+#include "cs_range_set.h"
 
 #include "cs_mesh_builder.h"
 
@@ -125,6 +126,8 @@ typedef struct {
 
   cs_interface_set_t  *vtx_interfaces;   /* Vertices interface set */
   cs_halo_t           *halo;             /* Ghost cells structure */
+  cs_range_set_t      *vtx_range_set;    /* Handle local/distant ranges for
+                                            vertices in parallel */
 
   cs_numbering_t  *cell_numbering;   /* Cell numbering info */
   cs_numbering_t  *i_face_numbering; /* Interior face numbering info */
@@ -149,16 +152,16 @@ typedef struct {
 
   /* Group and family features */
 
-  cs_lnum_t   n_groups;            /* Number of groups */
-  cs_lnum_t  *group_idx;           /* Starting index in the in group_lst */
-  char       *group_lst;           /* List of group names */
+  int         n_groups;            /* Number of groups */
+  int        *group_idx;           /* Starting index in group */
+  char       *group;               /* List of group names */
 
-  cs_lnum_t   n_families;          /* Number of families */
-  cs_lnum_t   n_max_family_items;  /* Max. number of items for one family */
-  cs_lnum_t  *family_item;         /* Family items */
-  cs_lnum_t  *cell_family;         /* Cell family */
-  cs_lnum_t  *i_face_family;       /* Interior face family */
-  cs_lnum_t  *b_face_family;       /* Boundary face family */
+  int         n_families;          /* Number of families */
+  int         n_max_family_items;  /* Max. number of items for one family */
+  int        *family_item;         /* Family items */
+  int        *cell_family;         /* Cell family */
+  int        *i_face_family;       /* Interior face family */
+  int        *b_face_family;       /* Boundary face family */
 
   fvm_group_class_set_t *class_defs;  /* Definition of group classes for
                                          selection and postprocessing (built
@@ -290,24 +293,6 @@ void CS_PROCF(synvie, SYNVIE)
  cs_real_t  var[]
 );
 
-/*----------------------------------------------------------------------------
- * Update a vector array in case of parallelism and/or periodicity,
- * ignoring periodicity of rotation.
- *
- * Fortran interface:
- *
- * subroutine synvnr(var)
- * *****************
- *
- * var   : <-> : interleaved vector (of dimension 3)
- *----------------------------------------------------------------------------*/
-
-void
-CS_PROCF (synvnr, SYNVNR)
-(
- cs_real_t  var[]
-);
-
 /*----------------------------------------------------------------------------
  * Update a diagonal tensor array in case of parallelism and/or periodicity.
  *
@@ -442,6 +427,16 @@ cs_mesh_free_rebuildable(cs_mesh_t  *mesh,
 void
 cs_mesh_discard_free_faces(cs_mesh_t  *mesh);
 
+/*----------------------------------------------------------------------------
+ * Generate or update list of mesh boundary cells.
+ *
+ * parameters:
+ *   mesh  <->  pointer to a cs_mesh_t structure
+ *----------------------------------------------------------------------------*/
+
+void
+cs_mesh_update_b_cells(cs_mesh_t  *mesh);
+
 /*----------------------------------------------------------------------------
  * Compute or update mesh structure members that depend on other members,
  * but whose results may be reused, such as global number of elements
@@ -687,6 +682,16 @@ cs_mesh_init_group_classes(cs_mesh_t  *mesh);
 void
 cs_mesh_init_selectors(void);
 
+/*----------------------------------------------------------------------------
+ * Update selector and associated structures.
+ *
+ * parameters:
+ *   mesh  <-> pointer to a mesh structure
+ *----------------------------------------------------------------------------*/
+
+void
+cs_mesh_update_selectors(cs_mesh_t  *mesh);
+
 /*----------------------------------------------------------------------------
  * Get global lists of periodic face couples.
  *
