@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -2811,7 +2811,7 @@ _create_gcell_faces_connect(cs_mesh_t                 *mesh,
 
   const cs_lnum_t n_i_faces = mesh->n_i_faces;
   const cs_lnum_t n_cells = mesh->n_cells;
-  const cs_lnum_2_t *face_cells = mesh->i_face_cells;
+  const cs_lnum_2_t *face_cells = (const cs_lnum_2_t *)(mesh->i_face_cells);
   const cs_lnum_t *fac_vtx_idx = mesh->i_face_vtx_idx;
   const cs_lnum_t *fac_vtx_lst = mesh->i_face_vtx_lst;
 
