@@ -9,7 +9,7 @@
   This file is part of the Code_Saturne Kernel, element of the
   Code_Saturne CFD tool.
 
-  Copyright (C) 1998-2015 EDF S.A., France
+  Copyright (C) 1998-2018 EDF S.A., France
 
   contact: saturne-support@edf.fr
 
@@ -87,14 +87,15 @@ typedef struct _cs_mesh_location_t cs_mesh_location_t;
  *   m           <-- pointer to associated mesh structure.
  *   location_id <-- id of associated location.
  *   n_elts      --> number of selected elements
- *   elt_list    --> list of selected elements (0 to n-1 numbering).
+ *   elt_ids     --> list of selected elements (0 to n-1 numbering).
  *----------------------------------------------------------------------------*/
 
 typedef void
-(cs_mesh_location_select_t) (const cs_mesh_t   *m,
+(cs_mesh_location_select_t) (void              *input,
+                             const cs_mesh_t   *m,
                              int                location_id,
                              cs_lnum_t         *n_elts,
-                             cs_lnum_t        **elt_list);
+                             cs_lnum_t        **elt_ids);
 
 /*=============================================================================
  * Global variables
@@ -193,8 +194,9 @@ cs_mesh_location_add(const char               *name,
                      cs_mesh_location_type_t   type,
                      const char               *criteria);
 
-/*----------------------------------------------------------------------------
- * Define a new mesh location.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define a new mesh location with an associated selection function.
  *
  * So as to define a subset of mesh entities of a given type, a pointer
  * to a selection function may be given.
@@ -202,19 +204,22 @@ cs_mesh_location_add(const char               *name,
  * This requires more programming but allows finer control than selection
  * criteria, as the function has access to the complete mesh structure.
  *
- * parameters:
- *   name <-- name of location to define
- *   type <-- type of location to define
- *   func <-- pointer to selection function for associated elements, or NULL
+ * \param[in]  name        name of location to define
+ * \param[in]  type        type of location to define
+ * \param[in]  func        pointer to selection function for associated
+ *                         elements, or NULL
+ * \param[in, out]  input  pointer to optional (untyped) value
+ *                         or structure.
  *
- * returns:
- *   id of newly created mesh location
- *----------------------------------------------------------------------------*/
+ * \return  id of newly defined created mesh location
+ */
+/*----------------------------------------------------------------------------*/
 
 int
 cs_mesh_location_add_by_func(const char                 *name,
                              cs_mesh_location_type_t     type,
-                             cs_mesh_location_select_t  *func);
+                             cs_mesh_location_select_t  *func,
+                             void                       *input);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -290,6 +295,9 @@ cs_mesh_location_get_n_elts(int id);
  * A list of elements is defined if the location is a subset of a main
  * location type.
  *
+ * \deprecated  Use \ref cs_mesh_location_get_elt_ids_try or
+ * \ref cs_mesh_location_get_elt_ids instead.
+ *
  * parameters:
  *   id <-- id of mesh location
  *
@@ -300,6 +308,94 @@ cs_mesh_location_get_n_elts(int id);
 const cs_lnum_t *
 cs_mesh_location_get_elt_list(int id);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Get a mesh location's element ids, if present.
+ *
+ * An array of element ids is returned if the location is a subset of a main
+ * main location type, and NULL is returned when the id array would map to
+ * the identity function (i.e. {0, 1, 2, ..., n_elts-1}).
+ *
+ * \param[in]  id  id of mesh location
+ *
+ * \return  pointer to elements array (0 to n-1 numbering).
+ */
+/*----------------------------------------------------------------------------*/
+
+const cs_lnum_t *
+cs_mesh_location_get_elt_ids_try(int id);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Get a mesh location's element ids.
+ *
+ * \This function may only be used with a given location if
+ * \ref cs_mesh_location_set_explicit_ids has been used to indicate
+ * explicit ids are needed for this location.
+ *
+ * \param[in]  id  id of mesh location
+ *
+ * \return  pointer to elements array (0 to n-1 numbering).
+ */
+/*----------------------------------------------------------------------------*/
+
+const cs_lnum_t *
+cs_mesh_location_get_elt_ids(int id);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Get a mesh location's selection criteria string
+ *
+ * \param[in]  id  id of mesh location
+ *
+ * \return  pointer to mesh location selection criteria, or NULL
+ */
+/*----------------------------------------------------------------------------*/
+
+const char *
+cs_mesh_location_get_selection_string(int  id);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Get a mesh location's selection function pointer
+ *
+ * \param[in]  id  id of mesh location
+ *
+ * \return  pointer to mesh location selection function pointer, or NULL
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_mesh_location_select_t *
+cs_mesh_location_get_selection_function(int  id);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Check if \ref cs_mesh_location_get_elt_ids always returns
+ *         explicit element ids for a given mesh location.
+ *
+ * \param[in]  id  id or type of mesh location
+ *
+ * \return true if explicit element ids are required, false otherwise
+ */
+/*----------------------------------------------------------------------------*/
+
+bool
+cs_mesh_location_get_explicit_ids(int id);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Set behavior of \ref cs_mesh_location_get_elt_ids for a given
+ *         mesh location and locations based on it.
+ *
+ * \param[in]  id                id or type of mesh location
+ * \param[in]  explicit_elt_ids  indicate if explicit element ids are required
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_mesh_location_set_explicit_ids(int   id,
+                                  bool  explicit_elt_ids);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
