@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -28,7 +28,6 @@
 !______________________________________________________________________________
 
 subroutine ppvarp
-!================
 
 !===============================================================================
 !  FONCTION  :
@@ -56,13 +55,13 @@ use ppthch
 use coincl
 use cpincl
 use ppincl
-use elincl
 use cs_coal_incl
 use cs_fuel_incl
 use ppcpfu
 use atincl
 use ihmpre
 use field
+use cs_c_bindings
 
 !===============================================================================
 
@@ -93,6 +92,16 @@ interface
 
 end interface
 
+interface
+
+  subroutine cs_field_pointer_map_groundwater()  &
+    bind(C, name='cs_field_pointer_map_groundwater')
+    use, intrinsic :: iso_c_binding
+    implicit none
+  end subroutine cs_field_pointer_map_groundwater
+
+end interface
+
 !===============================================================================
 
 ! Key ids for clipping
@@ -106,7 +115,6 @@ if (     ippmod(icod3p).ge.0   &
     .or. ippmod(icoebu).ge.0   &
     .or. ippmod(icolwc).ge.0) then
   call covarp
-  !==========
 endif
 
 ! Number of Diracs for LWC model
@@ -129,14 +137,12 @@ endif
 
 if (ippmod(iccoal).ge.0) then
   call cs_coal_varpos
-  !==================
 endif
 
 ! Pulverized coal combustion coupled with Lagrangian model
 
 if (ippmod(icpl3c).ge.0) then
   call cplvar
-  !==========
 endif
 
 ! 3. Compressible model
@@ -144,17 +150,14 @@ endif
 
 if (ippmod(icompf).ge.0) then
   call cfvarp
-  !==========
 endif
 
 ! 4. Electric arcs model
 !-----------------------
 
 if ( ippmod(ieljou).ge.1 .or.                                     &
-     ippmod(ielarc).ge.1 .or.                                     &
-     ippmod(ielion).ge.1       ) then
-  call elvarp
-  !==========
+     ippmod(ielarc).ge.1       ) then
+  call elvarp(ippmod(ieljou), ippmod(ielarc))
 endif
 
 ! 5. Fuel combustion model
@@ -162,7 +165,6 @@ endif
 
 if (ippmod(icfuel).ge.0) then
   call cs_fuel_varpos
-  !==================
 endif
 
 ! 6. Atmospheric model
@@ -170,15 +172,13 @@ endif
 
 if (ippmod(iatmos).ge.0) then
   call atvarp
-  !==========
 endif
 
 ! 7. Cooling towers model
 !------------------------
 
 if (ippmod(iaeros).ge.0) then
-  call ctvarp
-  !==========
+  call ctvarp(ippmod(iatmos))
 endif
 
 ! 8. Gas mixtures modelling
@@ -203,57 +203,64 @@ if (ippmod(igmix).ge.0) then
   if (ippmod(igmix).lt.5) then
     call add_model_scalar_field('y_o2', 'Y_O2', iscasp(1))
     f_id = ivarfl(isca(iscasp(1)))
+    call gas_mix_add_species(f_id)
     call field_set_key_int(f_id, kivisl, 0)
     call field_set_key_double(f_id, kscmin, scminp)
     call field_set_key_double(f_id, kscmax, scmaxp)
 
     call add_model_scalar_field('y_n2', 'Y_N2', iscasp(2))
     f_id = ivarfl(isca(iscasp(2)))
+    call gas_mix_add_species(f_id)
     call field_set_key_int(f_id, kivisl, 0)
     call field_set_key_double(f_id, kscmin, scminp)
     call field_set_key_double(f_id, kscmax, scmaxp)
 
-    nscasp = 2
-
     if (ippmod(igmix).eq.3) then
       call add_model_scalar_field('y_he', 'Y_He', iscasp(3))
       f_id = ivarfl(isca(iscasp(3)))
+      call gas_mix_add_species(f_id)
       call field_set_key_int(f_id, kivisl, 0)
       call field_set_key_double(f_id, kscmin, scminp)
       call field_set_key_double(f_id, kscmax, scmaxp)
 
-      nscasp = 3
-
     elseif (ippmod(igmix).eq.4) then
       call add_model_scalar_field('y_h2', 'Y_H2', iscasp(3))
       f_id = ivarfl(isca(iscasp(3)))
+      call gas_mix_add_species(f_id)
       call field_set_key_int(f_id, kivisl, 0)
       call field_set_key_double(f_id, kscmin, scminp)
       call field_set_key_double(f_id, kscmax, scmaxp)
 
-      nscasp = 3
     endif
   else ! ippmod(igmix).eq.5
 
     call add_model_scalar_field('y_n2', 'Y_N2', iscasp(1))
     f_id = ivarfl(isca(iscasp(1)))
+    call gas_mix_add_species(f_id)
     call field_set_key_int(f_id, kivisl, 0)
     call field_set_key_double(f_id, kscmin, scminp)
     call field_set_key_double(f_id, kscmax, scmaxp)
 
     call add_model_scalar_field('y_he', 'Y_He', iscasp(2))
     f_id = ivarfl(isca(iscasp(2)))
+    call gas_mix_add_species(f_id)
     call field_set_key_int(f_id, kivisl, 0)
     call field_set_key_double(f_id, kscmin, scminp)
     call field_set_key_double(f_id, kscmax, scmaxp)
 
-    nscasp = 2
-
   endif
 
   ! MAP to C API
   call cs_field_pointer_map_gas_mix
 endif
 
+! 9. Groundwater flows
+!---------------------
+
+if (ippmod(idarcy).ge.0) then
+  ! MAP to C API
+  call cs_field_pointer_map_groundwater
+endif
+
 return
 end subroutine
