@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -56,7 +56,6 @@
 !>                               (see \ref cs_user_mass_source_terms)
 !> \param[in]     viscf         visc*surface/dist at internal faces
 !> \param[in]     viscb         visc*surface/dist at edge faces
-!> \param[in]     tslagr        coupling term for lagrangian
 !> \param[in]     smbr          working array
 !> \param[in]     rovsdt        working array
 !_______________________________________________________________________________
@@ -68,7 +67,6 @@ subroutine reseps &
    gradv  , produc , gradro ,                                     &
    ckupdc , smacel ,                                              &
    viscf  , viscb  ,                                              &
-   tslagr ,                                                       &
    smbr   , rovsdt )
 
 !===============================================================================
@@ -109,11 +107,11 @@ double precision produc(6,ncelet), gradv(3, 3, ncelet)
 double precision gradro(3,ncelet)
 double precision ckupdc(ncepdp,6), smacel(ncesmp,nvar)
 double precision viscf(nfac), viscb(nfabor)
-double precision tslagr(ncelet,*)
 double precision smbr(ncelet), rovsdt(ncelet)
 
 ! Local variables
 
+integer          init
 integer          ivar
 integer          iel
 integer          iiun
@@ -147,11 +145,13 @@ double precision, dimension(:,:), pointer :: visten
 double precision, dimension(:), pointer :: cvar_ep, cvara_ep, cvar_al
 double precision, dimension(:), pointer :: cvara_r11, cvara_r22, cvara_r33
 double precision, dimension(:), pointer :: cvara_r12, cvara_r13, cvara_r23
-double precision, dimension(:,:), pointer :: cvara_rij
+double precision, dimension(:,:), pointer :: cvara_rij, lagr_st_rij
 double precision, dimension(:), pointer :: viscl, visct, c_st_prv
 
 character(len=80) :: label
 
+type(var_cal_opt) :: vcopt
+
 !===============================================================================
 
 !===============================================================================
@@ -168,14 +168,16 @@ allocate(viscce(6,ncelet))
 allocate(weighf(2,nfac))
 allocate(weighb(nfabor))
 
-if (iwarni(ivar).ge.1) then
+call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+
+if (vcopt%iwarni.ge.1) then
   call field_get_label(ivarfl(ivar), label)
   write(nfecra,1000) label
 endif
 
 call field_get_val_s(icrom, crom)
-call field_get_val_s(iprpfl(iviscl), viscl)
-call field_get_val_s(iprpfl(ivisct), visct)
+call field_get_val_s(iviscl, viscl)
+call field_get_val_s(ivisct, visct)
 
 call field_get_val_s(ivarfl(iep), cvar_ep)
 call field_get_val_prev_s(ivarfl(iep), cvara_ep)
@@ -211,7 +213,7 @@ endif
 
 ! S as Source, V as Variable
 thets  = thetst
-thetv  = thetav(ivar )
+thetv  = vcopt%thetav
 
 call field_get_key_int(ivarfl(ivar), kstprv, st_prv_id)
 if (st_prv_id.ge.0) then
@@ -272,16 +274,18 @@ endif
 !     Second order is not taken into account
 if (iilagr.eq.2 .and. ltsdyn.eq.1) then
 
+  call field_get_val_v_by_name('rij_st_lagr', lagr_st_rij)
+
   do iel = 1, ncel
     ! Source terms with eps
-    tseps = -0.5d0 * ( tslagr(iel,itsr11)                        &
-                     + tslagr(iel,itsr22)                        &
-                     + tslagr(iel,itsr33) )
+    tseps = -0.5d0 * ( lagr_st_rij(1,iel)                        &
+                     + lagr_st_rij(2,iel)                        &
+                     + lagr_st_rij(3,iel))
     ! quotient k/eps
     if (irijco.eq.1) then
-      kseps = 0.5d0 * ( cvara_rij(1,iel)                           &
-                      + cvara_rij(2,iel)                           &
-                      + cvara_rij(3,iel) )                         &
+      kseps = 0.5d0 * ( cvara_rij(1,iel)                         &
+                      + cvara_rij(2,iel)                         &
+                      + cvara_rij(3,iel) )                       &
                       / cvara_ep(iel)
     else
       kseps = 0.5d0 * ( cvara_r11(iel)                           &
@@ -331,7 +335,7 @@ endif
 
 do iel = 1, ncel
   rovsdt(iel) = rovsdt(iel)                                          &
-              + istat(ivar)*(crom(iel)/dt(iel))*volume(iel)
+              + vcopt%istat*(crom(iel)/dt(iel))*cell_f_vol(iel)
 enddo
 
 !===============================================================================
@@ -401,16 +405,16 @@ if (iturb.eq.32) then
 
     ! Production (explicit)
     ! Compute of C_eps_1'
-    w1(iel) = cromo(iel)*volume(iel)*                                 &
+    w1(iel) = cromo(iel)*cell_f_vol(iel)*                                 &
               ce1*(1.d0+xa1*(1.d0-alpha3)*prdeps)*trprod/xttdrb
 
 
     ! Dissipation (implicit)
-    smbr(iel) = smbr(iel) - crom(iel)*volume(iel)*                    &
+    smbr(iel) = smbr(iel) - crom(iel)*cell_f_vol(iel)*                    &
                              ceps2*cvara_ep(iel)/xttdrb
 
     rovsdt(iel) = rovsdt(iel)                                         &
-                + ceps2*crom(iel)*volume(iel)*thetap/xttdrb
+                + ceps2*crom(iel)*cell_f_vol(iel)*thetap/xttdrb
   enddo
 
 ! SSG and LRR
@@ -425,14 +429,16 @@ else
       trrij  = 0.5d0 * (cvara_r11(iel) + cvara_r22(iel) + cvara_r33(iel))
     endif
     xttke  = trrij/cvara_ep(iel)
-    ! Production (explicit)
-    w1(iel) = cromo(iel)*volume(iel)*ce1/xttke*trprod
+    ! Production (explicit, a part might be implicit)
+    rovsdt(iel) = rovsdt(iel)                                        &
+                + max(- cromo(iel)*cell_f_vol(iel)*ce1*trprod/trrij, 0.d0)
+    w1(iel) = cromo(iel)*cell_f_vol(iel)*ce1/xttke*trprod
 
     ! Dissipation (implicit)
     smbr(iel) = smbr(iel)                                            &
-              - crom(iel)*volume(iel)*ceps2*cvara_ep(iel)**2/trrij
+              - crom(iel)*cell_f_vol(iel)*ceps2*cvara_ep(iel)**2/trrij
     rovsdt(iel) = rovsdt(iel)                                        &
-                + ceps2*crom(iel)*volume(iel)/xttke*thetap
+                + ceps2*crom(iel)*cell_f_vol(iel)/xttke*thetap
   enddo
 
 endif
@@ -452,6 +458,7 @@ endif
 ! 7. Buoyancy term
 !===============================================================================
 
+!FIXME use beta ... WARNING
 if (igrari.eq.1) then
 
   ! Allocate a work array
@@ -461,12 +468,12 @@ if (igrari.eq.1) then
     w7(iel) = 0.d0
   enddo
 
+  ! Note that w7 is positive.
   if(irijco.eq.1) then
     call rijtheps(nscal, gradro, w7)
   else
     call rijthe(nscal, ivar, gradro, w7)
   endif
-  !==========
 
   ! Extrapolation of source terms (2nd order in time)
   if (st_prv_id.ge.0) then
@@ -489,7 +496,7 @@ endif
 !===============================================================================
 
 ! Symmetric tensor diffusivity (GGDH)
-if (idften(ivar).eq.6) then
+if (iand(vcopt%idften, ANISOTROPIC_DIFFUSION).ne.0) then
 
   call field_get_val_v(ivsten, visten)
 
@@ -502,7 +509,7 @@ if (idften(ivar).eq.6) then
     viscce(6,iel) = visten(6,iel)/sigmae
   enddo
 
-  iwarnp = iwarni(ivar)
+  iwarnp = vcopt%iwarni
 
   call vitens &
  ( viscce , iwarnp ,             &
@@ -513,7 +520,7 @@ if (idften(ivar).eq.6) then
 else
 
   do iel = 1, ncel
-    w1(iel) = viscl(iel) + idifft(ivar)*visct(iel)/sigmae
+    w1(iel) = viscl(iel) + vcopt%idifft*visct(iel)/sigmae
   enddo
 
   call viscfa                    &
@@ -534,32 +541,33 @@ if (st_prv_id.ge.0) then
   enddo
 endif
 
-iconvp = iconv (ivar)
-idiffp = idiff (ivar)
+iconvp = vcopt%iconv
+idiffp = vcopt%idiff
 ndircp = ndircl(ivar)
-nswrsp = nswrsm(ivar)
-nswrgp = nswrgr(ivar)
-imligp = imligr(ivar)
-ircflp = ircflu(ivar)
-ischcp = ischcv(ivar)
-isstpp = isstpc(ivar)
+nswrsp = vcopt%nswrsm
+nswrgp = vcopt%nswrgr
+imligp = vcopt%imligr
+ircflp = vcopt%ircflu
+ischcp = vcopt%ischcv
+isstpp = vcopt%isstpc
 iescap = 0
 imucpp = 0
-idftnp = idften(ivar)
-iswdyp = iswdyn(ivar)
-iwarnp = iwarni(ivar)
-blencp = blencv(ivar)
-epsilp = epsilo(ivar)
-epsrsp = epsrsm(ivar)
-epsrgp = epsrgr(ivar)
-climgp = climgr(ivar)
-extrap = extrag(ivar)
-relaxp = relaxv(ivar)
+idftnp = vcopt%idften
+iswdyp = vcopt%iswdyn
+iwarnp = vcopt%iwarni
+blencp = vcopt%blencv
+epsilp = vcopt%epsilo
+epsrsp = vcopt%epsrsm
+epsrgp = vcopt%epsrgr
+climgp = vcopt%climgr
+extrap = vcopt%extrag
+relaxp = vcopt%relaxv
 ! all boundary convective flux with upwind
 icvflb = 0
+init   = 1
 
 call codits &
- ( idtvar , ivar   , iconvp , idiffp , ndircp ,                   &
+ ( idtvar , init   , ivarfl(ivar)    , iconvp , idiffp , ndircp , &
    imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
    iwarnp ,                                                       &
