@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -59,10 +59,10 @@
 !>                              (without rho volume) only for iturb=30
 !> \param[in]     ckupdc        work array for the head loss
 !> \param[in]     smacel        value associated to each variable in the mass
-!>                               source terms or mass rate (see \ref cs_user_mass_source_terms)
+!>                              source terms or mass rate (see
+!>                              \ref cs_user_mass_source_terms)
 !> \param[in]     viscf         visc*surface/dist at internal faces
 !> \param[in]     viscb         visc*surface/dist at edge faces
-!> \param[in]     tslage        explicit source terms for the Lagrangian module
 !> \param[in]     tslagi        implicit source terms for the Lagrangian module
 !> \param[in]     smbr          working array
 !> \param[in]     rovsdt        working array
@@ -76,7 +76,7 @@ subroutine resrij &
    produc , gradro ,                                              &
    ckupdc , smacel ,                                              &
    viscf  , viscb  ,                                              &
-   tslage , tslagi ,                                              &
+   tslagi ,                                                       &
    smbr   , rovsdt )
 
 !===============================================================================
@@ -119,13 +119,13 @@ double precision produc(6,ncelet)
 double precision gradro(3,ncelet)
 double precision ckupdc(ncepdp,6), smacel(ncesmp,nvar)
 double precision viscf(nfac), viscb(nfabor)
-double precision tslage(ncelet),tslagi(ncelet)
+double precision tslagi(ncelet)
 double precision smbr(ncelet), rovsdt(ncelet)
 
 ! Local variables
 
 integer          iel
-integer          ii    , jj    , kk    , iiun
+integer          ii    , jj    , kk    , iiun, comp_id
 integer          iflmas, iflmab
 integer          nswrgp, imligp, iwarnp
 integer          iconvp, idiffp, ndircp
@@ -135,6 +135,7 @@ integer          isoluc
 integer          imucpp, idftnp, iswdyp
 integer          ivar_r(3,3)
 integer          icvflb
+integer          init
 integer          ivoid(1)
 
 double precision blencp, epsilp, epsrgp, climgp, extrap, relaxp
@@ -157,13 +158,15 @@ double precision, allocatable, dimension(:) :: weighb
 double precision, dimension(:), pointer :: imasfl, bmasfl
 double precision, dimension(:), pointer :: crom, cromo
 double precision, dimension(:), pointer :: coefap, coefbp, cofafp, cofbfp
-double precision, dimension(:,:), pointer :: visten
+double precision, dimension(:,:), pointer :: visten, lagr_st_rij
 double precision, dimension(:), pointer :: cvara_ep
 double precision, dimension(:), pointer :: cvara_r11, cvara_r22, cvara_r33
 double precision, dimension(:), pointer :: cvar_var, cvara_var
 double precision, dimension(:), pointer :: cvara_rik, cvara_rjk
 double precision, dimension(:), pointer :: viscl, c_st_prv
 
+type(var_cal_opt) :: vcopt
+
 !===============================================================================
 
 !===============================================================================
@@ -178,13 +181,15 @@ allocate(viscce(6,ncelet))
 allocate(weighf(2,nfac))
 allocate(weighb(nfabor))
 
-if (iwarni(ivar).ge.1) then
+call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+
+if (vcopt%iwarni.ge.1) then
   call field_get_label(ivarfl(ivar), label)
   write(nfecra,1000) label
 endif
 
 call field_get_val_s(icrom, crom)
-call field_get_val_s(iprpfl(iviscl), viscl)
+call field_get_val_s(iviscl, viscl)
 call field_get_key_int(ivarfl(iu), kimasf, iflmas)
 call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
 call field_get_val_s(iflmas, imasfl)
@@ -213,7 +218,7 @@ d2s3 = 2.d0/3.d0
 
 !     S as Source, V as Variable
 thets  = thetst
-thetv  = thetav(ivar)
+thetv  = vcopt%thetav
 
 call field_get_key_int(ivarfl(ivar), kstprv, st_prv_id)
 if (st_prv_id.ge.0) then
@@ -251,7 +256,6 @@ endif
 !===============================================================================
 
 call cs_user_turbulence_source_terms &
-!===================================
  ( nvar   , nscal  , ncepdp , ncesmp ,                            &
    ivarfl(ivar)    ,                                              &
    icepdc , icetsm , itypsm ,                                     &
@@ -285,8 +289,15 @@ endif
 
 !     Second order is not taken into account
 if (iilagr.eq.2 .and. ltsdyn.eq.1) then
+  call field_get_val_v_by_name('rij_st_lagr', lagr_st_rij)
+  comp_id = isou
+  if (isou .eq. 5) then
+    comp_id = 6
+  else if (isou .eq.6) then
+    comp_id = 5
+  endif
   do iel = 1,ncel
-    smbr(iel)   = smbr(iel)   + tslage(iel)
+    smbr(iel)   = smbr(iel)   + lagr_st_rij(comp_id,iel)
     rovsdt(iel) = rovsdt(iel) + max(-tslagi(iel),zero)
   enddo
 endif
@@ -327,7 +338,7 @@ endif
 
 do iel=1,ncel
   rovsdt(iel) = rovsdt(iel)                                          &
-              + istat(ivar)*(crom(iel)/dt(iel))*cell_f_vol(iel)
+              + vcopt%istat*(crom(iel)/dt(iel))*cell_f_vol(iel)
 enddo
 
 
@@ -578,7 +589,7 @@ endif
 !===============================================================================
 
 ! Symmetric tensor diffusivity (GGDH)
-if (idften(ivar).eq.6) then
+if (iand(vcopt%idften, ANISOTROPIC_DIFFUSION).ne.0) then
 
   call field_get_val_v(ivsten, visten)
 
@@ -591,7 +602,7 @@ if (idften(ivar).eq.6) then
     viscce(6,iel) = visten(6,iel)
   enddo
 
-  iwarnp = iwarni(ivar)
+  iwarnp = vcopt%iwarni
 
   call vitens &
  ( viscce , iwarnp ,             &
@@ -604,7 +615,7 @@ else
   do iel = 1, ncel
     trrij = 0.5d0 * (cvara_r11(iel) + cvara_r22(iel) + cvara_r33(iel))
     rctse = crom(iel) * csrij * trrij**2 / cvara_ep(iel)
-    w1(iel) = viscl(iel) + idifft(ivar)*rctse
+    w1(iel) = viscl(iel) + vcopt%idifft*rctse
   enddo
 
   call viscfa                    &
@@ -625,32 +636,33 @@ if (st_prv_id.ge.0) then
   enddo
 endif
 
-iconvp = iconv (ivar)
-idiffp = idiff (ivar)
+iconvp = vcopt%iconv
+idiffp = vcopt%idiff
 ndircp = ndircl(ivar)
-nswrsp = nswrsm(ivar)
-nswrgp = nswrgr(ivar)
-imligp = imligr(ivar)
-ircflp = ircflu(ivar)
-ischcp = ischcv(ivar)
-isstpp = isstpc(ivar)
+nswrsp = vcopt%nswrsm
+nswrgp = vcopt%nswrgr
+imligp = vcopt%imligr
+ircflp = vcopt%ircflu
+ischcp = vcopt%ischcv
+isstpp = vcopt%isstpc
 iescap = 0
 imucpp = 0
-idftnp = idften(ivar)
-iswdyp = iswdyn(ivar)
-iwarnp = iwarni(ivar)
-blencp = blencv(ivar)
-epsilp = epsilo(ivar)
-epsrsp = epsrsm(ivar)
-epsrgp = epsrgr(ivar)
-climgp = climgr(ivar)
-extrap = extrag(ivar)
-relaxp = relaxv(ivar)
+idftnp = vcopt%idften
+iswdyp = vcopt%iswdyn
+iwarnp = vcopt%iwarni
+blencp = vcopt%blencv
+epsilp = vcopt%epsilo
+epsrsp = vcopt%epsrsm
+epsrgp = vcopt%epsrgr
+climgp = vcopt%climgr
+extrap = vcopt%extrag
+relaxp = vcopt%relaxv
 ! all boundary convective flux with upwind
 icvflb = 0
+init   = 1
 
 call codits &
- ( idtvar , ivar   , iconvp , idiffp , ndircp ,                   &
+ ( idtvar , init   , ivarfl(ivar)    , iconvp , idiffp , ndircp , &
    imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
    iwarnp ,                                                       &
