@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -47,7 +47,6 @@
 !> \param[in]     nscal         total number of scalars
 !> \param[in]     ncepdp        number of cells with head loss
 !> \param[in]     ncesmp        number of cells with mass source term
-!> \param[in]     ivar          variable number
 !> \param[in]     icepdc        index of cells with head loss
 !> \param[in]     icetsm        index of cells with mass source term
 !> \param[in]     itypsm        type of mass source term for each variable
@@ -61,7 +60,6 @@
 !>                               source terms or mass rate (see \ref cs_user_mass_source_terms)
 !> \param[in]     viscf         visc*surface/dist at internal faces
 !> \param[in]     viscb         visc*surface/dist at edge faces
-!> \param[in]     tslage        explicit source terms for the Lagrangian module
 !> \param[in]     tslagi        implicit source terms for the Lagrangian module
 !> \param[in]     smbr          working array
 !> \param[in]     rovsdt        working array
@@ -69,13 +67,12 @@
 
 subroutine resrij2 &
  ( nvar   , nscal  , ncepdp , ncesmp ,                            &
-   ivar   ,                                                       &
    icepdc , icetsm , itypsm ,                                     &
    dt     ,                                                       &
    produc , gradro ,                                              &
    ckupdc , smacel ,                                              &
    viscf  , viscb  ,                                              &
-   tslage , tslagi ,                                              &
+   tslagi ,                                                       &
    smbr   , rovsdt )
 
 !===============================================================================
@@ -108,7 +105,6 @@ implicit none
 
 integer          nvar   , nscal
 integer          ncepdp , ncesmp
-integer          ivar
 
 integer          icepdc(ncepdp)
 integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
@@ -118,7 +114,7 @@ double precision produc(6,ncelet)
 double precision gradro(3,ncelet)
 double precision ckupdc(ncepdp,6), smacel(ncesmp,nvar)
 double precision viscf(nfac), viscb(nfabor)
-double precision tslage(6,ncelet),tslagi(6,ncelet)
+double precision tslagi(ncelet)
 double precision smbr(6,ncelet), rovsdt(6,6,ncelet)
 
 ! Local variables
@@ -161,9 +157,11 @@ double precision, dimension(:,:), pointer :: visten
 double precision, dimension(:), pointer :: cvara_ep
 double precision, dimension(:,:), pointer :: cvar_var, cvara_var
 double precision, allocatable, dimension(:,:) :: cvara_r
-double precision, dimension(:,:), pointer :: c_st_prv
+double precision, dimension(:,:), pointer :: c_st_prv, lagr_st_rij
 double precision, dimension(:), pointer :: viscl
 
+type(var_cal_opt) :: vcopt_rij
+
 !===============================================================================
 
 !===============================================================================
@@ -179,13 +177,15 @@ allocate(viscce(6,ncelet))
 allocate(weighf(2,nfac))
 allocate(weighb(nfabor))
 
-if (iwarni(ivar).ge.1) then
-  call field_get_label(ivarfl(ivar), label)
+call field_get_key_struct_var_cal_opt(ivarfl(irij), vcopt_rij)
+
+if (vcopt_rij%iwarni.ge.1) then
+  call field_get_label(ivarfl(irij), label)
   write(nfecra,1000) label
 endif
 
 call field_get_val_s(icrom, crom)
-call field_get_val_s(iprpfl(iviscl), viscl)
+call field_get_val_s(iviscl, viscl)
 call field_get_key_int(ivarfl(iu), kimasf, iflmas)
 call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
 call field_get_val_s(iflmas, imasfl)
@@ -194,14 +194,14 @@ call field_get_val_s(iflmab, bmasfl)
 call field_get_val_prev_s(ivarfl(iep), cvara_ep)
 
 
-call field_get_val_v(ivarfl(ivar), cvar_var)
-call field_get_val_prev_v(ivarfl(ivar), cvara_var)
-call field_get_dim(ivarfl(ivar),dimrij)! dimension of Rij
+call field_get_val_v(ivarfl(irij), cvar_var)
+call field_get_val_prev_v(ivarfl(irij), cvara_var)
+call field_get_dim(ivarfl(irij),dimrij)! dimension of Rij
 
-call field_get_coefa_v(ivarfl(ivar), coefap)
-call field_get_coefb_v(ivarfl(ivar), coefbp)
-call field_get_coefaf_v(ivarfl(ivar), cofafp)
-call field_get_coefbf_v(ivarfl(ivar), cofbfp)
+call field_get_coefa_v(ivarfl(irij), coefap)
+call field_get_coefb_v(ivarfl(irij), coefbp)
+call field_get_coefaf_v(ivarfl(irij), cofafp)
+call field_get_coefbf_v(ivarfl(irij), cofbfp)
 
 do isou = 1, 6
   deltij(isou) = 1.0d0
@@ -214,9 +214,9 @@ d2s3 = 2.d0/3.d0
 
 !     S as Source, V as Variable
 thets  = thetst
-thetv  = thetav(ivar)
+thetv  = vcopt_rij%thetav
 
-call field_get_key_int(ivarfl(ivar), kstprv, st_prv_id)
+call field_get_key_int(ivarfl(irij), kstprv, st_prv_id)
 if (st_prv_id.ge.0) then
   call field_get_val_v(st_prv_id, c_st_prv)
 else
@@ -258,17 +258,16 @@ endif
 !===============================================================================
 
 call cs_user_turbulence_source_terms2 &
-!===================================
  ( nvar   , nscal  , ncepdp , ncesmp ,                            &
-   ivarfl(ivar)    ,                                              &
+   ivarfl(irij)    ,                                              &
    icepdc , icetsm , itypsm ,                                     &
    ckupdc , smacel ,                                              &
    smbr   , rovsdt )
 
   !     If we extrapolate the source terms
-do isou = 1, dimrij
-  if (st_prv_id.ge.0) then
-    do iel = 1, ncel
+if (st_prv_id.ge.0) then
+  do iel = 1, ncel
+    do isou = 1, dimrij
       !       Save for exchange
       tuexpr = c_st_prv(isou,iel)
       !       For continuation and the next time step
@@ -280,44 +279,46 @@ do isou = 1, dimrij
       !       Diagonal
       rovsdt(isou,isou,iel) = - thetv*rovsdt(isou,isou,iel)
     enddo
-  else
-    do iel = 1, ncel
+  enddo
+else
+  do iel = 1, ncel
+    do isou = 1, dimrij
       smbr(isou,iel)   = rovsdt(isou,isou,iel)*cvara_var(isou,iel) + smbr(isou,iel)
       rovsdt(isou,isou,iel) = max(-rovsdt(isou,isou,iel),zero)
     enddo
-  endif
-enddo
+  enddo
+endif
 
 !===============================================================================
 ! 3. Lagrangian source terms
 !===============================================================================
 
 !     2nd order is not taken into account
-do isou = 1, dimrij
-        if (iilagr.eq.2 .and. ltsdyn.eq.1) then
-    do iel = 1,ncel
-      smbr(isou, iel)   = smbr(isou, iel)   + tslage(isou,iel)
-      rovsdt(isou,isou,iel) = rovsdt(isou,isou, iel) + max(-tslagi(isou, iel),zero)
+if (iilagr.eq.2 .and. ltsdyn.eq.1) then
+  call field_get_val_v_by_name('rij_st_lagr', lagr_st_rij)
+  do iel = 1,ncel
+    do isou = 1, dimrij
+      smbr(isou, iel)   = smbr(isou, iel) + lagr_st_rij(isou,iel)
+      rovsdt(isou,isou,iel) = rovsdt(isou,isou, iel) + max(-tslagi(iel),zero)
     enddo
-  endif
-enddo
+  enddo
+endif
 
 !===============================================================================
 ! 4. Mass source term
 !===============================================================================
 
-do isou = 1, dimrij
-  if (ncesmp.gt.0) then
+if (ncesmp.gt.0) then
 
+  do isou = 1, dimrij
     !       Integer equal to 1 (for navsto: nb of sur-iter)
     iiun = 1
 
     ! We increment smbr with -Gamma.var_prev and rovsdr with Gamma
     call catsmt &
-    !==========
    ( ncelet , ncel   , ncesmp , iiun     , isto2t ,                   &
-     icetsm , itypsm(:,ivar + isou - 1)  ,                            &
-     cell_f_vol , cvara_var  , smacel(:,ivar+isou-1)  , smacel(:,ipr) ,   &
+     icetsm , itypsm(:,irij + isou - 1)  ,                            &
+     cell_f_vol , cvara_var  , smacel(:,irij+isou-1)  , smacel(:,ipr) ,   &
      smbr   ,  rovsdt    , w1 )
 
     ! If we extrapolate the source terms we put Gamma Pinj in the previous st
@@ -332,8 +333,8 @@ do isou = 1, dimrij
       enddo
     endif
 
-  endif
-enddo
+  enddo
+endif
 
 !===============================================================================
 ! 5. Unsteady term
@@ -341,10 +342,10 @@ enddo
 
 ! ---> Added in the matrix diagonal
 
-do isou = 1, dimrij
-  do iel=1,ncel
-    rovsdt(isou,isou,iel) = rovsdt(isou, isou,iel)                                       &
-              + istat(ivar+isou-1)*(crom(iel)/dt(iel))*cell_f_vol(iel)
+do iel=1,ncel
+  do isou = 1, dimrij
+    rovsdt(isou,isou,iel) = rovsdt(isou,isou,iel)                              &
+              + vcopt_rij%istat*(crom(iel)/dt(iel))*cell_f_vol(iel)
   enddo
 enddo
 
@@ -595,20 +596,21 @@ if (igrari.eq.1) then
 
   call rijthe2(nscal, gradro, w7)
 
-  do isou = 1, dimrij
-    ! If we extrapolate the source terms: previous ST
-    if (st_prv_id.ge.0) then
-      do iel = 1, ncel
+  ! If we extrapolate the source terms: previous ST
+  if (st_prv_id.ge.0) then
+    do iel = 1, ncel
+      do isou = 1, dimrij
         c_st_prv(isou,iel) = c_st_prv(isou,iel) + w7(isou,iel)
       enddo
+    enddo
     ! Otherwise smbr
-    else
-      do iel = 1, ncel
+  else
+    do iel = 1, ncel
+      do isou = 1, dimrij
         smbr(isou,iel) = smbr(isou,iel) + w7(isou,iel)
       enddo
-    endif
-  enddo
-
+    enddo
+  endif
 
 endif
 
@@ -617,7 +619,7 @@ endif
 !===============================================================================
 
 ! Symmetric tensor diffusivity (GGDH)
-if (idften(ivar).eq.6) then
+if (iand(vcopt_rij%idften, ANISOTROPIC_RIGHT_DIFFUSION).ne.0) then
 
   call field_get_val_v(ivsten, visten)
 
@@ -630,7 +632,7 @@ if (idften(ivar).eq.6) then
     viscce(6,iel) = visten(6,iel)
   enddo
 
-  iwarnp = iwarni(ivar)
+  iwarnp = vcopt_rij%iwarni
 
   call vitens &
  ( viscce , iwarnp ,             &
@@ -643,13 +645,10 @@ else
   do iel = 1, ncel
     trrij = 0.5d0 * (cvara_var(1,iel) + cvara_var(2,iel) + cvara_var(3,iel))
     rctse = crom(iel) * csrij * trrij**2 / cvara_ep(iel)
-    w1(iel) = viscl(iel) + idifft(ivar)*rctse
+    w1(iel) = viscl(iel) + vcopt_rij%idifft*rctse
   enddo
 
-  call viscfa                    &
- ( imvisf ,                      &
-   w1     ,                      &
-   viscf  , viscb  )
+  call viscfa(imvisf, w1, viscf, viscb)
 
 endif
 
@@ -666,31 +665,30 @@ if (st_prv_id.ge.0) then
   enddo
 endif
 
-iconvp = iconv (ivar)
-idiffp = idiff (ivar)
-ndircp = ndircl(ivar)
-nswrsp = nswrsm(ivar)
-nswrgp = nswrgr(ivar)
-imligp = imligr(ivar)
-ircflp = ircflu(ivar)
-ischcp = ischcv(ivar)
-isstpp = isstpc(ivar)
-idftnp = idften(ivar)
-iswdyp = iswdyn(ivar)
-iwarnp = iwarni(ivar)
-blencp = blencv(ivar)
-epsilp = epsilo(ivar)
-epsrsp = epsrsm(ivar)
-epsrgp = epsrgr(ivar)
-climgp = climgr(ivar)
-extrap = extrag(ivar)
-relaxp = relaxv(ivar)
+iconvp = vcopt_rij%iconv
+idiffp = vcopt_rij%idiff
+ndircp = ndircl(irij)
+nswrsp = vcopt_rij%nswrsm
+nswrgp = vcopt_rij%nswrgr
+imligp = vcopt_rij%imligr
+ircflp = vcopt_rij%ircflu
+ischcp = vcopt_rij%ischcv
+isstpp = vcopt_rij%isstpc
+idftnp = vcopt_rij%idften
+iswdyp = vcopt_rij%iswdyn
+iwarnp = vcopt_rij%iwarni
+blencp = vcopt_rij%blencv
+epsilp = vcopt_rij%epsilo
+epsrsp = vcopt_rij%epsrsm
+epsrgp = vcopt_rij%epsrgr
+climgp = vcopt_rij%climgr
+extrap = vcopt_rij%extrag
+relaxp = vcopt_rij%relaxv
 ! all boundary convective flux with upwind
 icvflb = 0
 
 call coditts &
-!==========
- ( idtvar , ivar   , iconvp , idiffp , ndircp ,                   &
+ ( idtvar , ivarfl(irij)    , iconvp , idiffp , ndircp ,          &
    imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , idftnp , iswdyp ,                            &
    iwarnp ,                                                       &
