@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -63,7 +63,6 @@
 !>                               (see \ref cs_user_mass_source_terms)
 !> \param[in]     viscf         visc*surface/dist at internal faces
 !> \param[in]     viscb         visc*surface/dist at edge faces
-!> \param[in]     tslage        explicit source terms for the Lagrangian module
 !> \param[in]     tslagi        implicit source terms for the Lagrangian module
 !> \param[in]     smbr          working array
 !> \param[in]     rovsdt        working array
@@ -71,13 +70,13 @@
 
 subroutine resssg2 &
  ( nvar   , nscal  , ncepdp , ncesmp ,                            &
-   ivar   ,                                               &
+   ivar   ,                                                       &
    icepdc , icetsm , itypsm ,                                     &
    dt     ,                                                       &
    gradv  , gradro ,                                              &
    ckupdc , smacel ,                                              &
    viscf  , viscb  ,                                              &
-   tslage , tslagi ,                                              &
+   tslagi ,                                                       &
    smbr   , rovsdt )
 
 !===============================================================================
@@ -121,7 +120,7 @@ double precision gradv(3, 3, ncelet)
 double precision gradro(3,ncelet)
 double precision ckupdc(ncepdp,6), smacel(ncesmp,nvar)
 double precision viscf(nfac), viscb(nfabor)
-double precision tslage(6,ncelet),tslagi(6,ncelet)
+double precision tslagi(ncelet)
 double precision smbr(6,ncelet)
 double precision rovsdt(6,6,ncelet)
 
@@ -139,14 +138,17 @@ integer          idftnp, iswdyp
 integer          icvflb
 integer          ivoid(1)
 integer          dimrij
+integer          t2v(3,3)
+integer          iv2t(6), jv2t(6)
 
 double precision blencp, epsilp, epsrgp, climgp, extrap, relaxp
 double precision epsrsp
 double precision trprod, trrij
-double precision deltij(6)
+double precision deltij(6), dij(3,3)
 double precision tuexpr, thets , thetv , thetp1
 double precision aiksjk, aikrjk, aii ,aklskl, aikakj
 double precision xaniso(3,3), xstrai(3,3), xrotac(3,3), xprod(3,3), matrot(3,3)
+double precision sym_strain(6)
 double precision xrij(3,3), xnal(3), xnoral, xnnd
 double precision d1s2, d1s3, d2s3
 double precision alpha3
@@ -154,6 +156,16 @@ double precision pij, phiij1, phiij2, epsij
 double precision phiijw, epsijw
 double precision ccorio
 double precision rctse
+double precision eigen_max
+double precision eigen_vals(3)
+double precision turb_schmidt
+double precision gradchk, gradro_impl
+double precision kseps
+double precision matrn(6), oo_matrn(6)
+double precision ceps_impl, cphi3impl, cphiw_impl
+double precision impl_drsm(6,6)
+double precision implmat2add(3,3)
+double precision impl_lin_cst, impl_id_cst
 
 character(len=80) :: label
 double precision, allocatable, dimension(:,:) :: grad
@@ -171,10 +183,12 @@ double precision, dimension(:,:), pointer :: visten
 double precision, dimension(:), pointer :: cvara_ep, cvar_al
 double precision, dimension(:,:), pointer :: cvar_var, cvara_var
 double precision, dimension(:), pointer :: viscl
-double precision, dimension(:,:), pointer:: c_st_prv
+double precision, dimension(:,:), pointer:: c_st_prv, lagr_st_rij
 
 double precision, allocatable, dimension(:,:) :: cvara_r
 
+type(var_cal_opt) :: vcopt
+
 !===============================================================================
 
 !===============================================================================
@@ -192,13 +206,31 @@ allocate(weighb(nfabor))
 iii = 0
 jjj = 0
 
-if (iwarni(ivar).ge.1) then
+! Generating the tensor to vector (t2v) and vector to tensor (v2t) mask
+! arrays
+! a) t2v
+t2v(1,1) = 1; t2v(1,2) = 4; t2v(1,3) = 6;
+t2v(2,1) = 4; t2v(2,2) = 2; t2v(2,3) = 5;
+t2v(3,1) = 6; t2v(3,2) = 5; t2v(3,3) = 3;
+! b) i index of v2t
+iv2t(1) = 1; iv2t(2) = 2; iv2t(3) = 3;
+iv2t(4) = 1; iv2t(5) = 2; iv2t(6) = 1;
+! c) j index of v2t
+jv2t(1) = 1; jv2t(2) = 2; jv2t(3) = 3;
+jv2t(4) = 2; jv2t(5) = 3; jv2t(6) = 3;
+! d) kronecker symbol
+dij(:,:) = 0.0d0;
+dij(1,1) = 1.0d0; dij(2,2) = 1.0d0; dij(3,3) = 1.0d0;
+
+call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+
+if (vcopt%iwarni.ge.1) then
   call field_get_label(ivarfl(ivar), label)
   write(nfecra,1000) label
 endif
 
 call field_get_val_s(icrom, crom)
-call field_get_val_s(iprpfl(iviscl), viscl)
+call field_get_val_s(iviscl, viscl)
 
 call field_get_val_prev_s(ivarfl(iep), cvara_ep)
 if (iturb.ne.31) call field_get_val_s(ivarfl(ial), cvar_al)
@@ -225,7 +257,7 @@ do isou = 1, 6
 enddo
 !     S as Source, V as Variable
 thets  = thetst
-thetv  = thetav(ivar)
+thetv  = vcopt%thetav
 
 call field_get_key_int(ivarfl(ivar), kstprv, st_prv_id)
 if (st_prv_id .ge. 0) then
@@ -271,7 +303,6 @@ endif
 ! 2. User source terms
 !===============================================================================
 call cs_user_turbulence_source_terms2 &
-!===================================
  ( nvar   , nscal  , ncepdp , ncesmp ,                            &
    ivarfl(ivar)    ,                                              &
    icepdc , icetsm , itypsm ,                                     &
@@ -282,15 +313,15 @@ call cs_user_turbulence_source_terms2 &
 do isou = 1, dimrij
   if (st_prv_id.ge.0) then
     do iel = 1, ncel
-      !       Save for exchange
+      ! Save for exchange
       tuexpr = c_st_prv(isou,iel)
-      !       For continuation and the next time step
+      ! For continuation and the next time step
       c_st_prv(isou,iel) = smbr(isou,iel)
-      !       Second member of the previous time step
-      !       We suppose -rovsdt > 0: we implicite
-      !          the user source term (the rest)
+      ! Second member of the previous time step
+      ! We suppose -rovsdt > 0: we implicite
+      !    the user source term (the rest)
       smbr(isou,iel) = rovsdt(isou,isou,iel)*cvara_var(isou,iel)  - thets*tuexpr
-      !       Diagonal
+      ! Diagonal
       rovsdt(isou,isou,iel) = - thetv*rovsdt(isou,isou,iel)
     enddo
   else
@@ -306,34 +337,34 @@ enddo
 !===============================================================================
 
 !     2nd order is not taken into account
-do isou = 1, dimrij
-        if (iilagr.eq.2 .and. ltsdyn.eq.1) then
-    do iel = 1,ncel
-      smbr(isou, iel)   = smbr(isou, iel)   + tslage(isou,iel)
-      rovsdt(isou,isou,iel) = rovsdt(isou,isou, iel) + max(-tslagi(isou, iel),zero)
+if (iilagr.eq.2 .and. ltsdyn.eq.1) then
+  call field_get_val_v_by_name('rij_st_lagr', lagr_st_rij)
+  do iel = 1,ncel
+    do isou = 1, dimrij
+      smbr(isou, iel) = smbr(isou, iel) + lagr_st_rij(isou,iel)
+      rovsdt(isou,isou,iel) = rovsdt(isou,isou, iel) + max(-tslagi(iel),zero)
     enddo
-  endif
-enddo
+  enddo
+endif
 
 !===============================================================================
 ! 4. Mass source term
 !===============================================================================
 
-do isou = 1, dimrij
-  if (ncesmp.gt.0) then
+if (ncesmp.gt.0) then
 
+  do isou = 1, dimrij
     !       Integer equal to 1 (for navsto: nb of sur-iter)
     iiun = 1
 
     ! We increment smbr with -Gamma.var_prev and rovsdr with Gamma
     call catsmt &
-    !==========
    ( ncelet , ncel   , ncesmp , iiun     , isto2t ,                   &
      icetsm , itypsm(:,ivar + isou - 1)  ,                            &
      cell_f_vol , cvara_var  , smacel(:,ivar+isou-1)  , smacel(:,ipr) ,   &
      smbr   ,  rovsdt    , w1 )
 
-    ! If we extrapolate the source terms we put Gamma Pinj in the previous st
+   ! If we extrapolate the source terms we put Gamma Pinj in the previous st
     if (st_prv_id.ge.0) then
       do iel = 1, ncel
         c_st_prv(isou,iel) = c_st_prv(isou,iel) + w1(iel)
@@ -345,8 +376,9 @@ do isou = 1, dimrij
       enddo
     endif
 
-  endif
-enddo
+  enddo
+
+endif
 
 !===============================================================================
 ! 5. Unsteady term
@@ -354,10 +386,11 @@ enddo
 
 ! ---> Added in the matrix diagonal
 
-do isou = 1, dimrij
-  do iel=1,ncel
-    rovsdt(isou,isou,iel) = rovsdt(isou, isou,iel)                                       &
-              + istat(ivar+isou-1)*(crom(iel)/dt(iel))*cell_f_vol(iel)
+call field_get_key_struct_var_cal_opt(ivarfl(ivar+isou-1), vcopt)
+do iel=1,ncel
+  do isou = 1, dimrij
+    rovsdt(isou,isou,iel) = rovsdt(isou, isou,iel)                            &
+              + vcopt%istat*(crom(iel)/dt(iel))*cell_f_vol(iel)
   enddo
 enddo
 
@@ -372,6 +405,7 @@ enddo
 !     +Cr5*rho*k*(aik*rjk + ajk*rik)
 !     -2/3*epsilon*deltaij
 
+call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
 
 ! EBRSM
 if (iturb.eq.32) then
@@ -388,7 +422,7 @@ if (iturb.eq.32) then
 
 endif
 
-do iel=1,ncel
+do iel = 1, ncel
 
   ! EBRSM
   if (iturb.eq.32) then
@@ -409,6 +443,21 @@ do iel=1,ncel
     endif
   endif
 
+  ! Initalize implicit matrices at 0
+  do isou = 1, 6
+    do jsou = 1, 6
+      impl_drsm(isou, jsou) = 0.0d0
+    end do
+  end do
+  do isou = 1, 3
+    do jsou = 1, 3
+      implmat2add(isou, jsou) = 0.0d0
+    end do
+  end do
+
+  impl_lin_cst = 0.0d0
+  impl_id_cst  = 0.0d0
+
   ! Pij
   xprod(1,1) = -2.0d0*(cvara_var(1 ,iel)*gradv(1, 1, iel) +         &
                        cvara_var(4 ,iel)*gradv(2, 1, iel) +         &
@@ -457,8 +506,8 @@ do iel=1,ncel
         do jj = ii, 3
           do kk = 1, 3
             xprod(ii,jj) = xprod(ii,jj)                             &
-                     - ccorio*( matrot(ii,kk)*cvara_r(jj,kk) &
-                     + matrot(jj,kk)*cvara_r(ii,kk) )
+                            - ccorio*( matrot(ii,kk)*cvara_r(jj,kk) &
+                            + matrot(jj,kk)*cvara_r(ii,kk) )
           enddo
         enddo
       enddo
@@ -497,6 +546,12 @@ do iel=1,ncel
   xstrai(3,1) = xstrai(1,3)
   xstrai(3,2) = xstrai(2,3)
   xstrai(3,3) = gradv(3, 3, iel)
+  sym_strain(1) = xstrai(1,1)
+  sym_strain(2) = xstrai(2,2)
+  sym_strain(3) = xstrai(3,3)
+  sym_strain(4) = xstrai(1,2)
+  sym_strain(5) = xstrai(2,3)
+  sym_strain(6) = xstrai(1,3)
   ! omegaij
   xrotac(1,1) = 0.d0
   xrotac(1,2) = d1s2*(gradv(2, 1, iel)-gradv(1, 2, iel))
@@ -508,6 +563,117 @@ do iel=1,ncel
   xrotac(3,2) = -xrotac(2,3)
   xrotac(3,3) = 0.d0
 
+  do ii=1,3
+    do jj = 1,3
+      ! aii = aij.aij
+      aii    = aii+xaniso(ii,jj)*xaniso(ii,jj)
+      ! aklskl = aij.Sij
+      aklskl = aklskl + xaniso(ii,jj)*xstrai(ii,jj)
+    enddo
+  enddo
+
+  ! Computation of implicit components
+  ! -----------------------------------
+  ! Global variables needed for SSG and EBRSM
+  ! -------------
+  ! Computing the inverse matrix of R^n
+  ! Scaling by tr(R) in order to dodge inversion errors
+  do isou = 1, 6
+    matrn(isou) = cvara_var(isou,iel)/trrij
+    oo_matrn(isou) = 0.0d0
+  end do
+
+  ! Inversing the matrix
+  call symmetric_matrix_inverse(matrn, oo_matrn)
+  do isou = 1, dimrij
+    oo_matrn(isou) = oo_matrn(isou)/trrij
+  end do
+
+  ! Computing the maximal eigenvalue (in terms of norm!) of S
+  call calc_symtens_eigvals(sym_strain, eigen_vals)
+  eigen_max = maxval(abs(eigen_vals))
+
+  ! Constant for the dissipation
+  ceps_impl = d1s3 * cvara_ep(iel)
+
+
+  if (iturb .eq. 31 ) then ! SSG - Epsilon
+
+    ! Identity constant for phi3
+    cphi3impl = abs(cssgr2 - cssgr3*sqrt(aii))
+
+    ! Identity constant
+    impl_id_cst = - d2s3*cssgr1*min(trprod,0.0d0)         & ! Phi1
+                  - d1s3*cssgs2*cvara_ep(iel)*aii         & ! Phi2
+                  + cphi3impl * trrij * eigen_max         & ! Phi3
+                  + 2.0d0*d2s3*cssgr4*trrij*eigen_max     & ! Phi4
+                  + d2s3*trrij*cssgr4*max(aklskl,0.0d0)     ! Phi4
+
+    ! Linear constant
+    impl_lin_cst = eigen_max *     ( &
+                   1.0d0             & ! Production
+                 + cssgr4            & ! Phi 4 linear part
+                 + cssgr5          )   ! Phi 5 linear part
+
+    do jsou = 1, 3
+      do isou = 1 ,3
+        iii = t2v(isou,jsou)
+        implmat2add(isou,jsou) = xrotac(isou,jsou)              &
+                               + impl_lin_cst*deltij(iii)       &
+                               + impl_id_cst*d1s2*oo_matrn(iii) &
+                               + ceps_impl*oo_matrn(iii)
+      end do
+    end do
+
+    impl_drsm(:,:) = 0.0d0
+    call reduce_symprod33_to_66(implmat2add, impl_drsm)
+
+  else ! EBRSM
+
+    alpha3 = cvar_al(iel)**3
+
+    ! Phi3 constant
+    cphi3impl = abs(cebmr2 - cebmr3*sqrt(aii))
+
+    ! PhiWall + epsilon_wall constants for EBRSM
+    cphiw_impl = 6.0d0*(1.0d0-alpha3)*cvara_ep(iel)/trrij
+
+    ! -------------
+    ! The implicit components of Phi (pressure-velocity fluctuations)
+    ! are split into the linear part (A*R) and Id part (A*Id).
+    ! -------------
+
+    ! Identity constant
+    impl_id_cst = alpha3 * (                             &
+                  - d2s3*cebmr1*min(trprod,0.0d0)        & ! Phi1
+                  + cphi3impl * trrij * eigen_max        & ! Phi3
+                  + 2.0d0*d2s3*cebmr4*trrij*eigen_max    & ! Phi4
+                  + d2s3*trrij*cebmr4*max(aklskl,0.d0) )   ! Phi4
+
+
+    ! Linear constant
+    impl_lin_cst = eigen_max * (                  &
+                   1.0d0                          & ! Production
+                   + cebmr4 * alpha3              & ! Phi4 Linear part
+                   + cebmr5 * alpha3            ) & ! Phi5 Linear part
+                   + cphiw_impl                     ! Epsilon + Phi wall
+
+
+    do jsou = 1, 3
+      do isou = 1, 3
+        iii = t2v(isou,jsou)
+        implmat2add(isou,jsou) = xrotac(isou,jsou)                  &
+                               + impl_lin_cst*deltij(iii)           &
+                               + impl_id_cst*d1s2*oo_matrn(iii)     &
+                               + alpha3*ceps_impl*oo_matrn(iii)
+      end do
+    end do
+
+    impl_drsm(:,:) = 0.0d0
+    call reduce_symprod33_to_66(implmat2add, impl_drsm)
+
+  end if
+
   ! Rotating frame of reference => "absolute" vorticity
   if (icorio.eq.1) then
     do ii = 1, 3
@@ -517,14 +683,6 @@ do iel=1,ncel
     enddo
   endif
 
-  do ii=1,3
-    do jj = 1,3
-      ! aii = aij.aij
-      aii    = aii+xaniso(ii,jj)*xaniso(ii,jj)
-      ! aklskl = aij.Sij
-      aklskl = aklskl + xaniso(ii,jj)*xstrai(ii,jj)
-    enddo
-  enddo
 
   do isou = 1, dimrij
     if (isou.eq.1)then
@@ -574,6 +732,7 @@ do iel=1,ncel
 
     if (iturb.eq.31) then
 
+      ! Explicit terms
       pij = xprod(iii,jjj)
       phiij1 = -cvara_ep(iel)* &
          (cssgs1*xaniso(iii,jjj)+cssgs2*(aikakj-d1s3*deltij(isou)*aii))
@@ -585,9 +744,11 @@ do iel=1,ncel
 
       w1(iel) = cromo(iel)*cell_f_vol(iel)*(pij+phiij1+phiij2+epsij)
 
+      ! Implicit terms
       w2(iel) = cell_f_vol(iel)/trrij*crom(iel)*(                              &
              cssgs1*cvara_ep(iel) + cssgr1*max(trprod,0.d0) )
 
+
     ! EBRSM
     else
 
@@ -607,6 +768,9 @@ do iel=1,ncel
       ! of phi and epsilon
 
       ! Calculation of the term near the wall \f$ \Phi_{ij}^w \f$ --> W3
+      !
+      ! Phiw = -5.0 * (eps/k) * [ R*Xn + Xn^T*R - 0.5*tr(Xn*R)*(Xn + Id) ]
+      !
       phiijw = 0.d0
       xnnd = d1s2*( xnal(iii)*xnal(jjj) + deltij(isou) )
       do kk = 1, 3
@@ -649,17 +813,27 @@ do iel=1,ncel
       ! \f$ \alpha^3 \f$
       w2(iel) = cell_f_vol(iel)*crom(iel)*(                             &
                 cebms1*cvara_ep(iel)/trrij*alpha3                       &
-               +cebmr1*max(trprod/trrij,0.d0)*alpha3                &
-      ! Implicitation of epsijw
-      ! (the factor 5 appears when we calculate \f$ Phi_{ij}^w - epsijw\f$)
-              + 5.d0 * (1.d0-alpha3)*cvara_ep(iel)/trrij                &
-              +        (1.d0-alpha3)*cvara_ep(iel)/trrij)
+               +cebmr1*max(trprod/trrij,0.d0)*alpha3)
+
+!FIXME               +cebmr1*max(trprod/trrij,0.d0)*alpha3                &
+!      ! Implicitation of epsijw
+!      ! (the factor 5 appears when we calculate \f$ Phi_{ij}^w - epsijw\f$)
+!              + 5.d0 * (1.d0-alpha3)*cvara_ep(iel)/trrij                &
+!              +        (1.d0-alpha3)*cvara_ep(iel)/trrij)
     endif
     if (st_prv_id.ge.0) then
       c_st_prv(isou,iel) = c_st_prv(isou,iel) + w1(iel)
     else
       smbr(isou,iel) = smbr(isou,iel) + w1(iel)
       rovsdt(isou,isou,iel) = rovsdt(isou,isou,iel) + w2(iel)
+
+      ! Carefull ! Inversion of the order of the coefficients since
+      ! rovsdt matrix is then used by a c function for the linear solving
+      do jsou = 1, 6
+        rovsdt(jsou,isou,iel) = rovsdt(jsou,isou,iel) + cell_f_vol(iel) &
+                                *crom(iel) * impl_drsm(isou,jsou)
+      end do
+
     endif
   enddo
 enddo
@@ -672,7 +846,6 @@ if (iturb.eq.32) then
   deallocate(grad)
 endif
 
-
 !===============================================================================
 ! 7. Buoyancy source term
 !===============================================================================
@@ -706,6 +879,57 @@ if (igrari.eq.1) then
   ! Free memory
   deallocate(w7)
 
+  ! Implicit buoyancy term
+  if (iscalt.gt.0 .and. nscal.ge.iscalt) then
+    call field_get_key_double(ivarfl(isca(iscalt)), ksigmas, turb_schmidt)
+    if (iturb .eq. 32) then
+      gradro_impl = -1.5d0*cmu/turb_schmidt * (1.0d0-cebmr6)
+    else
+      gradro_impl = -1.5d0*cmu/turb_schmidt * (1.0d0-crij3)
+    end if
+  else
+    if (iturb .eq. 32) then
+      gradro_impl = -1.5d0*cmu*(1.0d0-cebmr6)
+    else
+      gradro_impl = -1.5d0*cmu*(1.0d0-crij3)
+    end if
+  end if
+
+  do iel = 1, ncel
+
+    do jsou = 1, 6
+      do isou = 1, 6
+        impl_drsm(isou,jsou) = 0.0d0
+      end do
+    end do
+    implmat2add(:,:) = 0.0d0
+
+    gradchk = gx*gradro(1,iel) + gy*gradro(2,iel) + gz*gradro(3,iel)
+    if (gradchk .gt. 0.0d0) then
+      kseps = (cvara_var(1,iel) + cvara_var(2,iel) + cvara_var(3,iel)) &
+              / (2.0d0*cvara_ep(iel))
+
+      do jsou = 1, 3
+        implmat2add(1,jsou) = kseps * gradro_impl * gx * gradro(jsou,iel)
+        implmat2add(2,jsou) = kseps * gradro_impl * gy * gradro(jsou,iel)
+        implmat2add(3,jsou) = kseps * gradro_impl * gz * gradro(jsou,iel)
+      end do
+
+    end if
+
+    ! Compute the 6x6 matrix A which verifies
+    ! A . R = M . R + R . M^t
+    call reduce_symprod33_to_66(implmat2add, impl_drsm)
+
+    do isou = 1, dimrij
+      do jsou = 1, dimrij
+        rovsdt(jsou,isou,iel) = rovsdt(jsou,isou,iel) - cell_f_vol(iel) &
+                                * impl_drsm(isou,jsou)
+      end do
+    end do
+
+  end do
+
 endif
 
 !===============================================================================
@@ -713,7 +937,7 @@ endif
 !===============================================================================
 
 ! Symmetric tensor diffusivity (GGDH)
-if (idften(ivar).eq.6) then
+if (iand(vcopt%idften, ANISOTROPIC_RIGHT_DIFFUSION).ne.0) then
 
   call field_get_val_v(ivsten, visten)
 
@@ -726,10 +950,9 @@ if (idften(ivar).eq.6) then
     viscce(6,iel) = visten(6,iel)
   enddo
 
-  iwarnp = iwarni(ivar)
+  iwarnp = vcopt%iwarni
 
   call vitens &
-  !==========
  ( viscce , iwarnp ,             &
    weighf , weighb ,             &
    viscf  , viscb  )
@@ -740,12 +963,11 @@ else
   do iel = 1, ncel
     trrij = 0.5d0 * (cvara_var(1,iel) + cvara_var(2,iel) + cvara_var(3,iel))
     rctse = crom(iel) * csrij * trrij**2 / cvara_ep(iel)
-    w1(iel) = viscl(iel) + idifft(ivar)*rctse
+    w1(iel) = viscl(iel) + vcopt%idifft*rctse
   enddo
 
   call viscfa                    &
-  !==========
-  ( imvisf ,                      &
+  ( imvisf ,                     &
    w1     ,                      &
    viscf  , viscb  )
 
@@ -764,25 +986,25 @@ if (st_prv_id.ge.0) then
   enddo
 endif
 
-iconvp = iconv (ivar)
-idiffp = idiff (ivar)
+iconvp = vcopt%iconv
+idiffp = vcopt%idiff
 ndircp = ndircl(ivar)
-nswrsp = nswrsm(ivar)
-nswrgp = nswrgr(ivar)
-imligp = imligr(ivar)
-ircflp = ircflu(ivar)
-ischcp = ischcv(ivar)
-isstpp = isstpc(ivar)
-idftnp = idften(ivar)
-iswdyp = iswdyn(ivar)
-iwarnp = iwarni(ivar)
-blencp = blencv(ivar)
-epsilp = epsilo(ivar)
-epsrsp = epsrsm(ivar)
-epsrgp = epsrgr(ivar)
-climgp = climgr(ivar)
-extrap = extrag(ivar)
-relaxp = relaxv(ivar)
+nswrsp = vcopt%nswrsm
+nswrgp = vcopt%nswrgr
+imligp = vcopt%imligr
+ircflp = vcopt%ircflu
+ischcp = vcopt%ischcv
+isstpp = vcopt%isstpc
+idftnp = vcopt%idften
+iswdyp = vcopt%iswdyn
+iwarnp = vcopt%iwarni
+blencp = vcopt%blencv
+epsilp = vcopt%epsilo
+epsrsp = vcopt%epsrsm
+epsrgp = vcopt%epsrgr
+climgp = vcopt%climgr
+extrap = vcopt%extrag
+relaxp = vcopt%relaxv
 ! all boundary convective flux with upwind
 icvflb = 0
 
@@ -792,8 +1014,7 @@ call field_get_coefaf_v(ivarfl(ivar), cofafp)
 call field_get_coefbf_v(ivarfl(ivar), cofbfp)
 
 call coditts &
-!==========
- ( idtvar , ivar   , iconvp , idiffp , ndircp ,                   &
+ ( idtvar , ivarfl(ivar)    , iconvp , idiffp , ndircp ,          &
    imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , idftnp , iswdyp ,                            &
    iwarnp ,                                                       &
