@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -109,10 +109,11 @@ integer          ivoid(1)
 double precision blencp, epsilp, epsrgp, climgp, extrap, relaxp
 double precision epsrsp
 double precision tuexpe, thets , thetv , thetap, thetp1
-double precision d2s3, d1s4, d3s2
+double precision d2s3
 double precision xk, xe, xnu, xrom, ttke, ttmin, llke, llmin, tt
 double precision fhomog
 double precision hint
+double precision l2, time_scale
 
 double precision rvoid(1)
 
@@ -121,7 +122,7 @@ double precision, allocatable, dimension(:) :: viscf, viscb
 double precision, allocatable, dimension(:) :: smbr, rovsdt
 double precision, allocatable, dimension(:,:) :: gradp, gradk
 double precision, allocatable, dimension(:) :: w1, w2, w3
-double precision, allocatable, dimension(:) :: w4, w5
+double precision, allocatable, dimension(:) :: w5
 double precision, allocatable, dimension(:) :: dpvar
 double precision, dimension(:), pointer :: imasfl, bmasfl
 double precision, dimension(:), pointer :: crom, cromo
@@ -134,6 +135,8 @@ double precision, dimension(:), pointer :: cpro_pcvlo, cpro_pcvto
 double precision, dimension(:), pointer :: viscl, visct
 double precision, dimension(:), pointer :: c_st_phi_p, c_st_a_p
 
+type(var_cal_opt) :: vcopt
+
 !===============================================================================
 
 !===============================================================================
@@ -146,12 +149,12 @@ allocate(smbr(ncelet), rovsdt(ncelet))
 
 ! Allocate work arrays
 allocate(w1(ncelet), w2(ncelet), w3(ncelet))
-allocate(w4(ncelet), w5(ncelet))
+allocate(w5(ncelet))
 allocate(dpvar(ncelet))
 
 call field_get_val_s(icrom, crom)
-call field_get_val_s(iprpfl(iviscl), viscl)
-call field_get_val_s(iprpfl(ivisct), visct)
+call field_get_val_s(iviscl, viscl)
+call field_get_val_s(ivisct, visct)
 call field_get_key_int(ivarfl(iu), kimasf, iflmas)
 call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
 call field_get_val_s(iflmas, imasfl)
@@ -191,10 +194,10 @@ if (istprv.ge.0) then
 endif
 
 d2s3 = 2.0d0/3.0d0
-d1s4 = 1.0d0/4.0d0
-d3s2 = 3.0d0/2.0d0
 
-if (iwarni(iphi).ge.1) then
+call field_get_key_struct_var_cal_opt(ivarfl(iphi), vcopt)
+
+if (vcopt%iwarni.ge.1) then
   write(nfecra,1000)
 endif
 
@@ -245,23 +248,25 @@ elseif (iturb.eq.51) then
   cvara_var => cvara_al
 endif
 
-if (iwarni(ivar).ge.1) then
+call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+
+if (vcopt%iwarni.ge.1) then
   call field_get_label(ivarfl(ivar), label)
   write(nfecra,1100) label
 endif
 
 !     S as Source, V as Variable
 thets  = thetst
-thetv  = thetav(ivar )
+thetv  = vcopt%thetav
 
 call field_get_val_s(icrom, cromo)
-call field_get_val_s(iprpfl(iviscl), cpro_pcvlo)
+call field_get_val_s(iviscl, cpro_pcvlo)
 if (istprv.ge.0) then
   if (iroext.gt.0) then
     call field_get_val_prev_s(icrom, cromo)
   endif
   if (iviext.gt.0) then
-    call field_get_val_s(iprpfl(ivisla), cpro_pcvlo)
+    call field_get_val_prev_s(iviscl, cpro_pcvlo)
   endif
 endif
 
@@ -287,11 +292,11 @@ call cs_user_turbulence_source_terms &
 if (istprv.ge.0) then
   do iel = 1, ncel
     !       Save for exchange
-    tuexpe = c_st_phi_p(iel)
+    tuexpe = c_st_a_p(iel)
     !       For the futur and the next step time
     !       We put a mark "-" because in fact we solve
     !       \f$-\div{\grad {\dfrac{\overline{f}}{\alpha}}} = ... \f$
-    c_st_phi_p(iel) = - smbr(iel)
+    c_st_a_p(iel) = - smbr(iel)
     !       Second member of the previous step time
     !       we implicit the user source term (the rest)
     smbr(iel) = - rovsdt(iel)*cvara_var(iel) - thets*tuexpe
@@ -311,7 +316,7 @@ endif
 
 
 !===============================================================================
-! 3.2 Source term of f_barre/alpha
+! 3.2 Source term of f_barre or alpha
 !   For f_barre (phi_fbar)
 !     \f[ smbr =\dfrac{1}{L^2*(f_b + \dfrac{1}{T(C1-1)(phi-2/3)}
 !                                               - \dfrac{C2 Pk}{k \rho}
@@ -358,12 +363,14 @@ iccocg = 1
 inc = 1
 init = 1
 
-nswrgp = nswrgr(iphi)
-imligp = imligr(iphi)
-iwarnp = iwarni(iphi)
-epsrgp = epsrgr(iphi)
-climgp = climgr(iphi)
-extrap = extrag(iphi)
+call field_get_key_struct_var_cal_opt(ivarfl(iphi), vcopt)
+
+nswrgp = vcopt%nswrgr
+imligp = vcopt%imligr
+iwarnp = vcopt%iwarni
+epsrgp = vcopt%epsrgr
+climgp = vcopt%climgr
+extrap = vcopt%extrag
 iphydp = 0
 
 call itrgrp &
@@ -375,114 +382,106 @@ call itrgrp &
    cvara_phi       ,                                                 &
    coefap , coefbp , cofafp , cofbfp ,                               &
    viscf  , viscb  ,                                                 &
-   w3     , w3     , w3     ,                                        &
+   w3     ,                                                          &
    w2     )
 
-!      We store T in W3 et L^2 in W4
-!      In this case of the second-order in time, T is calculated in n
-!      (it will be extrapolated) and L^2 in n+theta
-!      (even if k and eps stay in n)
-do iel=1,ncel
+!     Explicit term, stores ke temporarily in w5
+!     w2 is already multipicated by the volume which already contains
+!     a mark "-" (coming from itrgrp)
+do iel = 1, ncel
+  ! Compute the time scale
   xk = cvara_k(iel)
   xe = cvara_ep(iel)
-  xnu  = cpro_pcvlo(iel)/cromo(iel)
+  xrom = cromo(iel)
+  xnu  = cpro_pcvlo(iel)/xrom
   ttke = xk / xe
   if (iturb.eq.50) then
     ttmin = cv2fct*sqrt(xnu/xe)
-    w3(iel) = max(ttke,ttmin)
+    time_scale = max(ttke,ttmin)
   elseif (iturb.eq.51) then
     ttmin = cpalct*sqrt(xnu/xe)
-    w3(iel) = sqrt(ttke**2 + ttmin**2)
+    time_scale = sqrt(ttke**2 + ttmin**2)
   endif
 
-  xnu  = viscl(iel)/crom(iel)
-  llke = xk**d3s2/xe
   if (iturb.eq.50) then
-    llmin = cv2fet*(xnu**3/xe)**d1s4
-    w4(iel) = ( cv2fcl*max(llke,llmin) )**2
+    w5(iel) = - cell_f_vol(iel)*                                    &
+         ( (cv2fc1-1.d0)*(cvara_phi(iel)-d2s3)/time_scale           &
+               -cv2fc2*prdv2f(iel)/xrom/xk                          &
+          -2.0d0*xnu/xe/time_scale * w1(iel) ) - xnu*w2(iel)
   elseif (iturb.eq.51) then
-    llmin = cpalet*(xnu**3/xe)**d1s4
-    w4(iel) = cpalcl**2*(llke**2 + llmin**2)
+    w5(iel) = cell_f_vol(iel)
   endif
 enddo
 
-!     Explicit term, stores ke temporarily in W5
-!     W2 is already multipicated by the volume which already contains
-!     a mark "-" (coming from itrgrp)
-do iel = 1, ncel
-    xrom = cromo(iel)
-    xnu  = cpro_pcvlo(iel)/xrom
-    xk = cvara_k(iel)
-    xe = cvara_ep(iel)
-    if (iturb.eq.50) then
-      w5(iel) = - cell_f_vol(iel)*                                    &
-           ( (cv2fc1-1.d0)*(cvara_phi(iel)-d2s3)/w3(iel)              &
-             -cv2fc2*prdv2f(iel)/xrom/xk                          &
-             -2.0d0*xnu/xe/w3(iel)*w1(iel) ) - xnu*w2(iel)
-    elseif (iturb.eq.51) then
-      w5(iel) = cell_f_vol(iel)
-    endif
-enddo
-!     If we extrapolate the source term:
+! If we extrapolate the source term:
 if (istprv.ge.0) then
   thetp1 = 1.d0 + thets
   do iel = 1, ncel
-    c_st_phi_p(iel) = c_st_phi_p(iel) + w5(iel)
-    smbr(iel) = smbr(iel) + thetp1*c_st_phi_p(iel)
+    c_st_a_p(iel) = c_st_a_p(iel) + w5(iel)
+    smbr(iel) = smbr(iel) + thetp1*c_st_a_p(iel)
   enddo
-!     Otherwise: smbr
+! Otherwise: smbr
 else
   do iel = 1, ncel
     smbr(iel) = smbr(iel) + w5(iel)
   enddo
 endif
 
-!     Implicit term
-do iel = 1, ncel
-  if (iturb.eq.50) then
-    smbr(iel) = ( - cell_f_vol(iel)*cvara_fb(iel) + smbr(iel) ) / w4(iel)
-  elseif (iturb.eq.51) then
-    smbr(iel) = ( - cell_f_vol(iel)*cvara_al(iel) + smbr(iel) ) / w4(iel)
-  endif
-enddo
-
-! ---> Matrix
-
 if (istprv.ge.0) then
   thetap = thetv
 else
   thetap = 1.d0
 endif
+
+! Implicit term
 do iel = 1, ncel
-  rovsdt(iel) = (rovsdt(iel) + cell_f_vol(iel)*thetap)/w4(iel)
+
+  ! Compute L^2
+  xk = cvara_k(iel)
+  xe = cvara_ep(iel)
+  xnu  = viscl(iel)/crom(iel)
+
+  llke = xk**1.5d0/xe
+  if (iturb.eq.50) then
+    llmin = cv2fet*(xnu**3/xe)**0.25d0
+    l2 = ( cv2fcl*max(llke,llmin) )**2
+  elseif (iturb.eq.51) then
+    llmin = cpalet*(xnu**3/xe)**0.25d0
+    l2 = cpalcl**2*(llke**2 + llmin**2)
+  endif
+
+  smbr(iel) = ( - cell_f_vol(iel)*cvara_var(iel) + smbr(iel) ) / l2
+  ! ---> Matrix
+  rovsdt(iel) = (rovsdt(iel) + cell_f_vol(iel)*thetap) / l2
 enddo
 
-
 !===============================================================================
-! 3.3 Effective resolution in the equation of f_barre / alpha
+! 3.3 Effective resolution in the equation of f_barre or alpha
 !===============================================================================
 
-iconvp = iconv (ivar)
-idiffp = idiff (ivar)
+call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+
+iconvp = vcopt%iconv
+idiffp = vcopt%idiff
 ndircp = ndircl(ivar)
-nswrsp = nswrsm(ivar)
-nswrgp = nswrgr(ivar)
-imligp = imligr(ivar)
-ircflp = ircflu(ivar)
-ischcp = ischcv(ivar)
-isstpp = isstpc(ivar)
+nswrsp = vcopt%nswrsm
+nswrgp = vcopt%nswrgr
+imligp = vcopt%imligr
+ircflp = vcopt%ircflu
+ischcp = vcopt%ischcv
+isstpp = vcopt%isstpc
 iescap = 0
 imucpp = 0
-idftnp = idften(ivar)
-iswdyp = iswdyn(ivar)
-iwarnp = iwarni(ivar)
-blencp = blencv(ivar)
-epsilp = epsilo(ivar)
-epsrsp = epsrsm(ivar)
-epsrgp = epsrgr(ivar)
-climgp = climgr(ivar)
-extrap = extrag(ivar)
-relaxp = relaxv(ivar)
+idftnp = vcopt%idften
+iswdyp = vcopt%iswdyn
+iwarnp = vcopt%iwarni
+blencp = vcopt%blencv
+epsilp = vcopt%epsilo
+epsrsp = vcopt%epsrsm
+epsrgp = vcopt%epsrgr
+climgp = vcopt%climgr
+extrap = vcopt%extrag
+relaxp = vcopt%relaxv
 ! all boundary convective flux with upwind
 icvflb = 0
 
@@ -492,7 +491,7 @@ call field_get_coefaf_s(ivarfl(ivar), cofafp)
 call field_get_coefbf_s(ivarfl(ivar), cofbfp)
 
 call codits &
- ( idtvar , ivar   , iconvp , idiffp , ndircp ,                   &
+ ( idtvar , init   , ivarfl(ivar)    , iconvp , idiffp , ndircp , &
    imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
    iwarnp ,                                                       &
@@ -513,22 +512,24 @@ call codits &
 
 ivar = iphi
 
+call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+
 call field_get_val_s(ivarfl(ivar), cvar_var)
 cvara_var => cvara_phi
 
-if (iwarni(ivar).ge.1) then
+if (vcopt%iwarni.ge.1) then
   call field_get_label(ivarfl(ivar), label)
   write(nfecra,1100) label
 endif
 
 !     S as Source, V as Variable
 thets  = thetst
-thetv  = thetav(ivar )
+thetv  = vcopt%thetav
 
-call field_get_val_s(iprpfl(ivisct), cpro_pcvto)
+call field_get_val_s(ivisct, cpro_pcvto)
 if (istprv.ge.0) then
   if (iviext.gt.0) then
-    call field_get_val_s(iprpfl(ivista), cpro_pcvto)
+    call field_get_val_prev_s(ivisct, cpro_pcvto)
   endif
 endif
 
@@ -554,9 +555,9 @@ call cs_user_turbulence_source_terms &
 if (istprv.ge.0) then
   do iel = 1, ncel
     !       Save for exchange
-    tuexpe = c_st_a_p(iel)
+    tuexpe = c_st_phi_p(iel)
     !       For the future and the next time step
-    c_st_a_p(iel) = smbr(iel)
+    c_st_phi_p(iel) = smbr(iel)
     !       Second member of previous time step
     !       We suppose -rovsdt > 0: we implicit
     !       the user source term (the rest)
@@ -591,7 +592,7 @@ if (ncesmp.gt.0) then
   ! If we extrapolate the source term we put Gamma Pinj in the prev. TS
   if (istprv.ge.0) then
     do iel = 1, ncel
-      c_st_a_p(iel) = c_st_a_p(iel) + w2(iel)
+      c_st_phi_p(iel) = c_st_phi_p(iel) + w2(iel)
     enddo
   !       Otherwise we put it directly in smbr
   else
@@ -612,7 +613,7 @@ endif
 
 do iel = 1, ncel
   rovsdt(iel) = rovsdt(iel)                                       &
-           + istat(ivar)*(crom(iel)/dt(iel))*cell_f_vol(iel)
+           + vcopt%istat*(crom(iel)/dt(iel))*cell_f_vol(iel)
 enddo
 
 !===============================================================================
@@ -633,8 +634,7 @@ enddo
 do iel = 1, ncel
   xk = cvara_k(iel)
   xe = cvara_ep(iel)
-  xrom = cromo(iel)
-  xnu  = cpro_pcvlo(iel)/xrom
+  xnu  = viscl(iel)/crom(iel)
   if (iturb.eq.50) then
     ! The term in f_bar is taken at the current and not previous time step
     ! ... a priori better
@@ -653,6 +653,7 @@ do iel = 1, ncel
     w2(iel)   = cell_f_vol(iel)*                                    &
          ( cvara_al(iel)**3*fhomog*xrom                             &
            +2.d0/xk*cpro_pcvto(iel)/sigmak*w1(iel) )
+         !FIXME implit negativ w1 and fhomog
   endif
 
 enddo
@@ -661,8 +662,8 @@ enddo
 if (istprv.ge.0) then
   thetp1 = 1.d0 + thets
   do iel = 1, ncel
-    c_st_a_p(iel) = c_st_a_p(iel) + w2(iel)
-    smbr(iel) = smbr(iel) + thetp1*c_st_a_p(iel)
+    c_st_phi_p(iel) = c_st_phi_p(iel) + w2(iel)
+    smbr(iel) = smbr(iel) + thetp1*c_st_phi_p(iel)
   enddo
 !     Otherwise: smbr
 else
@@ -724,7 +725,7 @@ call field_get_coefb_s(ivarfl(ivar), coefbp)
 call field_get_coefaf_s(ivarfl(ivar), cofafp)
 call field_get_coefbf_s(ivarfl(ivar), cofbfp)
 
-if (idiff(ivar).ge.1) then
+if (vcopt%idiff.ge.1) then
   do iel = 1, ncel
     if (iturb.eq.50) then
       w2(iel) = viscl(iel)      + visct(iel)/sigmak
@@ -773,31 +774,31 @@ endif
 if (istprv.ge.0) then
   thetp1 = 1.d0 + thets
   do iel = 1, ncel
-    smbr(iel) = smbr(iel) + thetp1*c_st_a_p(iel)
+    smbr(iel) = smbr(iel) + thetp1*c_st_phi_p(iel)
   enddo
 endif
 
-iconvp = iconv (ivar)
-idiffp = idiff (ivar)
+iconvp = vcopt%iconv
+idiffp = vcopt%idiff
 ndircp = ndircl(ivar)
-nswrsp = nswrsm(ivar)
-nswrgp = nswrgr(ivar)
-imligp = imligr(ivar)
-ircflp = ircflu(ivar)
-ischcp = ischcv(ivar)
-isstpp = isstpc(ivar)
+nswrsp = vcopt%nswrsm
+nswrgp = vcopt%nswrgr
+imligp = vcopt%imligr
+ircflp = vcopt%ircflu
+ischcp = vcopt%ischcv
+isstpp = vcopt%isstpc
 iescap = 0
 imucpp = 0
-idftnp = idften(ivar)
-iswdyp = iswdyn(ivar)
-iwarnp = iwarni(ivar)
-blencp = blencv(ivar)
-epsilp = epsilo(ivar)
-epsrsp = epsrsm(ivar)
-epsrgp = epsrgr(ivar)
-climgp = climgr(ivar)
-extrap = extrag(ivar)
-relaxp = relaxv(ivar)
+idftnp = vcopt%idften
+iswdyp = vcopt%iswdyn
+iwarnp = vcopt%iwarni
+blencp = vcopt%blencv
+epsilp = vcopt%epsilo
+epsrsp = vcopt%epsrsm
+epsrgp = vcopt%epsrgr
+climgp = vcopt%climgr
+extrap = vcopt%extrag
+relaxp = vcopt%relaxv
 ! all boundary convective flux with upwind
 icvflb = 0
 
@@ -807,7 +808,7 @@ call field_get_coefaf_s(ivarfl(ivar), cofafp)
 call field_get_coefbf_s(ivarfl(ivar), cofbfp)
 
 call codits &
- ( idtvar , ivar   , iconvp , idiffp , ndircp ,                   &
+ ( idtvar , init   , ivarfl(ivar)    , iconvp , idiffp , ndircp , &
    imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
    iwarnp ,                                                       &
@@ -826,13 +827,15 @@ call codits &
 ! 5. Clipping
 !===============================================================================
 
-call clpv2f(ncel, iwarni(iphi))
+call field_get_key_struct_var_cal_opt(ivarfl(iphi), vcopt)
+
+call clpv2f(ncel, vcopt%iwarni)
 
 ! Free memory
 deallocate(viscf, viscb)
 deallocate(smbr, rovsdt)
 deallocate(w1, w2, w3)
-deallocate(w4, w5)
+deallocate(w5)
 deallocate(dpvar)
 
 !--------
