@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -59,6 +59,7 @@ use parall
 use period
 use mesh
 use field
+use field_operator
 use cs_c_bindings
 
 !===============================================================================
@@ -76,8 +77,7 @@ double precision smbr(ncelet)
 
 integer          iel   , ii    , jj    , kk    , mm
 integer          iskm  , iski  , iskj
-integer          ifac
-integer          inc   , iccocg, f_id0
+integer          inc   , iccocg, f_id
 
 double precision cmu075, distxn, d2s3  , trrij , xk
 double precision vnk   , vnm   , vni   , vnj
@@ -87,8 +87,8 @@ double precision aa    , bb    , xnorme
 double precision, allocatable, dimension(:,:) :: grad
 double precision, allocatable, dimension(:) :: produk, epsk
 double precision, allocatable, dimension(:) :: w2, w3, w4, w6
-double precision, allocatable, dimension(:) :: coefax, coefbx
 double precision, dimension(:), pointer :: crom, cromo
+double precision, dimension(:), pointer :: w_dist
 double precision, dimension(:), pointer :: cvara_ep
 double precision, dimension(:), pointer :: cvara_rkm, cvara_rki, cvara_rkj
 double precision, dimension(:), pointer :: cvara_r11, cvara_r22, cvara_r33
@@ -143,50 +143,21 @@ d2s3   = 2.d0/3.d0
 ! 2.Calculation in the orthogonal straights cells in corresponding walls
 !===============================================================================
 
-!  We store the components in the work tables W2, W3 and W4
-
 !     The orthogonal straght is defined as -gradient of the distance
 !       to the wall
 
-!       The distance to the wall worth 0 at the wall
-!         by definition and obey a flux nowhere else
-
-! Allocate temporary arrays
-allocate(coefax(nfabor), coefbx(nfabor))
-
-do ifac = 1, nfabor
-  if (itypfb(ifac).eq.iparoi .or. itypfb(ifac).eq.iparug) then
-    coefax(ifac) = 0.0d0
-    coefbx(ifac) = 0.0d0
-  else
-    coefax(ifac) = 0.0d0
-    coefbx(ifac) = 1.0d0
-  endif
-enddo
-
-!       Calculation of gradient
-
-! Allocate a temporary array for the gradient calculation
 allocate(grad(3,ncelet))
 
-if (irangp.ge.0.or.iperio.eq.1) then
-  call synsca(dispar)
-endif
-
 inc    = 1
 iccocg = 1
-f_id0  = -1
 
-call gradient_s                                                  &
-( f_id0  , imrgra , inc    , iccocg , nswrgy , imligy , iwarny , &
-  epsrgy , climgy , extray ,                                     &
-  dispar , coefax , coefbx ,                                     &
-  grad   )
+call field_get_id("wall_distance", f_id)
+call field_get_val_s(f_id, w_dist)
 
-! Free memory
-deallocate(coefax, coefbx)
+! Current gradient: iprev = 0
+call field_gradient_scalar(f_id, 0, imrgra, inc, iccocg, grad)
 
-!     Normalization (warning, the gradient may be sometimes equal to 0)
+! Normalization (warning, the gradient may be sometimes equal to 0)
 
 do iel = 1 ,ncel
   xnorme = max(sqrt(grad(1,iel)**2+grad(2,iel)**2+grad(3,iel)**2),epzero)
@@ -402,7 +373,7 @@ enddo
 !      Apart from the loop
 
 do iel = 1, ncel
-  distxn =  max(dispar(iel),epzero)
+  distxn =  max(w_dist(iel),epzero)
   trrij  = 0.5d0 * (cvara_r11(iel) + cvara_r22(iel) + cvara_r33(iel))
   aa = 1.d0
   bb = cmu075*trrij**1.5d0/(xkappa*cvara_ep(iel)*distxn)
@@ -459,6 +430,7 @@ use parall
 use period
 use mesh
 use field
+use field_operator
 use cs_c_bindings
 
 !===============================================================================
@@ -476,19 +448,18 @@ double precision smbr(6,ncelet)
 
 integer          iel   , ii    , jj    , kk    , mm
 integer          iskm  , iski  , iskj
-integer          ifac
-integer          inc   , iccocg, f_id0
+integer          inc   , iccocg, f_id
 
 double precision cmu075, distxn, d2s3  , trrij , xk
 double precision vnk   , vnm   , vni   , vnj
 double precision deltki, deltkj, deltkm, deltij(6)
-double precision aa    , bb    , xnorme
+double precision bb    , xnorme
 
 double precision, allocatable, dimension(:,:) :: grad
 double precision, allocatable, dimension(:) :: produk, epsk
-double precision, allocatable, dimension(:) :: w2, w3, w4, w6
-double precision, allocatable, dimension(:) :: coefax, coefbx
+double precision, allocatable, dimension(:) :: w6
 double precision, dimension(:), pointer :: crom, cromo
+double precision, dimension(:), pointer :: w_dist
 double precision, dimension(:), pointer :: cvara_ep
 double precision, dimension(:,:), pointer :: cvara_var
 double precision, allocatable, dimension(:,:,:) :: cvara_rij
@@ -500,7 +471,8 @@ double precision, allocatable, dimension(:,:,:) :: cvara_rij
 
 ! Allocate temporary arrays
 allocate(produk(ncelet), epsk(ncelet))
-allocate(w2(ncelet), w3(ncelet), w4(ncelet), w6(ncelet))
+allocate(w6(ncelet))
+allocate(grad(3,ncelet))
 allocate(cvara_rij(3,3,ncel))
 ! Initialize variables to avoid compiler warnings
 
@@ -554,61 +526,29 @@ d2s3   = 2.d0/3.d0
 ! 2.Calculation in the orthogonal straights cells in corresponding walls
 !===============================================================================
 
-!  We store the components in the work tables W2, W3 and W4
-
 !     The orthogonal straght is defined as -gradient of the distance
 !       to the wall
 
-!       The distance to the wall worth 0 at the wall
-!         by definition and obey a flux nowhere else
-
-! Allocate temporary arrays
-allocate(coefax(nfabor), coefbx(nfabor))
-
-do ifac = 1, nfabor
-  if (itypfb(ifac).eq.iparoi .or. itypfb(ifac).eq.iparug) then
-    coefax(ifac) = 0.0d0
-    coefbx(ifac) = 0.0d0
-  else
-    coefax(ifac) = 0.0d0
-    coefbx(ifac) = 1.0d0
-  endif
-enddo
-
 !       Calculation of gradient
 
-! Allocate a temporary array for the gradient calculation
-allocate(grad(3,ncelet))
-
-if (irangp.ge.0.or.iperio.eq.1) then
-  call synsca(dispar)
-endif
-
 inc    = 1
 iccocg = 1
-f_id0  = -1
 
-call gradient_s                                                  &
-( f_id0  , imrgra , inc    , iccocg , nswrgy , imligy , iwarny , &
-  epsrgy , climgy , extray ,                                     &
-  dispar , coefax , coefbx ,                                     &
-  grad   )
+call field_get_id("wall_distance", f_id)
 
-! Free memory
-deallocate(coefax, coefbx)
+call field_get_val_s(f_id, w_dist)
 
-!     Normalization (warning, the gradient may be sometimes equal to 0)
+! Current gradient: iprev = 0
+call field_gradient_scalar(f_id, 0, imrgra, inc, iccocg, grad)
 
+! Normalization (warning, the gradient may be sometimes equal to 0)
 do iel = 1 ,ncel
-  xnorme = max(sqrt(grad(1,iel)**2+grad(2,iel)**2+grad(3,iel)**2),epzero)
-  w2(iel) = -grad(1,iel)/xnorme
-  w3(iel) = -grad(2,iel)/xnorme
-  w4(iel) = -grad(3,iel)/xnorme
+  xnorme = max(sqrt(grad(1,iel)**2+grad(2,iel)**2+grad(3,iel)**2), epzero)
+  grad(1, iel) = -grad(1,iel)/xnorme
+  grad(2, iel) = -grad(2,iel)/xnorme
+  grad(3, iel) = -grad(3,iel)/xnorme
 enddo
 
-! Free memory
-deallocate(grad)
-
 !===============================================================================
 ! 3. Calculation of work variables
 !===============================================================================
@@ -621,8 +561,6 @@ do iel = 1 , ncel
   epsk(iel)   = cvara_ep(iel)/xk
 enddo
 
-
-
 ! ---> Tension rating
 do isou = 1, 6
   if     ((isou.eq.1).or.(isou.eq.4).or.(isou.eq.5)) then
@@ -680,22 +618,8 @@ do isou = 1, 6
       !  --> Terms with R km and Phi km
 
       do iel = 1, ncel
-
-        if    (kk.eq.1) then
-          vnk    = w2(iel)
-        elseif(kk.eq.2) then
-          vnk    = w3(iel)
-        elseif(kk.eq.3) then
-          vnk    = w4(iel)
-        endif
-
-        if    (mm.eq.1) then
-          vnm    = w2(iel)
-        elseif(mm.eq.2) then
-          vnm    = w3(iel)
-        elseif(mm.eq.3) then
-          vnm    = w4(iel)
-        endif
+        vnk    = grad(kk, iel)
+        vnm    = grad(mm, iel)
 
         w6(iel) = w6(iel) + vnk*vnm*deltij(isou)*(                        &
                crijp1*cvara_rij(kk,mm,iel)*epsk(iel)                      &
@@ -758,29 +682,9 @@ do isou = 1, 6
 
     do iel = 1, ncel
 
-        if    (kk.eq.1) then
-          vnk    = w2(iel)
-        elseif(kk.eq.2) then
-          vnk    = w3(iel)
-        elseif(kk.eq.3) then
-          vnk    = w4(iel)
-        endif
-
-        if    (ii.eq.1) then
-          vni    = w2(iel)
-        elseif(ii.eq.2) then
-          vni    = w3(iel)
-        elseif(ii.eq.3) then
-          vni    = w4(iel)
-        endif
-
-        if    (jj.eq.1) then
-          vnj    = w2(iel)
-        elseif(jj.eq.2) then
-          vnj    = w3(iel)
-        elseif(jj.eq.3) then
-          vnj    = w4(iel)
-        endif
+        vnk    = grad(kk, iel)
+        vni    = grad(ii, iel)
+        vnj    = grad(jj, iel)
 
       w6(iel) = w6(iel) + 1.5d0*vnk*(                               &
       -crijp1*(cvara_rij(kk,ii,iel)*vnj+cvara_rij(kk,jj,iel)*vni)*epsk(iel)     &
@@ -798,25 +702,21 @@ enddo
 !      Apart from the loop
 
 do iel = 1, ncel
-  distxn =  max(dispar(iel),epzero)
+  distxn =  max(w_dist(iel), epzero)!FIXME
   trrij  = 0.5d0 * (cvara_var(1,iel) + cvara_var(2,iel) + cvara_var(3,iel))
-  aa = 1.d0
   bb = cmu075*trrij**1.5d0/(xkappa*cvara_ep(iel)*distxn)
-  w3(iel) = min(aa, bb)
-enddo
+  bb = min(bb, 1.d0)
 
-! ---> Increment of source term
-
-do iel = 1, ncel
+  ! ---> Increment of source term
   do isou = 1, 6
-    smbr(isou,iel) = smbr(isou,iel) + cromo(iel)*volume(iel)*w6(iel)*w3(iel)
+    smbr(isou,iel) = smbr(isou,iel) + cromo(iel)*cell_f_vol(iel)*w6(iel)*bb
   enddo
 enddo
 
-! Allocate temporary arrays
+! Free memory
 deallocate(produk, epsk)
-deallocate(w2, w3, w4, w6)
-
+deallocate(w6)
+deallocate(grad)
 
 return
 end subroutine rijech2
