@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -27,7 +27,10 @@
 !> \file turbsa.f90
 !> \brief Solving op the equation of \f$ \tilde{\nu} \f$, which is the scalar
 !>        quantity defined by the Spalart-Allmaras model for 1 time-step.
-
+!>
+!> Please refer to the
+!> <a href="../../theory.pdf#spalart"><b>Spalart-Allmaras model</b></a>
+!> section of the theory guide for more informations.
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
@@ -72,7 +75,6 @@ use mesh
 use field
 use field_operator
 use parall
-use pointe, only: dispar
 use cs_c_bindings
 
 !===============================================================================
@@ -105,6 +107,8 @@ integer          iwarnp
 integer          istprv
 integer          ipatrg
 integer          imucpp, idftnp, iswdyp
+integer          f_id
+integer          init
 
 double precision romvsd
 double precision visct , rom
@@ -136,6 +140,9 @@ double precision, dimension(:), pointer :: cpro_pcvto
 double precision, dimension(:), pointer :: viscl, cvisct
 double precision, dimension(:), pointer :: coefap, coefbp, cofafp, cofbfp
 double precision, dimension(:), pointer :: c_st_nusa_p
+double precision, dimension(:), pointer :: w_dist
+
+type(var_cal_opt) :: vcopt_nusa
 
 !===============================================================================
 
@@ -156,8 +163,8 @@ allocate(tsexp(ncelet))
 allocate(dpvar(ncelet))
 
 call field_get_val_s(icrom, crom)
-call field_get_val_s(iprpfl(ivisct), cvisct)
-call field_get_val_s(iprpfl(iviscl), viscl)
+call field_get_val_s(ivisct, cvisct)
+call field_get_val_s(iviscl, viscl)
 call field_get_val_s(ibrom, brom)
 call field_get_val_s(ivarfl(inusa), cvar_nusa)
 call field_get_val_prev_s(ivarfl(inusa), cvara_nusa)
@@ -167,8 +174,14 @@ call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
 call field_get_val_s(iflmas, imasfl)
 call field_get_val_s(iflmab, bmasfl)
 
+call field_get_id("wall_distance", f_id)
+call field_get_val_s(f_id, w_dist)
+
 ivar   = inusa
-thetv  = thetav(ivar)
+
+call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt_nusa)
+
+thetv  = vcopt_nusa%thetav
 
 call field_get_key_int(ivarfl(inusa), kstprv, istprv)
 if (istprv.ge.0) then
@@ -183,11 +196,11 @@ if (istprv.ge.0) then
     call field_get_val_prev_s(icrom, cromo)
   endif
   if (iviext.gt.0) then
-    call field_get_val_s(iprpfl(ivista), cpro_pcvto)
+    call field_get_val_prev_s(ivisct, cpro_pcvto)
   endif
 endif
 
-if (iwarni(inusa).ge.1) then
+if (vcopt_nusa%iwarni.ge.1) then
   write(nfecra,1000)
 endif
 
@@ -329,7 +342,7 @@ do iel = 1, ncel
   ! kinematic viscosity
   nu0   = viscl(iel)/rom
   ! We have to know if there is any rough wall
-  distbf= dispar(iel)
+  distbf= w_dist(iel)
   ! viscosity of SA
   nusa  = cvara_nusa(iel)
   chi   = nusa/nu0
@@ -444,7 +457,7 @@ do iel = 1, ncel
 
   ! tinssa already contains the negativ implicited source term
   tinssa(iel) = tinssa(iel)                                        &
-               +istat(inusa)*romvsd
+               +vcopt_nusa%istat*romvsd
 enddo
 
 
@@ -511,14 +524,14 @@ call field_get_coefbf_s(ivarfl(ivar), cofbfp)
 
 ! Face viscosity
 
-if (idiff(ivar).ge.1) then
+if (vcopt_nusa%idiff.ge.1) then
 
   do iel = 1, ncel
     rom = crom(iel)
 
     ! diffusibility: 1/sigma*(mu_laminaire+ rho*nusa)
     w1(iel) = dsigma *( viscl(iel)                                &
-                        + idifft(ivar)*cvara_nusa(iel)*rom )
+                        + vcopt_nusa%idifft*cvara_nusa(iel)*rom )
   enddo
 
   call viscfa                                                     &
@@ -539,33 +552,34 @@ endif
 
 ! --- Solving
 
-iconvp = iconv (ivar)
-idiffp = idiff (ivar)
+iconvp = vcopt_nusa%iconv
+idiffp = vcopt_nusa%idiff
 ndircp = ndircl(ivar)
-nswrsp = nswrsm(ivar)
-nswrgp = nswrgr(ivar)
-imligp = imligr(ivar)
-ircflp = ircflu(ivar)
-ischcp = ischcv(ivar)
-isstpp = isstpc(ivar)
+nswrsp = vcopt_nusa%nswrsm
+nswrgp = vcopt_nusa%nswrgr
+imligp = vcopt_nusa%imligr
+ircflp = vcopt_nusa%ircflu
+ischcp = vcopt_nusa%ischcv
+isstpp = vcopt_nusa%isstpc
 iescap = 0
 imucpp = 0
-idftnp = idften(ivar)
-iswdyp = iswdyn(ivar)
-iwarnp = iwarni(ivar)
-blencp = blencv(ivar)
-epsilp = epsilo(ivar)
-epsrsp = epsrsm(ivar)
-epsrgp = epsrgr(ivar)
-climgp = climgr(ivar)
-extrap = extrag(ivar)
-relaxp = relaxv(ivar)
-thetap = thetav(ivar)
+idftnp = vcopt_nusa%idften
+iswdyp = vcopt_nusa%iswdyn
+iwarnp = vcopt_nusa%iwarni
+blencp = vcopt_nusa%blencv
+epsilp = vcopt_nusa%epsilo
+epsrsp = vcopt_nusa%epsrsm
+epsrgp = vcopt_nusa%epsrgr
+climgp = vcopt_nusa%climgr
+extrap = vcopt_nusa%extrag
+relaxp = vcopt_nusa%relaxv
+thetap = vcopt_nusa%thetav
 ! all boundary convective flux with upwind
 icvflb = 0
+init   = 1
 
 call codits &
- ( idtvar , ivar   , iconvp , idiffp , ndircp ,                   &
+ ( idtvar , init   , ivarfl(ivar)    , iconvp , idiffp , ndircp , &
    imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
    iwarnp ,                                                       &
