@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2015 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -36,6 +36,10 @@
 !> We have at edge faces types at previous time step
 !>   (except at first time step, when tables itypfb and itrifb
 !>   have not been filled).
+!>
+!> Please refer to the
+!> <a href="../../theory.pdf#dynsmago"><b>dynamic Smagorinsky model</b></a>
+!> section of the theory guide for more informations.
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
@@ -56,16 +60,13 @@
 !> \param[in]     smacel        value of variables associated to the
 !>                               mass source
 !>                               for ivar = ipr, smacel = mass flux
-!> \param[in,out] smagor        smagorinski's constant in the cas of
-!>                              a dynamic model
 !______________________________________________________________________________!
 
 subroutine visdyn &
  ( nvar   , nscal  , ncepdp , ncesmp ,                            &
    icepdc , icetsm , itypsm ,                                     &
    dt     ,                                                       &
-   ckupdc , smacel ,                                              &
-   smagor )
+   ckupdc , smacel )
 
 !===============================================================================
 ! Module files
@@ -98,7 +99,6 @@ integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
 
 double precision dt(ncelet)
 double precision ckupdc(ncepdp,6), smacel(ncesmp,nvar)
-double precision smagor(ncelet)
 
 ! Local variables
 
@@ -110,10 +110,10 @@ double precision coef, radeux, deux, delta, deltaf
 double precision s11, s22, s33, s11f, s22f, s33f
 double precision dudy, dudz, dvdx, dvdz, dwdx, dwdy
 double precision dudyf, dudzf, dvdxf, dvdzf, dwdxf, dwdyf
-double precision xfil, xa, xb, xfil2, xsmgmx
+double precision xfil, xa, xb, xfil2, xsmgmx, xsmgmn
 double precision xl11, xl22, xl33, xl12, xl13, xl23
 double precision xm11, xm22, xm33, xm12, xm13, xm23
-double precision smagma, smagmn, smagmy
+double precision smagma, smagmi, smagmy
 
 double precision, allocatable, dimension(:) :: w1, w2, w3
 double precision, allocatable, dimension(:) :: w4, w5, w6
@@ -125,7 +125,9 @@ double precision, dimension(:,:), pointer :: coefau
 double precision, dimension(:,:,:), pointer :: coefbu
 double precision, dimension(:), pointer :: crom
 double precision, dimension(:,:), pointer :: vel
-double precision, dimension(:), pointer :: visct
+double precision, dimension(:), pointer :: visct, cpro_smago
+
+type(var_cal_opt) :: vcopt
 
 !===============================================================================
 
@@ -134,14 +136,16 @@ double precision, dimension(:), pointer :: visct
 !===============================================================================
 
 ! Map field arrays
-call field_get_val_prev_v(ivarfl(iu), vel)
+call field_get_val_v(ivarfl(iu), vel)
 
 call field_get_coefa_v(ivarfl(iu), coefau)
 call field_get_coefb_v(ivarfl(iu), coefbu)
 
-call field_get_val_s(iprpfl(ivisct), visct)
+call field_get_val_s(ivisct, visct)
 call field_get_val_s(icrom, crom)
 
+call field_get_val_s(ismago, cpro_smago)
+
 ! For the calculation of the viscosity of the sub-mesh
 xfil   = xlesfl
 xfil2  = xlesfd
@@ -150,6 +154,7 @@ xb     = bles
 deux   = 2.d0
 radeux = sqrt(deux)
 xsmgmx = smagmx
+xsmgmn = smagmn
 
 ! Allocate some work arrays
 
@@ -165,7 +170,7 @@ allocate(xmij(6,ncelet))
 allocate(gradv(3,3,ncelet), gradvf(3,3,ncelet))
 
 inc = 1
-iprev = 1
+iprev = 0
 
 call field_gradient_vector(ivarfl(iu), iprev, imrgra, inc, gradv)
 
@@ -369,9 +374,9 @@ call les_filter(1, w2, w4)
 
 do iel = 1, ncel
   if(abs(w4(iel)).le.epzero) then
-    smagor(iel) = xsmgmx**2
+    cpro_smago(iel) = xsmgmx
   else
-    smagor(iel) = w3(iel)/w4(iel)
+    cpro_smago(iel) = w3(iel)/w4(iel)
   endif
 enddo
 
@@ -380,15 +385,15 @@ call ussmag                                                       &
    icepdc , icetsm , itypsm ,                                     &
    dt     ,                                                       &
    ckupdc , smacel ,                                              &
-   smagor , w1     , w2     )
+   w1     , w2     )
 
 iclipc = 0
 do iel = 1, ncel
-  if(smagor(iel).ge.xsmgmx**2) then
-    smagor(iel) = xsmgmx**2
+  if(cpro_smago(iel).ge.xsmgmx) then
+    cpro_smago(iel) = xsmgmx
     iclipc = iclipc + 1
-  elseif(smagor(iel).le.-xsmgmx**2) then
-    smagor(iel) = -xsmgmx**2
+  elseif(cpro_smago(iel).le.xsmgmn) then
+    cpro_smago(iel) = xsmgmn
     iclipc = iclipc + 1
   endif
 enddo
@@ -400,32 +405,34 @@ enddo
 ! Clipping in (mu + mu_t)>0 in phyvar
 
 do iel = 1, ncel
-  coef = smagor(iel)
+  coef = cpro_smago(iel)
   delta  = xfil * (xa*volume(iel))**xb
   visct(iel) = crom(iel) * coef * delta**2 * visct(iel)
 enddo
 
+call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt)
+
 !     Some printings
-if (iwarni(iu).ge.1) then
+if (vcopt%iwarni.ge.1) then
 
   smagma = -1.0d12
-  smagmn =  1.0d12
+  smagmi =  1.0d12
   smagmy =  0.d0
   do iel = 1, ncel
-    smagma = max(smagma,smagor(iel))
-    smagmn = min(smagmn,smagor(iel))
-    smagmy = smagmy + smagor(iel)*volume(iel)
+    smagma = max(smagma,cpro_smago(iel))
+    smagmi = min(smagmi,cpro_smago(iel))
+    smagmy = smagmy + cpro_smago(iel)*volume(iel)
   enddo
   if (irangp.ge.0) then
     call parmax(smagma)
-    call parmin(smagmn)
+    call parmin(smagmi)
     call parsom(smagmy)
     call parcpt(iclipc)
   endif
   smagmy = smagmy / voltot
   write(nfecra,1000) iclipc
   write(nfecra,2001)
-  write(nfecra,2002) smagma, smagmn, smagmy
+  write(nfecra,2002) smagmy, smagmi, smagma
   write(nfecra,2003)
 
 endif
