@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -218,7 +218,7 @@ if (iappel.eq.2) then
 
   do ii = 1, nfbpcd
     iz = izzftcd(ii)
-    if (iz.eq.1.and.icond.eq.0) then
+    if (iz.eq.1.and.icondb.eq.0) then
       ! Turbulent law and empiric correlations used to
       ! define the exchange coefficients of the sink
       ! source term and heat transfer to the cooling
@@ -321,7 +321,7 @@ elseif (iappel.eq.3) then
 !===============================================================================
 
   !-- pointer to the specific heat
-  if (icp.gt.0) call field_get_val_s(iprpfl(icp), cpro_cp)
+  if (icp.ge.0) call field_get_val_s(icp, cpro_cp)
 
   !-- pointer to the enthalpy value
   ivarh = isca(iscalt)
@@ -334,7 +334,7 @@ elseif (iappel.eq.3) then
 
     iz = izzftcd(ii)
 
-    if (icond.eq.0) then
+    if (icondb.eq.0) then
       if (iztag1d(iz).eq.1) then
         !-------------------------------------------
         !Boundary conditions of the 1-D thermal model
@@ -354,7 +354,7 @@ elseif (iappel.eq.3) then
         ! with a value specified by the user
         ztpar(iz) = 26.57d0
       endif
-    elseif (iz.eq.2.and.icond.eq.0) then
+    elseif (iz.eq.2.and.icondb.eq.0) then
       if (iztag1d(iz).eq.1) then
         !-------------------------------------------
         !Boundary conditions of the 1-D thermal model
