@@ -1,7 +1,7 @@
 /*============================================================================
  * This function is called at the end of each time step, and has a very
  *  general purpose
- *  (i.e. anything that does not have another dedicated user subroutine)
+ *  (i.e. anything that does not have another dedicated user function)
  *============================================================================*/
 
 /* VERS */
@@ -9,7 +9,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -55,8 +55,6 @@
 #include "bft_error.h"
 #include "bft_printf.h"
 
-#include "fvm_writer.h"
-
 #include "cs_base.h"
 #include "cs_field.h"
 #include "cs_field_pointer.h"
@@ -91,8 +89,7 @@ BEGIN_C_DECLS
  *
  * \brief This function is called at the end of each time step, and has a very
  * general purpose (i.e. anything that does not have another dedicated
- * user subroutine)
- *
+ * user function).
  */
 /*----------------------------------------------------------------------------*/
 
