@@ -1,7 +1,13 @@
+/*============================================================================
+ * Functions dealing with particle tracking
+ *============================================================================*/
+
+/* VERS */
+
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -20,10 +26,6 @@
 
 /*----------------------------------------------------------------------------*/
 
-/*============================================================================
- * Functions dealing with particle tracking
- *============================================================================*/
-
 #include "cs_defs.h"
 
 /*----------------------------------------------------------------------------
@@ -59,6 +61,7 @@
 #include "cs_order.h"
 #include "cs_parall.h"
 #include "cs_prototypes.h"
+#include "cs_random.h"
 #include "cs_search.h"
 #include "cs_time_step.h"
 #include "cs_timer_stats.h"
@@ -67,12 +70,11 @@
 #include "cs_field.h"
 #include "cs_field_pointer.h"
 
-#include "cs_prototypes.h"
-
 #include "cs_lagr.h"
 #include "cs_lagr_new.h"
 #include "cs_lagr_particle.h"
 #include "cs_lagr_stat.h"
+#include "cs_lagr_sde.h"
 #include "cs_lagr_geom.h"
 
 /*----------------------------------------------------------------------------
@@ -91,7 +93,7 @@ BEGIN_C_DECLS
  * Global variables
  *============================================================================*/
 
-static cs_real_t _debm[4];
+static cs_real_t _m_flow[4];
 
 /*============================================================================
  * Local (user defined) function definitions
@@ -180,10 +182,9 @@ _inlet2(cs_lagr_particle_set_t  *p_set,
 
   /* Calculations of the instantaneous particle velocity */
 
-  cs_lnum_t two = 2;
   cs_real_t vgauss[2];
 
-  CS_PROCF(normalen,NORMALEN) (&two,vgauss);
+  cs_random_normal(2, vgauss);
 
   cs_real_t *part_vel
     = cs_lagr_particle_attr(particle, p_set->p_am, CS_LAGR_VELOCITY);
@@ -215,7 +216,7 @@ _inlet2(cs_lagr_particle_set_t  *p_set,
  * \param[in]     vagaus   Gaussian random variables
  * \param[in]     gradpr   pressure gradient
  * \param[in]     gradvf   gradient of the flow velocity
- * \param[in,out] romp     particle density
+ * \param[in,out] rho_p     particle density
  * \param[out]    fextla   user external force field (m/s^2)$
  */
 /*----------------------------------------------------------------------------*/
@@ -225,12 +226,12 @@ cs_user_lagr_ef(cs_real_t            dt_p,
                 const cs_real_t      taup[],
                 const cs_real_3_t    tlag[],
                 const cs_real_3_t    piil[],
-                const cs_real_t      bx[],
+                const cs_real_33_t   bx[],
                 const cs_real_t      tsfext[],
                 const cs_real_33_t   vagaus[],
                 const cs_real_3_t    gradpr[],
                 const cs_real_33_t   gradvf[],
-                cs_real_t            romp[],
+                cs_real_t            rho_p[],
                 cs_real_3_t          fextla[])
 {
   cs_lagr_particle_set_t  *p_set = cs_lagr_get_particle_set();
@@ -244,78 +245,188 @@ cs_user_lagr_ef(cs_real_t            dt_p,
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief User setting of particle inlet conditions for the particles (inlet
- *        and treatment for the other boundaries)
+ * \brief User function (non-mandatory intervention)
  *
- *  This function is called after the initialization of the new particles in
- *  order to modify them according to new particle profiles (injection
- *  profiles, position of the injection point, statistical weights,
- *  correction of the diameter if the standard-deviation option is activated).
+ * User-defined modifications on the variables at the end of the
+ * Lagrangian time step and calculation of user-defined
+ * additional statistics on the particles.
  *
- * \param[in] time_id         time step indicator for fields
- *                            0: use fields at current time step
- *                            1: use fields at previous time step
- * \param[in] injfac          array of injection face id for every particles
- * \param[in] local_userdata  local_userdata pointer to zone/cluster specific
- *                            boundary conditions (number of injected
- *                            particles, velocity profile...)
+ * \param[in]  dt      time step (per cell)
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_user_lagr_in(int                         time_id,
-                int                        *injfac,
-                cs_lagr_zone_class_data_t  *local_userdata,
-                cs_real_t                   vislen[] )
+cs_user_lagr_extra_operations(const cs_real_t  dt[])
 {
-  const int ntcabs = cs_glob_time_step->nt_cur;
-
-  cs_lagr_bdy_condition_t  *lagr_bdy_conditions = cs_lagr_get_bdy_conditions();
-
   cs_lagr_particle_set_t  *p_set = cs_lagr_get_particle_set();
   const cs_lagr_attribute_map_t *p_am = p_set->p_am;
 
-  if (p_set->n_part_new == 0)
-    return;
+  /* Example: computation of the particle mass flow rate on 4 planes
+     --------------------------------------------------------------- */
 
-  /* Modifications occur after all the initializations related to
-     the particle injection, but before the treatment of the continuous
-     injection: it is thus possible to impose an injection profile with
-     the continous-injection option. */
+  {
+    cs_real_t zz[4] = {0.1e0, 0.15e0, 0.20e0, 0.25e0};
 
-  /* reinitialization of the counter of the new particles */
-  cs_lnum_t npt  = p_set->n_particles;
+    /* If we are in an unsteady case, or if the beginning of the steady stats
+     * is not reached yet, all statistics are reset to zero at each time
+     step before entering this function.*/
 
-  /* for each boundary zone */
-  for (cs_lnum_t ii = 0; ii < lagr_bdy_conditions->n_b_zones; ii++) {
+    if(   cs_glob_lagr_time_scheme->isttio == 0
+       || cs_glob_time_step->nt_cur <= cs_glob_lagr_stat_options->nstist) {
+      for (cs_lnum_t iplan = 0; iplan < 4; iplan++)
+        _m_flow[iplan] = 0.0;
 
-    cs_lnum_t izone = lagr_bdy_conditions->b_zone_id[ii];
+    }
 
-    /* for each class */
-    for (cs_lnum_t iclas = 0;
-         iclas < lagr_bdy_conditions->b_zone_classes[izone];
-         iclas++) {
+    for (cs_lnum_t iplan = 0; iplan < 4; iplan++) {
 
-      cs_lagr_zone_class_data_t *userdata
-        = &(local_userdata[iclas * cs_glob_lagr_nzone_max + izone]);
+      for (cs_lnum_t npt = 0; p_set->n_particles; npt++) {
 
-      /* if new particles must enter the domain:  */
-      if (ntcabs % userdata->injection_frequency == 0) {
+        unsigned char *part = p_set->p_buffer + p_am->extents * npt;
 
-        for (cs_lnum_t ip = npt; ip < npt + userdata->nb_part; ip++) {
+        cs_lnum_t iel = cs_lagr_particle_get_cell_id(part, p_am);
 
-          cs_lnum_t face_id = injfac[ip]; /* id of injection face */
+        if( iel >= 0 ) {
 
-          _inlet2(p_set, ip);
+          const cs_real_t *part_coords
+            = cs_lagr_particle_attr_const(part, p_am, CS_LAGR_COORDS);
+          const cs_real_t *prev_part_coords
+            = cs_lagr_particle_attr_n_const(part, p_am, 1, CS_LAGR_COORDS);
+
+          if (    part_coords[0] > zz[iplan]
+              && prev_part_coords[0] <= zz[iplan])
+            _m_flow[iplan] +=  cs_lagr_particle_get_real(part, p_am,
+                                                        CS_LAGR_STAT_WEIGHT)
+                             * cs_lagr_particle_get_real(part, p_am,
+                                                         CS_LAGR_MASS);
 
         }
 
-        npt += userdata->nb_part;
-
       }
 
     }
 
+    cs_real_t stat_age = cs_lagr_stat_get_age();
+
+    for (cs_lnum_t iplan = 0; iplan < 4; iplan++)
+      bft_printf(" Particle mass flow at Z(%d): %e14.5)",
+                 iplan,
+                 _m_flow[iplan]/stat_age);
+  }
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Impose the motion of a particle flagged CS_LAGR_PART_IMPOSED_MOTION.
+ *
+ * User-defined modifications on the particle position and its
+ * velocity.
+ *
+ * \param[in]   coords    old particle coordinates
+ * \param[in]   dt        time step (per particle)
+ * \param[out]  disp      particle dispacement
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_lagr_imposed_motion(const cs_real_t  coords[3],
+                            cs_real_t        dt,
+                            cs_real_t        disp[3])
+{
+  /* Angular velocity */
+  cs_real_t omega = 1.0;
+
+  /* Here we impose the particle to move arround a cylinder with
+   * the axis  is (*, 0, 1) */
+  cs_real_t rcost = (coords[1] - 0.0);
+  cs_real_t rsint = (coords[2] - 1.0);
+
+  /* Imposed displacement */
+  disp[0] = 0.;
+  disp[1] = rcost * (cos(omega*dt) - 1.0 ) - rsint * sin(omega*dt);
+  disp[2] = rsint * (cos(omega*dt) - 1.0 ) + rcost * sin(omega*dt);
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief User modification of newly injected particles.
+ *
+ * This function is called after the initialization of the new particles in
+ * order to modify them according to new particle profiles (injection
+ * profiles, position of the injection point, statistical weights,
+ * correction of the diameter if the standard-deviation option is activated).
+ *
+ * This function is called for each injection zone and set. Particles
+ * with ids between \c pset->n_particles and \c n_elts are initialized
+ * but may be modidied by this function.
+ *
+ * \param[in,out]  particles         particle set
+ * \param[in]      zis               zone injection set data
+ * \param[in]      particle_range    start and past-the-end ids of new particles
+ *                                   for this zone and class
+ * \param[in]      particle_face_id  face ids of new particles if zone is
+ *                                   a boundary,  NULL otherwise
+ * \param[in]      visc_length       viscous layer thickness
+ *                                   (size: number of mesh boundary faces)
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_lagr_in(cs_lagr_particle_set_t         *particles,
+                const cs_lagr_injection_set_t  *zis,
+                const cs_lnum_t                 particle_range[2],
+                const cs_lnum_t                 particle_face_id[],
+                const cs_real_t                 visc_length[])
+{
+  const int ntcabs = cs_glob_time_step->nt_cur;
+
+  cs_lagr_zone_data_t  *lagr_bdy_conditions
+    = cs_lagr_get_boundary_conditions();
+
+  /* Simple changes to selected attributes
+     ------------------------------------- */
+
+  for (cs_lnum_t p_id = particle_range[0]; p_id < particle_range[1]; p_id++) {
+
+    /* velocity */
+
+    cs_real_t *part_vel
+      = cs_lagr_particles_attr(particles, p_id, CS_LAGR_VELOCITY);
+    part_vel[0] = 1.0;
+    part_vel[1] = 0.0;
+    part_vel[2] = 0.0;
+
+    /* diameter */
+
+    cs_lagr_particles_set_real(particles, p_id, CS_LAGR_DIAMETER, 5e-05);
+
+    /* Temperature profile */
+
+    cs_lagr_particles_set_real(particles, p_id, CS_LAGR_TEMPERATURE, 20.0);
+
+    /* Statistical weight profile */
+
+    cs_lagr_particles_set_real(particles, p_id, CS_LAGR_STAT_WEIGHT, 0.01);
+
+    /* User variables */
+    for (int attr_id = CS_LAGR_USER;
+         attr_id < CS_LAGR_USER + cs_glob_lagr_model->n_user_variables;
+         attr_id++) {
+      cs_real_t *user_var = cs_lagr_particle_attr(particles, p_id, attr_id);
+      *user_var = 0.;
+    }
+
+  }
+
+  if (particle_range[1] <= particle_range[0])
+    return;
+
+  /* Modifications occur after all the initializations related to
+     the particle injection. */
+
+  /* if new particles have entered the domain  */
+  for (cs_lnum_t ip = particle_range[0]; ip < particle_range[1]; ip++) {
+    _inlet2(particles, ip);
   }
 
   /*
@@ -339,188 +450,21 @@ cs_user_lagr_in(int                         time_id,
    *
    * When the velocity of the flow is modified as above, most of the time
    * the user knows only the mean value. In some flow configurations and some
-   * injection conditions, it may be necessary to reconstruct the fluctuating part.
+   * injection conditions, it may be necessary to reconstruct the
+   * fluctuating part.
    * That is why the following function may be called.
    * Caution:
    *   - this turbulent component must be reconstructed only on the modified
    *     velocities of the flow seen.
-   *   - the reconstruction is must be adapted to the case. */
+   *   - the reconstruction must be adapted to the case. */
 
   if (false) {
-    cs_lnum_t npar1 = p_set->n_particles;
-    cs_lnum_t npar2 = p_set->n_particles + p_set->n_part_new;
-    cs_lagr_new_particle_init(npar1, npar2, time_id, vislen);
-  }
-}
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief Prescribe some attributes for newly injected particles.
- *
- * This function is called at different points, at which different attributes
- * may be modified.
- *
- * \param[inout]  particle  particle structure
- * \param[in]     p_am      particle attributes map
- * \param[in]     face_id   id of particle injection face
- * \param[in]     attr_id   id of variable modifiable by this call. called for
-                            CS_LAGR_VELOCITY, CS_LAGR_DIAMETER,
-                            CS_LAGR_TEMPERATURE, CS_LAGR_STAT_WEIGHT
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_user_lagr_new_p_attr(unsigned char                  *particle,
-                        const cs_lagr_attribute_map_t  *p_am,
-                        cs_lnum_t                       face_id,
-                        cs_lagr_attribute_t             attr_id)
-{
-  const cs_real_t pis6   = cs_math_pi / 6.0;
-
-  /* Velocity profile */
-
-  if (attr_id == CS_LAGR_VELOCITY) {
-
-    cs_real_t *part_vel = cs_lagr_particle_attr(particle, p_am, CS_LAGR_VELOCITY);
-    part_vel[0] = 1.0;
-    part_vel[1] = 0.0;
-    part_vel[2] = 0.0;
-
-  }
-
-  /* Diameter profile */
-
-  if (attr_id == CS_LAGR_DIAMETER)
-    cs_lagr_particle_set_real(particle, p_am, CS_LAGR_DIAMETER, 5e-05);
-
-  /* Temperature profile */
-
-  if (attr_id == CS_LAGR_TEMPERATURE)
-    cs_lagr_particle_set_real(particle, p_am, CS_LAGR_TEMPERATURE, 20.0);
-
-  /* Statistical weight profile */
-
-  if (attr_id == CS_LAGR_STAT_WEIGHT)
-    cs_lagr_particle_set_real(particle, p_am, CS_LAGR_STAT_WEIGHT, 0.01);
-}
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief Impose the motion of a particle falgged CS_LAGR_PART_IMPOSED_MOTION.
- *
- * User-defined modifications on the particle position and its
- * velocity.
- * \param[in]  coords    old particle coordinates
- * \param[in]  dt        time step (per particle)
- * \param[out] disp      particle dispacement
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_user_lagr_imposed_motion(const cs_real_3_t coords,
-                            const cs_real_t   dt,
-                            cs_real_3_t       disp)
-{
-
-  /* Angular velocity */
-  cs_real_t omega = 1.0;
-
-
-  /* Here we impose the particle to move arround a cylinder with
-   * the axe  is (*, 0, 1) */
-  cs_real_t rcost = (coords[1] - 0.0);
-  cs_real_t rsint = (coords[2] - 1.0);
-
-  /* Imposed displacement */
-  disp[0] = 0.;
-  disp[1] = rcost * (cos(omega*dt) - 1.0 ) - rsint * sin(omega*dt);
-  disp[2] = rsint * (cos(omega*dt) - 1.0 ) + rcost * sin(omega*dt);
-
-
-}
-
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief User function (non-mandatory intervention)
- *
- * User-defined modifications on the variables at the end of the
- * Lagrangian time step and calculation of user-defined
- * additional statistics on the particles.
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_user_lagr_extra_operations(const cs_real_t  dt[])
-{
-  cs_lagr_particle_set_t  *p_set = cs_lagr_get_particle_set();
-  const cs_lagr_attribute_map_t *p_am = p_set->p_am;
-
-  cs_lnum_t nxlist = 100;
-
-  const cs_lnum_t nfac        = cs_glob_mesh->n_i_faces;
-  const cs_lnum_t n_cells_ext = cs_glob_mesh->n_cells_with_ghosts;
-  const cs_real_3_t *ifacel   = cs_glob_mesh->i_face_cells;
-
-  const cs_lnum_t n_cells     = cs_glob_mesh->n_cells;
-  const cs_lnum_t n_vertices  = cs_glob_mesh->n_vertices;
-
-  const cs_real_t *cdgfac   = cs_glob_mesh_quantities->i_face_cog;
-  const cs_real_t *cell_cen = cs_glob_mesh_quantities->cell_cen;
-  const cs_real_t *pond     = cs_glob_mesh_quantities->weight;
-
-  /* Example: computation of the particle mass flow rate on 4 planes
-     --------------------------------------------------------------- */
-
-  if (false) {
-
-    cs_real_t zz[4] = {0.1e0, 0.15e0, 0.20e0, 0.25e0};
-
-    /* If we are in an unsteady case, or if the beginning of the steady stats
-     * is not reached yet, all statistics are reset to zero at each time
-     step before entering this function.*/
-
-    if(   cs_glob_lagr_time_scheme->isttio == 0
-       || cs_glob_time_step->nt_cur <= cs_glob_lagr_stat_options->nstist) {
-      for (cs_lnum_t iplan = 0; iplan < 4; iplan++)
-        _debm[iplan] = 0.0;
-
-    }
-
-    for (cs_lnum_t iplan = 0; iplan < 4; iplan++) {
-
-      for (cs_lnum_t npt = 0; p_set->n_particles; npt++) {
-
-        unsigned char *part = p_set->p_buffer + p_am->extents * npt;
-
-        cs_lnum_t iel = cs_lagr_particle_get_cell_id(part, p_am);
-
-        if( iel >= 0 ) {
-
-          const cs_real_t *part_coords
-            = cs_lagr_particle_attr_const(part, p_am, CS_LAGR_COORDS);
-          const cs_real_t *prev_part_coords
-            = cs_lagr_particle_attr_n_const(part, p_am, 1, CS_LAGR_COORDS);
-
-          if(    part_coords[0] > zz[iplan]
-              && prev_part_coords[0] <= zz[iplan])
-            _debm[iplan] +=  cs_lagr_particle_get_real(part, p_am,
-                                                       CS_LAGR_STAT_WEIGHT)
-              * cs_lagr_particle_get_real(part, p_am, CS_LAGR_MASS);
-
-        }
-
-      }
-
-    }
-
-    cs_real_t stat_age = cs_lagr_stat_get_age();
-
-    for (cs_lnum_t iplan = 0; iplan < 4; iplan++)
-      bft_printf(" Debit massique particulaire en Z(%d) : %E14.5)",
-                 iplan,
-                 _debm[iplan]/stat_age);
-
+    int time_id = 1;
+    if (cs_glob_time_step->nt_cur == 1)
+      time_id = 0;
+    cs_lagr_new_particle_init(particle_range,
+                              time_id,
+                              visc_length);
   }
 }
 
@@ -554,7 +498,7 @@ cs_user_lagr_extra_operations(const cs_real_t  dt[])
  *      h    : Coefficient of thermal exchange
  *       e
  *
- *  he coefficient of thermal exchange is calculated from a Nusselt number,
+ *  The coefficient of thermal exchange is calculated from a Nusselt number,
  *  itself evaluated by a correlation (Ranz-Marshall by default)
  *
  *             h  d
@@ -670,7 +614,7 @@ cs_user_lagr_rt(cs_lnum_t        id_p,
  * \param[out]  tauc   thermal relaxation time
  * \param[in]   dt     time step (per cell)
  */
-/*-------------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------------*/
 
 void
 cs_user_lagr_rt_t(cs_lnum_t        id_p,
@@ -759,9 +703,9 @@ cs_user_lagr_sde(const cs_real_t  dt[],
 
   /* Loop on the additional variables */
 
-  for (int iiii = 0;
-       iiii < cs_glob_lagr_model->n_user_variables;
-       iiii++) {
+  for (int i = 0;
+       i < cs_glob_lagr_model->n_user_variables;
+       i++) {
 
     for (cs_lnum_t npt = 0; npt < p_set->n_particles; npt++) {
 
@@ -782,12 +726,12 @@ cs_user_lagr_sde(const cs_real_t  dt[],
         /* Prediction at the first substep;
            This example must be adapted to the case */
         if (cs_glob_lagr_time_step->nor == 1)
-          pip[npt] = prev_usr_var[iiii];
+          pip[npt] = prev_usr_var[i];
 
         /* Correction at the second substep;
            This example must be adapted to the case */
         else
-          pip[npt] = usr_var[iiii];
+          pip[npt] = usr_var[i];
 
       }
 
