@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -56,16 +56,7 @@
 
 !===============================================================================
 
-subroutine usppmo &
-!================
- ( ixmlpu )
-
-
-!===============================================================================
-! Purpose:
-! -------
-
-!> \brief User subroutine.
+!> \brief User subroutine for selection of specific physics module
 
 !> Define the use of a specific physics amongst the following:
 !>   - combustion with gas / coal / heavy fuel oil
@@ -77,7 +68,6 @@ subroutine usppmo &
 !>
 !>    Only one specific physics module can be activated at once.
 
-
 !-------------------------------------------------------------------------------
 ! Arguments
 !______________________________________________________________________________.
@@ -87,6 +77,9 @@ subroutine usppmo &
 !>                              (1 : yes, 0 : no)
 !______________________________________________________________________________!
 
+subroutine usppmo &
+ ( ixmlpu )
+
 !===============================================================================
 ! Module files
 !===============================================================================
@@ -401,25 +394,7 @@ if (ixmlpu.eq.0) then
   ! Fuel combustion
 
   if (ippmod(icfuel).ge.0) then
-
     ficfpp = 'dp_FUE'
-
-  endif
-
-  ! Electric arcs
-
-  if (ippmod(ielarc).ge.1) then
-
-    ficfpp = 'dp_ELE'
-
-  endif
-
-  ! Joule effect
-
-  if (ippmod(ieljou).eq.1 .or. ippmod(ieljou).eq.2) then
-    ficfpp = 'dp_ELE'
-  else if (ippmod(ieljou).eq.3 .or. ippmod(ieljou).eq.4) then
-    ficfpp = 'dp_transfo'
   endif
 
   ! Atmospheric flows
@@ -430,12 +405,17 @@ if (ixmlpu.eq.0) then
 
  if ( ippmod(igmix).ge.0 ) then
    ! Specific condensation modelling
+
+   ! wall condensation
    !      if = -1 module not activated
    !      if =  0 condensation source terms activated
-   !      if =  1 condensation source terms with metal
-   !                               structures activate
-   icond = -1
+   icondb = -1
 
+   ! internal condensation
+   !      if = -1 module not activated
+   !      if =  0 condensation source terms with metal
+   !                               structures activate
+   icondv = -1
  endif
 
 endif
@@ -452,30 +432,25 @@ end subroutine usppmo
 
 !===============================================================================
 
-
-subroutine usipph &
-!================
- ( ixmlpu, iturb , itherm, iale , icavit )
-
-!===============================================================================
-! Purpose:
-! --------
-
-!> \brief User subroutine for input of parameters.
+!> \brief User subroutine for input of model selection parameters.
 
 !-------------------------------------------------------------------------------
 ! Arguments
 !______________________________________________________________________________.
 !  mode           name          role                                           !
 !______________________________________________________________________________!
-!> \param[in]     ixmlpu        indicates if the XML file from the GUI is used
+!> \param[in]      ixmlpu       indicates if the XML file from the GUI is used
 !>                              used (1: yes, 0: no
 !> \param[in, out] iturb        turbulence model
 !> \param[in, out] itherm       thermal model
 !> \param[in, out] iale         ale module
+!> \param[in, out] ivofmt       vof method
 !> \param[in, out] icavit       cavitation model
 !______________________________________________________________________________!
 
+subroutine usipph &
+ ( ixmlpu, iturb , itherm, iale , ivofmt, icavit )
+
 !===============================================================================
 ! Module files
 !===============================================================================
@@ -490,7 +465,7 @@ implicit none
 ! Arguments
 
 integer ixmlpu
-integer iturb, itherm, iale, icavit
+integer iturb, itherm, iale, ivofmt, icavit
 
 ! Local variables
 
@@ -542,7 +517,7 @@ endif
 
 ! Coupled solver for Rij components (when iturb=30, 31 or 32)
 
-irijco = 0
+irijco = 1
 
 ! --- Thermal model
 !      0: none
@@ -591,16 +566,6 @@ end subroutine usipph
 
 !===============================================================================
 
-
-subroutine usipsu &
-!================
-
- ( nmodpp )
-
-!===============================================================================
-! Purpose:
-! -------
-
 !> \brief User subroutine for the input of additional user parameters.
 !
 !-------------------------------------------------------------------------------
@@ -611,6 +576,9 @@ subroutine usipsu &
 !> \param[in]     nmodpp         number of active specific physics models
 !______________________________________________________________________________!
 
+subroutine usipsu &
+ ( nmodpp )
+
 !===============================================================================
 ! Module files
 !===============================================================================
@@ -633,7 +601,9 @@ use coincl
 use cpincl
 use field
 use cavitation
+use post
 use rotation
+use cs_c_bindings
 
 !===============================================================================
 
@@ -650,6 +620,8 @@ integer       ii, jj, ivar, kscmin, kscmax, keydri, kbfid, kccmin, kccmax
 integer       klimiter
 integer       f_id, idim1, itycat, ityloc, iscdri, iscal, ifcvsl, b_f_id
 
+type(var_cal_opt) :: vcopt
+
 !===============================================================================
 
 !>  This subroutine allows setting parameters
@@ -831,23 +803,28 @@ ipucou = 0
 !       if we suspect an excessive level of numerical diffusion on
 !         a variable ivar representing a user scalar
 !         iscal (with ivar=isca(iscal)), it may be useful to set
-!         blencv(ivar) = 1.0d0 to use a second-order scheme in space for
+!         blencv = 1.0d0 to use a second-order scheme in space for
 !         convection. For temperature or enthalpy in particular, we
 !         may thus choose in this case:
-!          blencv(isca(iscalt)) = 1.0d0
+!
+!         call field_get_key_struct_var_cal_opt(ivarfl(isca(iscalt)), vcopt)
+!         vcopt%blencv = 1.0d0
+!         call field_set_key_struct_var_cal_opt(ivarfl(isca(iscalt)), vcopt)
 
 !       For non-user scalars relative to specific physics (coal, combustion,
 !         electric arcs: see usppmo) implicitly defined by the model,
 !         the corresponding information is set automatically elsewhere:
 !         we do not modify blencv here.
 
+call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt)
+vcopt%blencv = 1.0d0
+call field_set_key_struct_var_cal_opt(ivarfl(iu), vcopt)
 
-blencv(iu) = 1.0d0
-blencv(iv) = 1.0d0
-blencv(iw) = 1.0d0
 if (nscaus.ge.1) then
   do ii = 1, nscaus
-    blencv(isca(ii)) = 1.0d0
+    call field_get_key_struct_var_cal_opt(ivarfl(isca(ii)), vcopt)
+    vcopt%blencv = 1.0d0
+    call field_set_key_struct_var_cal_opt(ivarfl(isca(ii)), vcopt)
   enddo
 endif
 
@@ -859,7 +836,9 @@ endif
 
 if (nscaus.ge.1) then
   do ii = 1, nscaus
-    epsilo(isca(ii)) = 1.d-6
+    call field_get_key_struct_var_cal_opt(ivarfl(isca(ii)), vcopt)
+    vcopt%epsilo = 1.d-6
+    call field_set_key_struct_var_cal_opt(ivarfl(isca(ii)), vcopt)
   enddo
 endif
 
@@ -867,14 +846,15 @@ endif
 ! --- Dynamic reconstruction sweeps to handle non-orthogonlaities
 !     This parameter computes automatically a dynamic relax factor,
 !     and can be activated for any variable.
-!      - iswdyn(ipr) = 1: means that the last increment is relaxed
-!      - iswdyn(ipr) = 2: means that the last two increments are used to
+!      - iswdyn = 1: means that the last increment is relaxed
+!      - iswdyn = 2: means that the last two increments are used to
 !                         relax
 !     NB: when iswdyn is greater than 1, then the number of
 !         non-orthogonality sweeps is increased to 20.
 
-iswdyn(ipr) = 1
-
+call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
+vcopt%iswdyn = 1
+call field_set_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
 
 ! --- Rotation/curvature correction for eddy-viscosity turbulence models
 !      0: deactivated
@@ -901,8 +881,13 @@ irccor = 1
 
 
 if (itytur.eq.2) then
-  ircflu(ik)   = 0
-  ircflu(iep)  = 0
+  call field_get_key_struct_var_cal_opt(ivarfl(ik), vcopt)
+  vcopt%ircflu = 0
+  call field_set_key_struct_var_cal_opt(ivarfl(ik), vcopt)
+
+  call field_get_key_struct_var_cal_opt(ivarfl(iep), vcopt)
+  vcopt%ircflu = 0
+  call field_set_key_struct_var_cal_opt(ivarfl(iep), vcopt)
 endif
 
 
@@ -1079,8 +1064,8 @@ p0 = 1.01325d5
 
 !     When no specific physics module is active, we may specify if the
 !       density, specific heat, and the molecular viscosity
-!       are constant (irovar=0, ivivar=0, icp=0), which is the default
-!       or variable (irovar=1, ivivar=1, icp=1)
+!       are constant (irovar=0, ivivar=0, icp=-1), which is the default
+!       or variable (irovar=1, ivivar=1, icp=0)
 
 !     For those properties we choose as variable, the corresponding law
 !       must be defined in usphyv
@@ -1089,7 +1074,7 @@ p0 = 1.01325d5
 
 irovar = 1
 ivivar = 1
-icp = 0
+icp = -1
 
 ! We only specify XYZ0 if we explicitely fix Dirichlet conditions
 ! for the pressure.
@@ -1136,23 +1121,30 @@ enddo
 
 ! --- Convective scheme for user (and non-user) scalars
 
-! ischcv(ivar) is the type of convective scheme:
+! ischcv is the type of convective scheme:
 !   - 0: second order linear upwind
 !   - 1: centered
 !   - 2: pure upwind gradient in SOLU
 
-! isstpc(ivar) is the slope test, Min/MAx limiter or Roe and Sweby limiters
+! isstpc is the slope test, Min/Max limiter or Roe and Sweby limiters
 !   - 0: swich on the slope test
 !   - 1: swich off the slope test (default)
-!   - 2: continuous limiter ensuring positivness
-!   - 3: Roe-Sweby limiter
-!        (ensuring  Decreasing Total Variation)
+!   - 2: continuous limiter ensuring boundedness (beta limiter)
+!   - 3: NVD/TVD Scheme
 !        Then "limiter_choice" keyword must be set:
-!       * 0: minmod
-!       * 1: Van-Leer
-!       * 2: Van-Albada
-!       * 3: Superbee
-
+!        * 0: Gamma
+!        * 1: SMART
+!        * 2: CUBISTA
+!        * 3: SUPERBEE
+!        * 4: MUSCL
+!        * 5: MINMOD
+!        * 6: CLAM
+!        * 7: STOIC
+!        * 8: OSHER
+!        * 9: WASEB
+!        * --- VOF scheme ---
+!        * 10: M-HRIC
+!        * 11: M-CICSAM
 
 ! Get the Key for the Sup and Inf for the convective scheme
 call field_get_key_id("min_scalar", kccmin)
@@ -1160,10 +1152,12 @@ call field_get_key_id("max_scalar", kccmax)
 
 ! Thermal model:
 if (iscalt.gt.0) then
-
   ivar = isca(iscalt)
-  ischcv(ivar) = 0
-  isstpc(ivar) = 3
+
+  call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+  vcopt%ischcv = 0
+  vcopt%isstpc = 3
+  call field_set_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
 
   ! Get the Key for the limiter choice of the studied scalar
   call field_get_key_id("limiter_choice", klimiter)
@@ -1177,10 +1171,12 @@ endif
 
 ! We loop on user scalars:
 do jj = 1, nscaus
-
   ivar = isca(jj)
-  ischcv(ivar) = 0
-  isstpc(ivar) = 2
+
+  call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+  vcopt%ischcv = 0
+  vcopt%isstpc = 2
+  call field_set_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
 
   ! Set the Value for the Sup and Inf of the studied scalar
   call field_set_key_double(ivarfl(ivar), kccmin, 0.d0)
@@ -1212,8 +1208,10 @@ enddo
 
 ! For thermal scalar
 if (ippmod(icompf).ge.0) then
+  ifcvsl = -1
   call field_set_key_int(ivarfl(isca(itempk)), kivisl, ifcvsl)
 else if (iscalt.gt.0) then
+  ifcvsl = -1
   call field_set_key_int(ivarfl(isca(iscalt)), kivisl, ifcvsl)
 endif
 
@@ -1225,14 +1223,41 @@ do iscal = 1, nscaus
 enddo
 
 
+! --- Variable density field id (ifcvsl>=0) or bulk
+!     density (ifcvsl=-1) for USER scalars.
+
+!     With ifcvsl = 0, the field will be added automatically, and later calls to
+!       field_get_key_int(ivarfl(isca(iscal)), kromsl, ifcvsl)
+!       will return its id.
+!     With ifcvsl > 0, the id of an existing, predifined field is given. This
+!       may allow sharing a density between multiple scalars.
+
+!     For user scalars iscal which represent the variance of another user
+!       scalar, the density of the variance of a scalar is assumed to
+!       have the same behavior as the density of this scalar,
+!       so values set here will be ignored.
+
+!     Caution:   complete usphyv with the law defining the density
+!     ========   if and only if ifcvsl = 0 has been set here.
+
+do iscal = 1, nscaus
+  if (iscavr(iscal).le.0) then
+    ifcvsl = -1
+    call field_set_key_int(ivarfl(isca(iscal)), kromsl, ifcvsl)
+  endif
+enddo
+
+
 ! --- Turbulent flux model u'T' for the scalar T
 !     Algebraic Model
 !      0  SGDH
 !      10 GGDH
+!      11 EB-GGDH (Elliptic Blending)
 !      20 AFM
+!      21 EB-AFM (Elliptic Blending)
 !     Model with transport equations
 !      30 DFM
-
+!      31 EB-DFM (Elliptic Blending)
 
 ! GGDH for thermal scalar:
 if (iscalt.gt.0) iturt(iscalt) = 10
@@ -1324,8 +1349,8 @@ inoprv = .false. ! no previous time step values needed
 call field_get_id_try('yplus', f_id)
 if (f_id.lt.0) then
   call field_create('yplus', itycat, ityloc, idim1, inoprv, f_id)
-  ! yplus postreated and in the log
-  call field_set_key_int(f_id, keyvis, 1)
+  ! yplus postprocessed and in the log
+  call field_set_key_int(f_id, keyvis, POST_ON_LOCATION + POST_MONITOR)
   call field_set_key_int(f_id, keylog, 1)
 endif
 
@@ -1333,7 +1358,7 @@ call field_get_id_try('tplus', f_id)
 if (f_id.lt.0) then
   call field_create('tplus', itycat, ityloc, idim1, inoprv, f_id)
   ! tplus postreated and in the log
-  call field_set_key_int(f_id, keyvis, 1)
+  call field_set_key_int(f_id, keyvis, POST_ON_LOCATION + POST_MONITOR)
   call field_set_key_int(f_id, keylog, 1)
 endif
 
@@ -1341,7 +1366,7 @@ call field_get_id_try('tstar', f_id)
 if (f_id.lt.0) then
   call field_create('tstar', itycat, ityloc, idim1, inoprv, f_id)
   ! tstar postreated and in the log
-  call field_set_key_int(f_id, keyvis, 1)
+  call field_set_key_int(f_id, keyvis, POST_ON_LOCATION + POST_MONITOR)
   call field_set_key_int(f_id, keylog, 1)
 endif
 
@@ -1398,17 +1423,6 @@ end subroutine usipsu
 
 !===============================================================================
 
-
-subroutine usipes &
-!================
-
- ( nmodpp )
-
-
-!===============================================================================
-! Purpose:
-! --------
-
 !> \brief User subroutine for the input of additional user parameters for
 !>        input/output.
 
@@ -1420,6 +1434,9 @@ subroutine usipes &
 !> \param[in]     nmodpp       number of active specific physics models
 !______________________________________________________________________________!
 
+subroutine usipes &
+ ( nmodpp )
+
 !===============================================================================
 ! Module files
 !===============================================================================
@@ -1435,9 +1452,11 @@ use field
 use parall
 use period
 use ihmpre
+use post
 use ppppar
 use ppthch
 use ppincl
+use cs_c_bindings
 
 !===============================================================================
 
@@ -1451,6 +1470,8 @@ integer nmodpp
 
 integer ii, ipp, f_id
 
+type(var_cal_opt) :: vcopt
+
 !===============================================================================
 
 !>     This subroutine allows setting parameters
@@ -1473,47 +1494,30 @@ ntlist = 1
 ! Log (listing) verbosity
 
 do ii = 1, nvar
-  iwarni(ii) = 1
+  call field_get_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+  vcopt%iwarni = 1
+  call field_set_key_struct_var_cal_opt(ivarfl(ii), vcopt)
 enddo
 
-iwarni(ipr) = 2
-iwarni(iu) = 2
-iwarni(iv) = 2
-iwarni(iw) = 2
+call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
+vcopt%iwarni = 2
+call field_set_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
+
+call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt)
+vcopt%iwarni = 2
+call field_set_key_struct_var_cal_opt(ivarfl(iu), vcopt)
 
 !===============================================================================
-! 2. Definition of probes
+! 2. Definition of deformable structure time plots
 !===============================================================================
 
-! --- probes output step
+! structures output step
 
 nthist = 1
 frhist = -1.d0
 
-
-! --- Number of monitoring points (probes) and their positions
-!     (limited to ncaptm=200)
-
-ncapt  = 4
 tplfmt = 1 ! time plot format (1: .dat, 2: .csv, 3: both)
 
-xyzcap(1,1) = 0.30d0
-xyzcap(2,1) = 0.15d0
-xyzcap(3,1) = 0.01d0
-
-xyzcap(1,2) = 0.30d0
-xyzcap(2,2) = 0.00d0
-xyzcap(3,2) = 0.01d0
-
-xyzcap(1,3) = 0.30d0
-xyzcap(2,3) =-0.08d0
-xyzcap(3,3) = 0.01d0
-
-xyzcap(1,4) = 0.60d0
-xyzcap(2,4) =-0.05d0
-xyzcap(3,4) = 0.01d0
-
-
 !===============================================================================
 ! 3. Fine control of variables output
 !===============================================================================
@@ -1526,19 +1530,15 @@ xyzcap(3,4) = 0.01d0
 if (isca(1).gt.0.and.nscaus.ge.1) then
   f_id = ivarfl(isca(1))
   call field_set_key_str(f_id, keylbl, 'Scalar 1')
-  call field_set_key_int(f_id, keyvis, 1)
+  call field_set_key_int(f_id, keyvis, POST_ON_LOCATION + POST_MONITOR)
   call field_set_key_int(f_id, keylog, 1)
-  ipp = field_post_id(f_id)
-  ihisvr(ipp,1)= -1
 endif
 
 if (isca(2).gt.0.and.nscaus.ge.2) then
   f_id = ivarfl(isca(2))
   call field_set_key_str(f_id, keylbl, 'Scalar 2')
-  call field_set_key_int(f_id, keyvis, 1)
+  call field_set_key_int(f_id, keyvis, POST_ON_LOCATION + POST_MONITOR)
   call field_set_key_int(f_id, keylog, 1)
-  ipp = field_post_id(f_id)
-  ihisvr(ipp,1)= -1
 endif
 
 
@@ -1549,14 +1549,6 @@ end subroutine usipes
 !===============================================================================
 
 
-subroutine usati1
-!================
-
-
-!===============================================================================
-! Purpose:
-! --------
-
 !> \brief Initialize non-standard calculation options for the atmospheric version.
 
 !-------------------------------------------------------------------------------
@@ -1565,6 +1557,8 @@ subroutine usati1
 !  mode           name          role                                           !
 !______________________________________________________________________________!
 
+subroutine usati1
+
 !===============================================================================
 ! Module files
 !===============================================================================
@@ -1738,7 +1732,6 @@ end subroutine usati1
 
 subroutine cs_user_combustion
 
-
 !===============================================================================
 ! Module files
 !===============================================================================
@@ -1828,14 +1821,6 @@ end subroutine cs_user_combustion
 
 !===============================================================================
 
-subroutine uscfx1
-!================
-
-
-!===============================================================================
-! Purpose:
-! -------
-
 !> \brief User subroutine.
 
 !> Initialize non standard options for the compressible flow scheme such
@@ -1848,6 +1833,8 @@ subroutine uscfx1
 !  mode           name          role                                           !
 !______________________________________________________________________________!
 
+subroutine uscfx1
+
 !===============================================================================
 ! Module files
 !===============================================================================
@@ -1898,10 +1885,10 @@ if (iihmpr.eq.0) then   !  Remove test to set values here when also using GUI.
   call field_set_key_int(ivarfl(isca(itempk)), kivisl, ifcvsl)
 
   ! --> Volumetric molecular viscosity
-  !       iviscv = 0 : uniform  in space and constant in time
-  !              = 1 : variable in space and time
+  !       iviscv = -1 : uniform  in space and constant in time
+  !              =  0 : variable in space and time
 
-  iviscv = 0
+  iviscv = -1
 
 endif
 
@@ -1918,13 +1905,6 @@ end subroutine uscfx1
 !===============================================================================
 
 
-subroutine uscfx2
-!================
-
-
-! Purpose:
-! -------
-
 !> \brief User subroutine.
 !>
 !> Set values for the reference volumic viscosity, the reference
@@ -1938,13 +1918,14 @@ subroutine uscfx2
 !> molecular viscosity is constant, its values being given in the user
 !> subroutine \ref usipsu .
 
-
 !-------------------------------------------------------------------------------
 ! Arguments
 !______________________________________________________________________________.
 !  mode           name          role                                           !
 !______________________________________________________________________________!
 
+subroutine uscfx2
+
 !===============================================================================
 ! Module files
 !===============================================================================
@@ -2032,15 +2013,6 @@ end subroutine uscfx2
 
 !===============================================================================
 
-
-subroutine uscti1
-!================
-
-
-!===============================================================================
-! Purpose:
-! -------
-
 !> \brief Definition of cooling tower model and exchange zones
 
 !-------------------------------------------------------------------------------
@@ -2049,6 +2021,8 @@ subroutine uscti1
 !  mode           name          role                                           !
 !______________________________________________________________________________!
 
+subroutine cs_user_cooling_towers
+
 !===============================================================================
 ! Module files
 !===============================================================================
@@ -2073,80 +2047,35 @@ implicit none
 
 !===============================================================================
 
-!< [uscti1]
-
 !===============================================================================
 ! 1. Parameters for prescibed temperature difference
 !===============================================================================
 
-! Activation
-iaeeri = 0
-
-! Temperature difference (cooling) to prescribe
-vaeeri = 13.d0
-
-! Temperature modification frequency
-iaeerp = 5
-
-! Temperature step to compute difference slope tref(teau)
-paseri = 0.015d0
-
-! Maximum average hot water temperature
-aetemx = 80.d0
-
-! Minimum average cooled water temperature
-aetemn = 10.d0
-
-! Number of excange zones with a water inlet boundary
-nbzsup = 2
-
-! List of the nbzsup exchange zones at water inlet boundary
-lizsup(1) = 1
-lizsup(2) = 2
-
-! Number of excange zones with a water outlet boundary
-nbzinf = 2
-
-! List of the nbzinf exchange zones at water outlet boundary
-lizinf(1) = 1
-lizinf(2) = 2
-
-! Prescribed difference activation start time
-
-inbaei = 1000.d0
-
-!===============================================================================
-! 2. Post-processing of exchange zones
-!===============================================================================
-
-ichrze = 1
-
-!===============================================================================
-! 3. Cooling tower restart
-!===============================================================================
-
-isuict = isuite
-
-!< [uscti1]
+! Air and liquid properties
+!< [cs_user_cooling_towers]
+cp_a    = 1006.0d0
+cp_v    = 1831.0d0
+cp_l    = 4179.0d0
+hv0    = 2501600.0d0
+rho_l   = 997.85615d0
+viscl0 = 1.765d-5
+lambda_l = 0.02493d0
+humidity0 = 0.d0
+droplet_diam = 0.005d0
+!< [cs_user_cooling_towers]
 
 !----
 ! End
 !----
 
 return
-end subroutine uscti1
+end subroutine cs_user_cooling_towers
 
 
 !===============================================================================
 
-subroutine user_darcy_ini1
-!========================
-
-!===============================================================================
-!  Purpose:
-!  --------
-
-!> \brief User routine for definition of computation parameters dealing with darcy module
+!> \brief User routine for definition of computation parameters dealing
+!>        with Darcy module
 
 !-------------------------------------------------------------------------------
 ! Arguments
@@ -2154,6 +2083,8 @@ subroutine user_darcy_ini1
 !  mode           name          role                                           !
 !______________________________________________________________________________!
 
+subroutine user_darcy_ini1
+
 !===============================================================================
 ! Module files
 !===============================================================================
