@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -32,6 +32,9 @@
  * Standard C library headers
  *----------------------------------------------------------------------------*/
 
+#include "stdlib.h"
+#include "string.h"
+
 /*----------------------------------------------------------------------------
  *  Local headers
  *----------------------------------------------------------------------------*/
@@ -39,16 +42,23 @@
 #include "bft_mem.h"
 #include "bft_error.h"
 
-#include "fvm_writer.h"
-
 #include "cs_base.h"
 #include "cs_field.h"
+#include "cs_geom.h"
+#include "cs_interpolate.h"
 #include "cs_mesh.h"
 #include "cs_selector.h"
-#include "cs_probe.h"
+#include "cs_parall.h"
 #include "cs_post.h"
+#include "cs_post_util.h"
+#include "cs_probe.h"
 #include "cs_time_plot.h"
 
+#include "cs_field_pointer.h"
+#include "cs_parameters.h"
+#include "cs_physical_constants.h"
+#include "cs_turbulence_model.h"
+
 /*----------------------------------------------------------------------------
  *  Header for the current file
  *----------------------------------------------------------------------------*/
@@ -81,6 +91,8 @@ _i_faces_select_example(void         *input,
                         cs_lnum_t    *n_faces,
                         cs_lnum_t   **face_ids)
 {
+  CS_UNUSED(input);
+
   cs_lnum_t i, face_id;
   cs_lnum_t n_families = 0;
   cs_int_t *family_list = NULL;
@@ -165,6 +177,8 @@ _b_faces_select_example(void         *input,
                         cs_lnum_t    *n_faces,
                         cs_lnum_t   **face_ids)
 {
+  CS_UNUSED(input);
+
   cs_lnum_t n_b_faces = 0;
   cs_lnum_t *b_face_ids = NULL;
 
@@ -208,6 +222,8 @@ _he_fraction_05_select(void        *input,
                        cs_lnum_t   *n_cells,
                        cs_lnum_t  **cell_ids)
 {
+  CS_UNUSED(input);
+
   cs_lnum_t _n_cells = 0;
   cs_lnum_t *_cell_ids = NULL;
 
@@ -276,15 +292,16 @@ cs_user_postprocess_writers(void)
   /* ----------------------- */
 
   /*! [post_define_writer_m1] */
-  cs_post_define_writer(-1,                /* writer_id */
-                        "results",         /* writer name */
-                        "postprocessing",  /* directory name */
-                        "EnSight Gold",    /* format_name */
-                        "",                /* format_options */
+  cs_post_define_writer(CS_POST_WRITER_DEFAULT,       /* writer_id */
+                        "results",                    /* writer name */
+                        "postprocessing",             /* directory name */
+                        "EnSight Gold",               /* format_name */
+                        "",                           /* format_options */
                         FVM_WRITER_FIXED_MESH,
-                        true,              /* output_at_end */
-                        -1,                /* frequency_n */
-                        -1.0);             /* frequency_t */
+                        false,                        /* output_at_start */
+                        true,                         /* output_at_end */
+                        -1,                           /* frequency_n */
+                        -1.0);                        /* frequency_t */
   /*! [post_define_writer_m1] */
 
   /* Define additional writers */
@@ -304,6 +321,7 @@ cs_user_postprocess_writers(void)
                         "MED",                        /* format name */
                         "divide_polyhedra",
                         FVM_WRITER_FIXED_MESH,
+                        false,                        /* output_at_start */
                         true,                         /* output_at_end */
                         -1,                           /* frequency_n */
                         -1.0);                        /* frequency_t */
@@ -316,7 +334,8 @@ cs_user_postprocess_writers(void)
                         "ensight",                    /* format name */
                         "text",
                         FVM_WRITER_TRANSIENT_CONNECT,
-                        false,
+                        false,                        /* output_at_start */
+                        false,                        /* output_at_end */
                         3,
                         frequency_t);
   /*! [post_define_writer_2] */
@@ -328,10 +347,24 @@ cs_user_postprocess_writers(void)
                         "plot",           /* format name */
                         "",               /* format options */
                         FVM_WRITER_FIXED_MESH,
-                        false,            // output_at_end
-                        100,              // nt_freq
-                        -1.0);            // dt_freq
+                        false,            /* output_at_start */
+                        false,            /* output_at_end */
+                        100,              /* nt_freq */
+                        -1.0);            /* dt_freq */
   /*! [post_define_writer_3] */
+
+  /*! [post_define_writer_4] */
+  cs_post_define_writer(6,                        /* writer_id */
+                        "Histogram",              /* writer name */
+                        "histograms",             /* directory name */
+                        "histogram",              /* format name */
+                        "10 tex",                 /* format options */
+                        FVM_WRITER_FIXED_MESH,
+                        false,                    /* output_at_start */
+                        true,                     /* output at end */
+                        -1,                       /* time step frequency */
+                        -1.0);                    /* time value frequency */
+  /*! [post_define_writer_4] */
 }
 
 /*----------------------------------------------------------------------------*/
@@ -352,7 +385,7 @@ cs_user_postprocess_meshes(void)
   /* De-activate boundary mesh output by redefining it with no writer
      association (default is:
      int n_writers = 1;
-     const int writer_ids[] = {-1});
+     const int writer_ids[] = {CS_POST_WRITER_DEFAULT});
   */
 
   /*! [post_define_mesh_m2] */
@@ -360,7 +393,7 @@ cs_user_postprocess_meshes(void)
     int n_writers = 0;
     const int *writer_ids = NULL;
 
-    cs_post_define_surface_mesh(-2,          /* mesh_id of main boundary mesh */
+    cs_post_define_surface_mesh(CS_POST_MESH_BOUNDARY,  /* mesh_id */
                                 "Boundary",  /* mesh name */
                                 NULL,        /* interior face selection criteria */
                                 "all[]",     /* boundary face selection criteria */
@@ -398,31 +431,6 @@ cs_user_postprocess_meshes(void)
 
   /*--------------------------------------------------------------------------*/
 
-  /* The same variables will be output through all writers associated
-   * to a mesh. In cases where different variables of a same mesh should
-   * be output throught different writers, the solution is to define one or
-   * several "aliases" of that mesh, allowing to assign a different id,
-   * writers, and variables to each secondary copy of the mesh, without the
-   * overhead of a full copy. The cs_post_define_alias_mesh() function
-   * may be used for such a purpose. */
-
-  /*! [post_define_mesh_2] */
-  {
-    /* Example: define an alias of the main surface mesh */
-
-    const int n_writers = 1;
-    const int writer_ids[] = {4};  /* Associate to writer 4 */
-
-    cs_post_define_alias_mesh(2,                 /* mesh id */
-                              -2,                /* aliased mesh id */
-                              false,             /* auto_variables */
-                              n_writers,
-                              writer_ids);
-  }
-  /*! [post_define_mesh_2] */
-
-  /*--------------------------------------------------------------------------*/
-
   /* Advanced example:
      Build a surface mesh containing interior faces separating cells of group "2"
      from those of group "3", (assuming no cell has both colors), as well as
@@ -487,6 +495,26 @@ cs_user_postprocess_meshes(void)
                               writer_ids);
   }
   /*! [post_define_mesh_5] */
+
+  /*--------------------------------------------------------------------------*/
+
+  /* Example: attach default txt histogram writer on boundary mesh */
+
+  /*! [post_attach_mesh_1] */
+  {
+    cs_post_mesh_attach_writer(CS_POST_MESH_BOUNDARY, CS_POST_WRITER_HISTOGRAMS);
+  }
+  /*! [post_attach_mesh_1] */
+
+  /*--------------------------------------------------------------------------*/
+
+  /* Example: attach user tex histogram writer of id 6 on volume mesh */
+
+  /*! [post_attach_mesh_2] */
+  {
+    cs_post_mesh_attach_writer(CS_POST_MESH_VOLUME, 6);
+  }
+  /*! [post_attach_mesh_2] */
 }
 
 /*----------------------------------------------------------------------------*/
@@ -500,20 +528,22 @@ cs_user_postprocess_meshes(void)
 void
 cs_user_postprocess_probes(void)
 {
-   /* Define monitoring probes */
+  /* Define monitoring probes */
+
+  /* A writer (id = CS_POST_WRITER_PROBES) using the format "time_plot" is
+     associated by default to a set of monitoring probes.
+     This is not the case for a profile. */
+
+  /*! [post_define_probes_1] */
   {
-    cs_real_3_t  m1 = {0.25, 0.025, 0.025};
-    cs_real_3_t  m2 = {0.50, 0.025, 0.025};
-    cs_real_3_t  m3 = {0.75, 0.025, 0.025};
     cs_probe_set_t  *pset = cs_probe_set_create("Monitoring");
 
-    cs_probe_set_add_probe(pset, m1, "M1");
-    cs_probe_set_add_probe(pset, m2, "M2");
-    cs_probe_set_add_probe(pset, m3, "M3");
+    cs_probe_set_add_probe(pset, 0.25, 0.025, 0.025, "M1");
+    cs_probe_set_add_probe(pset, 0.50, 0.025, 0.025, "M2");
+    cs_probe_set_add_probe(pset, 0.75, 0.025, 0.025, "M3");
 
-    /* A writer (id = -3) using the format "time_plot" is associated by default
-       to a set of monitoring probes. This is not the case for a profile. */
   }
+  /*! [post_define_probes_1] */
 
   /* Add a first profile */
   {
@@ -541,7 +571,7 @@ cs_user_postprocess_probes(void)
                                      end);    // end coordinates
   }
 
-  /* Add a second profile attached to border vertices */
+  /* Add a second profile attached to boundary vertices */
   {
     cs_coord_3_t  start = {0., 0., 0.};
     cs_coord_3_t  end = {1., 0., 0.};
@@ -556,11 +586,102 @@ cs_user_postprocess_probes(void)
     cs_probe_set_associate_writers(pset, 1, writer_ids);
 
     cs_probe_set_option(pset, "boundary", "true");
-    cs_probe_set_option(pset, "mode", "nearest_vertex");
+    cs_probe_set_snap_mode(pset, CS_PROBE_SNAP_VERTEX);
   }
 
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief User function for output of values on a post-processing mesh.
+ *
+ * \param[in]       mesh_name    name of the output mesh for the current call
+ * \param[in]       mesh_id      id of the output mesh for the current call
+ * \param[in]       cat_id       category id of the output mesh for the
+ *                               current call
+ * \param[in]       probes       pointer to associated probe set structure if
+ *                               the mesh is a probe set, NULL otherwise
+ * \param[in]       n_cells      local number of cells of post_mesh
+ * \param[in]       n_i_faces    local number of interior faces of post_mesh
+ * \param[in]       n_b_faces    local number of boundary faces of post_mesh
+ * \param[in]       n_vertices   local number of vertices faces of post_mesh
+ * \param[in]       cell_list    list of cells (0 to n-1) of post-processing
+ *                               mesh
+ * \param[in]       i_face_list  list of interior faces (0 to n-1) of
+ *                               post-processing mesh
+ * \param[in]       b_face_list  list of boundary faces (0 to n-1) of
+ *                               post-processing mesh
+ * \param[in]       vertex_list  list of vertices (0 to n-1) of
+ *                               post-processing mesh
+ * \param[in]       ts           time step status structure, or NULL
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_user_postprocess_values(const char            *mesh_name,
+                           int                    mesh_id,
+                           int                    cat_id,
+                           cs_probe_set_t        *probes,
+                           cs_lnum_t              n_cells,
+                           cs_lnum_t              n_i_faces,
+                           cs_lnum_t              n_b_faces,
+                           cs_lnum_t              n_vertices,
+                           const cs_lnum_t        cell_list[],
+                           const cs_lnum_t        i_face_list[],
+                           const cs_lnum_t        b_face_list[],
+                           const cs_lnum_t        vertex_list[],
+                           const cs_time_step_t  *ts)
+{
+  /* Output of k=1/2(R11+R22+R33) for the Rij-epsilon model
+     ------------------------------------------------------ */
+
+  /*< [postprocess_values_ex_1] */
+  if (cat_id == CS_POST_MESH_VOLUME && cs_glob_turb_model->itytur == 3) {
+
+    cs_real_t *s_cell;
+    BFT_MALLOC(s_cell, n_cells, cs_real_t);
+
+    if (cs_glob_turb_rans_model->irijco) {
+      const cs_real_6_t *cvar_r = (const cs_real_6_t *)(CS_F_(rij)->val);
+      for (cs_lnum_t i = 0; i < n_cells; i++) {
+        cs_lnum_t cell_id = cell_list[i];
+        s_cell[i] = 0.5* (  cvar_r[cell_id][0]
+                          + cvar_r[cell_id][1]
+                          + cvar_r[cell_id][2]);
+      }
+    }
+
+    else {
+      const cs_real_t *cvar_r11 = CS_F_(r11)->val;
+      const cs_real_t *cvar_r22 = CS_F_(r22)->val;
+      const cs_real_t *cvar_r33 = CS_F_(r33)->val;
+
+      for (cs_lnum_t i = 0; i < n_cells; i++) {
+        cs_lnum_t cell_id = cell_list[i];
+        s_cell[i] = 0.5* (  cvar_r11[cell_id]
+                          + cvar_r22[cell_id]
+                          + cvar_r33[cell_id]);
+      }
+    }
+
+    cs_post_write_var(mesh_id,
+                      CS_POST_WRITER_ALL_ASSOCIATED,  /* writer id filter */
+                      "Turb energy",                  /* var_name */
+                      1,                              /* var_dim */
+                      true,                           /* interlace, */
+                      false,                          /* use_parent */
+                      CS_POST_TYPE_cs_real_t,         /* var_type */
+                      s_cell,                         /* cel_vals */
+                      NULL,                           /* i_face_vals */
+                      NULL,                           /* b_face_vals */
+                      ts);
+
+    BFT_FREE(s_cell);
+  }
+  /*< [postprocess_values_ex_1] */
+
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * Override default frequency or calculation end based output.
