@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -34,7 +34,6 @@
 !> - The volume mesh (ipart=-1)
 !> - The boundary mesh (ipart=-2)
 !> - SYRTHES coupling surface (ipart < -2)
-!> - Cooling tower exchange zone meshes (ipart < -2) if 'ichrze' = 1
 !>
 !> Additional meshes (cells or faces) may also be defined through the GUI or
 !> using the \ref cs_user_postprocess_meshes function from the
@@ -124,7 +123,7 @@ double precision, dimension(:), pointer :: bfpro_rom
 double precision, dimension(:), pointer :: cvar_r11, cvar_r22, cvar_r33
 double precision, dimension(:), pointer :: cvar_var
 
-double precision, dimension(:), pointer :: cmom_1, cmom_2, cdt
+double precision, dimension(:), pointer :: cmom_1, cmom_2
 
 double precision, dimension(:,:), pointer :: cvar_vel
 
@@ -216,11 +215,6 @@ endif
 ! Examples for initialization
 !===============================================================================
 
-! Pointer to local time step
-call field_get_id('dt', f_id)
-call field_get_val_s(f_id, cdt)
-
-!< [postprocess_var_ex_1]
 
 !===============================================================================
 ! Examples of volume variables on the main volume mesh (ipart = -1)
@@ -228,39 +222,6 @@ call field_get_val_s(f_id, cdt)
 
 if (ipart .eq. -1) then
 
-  ! Output of k=1/2(R11+R22+R33) for the Rij-epsilon model
-  ! ------------------------------------------------------
-
-  if (itytur .eq. 3) then
-
-    allocate(scel(ncelps))
-
-    call field_get_val_s(ivarfl(ir11), cvar_r11)
-    call field_get_val_s(ivarfl(ir22), cvar_r22)
-    call field_get_val_s(ivarfl(ir33), cvar_r33)
-
-    do iloc = 1, ncelps
-      iel = lstcel(iloc)
-      scel(iloc) = 0.5d0*(  cvar_r11(iel)  &
-                          + cvar_r22(iel)  &
-                          + cvar_r33(iel))
-    enddo
-
-    idimt = 1        ! 1: scalar, 3: vector, 6/9: symm/non-symm tensor
-    ientla = .true.  ! dimension 1 here, so no effect
-    ivarpr = .false. ! defined on the work array, not on the parent
-
-    ! Output values; as we have no face values, we can pass a
-    ! trivial array rvoid for those.
-    call post_write_var(ipart, 'Turb energy', idimt, ientla, ivarpr,  &
-                        ntcabs, ttcabs, scel, rvoid, rvoid)
-
-    deallocate(scel)
-
-  endif
-
-!< [postprocess_var_ex_1]
-
 !< [postprocess_var_ex_2]
 
   ! Output of a combination of moments
