@@ -12,7 +12,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2018 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -130,6 +130,7 @@ use parall
 use period
 use mesh
 use field
+use cs_c_bindings
 
 !===============================================================================
 
@@ -153,10 +154,14 @@ double precision crvexp(3,ncelet), crvimp(3,3,ncelet)
 
 character*80     chaine
 integer          iel
-double precision ckp, qdm
+double precision ckp, qdm, beta
 
 integer, allocatable, dimension(:) :: lstelt
+double precision, dimension(:), pointer ::  cvar_temperature
 double precision, dimension(:), pointer ::  cpro_rom
+
+type(var_cal_opt) :: vcopt
+
 !< [loc_var_dec_1]
 
 !===============================================================================
@@ -167,7 +172,10 @@ double precision, dimension(:), pointer ::  cpro_rom
 ! Allocate a temporary array for cells selection
 allocate(lstelt(ncel))
 
-if (iwarni(ivar).ge.1) then
+! --- Get variable calculation options
+call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+
+if (vcopt%iwarni.ge.1) then
   call field_get_label(ivarfl(ivar), chaine)
   write(nfecra,1000) chaine(1:8)
 endif
@@ -199,7 +207,7 @@ call field_get_val_s(icrom, cpro_rom)
 
 ! ----------------------------------------------
 
-!!< [remaining_1]
+!< [remaining_1]
 ckp  = 10.d0
 qdm  = 100.d0
 
@@ -213,6 +221,20 @@ enddo
 
 !< [remaining_1]
 
+!< [boussinesq_st]
+
+! Expension coefficient
+beta = 1.d0
+
+! Get temperature field
+call field_get_val_s_by_name("temperature", cvar_temperature)
+
+do iel = 1, ncel
+  crvexp(3, iel) = cell_f_vol(iel) * ro0 * beta * (cvar_temperature(iel) - t0)
+enddo
+
+!< [boussinesq_st]
+
 !--------
 ! Formats
 !--------
@@ -362,6 +384,7 @@ use parall
 use period
 use mesh
 use field
+use cs_c_bindings
 
 !===============================================================================
 
@@ -394,6 +417,8 @@ double precision tauf, prodf, voltf, pwatt
 integer, allocatable, dimension(:) :: lstelt
 double precision, dimension(:), pointer ::  cpro_rom
 
+type(var_cal_opt) :: vcopt
+
 !< [loc_var_dec_2]
 
 !===============================================================================
@@ -428,9 +453,12 @@ iiscvr = iscavr(iscal)
 !< [density_2]
 call field_get_val_s(icrom, cpro_rom)
 
-if (iwarni(ivar).ge.1) then
+call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+
+if (vcopt%iwarni.ge.1) then
   write(nfecra,1000) chaine(1:8)
 endif
+
 !< [density_2]
 
 !===============================================================================
@@ -668,7 +696,7 @@ double precision crvexp(ncelet), crvimp(ncelet)
 integer          iel
 double precision ff, tau
 
-type(var_cal_opt) vcopt
+type(var_cal_opt) :: vcopt
 
 character(len=80) :: fname
 
