@@ -1,7 +1,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -2173,32 +2173,6 @@ _dense_3_3_sv_lu(const cs_real_t   a[restrict 3][3],
   x[0] = (b[0] - a[0][1]*x[1] - a[0][2]*x[2])/a[0][0];
 }
 
-/*----------------------------------------------------------------------------
- * Invert 3x3 matrix using Cramer's rule / Cayley Hamilton theorem
- *
- * This assumes the matrix is invertible
- *
- * parameters:
- *   a      <-- block matrix
- *----------------------------------------------------------------------------*/
-
-static inline void
-_dense_3_3_ci(cs_real_t   a[3][3])
-{
-  double _a[3][3];
-  cs_real_t t[3], di;
-
-  /* Copy array */
-
-  for (int i = 0; i < 3; i++) {
-    for (int j = 0; j < 3; j++) {
-      _a[i][j] = a[i][j];
-    }
-  }
-
-  cs_math_33_inv(_a, a);
-}
-
 /*----------------------------------------------------------------------------
  * Compute matrix-vector product for dense blocks: y[i] = a[i].x[i]
  *
@@ -2349,7 +2323,7 @@ _solve_33_test(double  t_measure)
 
   /* Tell IBM compiler not to alias */
 # if defined(__xlc__)
-# pragma disjoint(*x, *y)
+# pragma disjoint(*x, *a, *b)
 # endif
 
   /* Gaussian elimination */
@@ -2690,7 +2664,7 @@ _solve_33_test(double  t_measure)
         wt3 = cs_timer_wtime();
 #       pragma omp parallel for
         for (ii = 0; ii < n; ii++)
-          _dense_3_3_ci(a[ii]);
+          cs_math_33_inv_cramer_in_place(a[ii]);
         test_sum += test_sum_mult*a[run_id%n][0][0];
         run_id++;
       }
@@ -2765,7 +2739,9 @@ main (int argc, char *argv[])
 
   /* Initialization and environment */
 
+#if defined(HAVE_MPI)
   _mpi_init();
+#endif
 
   _pi = 4 * atan(1);
 
