@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -175,7 +175,7 @@ const char *f_base[] =
 int
 main (int argc, char *argv[])
 {
-  bft_mem_init(getenv("CS_MEM_TRACE"));
+  bft_mem_init(getenv("CS_MEM_LOG"));
 
   /* Cont elements */
 
